(PackageInfo named: 'Tools') postscript: 'ProcessBrowser initialize.'!
SystemOrganization addCategory: #'Tools-ArchiveViewer'!
SystemOrganization addCategory: #'Tools-Base'!
SystemOrganization addCategory: #'Tools-Browser'!
SystemOrganization addCategory: #'Tools-Changes'!
SystemOrganization addCategory: #'Tools-Debugger'!
SystemOrganization addCategory: #'Tools-Explorer'!
SystemOrganization addCategory: #'Tools-File Contents Browser'!
SystemOrganization addCategory: #'Tools-FileList'!
SystemOrganization addCategory: #'Tools-Inspector'!
SystemOrganization addCategory: #'Tools-Menus'!
SystemOrganization addCategory: #'Tools-MethodFinder'!
SystemOrganization addCategory: #'Tools-Process Browser'!

Model subclass: #CPUWatcher
	instanceVariableNames: 'tally watcher threshold'
	classVariableNames: 'CpuWatcherEnabled CurrentCPUWatcher'
	poolDictionaries: ''
	category: 'Tools-Process Browser'!

!CPUWatcher commentStamp: '<historical>' prior: 0!
CPUWatcher implements a simple runaway process monitoring tool
that will suspend a process that is taking up too much of Squeak's
time and allow user interaction. By default it watches for a Process that
is taking more than 80% of the time; this threshold can be changed.

CPUWatcher can also be used to show cpu percentages for each process 
from within the ProcessBrowser.

	CPUWatcher startMonitoring.	"process period 20 seconds, sample rate 100 msec"
	CPUWatcher current monitorProcessPeriod: 10 sampleRate: 20.
	CPUWatcher current threshold: 0.5.	"change from 80% to 50%"
	CPUWatcher stopMonitoring.
!

!CPUWatcher class methodsFor: 'preferences' stamp: 'fbs 12/31/2013 13:48'!
cpuWatcherEnabled
	<preference: 'CPU Watcher enabled' category: #('debug' 'performance') description: 'If true, Squeak will monitor processes for CPU usage. If any uses too much CPU, you will get a notification menu that will allow you to debug, resume or terminate the process.' type: #Boolean>
	^ CpuWatcherEnabled ifNil: [false].! !

!CPUWatcher class methodsFor: 'preferences' stamp: 'fbs 12/31/2013 13:48'!
cpuWatcherEnabled: aBoolean
	CpuWatcherEnabled := aBoolean.
	self monitorPreferenceChanged.! !

!CPUWatcher class methodsFor: 'as yet unclassified' stamp: 'nk 3/8/2001 18:45'!
current
	^CurrentCPUWatcher
! !

!CPUWatcher class methodsFor: 'as yet unclassified' stamp: 'nk 3/14/2001 08:28'!
currentWatcherProcess
	^CurrentCPUWatcher ifNotNil: [ CurrentCPUWatcher watcherProcess ]
! !

!CPUWatcher class methodsFor: 'as yet unclassified' stamp: 'nk 3/8/2001 21:43'!
dumpTallyOnTranscript
	self current ifNotNil: [
		ProcessBrowser dumpTallyOnTranscript: self current tally
	]! !

!CPUWatcher class methodsFor: 'as yet unclassified' stamp: 'nk 6/18/2003 07:15'!
initialize
	"CPUWatcher initialize"
	Smalltalk addToStartUpList: self.
	Smalltalk addToShutDownList: self.! !

!CPUWatcher class methodsFor: 'as yet unclassified' stamp: 'nk 3/14/2001 08:06'!
isMonitoring

	^CurrentCPUWatcher notNil and: [ CurrentCPUWatcher isMonitoring ]
! !

!CPUWatcher class methodsFor: 'as yet unclassified' stamp: 'fbs 12/31/2013 13:48'!
monitorPreferenceChanged
	self cpuWatcherEnabled
		ifTrue: [ self startMonitoring ]
		ifFalse: [ self stopMonitoring ]! !

!CPUWatcher class methodsFor: 'as yet unclassified' stamp: 'nk 6/18/2003 07:14'!
shutDown
	self stopMonitoring.! !

!CPUWatcher class methodsFor: 'as yet unclassified' stamp: 'nk 3/14/2001 08:17'!
startMonitoring
	"CPUWatcher startMonitoring"

	^self startMonitoringPeriod: 20 rate: 100 threshold: 0.8! !

!CPUWatcher class methodsFor: 'as yet unclassified' stamp: 'nk 3/14/2001 08:16'!
startMonitoringPeriod: pd rate: rt threshold: th
	"CPUWatcher startMonitoring"

	CurrentCPUWatcher ifNotNil: [ ^CurrentCPUWatcher startMonitoring. ].
	CurrentCPUWatcher := (self new)
		monitorProcessPeriod: pd sampleRate: rt;
		threshold: th;
		yourself.
	^CurrentCPUWatcher
! !

!CPUWatcher class methodsFor: 'as yet unclassified' stamp: 'nk 6/18/2003 07:14'!
startUp
	self monitorPreferenceChanged.! !

!CPUWatcher class methodsFor: 'as yet unclassified' stamp: 'nk 3/14/2001 08:05'!
stopMonitoring
	"CPUWatcher stopMonitoring"

	CurrentCPUWatcher ifNotNil: [ CurrentCPUWatcher stopMonitoring. ].
	CurrentCPUWatcher := nil.
! !

!CPUWatcher methodsFor: 'porcine capture' stamp: 'nk 3/8/2001 20:47'!
catchThePig: aProcess
	| rules |
	"nickname, allow-stop, allow-debug"
	rules := ProcessBrowser nameAndRulesFor: aProcess.

	(ProcessBrowser isUIProcess: aProcess)
		ifTrue: [ "aProcess debugWithTitle: 'Interrupted from the CPUWatcher'." ]
		ifFalse: [ rules second ifFalse: [ ^self ].
				ProcessBrowser suspendProcess: aProcess.
				self openWindowForSuspendedProcess: aProcess ]
! !

!CPUWatcher methodsFor: 'process operations' stamp: 'nk 3/8/2001 17:13'!
debugProcess: aProcess
	| uiPriority oldPriority |
	uiPriority := Processor activeProcess priority.
	aProcess priority >= uiPriority ifTrue: [
		oldPriority := ProcessBrowser setProcess: aProcess toPriority: uiPriority - 1
	].
	ProcessBrowser debugProcess: aProcess.! !

!CPUWatcher methodsFor: 'process operations' stamp: 'nk 3/8/2001 17:27'!
debugProcess: aProcess fromMenu: aMenuMorph
	aMenuMorph delete.
	self debugProcess: aProcess.! !

!CPUWatcher methodsFor: 'porcine capture' stamp: 'nk 3/8/2001 16:05'!
findThePig
	"tally has been updated. Look at it to see if there is a bad process.
	This runs at a very high priority, so make it fast"
	| countAndProcess | 
	countAndProcess := tally sortedCounts first.
	(countAndProcess key / tally size > self threshold) ifTrue: [ | proc |
		proc := countAndProcess value.
		proc == Processor backgroundProcess ifTrue: [ ^self ].	"idle process? OK"
		self catchThePig: proc
	].
! !

!CPUWatcher methodsFor: 'accessing' stamp: 'nk 3/14/2001 07:56'!
isMonitoring
	^watcher notNil! !

!CPUWatcher methodsFor: 'startup-shutdown' stamp: 'nk 3/14/2001 08:39'!
monitorProcessPeriod: secs sampleRate: msecs
	self stopMonitoring.

	watcher := [ [ | promise |
		promise := Processor tallyCPUUsageFor: secs every: msecs.
		tally := promise value.
		promise := nil.
		self findThePig.
	] repeat ] forkAt: Processor highestPriority.
	Processor yield ! !

!CPUWatcher methodsFor: 'porcine capture' stamp: 'nk 3/8/2001 17:23'!
openMorphicWindowForSuspendedProcess: aProcess
	| menu rules |
	menu := MenuMorph new.
	"nickname  allow-stop  allow-debug"
	rules := ProcessBrowser nameAndRulesFor: aProcess.
	menu add: 'Dismiss this menu' target: menu selector: #delete; addLine.
	menu add: 'Open Process Browser' target: ProcessBrowser selector: #open.
	menu add: 'Resume'
		target: self
		selector: #resumeProcess:fromMenu:
		argumentList: { aProcess . menu }.
	menu add: 'Terminate'
		target: self
		selector: #terminateProcess:fromMenu:
		argumentList: { aProcess . menu }.
	rules third ifTrue: [
		menu add: 'Debug at a lower priority'
			target: self
			selector: #debugProcess:fromMenu:
			argumentList: { aProcess . menu }.
	].
	menu addTitle: aProcess identityHash asString,
		' ', rules first,
		' is taking too much time and has been suspended.
What do you want to do with it?'.
	menu stayUp: true.
	menu popUpInWorld
! !

!CPUWatcher methodsFor: 'porcine capture' stamp: 'dtl 2/9/2010 22:00'!
openMVCWindowForSuspendedProcess: aProcess
	ProcessBrowser open! !

!CPUWatcher methodsFor: 'porcine capture' stamp: 'nk 3/8/2001 18:35'!
openWindowForSuspendedProcess: aProcess

	Smalltalk isMorphic
		ifTrue: [ WorldState addDeferredUIMessage: [ self openMorphicWindowForSuspendedProcess: aProcess ] ]
		ifFalse: [ [ self openMVCWindowForSuspendedProcess: aProcess ] forkAt: Processor userSchedulingPriority ]
! !

!CPUWatcher methodsFor: 'process operations' stamp: 'nk 3/8/2001 17:21'!
resumeProcess: aProcess fromMenu: aMenuMorph
	aMenuMorph delete.
	ProcessBrowser resumeProcess: aProcess.! !

!CPUWatcher methodsFor: 'startup-shutdown' stamp: 'nk 3/14/2001 08:07'!
startMonitoring
	self
		monitorProcessPeriod: 20 sampleRate: 100! !

!CPUWatcher methodsFor: 'startup-shutdown' stamp: 'nk 3/8/2001 16:24'!
stopMonitoring
	watcher ifNotNil: [
		ProcessBrowser terminateProcess: watcher.
		watcher := nil.
	]! !

!CPUWatcher methodsFor: 'accessing' stamp: 'nk 3/8/2001 18:36'!
tally
	^tally copy! !

!CPUWatcher methodsFor: 'process operations' stamp: 'nk 3/8/2001 17:24'!
terminateProcess: aProcess fromMenu: aMenuMorph
	aMenuMorph delete.
	ProcessBrowser terminateProcess: aProcess.! !

!CPUWatcher methodsFor: 'accessing' stamp: 'nk 3/8/2001 18:49'!
threshold
	"What fraction of the time can a process be the active process before we stop it?"
	^threshold! !

!CPUWatcher methodsFor: 'accessing' stamp: 'nk 3/8/2001 18:38'!
threshold: thresh
	"What fraction of the time can a process be the active process before we stop it?"
	threshold := (thresh max: 0.02) min: 1.0! !

!CPUWatcher methodsFor: 'accessing' stamp: 'nk 3/14/2001 08:26'!
watcherProcess
	^watcher! !

Model subclass: #DualChangeSorter
	instanceVariableNames: 'leftCngSorter rightCngSorter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Changes'!

!DualChangeSorter commentStamp: '<historical>' prior: 0!
This class presents a view of a two change sets at once, and supports copying changes between change sets.
!

!DualChangeSorter class methodsFor: 'opening' stamp: 'sw 3/24/1999 17:50'!
open
	"Open a new instance of the receiver's class"

	self new open! !

!DualChangeSorter class methodsFor: 'opening' stamp: 'ar 8/7/2009 22:20'!
prototypicalToolWindow
	"Answer an example of myself seen in a tool window, for the benefit of parts-launching tools"

 	^ ToolBuilder build: self new! !

!DualChangeSorter class methodsFor: 'class initialization' stamp: 'KR 8/12/2007 10:47'!
registerInFlapsRegistry
	"Register the receiver in the system's flaps registry"
	self environment
		at: #Flaps
		ifPresent: [:cl | cl registerQuad: {#DualChangeSorter,		#prototypicalToolWindow.	'Change Sorter' translatedNoop.		'Shows two change sets side by side' translatedNoop}
						forFlapNamed: 'Tools']! !

!DualChangeSorter class methodsFor: 'class initialization' stamp: 'asm 4/11/2003 12:33'!
unload
	"Unload the receiver from global registries"

	self environment at: #Flaps ifPresent: [:cl |
	cl unregisterQuadsWithReceiver: self] ! !

!DualChangeSorter methodsFor: 'toolbuilder' stamp: 'ar 9/6/2009 16:07'!
buildWith: builder
	| windowSpec window |
	leftCngSorter := ChangeSorter new myChangeSet: ChangeSet current.
	leftCngSorter parent: self.
	rightCngSorter := ChangeSorter new myChangeSet: 
			ChangeSorter secondaryChangeSet.
	rightCngSorter parent: self.

	windowSpec := builder pluggableWindowSpec new.
	windowSpec model: self.
	windowSpec label: 'Change Sorter'.
	windowSpec children: OrderedCollection new.
	windowSpec label: #labelString.
	leftCngSorter buildWith: builder in: windowSpec rect: (0@0 extent: 0.5@1).
	rightCngSorter buildWith: builder in: windowSpec rect: (0.5@0 extent: 0.5@1).
	window := builder build: windowSpec.
	leftCngSorter addDependent: window.		"so it will get changed: #relabel"
	rightCngSorter addDependent: window.	"so it will get changed: #relabel"
	^window! !

!DualChangeSorter methodsFor: 'user interface' stamp: 'mt 7/9/2016 08:52'!
defaultWindowColor
	^ (Color r: 0.719 g: 0.9 b: 0.9)! !

!DualChangeSorter methodsFor: 'other' stamp: ''!
isLeftSide: theOne
	"Which side am I?"
	^ theOne == leftCngSorter! !

!DualChangeSorter methodsFor: 'other' stamp: 'ar 1/4/2010 02:01'!
labelString
	"The window label"
	^'Changes go to "', ChangeSet current name,  '"'.! !

!DualChangeSorter methodsFor: 'other' stamp: 'tk 5/8/1998 16:30'!
modelWakeUp
	"A window with me as model is being entered.  Make sure I am up-to-date with the changeSets."

	"Dumb way"
	leftCngSorter canDiscardEdits 
		ifTrue: [leftCngSorter update]	"does both"
		ifFalse: [rightCngSorter update].
! !

!DualChangeSorter methodsFor: 'initialization' stamp: 'di 5/20/1998 21:44'!
okToChange
	^ leftCngSorter okToChange & rightCngSorter okToChange! !

!DualChangeSorter methodsFor: 'initialization' stamp: 'ar 8/7/2009 22:23'!
open
	^ToolBuilder open: self! !

!DualChangeSorter methodsFor: 'other' stamp: ''!
other: theOne
	"Return the other side's ChangeSorter"
	^ theOne == leftCngSorter
		ifTrue: [rightCngSorter]
		ifFalse: [leftCngSorter]! !

!DualChangeSorter methodsFor: 'initialization' stamp: ''!
release
	leftCngSorter release.
	rightCngSorter release.! !

!Model methodsFor: '*Tools' stamp: 'cmm 2/17/2010 22:03'!
addItem: classAndMethod
	"Make a linked message list and put this method in it"
	

	self flag: #mref.	"classAndMethod is a String"

	MessageSet 
		parse: classAndMethod  
		toClassAndSelector: [ :class :sel | | list |
			class ifNil: [^self].
			list := OrderedCollection with: (
				MethodReference new
					setClass: class  
					methodSymbol: sel 
					stringVersion: classAndMethod
			).
			ToolSet 
				browseMessageSet: list 
				name: 'Linked by HyperText'
				autoSelect: nil
		]

! !

!Model methodsFor: '*Tools-keyboard' stamp: 'nk 6/29/2004 14:46'!
arrowKey: aChar from: view
	"backstop; all the PluggableList* classes actually handle arrow keys, and the models handle other keys."
	^false! !

!Model methodsFor: '*Tools-pluggable menus' stamp: 'topa 10/8/2015 00:10'!
buildMenu: aMenu withBuilders: builders shifted: aBoolean
	" We let every builder modify the menu.
	The builder should indicate whether to abort by returning nil."
	| menu |
	menu := aMenu.
	builders do: [:builder |
		menu := self perform: builder method selector withEnoughArguments: { menu . aBoolean }.
		menu ifNil: [^ aMenu]].
	^ menu
! !

!Model methodsFor: '*Tools-pluggable menus' stamp: 'topa 10/8/2015 00:10'!
menuBuildersFor: someMenus in: aClass shifted: aBoolean
	"Find all builders but reject the ones not matching the shift state "
	| pragmas |
	pragmas := (self menuPragmasFor: someMenus in: aClass) .
	^ aBoolean 
		ifTrue:  [pragmas reject: [:builder | builder arguments = #(false)]]
		ifFalse: [pragmas reject: [:builder | builder arguments = #(true)]].
! !

!Model methodsFor: '*Tools-pluggable menus' stamp: 'topa 10/8/2015 00:10'!
menuPragmasFor: aMenuSymbolOrCollection in: aClass
	
	^ aMenuSymbolOrCollection isCollection
		ifTrue: [aMenuSymbolOrCollection gather: [:aMenuSymbol |
				Pragma allNamed: aMenuSymbol from: aClass to: Object]]
		ifFalse: [Pragma allNamed: aMenuSymbolOrCollection from: aClass to: Object]! !

!Model methodsFor: '*Tools-pluggable menus' stamp: 'topa 10/8/2015 00:10'!
menu: aMenu for: aMenuSymbolOrCollection

	^ self menu: aMenu for: aMenuSymbolOrCollection shifted: false! !

!Model methodsFor: '*Tools-pluggable menus' stamp: 'topa 10/8/2015 00:10'!
menu: aMenu for: aMenuSymbolOrCollection shifted: aBoolean

	| builders |
	builders := self menuBuildersFor: aMenuSymbolOrCollection in: self class shifted: aBoolean.
	builders := self sortMenuBuilders: builders.
	^ self buildMenu: aMenu withBuilders: builders shifted: aBoolean
! !

!Model methodsFor: '*Tools-pluggable menus' stamp: 'topa 10/8/2015 00:10'!
methodMenuPriority: aMethod
	"The pirority of a menu method indicated by a <menuPriority: ...> pragma.
	Defaults to 500"
	^ (aMethod pragmas detect: [:p | p keyword == #menuPriority: ] ifNone: [^ 500])
		argumentAt: 1
! !

!Model methodsFor: '*Tools' stamp: 'tk 4/17/1998 17:28'!
selectedClass
	"All owners of TextViews are asked this during a doIt"
	^ nil! !

!Model methodsFor: '*Tools-pluggable menus' stamp: 'topa 10/8/2015 00:10'!
sortMenuBuilders: builders 
	" Sort them by
		1. Priority (default 500)
		2. selector name
	"
	^  builders sorted: [:a :b | 
		| ma mb pa pb |
		ma := a method.
		mb := b method.
		pa := self methodMenuPriority: ma.
		pb := self methodMenuPriority: mb.
		pa < pb or: [pa = pb and: [ma selector <= mb selector]]]
! !

!Model methodsFor: '*Tools-menus' stamp: 'sw 12/15/2000 13:21'!
trash
	"What should be displayed if a trash pane is restored to initial state"

	^ ''! !

!Model methodsFor: '*Tools-menus' stamp: 'sw 12/15/2000 13:21'!
trash: ignored
	"Whatever the user submits to the trash, it need not be saved."

	^ true! !

Model subclass: #PointerFinder
	instanceVariableNames: 'goal parents toDo toDoNext hasGemStone pointerList objectList parentsSize todoSize depth pointerListIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Debugger'!

!PointerFinder commentStamp: '<historical>' prior: 0!
I can search for reasons why a certain object isn't garbage collected.  I'm a quick port of a VisualWorks program written by Hans-Martin Mosner.  Call me as shown below.  I'll search for a path from a global variable to the given object, presenting it in a small morphic UI.

Examples:
	PointerFinder on: self currentHand
	PointerFinder on: StandardSystemView someInstance

Now, let's see why this image contains more HandMorphs as expected...

HandMorph allInstancesDo: [:e | PointerFinder on: e]!

!PointerFinder class methodsFor: 'instance creation' stamp: 'sma 6/6/2000 23:52'!
on: anObject
	^ self new goal: anObject; search; open! !

!PointerFinder class methodsFor: 'utilities' stamp: 'sd 9/24/2004 20:48'!
pointersToItem: index of: anArray
	"Find all occurrences in the system of pointers to the given element of the given array. 
	This is useful for tracing up a pointer chain from an inspector on the results of a previous 	call of pointersTo:. To find out who points to the second element of the results, one would 	evaluate:

		PointerFinder pointersToItem: 2 of: self

	in the inspector."

	^ self pointersTo: (anArray at: index) except: (Array with: anArray)! !

!PointerFinder class methodsFor: 'utilities' stamp: 'sd 9/24/2004 20:49'!
pointersTo: anObject
	"Find all occurrences in the system of pointers to the argument anObject."
	"(PointerFinder pointersTo: Browser) inspect."

	^ self pointersTo: anObject except: #()
! !

!PointerFinder class methodsFor: 'utilities' stamp: 'mtf 8/15/2008 23:52'!
pointersTo: anObject except: objectsToExclude
	"Find all occurrences in the system of pointers to the argument anObject. 
	Remove objects in the exclusion list from the results."
	
	^ anObject inboundPointersExcluding: objectsToExclude! !

!PointerFinder methodsFor: 'pointer-list' stamp: 'sma 6/7/2000 00:23'!
arrowKey: key from: aController
	key = $i ifTrue: [^ self inspectObject].
	^ super arrowKey: key from: aController! !

!PointerFinder methodsFor: 'application' stamp: 'cmm 2/3/2014 16:19'!
buildList
	| list obj parent object key |
	list := OrderedCollection new.
	obj := goal.
	
	[list addFirst: obj.
	obj := parents at: obj ifAbsent: [].
	obj == nil] whileFalse.
	list removeFirst.
	parent := Smalltalk.
	objectList := OrderedCollection new.
	pointerList := OrderedCollection new.
	[list isEmpty]
		whileFalse: 
			[object := list removeFirst.
			key := nil.
			(parent isKindOf: Dictionary)
				ifTrue: [list size >= 2
						ifTrue: 
							[key := parent keyAtValue: list second ifAbsent: [].
							key == nil
								ifFalse: 
									[object := list removeFirst; removeFirst.
									pointerList add: key printString , ' -> ' , object class name]]].
			key == nil
				ifTrue: 
					[parent class == object ifTrue: [key := 'CLASS'].
					key == nil ifTrue: [1 to: parent class instSize do: [:i | key == nil ifTrue: [(parent instVarAt: i)
									== object ifTrue: [key := parent class allInstVarNames at: i]]]].
					key == nil ifTrue: [1 to: parent basicSize do: [:i | key == nil ifTrue: [(parent basicAt: i)
									== object ifTrue: [key := i printString]]]].
					key == nil ifTrue: [(parent isMorph and: [object isKindOf: Array]) ifTrue: [key := 'submorphs?']].
					key == nil ifTrue: [(parent isCompiledMethod and: [object isVariableBinding]) ifTrue: [key := 'literals?']].
					key == nil ifTrue: [key := '???'].
					pointerList add: key , ': ' , object class name, (object isMorph ifTrue: [' (', object identityHash asString, ')'] ifFalse: [ String empty ]) ].
			objectList add: object.
			parent := object]! !

!PointerFinder methodsFor: 'ui-building' stamp: 'fbs 6/20/2013 22:04'!
buildWith: builder
	| listSpec windowSpec |
	windowSpec := builder pluggableWindowSpec new
		model: self;
		label: 'Pointer Finder'.

	listSpec := builder pluggableListSpec new
		model: self;
		frame: (0@0 corner: 1@1);
		list: #pointerList;
		getIndex: #pointerListIndex;
		setIndex: #pointerListIndex:;
		menu: #menu:shifted:;
		keyPress: #arrowKey:from:;
		doubleClick: #inspectObject;
		yourself.

	windowSpec children: {listSpec}.
		
	^ builder build: windowSpec.! !

!PointerFinder methodsFor: 'application' stamp: 'mtf 8/15/2008 17:28'!
followObject: anObject
	anObject outboundPointersDo: [:ea |
		(self follow: ea from: anObject)
			ifTrue: [^ true]].
	^ false! !

!PointerFinder methodsFor: 'application' stamp: 'cmm 5/26/2013 11:15'!
follow: anObject from: parentObject 
	anObject == goal ifTrue:
		[ parents
			at: anObject
			put: parentObject.
		^ true ].
	anObject shouldFollowOutboundPointers ifFalse: [ ^ false ].
	((parents includesKey: anObject) or: [ anObject class = self class ]) ifTrue: [ ^ false ].
	parents
		at: anObject
		put: parentObject.
	toDoNext add: anObject.
	^ false! !

!PointerFinder methodsFor: 'application' stamp: 'sma 6/6/2000 09:52'!
goal: anObject
	goal := anObject! !

!PointerFinder methodsFor: 'ui-building' stamp: 'sma 6/6/2000 23:48'!
initialExtent
	^ 300 @ 300! !

!PointerFinder methodsFor: 'application' stamp: 'sma 6/7/2000 00:19'!
initialize
	parents := IdentityDictionary new: 20000.
	parents at: Smalltalk put: nil.
	parents at: Processor put: nil.
	parents at: self put: nil.

	toDo := OrderedCollection new: 5000.
	toDo add: Smalltalk.
	toDoNext := OrderedCollection new: 5000! !

!PointerFinder methodsFor: 'pointer-list' stamp: 'nb 6/17/2003 12:25'!
inspectObject
	pointerListIndex = 0 ifTrue: [^ Beeper beep].
	(objectList at: pointerListIndex) inspect! !

!PointerFinder methodsFor: 'pointer-list' stamp: 'topa 3/18/2015 10:56'!
menu: aMenu shifted: shifted

	^ self menu: aMenu for: #(pointerListMenu pointerListMenuShifted:) shifted: shifted! !

!PointerFinder methodsFor: 'ui-building' stamp: 'fbs 6/20/2013 21:56'!
open
	^ ToolBuilder open: self.! !

!PointerFinder methodsFor: 'morphic ui' stamp: 'fbs 6/20/2013 21:53'!
openInMorphic
	| window list |
	window := (SystemWindow labelled: 'Pointer Finder')
		model: self.
	list := PluggableListMorph new
		doubleClickSelector: #inspectObject;

		on: self
		list: #pointerList
		selected: #pointerListIndex
		changeSelected: #pointerListIndex:
		menu: #menu:shifted:
		keystroke: #arrowKey:from:.
		"For doubleClick to work best disable autoDeselect"
		list autoDeselect: false.
	window addMorph: list frame: (0@0 extent: 1@1).
	list color: Color lightMagenta.
	window openInWorld! !

!PointerFinder methodsFor: 'message handling' stamp: 'sma 6/7/2000 00:15'!
perform: selector orSendTo: otherTarget
	selector == #inspectObject ifTrue: [^ self inspectObject].
	selector == #searchAgain ifTrue: [^ self searchAgain].
	^ super perform: selector orSendTo: otherTarget! !

!PointerFinder methodsFor: 'pointer-list' stamp: 'sma 6/6/2000 23:49'!
pointerList
	^ pointerList asArray! !

!PointerFinder methodsFor: 'pointer-list' stamp: 'sma 6/6/2000 23:27'!
pointerListIndex
	^ pointerListIndex ifNil: [0]! !

!PointerFinder methodsFor: 'pointer-list' stamp: 'sma 6/6/2000 23:51'!
pointerListIndex: anInteger
	pointerListIndex := anInteger.
	self changed: #pointerListIndex! !

!PointerFinder methodsFor: 'pointer-list' stamp: 'topa 3/18/2015 10:55'!
pointerListMenu: aMenu

	<pointerListMenu>	
	^ aMenu
		addList: #(
			('Inspect (i)' #inspectObject 'Live long and prosper!!')
			-
			('Search again' #searchAgain 'Search again\for the same object' withCRs));
		yourself.! !

!PointerFinder methodsFor: 'application' stamp: 'sma 6/7/2000 00:17'!
search
	Smalltalk garbageCollect.

	self initialize.
	
	Cursor wait showWhile: [
		[[toDo isEmpty or: [self followObject: toDo removeFirst]] whileFalse.
		toDo isEmpty and: [toDoNext isEmpty not]]
			whileTrue: 
				[toDo := toDoNext.
				toDoNext := OrderedCollection new: 5000]].

	self buildList! !

!PointerFinder methodsFor: 'pointer-list' stamp: 'sma 6/7/2000 00:16'!
searchAgain
	self pointerListIndex: 0.
	self search.
	self changed: #pointerList! !

!PointerFinder methodsFor: 'application' stamp: 'sma 6/6/2000 19:10'!
update
	('done: ' , parents size asString , ' todo: ' , toDo size asString , '   ') displayAt: 0@0! !

Model subclass: #ProcessBrowser
	instanceVariableNames: 'selectedProcess selectedContext methodText processList processListIndex stackList stackListIndex sourceMap selectedClass selectedSelector searchString autoUpdateProcess lastUpdate startedCPUWatcher'
	classVariableNames: 'Browsers SuspendedProcesses WellKnownProcesses'
	poolDictionaries: ''
	category: 'Tools-Process Browser'!

!ProcessBrowser commentStamp: '<historical>' prior: 0!
Change Set:		ProcessBrowser
Date:			14 March 2000
Author:			Ned Konz

email: ned@bike-nomad.com

This is distributed under the Squeak License.

Added 14 March:
	CPUWatcher integration
	automatically start and stop CPUWatcher
	added CPUWatcher to process list menu

Added 29 October:
	MVC version
	2.8, 2.7 compatibility
	rearranged menus
	added pointer inspection and chasing
	added suspend/resume
	recognized more well-known processes
	misc. bug fixes

Added 26 October: highlight pc in source code
Added 27 October: added 'signal semaphore'
added 'inspect receiver', 'explore receiver', 'message tally' to stack list menu
added 'find context', 'next context' to process list menu
added 'change priority' and 'debug' choices to process list menu

27 October mods by Bob Arning:

alters process display in Ned's ProcessBrowser to 
- show process priority
- drop 'a Process in' that appears on each line
- show in priority order
- prettier names for known processes
- fix to Utilities to forget update downloading process when it ends (1 less dead
process)
- correct stack dump for the active process
!

!ProcessBrowser class methodsFor: 'class initialization' stamp: 'mtf 1/12/2011 19:34'!
cleanUp
"Remove terminated processes from my suspended list"

	self suspendedProcesses keys do: [:ea |
		ea isTerminated ifTrue: [self suspendedProcesses removeKey: ea]]! !

!ProcessBrowser class methodsFor: 'process control' stamp: 'bf 1/11/2013 15:14'!
debugProcess: aProcess
	aProcess debugWithTitle: 'Interrupted from the Process Browser'.
! !

!ProcessBrowser class methodsFor: 'CPU utilization' stamp: 'nk 3/14/2001 08:59'!
dumpTallyOnTranscript: tally
	"tally is from ProcessorScheduler>>tallyCPUUsageFor:
	Dumps lines with percentage of time, hash of process, and a friendly name"

	tally sortedCounts do: [ :assoc | | procName |
		procName := (self nameAndRulesFor: assoc value) first.
		Transcript print: (((assoc key / tally size) * 100.0) roundTo: 1);
			nextPutAll: '%   ';
			print: assoc value identityHash; space;
			nextPutAll: procName;
			cr.
	].
	Transcript flush.! !

!ProcessBrowser class methodsFor: 'class initialization' stamp: 'sd 11/20/2005 21:28'!
initialize
	"ProcessBrowser initialize"
	Browsers ifNil: [ Browsers := WeakSet new ].
	SuspendedProcesses ifNil: [ SuspendedProcesses := IdentityDictionary new ].
	Smalltalk addToStartUpList: self.
	Smalltalk addToShutDownList: self.
	self registerInFlapsRegistry.
	self registerWellKnownProcesses! !

!ProcessBrowser class methodsFor: 'process control' stamp: 'dtl 2/27/2010 11:05'!
isUIProcess: aProcess
	^aProcess == Project uiProcess
! !

!ProcessBrowser class methodsFor: 'process control' stamp: 'dtl 2/5/2005 01:21'!
nameAndRulesFor: aProcess 
	"Answer a nickname and two flags: allow-stop, and allow-debug"
	^ [aProcess caseOf: WellKnownProcesses
		 otherwise: 
			[(aProcess priority = Processor timingPriority
					and: [aProcess suspendedContext receiver == Delay])
				ifTrue: [{'the timer interrupt watcher'. false. false}]
				ifFalse: [{aProcess suspendedContext asString. true. true}]]]
		ifError: [:err :rcvr | {aProcess suspendedContext asString. true. true}]! !

!ProcessBrowser class methodsFor: 'instance creation' stamp: 'ar 8/7/2009 22:23'!
open
	"ProcessBrowser open"
	"Create and schedule a ProcessBrowser."
	Smalltalk garbageCollect.
	^ToolBuilder open: self new! !

!ProcessBrowser class methodsFor: 'instance creation' stamp: 'fbs 3/8/2013 08:33'!
prototypicalToolWindow
	"Answer a window representing a prototypical instance of the receiver"

	^ ToolBuilder build: self new! !

!ProcessBrowser class methodsFor: 'class initialization' stamp: 'KR 8/12/2007 10:56'!
registerInFlapsRegistry
	"Register the receiver in the system's flaps registry"
	self environment
		at: #Flaps
		ifPresent: [:cl | 	cl registerQuad: {#ProcessBrowser. #prototypicalToolWindow.	'Processes' translatedNoop.			'A Process Browser shows you all the running processes' translatedNoop}
						forFlapNamed: 'Tools'.]! !

!ProcessBrowser class methodsFor: 'class initialization' stamp: 'mt 4/6/2016 17:51'!
registerWellKnownProcesses
	"Associate each well-known process with a nickname and two flags: allow-stop, and allow-debug.
	Additional processes may be added to this list as required"

	WellKnownProcesses := OrderedCollection new.
	self registerWellKnownProcess: []
		label: 'no process'
		allowStop: false
		allowDebug: false.
	self registerWellKnownProcess: [Smalltalk lowSpaceWatcherProcess]
		label: 'the low space watcher'
		allowStop: false
		allowDebug: false.
	self registerWellKnownProcess: [WeakArray runningFinalizationProcess]
		label: 'the WeakArray finalization process'
		allowStop: false
		allowDebug: false.
	self registerWellKnownProcess: [Processor activeProcess]
		label: 'the UI process'
		allowStop: false
		allowDebug: true.
	self registerWellKnownProcess: [Processor backgroundProcess]
		label: 'the idle process'
		allowStop: false
		allowDebug: false.
	self registerWellKnownProcess: [Sensor interruptWatcherProcess]
		label: 'the user interrupt watcher'
		allowStop: false
		allowDebug: false.
	self registerWellKnownProcess: [Sensor eventTicklerProcess]
		label: 'the event tickler'
		allowStop: false
		allowDebug: false.
	self registerWellKnownProcess: [Project uiProcess]
		label: 'the inactive Morphic UI process'
		allowStop: false
		allowDebug: false.
"	self registerWellKnownProcess:
			[Smalltalk at: #SoundPlayer ifPresent: [:sp | sp playerProcess]]
		label: 'the Sound Player'
		allowStop: false
		allowDebug: false."
	self registerWellKnownProcess:
			[ScheduledControllers ifNotNil: [ScheduledControllers activeControllerProcess]]
		label: 'the inactive MVC controller process'
		allowStop: false
		allowDebug: true.
	self registerWellKnownProcess:
			[Smalltalk at: #CPUWatcher ifPresent: [:cw | cw currentWatcherProcess]]
		label: 'the CPUWatcher'
		allowStop: false
		allowDebug: false
! !

!ProcessBrowser class methodsFor: 'process control' stamp: 'dtl 2/5/2005 09:07'!
registerWellKnownProcess: aBlockForProcess label: nickname allowStop: allowStop allowDebug: allowDebug
	"Add an entry to the registry of well known processes. aBlockForProcess
	evaluates to a known process to be identified by nickname, and allowStop
	and allowDebug are flags controlling allowable actions for this process
	in the browser."

	WellKnownProcesses add: aBlockForProcess->[{nickname . allowStop . allowDebug}]! !

!ProcessBrowser class methodsFor: 'process control' stamp: 'nk 2/12/2002 10:09'!
resumeProcess: aProcess
	| priority |
	priority := self suspendedProcesses
				removeKey: aProcess
				ifAbsent: [aProcess priority].
	aProcess priority: priority.
	aProcess suspendedContext ifNotNil: [ aProcess resume ]
! !

!ProcessBrowser class methodsFor: 'process control' stamp: 'nk 3/8/2001 17:07'!
setProcess: aProcess toPriority: priority
	| oldPriority |
	oldPriority := self suspendedProcesses at: aProcess ifAbsent: [ ].
	oldPriority ifNotNil: [ self suspendedProcesses at: aProcess put: priority ].
	aProcess priority: priority.
	^oldPriority! !

!ProcessBrowser class methodsFor: 'system startup' stamp: 'nk 6/18/2003 07:32'!
shutDown
	Browsers do: [ :ea | ea isAutoUpdating ifTrue: [ ea pauseAutoUpdate ]]! !

!ProcessBrowser class methodsFor: 'system startup' stamp: 'dtl 8/27/2012 18:44'!
startUp
	Browsers
		do: [:ea | ea isAutoUpdatingPaused
				ifTrue: [ea initialize; startAutoUpdate]
				ifFalse: [ea initialize]]! !

!ProcessBrowser class methodsFor: 'process control' stamp: 'nk 10/29/2000 08:55'!
suspendedProcesses
	"Answer a collection of processes that my instances have suspended.  
	This is so that they don't get garbage collected."
	^ SuspendedProcesses
		ifNil: [SuspendedProcesses := IdentityDictionary new]! !

!ProcessBrowser class methodsFor: 'process control' stamp: 'dew 9/16/2001 01:53'!
suspendProcess: aProcess
	| priority |
	priority := aProcess priority.
	self suspendedProcesses at: aProcess put: priority.
	"Need to take the priority down below the caller's
	so that it can keep control after signaling the Semaphore"
	(aProcess suspendingList isKindOf: Semaphore)
		ifTrue: [aProcess priority: Processor lowestPriority.
			aProcess suspendingList signal].
	[aProcess suspend]
		on: Error
		do: [:ex | self suspendedProcesses removeKey: aProcess].
	aProcess priority: priority.
! !

!ProcessBrowser class methodsFor: 'CPU utilization' stamp: 'nk 3/8/2001 12:49'!
tallyCPUUsageFor: seconds
	"Compute CPU usage using a 10-msec sample for the given number of seconds,
	then dump the usage statistics on the Transcript. The UI is free to continue, meanwhile"
	"ProcessBrowser tallyCPUUsageFor: 10"
	^self tallyCPUUsageFor: seconds every: 10! !

!ProcessBrowser class methodsFor: 'CPU utilization' stamp: 'dtl 9/23/2009 20:36'!
tallyCPUUsageFor: seconds every: msec
	"Compute CPU usage using a msec millisecond sample for the given number of seconds,
	then dump the usage statistics on the Transcript. The UI is free to continue, meanwhile"
	"ProcessBrowser tallyCPUUsageFor: 10 every: 100"

	| promise |
	promise := Processor tallyCPUUsageFor: seconds every: msec.

	[ | tally |
		tally := promise value.
		Smalltalk isMorphic
			ifTrue: [ Project current addDeferredUIMessage: [ self dumpTallyOnTranscript: tally ] ]
			ifFalse: [ [ Transcript open ] forkAt: Processor userSchedulingPriority.
					[ (Delay forSeconds: 1) wait.
					self dumpTallyOnTranscript: tally ] forkAt: Processor userSchedulingPriority.]
	] fork.! !

!ProcessBrowser class methodsFor: 'process control' stamp: 'nk 3/8/2001 13:25'!
terminateProcess: aProcess
	aProcess ifNotNil: [
		self suspendedProcesses
			removeKey: aProcess
			ifAbsent: [].
		aProcess terminate
	].
! !

!ProcessBrowser class methodsFor: 'class initialization' stamp: 'asm 4/11/2003 12:39'!
unload
	"Unload the receiver from global registries"

	self environment at: #Flaps ifPresent: [:cl |
	cl unregisterQuadsWithReceiver: self] ! !

!ProcessBrowser class methodsFor: 'process control' stamp: 'sd 11/20/2005 21:28'!
unregisterWellKnownProcess: aProcess
	"Remove the first registry entry that matches aProcess. Use
	with caution if more than one registry entry may match aProcess."

	"self unregisterWellKnownProcess: Smalltalk lowSpaceWatcherProcess"

	| entry |
	entry := WellKnownProcesses
		detect: [:e | e key value == aProcess]
		ifNone: [^ self].
	WellKnownProcesses remove: entry! !

!ProcessBrowser class methodsFor: 'process control' stamp: 'nk 3/8/2001 13:26'!
wasProcessSuspendedByProcessBrowser: aProcess
	^self suspendedProcesses includesKey: aProcess! !

!ProcessBrowser methodsFor: 'stack list' stamp: 'mt 1/22/2016 13:12'!
browseContext
	selectedContext
		ifNil: [^ self].
	ToolSet browse: self selectedClass selector: self selectedSelector! !

!ProcessBrowser methodsFor: 'toolbuilder' stamp: 'ar 2/11/2005 20:38'!
buildWith: builder
	"Create a pluggable version of me, answer a window"
	| windowSpec listSpec textSpec |
	windowSpec := builder pluggableWindowSpec new.
	windowSpec model: self.
	windowSpec label: 'Process Browser'.
	windowSpec children: OrderedCollection new.

	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #processNameList; 
		getIndex: #processListIndex; 
		setIndex: #processListIndex:; 
		menu: #processListMenu:; 
		keyPress: #processListKey:from:;
		frame: (0 @ 0 extent: 0.5 @ 0.5).
	windowSpec children add: listSpec.

	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #stackNameList; 
		getIndex: #stackListIndex; 
		setIndex: #stackListIndex:; 
		menu: #stackListMenu:; 
		keyPress: #stackListKey:from:;
		frame: (0.5 @ 0.0 extent: 0.5 @ 0.5).
	windowSpec children add: listSpec.

	textSpec := builder pluggableTextSpec new.
	textSpec 
		model: self;
		getText: #selectedMethod; 
		setText: nil; 
		selection: nil; 
		menu: nil;
		frame: (0 @ 0.5 corner: 1 @ 1).
	windowSpec children add: textSpec.

	^builder build: windowSpec! !

!ProcessBrowser methodsFor: 'process actions' stamp: 'rbb 3/1/2005 11:07'!
changePriority
	| str newPriority nameAndRules |
	nameAndRules := self nameAndRulesForSelectedProcess.
	nameAndRules third
		ifFalse: [self inform: 'Nope, won''t change priority of ' , nameAndRules first.
			^ self].
	str := UIManager default 
				request: 'New priority' 
		  initialAnswer: selectedProcess priority asString.
	newPriority := str asNumber asInteger.
	newPriority
		ifNil: [^ self].
	(newPriority < 1
			or: [newPriority > Processor highestPriority])
		ifTrue: [self inform: 'Bad priority'.
			^ self].
	self class setProcess: selectedProcess toPriority: newPriority.
	self updateProcessList! !

!ProcessBrowser methodsFor: 'stack list' stamp: 'RAA 10/27/2000 15:21'!
changeStackListTo: aCollection 

        stackList := aCollection.
        self changed: #stackNameList.
        self stackListIndex: 0! !

!ProcessBrowser methodsFor: 'process actions' stamp: 'nk 10/29/2000 10:18'!
chasePointers
	| saved |
	selectedProcess
		ifNil: [^ self].
	saved := selectedProcess.
	[selectedProcess := nil.
	(Smalltalk includesKey: #PointerFinder)
		ifTrue: [PointerFinder on: saved]
		ifFalse: [self inspectPointers]]
		ensure: [selectedProcess := saved]! !

!ProcessBrowser methodsFor: 'process actions' stamp: 'sd 11/20/2005 21:27'!
debugProcess
	| nameAndRules |
	nameAndRules := self nameAndRulesForSelectedProcess.
	nameAndRules third
		ifFalse: [self inform: 'Nope, won''t debug ' , nameAndRules first.
			^ self].
	self class debugProcess: selectedProcess.! !

!ProcessBrowser methodsFor: 'stack list' stamp: 'nk 10/27/2000 09:28'!
exploreContext
	selectedContext explore! !

!ProcessBrowser methodsFor: 'process list' stamp: 'nk 10/27/2000 09:24'!
exploreProcess
	selectedProcess explore! !

!ProcessBrowser methodsFor: 'stack list' stamp: 'nk 10/27/2000 09:41'!
exploreReceiver
	selectedContext ifNotNil: [ selectedContext receiver explore ]! !

!ProcessBrowser methodsFor: 'process list' stamp: 'rbb 3/1/2005 11:08'!
findContext
	| initialProcessIndex initialStackIndex found |
	initialProcessIndex := self processListIndex.
	initialStackIndex := self stackListIndex.
	searchString := UIManager default 
			request: 'Enter a string to search for in the process stack lists'
	  initialAnswer: searchString.
	searchString isEmpty
		ifTrue: [^ false].
	self processListIndex: 1.
	self stackListIndex: 1.
	found := self nextContext.
	found
		ifFalse: [self processListIndex: initialProcessIndex.
			self stackListIndex: initialStackIndex].
	^ found! !

!ProcessBrowser methodsFor: 'views' stamp: 'nk 10/28/2000 11:44'!
hasView
	^self dependents isEmptyOrNil not! !

!ProcessBrowser methodsFor: 'initialize-release' stamp: 'fbs 12/31/2013 14:03'!
initialize
	methodText := ''.
	stackListIndex := 0.
	searchString := ''.
	lastUpdate := 0.
	startedCPUWatcher := CPUWatcher cpuWatcherEnabled and: [ self startCPUWatcher ].
	self updateProcessList; processListIndex: 1.
	Browsers add: self! !

!ProcessBrowser methodsFor: 'stack list' stamp: 'nk 7/8/2000 20:23'!
inspectContext
	selectedContext inspect! !

!ProcessBrowser methodsFor: 'process actions' stamp: 'nk 2/22/2005 15:26'!
inspectPointers
	| tc pointers |
	selectedProcess ifNil: [^self].
	tc := thisContext.
	pointers := PointerFinder pointersTo: selectedProcess
				except: { 
						self processList.
						tc.
						self}.
	pointers isEmpty ifTrue: [^self].
	OrderedCollectionInspector 
		openOn: pointers
		withEvalPane: false
		withLabel: 'Objects pointing to ' , selectedProcess browserPrintString! !

!ProcessBrowser methodsFor: 'process list' stamp: ''!
inspectProcess
	selectedProcess inspect! !

!ProcessBrowser methodsFor: 'stack list' stamp: 'nk 10/27/2000 09:41'!
inspectReceiver
	selectedContext
		ifNotNil: [selectedContext receiver inspect]! !

!ProcessBrowser methodsFor: 'updating' stamp: 'nk 10/28/2000 21:48'!
isAutoUpdating
	^autoUpdateProcess notNil and: [ autoUpdateProcess isSuspended  not ]! !

!ProcessBrowser methodsFor: 'updating' stamp: 'nk 6/18/2003 07:20'!
isAutoUpdatingPaused
	^autoUpdateProcess notNil and: [ autoUpdateProcess isSuspended ]! !

!ProcessBrowser methodsFor: 'stack list' stamp: 'rbb 3/1/2005 11:08'!
messageTally
	| secString secs |
	secString := UIManager default request: 'Profile for how many seconds?' initialAnswer: '4'.
	secs := secString asNumber asInteger.
	(secs isNil
			or: [secs isZero])
		ifTrue: [^ self].
	[ TimeProfileBrowser spyOnProcess: selectedProcess forMilliseconds: secs * 1000 ] forkAt: selectedProcess priority + 1.! !

!ProcessBrowser methodsFor: 'stack list' stamp: 'nk 10/28/2000 12:13'!
moreStack
	self updateStackList: 2000! !

!ProcessBrowser methodsFor: 'process actions' stamp: 'nk 10/28/2000 20:31'!
nameAndRulesForSelectedProcess
	"Answer a nickname and two flags: allow-stop, and allow-debug"
	^self nameAndRulesFor: selectedProcess! !

!ProcessBrowser methodsFor: 'process actions' stamp: 'nk 3/8/2001 13:35'!
nameAndRulesFor: aProcess 
	"Answer a nickname and two flags: allow-stop, and allow-debug"
	aProcess == autoUpdateProcess ifTrue: [ ^{'my auto-update process'. true. true} ].
	^self class nameAndRulesFor: aProcess 
! !

!ProcessBrowser methodsFor: 'process list' stamp: 'ul 8/14/2015 18:03'!
nextContext
	| initialProcessIndex initialStackIndex |
	searchString isEmpty ifTrue: [ ^false ].
	initialProcessIndex := self processListIndex.
	initialStackIndex := self stackListIndex.
	initialProcessIndex
		to: self processList size
		do: [:pi | self processListIndex: pi.
					self stackNameList
						withIndexDo: [:name :si | (pi ~= initialProcessIndex
											or: [si > initialStackIndex])
								ifTrue: [(name includesSubstring: searchString)
										ifTrue: [self stackListIndex: si.
											^true]]]].
	self processListIndex: initialProcessIndex.
	self stackListIndex: initialStackIndex.
	^ false! !

!ProcessBrowser methodsFor: 'process list' stamp: 'nk 10/28/2000 08:19'!
notify: errorString at: location in: aStream 
	"A syntax error happened when I was trying to highlight my pc. 
	Raise a signal so that it can be ignored."
	Warning signal: 'syntax error'! !

!ProcessBrowser methodsFor: 'updating' stamp: 'nk 6/18/2003 07:20'!
pauseAutoUpdate
	self isAutoUpdating
		ifTrue: [ autoUpdateProcess suspend ].
	self updateProcessList! !

!ProcessBrowser methodsFor: 'stack list' stamp: 'ul 12/12/2009 12:51'!
pcRange
	"Answer the indices in the source code for the method corresponding to  
	the selected context's program counter value."
	(selectedContext isNil or: [methodText isEmptyOrNil])
		ifTrue: [^ 1 to: 0].
	^selectedContext debuggerMap
		rangeForPC: (selectedContext pc ifNotNil: [:pc| pc] ifNil: [selectedContext method endPC])
		contextIsActiveContext: stackListIndex = 1! !

!ProcessBrowser methodsFor: 'message handling' stamp: 'nk 10/28/2000 20:53'!
perform: selector orSendTo: otherTarget 
	"Selector was just chosen from a menu by a user. If can respond, then  
	perform it on myself. If not, send it to otherTarget, presumably the  
	editPane from which the menu was invoked."
	(self respondsTo: selector)
		ifTrue: [^ self perform: selector]
		ifFalse: [^ super perform: selector orSendTo: otherTarget]! !

!ProcessBrowser methodsFor: 'process list' stamp: 'nk 6/30/2004 07:00'!
prettyNameForProcess: aProcess 
	| nameAndRules |
	aProcess ifNil: [ ^'<nil>' ].
	nameAndRules := self nameAndRulesFor: aProcess.
	^ aProcess browserPrintStringWith: nameAndRules first! !

!ProcessBrowser methodsFor: 'accessing' stamp: ''!
processList
	^ processList! !

!ProcessBrowser methodsFor: 'accessing' stamp: ''!
processListIndex
	^ processListIndex! !

!ProcessBrowser methodsFor: 'accessing' stamp: 'sd 11/20/2005 21:27'!
processListIndex: index 
	processListIndex := index.
	selectedProcess := processList
				at: index
				ifAbsent: [].
	self updateStackList.
	self changed: #processListIndex.! !

!ProcessBrowser methodsFor: 'process list' stamp: 'nk 10/29/2000 10:20'!
processListKey: aKey from: aView 
	^ aKey caseOf: {
		[$i] -> [self inspectProcess].
		[$I] -> [self exploreProcess].
		[$c] -> [self chasePointers].
		[$P] -> [self inspectPointers].
		[$t] -> [self terminateProcess].
		[$r] -> [self resumeProcess].
		[$s] -> [self suspendProcess].
		[$d] -> [self debugProcess].
		[$p] -> [self changePriority].
		[$m] -> [self messageTally].
		[$f] -> [self findContext].
		[$g] -> [self nextContext].
		[$a] -> [self toggleAutoUpdate].
		[$u] -> [self updateProcessList].
		[$S] -> [self signalSemaphore].
		[$k] -> [self moreStack]}
		 otherwise: [self arrowKey: aKey from: aView]! !

!ProcessBrowser methodsFor: 'process list' stamp: 'LC 1/7/2002 16:35'!
processListMenu: menu 
	| pw |

	selectedProcess
		ifNotNil: [| nameAndRules | 
			nameAndRules := self nameAndRulesForSelectedProcess.
			menu addList: {{'inspect (i)'. #inspectProcess}. {'explore (I)'. #exploreProcess}. {'inspect Pointers (P)'. #inspectPointers}}.
	(Smalltalk includesKey: #PointerFinder)
		ifTrue: [ menu add: 'chase pointers (c)' action: #chasePointers.  ].
			nameAndRules second
				ifTrue: [menu add: 'terminate (t)' action: #terminateProcess.
					selectedProcess isSuspended
						ifTrue: [menu add: 'resume (r)' action: #resumeProcess]
						ifFalse: [menu add: 'suspend (s)' action: #suspendProcess]].
			nameAndRules third
				ifTrue: [menu addList: {{'change priority (p)'. #changePriority}. {'debug (d)'. #debugProcess}}].
			menu addList: {{'profile messages (m)'. #messageTally}}.
			(selectedProcess suspendingList isKindOf: Semaphore)
				ifTrue: [menu add: 'signal Semaphore (S)' action: #signalSemaphore].
			menu add: 'full stack (k)' action: #moreStack.
			menu addLine].

	menu addList: {{'find context... (f)'. #findContext}. {'find again (g)'. #nextContext}}.
	menu addLine.

	menu
		add: (self isAutoUpdating
				ifTrue: ['turn off auto-update (a)']
				ifFalse: ['turn on auto-update (a)'])
		action: #toggleAutoUpdate.
	menu add: 'update list (u)' action: #updateProcessList.

	pw := Smalltalk at: #CPUWatcher ifAbsent: [].
	pw ifNotNil: [
		menu addLine.
		pw isMonitoring
				ifTrue: [ menu add: 'stop CPUWatcher' action: #stopCPUWatcher ]
				ifFalse: [ menu add: 'start CPUWatcher' action: #startCPUWatcher  ]
	].

	^ menu! !

!ProcessBrowser methodsFor: 'process list' stamp: 'nk 6/21/2004 09:59'!
processNameList
	"since processList is a WeakArray, we have to strengthen the result"
	| pw tally |
	pw := Smalltalk at: #CPUWatcher ifAbsent: [ ].
	tally := pw ifNotNil: [ pw current ifNotNil: [ pw current tally ] ].
	^ (processList asOrderedCollection
		copyWithout: nil)
		collect: [:each | | percent |
			percent := tally
				ifNotNil: [ ((((tally occurrencesOf: each) * 100.0 / tally size) roundTo: 1)
						asString padded: #left to: 2 with: $ ), '% '  ]
				ifNil: [ '' ].
			percent, (self prettyNameForProcess: each)
		] ! !

!ProcessBrowser methodsFor: 'process actions' stamp: 'nk 3/8/2001 13:23'!
resumeProcess
	selectedProcess
		ifNil: [^ self].
	self class resumeProcess: selectedProcess.
	self updateProcessList! !

!ProcessBrowser methodsFor: 'accessing' stamp: 'md 2/17/2006 09:32'!
selectedClass
	"Answer the class in which the currently selected context's method was  
	found."
	^ selectedClass
		ifNil: [selectedClass := selectedContext receiver
				ifNil: [selectedSelector := selectedContext method selector.
					   selectedContext method methodClass]
				ifNotNil: [selectedContext methodClass]]! !

!ProcessBrowser methodsFor: 'accessing' stamp: 'nk 2/16/2001 13:39'!
selectedMethod
	^ methodText ifNil: [methodText := selectedContext
						ifNil: ['']
						ifNotNil: [| pcRange | 
							methodText := [ selectedContext sourceCode ]
								ifError: [ :err :rcvr | 'error getting method text' ].
							pcRange := self pcRange.
							methodText asText
								addAttribute: TextColor red
								from: pcRange first
								to: pcRange last;
								
								addAttribute: TextEmphasis bold
								from: pcRange first
								to: pcRange last]]! !

!ProcessBrowser methodsFor: 'accessing' stamp: 'md 2/17/2006 12:07'!
selectedSelector
	"Answer the class in which the currently selected context's method was  
	found."
	^ selectedSelector
		ifNil: [selectedSelector := selectedContext receiver
				ifNil: [selectedClass := selectedContext method methodClass
					   selectedContext method selector]
				ifNotNil: [selectedContext selector]]! !

!ProcessBrowser methodsFor: 'updating' stamp: 'nice 2/8/2010 09:04'!
setUpdateCallbackAfter: seconds 

	[(Delay forSeconds: seconds) wait.
	Project current addDeferredUIMessage: [self updateProcessList]] fork! !

!ProcessBrowser methodsFor: 'process actions' stamp: 'nk 10/29/2000 09:58'!
signalSemaphore
	(selectedProcess suspendingList isKindOf: Semaphore)
		ifFalse: [^ self].
	[selectedProcess suspendingList signal] fork.
	(Delay forMilliseconds: 300) wait.
	"Hate to make the UI wait, but it's convenient..."
	self updateProcessList! !

!ProcessBrowser methodsFor: 'accessing' stamp: ''!
stackList
	^ stackList! !

!ProcessBrowser methodsFor: 'accessing' stamp: ''!
stackListIndex
	^ stackListIndex! !

!ProcessBrowser methodsFor: 'accessing' stamp: 'eem 6/12/2008 12:41'!
stackListIndex: index 
	stackListIndex := index.
	selectedContext := (stackList notNil
						and: [index > 0]) ifTrue:
							[stackList at: index ifAbsent: []].
	selectedClass := nil.
	selectedSelector := nil.
	methodText := nil.
	self changed: #stackListIndex.
	self changed: #selectedMethod! !

!ProcessBrowser methodsFor: 'views' stamp: 'nk 10/28/2000 16:53'!
stackListKey: aKey from: aView 
	^ aKey caseOf: {
		[$c] -> [self inspectContext].
		[$C] -> [self exploreContext].
		[$i] -> [self inspectReceiver].
		[$I] -> [self exploreReceiver].
		[$b] -> [self browseContext]}
		 otherwise: [self arrowKey: aKey from: aView]! !

!ProcessBrowser methodsFor: 'stack list' stamp: 'nk 10/28/2000 16:53'!
stackListMenu: aMenu 
	| menu |
	selectedContext
		ifNil: [^ aMenu].
	menu := aMenu
				labels: 'inspect context (c)
explore context (C)
inspect receiver (i)
explore receiver (I)
browse (b)'
				lines: #(2 4 )
				selections: #(#inspectContext #exploreContext #inspectReceiver #exploreReceiver #browseContext ).
	^ menu! !

!ProcessBrowser methodsFor: 'stack list' stamp: 'nk 10/28/2000 16:18'!
stackNameList
	^ stackList
		ifNil: [#()]
		ifNotNil: [stackList
				collect: [:each | each asString]]! !

!ProcessBrowser methodsFor: 'updating' stamp: 'dtl 9/23/2009 20:36'!
startAutoUpdate
	self isAutoUpdatingPaused
		ifTrue: [^ autoUpdateProcess resume].
	self isAutoUpdating
		ifFalse: [autoUpdateProcess := [[self hasView]
						whileTrue: [(Delay forSeconds: 2) wait.
							Project current addDeferredUIMessage: [self updateProcessList]].
					autoUpdateProcess := nil] fork].
	self updateProcessList
! !

!ProcessBrowser methodsFor: 'initialize-release' stamp: 'nk 3/14/2001 09:26'!
startCPUWatcher
	"Answers whether I started the CPUWatcher"

	| pw |
	pw := Smalltalk at: #CPUWatcher ifAbsent: [ ^self ].
	pw ifNotNil: [
		pw isMonitoring ifFalse: [
			pw startMonitoringPeriod: 5 rate: 100 threshold: 0.85.
			self setUpdateCallbackAfter: 7.
			^true
		]
	].
	^false
! !

!ProcessBrowser methodsFor: 'updating' stamp: 'nk 6/18/2003 07:22'!
stopAutoUpdate
	autoUpdateProcess ifNotNil: [
		autoUpdateProcess terminate.
		autoUpdateProcess := nil].
	self updateProcessList! !

!ProcessBrowser methodsFor: 'initialize-release' stamp: 'nk 3/14/2001 09:26'!
stopCPUWatcher
	| pw |
	pw := Smalltalk at: #CPUWatcher ifAbsent: [ ^self ].
	pw ifNotNil: [
		pw stopMonitoring.
		self updateProcessList.
		startedCPUWatcher := false.	"so a manual restart won't be killed later"
	]
! !

!ProcessBrowser methodsFor: 'process actions' stamp: 'sd 11/20/2005 21:27'!
suspendProcess
	| nameAndRules |
	selectedProcess isSuspended
		ifTrue: [^ self].
	nameAndRules := self nameAndRulesForSelectedProcess.
	nameAndRules second
		ifFalse: [self inform: 'Nope, won''t suspend ' , nameAndRules first.
			^ self].
	self class suspendProcess: selectedProcess.
	self updateProcessList! !

!ProcessBrowser methodsFor: 'process actions' stamp: 'sd 11/20/2005 21:27'!
terminateProcess
	| nameAndRules |
	nameAndRules := self nameAndRulesForSelectedProcess.
	nameAndRules second
		ifFalse: [self inform: 'Nope, won''t kill ' , nameAndRules first.
			^ self].
	self class terminateProcess: selectedProcess.	
	self updateProcessList! !

!ProcessBrowser methodsFor: 'accessing' stamp: 'nk 10/28/2000 08:36'!
text
	^methodText! !

!ProcessBrowser methodsFor: 'updating' stamp: 'nk 10/28/2000 21:50'!
toggleAutoUpdate
	self isAutoUpdating
		ifTrue: [ self stopAutoUpdate ]
		ifFalse: [ self startAutoUpdate ].
! !

!ProcessBrowser methodsFor: 'process list' stamp: 'mt 8/12/2016 10:20'!
updateProcessList
	| oldSelectedProcess newIndex now |
	now := Time millisecondClockValue.
	now - lastUpdate < 500
		ifTrue: [^ self].
	"Don't update too fast"
	lastUpdate := now.
	oldSelectedProcess := selectedProcess.
	processList := selectedProcess := selectedSelector := nil.
	Smalltalk garbageCollectMost.
	"lose defunct processes"

	processList := Process allSubInstances
				reject: [:each | each isTerminated].
	processList := processList
				sorted: [:a :b | a priority >= b priority].
	processList := WeakArray withAll: processList.
	newIndex := processList
				indexOf: oldSelectedProcess
				ifAbsent: [0].
	self changed: #processNameList.
	self processListIndex: newIndex! !

!ProcessBrowser methodsFor: 'stack list' stamp: 'nk 7/8/2000 20:24'!
updateStackList
	self updateStackList: 20! !

!ProcessBrowser methodsFor: 'stack list' stamp: 'nk 10/28/2000 09:00'!
updateStackList: depth 
	| suspendedContext oldHighlight |
	selectedProcess
		ifNil: [^ self changeStackListTo: nil].
	(stackList notNil and: [ stackListIndex > 0 ])
		ifTrue: [oldHighlight := stackList at: stackListIndex].
	selectedProcess == Processor activeProcess
		ifTrue: [self
				changeStackListTo: (thisContext stackOfSize: depth)]
		ifFalse: [suspendedContext := selectedProcess suspendedContext.
			suspendedContext
				ifNil: [self changeStackListTo: nil]
				ifNotNil: [self
						changeStackListTo: (suspendedContext stackOfSize: depth)]].
	self
		stackListIndex: (stackList
				ifNil: [0]
				ifNotNil: [stackList indexOf: oldHighlight])! !

!ProcessBrowser methodsFor: 'process actions' stamp: 'nk 10/29/2000 08:56'!
wasProcessSuspendedByProcessBrowser: aProcess
	^self class suspendedProcesses includesKey: aProcess! !

!ProcessBrowser methodsFor: 'initialize-release' stamp: 'nk 3/14/2001 08:03'!
windowIsClosing
	startedCPUWatcher ifTrue: [ CPUWatcher stopMonitoring ]! !

!FloatArray methodsFor: '*Tools-Inspector' stamp: 'apb 7/14/2004 12:18'!
inspectorClass 
	"Answer the class of the inspector to be used on the receiver.  Called by inspect; 
	use basicInspect to get a normal (less useful) type of inspector."

	^OrderedCollectionInspector! !

!Boolean methodsFor: '*Tools-Debugger' stamp: ''!
canonicalArgumentName
	^ 'Boolean'! !

ObjectExplorerWrapper subclass: #PointerExplorerWrapper
	instanceVariableNames: 'weakOnly'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Explorer'!

!PointerExplorerWrapper commentStamp: 'avi 8/21/2004 19:58' prior: 0!
A subclass of ObjectExplorerWrapper for use with PointerExplorer.  #contents is overridden to work backwards: it returns wrappers for the objects pointing to item rather than for the objects that item points to.!

!PointerExplorerWrapper methodsFor: 'accessing' stamp: 'mt 2/3/2016 20:54'!
contents
	"Return the wrappers with the objects holding references to item. Eldest objects come first, weak only referencers are at the end and have parentheses around their identity hash."

	| objects weakOnlyReferences |
	objects := self object inboundPointersExcluding: { self. self item. model }.
	weakOnlyReferences := OrderedCollection new.
	objects removeAllSuchThat: [ :each |
		each class == self class 
			or: [ each class == PointerExplorer
			or: [ (each class == MethodContext
				and: [ each receiver class == PointerExplorer ] )
			or: [ (each pointsOnlyWeaklyTo: self object)
				ifTrue: [ weakOnlyReferences add: each. true ]
				ifFalse: [ false ] ] ] ] ].
	 ^(objects replace: [ :each |	
		self class with: each name: each identityHash asString model: self object ])
		addAll: (weakOnlyReferences replace: [ :each |
			(self class with: each name: '(', each identityHash asString, ')' model: self object)
				weakOnly: true;
				yourself ]);
		yourself! !

!PointerExplorerWrapper methodsFor: 'testing' stamp: 'ab 8/22/2003 18:39'!
hasContents
	^true! !

!PointerExplorerWrapper methodsFor: 'testing' stamp: 'ul 9/4/2012 01:50'!
isWeakOnly

	^weakOnly ifNil: [ false ] "This #ifNil: message is only here to avoid errors when updating an image with opened PointerExplorers."! !

!PointerExplorerWrapper methodsFor: 'accessing' stamp: 'mt 7/24/2016 10:06'!
preferredColor

	^self isWeakOnly
		ifTrue: [ Color gray ]
		ifFalse: [ super preferredColor ]! !

!PointerExplorerWrapper methodsFor: 'accessing' stamp: 'ul 9/3/2012 12:04'!
weakOnly: aBoolean

	weakOnly := aBoolean! !

PluggableTextMorph subclass: #BrowserCommentTextMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Browser'!

!BrowserCommentTextMorph commentStamp: '<historical>' prior: 0!
I am a PluggableTextMorph that knows enough to make myself invisible when necessary.!

!BrowserCommentTextMorph methodsFor: 'displaying' stamp: 'nk 2/15/2004 13:41'!
hideOrShowPane
	(self model editSelection == #editClass)
		ifTrue: [ self showPane ]
		ifFalse: [ self hidePane ]! !

!BrowserCommentTextMorph methodsFor: 'displaying' stamp: 'ul 12/12/2009 14:12'!
hidePane
	| win |
	self window ifNotNil: [:window | window removePaneSplitters].
	
	self lowerPane ifNotNil:
		[ :lp | 
		lp layoutFrame bottomFraction: self layoutFrame bottomFraction.
		lp layoutFrame bottomOffset: SystemWindow borderWidth negated].
	win := self window ifNil: [ ^self ].
	self delete.
	win updatePanesFromSubmorphs.
	win addPaneSplitters! !

!BrowserCommentTextMorph methodsFor: 'accessing' stamp: 'nk 2/15/2004 14:12'!
lowerPane
	"Answer the AlignmentMorph that I live beneath"
	^self valueOfProperty: #browserLowerPane! !

!BrowserCommentTextMorph methodsFor: 'updating' stamp: 'nice 1/1/2010 21:53'!
noteNewOwner: win
	super noteNewOwner: win.
	self setProperty: #browserWindow toValue: win.
	win ifNil: [ ^self ].
	win setProperty: #browserClassCommentPane toValue: self.
	self setProperty: #browserLowerPane
		toValue: (win submorphThat: [ :m |
			m isAlignmentMorph
			and: [ m layoutFrame bottomFraction notNil
			and: [ m layoutFrame bottomFraction >= self layoutFrame topFraction ]]]
		ifNone: [])! !

!BrowserCommentTextMorph methodsFor: 'displaying' stamp: 'ul 12/12/2009 14:12'!
showPane
	owner ifNil: [
		| win |
		win := self window ifNil: [ ^self ].
		win addMorph: self fullFrame: self layoutFrame.
		win updatePanesFromSubmorphs ].

	self lowerPane ifNotNil: [ :lp | lp layoutFrame bottomFraction: self layoutFrame topFraction ].
	
	self window ifNotNil: [:win | win addPaneSplitters]! !

!BrowserCommentTextMorph methodsFor: 'updating' stamp: 'nk 2/15/2004 13:42'!
update: anAspect
	super update: anAspect.
	anAspect == #editSelection ifFalse: [ ^self ].
	self hideOrShowPane! !

!BrowserCommentTextMorph methodsFor: 'accessing' stamp: 'nk 2/15/2004 14:07'!
window
	^self owner ifNil: [ self valueOfProperty: #browserWindow ].! !

!PluggableTextMorph methodsFor: '*Tools-multi-window support' stamp: 'eem 6/30/2010 14:23'!
canHaveUnacceptedEdits
	"Answer if the receiver is an object that can hold unaccepted edits (such as a text editor widget)"

	^true! !

!PluggableTextMorph methodsFor: '*Tools-multi-window support' stamp: 'eem 6/30/2010 14:56'!
unacceptedEditState
	^hasUnacceptedEdits ifTrue: [self text]! !

!PluggableTextMorph methodsFor: '*Tools-multi-window support' stamp: 'eem 6/30/2010 21:05'!
unacceptedEditState: stateOrNil
	stateOrNil
		ifNotNil:
			[self setText: stateOrNil.
			 hasUnacceptedEdits := true "setText: clears hasUnacceptedEdits, so must do this post setText:"]
		ifNil: [hasUnacceptedEdits := false]! !

!BlockClosure methodsFor: '*Tools' stamp: 'ul 10/9/2009 00:19'!
timeProfile

	^TimeProfileBrowser onBlock: self! !

Object subclass: #ChangesOrganizer
	instanceVariableNames: ''
	classVariableNames: 'ChangeSetNamesInRelease RecentUpdateMarker'
	poolDictionaries: ''
	category: 'Tools-Changes'!

!ChangesOrganizer commentStamp: 'cbr 7/27/2010 19:17' prior: 0!
I manage ChangeSets in the system. See the protocols on my class side.

For an example of what I can do, select the following line in a Workspace and print it.


ChangesOrganizer allChangeSetNames!

!ChangesOrganizer class methodsFor: 'enumerating' stamp: 'pk 2/6/2006 09:49'!
allChangeSetNames
	^ self allChangeSets collect: [:c | c name]! !

!ChangesOrganizer class methodsFor: 'enumerating' stamp: 'pk 2/6/2006 09:49'!
allChangeSets
	"Return the list of all current ChangeSets"

	^ChangeSet allChangeSets! !

!ChangesOrganizer class methodsFor: 'enumerating' stamp: 'pk 2/6/2006 09:49'!
allChangeSetsWithClass: class selector: selector
	class ifNil: [^ #()].
	^ self allChangeSets select: 
		[:cs | (cs atSelector: selector class: class) ~~ #none]! !

!ChangesOrganizer class methodsFor: 'services' stamp: 'pk 2/6/2006 09:49'!
assuredChangeSetNamed: aName
	"Answer a change set of the given name.  If one already exists, answer that, else create a new one and answer it."

	| existing |
	^ (existing := self changeSetNamed: aName)
		ifNotNil:
			[existing]
		ifNil:
			[self basicNewChangeSet: aName]! !

!ChangesOrganizer class methodsFor: 'adding' stamp: 'pk 2/6/2006 09:49'!
basicNewChangeSet: newName
	^ChangeSet basicNewChangeSet: newName! !

!ChangesOrganizer class methodsFor: 'class initialization' stamp: 'pk 2/6/2006 09:49'!
belongsInAdditions: aChangeSet
	"Answer whether a change set belongs in the Additions category, which is fed by all change sets that are neither numbered nor in the initial release"

	^ (((self belongsInProjectsInRelease: aChangeSet) or:
		[self belongsInNumbered: aChangeSet])) not! !

!ChangesOrganizer class methodsFor: 'class initialization' stamp: 'pk 2/6/2006 09:49'!
belongsInAll: aChangeSet
	"Answer whether a change set belongs in the All category"

	^ true ! !

!ChangesOrganizer class methodsFor: 'class initialization' stamp: 'pk 2/6/2006 09:49'!
belongsInMyInitials: aChangeSet
	"Answer whether a change set belongs in the MyInitials category. "

	^ aChangeSet name endsWith: ('-', Utilities authorInitials)! !

!ChangesOrganizer class methodsFor: 'class initialization' stamp: 'pk 10/17/2006 09:04'!
belongsInNumbered: aChangeSet
	"Answer whether a change set belongs in the Numbered category. "

	^  aChangeSet name startsWithDigit! !

!ChangesOrganizer class methodsFor: 'class initialization' stamp: 'pk 2/6/2006 09:49'!
belongsInProjectChangeSets: aChangeSet
	"Answer whether a change set belongs in the MyInitials category. "

	^ aChangeSet belongsToAProject! !

!ChangesOrganizer class methodsFor: 'class initialization' stamp: 'pk 2/6/2006 09:49'!
belongsInProjectsInRelease:  aChangeSet
	"Answer whether a change set belongs in the ProjectsInRelease category.  You can hand-tweak this to suit your working style.  This just covers the space of project names in the 2.9, 3.0, and 3.1a systems"

	| aString |
	^ ((aString := aChangeSet name) beginsWith: 'Play With Me') or: [self changeSetNamesInReleaseImage includes: aString]! !

!ChangesOrganizer class methodsFor: 'class initialization' stamp: 'pk 2/6/2006 09:49'!
belongsInRecentUpdates: aChangeSet
	"Answer whether a change set belongs in the RecentUpdates category."

	^ aChangeSet name startsWithDigit and:
			[aChangeSet name asInteger >= self recentUpdateMarker]! !

!ChangesOrganizer class methodsFor: 'services' stamp: 'pk 2/6/2006 09:49'!
buildAggregateChangeSet
	"Establish a change-set named Aggregate which bears the union of all the changes in all the existing change-sets in the system (other than any pre-existing Aggregate).  This can be useful when wishing to discover potential conflicts between a disk-resident change-set and an image.  Formerly very useful, now some of its unique contributions have been overtaken by new features"

	| aggregateChangeSet |
	aggregateChangeSet := self existingOrNewChangeSetNamed: 'Aggregate'.
	aggregateChangeSet clear.
	self allChangeSets do:
		[:aChangeSet | aChangeSet == aggregateChangeSet ifFalse:
			[aggregateChangeSet assimilateAllChangesFoundIn: aChangeSet]]

"ChangeSorter buildAggregateChangeSet"

	! !

!ChangesOrganizer class methodsFor: 'enumerating' stamp: 'pk 2/6/2006 09:49'!
changeSetNamed: aName
	"Return the change set of the given name, or nil if none found.  1/22/96 sw"
	^ChangeSet named: aName! !

!ChangesOrganizer class methodsFor: 'class initialization' stamp: 'pk 2/6/2006 09:49'!
changeSetNamesInReleaseImage
	"Answer a list of names of project change sets that come pre-shipped in the latest sytem release.  On the brink of shipping a new release, call 'ChangeSorter noteChangeSetsInRelease'  "

	^ ChangeSetNamesInRelease ifNil:
		[ChangeSetNamesInRelease := self changeSetNamesInThreeOh]! !

!ChangesOrganizer class methodsFor: 'class initialization' stamp: 'pk 2/6/2006 09:49'!
changeSetNamesInThreeOh
	"Hard-coded: answer a list of names of project change sets that came pre-shipped in Squeak 3.0"

	^ #('The Worlds of Squeak' 'Fun with Morphic' 'Games' 'Fun With Music' 'Building with Squeak' 'Squeak and the Internet' 'Squeak in 3D' 'More About Sound' ) ! !

!ChangesOrganizer class methodsFor: 'enumerating' stamp: 'pk 2/6/2006 09:49'!
changeSetsNamedSuchThat: nameBlock
	^ChangeSet changeSetsNamedSuchThat: nameBlock! !

!ChangesOrganizer class methodsFor: 'enumerating' stamp: 'pk 2/6/2006 09:49'!
changeSet: aChangeSet containsClass: aClass
	| theClass |
	theClass := Smalltalk classNamed: aClass.
	theClass ifNil: [^ false].
	^ aChangeSet containsClass: theClass! !

!ChangesOrganizer class methodsFor: 'services' stamp: 'pk 2/6/2006 09:49'!
countOfChangeSetsWithClass: aClass andSelector: aSelector
	"Answer how many change sets record a change for the given class and selector"

	^ (self allChangeSetsWithClass: aClass selector: aSelector) size! !

!ChangesOrganizer class methodsFor: 'removing' stamp: 'pk 2/6/2006 09:49'!
deleteChangeSetsNumberedLowerThan: anInteger
	"Delete all changes sets whose names start with integers smaller than anInteger"

	self removeChangeSetsNamedSuchThat:
		[:aName | aName first isDigit and: [aName initialIntegerOrNil < anInteger]].

	"ChangesOrganizer deleteChangeSetsNumberedLowerThan: (ChangeSorter highestNumberedChangeSet name initialIntegerOrNil - 500)"
! !

!ChangesOrganizer class methodsFor: 'services' stamp: 'pk 2/6/2006 09:49'!
doesAnyChangeSetHaveClass: aClass andSelector: aSelector
	"Answer whether any known change set bears a change for the given class and selector"

	^ (self countOfChangeSetsWithClass: aClass andSelector: aSelector) > 0! !

!ChangesOrganizer class methodsFor: 'enumerating' stamp: 'pk 2/6/2006 09:49'!
existingOrNewChangeSetNamed: aName
	^ChangeSet existingOrNewChangeSetNamed: aName! !

!ChangesOrganizer class methodsFor: 'utilities' stamp: 'nice 12/27/2009 03:12'!
fileOutChangeSetsNamed: nameList
	"File out the list of change sets whose names are provided"
     "ChangeSorter fileOutChangeSetsNamed: #('New Changes' 'miscTidies-sw')"

	| notFound empty infoString |
	notFound := OrderedCollection new.
	empty := OrderedCollection new.
	nameList do:
		[:aName | | aChangeSet | (aChangeSet := self changeSetNamed: aName)
			ifNotNil:
				[aChangeSet isEmpty
					ifTrue:
						[empty add: aName]
					ifFalse:
						[aChangeSet fileOut]]
			ifNil:
				[notFound add: aName]].

	infoString := (nameList size - notFound size) printString, ' change set(s) filed out'.
	notFound size > 0 ifTrue:
		[infoString := infoString, '

', notFound size printString, ' change set(s) not found:'.
		notFound do:
			[:aName | infoString := infoString, '
', aName]].
	empty size > 0 ifTrue:
		[infoString := infoString, '
', empty size printString, ' change set(s) were empty:'.
		empty do:
			[:aName | infoString := infoString, '
', aName]].

	self inform: infoString! !

!ChangesOrganizer class methodsFor: 'enumerating' stamp: 'pk 2/6/2006 09:49'!
gatherChangeSets		"ChangeSorter gatherChangeSets"
	^ChangeSet gatherChangeSets! !

!ChangesOrganizer class methodsFor: 'enumerating' stamp: 'pk 2/6/2006 09:49'!
highestNumberedChangeSet
	"ChangeSorter highestNumberedChangeSet"
	| aList |
	aList := (ChangeSet allChangeSetNames select: [:aString | aString startsWithDigit] thenCollect:
		[:aString | aString initialIntegerOrNil]).
	^ (aList size > 0)
		ifTrue:
			[aList max]
		ifFalse:
			[nil]
! !

!ChangesOrganizer class methodsFor: 'class initialization' stamp: 'ar 1/4/2010 02:02'!
initialize

	"Initialize the class variables"
	RecentUpdateMarker := 0.


! !

!ChangesOrganizer class methodsFor: 'enumerating' stamp: 'pk 2/6/2006 09:49'!
mostRecentChangeSetWithChangeForClass: class selector: selector
	| hits |
	hits := self allChangeSets select: 
		[:cs | (cs atSelector: selector class: class) ~~ #none].
	hits isEmpty ifTrue: [^ 'not in any change set'].
	^ 'recent cs: ', hits last name! !

!ChangesOrganizer class methodsFor: 'adding' stamp: 'pk 2/6/2006 09:49'!
newChangeSet
	"Prompt the user for a name, and establish a new change set of
	that name (if ok), making it the current changeset.  Return nil
	of not ok, else return the actual changeset."

	| newName newSet |
	newName := UIManager default
		request: 'Please name the new change set:'
		initialAnswer: ChangeSet defaultName.
	newName isEmptyOrNil ifTrue:
		[^ nil].
	newSet := self basicNewChangeSet: newName.
	newSet ifNotNil:
		[ChangeSet  newChanges: newSet].
	^ newSet! !

!ChangesOrganizer class methodsFor: 'adding' stamp: 'pk 2/6/2006 09:49'!
newChangeSet: aName
	"Makes a new change set called aName, add author initials to try to
	ensure a unique change set name."

	| newName |
	newName := aName , FileDirectory dot , Utilities authorInitials.
	^ self basicNewChangeSet: newName! !

!ChangesOrganizer class methodsFor: 'adding' stamp: 'pk 2/6/2006 09:49'!
newChangesFromStream: aStream named: aName
	^ChangeSet newChangesFromStream: aStream named: aName
! !

!ChangesOrganizer class methodsFor: 'class initialization' stamp: 'pk 2/6/2006 09:49'!
noteChangeSetsInRelease
	"Freshly compute what the change sets in the release are; to be called manually just before a release"

	ChangeSetNamesInRelease := (Project allProjects collect: [:p | p name]) asSet asOrderedCollection.

"ChangeSorter noteChangeSetsInRelease"! !

!ChangesOrganizer class methodsFor: 'enumerating' stamp: 'pk 2/6/2006 09:49'!
promoteToTop: aChangeSet
	"Make aChangeSet the first in the list from now on"
	^ChangeSet promoteToTop: aChangeSet! !

!ChangesOrganizer class methodsFor: 'services' stamp: 'pk 2/6/2006 09:49'!
recentUpdateMarker
	"Answer the number representing the threshold of what counts as 'recent' for an update number.  This allow you to use the RecentUpdates category in a ChangeSorter to advantage"

	^ RecentUpdateMarker ifNil: [RecentUpdateMarker := 0]! !

!ChangesOrganizer class methodsFor: 'services' stamp: 'pk 2/6/2006 09:49'!
recentUpdateMarker: aNumber
	"Set the recent update marker as indicated"

	^ RecentUpdateMarker := aNumber! !

!ChangesOrganizer class methodsFor: 'removing' stamp: 'pk 2/6/2006 09:49'!
removeChangeSetsNamedSuchThat: nameBlock
	(self changeSetsNamedSuchThat: nameBlock)
		do: [:cs | self removeChangeSet: cs]! !

!ChangesOrganizer class methodsFor: 'removing' stamp: 'pk 2/6/2006 09:49'!
removeChangeSet: aChangeSet
	"Remove the given changeSet.  Caller must assure that it's cool to do this"
	^ChangeSet removeChangeSet: aChangeSet! !

!ChangesOrganizer class methodsFor: 'removing' stamp: 'pk 2/6/2006 09:49'!
removeEmptyUnnamedChangeSets
	"Remove all change sets that are empty, whose names start with Unnamed,
		and which are not nailed down by belonging to a Project."
	"ChangeSorter removeEmptyUnnamedChangeSets"
	| toGo |
	(toGo := (self changeSetsNamedSuchThat: [:csName | csName beginsWith: 'Unnamed'])
		select: [:cs | cs isEmpty and: [cs okayToRemoveInforming: false]])
		do: [:cs | self removeChangeSet: cs].
	self inform: toGo size printString, ' change set(s) removed.'! !

!ChangesOrganizer class methodsFor: 'services' stamp: 'dtl 12/28/2009 22:55'!
reorderChangeSets
	"Change the order of the change sets to something more convenient:
		First come the project changesets that come with the release.  These are mostly empty.
		Next come all numbered updates.
		Next come all remaining changesets
	In a ChangeSorter, they will appear in the reversed order."

	"ChangeSorter reorderChangeSets"

	| newHead newMid newTail |
	newHead := OrderedCollection new.
	newMid := OrderedCollection new.
	newTail := OrderedCollection new.
	ChangeSet allChangeSets do:
		[:aChangeSet |
			(self belongsInProjectsInRelease: aChangeSet)
				ifTrue:
					[newHead add: aChangeSet]
				ifFalse:
					[(self belongsInNumbered: aChangeSet)
						ifTrue:
							[newMid add: aChangeSet]
						ifFalse:
							[newTail add: aChangeSet]]].
	ChangeSet allChangeSets: newHead, newMid, newTail.
	Project current wakeUpTopWindow! !

!ChangesOrganizer class methodsFor: 'services' stamp: 'pk 2/6/2006 09:49'!
secondaryChangeSet
	^ChangeSet secondaryChangeSet! !

!ChangesOrganizer class methodsFor: 'class initialization' stamp: 'pk 2/6/2006 09:49'!
unload
	"Unload the receiver from global registries"

	self environment at: #FileList ifPresent: [:cl |
	cl unregisterFileReader: self].
	self environment at: #Flaps ifPresent: [:cl |
	cl unregisterQuadsWithReceiver: self] ! !

Object subclass: #DebuggerMethodMap
	instanceVariableNames: 'timestamp methodReference methodNode abstractSourceRanges sortedSourceMap'
	classVariableNames: 'AccessLock MapCache MapCacheEntries'
	poolDictionaries: ''
	category: 'Tools-Debugger'!

!DebuggerMethodMap commentStamp: '<historical>' prior: 0!
I am a place-holder for information needed by the Debugger to inspect method activations.  I insulate the debugger from details of code generation such as exact bytecode offsets and temporary variable locations.  I have two concreate subclasses, one for methods compiled using BlueBook blocks and one for methods compiled using Closures.  These classes deal with temporary variable access. My function is to abstract the source map away from actual bytecode pcs to abstract bytecode pcs.

To reduce compilation time I try and defer as much computation to access time as possible as instances of me will be created after each compilation.

I maintain a WeakIdentityDictionary of method to DebuggerMethodMap to cache maps.  I refer to my method through a WeakArray to keep the map cache functional. If the reference from a DebuggerMethodMap to its method were strong then the method would never be dropped from the cache because the reference from its map would keep it alive.!

DebuggerMethodMap subclass: #DebuggerMethodMapForBlueBookMethods
	instanceVariableNames: 'tempNames'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Debugger'!

!DebuggerMethodMapForBlueBookMethods commentStamp: '<historical>' prior: 0!
I am a place-holder for information needed by the Debugger to inspect method activations.  See my superclass's comment. I map methods compiled using Closures.!

!DebuggerMethodMapForBlueBookMethods methodsFor: 'initialize-release' stamp: 'eem 6/5/2008 10:34'!
forMethod: aMethod "<CompiledMethod>" methodNode: aMethodNode "<MethodNode>"
	super forMethod: aMethod methodNode: aMethodNode.
	tempNames := methodNode encoder tempNames! !

!DebuggerMethodMapForBlueBookMethods methodsFor: 'accessing' stamp: 'eem 6/3/2008 11:43'!
namedTempAt: index in: aContext
	"Answer the value of the temp at index in aContext where index is relative
	 to the array of temp names answered by tempNamesForContext:"
	^aContext tempAt: index! !

!DebuggerMethodMapForBlueBookMethods methodsFor: 'accessing' stamp: 'eem 6/3/2008 11:43'!
namedTempAt: index put: aValue in: aContext
	"Assign the value of the temp at index in aContext where index is relative
	 to the array of temp names answered by tempNamesForContext:"
	^aContext tempAt: index put: aValue! !

!DebuggerMethodMapForBlueBookMethods methodsFor: 'accessing' stamp: 'eem 6/3/2008 11:42'!
tempNamesForContext: aContext
	"Answer an Array of all the temp names in scope in aContext starting with
	 the home's first local (the first argument or first temporary if no arguments)."
	^tempNames! !

DebuggerMethodMap subclass: #DebuggerMethodMapForClosureCompiledMethods
	instanceVariableNames: 'blockExtentsToTempRefs startpcsToTempRefs'
	classVariableNames: 'FirstTime'
	poolDictionaries: ''
	category: 'Tools-Debugger'!

!DebuggerMethodMapForClosureCompiledMethods commentStamp: '<historical>' prior: 0!
I am a place-holder for information needed by the Debugger to inspect method activations.  See my superclass's comment. I map methods compiled using BlueBook blocks.

Instance variables
	blockExtentsToTempsRefs <Dictionary of: Interval -> Array of: (Array with: String with: (Integer | (Array with: Integer with: Integer)))>
		maps a block extent to an Array of temp references for that block/method.
		Each reference is a pair of temp name and index, where the index can itself be a pair for a remote temp.
	startpcsToTempRefs <Dictionary of: Integer -> Array of: (Array with: String with: temp reference)> where
		temp reference ::= Integer
						| (Array with: Integer with: Integer)
						| (Array with: #outer with: temp reference)!

!DebuggerMethodMapForClosureCompiledMethods methodsFor: 'private' stamp: 'eem 7/29/2008 20:09'!
ensureExtentsMapsInitialized
	| encoderTempRefs "<Dictionary of: Interval -> <Array of: <String | <Array of: String>>>>" |
	blockExtentsToTempRefs ifNotNil: [^self].
	blockExtentsToTempRefs := Dictionary new.
	startpcsToTempRefs := Dictionary new.
	encoderTempRefs := methodNode blockExtentsToTempRefs.
	encoderTempRefs keysAndValuesDo:
		[:blockExtent :tempVector|
		blockExtentsToTempRefs
			at: blockExtent
			put: (Array streamContents:
					[:stream|
					tempVector withIndexDo:
						[:nameOrSequence :index|
						nameOrSequence isString
							ifTrue:
								[stream nextPut: {nameOrSequence. index}]
							ifFalse:
								[nameOrSequence withIndexDo:
									[:name :indirectIndex|
									stream nextPut: { name. { index. indirectIndex }}]]]])]! !

!DebuggerMethodMapForClosureCompiledMethods methodsFor: 'accessing' stamp: 'eem 7/29/2008 19:28'!
namedTempAt: index in: aContext
	"Answer the value of the temp at index in aContext where index is relative
	 to the array of temp names answered by tempNamesForContext:"
	^self
		privateTempAt: index
		in: aContext
		startpcsToBlockExtents: aContext method startpcsToBlockExtents! !

!DebuggerMethodMapForClosureCompiledMethods methodsFor: 'accessing' stamp: 'eem 7/29/2008 19:33'!
namedTempAt: index put: aValue in: aContext
	"Assign the value of the temp at index in aContext where index is relative
	 to the array of temp names answered by tempNamesForContext:.
	 If the value is a copied value we also need to set it along the lexical chain."
	^self
		privateTempAt: index
		in: aContext
		put: aValue
		startpcsToBlockExtents: aContext method startpcsToBlockExtents! !

!DebuggerMethodMapForClosureCompiledMethods methodsFor: 'private' stamp: 'eem 4/16/2012 13:45'!
privateDereference: tempReference in: aContext
	"Fetch the temporary with reference tempReference in aContext.
	 tempReference can be
		integer - direct temp reference
		#( indirectionVectorIndex tempIndex ) - remote temp in indirectionVector at index
		#( outer. temp reference ) - a temp reference in an outer context."
	^tempReference isInteger
		ifTrue:
			[tempReference <= aContext stackPtr ifTrue:
				[aContext tempAt: tempReference]]
		ifFalse:
			[tempReference first == #outer
				ifTrue:
					[self privateDereference: tempReference last
						in: aContext outerContext]
				ifFalse: "If stopped before indirection vectors are created they will be nil. Simply answer nil"
					[tempReference first <= aContext stackPtr ifTrue:
						[(aContext tempAt: tempReference first) ifNotNil:
							[:indirectionVector|
							indirectionVector at: tempReference second]]]]! !

!DebuggerMethodMapForClosureCompiledMethods methodsFor: 'private' stamp: 'eem 3/13/2013 14:53'!
privateDereference: tempReference in: aContext put: aValue
	"Assign the temporary with reference tempReference in aContext.
	 tempReference can be
		integer - direct temp reference
		#( indirectionVectorIndex tempIndex ) - remote temp in indirectionVector at index
		#( outer. temp reference ) - a temp reference in an outer context."
	^tempReference isInteger
		ifTrue:
			[tempReference <= aContext stackPtr
				ifTrue:
					[aContext tempAt: tempReference put: aValue]
				ifFalse:
					[UIManager default inform: 'Cannot assign temp because it is no longer on stack.\Activation has returned?' withCRs.
							nil]]
		ifFalse:
			[tempReference first == #outer
				ifTrue:
					[self privateDereference: tempReference last
						in: aContext outerContext
						put: aValue]
				ifFalse: "If stopped before indirection vectors are created they will be nil."
					[tempReference first <= aContext stackPtr
						ifTrue:
							[(aContext tempAt: tempReference first)
								ifNil: [UIManager default inform: 'Cannot assign remote temp because indirection vector is nil.\Too early in method execution?' withCRs.
									nil]
								ifNotNil:
									[:indirectionVector|
									indirectionVector
										at: tempReference second
										put: aValue]]
						ifFalse:
							[UIManager default inform: 'Cannot assign remote temp because it is no longer on stack.\Activation has returned?' withCRs.
							nil]]]! !

!DebuggerMethodMapForClosureCompiledMethods methodsFor: 'private' stamp: 'eem 7/29/2008 20:03'!
privateTempAt: index in: aContext put: aValue startpcsToBlockExtents: theContextsStartpcsToBlockExtents
	| nameRefPair |
	nameRefPair := (self privateTempRefsForContext: aContext
						 startpcsToBlockExtents: theContextsStartpcsToBlockExtents)
						at: index
						ifAbsent: [aContext errorSubscriptBounds: index].
	^self privateDereference: nameRefPair last in: aContext put: aValue! !

!DebuggerMethodMapForClosureCompiledMethods methodsFor: 'private' stamp: 'eem 7/29/2008 20:02'!
privateTempAt: index in: aContext startpcsToBlockExtents: theContextsStartpcsToBlockExtents
	| nameRefPair |
	nameRefPair := (self privateTempRefsForContext: aContext
						 startpcsToBlockExtents: theContextsStartpcsToBlockExtents)
						at: index
						ifAbsent: [aContext errorSubscriptBounds: index].
	^self privateDereference: nameRefPair last in: aContext! !

!DebuggerMethodMapForClosureCompiledMethods methodsFor: 'private' stamp: 'eem 9/14/2011 14:50'!
privateTempRefsForContext: aContext startpcsToBlockExtents: theContextsStartpcsToBlockExtents
	"Answer the sequence of temps in scope in aContext in the natural order,
	 outermost arguments and temporaries first, innermost last.  Each temp is
	 a pair of the temp's name followed by a reference.  The reference can be
		integer - index of temp in aContext
		#( indirectionVectorIndex tempIndex ) - remote temp in indirectionVector at index in aContext
		#( outer. temp reference ) - a temp reference in an outer context."
	blockExtentsToTempRefs ifNil:
		[blockExtentsToTempRefs := (aContext method holdsTempNames
										ifTrue: [aContext method]
										ifFalse: [methodNode]) blockExtentsToTempsMap.
		 blockExtentsToTempRefs ifNil:
			["an empty method.  shouldn't be able to step into here but it
			  can happen in weird circumstances (i.e. with MethodWrapper)."
			blockExtentsToTempRefs := Dictionary new.
			blockExtentsToTempRefs
				at: (theContextsStartpcsToBlockExtents at: aContext startpc)
				put: {}].
		 startpcsToTempRefs := Dictionary new].
	^startpcsToTempRefs
		at: aContext startpc
		ifAbsentPut:
			[| localRefs |
			 localRefs := blockExtentsToTempRefs at: (theContextsStartpcsToBlockExtents at: aContext startpc).
			 aContext outerContext
				ifNil: [localRefs]
				ifNotNil:
					[:outer| | outerTemps |
					"Present temps in the order outermost to innermost left-to-right, but replace
					 copied outermost temps with their innermost copies"
					 outerTemps := (self
										privateTempRefsForContext: outer
										startpcsToBlockExtents: theContextsStartpcsToBlockExtents) collect:
						[:outerPair|
						localRefs
							detect: [:localPair| outerPair first = localPair first]
							ifNone: [{ outerPair first. { #outer. outerPair last } }]].
					outerTemps,
					 (localRefs reject: [:localPair| outerTemps anySatisfy: [:outerPair| localPair first = outerPair first]])]]! !

!DebuggerMethodMapForClosureCompiledMethods methodsFor: 'accessing' stamp: 'eem 7/29/2008 18:26'!
tempNamesForContext: aContext
	"Answer an Array of all the temp names in scope in aContext starting with
	 the home's first local (the first argument or first temporary if no arguments)."
	^(self
		privateTempRefsForContext: aContext
		startpcsToBlockExtents: aContext method startpcsToBlockExtents) collect:
			[:pair| pair first]! !

!DebuggerMethodMap class methodsFor: 'debugger support' stamp: 'ul 11/15/2010 08:44'!
cacheDebugMap: aDebuggerMethodMap forMethod: aCompiledMethod
	
	^self protected: [ 
		MapCache size >= MapCacheEntries ifTrue: [
			MapCache slowSize >= MapCacheEntries 
				ifFalse: [ MapCache rehash ]
				ifTrue: [
					| mapsByAge |
					mapsByAge := MapCache keys sort: [ :m1 :m2 |
						"We are holding strongly on the keys, so #at: is suitable."
						(MapCache at: m1) timestamp < (MapCache at: m2) timestamp].
					mapsByAge from: 1 to: mapsByAge size - MapCacheEntries do: [ :each |
						MapCache removeKey: each ] ] ].
		MapCache
			at: aCompiledMethod
			put: aDebuggerMethodMap ]! !

!DebuggerMethodMap class methodsFor: 'class initialization' stamp: 'mtf 9/19/2010 19:50'!
cleanUp
	self initialize! !

!DebuggerMethodMap class methodsFor: 'instance creation' stamp: 'ul 11/15/2010 08:39'!
forMethod: aMethod "<CompiledMethod>"
	"Answer a DebuggerMethodMap suitable for debugging activations of aMethod.
	 Answer an existing instance from the cache if it exists, cacheing a new one if required."
	
	^self protected: [ 
		MapCache
			at: aMethod
			ifAbsent: [self
						cacheDebugMap:
							(self
								forMethod: aMethod
								methodNode: aMethod methodNode)
						forMethod: aMethod] ]! !

!DebuggerMethodMap class methodsFor: 'instance creation' stamp: 'eem 7/29/2008 16:54'!
forMethod: aMethod "<CompiledMethod>" methodNode: methodNode "<MethodNode>"
	"Uncached instance creation method for private use or for tests.
	 Please consider using forMethod: instead."
	^(aMethod isBlueBookCompiled
			ifTrue: [DebuggerMethodMapForBlueBookMethods]
			ifFalse: [DebuggerMethodMapForClosureCompiledMethods]) new
		forMethod: aMethod
		methodNode: methodNode! !

!DebuggerMethodMap class methodsFor: 'class initialization' stamp: 'eem 6/5/2008 09:14'!
initialize
	"DebuggerMethodMap initialize"

	self voidMapCache! !

!DebuggerMethodMap class methodsFor: 'synchronization' stamp: 'ul 11/15/2010 08:38'!
protected: aBlock

	^(AccessLock ifNil: [ AccessLock := Mutex new ]) critical: aBlock! !

!DebuggerMethodMap class methodsFor: 'class initialization' stamp: 'ul 11/15/2010 08:38'!
voidMapCache

	self protected: [ 
		MapCache := WeakIdentityKeyDictionary new.
		MapCacheEntries := 16 ]! !

!DebuggerMethodMap methodsFor: 'source mapping' stamp: 'eem 7/29/2008 17:12'!
abstractSourceMap
	"Answer with a Dictionary of abstractPC <Integer> to sourceRange <Interval>."
	| theMethodToScan rawSourceRanges concreteSourceRanges abstractPC scanner client |
	abstractSourceRanges ifNotNil:
		[^abstractSourceRanges].
	"If the methodNode hasn't had a method generated it doesn't have pcs set in its
	 nodes so we must generate a new method and might as well use it for scanning."
	methodNode rawSourceRangesAndMethodDo:
		[:ranges :method|
		 rawSourceRanges := ranges.
		 theMethodToScan := method].
	concreteSourceRanges := Dictionary new.
	rawSourceRanges keysAndValuesDo:
		[:node :range|
		node pc ~= 0 ifTrue:
			[concreteSourceRanges at: node pc put: range]].
	abstractPC := 1.
	abstractSourceRanges := Dictionary new.
	scanner := InstructionStream on: theMethodToScan.
	client := InstructionClient new.
	[(concreteSourceRanges includesKey: scanner pc) ifTrue:
		[abstractSourceRanges at: abstractPC put: (concreteSourceRanges at: scanner pc)].
	 abstractPC := abstractPC + 1.
	 scanner interpretNextInstructionFor: client.
	 scanner atEnd] whileFalse.
	^abstractSourceRanges! !

!DebuggerMethodMap methodsFor: 'initialize-release' stamp: 'eem 6/5/2008 09:21'!
forMethod: aMethod "<CompiledMethod>" methodNode: theMethodNode "<MethodNode>"
	methodReference := WeakArray with: aMethod.
	methodNode := theMethodNode.
	self markRecentlyUsed! !

!DebuggerMethodMap methodsFor: 'accessing' stamp: 'eem 6/3/2008 12:21'!
markRecentlyUsed
	timestamp := Time totalSeconds! !

!DebuggerMethodMap methodsFor: 'accessing' stamp: 'eem 6/5/2008 09:21'!
method
	^methodReference at: 1! !

!DebuggerMethodMap methodsFor: 'accessing' stamp: 'eem 6/10/2008 09:44'!
namedTempAt: index in: aContext
	"Answer the value of the temp at index in aContext where index is relative
	 to the array of temp names answered by tempNamesForContext:"
	self subclassResponsibility! !

!DebuggerMethodMap methodsFor: 'accessing' stamp: 'eem 6/10/2008 09:44'!
namedTempAt: index put: aValue in: aContext
	"Assign the value of the temp at index in aContext where index is relative
	 to the array of temp names answered by tempNamesForContext:"
	self subclassResponsibility! !

!DebuggerMethodMap methodsFor: 'source mapping' stamp: 'eem 9/8/2011 10:30'!
rangeForPC: contextsConcretePC contextIsActiveContext: contextIsActiveContext
	"Answer the indices in the source code for the supplied pc.
	 If the context is the actve context (is at the hot end of the stack)
	 then its pc is the current pc.  But if the context isn't, because it is
	 suspended sending a message, then its current pc is the previous pc."

	| pc i end |
	pc := self method abstractPCForConcretePC: (contextIsActiveContext
													ifTrue: [contextsConcretePC]
													ifFalse: [(self method pcPreviousTo: contextsConcretePC)
																ifNotNil: [:prevpc| prevpc]
																ifNil: [contextsConcretePC]]).
	(self abstractSourceMap includesKey: pc) ifTrue:
		[^self abstractSourceMap at: pc].
	sortedSourceMap ifNil:
		[sortedSourceMap := self abstractSourceMap.
		 sortedSourceMap := (sortedSourceMap keys collect: 
								[:key| key -> (sortedSourceMap at: key)]) asSortedCollection].
	(sortedSourceMap isNil or: [sortedSourceMap isEmpty]) ifTrue: [^1 to: 0].
	i := sortedSourceMap findNearbyBinaryIndex: [:assoc| pc - assoc key].
	i < 1 ifTrue: [^1 to: 0].
	i > sortedSourceMap size ifTrue:
		[end := sortedSourceMap inject: 0 into:
			[:prev :this | prev max: this value last].
		^end+1 to: end].
	^(sortedSourceMap at: i) value

	"| method source scanner map |
	 method := DebuggerMethodMap compiledMethodAt: #rangeForPC:contextIsActiveContext:.
	 source := method getSourceFromFile asString.
	 scanner := InstructionStream on: method.
	 map := method debuggerMap.
	 Array streamContents:
		[:ranges|
		[scanner atEnd] whileFalse:
			[| range |
			 range := map rangeForPC: scanner pc contextIsActiveContext: true.
			 ((map abstractSourceMap includesKey: scanner abstractPC)
			  and: [range first ~= 0]) ifTrue:
				[ranges nextPut: (source copyFrom: range first to: range last)].
			scanner interpretNextInstructionFor: InstructionClient new]]"! !

!DebuggerMethodMap methodsFor: 'source mapping' stamp: 'eem 7/6/2009 10:13'!
sourceText
	self method ifNotNil:
		[:method|
		method holdsTempNames ifTrue:
			[^method
				getSourceFor: (method selector ifNil: [method defaultSelector])
				in: method methodClass]].
	^methodNode sourceText! !

!DebuggerMethodMap methodsFor: 'accessing' stamp: 'eem 6/10/2008 09:45'!
tempNamesForContext: aContext
	"Answer an Array of all the temp names in scope in aContext starting with
	 the home's first local (the first argument or first temporary if no arguments)."
	self subclassResponsibility! !

!DebuggerMethodMap methodsFor: 'accessing' stamp: 'eem 6/10/2008 09:47'!
tempsAndValuesForContext: aContext
	"Return a string of the temporary variabls and their current values"
	| aStream |
	aStream := WriteStream on: (String new: 100).
	(self tempNamesForContext: aContext) doWithIndex:
		[:title :index |
		 aStream nextPutAll: title; nextPut: $:; space; tab.
		 aContext print: (self namedTempAt: index in: aContext) on: aStream.
		 aStream cr].
	^aStream contents! !

!DebuggerMethodMap methodsFor: 'accessing' stamp: 'eem 6/2/2008 18:32'!
timestamp
	^timestamp! !

Object subclass: #MethodFinder
	instanceVariableNames: 'data answers selector argMap thisData mapStage mapList expressions cachedClass cachedArgNum cachedSelectorLists'
	classVariableNames: 'AddAndRemove Approved Blocks Dangerous'
	poolDictionaries: ''
	category: 'Tools-MethodFinder'!

!MethodFinder commentStamp: '<historical>' prior: 0!
Find a method in the system from a set of examples.  Done by brute force, trying every possible selector.  Errors are skipped over using ( [3 + 'xyz'] ifError: [^ false] ).
Submit an array of the form ((data1 data2) answer  (data1 data2) answer).

	MethodFinder methodFor: #( (4 3) 7  (0 5) 5  (5 5) 10).

answer:  'data1 + data2'

More generally, use the brace notation to construct live examples.

The program tries data1 as the receiver, and
	tries all other permutations of the data for the receiver and args, and
	tries leaving out one argument, and
	uses all selectors data understands, and
	uses all selectors in all od data's superclasses.

Floating point values must be precise to 0.01 percent, or (X * 0.0001).

If you get an error, you have probably discovered a selector that needs to be removed from the Approved list.  See MethodFinder.initialize.  Please email the Squeak Team.

Only considers 0, 1, 2, and 3 argument messages.  The argument data may have 1 to 5 entries, but only a max of 4 used at a time.  For now, we only test messages that use given number of args or one fewer.  For example, this data (100 true 0.6) would test the receiver plus two args, and the receiver plus one arg, but not any other patterns.

Three sets of selectors:  Approved, AddAndRemove, and Blocks selectors.  When testing a selector in AddAndRemove, deepCopy the receiver.  We do not handle selectors that modify an argument (printOn: etc.).  Blocks is a set of (selector argNumber) where that argument must be a block.

For perform, the selector is tested.  It must be in the Approved list.

do: is not on the Approved list.  It does not produce a result that can be tested.  Type 'do' into the upper pane of the Selector Finder to find messages list that.

[Later, allow the user to supply a block that tests the answer, not just the literal answer.]
	MethodFinder methodFor: { { true. [3]. [4]}. 3}. 
Later allow this to work without the blocks around 3 and 4.!

!MethodFinder class methodsFor: 'as yet unclassified' stamp: 'ar 9/27/2005 22:45'!
methodFor: dataAndAnswers
	"Return a Squeak expression that computes these answers.  (This method is called by the comment in the bottom pane of a MethodFinder.  Do not delete this method.)"

	| resultOC resultString |
	resultOC := (self new) load: dataAndAnswers; findMessage.
	resultString := String streamContents: [:strm |
		resultOC do: [:exp | strm nextPut: $(; nextPutAll: exp; nextPut: $); space]].
	^ resultString! !

!MethodFinder methodsFor: 'find a constant' stamp: 'tk 12/29/2000 22:34'!
allNumbers
	"Return true if all answers and all data are numbers."

	answers do: [:aa | aa isNumber ifFalse: [^ false]].
	thisData do: [:vec |
			vec do: [:nn | nn isNumber ifFalse: [^ false]]].
	^ true! !

!MethodFinder methodsFor: 'access' stamp: 'tk 12/29/2000 13:39'!
answers

	^ answers! !

!MethodFinder methodsFor: 'arg maps' stamp: 'tk 4/24/1999 19:29'!
argMap
	^ argMap ! !

!MethodFinder methodsFor: 'initialize' stamp: 'tk 7/1/2000 23:11'!
cleanInputs: dataAndAnswerString
	"Find an remove common mistakes.  Complain when ill formed."

| fixed ddd rs places |
ddd := dataAndAnswerString.
fixed := false.

rs := ReadStream on: ddd, ' '.
places := OrderedCollection new.
[rs upToAll: '#true'.  rs atEnd] whileFalse: [places addFirst: rs position-4]. 
places do: [:pos | ddd := ddd copyReplaceFrom: pos to: pos with: ''.
	fixed := true]. 	"remove #"

rs := ReadStream on: ddd.
places := OrderedCollection new.
[rs upToAll: '#false'.  rs atEnd] whileFalse: [places addFirst: rs position-5]. 
places do: [:pos | ddd := ddd copyReplaceFrom: pos to: pos with: ''.
	fixed := true]. 	"remove #"

fixed ifTrue: [self inform: '#(true false) are Symbols, not Booleans.  
Next time use { true. false }.'].

fixed := false.
rs := ReadStream on: ddd.
places := OrderedCollection new.
[rs upToAll: '#nil'.  rs atEnd] whileFalse: [places addFirst: rs position-3]. 
places do: [:pos | ddd := ddd copyReplaceFrom: pos to: pos with: ''.
	fixed := true]. 	"remove #"

fixed ifTrue: [self inform: '#nil is a Symbol, not the authentic UndefinedObject.  
Next time use nil instead of #nil'].

^ ddd
! !

!MethodFinder methodsFor: 'find a constant' stamp: 'tk 1/18/2001 22:45'!
const
	| const |
	"See if (^ constant) is the answer"

	"quick test"
	((const := answers at: 1) closeTo: (answers at: 2)) ifFalse: [^ false].
	3 to: answers size do: [:ii | (const closeTo: (answers at: ii)) ifFalse: [^ false]].
	expressions add: '^ ', const printString.
	selector add: #yourself.
	^ true! !

!MethodFinder methodsFor: 'find a constant' stamp: 'tk 1/8/2001 17:49'!
constDiv
	| const subTest got |
	"See if (data1 // C) is the answer"

	const := ((thisData at: 1) at: 1) // (answers at: 1).  "May not be right!!"
	got := (subTest := MethodFinder new copy: self addArg: const) 
				searchForOne isEmpty not.
	got ifFalse: [^ false]. 

	"replace data2 with const in expressions"
	subTest expressions do: [:exp |
		expressions add: (exp copyReplaceAll: 'data2' with: const printString)].
	selector addAll: subTest selectors.
	^ true! !

!MethodFinder methodsFor: 'find a constant' stamp: 'md 11/14/2003 16:47'!
constEquiv
	| const subTest got jj |
	"See if (data1 = C) or (data1 ~= C) is the answer"

	"quick test"
	((answers at: 1) class superclass == Boolean) ifFalse: [^ false].
	2 to: answers size do: [:ii | 
		((answers at: ii) class superclass == Boolean) ifFalse: [^ false]].

	const := (thisData at: 1) at: 1.
	got := (subTest := MethodFinder new copy: self addArg: const) 
				searchForOne isEmpty not.
	got ifFalse: ["try other polarity for ~~ "
		(jj := answers indexOf: (answers at: 1) not) > 0 ifTrue: [
		const := (thisData at: jj) at: 1.
		got := (subTest := MethodFinder new copy: self addArg: const) 
				searchForOne isEmpty not]]. 
	got ifFalse: [^ false]. 

	"replace data2 with const in expressions"
	subTest expressions do: [:exp |
		expressions add: (exp copyReplaceAll: 'data2' with: const printString)].
	selector addAll: subTest selectors.
	^ true! !

!MethodFinder methodsFor: 'find a constant' stamp: 'tk 1/8/2001 17:47'!
constLinear
	| const subTest got denom num slope offset |
	"See if (data1 * C1) + C2 is the answer.  In the form  #(C2 C1) polynomialEval: data1 "

	denom := ((thisData at: 2) at: 1) - ((thisData at: 1) at: 1).
	denom = 0 ifTrue: [^ false].   "will divide by it"
	num := (answers at: 2) - (answers at: 1).

    slope := (num asFloat / denom) reduce.
    offset := ((answers at: 2) - (((thisData at: 2) at: 1) * slope)) reduce.

	const := Array with: offset with: slope.
	got := (subTest := MethodFinder new copy: self addArg: const) 
				searchForOne isEmpty not.
	got ifFalse: [^ false]. 

	"replace data2 with const in expressions"
	subTest expressions do: [:exp |
		expressions add: (exp copyReplaceAll: 'data2' with: const printString)].
	selector addAll: subTest selectors.
	^ true! !

!MethodFinder methodsFor: 'find a constant' stamp: 'tk 1/18/2001 22:46'!
constMod
	| subTest low |
	"See if mod, (data1 \\ C) is the answer"

	low := answers max.
	low+1 to: low+20 do: [:const |
		subTest := MethodFinder new copy: self addArg: const.
		(subTest testPerfect: #\\) ifTrue: [
			expressions add: 'data1 \\ ', const printString.
			selector add: #\\.
			^ true]].
	^ false! !

!MethodFinder methodsFor: 'find a constant' stamp: 'tk 1/8/2001 17:49'!
constMult
	| const subTest got |
	"See if (data1 * C) is the answer"

	((thisData at: 1) at: 1) = 0 ifTrue: [^ false].
	const := ((answers at: 1) asFloat / ((thisData at: 1) at: 1)) reduce.
	got := (subTest := MethodFinder new copy: self addArg: const) 
				searchForOne isEmpty not.
	got ifFalse: [^ false]. 

	"replace data2 with const in expressions"
	subTest expressions do: [:exp |
		expressions add: (exp copyReplaceAll: 'data2' with: const printString)].
	selector addAll: subTest selectors.
	^ true! !

!MethodFinder methodsFor: 'find a constant' stamp: 'tk 1/8/2001 17:48'!
constPlus
	| const subTest got |
	"See if (data1 + C) is the answer"

	const := (answers at: 1) - ((thisData at: 1) at: 1).
	got := (subTest := MethodFinder new copy: self addArg: const) 
				searchForOne isEmpty not.
	got ifFalse: [^ false]. 

	"replace data2 with const in expressions"
	subTest expressions do: [:exp |
		expressions add: (exp copyReplaceAll: 'data2' with: const printString)].
	selector addAll: subTest selectors.
	^ true! !

!MethodFinder methodsFor: 'find a constant' stamp: 'nice 12/27/2009 03:10'!
constUsingData1Value
	| subTest |
	"See if (data1 <= C) or (data1 >= C) is the answer"

	"quick test"
	((answers at: 1) class superclass == Boolean) ifFalse: [^ false].
	2 to: answers size do: [:ii | 
		((answers at: ii) class superclass == Boolean) ifFalse: [^ false]].

	thisData do: [:datums | | got const | 
		const := datums first.	"use data as a constant!!"
		got := (subTest := MethodFinder new copy: self addArg: const) 
					searchForOne isEmpty not.
		got ifTrue: [
			"replace data2 with const in expressions"
			subTest expressions do: [:exp |
				expressions add: (exp copyReplaceAll: 'data2' with: const printString)].
			selector addAll: subTest selectors.
			^ true]].
	^ false! !

!MethodFinder methodsFor: 'initialize' stamp: 'md 11/14/2003 16:47'!
copy: mthFinder addArg: aConstant
	| more |
	"Copy inputs and answers, add an additional data argument to the inputs.  The same constant for every example"

	more := Array with: aConstant.
	data := mthFinder data collect: [:argList | argList, more].
	answers := mthFinder answers.
	self load: nil.
! !

!MethodFinder methodsFor: 'access' stamp: 'tk 12/29/2000 13:39'!
data

	^ data! !

!MethodFinder methodsFor: 'search' stamp: 'ar 4/10/2005 22:20'!
exceptions
	"Handle some very slippery selectors.
	asSymbol -- want to be able to produce it, but do not want to make every string submitted into a Symbol!!" 

	| aSel |
	answers first isSymbol ifFalse: [^ self].
	thisData first first isString ifFalse: [^ self].
	aSel := #asSymbol.
	(self testPerfect: aSel) ifTrue: [
		selector add: aSel.
		expressions add: (String streamContents: [:strm | 
			strm nextPutAll: 'data', argMap first printString.
			aSel keywords doWithIndex: [:key :ind |
				strm nextPutAll: ' ',key.
				(key last == $:) | (key first isLetter not)
					ifTrue: [strm nextPutAll: ' data', 
						(argMap at: ind+1) printString]]])].
! !

!MethodFinder methodsFor: 'access' stamp: 'tk 12/29/2000 13:20'!
expressions
	^ expressions! !

!MethodFinder methodsFor: 'search' stamp: 'ar 4/10/2005 18:48'!
findMessage
	"Control the search."

	data do: [:alist |
		(alist isKindOf: SequenceableCollection) ifFalse: [
			^ OrderedCollection with: 'first and third items are not Arrays']].
	Approved ifNil: [self initialize].	"Sets of allowed selectors"
	expressions := OrderedCollection new.
	self search: true.	"multi"
	expressions isEmpty ifTrue: [^ OrderedCollection with: 'no single method does that function'].
	expressions isString ifTrue: [^ OrderedCollection with: expressions].
 	^ expressions! !

!MethodFinder methodsFor: 'initialize' stamp: 'mt 8/12/2016 10:19'!
initialize
	"The methods we are allowed to use.  (MethodFinder new initialize) "

	Approved := Set new.
	AddAndRemove := Set new.
	Blocks := Set new.
	"These modify an argument and are not used by the MethodFinder: longPrintOn: printOn: storeOn: sentTo: storeOn:base: printOn:base: absPrintExactlyOn:base: absPrintOn:base: absPrintOn:base:digitCount: writeOn: writeScanOn: possibleVariablesFor:continuedFrom: printOn:format:"

"Object"  
	#("in class, instance creation" categoryForUniclasses chooseUniqueClassName initialInstance isSystemDefined newFrom: officialClass readCarefullyFrom:
"accessing" at: basicAt: basicSize bindWithTemp: in: size yourself 
"testing" basicType ifNil: ifNil:ifNotNil: ifNotNil: ifNotNil:ifNil: isColor isFloat isFraction isInMemory isInteger isMorph isNil isNumber isPoint isPseudoContext isText isTransparent isWebBrowser knownName notNil pointsTo: wantsSteps 
"comparing" = == closeTo: hash identityHash identityHashPrintString ~= ~~ 
"copying" clone copy shallowCopy 
"dependents access" canDiscardEdits dependents hasUnacceptedEdits 
"updating" changed changed: okToChange update: windowIsClosing 
"printing" fullPrintString isLiteral longPrintString printString storeString stringForReadout stringRepresentation 
"class membership" class isKindOf: isKindOf:orOf: isMemberOf: respondsTo: xxxClass 
"error handling" 
"user interface" addModelMenuItemsTo:forMorph:hand: defaultBackgroundColor defaultLabelForInspector fullScreenSize initialExtent modelWakeUp mouseUpBalk: newTileMorphRepresentative windowActiveOnFirstClick windowReqNewLabel: 
"system primitives" asOop instVarAt: instVarNamed: 
"private" 
"associating" -> 
"converting" as: asOrderedCollection asString 
"casing" caseOf: caseOf:otherwise: 
"binding" bindingOf: 
"macpal" contentsChanged currentEvent currentHand currentWorld flash instanceVariableValues scriptPerformer 
"flagging" flag: 
"translation support" "objects from disk" "finalization" ) do: [:sel | Approved add: sel].
	#(at:add: at:modify: at:put: basicAt:put: "NOT instVar:at:"
"message handling" perform: perform:orSendTo: perform:with: perform:with:with: perform:with:with:with: perform:withArguments: perform:withArguments:inSuperclass: 
) do: [:sel | AddAndRemove add: sel].

"Boolean, True, False, UndefinedObject"  
	#("logical operations" & eqv: not xor: |
"controlling" and: ifFalse: ifFalse:ifTrue: ifTrue: ifTrue:ifFalse: or:
"copying" 
"testing" isEmptyOrNil) do: [:sel | Approved add: sel].

"Behavior" 
	#("initialize-release"
"accessing" compilerClass decompilerClass evaluatorClass format methodDict parserClass sourceCodeTemplate subclassDefinerClass
"testing" instSize instSpec isBits isBytes isFixed isPointers isVariable isWeak isWords
"copying"
"printing" defaultNameStemForInstances printHierarchy
"creating class hierarchy"
"creating method dictionary"
"instance creation" basicNew basicNew: new new:
"accessing class hierarchy" allSubclasses allSubclassesWithLevelDo:startingLevel: allSuperclasses subclasses superclass withAllSubclasses withAllSuperclasses
"accessing method dictionary" allSelectors changeRecordsAt: compiledMethodAt: compiledMethodAt:ifAbsent: firstCommentAt: lookupSelector: selectors selectorsDo: selectorsWithArgs: "slow but useful ->" sourceCodeAt: sourceCodeAt:ifAbsent: sourceMethodAt: sourceMethodAt:ifAbsent:
"accessing instances and variables" allClassVarNames allInstVarNames allSharedPools classVarNames instVarNames instanceCount sharedPools someInstance subclassInstVarNames
"testing class hierarchy" inheritsFrom: kindOfSubclass
"testing method dictionary" canUnderstand: classThatUnderstands: hasMethods includesSelector: whichClassIncludesSelector: whichSelectorsAccess: whichSelectorsReferTo: whichSelectorsReferTo:special:byte: whichMethodsStoreInto:
"enumerating"
"user interface") do: [:sel | Approved add: sel].

"ClassDescription"
	#("initialize-release" 
"accessing" classVersion isMeta name theNonMetaClass
"copying" 
"printing" classVariablesString instanceVariablesString sharedPoolsString
"instance variables" checkForInstVarsOK: 
"method dictionary" 
"organization" category organization whichCategoryIncludesSelector:
"compiling" acceptsLoggingOfCompilation wantsChangeSetLogging
"fileIn/Out" definition
"private" ) do: [:sel | Approved add: sel].

"Class"
	#("initialize-release" 
"accessing" classPool
"testing"
"copying" 
"class name" 
"instance variables" 
"class variables" classVarAt: classVariableAssociationAt:
"pool variables" 
"compiling" 
"subclass creation" 
"fileIn/Out" ) do: [:sel | Approved add: sel]. 

"Metaclass"
	#("initialize-release" 
"accessing" isSystemDefined soleInstance
"copying" "instance creation" "instance variables"  "pool variables" "class hierarchy"  "compiling"
"fileIn/Out"  nonTrivial ) do: [:sel | Approved add: sel].

"Context, BlockContext"
	#(receiver client method receiver tempAt: 
"debugger access" pc selector sender shortStack sourceCode tempNames tempsAndValues
"controlling"  "printing" "system simulation" 
"initialize-release" 
"accessing" hasMethodReturn home numArgs
"evaluating" value value:ifError: value:value: value:value:value: value:value:value:value: valueWithArguments:
"controlling"  "scheduling"  "instruction decoding"  "printing" "private"  "system simulation" ) do: [:sel | Approved add: sel].
	#(value: "<- Association has it as a store" ) do: [:sel | AddAndRemove add: sel].

"Message"
	#("inclass, instance creation" selector: selector:argument: selector:arguments:
"accessing" argument argument: arguments sends:
"printing" "sending" ) do: [:sel | Approved add: sel].
	#("private" setSelector:arguments:) do: [:sel | AddAndRemove add: sel].

"Magnitude"
	#("comparing" < <= > >= between:and:
"testing" max: min: min:max: ) do: [:sel | Approved add: sel].

"Date, Time"
	#("in class, instance creation" fromDays: fromSeconds: fromString: newDay:month:year: newDay:year: today
	"in class, general inquiries" dateAndTimeNow dayOfWeek: daysInMonth:forYear: daysInYear: firstWeekdayOfMonth:year: indexOfMonth: leapYear: nameOfDay: nameOfMonth:
"accessing" day leap monthIndex monthName weekday year
"arithmetic" addDays: subtractDate: subtractDays:
"comparing"
"inquiries" dayOfMonth daysInMonth daysInYear daysLeftInYear firstDayOfMonth previous:
"converting" asSeconds
"printing"  mmddyyyy printFormat: 
"private" weekdayIndex 
	"in class, instance creation" fromSeconds: now 
	"in class, general inquiries" dateAndTimeFromSeconds: dateAndTimeNow millisecondClockValue millisecondsToRun: totalSeconds
"accessing" hours minutes seconds
"arithmetic" addTime: subtractTime:
"comparing"
"printing" intervalString print24 
"converting") do: [:sel | Approved add: sel].
	#("private" 
		 ) do: [:sel | AddAndRemove add: sel].

"Number"
	#("in class" readFrom:base: 
"arithmetic" * + - / // \\ abs negated quo: reciprocal rem:
"mathematical functions" arcCos arcSin arcTan arcTan: cos exp floorLog: ln log log: raisedTo: raisedToInteger: sin sqrt squared tan
"truncation and round off" ceiling detentBy:atMultiplesOf:snap: floor roundTo: roundUpTo: rounded truncateTo: truncated
"comparing"
"testing" even isDivisibleBy: isInfinite isNaN isZero negative odd positive sign strictlyPositive
"converting" @ asInteger asNumber asPoint asSmallAngleDegrees degreesToRadians radiansToDegrees
"intervals" to: to:by: 
"printing" printStringBase: storeStringBase: ) do: [:sel | Approved add: sel].

"Integer"
	#("in class" primesUpTo:
"testing" isPowerOfTwo
"arithmetic" alignedTo:
"comparing"
"truncation and round off" atRandom normalize
"enumerating" timesRepeat:
"mathematical functions" degreeCos degreeSin factorial gcd: lcm: take:
"bit manipulation" << >> allMask: anyMask: bitAnd: bitClear: bitInvert bitInvert32 bitOr: bitShift: bitXor: lowBit noMask:
"converting" asCharacter asColorOfDepth: asFloat asFraction asHexDigit
"printing" asStringWithCommas hex hex8 radix:
"system primitives" lastDigit replaceFrom:to:with:startingAt:
"private" "benchmarks" ) do: [:sel | Approved add: sel].

"SmallInteger, LargeNegativeInteger, LargePositiveInteger"
	#("arithmetic" "bit manipulation" highBit "testing" "comparing" "copying" "converting" "printing" 
"system primitives" digitAt: digitLength 
"private" fromString:radix: ) do: [:sel | Approved add: sel].
	#(digitAt:put: ) do: [:sel | AddAndRemove add: sel].

"Float"
	#("arithmetic"
"mathematical functions" reciprocalLogBase2 timesTwoPower:
"comparing" "testing"
"truncation and round off" exponent fractionPart integerPart significand significandAsInteger
"converting" asApproximateFraction asIEEE32BitWord asTrueFraction
"copying") do: [:sel | Approved add: sel].

"Fraction, Random"
	#(denominator numerator reduced next nextValue) do: [:sel | Approved add: sel].
	#(setNumerator:denominator:) do: [:sel | AddAndRemove add: sel].

"Collection"
	#("accessing" anyOne
"testing" includes: includesAllOf: includesAnyOf: includesSubstringAnywhere: isEmpty isSequenceable occurrencesOf:
"enumerating" collect: collect:thenSelect: count: detect: detect:ifNone: detectMax: detectMin: detectSum: inject:into: reject: select: select:thenCollect: intersection:
"converting" asBag asCharacterSet asSet asSortedArray asSortedCollection asSortedCollection:
"printing"
"private" maxSize
"arithmetic"
"math functions" average max median min range sum) do: [:sel | Approved add: sel].
	#("adding" add: addAll: addIfNotPresent:
"removing" remove: remove:ifAbsent: removeAll: removeAllFoundIn: removeAllSuchThat: remove:ifAbsent:) do: [:sel | AddAndRemove add: sel].

"SequenceableCollection"
	#("comparing" hasEqualElements:
"accessing" allButFirst allButLast at:ifAbsent: atAll: atPin: atRandom: atWrap: fifth first fourth identityIndexOf: identityIndexOf:ifAbsent: indexOf: indexOf:ifAbsent: indexOf:startingAt:ifAbsent: indexOfSubCollection:startingAt: indexOfSubCollection:startingAt:ifAbsent: last second sixth third
"removing"
"copying" , copyAfterLast: copyAt:put: copyFrom:to: copyReplaceAll:with: copyReplaceFrom:to:with: copyUpTo: copyUpToLast: copyWith: copyWithout: copyWithoutAll: forceTo:paddingWith: shuffled sorted:
"enumerating" collectWithIndex: findFirst: findLast: pairsCollect: with:collect: withIndexCollect: polynomialEval:
"converting" asArray asDictionary asFloatArray asIntegerArray asStringWithCr asWordArray reversed
"private" copyReplaceAll:with:asTokens: ) do: [:sel | Approved add: sel].
	#( swap:with:) do: [:sel | AddAndRemove add: sel].

"ArrayedCollection, Bag"
	#("private" defaultElement 
"sorting" isSorted
"accessing" cumulativeCounts sortedCounts sortedElements "testing" "adding" add:withOccurrences: "removing" "enumerating" 
	) do: [:sel | Approved add: sel].
	#( mergeSortFrom:to:by: sort sort: add: add:withOccurrences:
"private" setDictionary ) do: [:sel | AddAndRemove add: sel].

"Other messages that modify the receiver"
	#(atAll:put: atAll:putAll: atAllPut: atWrap:put: replaceAll:with: replaceFrom:to:with:  removeFirst removeLast) do: [:sel | AddAndRemove add: sel].

	self initialize2.

"
MethodFinder new initialize.
MethodFinder new organizationFiltered: Set
"

! !

!MethodFinder methodsFor: 'initialize' stamp: 'mt 8/12/2016 10:19'!
initialize2
	"The methods we are allowed to use.  (MethodFinder new initialize) "

"Set"
	#("in class" sizeFor:
"testing" "adding" "removing" "enumerating"
"private" array scanFor: 
"accessing" someElement) do: [:sel | Approved add: sel].

"Dictionary, IdentityDictionary, IdentitySet"
	#("accessing" associationAt: associationAt:ifAbsent: at:ifPresent: keyAtIdentityValue: keyAtIdentityValue:ifAbsent: keyAtValue: keyAtValue:ifAbsent: keys
"testing" includesKey: ) do: [:sel | Approved add: sel].
	#(removeKey: removeKey:ifAbsent:
) do: [:sel | AddAndRemove add: sel].

"LinkedList, Interval, MappedCollection"
	#("in class"  from:to: from:to:by:
"accessing" contents) do: [:sel | Approved add: sel].
	#(
"adding" addFirst: addLast:) do: [:sel | AddAndRemove add: sel].

"OrderedCollection, SortedCollection"
	#("accessing" after: before:
"copying" copyEmpty
"adding"  growSize
"removing" "enumerating" "private" 
"accessing" sortBlock) do: [:sel | Approved add: sel].
	#("adding" add:after: add:afterIndex: add:before: addAllFirst: addAllLast: addFirst: addLast:
"removing" removeAt: removeFirst removeLast
"accessing" sortBlock:) do: [:sel | AddAndRemove add: sel].

"Character"
	#("in class, instance creation" allByteCharacters digitValue: new separators
	"accessing untypeable characters" backspace cr enter lf linefeed nbsp newPage space tab
	"constants" alphabet characterTable
"accessing" asciiValue digitValue
"comparing"
"testing" isAlphaNumeric isDigit isLetter isLowercase isSafeForHTTP isSeparator isSpecial isUppercase isVowel tokenish
"copying"
"converting" asIRCLowercase asLowercase asUppercase
	) do: [:sel | Approved add: sel].

"String"
	#("in class, instance creation" crlf fromPacked:
	"primitives" findFirstInString:inSet:startingAt: indexOfAscii:inString:startingAt: 	"internet" valueOfHtmlEntity:
"accessing" byteAt: endsWithDigit findAnySubStr:startingAt: findBetweenSubStrs: findDelimiters:startingAt: findString:startingAt: findString:startingAt:caseSensitive: findTokens: findTokens:includes: findTokens:keep: includesSubstring: includesSubstring:caseSensitive: indexOf:startingAt: indexOfAnyOf: indexOfAnyOf:ifAbsent: indexOfAnyOf:startingAt: indexOfAnyOf:startingAt:ifAbsent: lineCorrespondingToIndex: lineCount lineNumber: skipAnySubStr:startingAt: skipDelimiters:startingAt: startsWithDigit
"comparing" alike: beginsWith: caseSensitiveLessOrEqual: charactersExactlyMatching: compare: crc16 endsWith: endsWithAnyOf: sameAs: startingAt:match:startingAt:
"copying" copyReplaceTokens:with: padded:to:with:
"converting" asByteArray asDate asDisplayText asFileName asHtml asLegalSelector asPacked asParagraph asText asTime asUnHtml asUrl asUrlRelativeTo: capitalized compressWithTable: contractTo: correctAgainst: encodeForHTTP initialIntegerOrNil keywords quoted sansPeriodSuffix splitInteger stemAndNumericSuffix substrings surroundedBySingleQuotes truncateWithElipsisTo: withBlanksTrimmed withFirstCharacterDownshifted withNoLineLongerThan: withSeparatorsCompacted withoutLeadingDigits withoutTrailingBlanks
"displaying" "printing"
"system primitives" compare:with:collated: 
"Celeste" withCRs
"internet" decodeMimeHeader decodeQuotedPrintable unescapePercents withInternetLineEndings withSqueakLineEndings withoutQuoting
"testing" isAllSeparators lastSpacePosition
"paragraph support" indentationIfBlank:
"arithmetic" ) do: [:sel | Approved add: sel].
	#(byteAt:put: translateToLowercase match:) do: [:sel | AddAndRemove add: sel].

"Symbol"
	#("in class, private" hasInterned:ifTrue:
	"access" morePossibleSelectorsFor: possibleSelectorsFor: selectorsContaining: thatStarts:skipping:
"accessing" "comparing" "copying" "converting" "printing" 
"testing" isInfix isKeyword isPvtSelector isUnary) do: [:sel | Approved add: sel].

"Array"
	#("comparing" "converting" evalStrings 
"printing" "private" hasLiteralSuchThat:) do: [:sel | Approved add: sel].

"Array2D"
	#("access" at:at: atCol: atCol:put: atRow: extent extent:fromArray: height width width:height:type:) do: [:sel | Approved add: sel].
	#(at:at:add: at:at:put: atRow:put: ) do: [:sel | AddAndRemove add: sel].

"ByteArray"
	#("accessing" doubleWordAt: wordAt: 
"platform independent access" longAt:bigEndian: shortAt:bigEndian: unsignedLongAt:bigEndian: unsignedShortAt:bigEndian: 
"converting") do: [:sel | Approved add: sel].
	#(doubleWordAt:put: wordAt:put: longAt:put:bigEndian: shortAt:put:bigEndian: unsignedLongAt:put:bigEndian: unsignedShortAt:put:bigEndian:
	) do: [:sel | AddAndRemove add: sel].

"FloatArray"		"Dont know what happens when prims not here"
	false ifTrue: [#("accessing" "arithmetic" *= += -= /=
"comparing"
"primitives-plugin" primAddArray: primAddScalar: primDivArray: primDivScalar: primMulArray: primMulScalar: primSubArray: primSubScalar:
"primitives-translated" primAddArray:withArray:from:to: primMulArray:withArray:from:to: primSubArray:withArray:from:to:
"converting" "private" "user interface") do: [:sel | Approved add: sel].
	].

"IntegerArray, WordArray"
"RunArray"
	#("in class, instance creation" runs:values: scanFrom:
"accessing" runLengthAt: 
"adding" "copying"
"private" runs values) do: [:sel | Approved add: sel].
	#(coalesce addLast:times: repeatLast:ifEmpty: repeatLastIfEmpty:
		) do: [:sel | AddAndRemove add: sel].

"Stream  -- many operations change its state"
	#("testing" atEnd) do: [:sel | Approved add: sel].
	#("accessing" next: nextMatchAll: nextMatchFor: upToEnd
next:put: nextPut: nextPutAll: "printing" print: printHtml:
	) do: [:sel | AddAndRemove add: sel].

"PositionableStream"
	#("accessing" contentsOfEntireFile originalContents peek peekFor: "testing"
"positioning" position ) do: [:sel | Approved add: sel].
	#(nextDelimited: nextLine upTo: position: reset resetContents setToEnd skip: skipTo: upToAll: ) do: [:sel | AddAndRemove add: sel].
	"Because it is so difficult to test the result of an operation on a Stream (you have to supply another Stream in the same state), we don't support Streams beyond the basics.  We want to find the messages that convert Streams to other things."

"ReadWriteStream"
	#("file status" closed) do: [:sel | Approved add: sel].
	#("accessing" next: on: ) do: [:sel | AddAndRemove add: sel].

"WriteStream"
	#("in class, instance creation" on:from:to: with: with:from:to:
		) do: [:sel | Approved add: sel].
	#("positioning" resetToStart
"character writing" crtab crtab:) do: [:sel | AddAndRemove add: sel].

"LookupKey, Association, Link"
	#("accessing" key nextLink) do: [:sel | Approved add: sel].
	#(key: key:value: nextLink:) do: [:sel | AddAndRemove add: sel].

"Point"
	#("in class, instance creation" r:degrees: x:y:
"accessing" x y "comparing" "arithmetic" "truncation and round off"
"polar coordinates" degrees r theta
"point functions" bearingToPoint: crossProduct: dist: dotProduct: eightNeighbors flipBy:centerAt: fourNeighbors grid: nearestPointAlongLineFrom:to: nearestPointOnLineFrom:to: normal normalized octantOf: onLineFrom:to: onLineFrom:to:within: quadrantOf: rotateBy:centerAt: transposed unitVector
"converting" asFloatPoint asIntegerPoint corner: extent: rect:
"transforming" adhereTo: rotateBy:about: scaleBy: scaleFrom:to: translateBy: "copying"
"interpolating" interpolateTo:at:) do: [:sel | Approved add: sel].

"Rectangle"
	#("in class, instance creation" center:extent: encompassing: left:right:top:bottom: 
	merging: origin:corner: origin:extent: 
"accessing" area bottom bottomCenter bottomLeft bottomRight boundingBox center corner corners innerCorners left leftCenter origin right rightCenter top topCenter topLeft topRight
"comparing"
"rectangle functions" adjustTo:along: amountToTranslateWithin: areasOutside: bordersOn:along: encompass: expandBy: extendBy: forPoint:closestSideDistLen: insetBy: insetOriginBy:cornerBy: intersect: merge: pointNearestTo: quickMerge: rectanglesAt:height: sideNearestTo: translatedToBeWithin: withBottom: withHeight: withLeft: withRight: withSide:setTo: withTop: withWidth:
"testing" containsPoint: containsRect: hasPositiveExtent intersects: isTall isWide
"truncation and round off"
"transforming" align:with: centeredBeneath: newRectFrom: squishedWithin: "copying"
	) do: [:sel | Approved add: sel].

"Color"
	#("in class, instance creation" colorFrom: colorFromPixelValue:depth: fromRgbTriplet: gray: h:s:v: r:g:b: r:g:b:alpha: r:g:b:range:
	"named colors" black blue brown cyan darkGray gray green lightBlue lightBrown lightCyan lightGray lightGreen lightMagenta lightOrange lightRed lightYellow magenta orange red transparent veryDarkGray veryLightGray veryVeryDarkGray veryVeryLightGray white yellow
	"other" colorNames indexedColors pixelScreenForDepth: quickHighLight:
"access" alpha blue brightness green hue luminance red saturation
"equality"
"queries" isBitmapFill isBlack isGray isSolidFill isTranslucent isTranslucentColor
"transformations" alpha: dansDarker darker lighter mixed:with: muchLighter slightlyDarker slightlyLighter veryMuchLighter alphaMixed:with:
"groups of shades" darkShades: lightShades: mix:shades: wheel:
"printing" shortPrintString
"other" colorForInsets rgbTriplet
"conversions" asB3DColor asColor balancedPatternForDepth: bitPatternForDepth: closestPixelValue1 closestPixelValue2 closestPixelValue4 closestPixelValue8 dominantColor halfTonePattern1 halfTonePattern2 indexInMap: pixelValueForDepth: pixelWordFor:filledWith: pixelWordForDepth: scaledPixelValue32
"private" privateAlpha privateBlue privateGreen privateRGB privateRed "copying"
	) do: [:sel | Approved add: sel].

"	For each selector that requires a block argument, add (selector argNum) 
		to the set Blocks."
"ourClasses := #(Object Boolean True False UndefinedObject Behavior ClassDescription Class Metaclass MethodContext BlockContext Message Magnitude Date Time Number Integer SmallInteger LargeNegativeInteger LargePositiveInteger Float Fraction Random Collection SequenceableCollection ArrayedCollection Bag Set Dictionary IdentityDictionary IdentitySet LinkedList Interval MappedCollection OrderedCollection SortedCollection Character String Symbol Array Array2D ByteArray FloatArray IntegerArray WordArray RunArray Stream PositionableStream ReadWriteStream WriteStream LookupKey Association Link Point Rectangle Color).
ourClasses do: [:clsName | cls := Smalltalk at: clsName.
	(cls selectors) do: [:aSel |
		((Approved includes: aSel) or: [AddAndRemove includes: aSel]) ifTrue: [
			(cls formalParametersAt: aSel) withIndexDo: [:tName :ind |
				(tName endsWith: 'Block') ifTrue: [
					Blocks add: (Array with: aSel with: ind)]]]]].
"
#((timesRepeat: 1 ) (indexOf:ifAbsent: 2 ) (pairsCollect: 1 ) (mergeSortFrom:to:by: 3 ) (ifNotNil:ifNil: 1 ) (ifNotNil:ifNil: 2 ) (ifNil: 1 ) (at:ifAbsent: 2 ) (ifNil:ifNotNil: 1 ) (ifNil:ifNotNil: 2 ) (ifNotNil: 1 ) (at:modify: 2 ) (identityIndexOf:ifAbsent: 2 ) (sort: 1 ) (sortBlock: 1 ) (detectMax: 1 ) (repeatLastIfEmpty: 1 ) (allSubclassesWithLevelDo:startingLevel: 1 ) (keyAtValue:ifAbsent: 2 ) (in: 1 ) (ifTrue: 1 ) (or: 1 ) (select: 1 ) (inject:into: 2 )  (forPoint:closestSideDistLen: 2 ) (value:ifError: 2 ) (selectorsDo: 1 ) (removeAllSuchThat: 1 ) (keyAtIdentityValue:ifAbsent: 2 ) (detectMin: 1 ) (detect:ifNone: 1 ) (ifTrue:ifFalse: 1 ) (ifTrue:ifFalse: 2 ) (detect:ifNone: 2 ) (hasLiteralSuchThat: 1 ) (indexOfAnyOf:ifAbsent: 2 ) (reject: 1 ) (newRectFrom: 1 ) (removeKey:ifAbsent: 2 ) (at:ifPresent: 2 ) (associationAt:ifAbsent: 2 ) (withIndexCollect: 1 ) (repeatLast:ifEmpty: 2 ) (findLast: 1 ) (indexOf:startingAt:ifAbsent: 3 ) (remove:ifAbsent: 2 ) (ifFalse:ifTrue: 1 ) (ifFalse:ifTrue: 2 ) (caseOf:otherwise: 2 ) (count: 1 ) (collect: 1 ) (sorted: 1 ) (and: 1 ) (asSortedCollection: 1 ) (with:collect: 2 ) (sourceCodeAt:ifAbsent: 2 ) (detect: 1 ) (collectWithIndex: 1 ) (compiledMethodAt:ifAbsent: 2 ) (bindWithTemp: 1 ) (detectSum: 1 ) (indexOfSubCollection:startingAt:ifAbsent: 3 ) (findFirst: 1 ) (sourceMethodAt:ifAbsent: 2 ) (collect:thenSelect: 1 ) (collect:thenSelect: 2 ) (select:thenCollect: 1 ) (select:thenCollect: 2 ) (ifFalse: 1 ) (indexOfAnyOf:startingAt:ifAbsent: 3 ) (indentationIfBlank: 1 ) ) do: [:anArray |
	Blocks add: anArray].

self initialize3.

"
MethodFinder new initialize.
MethodFinder new organizationFiltered: TranslucentColor class 
"
"Do not forget class messages for each of these classes"
! !

!MethodFinder methodsFor: 'initialize' stamp: 'tk 4/1/2002 11:33'!
initialize3
	"additional selectors to consider"

#(asWords threeDigitName ) do: [:sel | Approved add: sel].! !

!MethodFinder methodsFor: 'search' stamp: 'tk 4/12/2001 10:47'!
insertConstants
	"see if one of several known expressions will do it. C is the constant we discover here."
	"C  data1+C  data1*C  data1//C  (data1*C1 + C2) (data1 = C) (data1 ~= C) (data1 <= C) (data1 >= C) 
 (data1 mod C)"

	thisData size >= 2 ifFalse: [^ false].	"need 2 examples"
	(thisData at: 1) size = 1 ifFalse: [^ false].	"only one arg, data1"

	self const ifTrue: [^ true].
	self constUsingData1Value ifTrue: [^ true].
		"(data1 ?? const), where const is one of the values of data1"
		" == ~~ ~= = <= >= "

	self allNumbers ifFalse: [^ false].
	self constMod ifTrue: [^ true].
	self constPlus ifTrue: [^ true].
	self constMult ifTrue: [^ true].
	self constDiv ifTrue: [^ true].
	self constLinear ifTrue: [^ true].
	^ false! !

!MethodFinder methodsFor: 'initialize' stamp: 'tk 12/29/2000 13:22'!
load: dataWithAnswers
	"Find a function that takes the data and gives the answers.  Odd list entries are data for it, even ones are the answers.  nil input means data and answers were supplied already."
"  (MethodFinder new) load: #( (4 3) 7  (-10 5) -5  (-3 11) 8);
		findMessage  "

dataWithAnswers ifNotNil: [
	data := Array new: dataWithAnswers size // 2.
	1 to: data size do: [:ii | data at: ii put: (dataWithAnswers at: ii*2-1)].
	answers := Array new: data size.
	1 to: answers size do: [:ii | answers at: ii put: (dataWithAnswers at: ii*2)]].
data do: [:list | 
	(list isKindOf: SequenceableCollection) ifFalse: [
		^ self inform: 'first and third items are not Arrays'].
	].
argMap := (1 to: data first size) asArray.
data do: [:list | list size = argMap size ifFalse: [
		self inform: 'data arrays must all be the same size']].
argMap size > 4 ifTrue: [self inform: 'No more than a receiver and 
three arguments allowed'].
	"Really only test receiver and three args." 
thisData := data copy.
mapStage := mapList := nil.
! !

!MethodFinder methodsFor: 'arg maps' stamp: 'tk 5/18/1999 14:46'!
makeAllMaps 
	"Make a giant list of all permutations of the args.  To find the function, we will try these permutations of the input data.  receiver, args."

	| ii |
	mapList := Array new: argMap size factorial.
	ii := 1.
	argMap permutationsDo: [:perm |
		mapList at: ii put: perm copy.
		ii := ii + 1].
	mapStage := 1.	"about to be bumped"! !

!MethodFinder methodsFor: 'arg maps' stamp: 'tk 4/24/1999 19:29'!
mapData 
	"Force the data through the map (permutation) to create the data to test."

	thisData := data collect: [:realData |
					argMap collect: [:ind | realData at: ind]].
		! !

!MethodFinder methodsFor: 'initialize' stamp: 'eem 1/18/2016 18:44'!
noteDangerous
	"Remember the methods with really bad side effects."

	Dangerous := Set new.
"Object accessing, testing, copying, dependent access, macpal, flagging"
	#(addInstanceVarNamed:withValue: haltIfNil copyAddedStateFrom: veryDeepCopy veryDeepCopyWith: veryDeepFixupWith: veryDeepInner: addDependent: evaluate:wheneverChangeIn: codeStrippedOut: playSoundNamed: isThisEverCalled isThisEverCalled: logEntry logExecution logExit)
		do: [:sel | Dangerous add: sel].

"Object error handling"
	#(cannotInterpret: caseError confirm: confirm:orCancel: doesNotUnderstand: error: halt halt: notify: notify:at: primitiveFailed shouldNotImplement subclassResponsibility)
		do: [:sel | Dangerous add: sel].

"Object user interface"
	#(basicInspect beep inform: inspect inspectWithLabel: notYetImplemented inspectElement )
		do: [:sel | Dangerous add: sel].

"Object system primitives"
	#(become: becomeForward: instVarAt:put: instVarNamed:put: nextInstance nextObject rootStubInImageSegment: someObject tryPrimitive:withArgs:)
		do: [:sel | Dangerous add: sel].

"Object private"
	#(errorImproperStore errorNonIntegerIndex errorNotIndexable errorSubscriptBounds: mustBeBoolean primitiveError: species storeAt:inTempFrame:)
		do: [:sel | Dangerous add: sel].

"Object, translation support"
	#(cCode: cCode:inSmalltalk: cCoerce:to: export: inline: returnTypeC: sharedCodeNamed:inCase: var:declareC:)
		do: [:sel | Dangerous add: sel].

"Object, objects from disk, finalization.  And UndefinedObject"
	#(comeFullyUpOnReload: objectForDataStream: readDataFrom:size: rehash saveOnFile storeDataOn: actAsExecutor executor finalize retryWithGC:until:   suspend)
		do: [:sel | Dangerous add: sel].

"No Restrictions:   Boolean, False, True, "

"Morph"
	#()
		do: [:sel | Dangerous add: sel].

"Behavior"
	#(obsolete confirmRemovalOf: copyOfMethodDictionary literalScannedAs:notifying: storeLiteral:on: addSubclass: removeSubclass: superclass: 
"creating method dictionary" addSelector:withMethod: compile: compile:notifying: compileAll compileAllFrom: compress decompile: defaultSelectorForMethod: methodDictionary: recompile:from: recompileChanges removeSelector: compressedSourceCodeAt: selectorAtMethod:setClass: allInstances allSubInstances inspectAllInstances inspectSubInstances thoroughWhichSelectorsReferTo:special:byte: "enumerating" allInstancesDo: allSubInstancesDo: allSubclassesDo: allSuperclassesDo: selectSubclasses: selectSuperclasses: subclassesDo: withAllSubclassesDo:
   "too slow->" crossReference removeUninstantiatedSubclassesSilently "too slow->" unreferencedInstanceVariables
"private" flushCache format:variable:words:pointers: format:variable:words:pointers:weak: format:variable:bitsUnitSize:pointers:weak: printSubclassesOn:level: basicRemoveSelector: addSelector:withMethod:notifying: addSelectorSilently:withMethod:)
		do: [:sel | Dangerous add: sel].

"CompiledMethod"
	#(defaultSelector)
		do: [:sel | Dangerous add: sel].

"Others "
	#("no tangible result" do: associationsDo:  
"private" adaptToCollection:andSend: adaptToNumber:andSend: adaptToPoint:andSend: adaptToString:andSend: instVarAt:put: asDigitsToPower:do: combinations:atATimeDo: doWithIndex: pairsDo: permutationsDo: reverseDo: reverseWith:do: with:do: withIndexDo: asDigitsAt:in:do: combinationsAt:in:after:do: errorOutOfBounds permutationsStartingAt:do: fromUser)
		do: [:sel | Dangerous add: sel].


	#(    fileOutPrototype addSpareFields makeFileOutFile )
		do: [:sel | Dangerous add: sel].
	#(recompile:from: recompileAllFrom: recompileChanges asPrototypeWithFields: asPrototype addInstanceVarNamed:withValue: addInstanceVariable addClassVarName: removeClassVarName: findOrAddClassVarName: instanceVariableNames: )
		do: [:sel | Dangerous add: sel].

 ! !

!MethodFinder methodsFor: 'initialize' stamp: 'tk 4/14/1999 11:16'!
organizationFiltered: aClass
	"Return the organization of the class with all selectors defined in superclasses removed.  (except those in Object)"

	| org str |
	org := aClass organization deepCopy.
	Dangerous do: [:sel |
			org removeElement: sel].
	Approved do: [:sel |
			org removeElement: sel].
	AddAndRemove do: [:sel |
			org removeElement: sel].
	str := org printString copyWithout: $(.
	str := '(', (str copyWithout: $) ).
	str := str replaceAll: $' with: $".
	^ str
! !

!MethodFinder methodsFor: 'arg maps' stamp: 'tk 5/24/1999 16:31'!
permuteArgs 
	"Run through ALL the permutations.  First one was as presented."

	data first size <= 1 ifTrue: [^ false].	"no other way"
	mapList ifNil: [self makeAllMaps].
	mapStage := mapStage + 1.
	mapStage > mapList size ifTrue: [^ false].
	argMap := mapList at: mapStage.
	self mapData.
	^ true
	! !

!MethodFinder methodsFor: 'search' stamp: 'tk 1/8/2001 17:53'!
searchForOne
	"Look for and return just one answer"

	expressions := OrderedCollection new.
	self search: false.	"non-multi"
	^ expressions
			! !

!MethodFinder methodsFor: 'search' stamp: 'md 2/22/2006 21:23'!
search: multi
	"if Multi is true, collect all selectors that work."
	selector := OrderedCollection new.	"list of them"
	self simpleSearch.
	multi not & (selector isEmpty not) ifTrue:[^ selector].

	[self permuteArgs] whileTrue:
		[self simpleSearch.
		multi not & (selector isEmpty not) ifTrue: [^ selector]].

	self insertConstants.
	"(selector isEmpty not) ifTrue: [^ selector]].    expression is the answer, not a selector"
	^ #()! !

!MethodFinder methodsFor: 'access' stamp: 'tk 1/4/2001 17:18'!
selectors
	"Note the inst var does not have an S on the end"

	^ selector! !

!MethodFinder methodsFor: 'search' stamp: 'tk 1/8/2001 18:17'!
simpleSearch
	"Run through first arg's class' selectors, looking for one that works."

| class supers listOfLists |
self exceptions.
class := thisData first first class.
"Cache the selectors for the receiver class"
(class == cachedClass and: [cachedArgNum = ((argMap size) - 1)]) 
	ifTrue: [listOfLists := cachedSelectorLists]
	ifFalse: [supers := class withAllSuperclasses.
		listOfLists := OrderedCollection new.
		supers do: [:cls |
			listOfLists add: (cls selectorsWithArgs: (argMap size) - 1)].
		cachedClass := class.
		cachedArgNum := (argMap size) - 1.
		cachedSelectorLists := listOfLists].
listOfLists do: [:selectorList |
	selectorList do: [:aSel |
		(selector includes: aSel) ifFalse: [
			((Approved includes: aSel) or: [AddAndRemove includes: aSel]) ifTrue: [
				(self testPerfect: aSel) ifTrue: [
					selector add: aSel.
					expressions add: (String streamContents: [:strm | 
						strm nextPutAll: 'data', argMap first printString.
						aSel keywords doWithIndex: [:key :ind |
							strm nextPutAll: ' ',key.
							(key last == $:) | (key first isLetter not)
								ifTrue: [strm nextPutAll: ' data', 
									(argMap at: ind+1) printString]]])
					]]]]].
! !

!MethodFinder methodsFor: 'initialize' stamp: 'ar 4/10/2005 18:48'!
test2: anArray
	"look for bad association"

	anArray do: [:sub |
		sub class == Association ifTrue: [
			(#('true' '$a' '2' 'false') includes: sub value printString) ifFalse: [
				self error: 'bad assn'].
			(#('3' '5.6' 'x' '''abcd''') includes: sub key printString) ifFalse: [
				self error: 'bad assn'].
		].
		sub class == Array ifTrue: [
			sub do: [:element | 
				element isString ifTrue: [element first asciiValue < 32 ifTrue: [
						self error: 'store into string in data']].
				element class == Association ifTrue: [
					element value class == Association ifTrue: [
						self error: 'bad assn']]]].
		sub class == Date ifTrue: [sub year isInteger ifFalse: [
				self error: 'stored into input date!!!!']].
		sub class == Dictionary ifTrue: [
				sub size > 0 ifTrue: [
					self error: 'store into dictionary']].
		sub class == OrderedCollection ifTrue: [
				sub size > 4 ifTrue: [
					self error: 'store into OC']].
		].! !

!MethodFinder methodsFor: 'initialize' stamp: 'tk 4/24/1999 19:34'!
test3
	"find the modification of the caracter table"

	(#x at: 1) asciiValue = 120 ifFalse: [self error: 'Character table mod'].! !

!MethodFinder methodsFor: 'initialize' stamp: 'nice 12/29/2010 14:49'!
testFromTuple: nth
	"verify that the methods allowed don't crash the system.  Try N of each of the fundamental types.  up to 4 of each kind." 

| objects nonRepeating even other aa cnt |
objects := #((1 4 17 42) ($a $b $c $d) ('one' 'two' 'three' 'four')
	(x + rect: new) ((a b 1 4) (c 1 5) ($a 3 d) ()) (4.5 0.0 3.2 100.3)
	).

objects := objects, {{true. false. true. false}. {Point. SmallInteger. Association. Array}.
	{Point class. SmallInteger class. Association class. Array class}.
	"{ 4 blocks }."
	{Date today. '1 Jan 1950' asDate. '25 Aug 1987' asDate. '1 Jan 2000' asDate}.
	{'15:16' asTime. '1:56' asTime. '4:01' asTime. '6:23' asTime}.
	{Dictionary new. Dictionary new. Dictionary new. Dictionary new}.
	{#(a b 1 4) asOrderedCollection. #(c 1 5) asOrderedCollection. 
		#($a 3 d) asOrderedCollection. #() asOrderedCollection}.
	{3 -> true. 5.6 -> $a. #x -> 2. 'abcd' -> false}.
	{9 @ 3 extent: 5 @ 4. 0 @ 0 extent: 45 @ 9. -3 @ -7 extent: 2 @ 2. 4 @ 4 extent: 16 @ 16}.
	{Color red.  Color blue. Color black. Color gray}}.

self test2: objects.
"rec+0, rec+1, rec+2, rec+3 need to be tested.  " 
cnt := 0.
nth to: 4 do: [:take |
	nonRepeating := OrderedCollection new.
	objects do: [:each |
		nonRepeating addAll: (each copyFrom: 1 to: take)].
	"all combinations of take, from nonRepeating"
	even := true.
	nonRepeating combinations: take atATimeDo: [:tuple |
		even ifTrue: [other := tuple clone]
			ifFalse: [self load: (aa := Array with: tuple with: 1 with: other with: 7).
				(cnt := cnt + 1) \\ 50 = 0 ifTrue: [
					Transcript cr; show: aa first printString].
				self search: true.
				self test2: aa.
				self test2: nonRepeating.
				"self test2: objects"].
		even := even not].
	].! !

!MethodFinder methodsFor: 'search' stamp: 'eem 7/21/2008 12:08'!
testPerfect: aSelector
	"Try this selector!! Return true if it answers every example perfectly.  Take the args in the order they are.  Do not permute them.  Survive errors.  later cache arg lists."

| sz argList val rec activeSel perform |
	"Transcript cr; show: aSelector.		debug"
perform := aSelector beginsWith: 'perform:'.
sz := argMap size.
1 to: thisData size do: [:ii | "each example set of args"
	argList := (thisData at: ii) copyFrom: 2 to: sz.
	perform
		ifFalse: [activeSel := aSelector]
		ifTrue: [activeSel := argList first.	"what will be performed"
			((Approved includes: activeSel) or: [AddAndRemove includes: activeSel])
				ifFalse: [^ false].	"not approved"
			aSelector == #perform:withArguments: 
				ifTrue: [activeSel numArgs = (argList at: 2) basicSize "avoid error" 
							ifFalse: [^ false]]
				ifFalse: [activeSel numArgs = (aSelector numArgs - 1) 
							ifFalse: [^ false]]].
	1 to: sz do: [:num | 
		(Blocks includes: (Array with: activeSel with: num)) ifTrue: [
			(argList at: num) isBlock ifFalse: [^ false]]].
	rec := (AddAndRemove includes: activeSel) 
			ifTrue: [(thisData at: ii) first isSymbol ifTrue: [^ false].
						"vulnerable to modification"
				(thisData at: ii) first copyTwoLevel] 	"protect from damage"
			ifFalse: [(thisData at: ii) first].
	val := [rec perform: aSelector withArguments: argList] 
				ifError: [:aString :aReceiver | 
							"self test3."
							"self test2: (thisData at: ii)."
							^ false].
	"self test3."
	"self test2: (thisData at: ii)."
	((answers at: ii) closeTo: val) ifFalse: [^ false].
	].
^ true! !

!MethodFinder methodsFor: 'initialize' stamp: 'ul 12/18/2014 10:31'!
testRandom
	"verify that the methods allowed don't crash the system.  Pick 3 or 4 from a mixed list of the fundamental types." 

	| objects other aa cnt take tuple fName sss random |
	objects := #(
		(1 4 17 42) ($a $b $c $d) ('one' 'two' 'three' 'four')
		(x + rect: new) ((a b 1 4) (c 1 5) ($a 3 d) ()) (4.5 0.0 3.2 100.3)
	).

	objects := objects, {{true. false. true. false}. {Point. SmallInteger. Association. Array}.
		{Point class. SmallInteger class. Association class. Array class}.
		"{ 4 blocks }."
		{Date today. '1 Jan 1950' asDate. '25 Aug 1987' asDate. '1 Jan 2000' asDate}.
		{'15:16' asTime. '1:56' asTime. '4:01' asTime. '6:23' asTime}.
		{Dictionary new. Dictionary new. Dictionary new. Dictionary new}.
		{#(a b 1 4) asOrderedCollection. #(c 1 5) asOrderedCollection. 
			#($a 3 d) asOrderedCollection. #() asOrderedCollection}.
		{3 -> true. 5.6 -> $a. #x -> 2. 'abcd' -> false}.
		{9 @ 3 extent: 5 @ 4. 0 @ 0 extent: 45 @ 9. -3 @ -7 extent: 2 @ 2. 4 @ 4 extent: 16 @ 16}.
		{Color red.  Color blue. Color black. Color gray}}.

	self test2: objects.
	"rec+0, rec+1, rec+2, rec+3 need to be tested.  " 
	fName := (FileDirectory default fileNamesMatching: '*.ran') first.
	sss := fName splitInteger first.
	random := Random seed: sss.
	cnt := 0.
	
	[take := #(3 4) atRandom: random.
	tuple := (1 to: take) collect: [:ind | (objects atRandom: random) atRandom: random].
	other := (1 to: take) collect: [:ind | (objects atRandom: random) atRandom: random].
	self load: (aa := Array with: tuple with: 1 with: other with: 7).
	((cnt := cnt + 1) \\ 10 = 0) " | (cnt > Skip)" ifTrue:
		[Transcript cr; show: cnt printString; tab; tab; show: aa first printString].
	cnt > (Smalltalk at: #StopHere) ifTrue: [self halt].		"stop just before crash"
	cnt > (Smalltalk at: #Skip)
		ifTrue:
			["skip this many at start"
			self search: true.
			self test2: aa first.  self test2: (aa at: 3).
			"self test2: objects"]] repeat.
	! !

!MethodFinder methodsFor: 'arg maps' stamp: 'tk 4/24/1999 19:29'!
thisData
	^ thisData ! !

!MethodFinder methodsFor: 'initialize' stamp: 'tk 5/18/2001 19:18'!
verify
	"Test a bunch of examples"
	"	MethodFinder new verify    "
Approved ifNil: [self initialize].	"Sets of allowed selectors"
(MethodFinder new load: #( (0) 0  (30) 0.5  (45) 0.707106  (90) 1)
	) searchForOne asArray = #('data1 degreeSin') ifFalse: [self error: 'should have found it'].
(MethodFinder new load:  { { true. [3]. [4]}. 3.  { false. [0]. [6]}. 6}
	) searchForOne asArray = #('data1 ifTrue: data2 ifFalse: data3') ifFalse: [
		self error: 'should have found it'].
(MethodFinder new load: {#(1). true. #(2). false. #(5). true. #(10). false}
	) searchForOne asArray = #('data1 odd') ifFalse: [self error: 'should have found it'].
		"will correct the date type of #true, and complain"
(MethodFinder new load: #((4 2) '2r100'   (255 16) '16rFF'    (14 8) '8r16')
	) searchForOne asArray = 
		#('data1 radix: data2' 'data1 printStringBase: data2' 'data1 storeStringBase: data2')
			  ifFalse: [self error: 'should have found it'].	
(MethodFinder new load: {{Point x: 3 y: 4}. 4.  {Point x: 1 y: 5}. 5}
	) searchForOne asArray = #('data1 y') ifFalse: [self error: 'should have found it'].	
(MethodFinder new load: #(('abcd') $a  ('TedK') $T)
	) searchForOne asArray = #('data1 asCharacter' 'data1 first' 'data1 anyOne')
		 ifFalse: [self error: 'should have found it'].	
(MethodFinder new load: #(('abcd' 1) $a  ('Ted ' 3) $d )
	) searchForOne asArray = #('data1 at: data2' 'data1 atPin: data2' 'data1 atWrap: data2')
		ifFalse: [self error: 'should have found it'].	
(MethodFinder new load: #(((12 4 8)) 24  ((1 3 6)) 10 )
	) searchForOne asArray=  #('data1 sum') ifFalse: [self error: 'should have found it'].	
		"note extra () needed for an Array object as an argument"

(MethodFinder new load: #((14 3) 11  (-10 5) -15  (4 -3) 7)
	) searchForOne asArray = #('data1 - data2') ifFalse: [self error: 'should have found it'].
(MethodFinder new load: #((4) 4  (-10) 10 (-3) 3 (2) 2 (-6) 6 (612) 612)
	) searchForOne asArray = #('data1 abs') ifFalse: [self error: 'should have found it'].
(MethodFinder new load: {#(4 3). true.  #(-7 3). false.  #(5 1). true.  #(5 5). false}
	) searchForOne asArray = #('data1 > data2') ifFalse: [self error: 'should have found it'].	
(MethodFinder new load: #((5) 0.2   (2) 0.5)
	) searchForOne asArray = #('data1 reciprocal') ifFalse: [self error: 'should have found it'].	
(MethodFinder new load: #((12 4 8) 2  (1 3 6) 2  (5 2 16) 8)
	) searchForOne asArray = #()     " '(data3 / data2) ' want to be able to leave out args"  
		ifFalse: [self error: 'should have found it'].	
(MethodFinder new load: #((0.0) 0.0  (1.5) 0.997495  (0.75) 0.681639)
	) searchForOne asArray = #('data1 sin') ifFalse: [self error: 'should have found it'].	
(MethodFinder new load: #((7 5) 2   (4 5) 4   (-9 4) 3)
	) searchForOne asArray = #('data1 \\ data2') ifFalse: [self error: 'should have found it'].	

(MethodFinder new load: #((7) 2   (4) 2 )
	) searchForOne asArray = #('^ 2')  ifFalse: [self error: 'should have found it'].	
(MethodFinder new load: {#(7). true.   #(4.1).  true.   #(1.5). false}
	) searchForOne asArray = #('data1 >= 4.1') ifFalse: [self error: 'should have found it'].	
(MethodFinder new load: #((35) 3   (17) 1   (5) 5)
	) searchForOne asArray = #('data1 \\ 8') ifFalse: [self error: 'should have found it'].	
(MethodFinder new load: #((36) 7   (50) 10 )
	) searchForOne asArray = #('data1 quo: 5' 'data1 // 5') ifFalse: [
		self error: 'should have found it'].	
(MethodFinder new load: #( ((2 3) 2) 8   ((2 3) 5) 17 )
	) searchForOne asArray = #('data1 polynomialEval: data2') ifFalse: [
		self error: 'should have found it'].	
(MethodFinder new load: #((2) 8   (5) 17 )
	) searchForOne asArray = #('#(2 3) polynomialEval: data1') ifFalse: [
		self error: 'should have found it'].	
! !

!Object methodsFor: '*Tools-inspecting' stamp: 'ar 9/27/2005 18:31'!
basicInspect
	"Create and schedule an Inspector in which the user can examine the 
	receiver's variables. This method should not be overriden."
	^ToolSet basicInspect: self! !

!Object methodsFor: '*Tools-Browsing' stamp: 'nice 11/3/2013 00:33'!
browse
	^ToolSet browseClass: self class! !

!Object methodsFor: '*Tools-MessageSets' stamp: 'cmm 2/19/2010 15:51'!
browseAllCallsOn: selectorSymbol
	"Models get the first chance to handle this, so a message-tracer can be built..  Not all Tool 'models' inherit from Model, otherwise this would be there."
	self systemNavigation browseAllCallsOn: selectorSymbol! !

!Object methodsFor: '*Tools-MessageSets' stamp: 'cmm 2/18/2010 21:51'!
browseAllImplementorsOf: selectorSymbol
	"Models get first chance to handle browseAllImplementorsOf:, so a tracing-messages browser can be built..  Not all Tool 'models' inherit from Model, otherwise this would be there."
	self systemNavigation browseAllImplementorsOf: selectorSymbol! !

!Object methodsFor: '*tools-browser' stamp: 'mu 3/11/2004 16:00'!
browseHierarchy
	self systemNavigation browseHierarchy: self class! !

!Object methodsFor: '*Tools-multi-window support' stamp: 'eem 6/30/2010 14:23'!
canHaveUnacceptedEdits
	"Answer if the receiver is an object that can hold unaccepted edits (such as a text editor widget)"

	^false! !

!Object methodsFor: '*Tools-Debugger' stamp: ''!
canonicalArgumentName
	^ self class name! !

!Object methodsFor: '*Tools-Debugger' stamp: 'mtf 8/19/2008 22:38'!
chasePointers
	PointerFinder on: self! !

!Object methodsFor: '*Tools-Explorer' stamp: 'ar 9/27/2005 20:29'!
explore
	^ToolSet explore: self! !

!Object methodsFor: '*Tools-Explorer' stamp: 'stephaneducasse 9/17/2005 21:52'!
exploreAndYourself
	"i.e. explore; yourself. Thisway i can peek w/o typing all the parentheses"
	self explore. 
     ^self! !

!Object methodsFor: '*Tools-Debugger' stamp: 'mt 4/2/2015 11:04'!
explorePointers
	PointerExplorer openOn: self.! !

!Object methodsFor: '*Tools-Explorer' stamp: 'mt 4/2/2015 11:03'!
exploreWithLabel: label

	^ ObjectExplorer openOn: self withLabel: label! !

!Object methodsFor: '*Tools-inspecting' stamp: 'mt 4/12/2015 16:29'!
inspect
	"Create and schedule an Inspector in which the user can examine the receiver's variables."
	^ ToolSet inspect: self! !

!Object methodsFor: '*Tools-inspecting' stamp: 'apb 7/14/2004 12:19'!
inspectorClass
	"Answer the class of the inspector to be used on the receiver.  Called by inspect; 
	use basicInspect to get a normal (less useful) type of inspector."

	^ Inspector! !

!Object methodsFor: '*Tools-inspecting' stamp: 'ar 9/27/2005 20:30'!
inspectWithLabel: aLabel
	"Create and schedule an Inspector in which the user can examine the receiver's variables."
	^ToolSet inspect: self label: aLabel! !

!Object methodsFor: '*Tools-error handling' stamp: 'ar 9/27/2005 20:24'!
notifyWithLabel: aString 
	"Create and schedule a Notifier with aString as the window label as well as the contents of the window, in  order to request confirmation before a process can proceed."

	ToolSet
		debugContext: thisContext
		label: aString
		contents: aString

	"nil notifyWithLabel: 'let us see if this works'"! !

!Object methodsFor: '*Tools-Debugger' stamp: 'cmm 2/3/2014 15:37'!
shouldFollowOutboundPointers
	"Allow other subclasses of ProtoObject to hook this method."
	(self class isPointers or: [ self isCompiledMethod ]) ifFalse: [ ^ false ].
	^ (self class isWeak not or: [ self class instSize > 0 ]) and: [ self isLiteral not ]! !

Object subclass: #PopUpMenu
	instanceVariableNames: 'labelString font lineArray frame form marker selection'
	classVariableNames: 'CacheMenuForms MenuStyle'
	poolDictionaries: ''
	category: 'Tools-Menus'!

!PopUpMenu commentStamp: '<historical>' prior: 0!
I represent a list of items. My instances are presented on the display screen in a rectangular area. The user points to an item, pressing a mouse button; the item is highlighted. When the button is released, the highlighted item indicates the selection.!

!PopUpMenu class methodsFor: 'class initialization' stamp: 'jla 4/2/2001 20:41'!
alignment

	^ MenuStyle alignment! !

!PopUpMenu class methodsFor: 'class initialization' stamp: 'jla 4/2/2001 20:46'!
alignment: anAlignment

	^ MenuStyle alignment: anAlignment! !

!PopUpMenu class methodsFor: 'class initialization' stamp: 'ar 2/26/2010 23:04'!
cleanUp
	"Re-initialize"

	self initialize.! !

!PopUpMenu class methodsFor: 'dialogs' stamp: 'dgd 9/5/2003 18:24'!
confirm: queryString
	"Put up a yes/no menu with caption queryString. Answer true if the 
	response is yes, false if no. This is a modal question--the user must 
	respond yes or no."

	"PopUpMenu confirm: 'Are you hungry?'"

	^ self confirm: queryString trueChoice: 'Yes' translated falseChoice: 'No' translated! !

!PopUpMenu class methodsFor: 'dialogs' stamp: 'dgd 10/12/2004 13:49'!
confirm: queryString orCancel: cancelBlock 
	"Put up a yes/no/cancel menu with caption aString. Answer 
	true if  
	the response is yes, false if no. If cancel is chosen, evaluate  
	cancelBlock. This is a modal question--the user must respond 
	yes or no."
	"PopUpMenu confirm: 'Reboot universe' orCancel: 
	[^'Nevermind'] "
	| menu choice |
	menu := PopUpMenu labelArray: {'Yes' translated. 'No' translated. 'Cancel' translated}.
	choice := menu startUpWithCaption: queryString icon: MenuIcons confirmIcon.
	choice = 1
		ifTrue: [^ true].
	choice = 2
		ifTrue: [^ false].
	^ cancelBlock value! !

!PopUpMenu class methodsFor: 'dialogs' stamp: 'dgd 10/12/2004 13:49'!
confirm: queryString trueChoice: trueChoice falseChoice: falseChoice 
	"Put up a yes/no menu with caption queryString. The actual 
	wording  
	for the two choices will be as provided in the trueChoice and  
	falseChoice parameters. Answer true if the response is the 
	true-choice,  
	false if it's the false-choice. 
	This is a modal question -- the user must respond one way or 
	the other."
	"PopUpMenu  
	confirm: 'Are you hungry?' 
	trueChoice: 'yes, I''m famished' 
	falseChoice: 'no, I just ate'"
	| menu choice |
	menu := PopUpMenu labelArray: {trueChoice. falseChoice}.
	[(choice := menu startUpWithCaption: queryString icon: MenuIcons confirmIcon) isNil] whileTrue.
	^ choice = 1! !

!PopUpMenu class methodsFor: 'dialogs' stamp: 'dtl 2/11/2010 23:08'!
inform: aString
	"PopUpMenu inform: 'I like Squeak'"

	| iconOrNil |
	iconOrNil := Smalltalk at: #MenuIcons ifPresent: [:cls | cls confirmIcon].
	(PopUpMenu labels: ' OK ' translated)
		startUpWithCaption: aString
		icon: iconOrNil
! !

!PopUpMenu class methodsFor: 'class initialization' stamp: 'sw 12/6/1999 13:08'!
initialize  "PopUpMenu initialize"
	(MenuStyle := TextStyle default copy)
		gridForFont: TextStyle default defaultFontIndex withLead: 0;
		centered.
	PopUpMenu allSubInstancesDo: [:m | m rescan]! !

!PopUpMenu class methodsFor: 'instance creation' stamp: 'sma 5/28/2000 15:44'!
labelArray: labelArray
	"Answer an instance of me whose items are in labelArray."

	^ self labelArray: labelArray lines: nil! !

!PopUpMenu class methodsFor: 'instance creation' stamp: 'sma 5/28/2000 15:43'!
labelArray: labelArray lines: lineArray
	"Answer an instance of me whose items are in labelArray, with lines 
	drawn after each item indexed by anArray. 2/1/96 sw"

	labelArray isEmpty ifTrue: [self error: 'Menu must not be zero size'].
	^ self
		labels: (String streamContents: 
			[:stream |
			labelArray do: [:each | stream nextPutAll: each; cr].
			stream skip: -1 "remove last CR"])
		lines: lineArray

"Example:
	(PopUpMenu labelArray: #('frog' 'and' 'toad') lines: #()) startUp"! !

!PopUpMenu class methodsFor: 'instance creation' stamp: 'sma 5/28/2000 15:36'!
labels: aString
	"Answer an instance of me whose items are in aString."

	^ self labels: aString lines: nil! !

!PopUpMenu class methodsFor: 'instance creation' stamp: 'sw 12/6/1999 17:55'!
labels: aString lines: anArray
	"Answer an instance of me whose items are in aString, with lines drawn 
	after each item indexed by anArray."

	^ self new
		labels: aString
		font: MenuStyle defaultFont
		lines: anArray! !

!PopUpMenu class methodsFor: 'class initialization' stamp: 'jla 4/2/2001 20:56'!
leftFlush

	MenuStyle leftFlush! !

!PopUpMenu class methodsFor: 'dialogs' stamp: 'sma 5/28/2000 15:57'!
notify: message
	"Deprecated. Use #inform: instead."

	self inform: message! !

!PopUpMenu class methodsFor: 'class initialization' stamp: 'nk 9/1/2004 10:27'!
setMenuFontTo: aFont
	"Set the menu font as indicated"

	MenuStyle := TextStyle fontArray: { aFont }.
	MenuStyle 
		gridForFont: 1 withLead: 0;
		centered.
	self allSubInstancesDo: [:m | m rescan]! !

!PopUpMenu class methodsFor: 'instance creation' stamp: 'nk 8/30/2004 07:59'!
withCaption: cap chooseFrom: labels 
	"Simply put up a menu. Get the args in the right order with the caption 
	first. labels may be either an array of items or a string with CRs in it. 
	May use backslashes for returns."

	^ (labels isString
		ifTrue: [self labels: labels withCRs lines: nil]
		ifFalse: [self labelArray: labels lines: nil])
		startUpWithCaption: cap withCRs! !

!PopUpMenu methodsFor: 'accessing' stamp: 'sma 5/28/2000 11:44'!
center
	"Answer the point at the center of the receiver's rectangular area."

	^ frame center! !

!PopUpMenu methodsFor: 'private' stamp: 'BG 8/6/2003 12:34'!
computeForm
	"Compute and answer a Form to be displayed for this menu."

	| borderInset paraForm menuForm inside |
	borderInset := 4@4.
	paraForm := (DisplayText text: labelString asText textStyle: MenuStyle) form.
	menuForm := Form extent: paraForm extent + (borderInset * 2) depth: paraForm depth.
      menuForm fill: (0 @ 0 extent: menuForm  extent)
                        rule: Form over
                        fillColor: Color white.
	menuForm borderWidth: 2.
	paraForm displayOn: menuForm at: borderInset.
	lineArray == nil ifFalse:
		[lineArray do:
			[ :line |
			menuForm fillBlack: (4 @ ((line * font height) + borderInset y)
				extent: (menuForm width - 8 @ 1))]].

	frame := Quadrangle new.
	frame region: menuForm boundingBox.
	frame borderWidth: 4.
	inside := frame inside.
	marker := inside topLeft extent: (inside width @ MenuStyle lineGrid).
	selection := 1.

	^ form := menuForm
! !

!PopUpMenu methodsFor: 'basic control sequence' stamp: 'sma 6/1/2000 13:04'!
controlActivity
	"Do whatever a menu must do - now with keyboard support."

	| didNotMove downPos |
	didNotMove := true.
	Sensor anyButtonPressed
		ifFalse:
			[didNotMove := false.
			Sensor waitButtonOrKeyboard]. 
	
	Sensor keyboardPressed ifFalse: [self manageMarker].
	(didNotMove and: [selection = 0])
		ifTrue:
			[downPos := Sensor cursorPoint.
			[didNotMove and: [Sensor anyButtonPressed]]
				whileTrue:
					[(downPos dist: Sensor cursorPoint) < 2 ifFalse: [didNotMove := false]].
			didNotMove ifTrue: [Sensor waitButtonOrKeyboard]].

	[Sensor keyboardPressed] whileTrue:
		[self readKeyboard ifTrue: [^ self].
		Sensor waitButtonOrKeyboard].

	[Sensor anyButtonPressed] whileTrue: [self manageMarker]! !

!PopUpMenu methodsFor: 'displaying' stamp: 'ul 8/2/2011 20:07'!
displayAt: aPoint withCaption: captionOrNil during: aBlock
	"Display the receiver just to the right of aPoint while aBlock is evaluated.  If the receiver is forced off screen, display it just to the right."
	| delta savedArea captionForm captionSave outerFrame captionText tFrame frameSaveLoc captionBox |
	marker ifNil: [self computeForm].
	frame := frame align: marker leftCenter with: aPoint + (2@0).
	outerFrame := frame.
	captionOrNil notNil ifTrue:
		[captionText := (DisplayText
				text: captionOrNil asText
				textStyle: MenuStyle copy centered)
					foregroundColor: Color black
					backgroundColor: Color white.
		tFrame := captionText boundingBox insetBy: -2.
		outerFrame := frame merge: (tFrame align: tFrame bottomCenter
					with: frame topCenter + (0@2))].
	delta := outerFrame amountToTranslateWithin: Display boundingBox.
	frame right > Display boundingBox right
		ifTrue: [delta := 0 - frame width @ delta y].
	frame := frame translateBy: delta.
	captionOrNil notNil ifTrue:
		[captionForm := captionText form.
		captionBox := captionForm boundingBox expandBy: 4.
		captionBox := captionBox align: captionBox bottomCenter
								with: frame topCenter + (0@2).
		captionSave := Form fromDisplay: captionBox.
		Display border: captionBox width: 4 fillColor: Color white.
		Display border: captionBox width: 2 fillColor: Color black.
		captionForm displayAt: captionBox topLeft + 4].
	marker := marker align: marker leftCenter with: aPoint + delta +  (2@0).
	savedArea := Form fromDisplay: frame.
	self menuForm displayOn: Display at: (frameSaveLoc := frame topLeft).
	selection ~= 0 ifTrue: [Display reverse: marker].
	Cursor normal showWhile: aBlock.
	savedArea displayOn: Display at: frameSaveLoc.
	captionOrNil notNil ifTrue:
		[captionSave displayOn: Display at: captionBox topLeft]! !

!PopUpMenu methodsFor: 'accessing' stamp: 'di 4/20/1999 14:33'!
frameHeight
	"Designed to avoid the entire frame computation (includes MVC form),
	since the menu may well end up being displayed in Morphic anyway."
	| nItems |
	frame ifNotNil: [^ frame height].
	nItems := 1 + (labelString occurrencesOf: Character cr).
	^ (nItems * MenuStyle lineGrid) + 4 "border width"! !

!PopUpMenu methodsFor: 'accessing' stamp: 'sma 5/28/2000 14:55'!
labelString
	^ labelString! !

!PopUpMenu methodsFor: 'private' stamp: 'di 4/13/1999 16:21'!
labels: aString font: aFont lines: anArray

	labelString := aString.
	font := aFont.
	lineArray := anArray.
! !

!PopUpMenu methodsFor: 'accessing' stamp: 'sma 5/28/2000 14:55'!
lineArray
	^ lineArray! !

!PopUpMenu methodsFor: 'marker adjustment' stamp: 'di 4/13/1999 17:42'!
manageMarker
	"If the cursor is inside the receiver's frame, then highlight the marked 
	item. Otherwise no item is to be marked."
	| pt |
	"Don't let pt get far from display box, so scrolling will go all the way"
	pt := Sensor cursorPoint adhereTo: (Display boundingBox expandBy: 1).
	(frame inside containsPoint: pt)
		ifTrue: ["Need to cache the form for reasonable scrolling performance"
				((Display boundingBox insetBy: 0@3) containsPoint: pt)
					ifFalse: [pt := pt - (self scrollIntoView: pt)].
				self markerOn: pt]
		ifFalse: [self markerOff]! !

!PopUpMenu methodsFor: 'marker adjustment' stamp: 'sma 5/28/2000 15:27'!
markerOff
	"No item is selected. Reverse the highlight if any item has been marked 
	as selected."

	self setSelection: 0! !

!PopUpMenu methodsFor: 'marker adjustment' stamp: 'sma 6/1/2000 13:01'!
markerOn: aPoint 
	"The item whose bounding area contains aPoint should be marked as 
	selected. Highlight its area and set the selection to its index."

	selection = 0 | (marker containsPoint: aPoint) not 
		ifTrue: [selection = 0 & (marker containsPoint: aPoint)
					ifTrue: [Display reverse: marker]
					ifFalse: 
						[selection > 0 ifTrue: [Display reverse: marker].
						marker := 
							marker 
								align: marker topLeft 
								with: marker left @ (self markerTop: aPoint).
						Display reverse: marker]].
	selection := marker top - frame top // marker height + 1! !

!PopUpMenu methodsFor: 'marker adjustment' stamp: ''!
markerTop: aPoint 
	"Answer aPoint, gridded to lines in the receiver."

	^(aPoint y - frame inside top truncateTo: font height) + frame inside top! !

!PopUpMenu methodsFor: 'private' stamp: 'di 4/13/1999 17:51'!
menuForm
	"Answer a Form to be displayed for this menu."

	form == nil ifTrue: [self computeForm].
	^ form! !

!PopUpMenu methodsFor: 'accessing' stamp: 'sma 5/28/2000 12:32'!
nItems
	^ (labelString occurrencesOf: Character cr) + 1! !

!PopUpMenu methodsFor: 'basic control sequence' stamp: 'nice 6/11/2010 21:53'!
readKeyboard
	"Keyboard support for menus. ESC will abort the menu, Space or CR
	will select an item. Cursor up and cursor down will change the
	selection. Any other key will either select an item whose label starts
	with that character or select the next matching label.
	Answer true if the menu should be closed and false otherwise."

	| ch labels occurences |
	ch := Sensor keyboard asciiValue.
	(ch = 13 or: [ch = 32]) ifTrue: [^ true].
	ch = 27 ifTrue: [self setSelection: 0. ^ true].
	ch = 30
		ifTrue:
			[self setSelection: (selection <= 1
				ifTrue: [self nItems]
				ifFalse: [selection - 1])].
	ch = 31 ifTrue: [self setSelection: selection \\ self nItems + 1].
	ch := ch asCharacter asLowercase.
	labels := labelString lines.
	occurences := 0.
	1 + selection to: selection + labels size do:
		[:index |
		| i | i := index - 1 \\ labels size + 1.
		(labels at: i) withBlanksTrimmed first asLowercase = ch
			ifTrue: [(occurences := occurences + 1) = 1 ifTrue: [self setSelection: i]]].
	^ occurences = 1! !

!PopUpMenu methodsFor: 'private' stamp: 'sma 2/5/2000 11:56'!
rescan
	"Cause my form to be recomputed after a font change."

	labelString == nil ifTrue: [labelString := 'NoText!!'].
	self labels: labelString font: (MenuStyle fontAt: 1) lines: lineArray.
	frame := marker := form := nil.

	"PopUpMenu allSubInstancesDo: [:m | m rescan]"! !

!PopUpMenu methodsFor: 'marker adjustment' stamp: 'di 3/9/98 19:46'!
scrollIntoView: cursorLoc
	| dy |
	dy := 0.
	cursorLoc y < 2 ifTrue: [dy := font height].
	cursorLoc y > (Display height-3) ifTrue: [dy := font height negated].
	dy = 0 ifTrue: [^ 0@0].
	self markerOff.
	frame := frame translateBy: 0@dy.
	marker := marker translateBy: 0@dy.
	self menuForm displayOn: Display at: frame topLeft.
	^ 0@dy! !

!PopUpMenu methodsFor: 'selecting' stamp: 'sma 5/28/2000 12:27'!
selection
	"Answer the current selection."

	^ selection! !

!PopUpMenu methodsFor: 'selecting' stamp: 'sma 6/1/2000 11:01'!
setSelection: index
	| newSelection |
	selection = index ifTrue: [^ self].
	newSelection := (0 max: index) min: frame height // marker height.
	selection > 0 ifTrue: [Display reverse: marker].
	marker := marker translateBy: 0 @ (newSelection - selection * marker height).
	selection := newSelection.
	selection > 0 ifTrue: [Display reverse: marker]! !

!PopUpMenu methodsFor: 'basic control sequence' stamp: ''!
startUp
	"Display and make a selection from the receiver as long as the button 
	is pressed. Answer the current selection."
	
	^ self startUpWithCaption: nil! !

!PopUpMenu methodsFor: 'basic control sequence' stamp: 'ar 3/18/2001 00:55'!
startUpCenteredWithCaption: captionOrNil
	"Differs from startUpWithCaption: by appearing with cursor in the menu,
	and thus ready to act on mouseUp, without requiring user tweak to confirm"
	^ self startUpWithCaption: captionOrNil at: (ActiveHand ifNil:[Sensor]) cursorPoint - (20@0)! !

!PopUpMenu methodsFor: 'basic control sequence' stamp: 'dtl 1/30/2010 16:16'!
startUpLeftFlush
	"Build and invoke this menu with no initial selection.  By Jerry Archibald, 4/01.
	If in MVC, align menus items with the left margin.
	Answer the selection associated with the menu item chosen by the user or nil if none is chosen.  
	The mechanism for getting left-flush appearance in mvc leaves a tiny possibility for misadventure: if the user, in mvc, puts up the jump-to-project menu, then hits cmd period while it is up, then puts up a second jump-to-project menu before dismissing or proceeding through the debugger, it's possible for mvc popup-menus thereafter to appear left-aligned rather than centered; this very unlikely condition can be cleared by evaluating 'PopUpMenu alignment: 2'"

	^ Project current
		dispatchTo: self
		addPrefixAndSend: #StartUpLeftFlush
		withArguments: #()
! !

!PopUpMenu methodsFor: 'basic control sequence' stamp: 'ul 6/17/2011 12:40'!
startUpSegmented: segmentHeight withCaption: captionOrNil at: location
	"This menu is too big to fit comfortably on the screen.
	Break it up into smaller chunks, and manage the relative indices.
	Inspired by a special-case solution by Reinier van Loon."
"
(PopUpMenu labels: (String streamContents: [:s | 1 to: 100 do: [:i | s print: i; cr]. s skip: -1])
		lines: (5 to: 100 by: 5)) startUpWithCaption: 'Give it a whirl...'.
"
	| nLines nLinesPer allLabels from to subset subLines index |
	frame ifNil: [self computeForm].
	allLabels := labelString lines.
	nLines := allLabels size.
	lineArray ifNil: [lineArray := Array new].
	nLinesPer := segmentHeight // marker height - 3.
	from := 1.
	[to := (from + nLinesPer) min: nLines.
	subset := allLabels copyFrom: from to: to.
	subset add: (to = nLines ifTrue: ['start over...' translated] ifFalse: ['more...' translated])
		before: subset first.
	subLines := lineArray select: [:n | n >= from] thenCollect: [:n | n - (from-1) + 1].
	subLines := (Array with: 1) , subLines.
	index := (PopUpMenu labels: subset asStringWithCr lines: subLines)
				startUpWithCaption: captionOrNil at: location.
	index = 1
		ifTrue: [from := to + 1.
				from > nLines ifTrue: [ from := 1 ]]
		ifFalse: [index = 0 ifTrue: [^ 0].
				^ from + index - 2]] repeat! !

!PopUpMenu methodsFor: 'basic control sequence' stamp: 'ul 6/17/2011 12:41'!
startUpSegmented: segmentHeight withCaption: captionOrNil at: location allowKeyboard: aBoolean
	"This menu is too big to fit comfortably on the screen.
	Break it up into smaller chunks, and manage the relative indices.
	Inspired by a special-case solution by Reinier van Loon.  The boolean parameter indicates whether the menu should be given keyboard focus (if in morphic)"

"
(PopUpMenu labels: (String streamContents: [:s | 1 to: 100 do: [:i | s print: i; cr]. s skip: -1])
		lines: (5 to: 100 by: 5)) startUpWithCaption: 'Give it a whirl...'.
"
	| nLines nLinesPer allLabels from to subset subLines index |
	frame ifNil: [self computeForm].
	allLabels := labelString lines.
	nLines := allLabels size.
	lineArray ifNil: [lineArray := Array new].
	nLinesPer := segmentHeight // marker height - 3.
	from := 1.
	[to := (from + nLinesPer) min: nLines.
	subset := (allLabels copyFrom: from to: to) asOrderedCollection.
	subset add: (to = nLines ifTrue: ['start over...' translated] ifFalse: ['more...' translated])
		before: subset first.
	subLines := lineArray select: [:n | n >= from] thenCollect: [:n | n - (from-1) + 1].
	subLines := (Array with: 1) , subLines.
	index := (PopUpMenu labels: subset asStringWithCr lines: subLines)
				startUpWithCaption: captionOrNil at: location allowKeyboard: aBoolean.
	index = 1
		ifTrue: [from := to + 1.
				from > nLines ifTrue: [ from := 1 ]]
		ifFalse: [index = 0 ifTrue: [^ 0].
				^ from + index - 2]] repeat! !

!PopUpMenu methodsFor: 'basic control sequence' stamp: 'mt 8/16/2016 15:43'!
startUpWithCaption: captionOrNil
	"Display the menu, slightly offset from the cursor,
	so that a slight tweak is required to confirm any action."
	self flag: #fix. "mt: Could we manage to open pop-up menus in Morphic without accessing ActiveHand?"
	^ self startUpWithCaption: captionOrNil at: (ActiveHand ifNil:[Sensor]) cursorPoint! !

!PopUpMenu methodsFor: 'basic control sequence' stamp: 'ar 12/27/2001 22:47'!
startUpWithCaption: captionOrNil at: location
	"Display the menu, with caption if supplied. Wait for the mouse button to go down,
	then track the selection as long as the button is pressed. When the button is released, 
	answer the index of the current selection, or zero if the mouse is not released over 
	any menu item. Location specifies the desired topLeft of the menu body rectangle."

		^ self startUpWithCaption: captionOrNil at: location allowKeyboard: Preferences menuKeyboardControl! !

!PopUpMenu methodsFor: 'basic control sequence' stamp: 'dgd 4/4/2006 14:47'!
startUpWithCaption: captionOrNil at: location allowKeyboard: aBoolean
	"Display the menu, with caption if supplied. Wait for the mouse button to go down, then track the selection as long as the button is pressed. When the button is released,
	Answer the index of the current selection, or zero if the mouse is not released over  any menu item. Location specifies the desired topLeft of the menu body rectangle. The final argument indicates whether the menu should seize the keyboard focus in order to allow the user to navigate it via the keyboard."

	^ self
			startUpWithCaption: captionOrNil
			icon: nil
			at: location
			allowKeyboard: aBoolean! !

!PopUpMenu methodsFor: 'basic control sequence' stamp: 'dgd 10/12/2004 13:43'!
startUpWithCaption: captionOrNil icon: aForm
	"Display the menu, slightly offset from the cursor,
	so that a slight tweak is required to confirm any action."
	^ self
			startUpWithCaption: captionOrNil
			icon: aForm
			at: (ActiveHand ifNil:[Sensor]) cursorPoint
! !

!PopUpMenu methodsFor: 'basic control sequence' stamp: 'dgd 10/12/2004 13:44'!
startUpWithCaption: captionOrNil icon: aForm at: location
	"Display the menu, with caption if supplied. Wait for the mouse button to go down,
	then track the selection as long as the button is pressed. When the button is released, 
	answer the index of the current selection, or zero if the mouse is not released over 
	any menu item. Location specifies the desired topLeft of the menu body rectangle."

	^ self
			startUpWithCaption: captionOrNil
			icon: aForm
			at: location
			allowKeyboard: Preferences menuKeyboardControl
! !

!PopUpMenu methodsFor: 'basic control sequence' stamp: 'dtl 1/30/2010 16:10'!
startUpWithCaption: captionOrNil icon: aForm at: location allowKeyboard: aBoolean
	"Display the menu, with caption if supplied. Wait for the mouse button to go down, then track the selection as long as the button is pressed. When the button is released,
	Answer the index of the current selection, or zero if the mouse is not released over  any menu item. Location specifies the desired topLeft of the menu body rectangle. The final argument indicates whether the menu should seize the keyboard focus in order to allow the user to navigate it via the keyboard."

	| maxHeight |
	(ProvideAnswerNotification signal: captionOrNil) ifNotNil:
		[:answer | ^ selection := answer ifTrue: [1] ifFalse: [2]].
		 
	maxHeight := Display height*3//4.
	self frameHeight > maxHeight ifTrue:
		[^ self
			startUpSegmented: maxHeight
			withCaption: captionOrNil
			at: location
			allowKeyboard: aBoolean].

	^ Project current
		dispatchTo: self
		addPrefixAndSend: #StartUpWithCaption:icon:at:allowKeyboard:
		withArguments: {captionOrNil. aForm. location. aBoolean}! !

!PopUpMenu methodsFor: 'basic control sequence' stamp: 'sw 12/17/2001 17:01'!
startUpWithoutKeyboard
	"Display and make a selection from the receiver as long as the button  is pressed. Answer the current selection.  Do not allow keyboard input into the menu"
	
	^ self startUpWithCaption: nil at: ((ActiveHand ifNil:[Sensor]) cursorPoint) allowKeyboard: false! !

PopUpMenu subclass: #SelectionMenu
	instanceVariableNames: 'selections'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Menus'!

SelectionMenu subclass: #CustomMenu
	instanceVariableNames: 'labels dividers lastDivider title targets arguments'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Menus'!

!CustomMenu commentStamp: 'nice 3/24/2010 07:36' prior: 0!
I am used to build menus on-the-fly. I maintain lists of menu items, actions (usually symbols to be sent as messages), and menu section dividing lines to which my clients may append new entries and lines by sending me the messages:

	add: aString action: anAction
	addLine

After the menu is constructed, it may be invoked with one of the following messages:

	startUp: initialSelection
	startUp

I am a subclass of ActionMenu, so I inherit a number of instance variables. The ones I am particularly concerned with are:

	items := an OrderedCollection of strings to appear in the menu
	selectors := an OrderedCollection of Symbols to be used as message selectors
	lineArray := an OrderedCollection of line positions
	lastLine := used to keep track of the last line to avoid making duplicate entries in lineArray!

!CustomMenu class methodsFor: 'example' stamp: 'sw 11/8/1999 17:27'!
example
	"CustomMenu example"

	| menu |
	menu := CustomMenu new.
	menu add: 'apples' action: #apples.
	menu add: 'oranges' action: #oranges.
	menu addLine.
	menu addLine.  "extra lines ignored"
	menu add: 'peaches' action: #peaches.
	menu addLine.
	menu add: 'pears' action: #pears.
	menu addLine.
	^ menu startUp: #apples


"NB:  The following is equivalent to the above, but uses the compact #fromArray: consruct:
	(CustomMenu fromArray:
		#(	('apples'		apples)
			('oranges'		oranges)
			-
			-
			('peaches'		peaches)
			-
			('pears'			pears)
			-))
				startUp: #apples"! !

!CustomMenu methodsFor: 'construction' stamp: ''!
addLine
	"Append a line to the menu after the last entry. Suppress duplicate lines."

	(lastDivider ~= selections size) ifTrue: [
		lastDivider := selections size.
		dividers addLast: lastDivider].! !

!CustomMenu methodsFor: 'construction' stamp: 'sw 2/27/2001 07:52'!
addList: listOfTuplesAndDashes
	"Add a menu item to the receiver for each tuple in the given list of the form (<what to show> <selector>). Add a line for each dash (-) in the list.  The tuples may have an optional third element, providing balloon help for the item, but such an element is ignored in mvc."

	listOfTuplesAndDashes do: [:aTuple |
		aTuple == #-
			ifTrue: [self addLine]
			ifFalse: [self add: aTuple first action: aTuple second]]

	"CustomMenu new addList: #(
		('apples' buyApples)
		('oranges' buyOranges)
		-
		('milk' buyMilk)); startUp"

! !

!CustomMenu methodsFor: 'compatibility' stamp: 'nk 2/15/2004 16:02'!
addServices2: services for: served extraLines: linesArray

	services withIndexDo: [:service :i |
		service addServiceFor: served toMenu: self.
		(linesArray includes: i)  ifTrue: [self addLine] ]! !

!CustomMenu methodsFor: 'compatibility' stamp: 'sumim 2/10/2002 01:20'!
addServices: services for: served extraLines: linesArray

	services withIndexDo: [:service :i |
		self addService: service for: served.
		(linesArray includes: i) | service useLineAfter 
			ifTrue: [self addLine]]! !

!CustomMenu methodsFor: 'compatibility' stamp: 'nk 2/15/2004 16:19'!
addService: aService for: serviceUser
	"Append a menu item with the given service. If the item is selected, it will perform the given service."

	aService addServiceFor: serviceUser toMenu: self.! !

!CustomMenu methodsFor: 'construction' stamp: 'sw 8/12/2002 17:14'!
addStayUpItem
	"For compatibility with MenuMorph.  Here it is a no-op"! !

!CustomMenu methodsFor: 'construction' stamp: 'dtl 2/27/2010 07:22'!
addStayUpItemSpecial
	"For compatibility with MenuMorph.  Here it is a no-op"! !

!CustomMenu methodsFor: 'construction' stamp: 'dtl 2/27/2010 08:22'!
addTitle: aString
	"For compatibility with MenuMorph.  Here it is a no-op"! !

!CustomMenu methodsFor: 'construction' stamp: 'nk 11/25/2003 10:00'!
addTranslatedList: listOfTuplesAndDashes
	"Add a menu item to the receiver for each tuple in the given list of the form (<what to show> <selector>). Add a line for each dash (-) in the list.  The tuples may have an optional third element, providing balloon help for the item, but such an element is ignored in mvc.
	The first element will be translated."

	listOfTuplesAndDashes do: [:aTuple |
		aTuple == #-
			ifTrue: [self addLine]
			ifFalse: [self add: aTuple first translated action: aTuple second]]

	"CustomMenu new addTranslatedList: #(
		('apples' buyApples)
		('oranges' buyOranges)
		-
		('milk' buyMilk)); startUp"

! !

!CustomMenu methodsFor: 'construction' stamp: 'dhhi 9/14/2000 22:39'!
add: aString action: actionItem
	"Add the given string as the next menu item. If it is selected, the given action (usually but not necessarily a symbol) will be returned to the client."

	| s |
	aString ifNil: [^ self addLine].
	s := String new: aString size + 2.
	s at: 1 put: Character space.
	s replaceFrom: 2 to: s size - 1 with: aString.
	s at: s size put: Character space.
	labels addLast: s.
	selections addLast: actionItem.! !

!CustomMenu methodsFor: 'compatibility' stamp: 'ads 2/20/2003 08:59'!
add: aString subMenu: aMenu target: target selector: aSymbol argumentList: argList
	"Create a sub-menu with the given label. This isn't really a sub-menu the way Morphic does it; it'll just pop up another menu."

	self
		add: aString
		target: aMenu
		selector: #invokeOn:
		argumentList: argList asArray.! !

!CustomMenu methodsFor: 'compatibility' stamp: 'mt 4/12/2015 20:35'!
add: aString target: target selector: aSymbol 

	self add: aString
		target: target
		selector: aSymbol
		argumentList: #()! !

!CustomMenu methodsFor: 'compatibility' stamp: 'sumim 2/10/2002 01:18'!
add: aString target: target selector: aSymbol argumentList: argList
	"Append a menu item with the given label. If the item is selected, it will send the given selector to the target object with the given arguments. If the selector takes one more argument than the number of arguments in the given list, then the triggering event is supplied as as the last argument."

	self add: aString action: aSymbol.
	targets addLast: target.
	arguments addLast: argList asArray
! !

!CustomMenu methodsFor: 'compatibility' stamp: 'sumim 2/10/2002 01:23'!
add: aString target: target selector: aSymbol argument: arg
	"Append a menu item with the given label. If the item is selected, it will send the given selector to the target object with the given argument."

	self add: aString
		target: target
		selector: aSymbol
		argumentList: (Array with: arg)! !

!CustomMenu methodsFor: 'compatibility' stamp: 'sw 2/16/2002 00:57'!
arguments
	"Answer my arguments, initializing them to an empty collection if they're found to be nil."

	^ arguments ifNil: [arguments := OrderedCollection new]! !

!CustomMenu methodsFor: 'construction' stamp: 'sw 7/20/1999 18:47'!
balloonTextForLastItem: aString
	"Vacuous backstop provided for compatibility with MorphicMenu"! !

!CustomMenu methodsFor: 'private' stamp: 'sw 12/10/1999 11:21'!
build
	"Turn myself into an invokable ActionMenu."

	| stream |
	stream := WriteStream on: (String new).
	labels do: [:label | stream nextPutAll: label; cr].
	(labels isEmpty) ifFalse: [stream skip: -1].  "remove final cr"
	super labels: stream contents
		font: MenuStyle defaultFont
		lines: dividers! !

!CustomMenu methodsFor: 'initialize-release' stamp: 'sumim 2/10/2002 01:26'!
initialize

	labels := OrderedCollection new.
	selections := OrderedCollection new.
	dividers := OrderedCollection new.
	lastDivider := 0.
	targets := OrderedCollection new.
	arguments := OrderedCollection new	! !

!CustomMenu methodsFor: 'invocation' stamp: 'sw 2/17/2002 04:48'!
invokeOn: targetObject
	"Pop up this menu and return the result of sending to the target object the selector corresponding to the menu item selected by the user. Return nil if no item is selected.  If the chosen selector has arguments, obtain them from my arguments"

	^ self invokeOn: targetObject orSendTo: nil! !

!CustomMenu methodsFor: 'invocation' stamp: 'jm 11/17/97 16:54'!
invokeOn: targetObject defaultSelection: defaultSelection
	"Invoke the menu with the given default selection (i.e. one of my 'action' symbols). Answer the 'action' selector associated with the menu item chosen by the user or nil if none is chosen."

	| sel |
	sel := self startUp: defaultSelection.
	sel = nil ifFalse: [
		sel numArgs = 0
			ifTrue: [^ targetObject perform: sel]
			ifFalse: [^ targetObject perform: sel with: nil]].
	^ nil
! !

!CustomMenu methodsFor: 'invocation' stamp: 'ul 11/15/2010 11:35'!
invokeOn: targetObject orSendTo: anObject
	"Pop up this menu and return the result of sending to the target object the selector corresponding to the menu item selected by the user. Return  nil if no item is selected.  If the chosen selector has arguments, obtain appropriately.  If the recipient does not respond to the resulting message, send it to the alternate object provided"

	| aSelector anIndex recipient |
	^ (aSelector := self startUp) ifNotNil:
		[anIndex := self selection.
		recipient := ((targets := self targets) isEmptyOrNil or: [anIndex > targets size])
			ifTrue:
				[targetObject]
			ifFalse:
				[targets at: anIndex].
		aSelector numArgs = 0
			ifTrue:
				[recipient perform: aSelector orSendTo: anObject]
			ifFalse:
				[recipient perform: aSelector withArguments: (self arguments at: anIndex)]]! !

!CustomMenu methodsFor: 'construction' stamp: 'nice 6/11/2010 21:44'!
labels: aString font: aFont lines: anArrayOrNil
	"This method allows the receiver to accept old-style SelectionMenu creation messages. It should be used only for backward compatibility during the MVC-to-Morphic transition. New code should be written using the other menu construction protocol such as addList:."

	| labelList linesArray |
	labelList := aString lines asArray.
	anArrayOrNil
		ifNil: [linesArray := #()]
		ifNotNil: [linesArray := anArrayOrNil].
	1 to: labelList size do: [:i |
		self add: (labelList at: i) action: (labelList at: i).
		(linesArray includes: i) ifTrue: [self addLine]].
	font ifNotNil: [font := aFont].
! !

!CustomMenu methodsFor: 'construction' stamp: 'nice 6/11/2010 21:50'!
labels: labelList lines: linesArray selections: selectionsArray
	"This method allows the receiver to accept old-style SelectionMenu creation messages. It should be used only for backward compatibility during the MVC-to-Morphic transition. New code should be written using the other menu construction protocol such as addList:."
	"Labels can be either a sting with embedded crs, or a collection of strings."

	| labelArray |
	labelList isString
		ifTrue: [labelArray := labelList lines]
		ifFalse: [labelArray := labelList].
	1 to: labelArray size do: [:i |
		self add: (labelArray at: i) action: (selectionsArray at: i).
		(linesArray includes: i) ifTrue: [self addLine]].
! !

!CustomMenu methodsFor: 'private' stamp: 'di 4/14/1999 21:28'!
preSelect: action
	"Pre-select and highlight the menu item associated with the given action."

	| i |
	i := selections indexOf: action ifAbsent: [^ self].
	marker ifNil: [self computeForm].
	marker := marker
		align: marker topLeft
		with: (marker left)@(frame inside top + (marker height * (i - 1))).
	selection := i.! !

!CustomMenu methodsFor: 'invocation' stamp: ''!
startUp
	"Build and invoke this menu with no initial selection. Answer the selection associated with the menu item chosen by the user or nil if none is chosen."

	^ self startUp: nil! !

!CustomMenu methodsFor: 'invocation' stamp: 'sw 7/31/97 19:31'!
startUpWithCaption: caption
	"Build and invoke this menu with no initial selection. Answer the selection associated with the menu item chosen by the user or nil if none is chosen; use the provided caption"

	^ self startUp: nil withCaption: caption! !

!CustomMenu methodsFor: 'invocation' stamp: 'sw 8/18/1998 12:01'!
startUp: initialSelection
	"Build and invoke this menu with the given initial selection. Answer the selection associated with the menu item chosen by the user or nil if none is chosen."

	^ self startUp: initialSelection withCaption: title! !

!CustomMenu methodsFor: 'invocation' stamp: ''!
startUp: initialSelection withCaption: caption
	"Build and invoke this menu with the given initial selection and caption. Answer the selection associated with the menu item chosen by the user or nil if none is chosen."

	self build.
	(initialSelection notNil) ifTrue: [self preSelect: initialSelection].
	^ super startUpWithCaption: caption! !

!CustomMenu methodsFor: 'compatibility' stamp: 'sw 2/16/2002 00:57'!
targets
	"Answer my targets, initializing them to an empty collection if found to be nil"

	^ targets ifNil: [targets := OrderedCollection new]! !

!CustomMenu methodsFor: 'initialize-release' stamp: 'sw 8/18/1998 12:01'!
title: aTitle
	title := aTitle! !

SelectionMenu subclass: #EmphasizedMenu
	instanceVariableNames: 'emphases'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Menus'!

!EmphasizedMenu commentStamp: '<historical>' prior: 0!
A selection menu in which individual selections are allowed to have different emphases.  Emphases allowed are: bold, italic, struckThrough, and plain.  Provide an emphasis array, with one element per selection, to use.  Refer to the class method #example.!

!EmphasizedMenu class methodsFor: 'examples' stamp: 'fc 2/19/2004 22:06'!
example1
	"EmphasizedMenu example1"

	^ (self
		selections: #('how' 'well' 'does' 'this' 'work?' ) 
		emphases: #(#bold #normal #italic #struckOut #normal ))
			startUpWithCaption: 'A Menu with Emphases'! !

!EmphasizedMenu class methodsFor: 'examples' stamp: 'sw 9/11/97 16:14'!
example2
	"EmphasizedMenu example2"

	| aMenu |
	aMenu := EmphasizedMenu selections: #('One' 'Two' 'Three' 'Four').
	aMenu onlyBoldItem: 3.
	^ aMenu startUpWithCaption: 'Only the Bold'! !

!EmphasizedMenu class methodsFor: 'examples' stamp: 'fc 2/19/2004 22:08'!
example3
	"EmphasizedMenu example3"

	^ (self
		selectionAndEmphasisPairs: #('how' #bold 'well' #normal 'does' #italic 'this' #struckOut 'work' #normal))
		startUpWithCaption: 'A Menu with Emphases'! !

!EmphasizedMenu class methodsFor: 'instance creation' stamp: 'sw 12/23/96'!
selectionAndEmphasisPairs: interleavedList
	"An alternative form of call.  "
	| selList  emphList |
	selList := OrderedCollection new.
	emphList := OrderedCollection new.
	interleavedList pairsDo:
		[:aSel :anEmph |
			selList add: aSel.
			emphList add: anEmph].
	^ self selections:selList emphases: emphList! !

!EmphasizedMenu class methodsFor: 'instance creation' stamp: 'sma 5/28/2000 16:14'!
selections: selList emphases: emphList
	"Answer an instance of the receiver with the given selections and 
	emphases."

	^ (self selections: selList) emphases: emphList

"Example:
	(EmphasizedMenu
		selections: #('how' 'well' 'does' 'this' 'work?') 
		emphases: #(bold plain italic struckOut plain)) startUp"! !

!EmphasizedMenu methodsFor: 'emphasis' stamp: ''!
emphases: emphasisArray
	emphases := emphasisArray! !

!EmphasizedMenu methodsFor: 'emphasis' stamp: 'fc 2/19/2004 22:07'!
onlyBoldItem: itemNumber
	"Set up emphasis such that all items are plain except for the given item number.  "

	emphases := (Array new: selections size) atAllPut: #normal.
	emphases at: itemNumber put: #bold! !

!EmphasizedMenu methodsFor: 'private' stamp: 'fc 2/20/2004 11:01'!
setEmphasis
	"Set up the receiver to reflect the emphases in the emphases array.  "

	| selStart selEnd currEmphasis |
	
	labelString := labelString asText.
	emphases isEmptyOrNil ifTrue: [^ self].
	selStart := 1.
	1 to: selections size do:
		[:line |
			selEnd := selStart + (selections at: line) size - 1.
			((currEmphasis := emphases at: line) size > 0 and: [currEmphasis ~~ #normal]) ifTrue:
				[labelString addAttribute: (TextEmphasis perform: currEmphasis)
					from: selStart to: selEnd].
			selStart := selEnd + 2]! !

!EmphasizedMenu methodsFor: 'display' stamp: ''!
startUpWithCaption: captionOrNil
	self setEmphasis.
	^ super startUpWithCaption: captionOrNil! !

!SelectionMenu class methodsFor: 'instance creation' stamp: 'ul 11/15/2010 12:03'!
fromArray: anArray
	"Construct a menu from anArray.  The elements of anArray must be either:
	*  A pair of the form: <label> <selector>
or	*  The 'dash' (or 'minus sign') symbol

	Refer to the example at the bottom of the method"

	| labelList lines selections anIndex |
	labelList := OrderedCollection new.
	lines := OrderedCollection new.
	selections := OrderedCollection new.
	anIndex := 0.
	anArray do:
		[:anElement |
			anElement size = 1
				ifTrue:
					[(anElement == #-) ifFalse: [self error: 'badly-formed menu constructor'].
					lines add: anIndex]
				ifFalse:
					[anElement size = 2 ifFalse: [self error: 'badly-formed menu constructor'].
					anIndex := anIndex + 1.
					labelList add: anElement first.
					selections add: anElement second]].
	^ self labelList: labelList lines: lines selections: selections

"(SelectionMenu fromArray:
	#(	('first label'		moja)
		('second label'	mbili)
		-
		('third label' 	tatu)
		-
		('fourth label'	nne)
		('fifth label'	tano))) startUp"! !

!SelectionMenu class methodsFor: 'instance creation' stamp: 'sma 5/28/2000 16:04'!
labelList: labelList
	^ self labelArray: labelList! !

!SelectionMenu class methodsFor: 'instance creation' stamp: 'mha 1/18/2010 10:13'!
labelList: labelList lines: lines
	^ self labelArray: labelList lines: lines! !

!SelectionMenu class methodsFor: 'instance creation' stamp: 'sma 5/28/2000 16:04'!
labelList: labelList lines: lines selections: selections
	^ (self labelArray: labelList lines: lines) selections: selections! !

!SelectionMenu class methodsFor: 'instance creation' stamp: 'sma 5/28/2000 16:04'!
labelList: labelList selections: selections
	^ self
		labelList: labelList
		lines: #()
		selections: selections! !

!SelectionMenu class methodsFor: 'instance creation' stamp: 'yo 8/30/2002 17:00'!
labels: labels lines: linesArray
	"Answer an instance of me whose items are in labels, with lines drawn  
	after each item indexed by linesArray. Labels can be either a string 
	with embedded CRs, or a collection of strings."

	(labels isString)
		ifTrue: [^ super labels: labels lines: linesArray]
		ifFalse: [^ super labelArray: labels lines: linesArray]! !

!SelectionMenu class methodsFor: 'instance creation' stamp: 'yo 8/30/2002 17:02'!
labels: labels lines: linesArray selections: selectionsArray
	"Answer an instance of me whose items are in labels, with lines drawn  
	after each item indexed by linesArray. Labels can be either a string  
	with embedded CRs, or a collection of strings. Record the given array of 
	selections corresponding to the items in labels."

	| labelString |
	(labels isString)
		ifTrue: [labelString := labels]
		ifFalse: [labelString := String streamContents:
					[:s |
					labels do: [:l | s nextPutAll: l; cr].
					s skip: -1]].
	^ (self labels: labelString lines: linesArray) selections: selectionsArray
! !

!SelectionMenu class methodsFor: 'instance creation' stamp: 'sma 5/28/2000 16:10'!
labels: labels selections: selectionsArray
	"Answer an instance of me whose items are in labels, recording 
	the given array of selections corresponding to the items in labels."

	^ self
		labels: labels
		lines: #()
		selections: selectionsArray! !

!SelectionMenu class methodsFor: 'instance creation' stamp: 'sma 5/28/2000 16:10'!
selections: selectionsArray
	"Answer an instance of me whose labels and selections are identical."

	^ self selections: selectionsArray lines: nil! !

!SelectionMenu class methodsFor: 'instance creation' stamp: 'sma 5/28/2000 16:10'!
selections: selectionsArray lines: linesArray
	"Answer an instance of me whose labels and selections are identical."

	^ self
		labelList: (selectionsArray collect: [:each | each asString])
		lines: linesArray
		selections: selectionsArray! !

!SelectionMenu methodsFor: 'basic control sequence' stamp: 'sma 5/28/2000 15:28'!
invokeOn: targetObject
	"Pop up this menu and return the result of sending to the target object 
	the selector corresponding to the menu item selected by the user. Return 
	nil if no item is selected."

	| sel |
	sel := self startUp.
	sel = nil ifFalse: [^ targetObject perform: sel].
	^ nil

"Example:
	(SelectionMenu labels: 'sin
cos
neg' lines: #() selections: #(sin cos negated)) invokeOn: 0.7"! !

!SelectionMenu methodsFor: 'invocation' stamp: 'sw 11/18/2002 16:24'!
invokeOn: targetObject orSendTo: anObject
	"Pop up the receiver, obtaining a selector; return the result of having the target object perform the selector.  If it dos not understand the selector, give the alternate object a chance"

	| aSelector |
	^ (aSelector := self startUp) ifNotNil:
		[(targetObject respondsTo: aSelector)
			ifTrue:
				[targetObject perform: aSelector]
			ifFalse:
				[anObject perform: aSelector]]! !

!SelectionMenu methodsFor: 'accessing' stamp: 'sma 5/28/2000 11:38'!
selections
	^ selections! !

!SelectionMenu methodsFor: 'accessing' stamp: 'sma 5/28/2000 11:38'!
selections: selectionArray
	selections := selectionArray! !

!SelectionMenu methodsFor: 'basic control sequence' stamp: 'jmv 3/2/2010 11:23'!
startUpWithCaption: captionOrNil at: location allowKeyboard: aBoolean
	"Overridden to return value returned by manageMarker.  The boolean parameter indicates whether the menu should be given keyboard focus (if in morphic)"

	| index |
	index := super startUpWithCaption: captionOrNil at: location allowKeyboard: aBoolean.
	(selections isNil or: [(index between: 1 and: selections size) not])
		ifTrue: [^ nil].
	^ selections at: index! !

SelectionMenu subclass: #StandardFileMenu
	instanceVariableNames: 'canTypeFileName pattern'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Menus'!

!StandardFileMenu commentStamp: 'mp 8/15/2005 18:44' prior: 0!
I represent a SelectionMenu which operates like a modal dialog for selecting files, somewhat similar to the StandardFile dialogs in MacOS and Java Swing.

Try for example, the following:

	StandardFileMenu oldFile inspect

	StandardFileMenu oldFileStream inspect

	StandardFileMenu newFile inspect

	StandardFileMenu newFileStream inspect
	
	(StandardFileMenu oldFileMenu: FileDirectory default withPattern: '*') startUpWithCaption: 'Select a file:'
	
	(StandardFileMenu oldFileMenu: (FileDirectory default) withPatternList: {'*.txt'. '*.changes'}) startUpWithCaption: 'Select a file:'
!

!StandardFileMenu class methodsFor: 'standard file operations' stamp: 'tk 2/14/2000 14:28'!
newFile

	^self newFileFrom: (FileDirectory default)! !

!StandardFileMenu class methodsFor: 'standard file operations' stamp: 'dgd 9/21/2003 13:17'!
newFileFrom: aDirectory

	^(self newFileMenu: aDirectory)
		startUpWithCaption: 'Select a File:' translated! !

!StandardFileMenu class methodsFor: 'instance creation' stamp: 'sma 4/30/2000 10:14'!
newFileMenu: aDirectory
	Smalltalk isMorphic ifFalse: [^ PluggableFileList newFileMenu: aDirectory].
	^ super new newFileFrom: aDirectory! !

!StandardFileMenu class methodsFor: 'instance creation' stamp: 'zz 8/15/2005 18:21'!
newFileMenu: aDirectory withPatternList: aPatternList
	Smalltalk isMorphic ifFalse: [^ PluggableFileList newFileMenu: aDirectory].
	^ super new newFileFrom: aDirectory withPatternList: aPatternList! !

!StandardFileMenu class methodsFor: 'instance creation' stamp: 'rww 9/23/2001 09:56'!
newFileMenu: aDirectory withPattern: aPattern
	Smalltalk isMorphic ifFalse: [^ PluggableFileList newFileMenu: aDirectory].
	^ super new newFileFrom: aDirectory withPattern: aPattern! !

!StandardFileMenu class methodsFor: 'standard file operations' stamp: 'acg 4/15/1999 22:18'!
newFileStream

	^self newFileStreamFrom: (FileDirectory default)! !

!StandardFileMenu class methodsFor: 'standard file operations' stamp: 'tk 2/14/2000 14:28'!
newFileStreamFrom: aDirectory

	| sfmResult fileStream |
	sfmResult := self newFileFrom: aDirectory.
	sfmResult ifNil: [^nil].
	fileStream := sfmResult directory newFileNamed: sfmResult name.
	[fileStream isNil] whileTrue:
		[sfmResult := self newFileFrom: aDirectory.
		sfmResult ifNil: [^nil].
		fileStream := sfmResult directory newFileNamed: sfmResult name].
	^fileStream
! !

!StandardFileMenu class methodsFor: 'standard file operations' stamp: 'tk 2/14/2000 14:28'!
oldFile

	^self oldFileFrom: (FileDirectory default)! !

!StandardFileMenu class methodsFor: 'standard file operations' stamp: 'dgd 9/21/2003 13:17'!
oldFileFrom: aDirectory

	^(self oldFileMenu: aDirectory)
		startUpWithCaption: 'Select a File:' translated! !

!StandardFileMenu class methodsFor: 'standard file operations' stamp: 'MM 4/6/2004 22:56'!
oldFileFrom: aDirectory withPattern: aPattern
"
Select an existing file from a selection conforming to aPattern.
"
	^(self oldFileMenu: aDirectory withPattern: aPattern)
		startUpWithCaption: 'Select a File:' translated! !

!StandardFileMenu class methodsFor: 'instance creation' stamp: 'sma 4/30/2000 10:15'!
oldFileMenu: aDirectory
	Smalltalk isMorphic ifFalse: [^ PluggableFileList oldFileMenu: aDirectory].
	^ super new oldFileFrom: aDirectory! !

!StandardFileMenu class methodsFor: 'instance creation' stamp: 'zz 8/15/2005 17:41'!
oldFileMenu: aDirectory withPatternList: aPatternList

	Smalltalk isMorphic ifFalse: [^PluggableFileList oldFileMenu: aDirectory].
	^super new oldFileFrom: aDirectory withPatternList: aPatternList! !

!StandardFileMenu class methodsFor: 'instance creation' stamp: 'RAA 5/25/2000 09:30'!
oldFileMenu: aDirectory withPattern: aPattern

	Smalltalk isMorphic ifFalse: [^PluggableFileList oldFileMenu: aDirectory].
	^super new oldFileFrom: aDirectory withPattern: aPattern! !

!StandardFileMenu class methodsFor: 'standard file operations' stamp: 'acg 4/15/1999 22:17'!
oldFileStream

	^self oldFileStreamFrom: (FileDirectory default)
! !

!StandardFileMenu class methodsFor: 'standard file operations' stamp: 'tk 2/14/2000 14:27'!
oldFileStreamFrom: aDirectory

	| sfmResult fileStream |
	sfmResult := self oldFileFrom: aDirectory.
	sfmResult ifNil: [^nil].
	fileStream := sfmResult directory oldFileNamed: sfmResult name.
	[fileStream isNil] whileTrue:
		[sfmResult := self oldFileFrom: aDirectory.
		sfmResult ifNil: [^nil].
		fileStream := sfmResult directory oldFileNamed: sfmResult name].
	^fileStream
! !

!StandardFileMenu methodsFor: 'private' stamp: 'acg 4/15/1999 00:32'!
advance: anInteger containingDirectoriesFrom: aDirectory

	| theDirectory |
	theDirectory := aDirectory.
	1 to: anInteger do: [:i | theDirectory := theDirectory containingDirectory].
	^theDirectory! !

!StandardFileMenu methodsFor: 'basic control sequences' stamp: 'rbb 2/16/2005 16:59'!
confirmExistingFiles: aResult

	|choice|
	(aResult directory fileExists: aResult name) ifFalse: [^aResult].
	
	choice := (UIManager default chooseFrom: #('overwrite that file' 'choose another name'
 'cancel')
		title: aResult name, '
already exists.').

	choice = 1 ifTrue: [
		aResult directory 
			deleteFileNamed: aResult name
			ifAbsent: 
				[^self startUpWithCaption: 
'Can''t delete ', aResult name, '
Select another file'].
		^aResult].
	choice = 2 ifTrue: [^self startUpWithCaption: 'Select Another File'].
	^nil
 ! !

!StandardFileMenu methodsFor: 'menu building' stamp: 'di 5/12/2000 10:31'!
directoryNamesString: aDirectory
"Answer a string concatenating the directory name strings in aDirectory, each string followed by a '[...]' indicator, and followed by a cr."

	^ String streamContents:
		[:s | aDirectory directoryNames do: 
				[:dn | s nextPutAll: dn withBlanksTrimmed , ' [...]'; cr]]

! !

!StandardFileMenu methodsFor: 'menu building' stamp: 'zz 8/15/2005 17:33'!
fileNamesString: aDirectory
"Answer a string concatenating the file name strings in aDirectory, each string followed by a cr."

	^String streamContents:
		[:s | 
			aDirectory fileNames do: 
				[:fn |
					pattern do:[:each | (each match: fn) ifTrue: [
						s nextPutAll: fn withBlanksTrimmed; cr]]]]
		! !

!StandardFileMenu methodsFor: 'basic control sequences' stamp: 'rbb 3/1/2005 11:14'!
getTypedFileName: aResult

	| name |
	name := UIManager default 
		request: 'Enter a new file name' 
		initialAnswer: ''.
	name = '' ifTrue: [^self startUpWithCaption: 'Select a File:' translated].
	name := aResult directory fullNameFor: name.
	^ StandardFileMenuResult
			directory: (FileDirectory forFileName: name)
			name: (FileDirectory localNameFor: name)
! !

!StandardFileMenu methodsFor: 'menu building' stamp: 'zz 8/15/2005 16:28'!
makeFileMenuFor: aDirectory
"Initialize an instance of me to operate on aDirectory"

	| theMenu |
	pattern ifNil: [pattern := {'*'}].
	Cursor wait showWhile: 
		[self 
			labels: 	(self menuLabelsString: aDirectory)
			font: 	(MenuStyle fontAt: 1) 
			lines: 	(self menuLinesArray: aDirectory).
		theMenu := self selections: (self menuSelectionsArray: aDirectory)].
	^theMenu! !

!StandardFileMenu methodsFor: 'menu building' stamp: 'acg 4/15/1999 21:57'!
menuLabelsString: aDirectory
"Answer a menu labels object corresponding to aDirectory"

	^ String streamContents: 
		[:s | 
			canTypeFileName ifTrue: 
				[s nextPutAll: 'Enter File Name...'; cr].
			s nextPutAll: (self pathPartsString: aDirectory).
			s nextPutAll: (self directoryNamesString: aDirectory).
			s nextPutAll: (self fileNamesString: aDirectory).
			s skip: -1]! !

!StandardFileMenu methodsFor: 'menu building' stamp: 'tpr 11/28/2003 15:12'!
menuLinesArray: aDirectory
"Answer a menu lines object corresponding to aDirectory"

	| typeCount nameCnt dirDepth|
	typeCount := canTypeFileName 
		ifTrue: [1] 
		ifFalse: [0].
	nameCnt := aDirectory directoryNames size.
	dirDepth := aDirectory pathParts size.
	^Array streamContents: [:s |
		canTypeFileName ifTrue: [s nextPut: 1].
		s nextPut: dirDepth + typeCount + 1.
		s nextPut: dirDepth + nameCnt + typeCount + 1]! !

!StandardFileMenu methodsFor: 'menu building' stamp: 'zz 8/15/2005 18:18'!
menuSelectionsArray: aDirectory
"Answer a menu selections object corresponding to aDirectory.  The object is an array corresponding to each item, each element itself constituting a two-element array, the first element of which contains a selector to operate on and the second element of which contains the parameters for that selector."

	|dirSize|
	dirSize := aDirectory pathParts size.
	^Array streamContents: [:s |
		canTypeFileName ifTrue:
			[s nextPut: (StandardFileMenuResult
				directory: aDirectory
				name: nil)].
		s nextPut: (StandardFileMenuResult
			directory: (FileDirectory root)
			name: '').
		aDirectory pathParts doWithIndex: 
			[:d :i | s nextPut: (StandardFileMenuResult
					directory: (self 
						advance: dirSize - i
						containingDirectoriesFrom: aDirectory)
					name: '')].
		aDirectory directoryNames do: 
			[:dn |  s nextPut: (StandardFileMenuResult
						directory: (FileDirectory on: (aDirectory fullNameFor: dn))
						name: '')].
		aDirectory fileNames do: 
			[:fn | pattern do: [:pat | (pat match: fn) ifTrue: [
					s nextPut: (StandardFileMenuResult
						directory: aDirectory
						name: fn)]]]]! !

!StandardFileMenu methodsFor: 'private' stamp: 'acg 4/15/1999 22:03'!
newFileFrom: aDirectory

	canTypeFileName := true.
	^self makeFileMenuFor: aDirectory! !

!StandardFileMenu methodsFor: 'private' stamp: 'zz 8/15/2005 18:25'!
newFileFrom: aDirectory withPatternList: aPatternList

	canTypeFileName := true.
	pattern := aPatternList.
	^self makeFileMenuFor: aDirectory! !

!StandardFileMenu methodsFor: 'private' stamp: 'zz 8/15/2005 16:29'!
newFileFrom: aDirectory withPattern: aPattern

	canTypeFileName := true.
	pattern := {aPattern}.
	^self makeFileMenuFor: aDirectory! !

!StandardFileMenu methodsFor: 'private' stamp: 'acg 4/15/1999 22:03'!
oldFileFrom: aDirectory

	canTypeFileName := false.
	^self makeFileMenuFor: aDirectory! !

!StandardFileMenu methodsFor: 'private' stamp: 'zz 8/15/2005 17:28'!
oldFileFrom: aDirectory withPatternList: aPatternList

	canTypeFileName := false.
	pattern := aPatternList.
	^self makeFileMenuFor: aDirectory! !

!StandardFileMenu methodsFor: 'private' stamp: 'zz 8/15/2005 16:28'!
oldFileFrom: aDirectory withPattern: aPattern

	canTypeFileName := false.
	pattern := {aPattern}.
	^self makeFileMenuFor: aDirectory! !

!StandardFileMenu methodsFor: 'menu building' stamp: 'acg 4/15/1999 21:03'!
pathPartsString: aDirectory
"Answer a string concatenating the path parts strings in aDirectory, each string followed by a cr."

	^String streamContents:
		[:s | 
			s nextPutAll: '[]'; cr.
			aDirectory pathParts asArray doWithIndex: 
				[:part :i |
					s next: i put: $ .
					s nextPutAll: part withBlanksTrimmed; cr]]! !

!StandardFileMenu methodsFor: 'private' stamp: 'zz 8/15/2005 17:29'!
patternList: aPatternList

	pattern := aPatternList! !

!StandardFileMenu methodsFor: 'private' stamp: 'zz 8/15/2005 16:31'!
pattern: aPattern
	" * for all files, or '*.cs' for changeSets, etc.  Just like fileLists"

	pattern := {aPattern}! !

!StandardFileMenu methodsFor: 'basic control sequences' stamp: 'acg 9/28/1999 23:34'!
startUpWithCaption: aString at: location

	|result|
	result := super startUpWithCaption: aString at: location.
	result ifNil: [^nil].
	result isDirectory ifTrue:
		[self makeFileMenuFor: result directory.
		 self computeForm.
		 ^self startUpWithCaption: aString at: location].
	result isCommand ifTrue: 
		[result := self getTypedFileName: result.
		result ifNil: [^nil]].
	canTypeFileName ifTrue: [^self confirmExistingFiles: result].
	^result
	! !

Object subclass: #SavedMultiWindowState
	instanceVariableNames: 'models modelStates modelDependents currentModelIndex prototype'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Base'!

!SavedMultiWindowState commentStamp: 'eem 6/30/2010 15:45' prior: 0!
A SavedMultiWindowState is a collection of states of inactive windows for a multi-window tool ui where one can select between multiple copies of a tool sharing a single system window, each tol having its own state.  It manages switching between windows, swapping unaccepted edit state (e.g. in text views) as it does so.

Instance Variables
	currentModelIndex:		<Integer>
	modelDependents:		<IdentitySet>
	modelStates:			<OrderedCollection of: (Array
													with: modelState <Object>
													with: unacceptedEditStates <WeakIdentityKeyDictionary>) >
	models:					<OrderedCollection of: Model>

currentModelIndex
	- index into models and modelStates of current window selection

modelDependents
	- set of all dependents

modelStates
	- the pairs of model-specific selection state and widget-specfic unaccepted edit state for each unselected/inactive window

models
	- the models for each window
!

!SavedMultiWindowState class methodsFor: 'instance creation' stamp: 'eem 6/30/2010 13:46'!
on: aModel
	^self new on: aModel! !

!SavedMultiWindowState methodsFor: 'operations' stamp: 'eem 6/30/2010 17:14'!
addNewWindow
	self addWindow: prototype clone! !

!SavedMultiWindowState methodsFor: 'operations' stamp: 'eem 6/30/2010 19:01'!
addWindow: aModel
	aModel multiWindowState: self.
	self assert: models size = modelStates size.
	models := models
					copyReplaceFrom: currentModelIndex + 1
					to: currentModelIndex
					with: (Array with: aModel).
	modelStates := modelStates
					copyReplaceFrom: currentModelIndex + 1
					to: currentModelIndex
					with: (Array with: self emptyEditState).
	self assert: models size = modelStates size.
	self selectWindowIndex: (currentModelIndex = 0 ifTrue: [currentModelIndex := 1] ifFalse: [currentModelIndex + 1]).
	self assert: models size = modelStates size.
	1 to: models size do:
		[:idx|
		self assert: (idx = currentModelIndex
					or: [(modelStates at: idx) isArray and: [(modelStates at: idx) size = 2]])]! !

!SavedMultiWindowState methodsFor: 'operations' stamp: 'eem 6/30/2010 17:10'!
copyWindow
	self addWindow: (models at: currentModelIndex) copy breakDependents! !

!SavedMultiWindowState methodsFor: 'accessing' stamp: 'eem 6/30/2010 15:01'!
currentModelIndex
	^currentModelIndex! !

!SavedMultiWindowState methodsFor: 'selecting' stamp: 'eem 6/30/2010 14:57'!
editState
	| textState |
	textState := WeakIdentityKeyDictionary new.
	modelDependents do:
		[:dep|
		dep canHaveUnacceptedEdits ifTrue:
			[textState at: dep put: dep unacceptedEditState]].
	^textState! !

!SavedMultiWindowState methodsFor: 'accessing' stamp: 'eem 6/30/2010 21:07'!
emptyEditState
	^#(nil ())! !

!SavedMultiWindowState methodsFor: 'initialize-release' stamp: 'eem 6/30/2010 15:12'!
initialize
	models := OrderedCollection new.
	modelStates := OrderedCollection new.
	modelDependents := IdentitySet new.
	currentModelIndex := 0! !

!SavedMultiWindowState methodsFor: 'accessing' stamp: 'eem 6/30/2010 19:03'!
models
	^models! !

!SavedMultiWindowState methodsFor: 'testing' stamp: 'eem 6/30/2010 22:11'!
okToClose
	"Answer if the unaccepted edit states are void.  If they are not then
	 offer to select a window with non-empty unaccepted edit state."

	modelStates withIndexDo:
		[:savedState :index|
		savedState ifNotNil:
			[(savedState last anySatisfy: [:editState| editState notNil]) ifTrue:
				[(PopUpMenu
					confirm: 'One or more windows have unsaved changes.'
					trueChoice: 'Close Anyway'
					falseChoice: 'Select Unsaved') ifTrue: [^true].
				 self selectWindowIndex: index.
				 ^false]]].
	^true! !

!SavedMultiWindowState methodsFor: 'initialize-release' stamp: 'eem 6/30/2010 18:42'!
on: aModel
	prototype := aModel clone.
	self addWindow: aModel! !

!SavedMultiWindowState methodsFor: 'accessing' stamp: 'eem 7/1/2010 10:40'!
prototype
	^prototype! !

!SavedMultiWindowState methodsFor: 'operations' stamp: 'eem 6/30/2010 21:15'!
removeCurrentWindow
	"Remove the current pane."
	| deletedIndex |
	self assert: models size > 1.
	self assert: models size = modelStates size.
	(models at: currentModelIndex) okToChange ifFalse:
		[^self].
	deletedIndex := currentModelIndex.
	self selectWindowIndex: (currentModelIndex = 1
								ifTrue: [currentModelIndex + 1]
								ifFalse: [currentModelIndex - 1]).
	models := models copyWithoutIndex: deletedIndex.
	modelStates :=  modelStates copyWithoutIndex: deletedIndex.
	currentModelIndex > deletedIndex ifTrue:
		[currentModelIndex := currentModelIndex - 1].
	(models at: currentModelIndex) changed: #relabel! !

!SavedMultiWindowState methodsFor: 'selecting' stamp: 'eem 6/30/2010 14:57'!
restoreEditState: aWeakDictionary
	aWeakDictionary keysAndValuesDo:
		[:depOrNil :unacceptedEditState|
		depOrNil ifNotNil:
			[depOrNil unacceptedEditState: unacceptedEditState]]! !

!SavedMultiWindowState methodsFor: 'selecting' stamp: 'eem 6/30/2010 21:09'!
selectWindowIndex: index
	| currentModel newModel newModelState |
	index = currentModelIndex ifTrue: [^self].

	currentModel := models at: currentModelIndex.
	newModel := models at: index.
	modelDependents addAll: currentModel dependents.
	modelStates at: currentModelIndex put: { currentModel saveMultiWindowState. self editState }.
	"the become makes things confusing.  currentModel will end up being newModel and vice verce"
	models swap: currentModelIndex with: index.
	currentModel become: newModel.
	currentModel := models at: currentModelIndex.
	newModel := models at: index.
	newModel myDependents: (DependentsArray withAll: modelDependents asArray).
	currentModel myDependents: nil.
	"Change currentModelIndex before restoreMultiWIndowState: so label index is shown correctly"
	currentModelIndex := index.
	newModelState := modelStates at: index.
	newModelState first ifNotNil: "emptyState has nil for savedModelState.  Don;t restore state for a new window"
		[:savedModelState| newModel restoreMultiWindowState: savedModelState].
	self restoreEditState: newModelState last.
	modelStates at: index put: nil.
	newModel changed: #relabel! !

!SavedMultiWindowState methodsFor: 'operations' stamp: 'eem 8/11/2010 13:27'!
selectWindowsMenu
	| menu currentItem |
	menu := MenuMorph new defaultTarget: self.
	models withIndexDo:
		[:model :index| | modelLabel |
		modelLabel := index = currentModelIndex
							ifTrue: [model multiWindowName]
							ifFalse: [model multiWindowNameForState: (modelStates at: index) first].
		models size > 1 ifTrue:
			[modelLabel := index printString, '. ', modelLabel].
		menu
			add: (index = currentModelIndex
					ifTrue: [Text string: modelLabel attribute: TextEmphasis bold]
					ifFalse:
						[((modelStates at: index) last anySatisfy: [:editState| editState notNil])
							ifTrue: [Text string: modelLabel attribute: TextColor red]
							ifFalse: [modelLabel]])
			target: self
			selector: #selectWindowIndex:
			argument: index.
		index = currentModelIndex ifTrue:
			[currentItem := menu lastItem]].
	menu
		addLine;
		add: 'new window' target: self selector: #addNewWindow;
		add: 'copy this window' target: self selector: #copyWindow;
		add: 'remove this window' target: self selector: #removeCurrentWindow.
	models size <= 1 ifTrue:
		[menu lastItem isEnabled: false].
	menu selectItem: currentItem event: nil.
	^menu! !

Object subclass: #StandardFileMenuResult
	instanceVariableNames: 'directory name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Menus'!

!StandardFileMenuResult class methodsFor: 'instance creation' stamp: 'acg 4/15/1999 08:42'!
directory: aDirectory name: aString

	^super new directory: aDirectory name: aString! !

!StandardFileMenuResult methodsFor: 'accessing' stamp: 'acg 4/15/1999 08:43'!
directory

	^directory! !

!StandardFileMenuResult methodsFor: 'accessing' stamp: 'acg 4/15/1999 08:43'!
directory: aDirectory

	^directory := aDirectory! !

!StandardFileMenuResult methodsFor: 'private' stamp: 'acg 4/15/1999 08:42'!
directory: aDirectory name: aString

	directory := aDirectory.
	name := aString.
	^self! !

!StandardFileMenuResult methodsFor: 'testing' stamp: 'acg 4/15/1999 09:05'!
isCommand

	^name isNil! !

!StandardFileMenuResult methodsFor: 'testing' stamp: 'acg 4/15/1999 20:57'!
isDirectory

	^name = ''! !

!StandardFileMenuResult methodsFor: 'accessing' stamp: 'acg 4/15/1999 08:43'!
name

	^name! !

!StandardFileMenuResult methodsFor: 'accessing' stamp: 'acg 4/15/1999 08:43'!
name: aString

	^name := aString! !

!StandardFileMenuResult methodsFor: 'accessing' stamp: 'sw 6/9/1999 11:50'!
printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: ' with directory: '.
	directory printOn: aStream.
	aStream nextPutAll: ' name: '.
	name printOn: aStream

"StandardFileMenu oldFile"! !

Object subclass: #StandardToolSet
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Base'!

!StandardToolSet commentStamp: '<historical>' prior: 0!
Main comment stating the purpose of this class and relevant relationship to other classes.

Possible useful expressions for doIt or printIt.

Structure:
 instVar1		type -- comment about the purpose of instVar1
 instVar2		type -- comment about the purpose of instVar2

Any further useful comments about the general approach of this implementation.!

!StandardToolSet class methodsFor: 'inspecting' stamp: 'ar 7/15/2005 18:57'!
basicInspect: anObject
	"Open an inspector on the given object. The tool set must know which inspector type to use for which object - the object cannot possibly know what kind of inspectors the toolset provides."
	^BasicInspector openOn: anObject! !

!StandardToolSet class methodsFor: 'browsing' stamp: 'ar 7/16/2005 15:20'!
browseChangeSetsWithClass: aClass selector: aSelector
	"Browse all the change sets with the given class/selector"
	^ChangeSorter browseChangeSetsWithClass: aClass selector: aSelector! !

!StandardToolSet class methodsFor: 'browsing' stamp: 'fbs 9/13/2013 17:28'!
browseClass: aClass
	"Open a browser"
	^SystemBrowser default fullOnClass: aClass.! !

!StandardToolSet class methodsFor: 'browsing' stamp: 'mt 1/22/2016 13:14'!
browseClass: aClass category: aCategory
	"Open a browser for a class selecting a given message category."
	
	^ SystemBrowser default
		fullOnClass: aClass
		category: aCategory! !

!StandardToolSet class methodsFor: 'browsing' stamp: 'mt 1/22/2016 14:14'!
browseHierarchy: aClass selector: aSelector
	"Open a browser"
	
	^ HierarchyBrowser fullOnClass: aClass selector: aSelector.! !

!StandardToolSet class methodsFor: 'browsing' stamp: 'mt 1/22/2016 13:30'!
browseMessageCategory: aCategory inClass: aClass
	"Open a message category browser."
	
	^ SystemBrowser default
		newOnMessageCategory: aCategory inClass: aClass.! !

!StandardToolSet class methodsFor: 'browsing' stamp: 'ar 7/17/2005 11:12'!
browseMessageNames: aString
	^(MessageNames methodBrowserSearchingFor: aString) openInWorld! !

!StandardToolSet class methodsFor: 'browsing' stamp: 'cmm 3/2/2010 17:23'!
browseMessageSet: messageList name: title autoSelect: autoSelectString
	"Open a message set or message-trace browser, depending on the #traceMessages setting."
	^ Preferences traceMessages
		ifTrue:
			[ MessageTrace
				openMessageList: messageList 
				name: title 
				autoSelect: autoSelectString ]
		ifFalse:
			[ MessageSet
				openMessageList: messageList 
				name: title 
				autoSelect: autoSelectString ]! !

!StandardToolSet class methodsFor: 'browsing' stamp: 'eem 1/7/2016 11:28'!
browseVersionsOf: class selector: selector
	"Open and answer a browser on versions of the method.
	 If the method doesn't exist try and find version from the current change set.
	 If not found there, answer nil."
	| methodOrNil methodCategory |
	methodOrNil := class compiledMethodAt: selector ifAbsent: [].
	methodOrNil ifNil:
		[(ChangeSet current methodInfoFromRemoval: {class name. selector})
			ifNil: [^nil]
			ifNotNil:
				[:pair|
				methodOrNil := CompiledMethod toReturnSelfTrailerBytes:
									(CompiledMethodTrailer new sourcePointer: pair first).
				methodCategory := pair last]].
	^VersionsBrowser
		browseVersionsOf: methodOrNil
		class: class theNonMetaClass
		meta: class isMeta
		category: (methodCategory ifNil: [class organization categoryOfElement: selector])
		selector: selector! !

!StandardToolSet class methodsFor: 'browsing' stamp: 'md 3/10/2006 21:43'!
browse: aClass selector: aSelector
	"Open a browser"
	^SystemBrowser default fullOnClass: aClass selector: aSelector! !

!StandardToolSet class methodsFor: 'completion' stamp: 'damiencassou 7/29/2009 16:12'!
codeCompletionAround: aBlock textMorph: aTextMorph keyStroke: evt
	^ aBlock value! !

!StandardToolSet class methodsFor: 'debugging' stamp: 'ar 7/15/2005 19:15'!
debugContext: aContext label: aString contents: contents
	"Open a debugger on the given process and context."
	^Debugger openContext: aContext label: aString contents: contents! !

!StandardToolSet class methodsFor: 'debugging' stamp: 'ar 7/17/2005 11:16'!
debugError: anError
	"Handle an otherwise unhandled error"
	^Processor activeProcess
		debug: anError signalerContext
		title: anError description! !

!StandardToolSet class methodsFor: 'debugging' stamp: 'mt 4/18/2016 16:33'!
debugMethod: aCompiledMethod forReceiver: anObject inContext: aContext

	^ Project current debugMethod: aCompiledMethod forReceiver: anObject inContext: aContext! !

!StandardToolSet class methodsFor: 'debugging' stamp: 'nice 9/18/2013 02:46'!
debugSyntaxError: aSyntaxErrorNotification
	"Handle a syntax error"
	| notifier |
	notifier :=  SyntaxError new setNotification: aSyntaxErrorNotification.
	SyntaxError open: notifier.! !

!StandardToolSet class methodsFor: 'debugging' stamp: 'ar 7/15/2005 18:57'!
debug: aProcess context: aContext label: aString contents: contents fullView: aBool
	"Open a debugger on the given process and context."
	^Debugger openOn: aProcess context: aContext label: aString contents: contents fullView: aBool! !

!StandardToolSet class methodsFor: 'inspecting' stamp: 'mt 4/2/2015 11:04'!
explore: anObject

	^ ObjectExplorer openOn: anObject! !

!StandardToolSet class methodsFor: 'class initialization' stamp: 'mt 7/2/2016 12:41'!
initialize
	ToolSet register: self.! !

!StandardToolSet class methodsFor: 'inspecting' stamp: 'ar 7/15/2005 19:54'!
inspectorClassOf: anObject
	"Answer the inspector class for the given object. The tool set must know which inspector type to use for which object - the object cannot possibly know what kind of inspectors the toolset provides."
	| map |
	map := Dictionary new.
	#(
		(CompiledMethod		CompiledMethodInspector)
		(CompositeEvent		OrderedCollectionInspector)
		(Dictionary			DictionaryInspector)
		(ExternalStructure	ExternalStructureInspector)
		(FloatArray			OrderedCollectionInspector)
		(OrderedCollection	OrderedCollectionInspector)
		(Set					SetInspector)
		(WeakSet			WeakSetInspector)
	) do:[:spec|
		map at: spec first put: spec last.
	].
	anObject class withAllSuperclassesDo:[:cls|
		map at: cls name ifPresent:[:inspectorName| ^Smalltalk classNamed: inspectorName].
	].
	^Inspector! !

!StandardToolSet class methodsFor: 'inspecting' stamp: 'ar 7/15/2005 18:58'!
inspect: anObject
	"Open an inspector on the given object. The tool set must know which inspector type to use for which object - the object cannot possibly know what kind of inspectors the toolset provides."
	^(self inspectorClassOf: anObject) openOn: anObject! !

!StandardToolSet class methodsFor: 'inspecting' stamp: 'ar 7/15/2005 19:57'!
inspect: anObject label: aString
	"Open an inspector on the given object. The tool set must know which inspector type to use for which object - the object cannot possibly know what kind of inspectors the toolset provides."
	^(self inspectorClassOf: anObject) openOn: anObject withEvalPane: true withLabel: aString! !

!StandardToolSet class methodsFor: 'debugging' stamp: 'ar 7/17/2005 11:16'!
interrupt: aProcess label: aString
	"Open a debugger on the given process and context."
	Debugger
		openInterrupt: aString
		onProcess: aProcess! !

!StandardToolSet class methodsFor: 'menu' stamp: 'ar 7/17/2005 13:19'!
menuItems
	"Answer the menu items available for this tool set"
	^#(
		('class browser' 			#openClassBrowser)
		('workspace'				#openWorkspace)
		('file list'					#openFileList)
		('package pane browser' 	#openPackagePaneBrowser)
		('process browser' 			#openProcessBrowser)
		-
		('method finder'				#openSelectorBrowser)
		('message names'			#openMessageNames)
		-
		('simple change sorter'		#openChangeSorter)
		('dual change sorter'		#openDualChangeSorter)
	)
! !

!StandardToolSet class methodsFor: 'browsing' stamp: 'ar 7/15/2005 18:58'!
openChangedMessageSet: aChangeSet
	"Open a ChangedMessageSet for aChangeSet"
	ChangedMessageSet openFor: aChangeSet! !

!StandardToolSet class methodsFor: 'menu' stamp: 'ar 8/7/2009 22:21'!
openChangeSorter
	ToolBuilder open: ChangeSorter new! !

!StandardToolSet class methodsFor: 'menu' stamp: 'md 3/10/2006 21:47'!
openClassBrowser
	SystemBrowser default open! !

!StandardToolSet class methodsFor: 'browsing' stamp: 'ar 7/15/2005 18:58'!
openClassListBrowser: anArray title: aString
	"Open a class list browser"
	^ClassListBrowser new initForClassesNamed: anArray title: aString
! !

!StandardToolSet class methodsFor: 'menu' stamp: 'ar 8/7/2009 22:21'!
openDualChangeSorter
	ToolBuilder open: DualChangeSorter new! !

!StandardToolSet class methodsFor: 'menu' stamp: 'ar 10/4/2009 22:19'!
openFileList
	FileList open! !

!StandardToolSet class methodsFor: 'menu' stamp: 'ar 7/17/2005 12:59'!
openMessageNames
	"Bring a MessageNames tool to the front"
	MessageNames openMessageNames! !

!StandardToolSet class methodsFor: 'menu' stamp: 'ar 7/17/2005 13:04'!
openPackagePaneBrowser
	PackagePaneBrowser openBrowser.! !

!StandardToolSet class methodsFor: 'menu' stamp: 'ar 7/17/2005 12:59'!
openProcessBrowser
	ProcessBrowser open! !

!StandardToolSet class methodsFor: 'menu' stamp: 'ar 8/7/2009 22:21'!
openSelectorBrowser
	ToolBuilder open: SelectorBrowser new! !

!StandardToolSet class methodsFor: 'menu' stamp: 'md 1/18/2006 19:08'!
openWorkspace
	Workspace open! !

!StandardToolSet class methodsFor: 'class initialization' stamp: 'ar 7/16/2005 16:18'!
unload
	ToolSet unregister: self.! !

Object subclass: #ToolIconHelp
	instanceVariableNames: ''
	classVariableNames: 'HelpTexts'
	poolDictionaries: ''
	category: 'Tools-Base'!

!ToolIconHelp class methodsFor: 'as yet unclassified' stamp: 'pre 12/8/2015 19:05'!
abstract

	^ 'This method is abstract.' ! !

!ToolIconHelp class methodsFor: 'as yet unclassified' stamp: 'pre 12/8/2015 19:06'!
arrowDown

	^ 'This method is overriden by another method.'! !

!ToolIconHelp class methodsFor: 'as yet unclassified' stamp: 'pre 12/8/2015 19:05'!
arrowUp

	^ 'This method overrides a super method.'! !

!ToolIconHelp class methodsFor: 'as yet unclassified' stamp: 'pre 12/8/2015 19:05'!
arrowUpAndDown

	^ 'This method overrides and is overridden by other methods.'! !

!ToolIconHelp class methodsFor: 'as yet unclassified' stamp: 'pre 12/8/2015 19:09'!
blank

	^ ''! !

!ToolIconHelp class methodsFor: 'as yet unclassified' stamp: 'pre 12/8/2015 17:48'!
breakpoint 

	^ 'This method contains a breakpoint.'! !

!ToolIconHelp class methodsFor: 'as yet unclassified' stamp: 'pre 12/8/2015 19:04'!
flag

	^ 'This method needs attention of some kind.' ! !

!ToolIconHelp class methodsFor: 'as yet unclassified' stamp: 'pre 12/8/2015 19:08'!
helpTexts

	^ HelpTexts ifNil: [HelpTexts := IdentityDictionary new]! !

!ToolIconHelp class methodsFor: 'as yet unclassified' stamp: 'topa 2/23/2016 18:51'!
iconHelpNamed: aSymbol

	^ (self respondsTo: aSymbol)
		ifTrue: [self helpTexts at: aSymbol ifAbsentPut: [self perform: aSymbol]]
		ifFalse: [String empty]
	! !

!ToolIconHelp class methodsFor: 'as yet unclassified' stamp: 'pre 12/8/2015 19:05'!
no

	^  'This method should not be implemented.' ! !

!ToolIconHelp class methodsFor: 'as yet unclassified' stamp: 'pre 12/8/2015 19:05'!
notOverridden

	^ 'This method is abstract and has not been overridden.' ! !

!ToolIconHelp class methodsFor: 'as yet unclassified' stamp: 'pre 12/8/2015 19:05'!
primitive

	^ 'This method implements a primitive.'! !

Object subclass: #ToolIcons
	instanceVariableNames: ''
	classVariableNames: 'Icons'
	poolDictionaries: ''
	category: 'Tools-Base'!

!ToolIcons class methodsFor: 'private' stamp: 'topa 3/9/2015 00:29'!
abstract

	^  Form
	extent: 12@12
	depth: 32
	fromArray: #( 165 165 165 165 165 16777215 16777215 165 165 165 165 165 16777215 16777215 165 165 16777215 2712832 2712832 16777215 165 165 165 165 165 165 165 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 165 165 165 165 165 4294967295 4284714821 4284714821 4284714821 4284714821 4284714821 4284714821 4294967295 165 165 165 16777215 4294967295 4284714821 4288074840 4288074840 4288074840 4287677545 4284714821 4294967295 16777215 165 165 16777215 4294967295 4284714821 4288074840 4287679828 4287810910 4287677545 4284714821 4294967295 16777215 165 165 16777215 4294967295 4284714821 4288074840 4287614291 4287810910 4286361178 4284714821 4294967295 16777215 165 165 16777215 4294967295 4284714821 4288074840 4287677545 4286361178 4286361178 4284714821 4294967295 16777215 165 165 165 4294967295 4284714821 4284714821 4284714821 4284714821 4284714821 4284714821 4294967295 165 165 165 165 165 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 165 165 165 165 165 165 165 16777215 2712832 2712832 16777215 165 165 165 165 165 165 165 165 165 16777215 16777215 165 165 165 165 165)
	offset: 0@0! !

!ToolIcons class methodsFor: 'private' stamp: 'topa 3/9/2015 00:30'!
arrowDown
	^ Form
	extent: 12@12
	depth: 32
	fromArray: #( 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 0 0 0 0 0 0 4294967295 4284714821 4284714821 4284714821 4284714821 4294967295 0 0 0 0 0 0 4294967295 4284714821 4287614291 4287810910 4284714821 4294967295 0 0 0 0 4294967295 4294967295 4294967295 4284714821 4287679828 4287810910 4284714821 4294967295 4294967295 4294967295 0 0 4294967295 4284714821 4284714821 4284714821 4288074840 4287677545 4284714821 4284714821 4284714821 4294967295 0 0 0 4294967295 4284714821 4287810910 4288272218 4287876447 4287350105 4284714821 4294967295 0 0 0 0 0 4294967295 4284714821 4287810910 4286361178 4284714821 4294967295 0 0 0 0 0 0 0 4294967295 4284714821 4284714821 4294967295 0 0 0 0 0 0 0 0 0 4294967295 4294967295 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0)
	offset: 0@0! !

!ToolIcons class methodsFor: 'private' stamp: 'topa 3/9/2015 00:31'!
arrowUp
	^ Form
	extent: 12@12
	depth: 32
	fromArray: #( 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 4294967295 4294967295 0 0 0 0 0 0 0 0 0 4294967295 4284714821 4284714821 4294967295 0 0 0 0 0 0 0 4294967295 4284714821 4287810910 4286361178 4284714821 4294967295 0 0 0 0 0 4294967295 4284714821 4287810910 4288272218 4287876447 4287350105 4284714821 4294967295 0 0 0 4294967295 4284714821 4284714821 4284714821 4288074840 4287677545 4284714821 4284714821 4284714821 4294967295 0 0 4294967295 4294967295 4294967295 4284714821 4287679828 4287810910 4284714821 4294967295 4294967295 4294967295 0 0 0 0 4294967295 4284714821 4287614291 4287810910 4284714821 4294967295 0 0 0 0 0 0 4294967295 4284714821 4284714821 4284714821 4284714821 4294967295 0 0 0 0 0 0 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0)
	offset: 0@0! !

!ToolIcons class methodsFor: 'private' stamp: 'topa 3/9/2015 00:31'!
arrowUpAndDown
	^ Form
	extent: 12@12
	depth: 32
	fromArray: #( 0 0 0 0 0 4294967295 4294967295 0 0 0 0 0 0 0 0 0 4294967295 4284714821 4284714821 4294967295 0 0 0 0 0 0 0 4294967295 4284714821 4287810910 4286361178 4284714821 4294967295 0 0 0 0 0 4294967295 4284714821 4287810910 4288272218 4287876447 4287350105 4284714821 4294967295 0 0 0 4294967295 4284714821 4284714821 4284714821 4288074840 4287677545 4284714821 4284714821 4284714821 4294967295 0 0 4294967295 4294967295 4294967295 4284714821 4287679828 4287810910 4284714821 4294967295 4294967295 4294967295 0 0 4294967295 4294967295 4294967295 4284714821 4287614291 4287810910 4284714821 4294967295 4294967295 4294967295 0 0 4294967295 4284714821 4284714821 4284714821 4288074840 4287677545 4284714821 4284714821 4284714821 4294967295 0 0 0 4294967295 4284714821 4287810910 4288272218 4287876447 4287350105 4284714821 4294967295 0 0 0 0 0 4294967295 4284714821 4287810910 4286361178 4284714821 4294967295 0 0 0 0 0 0 0 4294967295 4284714821 4284714821 4294967295 0 0 0 0 0 0 0 0 0 4294967295 4294967295 0 0 0 0 0)
	offset: 0@0! !

!ToolIcons class methodsFor: 'private' stamp: 'mt 3/5/2015 10:26'!
blank
	^ Form extent: 12 @ 12 depth: 8! !

!ToolIcons class methodsFor: 'private' stamp: 'topa 3/9/2015 00:32'!
breakpoint
	^ Form
	extent: 12@12
	depth: 32
	fromArray: #( 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 4294156190 4294219671 4294085257 4294146939 4294140771 4294138201 4294136145 4284036633 4284036633 4284036633 4294967295 4294967295 4294219671 4294830826 4294697184 4294764005 4294156190 4294156190 4294156190 4291318347 4289820229 4284036633 4294967295 4294967295 4294085257 4294697184 4294140771 4294139229 4294136145 4294136145 4294134860 4291318347 4289820229 4284496668 4294967295 4294967295 4294140771 4294156190 4294138201 4294137173 4294136145 4294134860 4294134860 4288365876 4289820229 4284036633 4294967295 4294967295 4294139229 4294219671 4294136145 4294136145 4294134860 4294134860 4294134860 4288365876 4289820229 4283708184 4294967295 4294967295 4294137173 4294085257 4293619053 4293619053 4293619053 4293619053 4293619053 4287311645 4289820229 4283708184 4294967295 4294967295 4293473589 4292813603 4292418331 4291366414 4290250246 4290250246 4290250246 4284036633 4289820229 4283708184 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 4284036633 4289820229 4283708184 4294967295 0 0 0 0 0 0 0 4294967295 4284036633 4289820229 4283708184 4294967295 0 0 0 0 0 0 0 4294967295 4284036633 4289820229 4284036633 4294967295 0 0 0 0 0 0 0 4294967295 4284036633 4284036633 4284036633 4294967295)
	offset: 0@0! !

!ToolIcons class methodsFor: 'private' stamp: 'topa 3/9/2015 00:32'!
collection
	^  Form
	extent: 12@12
	depth: 32
	fromArray: #( 4294964203 4294947198 4294938953 4294947198 4294964203 0 0 0 0 0 0 0 4294946167 4294483041 4293435505 4293693782 4294945395 0 4293916398 4287874445 4285050207 4287874445 4293916398 0 4294939470 4293566834 4292777576 4291988059 4294938953 0 4287545991 4287348353 4288922521 4286558838 4287283332 0 4294946167 4293891161 4292119903 4293035852 4294945395 0 4285247331 4288988570 4288461714 4287606661 4285050207 0 4294963945 4294946426 4294938181 4294946426 4294963945 0 4287545991 4286821753 4287803784 4286031980 4287283332 0 0 0 0 0 0 0 4293785068 4287677321 4284787547 4287677321 4293785068 0 0 0 4294834926 4294238348 4293907294 4294238348 4294834926 0 0 0 0 0 0 0 4294172295 4293777015 4292988807 4293053292 4294172035 0 0 0 0 0 0 0 4293973090 4293186184 4292462206 4291672945 4293907294 0 0 0 0 0 0 0 4294172295 4293184879 4291738741 4292460898 4294172035 0 0 0 0 0 0 0 4294834924 4294238089 4293907034 4294238089 4294834924 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0)
	offset: 0@0! !

!ToolIcons class methodsFor: 'private' stamp: 'topa 3/9/2015 00:32'!
exception
	^ Form
	extent: 12@12
	depth: 32
	fromArray: #( 0 0 4294371495 4293908066 4293842277 4293841507 4293775456 4293708891 4293577045 4293378633 4294371516 4294834928 0 0 4294172547 4294106220 4294040432 4294039405 4293973096 4293906533 4293576270 4293245752 4294504141 4294835187 0 4294768349 4293974633 4294634386 4294368606 4294235479 4294301290 4294236810 4293444430 4294636250 0 0 0 4294239131 4294370178 4294368870 4294102843 4294036547 4294236543 4293510488 4294437573 0 0 0 4294834665 4293973866 4294502031 4294169671 4293970487 4294170225 4293840750 4294239413 0 0 0 0 4294371762 4294172025 4294435187 4294168905 4293969724 4294169977 4293246022 4293444706 4293378916 4293246298 4293048405 4293511033 4293577033 4294105718 4294171257 4294038641 4294170495 4294103156 4293970801 4293971324 4293640551 4292915263 4293709969 4294371787 4294239138 4294107034 4294040985 4293842310 4293510493 4294036070 4293969512 4293310802 4293378927 4294437840 0 0 0 0 0 4294635995 4293509979 4293773681 4293113677 4293974434 4294967037 0 0 0 0 0 0 4293180237 4293245257 4293312869 4294702567 0 0 0 0 0 0 0 4294106534 4292849719 4294040744 0 0 0 0 0 0 0 0 0 4294239161 4294570202 0 0 0 0 0 0 0 0)
	offset: 0@0! !

!ToolIcons class methodsFor: 'private' stamp: 'topa 3/9/2015 00:33'!
flag
	^ Form
	extent: 12@12
	depth: 32
	fromArray: #( 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 4288859642 4288400378 4287284473 4286102264 4284132854 4283673334 4283016693 4284036633 4284036633 4284036633 4294967295 4294967295 4288400378 4293259774 4293259774 4293259774 4289188346 4288859642 4288859642 4282294213 4289820229 4284036633 4294967295 4294967295 4287284473 4293259774 4284132854 4283673334 4283016693 4282688501 4282425845 4282023874 4289820229 4284627477 4294967295 4294967295 4284132854 4289188346 4283279606 4283016693 4282425845 4282425845 4282425845 4281886101 4289820229 4284036633 4294967295 4294967295 4283673334 4288400378 4283016693 4282819829 4282425845 4282425845 4282425845 4281886101 4289820229 4283708184 4294967295 4294967295 4283016693 4287284473 4285250295 4285250295 4284856054 4284856054 4284856054 4284627477 4289820229 4283708184 4294967295 4294967295 4282232813 4282232813 4282167001 4282167001 4282034889 4282294213 4281965995 4284036633 4289820229 4283708184 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 4284036633 4289820229 4283708184 4294967295 0 0 0 0 0 0 0 4294967295 4284036633 4289820229 4283708184 4294967295 0 0 0 0 0 0 0 4294967295 4284036633 4289820229 4284036633 4294967295 0 0 0 0 0 0 0 4294967295 4284036633 4284036633 4284036633 4294967295)
	offset: 0@0! !

!ToolIcons class methodsFor: 'support' stamp: 'topa 3/26/2015 16:13'!
iconForClass: aClassNameSymbol
	
	self environment
		at: aClassNameSymbol
		ifPresent: [:global | 
			(global respondsTo: #toolIcon) ifTrue: [
				^ global toolIcon]]
		ifAbsent: [^ #blank].
	^ #blank! !

!ToolIcons class methodsFor: 'support' stamp: 'topa 3/26/2015 16:45'!
iconForClass: aBehavior selector: aSelector
	"Answer an icon to be shown for the selector aSymbol in aBehavior."
	(aBehavior respondsTo: #toolIconSelector:) ifTrue: [
		^ aBehavior toolIconSelector: aSelector].
	^ #blank
! !

!ToolIcons class methodsFor: 'accessing' stamp: 'mt 3/5/2015 10:25'!
iconNamed: aSymbol

	^self icons at: aSymbol ifAbsentPut: [self perform: aSymbol]! !

!ToolIcons class methodsFor: 'accessing' stamp: 'topa 3/26/2015 15:21'!
icons

	^ Icons ifNil: [Icons := IdentityDictionary new]! !

!ToolIcons class methodsFor: 'private' stamp: 'topa 3/9/2015 00:33'!
magnitude
	^ Form
	extent: 12@12
	depth: 32
	fromArray: #( 0 4292861919 4286677377 4286479998 4287993237 4288059030 4287598479 4285756275 4285164138 4287072135 0 0 0 0 4288059030 4290690750 4287730065 4289901234 4289703855 4289243304 4289309097 4288322202 0 0 0 0 4293322470 4287335307 4287664272 4292532954 0 0 0 4286611584 0 0 0 0 0 4290295992 4288651167 4286151033 4294309365 0 0 0 0 0 0 0 0 0 4286677377 4288387995 4290164406 0 0 0 0 0 0 0 0 0 4290953922 4284637794 4288124823 0 0 0 0 0 0 0 0 4294309365 4285229931 4285164138 0 0 0 0 0 0 0 0 0 4287269514 4285690482 4288387995 0 0 0 4281611316 0 0 0 0 4290690750 4285493103 4285361517 0 0 4294177779 0 4282137660 0 0 0 4292796126 4283914071 4283782485 4288716960 4288585374 4287598479 4283190348 4281216558 4284440415 0 0 0 4290493371 4285822068 4285558896 4285229931 4285032552 4284835173 4284637794 4284440415 4286940549 0 0 0 0 0 0 0 0 0 0 0 0 0 0)
	offset: 0@0! !

!ToolIcons class methodsFor: 'private' stamp: 'topa 3/9/2015 00:29'!
morph
	^ Form
	extent: 12@12
	depth: 32
	fromArray: #( 4289839580 4288331238 4287542498 4287607777 4287475934 4287344604 4287278554 4287146969 4286949590 4287278037 4288525266 4290234076 4287934423 4287543013 4287016930 4287016930 4286950879 4286885086 4286819293 4286753500 4286621914 4286884827 4286751180 4288196048 4286754273 4287214049 4287608033 4287410913 4287345381 4287673834 4287476968 4287279842 4287542498 4287543013 4286161107 4286291404 4288593123 4293719549 4292209405 4293259773 4293587947 4291880146 4292931041 4293916413 4292668925 4293522685 4289708509 4287144908 4289249763 4291159551 4285840895 4289189887 4290435500 4284722775 4288005764 4291947263 4287417087 4290503935 4290234076 4287341514 4288986848 4291684863 4286957055 4290043135 4290830265 4285314924 4288597652 4292209919 4288008191 4290897919 4290036186 4287209670 4288461538 4293850862 4291814867 4293063142 4294767338 4294434247 4294568413 4294638062 4294176722 4294769125 4289576667 4286684358 4288920552 4290436015 4284919642 4287416979 4294953900 4294936894 4294946435 4294307254 4294235741 4294961804 4289445600 4286487243 4288788710 4291421627 4285904999 4288533664 4294956727 4294941008 4294950033 4294899137 4294696554 4294963868 4289642464 4286355142 4287079630 4292469742 4292075239 4292206827 4292468971 4292467684 4292468200 4292337385 4292271331 4292271590 4287932618 4286157760 4287933133 4287671254 4287867353 4287473106 4286882253 4286685904 4286554058 4286552260 4286486986 4286355142 4286617792 4288458446 0 0 0 0 0 0 0 0 0 0 0 0)
	offset: 0@0! !

!ToolIcons class methodsFor: 'private' stamp: 'topa 3/26/2015 15:20'!
no

	^  Form
	extent: 12@12
	depth: 32
	fromArray: #( 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1357600601 2162906969 2162906969 820729689 0 0 0 0 0 0 283858777 3488307033 4293613401 4293613401 4293613401 4293613401 3219871577 283858777 0 0 0 0 3488307033 4293613401 4293613401 4293613401 4293613401 4293613401 4293613401 3219871577 0 0 0 1357600601 4293613401 4293613401 4293613401 4293613401 4293613401 4293613401 4293613401 4293613401 820729689 0 0 2162906969 4293613401 4294628822 4294628822 4294628822 4294628822 4294628822 4294628822 4293613401 1894471513 0 0 2162906969 4293613401 4294628822 4294628822 4294628822 4294628822 4294628822 4294628822 4293613401 1894471513 0 0 820729689 4293613401 4293613401 4293613401 4293613401 4293613401 4293613401 4293613401 4293613401 820729689 0 0 0 3219871577 4293613401 4293613401 4293613401 4293613401 4293613401 4293613401 2951436121 0 0 0 0 283858777 3219871577 4293613401 4293613401 4293613401 4293613401 2951436121 283858777 0 0 0 0 0 0 820729689 1894471513 1894471513 820729689 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0)
	offset: 0@0! !

!ToolIcons class methodsFor: 'private' stamp: 'topa 3/9/2015 00:27'!
notOverridden

	^ Form
	extent: 12@12
	depth: 32
	fromArray: #( 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 4292955472 4292955472 4292955472 4292955472 4292955472 4292955472 0 0 0 0 0 0 4292955472 4294940569 4294940569 4294940569 4294936200 4292955472 0 0 0 0 0 0 4292955472 4294940569 4294937229 4294937999 4294936200 4292955472 0 0 0 0 0 0 4292955472 4294940569 4294937229 4294937999 4294535787 4292955472 0 0 0 0 0 0 4292955472 4294940569 4294936458 4294535787 4294535787 4292889936 0 0 0 0 0 0 4292955472 4292955472 4292955472 4292955472 4292759120 4292236111 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0)
	offset: 0@0! !

!ToolIcons class methodsFor: 'private' stamp: 'topa 3/26/2015 15:46'!
primitive

	^ Form
	extent: 12@12
	depth: 32
	fromArray: #( 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 4288124823 4288124823 9934743 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 4288124823 4288782753 4288782753 4288124823 9934743 16777215 16777215 16777215 16777215 16777215 16777215 4288124823 4288782753 4288782753 4289045925 4289440683 4288124823 9934743 16777215 16777215 16777215 16777215 4288124823 4288782753 4288782753 4289045925 4289045925 4289440683 4288848546 4288124823 16777215 16777215 16777215 16777215 4288124823 4288782753 4289045925 4289045925 4289045925 4288848546 4288848546 4288124823 16777215 16777215 16777215 16777215 9934743 4288124823 4289440683 4289440683 4288848546 4288848546 4288124823 10658466 16777215 16777215 16777215 16777215 9934743 10592673 4288124823 4288848546 4288848546 4288124823 10658466 10658466 16777215 16777215 16777215 16777215 16777215 9934743 11250603 4288124823 4288124823 10658466 10658466 9934743 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215 16777215)
	offset: 0@0! !

!ToolIcons class methodsFor: 'private' stamp: 'topa 3/26/2015 17:26'!
stream

	^ Form
	extent: 12@12
	depth: 32
	fromArray: #( 0 0 0 0 0 0 0 0 0 0 0 0 0 0 4294967295 4294967295 0 0 4294967295 4294967295 0 0 0 0 0 4294967295 4290038244 4292075503 4294967295 4294967295 4290038244 4292075503 4294967295 0 0 0 0 4294967295 4288921054 4284189380 4292535794 4294967295 4288921054 4284189380 4292535794 4294967295 0 0 0 4294967295 4288921054 4283597761 4284386245 4293850105 4289183968 4283597761 4284386245 4292995828 4294967295 0 0 4294967295 4288921054 4283597761 4283597761 4288461275 4289381089 4283597761 4283597761 4284583622 4293784313 4294967295 0 4294967295 4288921054 4283597761 4283597761 4288198106 4289381088 4283597761 4283597761 4284123588 4293192949 4294967295 0 4294967295 4288921054 4283597761 4283926211 4293127157 4289183968 4283597761 4283926211 4291812590 4294967295 0 0 4294967295 4288921054 4283794882 4291155690 4294967295 4288921054 4283794882 4291155690 4294967295 0 0 0 4294967295 4289381088 4290629607 4294967295 4294967295 4289381088 4290629607 4294967295 0 0 0 0 4294967295 4294967295 4294967295 0 4294967295 4294967295 4294967295 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0)
	offset: 0@0! !

!ToolIcons class methodsFor: 'private' stamp: 'topa 3/9/2015 00:28'!
string

	^ Form
	extent: 12@12
	depth: 32
	fromArray: #( 0 0 4292077049 4285510895 4285577969 4285577455 4285577199 4285577199 4285577199 4285771759 0 0 0 0 4285249007 4286956786 4287810035 4287218674 4285837807 0 4285771759 4285771503 0 0 0 0 4285577455 4287809779 4287809779 4287152882 4285771759 0 4285705967 4285705965 0 0 0 0 4285838063 4286562033 4287743987 4287152882 4285771503 0 4285705453 4285639661 0 0 0 0 4293981949 4286231022 4285642223 4286495729 4285705453 0 4285639661 4285639149 0 0 0 0 0 0 4293981693 4285377517 4285639661 0 4285638893 4285638892 0 0 0 0 0 0 0 4285639661 4285638893 0 4285638892 4285572844 0 0 0 0 0 0 0 4285638893 4285638892 0 4285572844 4285507052 0 0 0 0 0 0 0 4285638380 4285572844 0 4285507052 4285506540 0 0 0 0 0 0 0 4285507052 4285507052 0 4285506540 4285506284 0 0 0 0 0 0 0 4285506796 4285506540 0 4285506028 4285440492 0 0 0 0 0 0 0 4285506284 4285506284 0 4285440236 4285440234 0 0)
	offset: 0@0! !

!Set methodsFor: '*Tools-Inspector' stamp: 'apb 7/14/2004 12:19'!
inspectorClass 
	"Answer the class of the inspector to be used on the receiver.  Called by inspect; 
	use basicInspect to get a normal (less useful) type of inspector."

	^ SetInspector! !

!FutureMaker methodsFor: '*Tools-inspecting' stamp: 'das 9/7/2005 10:09'!
defaultLabelForInspector
	"Answer the default label to be used for an Inspector window on the receiver."
	^self class name! !

!FutureMaker methodsFor: '*Tools-inspecting' stamp: 'das 9/7/2005 10:09'!
inspectorClass
	"Answer the class of the inspector to be used on the receiver.  Called by inspect; 
	use basicInspect to get a normal (less useful) type of inspector."

	^ Inspector! !

!CompiledMethod methodsFor: '*Tools-Debugger-support' stamp: 'eem 3/20/2015 12:27'!
abstractPCForConcretePC: concretePC
	"Answer the abstractPC matching concretePC."

	| abstractPC scanner client endPC |
	self flag: 'belongs in DebuggerMethodMap?'.
	abstractPC := 1.
	scanner := InstructionStream on: self.
	client := InstructionClient new.
	"cache endPC for methods with embedded source; finding out the endPC is very slow in this case..."
	endPC := self endPC.
	[(scanner pc > endPC
	  or: [scanner pc >= concretePC]) ifTrue:
		[^abstractPC].
	 abstractPC := abstractPC + 1.
	 scanner interpretNextInstructionFor: client] repeat! !

!CompiledMethod methodsFor: '*Tools-Debugger-support' stamp: 'eem 6/3/2008 16:15'!
blockExtentsInto: aDictionary from: initialPC to: endPC scanner: scanner numberer: numbererBlock
	"Support routine for startpcsToBlockExtents"
	| extentStart blockSizeOrLocator |
	self flag: 'belongs in DebuggerMethodMap'.
	extentStart := numbererBlock value.
	[scanner pc <= endPC] whileTrue:
		[blockSizeOrLocator := scanner interpretNextInstructionFor: BlockStartLocator new.
		 blockSizeOrLocator isInteger ifTrue:
			[self
				blockExtentsInto: aDictionary
				from: scanner pc
				to: scanner pc + blockSizeOrLocator - 1
				scanner: scanner
				numberer: numbererBlock]].
	aDictionary at: initialPC put: (extentStart to: numbererBlock value).
	^aDictionary! !

!CompiledMethod methodsFor: '*Tools-Debugger-support' stamp: 'eem 6/30/2009 12:37'!
blockExtentsToTempsMap
	"If the receiver has been copied with temp names answer a
	 map from blockExtent to temps map in the same format as
	 BytecodeEncoder>>blockExtentsToTempNamesMap.  if the
	 receiver has not been copied with temps answer nil."
	^self holdsTempNames ifTrue:
		[self mapFromBlockKeys: ((self startpcsToBlockExtents associations asSortedCollection:
										[:a1 :a2| a1 key < a2 key]) collect:
									[:assoc| assoc value])
			toSchematicTemps: self tempNamesString]! !

!CompiledMethod methodsFor: '*Tools-Debugger' stamp: 'eem 3/6/2015 16:20'!
canonicalArgumentName
	^ 'CompiledMethod'! !

!CompiledMethod methodsFor: '*Tools-Debugger-support' stamp: 'eem 6/5/2008 09:10'!
debuggerMap
	^DebuggerMethodMap forMethod: self! !

!CompiledMethod methodsFor: '*Tools-Inspector' stamp: 'ar 9/27/2005 18:32'!
inspectorClass
	"Answer the class of the inspector to be used on the receiver.  Called by inspect; 
	use basicInspect to get a normal (less useful) type of inspector."

	^ CompiledMethodInspector! !

!CompiledMethod methodsFor: '*Tools-Debugger-support' stamp: 'eem 6/29/2009 09:48'!
mapFromBlockKeys: keys toSchematicTemps: schematicTempNamesString
	"Decode a schematicTempNamesString that encodes the layout of temp names
	 in a method and any closures/blocks within it, matching keys in keys to
	 vectors of temp names."
	| map tempNames |
	map := Dictionary new.
	tempNames := schematicTempNamesString readStream.
	keys do:
		[:key| | tempSequence tempIndex |
		tempSequence := OrderedCollection new.
		tempIndex := 0.
		[(tempNames skipSeparators; peek) ifNil: [true] ifNotNil: [:ch| '[]' includes: ch]] whileFalse:
			[tempNames peek = $(
				ifTrue: [tempSequence addAllLast: ((self tempsSubSequenceFrom: (tempNames next; yourself)) withIndexCollect:
														[:temp :index|
														{ temp. { tempIndex + 1. index } }]).
						tempNames peek ~= $) ifTrue: [self error: 'parse error'].
						tempIndex := tempIndex + 1.
						tempNames next]
				ifFalse: [tempSequence addAllLast: ((self tempsSubSequenceFrom: tempNames) withIndexCollect:
														[:temp :index|
														{ temp. tempIndex := tempIndex + 1 }])]].
		map at: key put: tempSequence asArray.
		[tempNames peek = $]] whileTrue: [tempNames next].
		tempNames peek = $[ ifTrue:
			[tempNames next]].
	^map! !

!CompiledMethod methodsFor: '*Tools-Debugger-support' stamp: 'eem 6/29/2009 09:50'!
startpcsToBlockExtents
	"Answer a Dictionary of startpc to Interval of blockExtent, using the
	 identical numbering scheme described in and orchestrated by
	 BlockNode>>analyseArguments:temporaries:rootNode:.  This is
	 used in part to find the temp names for any block in a method, as
	 needed by the debugger.  The other half is to recompile the method,
	 obtaining the temp names for each block extent.  By indirecting through
	 the blockExtent instead of using the startpc directly we decouple the
	 debugger's access to temp names from the exact bytecode; insulating
	 debugging from minor changes in the compiler (e.g. changes in literal
	 pooling, adding prefix bytecodes, adding inst vars to CompiledMethod
	 in literals towards the end of the literal frame, etc).  If the recompilation
	 doesn't produce exactly the same bytecode at exactly the same offset
	 no matter; the blockExtents will be the same."
	| index |
	self flag: 'belongs in DebuggerMethodMap'.
	index := 0.
	^self
		blockExtentsInto: Dictionary new
		from: self initialPC
		to: self endPC
		scanner: (InstructionStream on: self)
		numberer: [| value | value := index. index := index + 2. value]! !

!CompiledMethod methodsFor: '*Tools-Debugger-support' stamp: 'eem 7/1/2009 10:09'!
tempsSubSequenceFrom: tempNamesStream
	^Array streamContents:
		[:tsss|
		[tempNamesStream skipSeparators.
		 tempNamesStream atEnd
		 or: ['[]()' includes: tempNamesStream peek]] whileFalse:
			[tsss nextPut: (String streamContents:
							[:s|
							[s nextPut: tempNamesStream next.
							 tempNamesStream peek
								ifNil: [true]
								ifNotNil: [:peek| ' []()' includes: peek]] whileFalse])]]

	"thisContext method tempsSubSequenceFrom: 'les temps perdu(sont n''est pas la)' readStream"
	"thisContext method tempsSubSequenceFrom: ('les temps perdu(sont n''est pas la)' readStream skipTo: $(; yourself)"! !

!Stream class methodsFor: '*Tools-icons' stamp: 'topa 3/26/2015 17:27'!
toolIcon

	^ #stream! !

SystemWindow subclass: #ArchiveViewer
	instanceVariableNames: 'archive fileName memberIndex viewAllContents'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-ArchiveViewer'!

!ArchiveViewer commentStamp: '<historical>' prior: 0!
This is a viewer window that allows editing and viewing of Zip archives.!

!ArchiveViewer class methodsFor: 'instance creation' stamp: 'nk 1/30/2002 10:18'!
addFileToNewZip: fullName

	"Add the currently selected file to a new zip"
	| zip |
	zip := (ZipArchive new) 
			addFile: fullName 
			as: (FileDirectory localNameFor: fullName); yourself.
	(self open) archive: zip
! !

!ArchiveViewer class methodsFor: 'class initialization' stamp: 'nk 4/29/2004 11:05'!
deleteTemporaryDirectory
	"
	ArchiveViewer deleteTemporaryDirectory
	"

	| dir |
	(dir := self temporaryDirectory) exists ifTrue: [ dir recursiveDelete ].! !

!ArchiveViewer class methodsFor: 'parts bin' stamp: 'tak 7/25/2007 00:03'!
descriptionForPartsBin

	^ self partName: 'Zip Tool' translatedNoop
		categories: #()
		documentation: 'A viewer and editor for Zip archive files' translatedNoop
! !

!ArchiveViewer class methodsFor: 'file list services' stamp: 'nk 11/26/2002 12:46'!
extractAllFrom: aFileName
	(self new) fileName: aFileName; extractAll! !

!ArchiveViewer class methodsFor: 'fileIn/Out' stamp: 'nk 8/21/2004 16:01'!
fileReaderServicesForFile: fullName suffix: suffix 

	|  services |
	services := OrderedCollection new.
	services add: self serviceAddToNewZip.
	({'zip'.'sar'.'pr'. 'mcz'. '*'} includes: suffix)
		ifTrue: [services add: self serviceOpenInZipViewer.
				services add: self serviceExtractAll].
	^ services! !

!ArchiveViewer class methodsFor: 'class initialization' stamp: 'nk 4/29/2004 10:56'!
initialize
	"ArchiveViewer initialize"

	FileList registerFileReader: self.
	Smalltalk addToShutDownList: self.! !

!ArchiveViewer class methodsFor: 'instance creation' stamp: 'nk 2/23/2001 21:52'!
open
	^(self new) createWindow; openInWorld.! !

!ArchiveViewer class methodsFor: 'instance creation' stamp: 'nk 11/26/2002 12:45'!
openOn: aFileName
	| newMe |
	newMe := self new.
	newMe createWindow; fileName: aFileName; openInWorld.
	^newMe! !

!ArchiveViewer class methodsFor: 'file list services' stamp: 'KR 3/21/2008 21:08'!
serviceAddToNewZip
	"Answer a service for adding the file to a new zip"

	^ FileModifyingSimpleServiceEntry 
		provider: self
		label: 'add file to new zip' translatedNoop
		selector: #addFileToNewZip:
		description: 'add file to new zip' translatedNoop
		buttonLabel: 'to new zip' translatedNoop! !

!ArchiveViewer class methodsFor: 'file list services' stamp: 'KR 3/21/2008 21:03'!
serviceExtractAll
	"Answer a service for opening in a zip viewer"

	^ FileModifyingSimpleServiceEntry 
		provider: self
		label: 'extract all to...' translatedNoop
		selector: #extractAllFrom: 
		description: 'extract all files to a user-specified directory' translatedNoop
		buttonLabel: 'extract all' translatedNoop! !

!ArchiveViewer class methodsFor: 'class initialization' stamp: 'KR 3/21/2008 20:53'!
serviceOpenInZipViewer
	"Answer a service for opening in a zip viewer"

	^ SimpleServiceEntry
		provider: self
		label: 'open in zip viewer' translatedNoop
		selector: #openOn: 
		description: 'open in zip viewer' translatedNoop
		buttonLabel: 'open zip' translatedNoop! !

!ArchiveViewer class methodsFor: 'fileIn/Out' stamp: 'sd 2/1/2002 21:18'!
services
	
	^ Array 
		with: self serviceAddToNewZip
		with: self serviceOpenInZipViewer
		
					
			! !

!ArchiveViewer class methodsFor: 'class initialization' stamp: 'nk 4/29/2004 11:06'!
shutDown: quitting
	quitting ifTrue: [ self deleteTemporaryDirectory ].! !

!ArchiveViewer class methodsFor: 'fileIn/Out' stamp: 'nk 4/29/2004 10:56'!
temporaryDirectory
	"Answer a directory to use for unpacking files for the file list services."
	^FileDirectory default directoryNamed: '.archiveViewerTemp'! !

!ArchiveViewer class methodsFor: 'initialize-release' stamp: 'nk 1/30/2002 10:13'!
unload

	FileList unregisterFileReader: self ! !

!ArchiveViewer methodsFor: 'member operations' stamp: 'nk 2/24/2001 23:28'!
addDirectory
	| directory |
	self canAddMember ifFalse: [ ^self ].
	directory := FileList2 modalFolderSelector.
	directory
		ifNil: [^ self].
	archive addTree: directory removingFirstCharacters: directory pathName size + 1.
	self memberIndex: 0.
	self changed: #memberList.! !

!ArchiveViewer methodsFor: 'member operations' stamp: 'edc 7/13/2006 12:10'!
addMember
	| result local full |
	self canAddMember ifFalse: [ ^self ].
	result := FileList2 modalFileSelector .
	result ifNil: [ ^self ].
	
local := result directory localNameFor: result name.

	full := result directory fullNameFor: result name.
	
	(archive addFile: full as: local)
		desiredCompressionMethod: ZipArchive compressionDeflated.
	self memberIndex: self members size.
	self changed: #memberList.! !

!ArchiveViewer methodsFor: 'member operations' stamp: 'ar 9/2/2010 08:49'!
addMemberFromClipboard
	| string newName |
	self canAddMember ifFalse: [ ^self ].
	string := Clipboard clipboardText asString.
	newName := UIManager default
		request: 'New name for member:'
		initialAnswer: 'clipboardText'.
	newName notEmpty ifTrue: [
		(archive addString: string as: newName) desiredCompressionMethod: ZipArchive compressionDeflated.
		self memberIndex: self members size.
		self changed: #memberList.
	]
! !

!ArchiveViewer methodsFor: 'accessing' stamp: 'nk 2/24/2001 11:03'!
archive
	^archive! !

!ArchiveViewer methodsFor: 'initialization' stamp: 'nk 6/10/2004 08:16'!
archive: aZipArchive
	archive := aZipArchive.
	self model: aZipArchive.
	self setLabel: 'New Zip Archive'.
	self memberIndex: 0.
	self changed: #memberList! !

!ArchiveViewer methodsFor: 'initialization' stamp: 'nk 3/7/2004 16:43'!
briefContents
	"Trim to 5000 characters. If the member is longer, then point out that it is trimmed.
	Also warn if the member has a corrupt CRC-32."

	| stream subContents errorMessage |
	self selectedMember ifNil: [^ ''].
	errorMessage := ''.
	stream := WriteStream on: (String new: (self selectedMember uncompressedSize min: 5500)).

	[ self selectedMember uncompressedSize > 5000
		ifTrue: [ |  lastLineEndingIndex tempIndex |
			subContents := self selectedMember contentsFrom: 1 to: 5000.
			lastLineEndingIndex := subContents lastIndexOf: Character cr.
			tempIndex := subContents lastIndexOf: Character lf.
			tempIndex > lastLineEndingIndex ifTrue: [lastLineEndingIndex := tempIndex].
			lastLineEndingIndex = 0
				ifFalse: [subContents := subContents copyFrom: 1 to: lastLineEndingIndex]]
		ifFalse: [ subContents := self selectedMember contents ]]
			on: CRCError do: [ :ex |
				errorMessage := String streamContents: [ :s |
					s nextPutAll: '[ ';
						nextPutAll: (ex messageText copyUpToLast: $( );
						nextPutAll: ' ]' ].
				ex proceed ].

		(errorMessage isEmpty not or: [ self selectedMember isCorrupt ]) ifTrue: [
			stream nextPutAll: '********** WARNING!! Member is corrupt!! ';
					nextPutAll: errorMessage;
					nextPutAll: ' **********'; cr ].

	self selectedMember uncompressedSize > 5000
		ifTrue: [
			stream nextPutAll: 'File ';
				print: self selectedMember fileName;
				nextPutAll: ' is ';
				print: self selectedMember uncompressedSize;
				nextPutAll: ' bytes long.'; cr;
				nextPutAll: 'Click the ''View All Contents'' button above to see the entire file.'; cr; cr;
				nextPutAll: 'Here are the first ';
				print: subContents size;
				nextPutAll: ' characters...'; cr;
				next: 40 put: $-; cr;
				nextPutAll: subContents;
				next: 40 put: $-; cr;
				nextPutAll: '... end of the first ';
				print: subContents size;
				nextPutAll: ' characters.' ]
		ifFalse: [ stream nextPutAll: self selectedMember contents ].
		
		^stream contents
! !

!ArchiveViewer methodsFor: 'menu' stamp: 'nk 3/27/2002 12:48'!
buildWindowMenu
	| menu |
	menu := super buildWindowMenu.
	menu addLine.
	menu add: 'inspect archive' target: archive action: #inspect.
	menu add: 'write prepending file...' target: self action: #writePrependingFile.
	^menu.! !

!ArchiveViewer methodsFor: 'initialization' stamp: 'nk 6/10/2004 08:58'!
buttonColor
	^self defaultBackgroundColor darker! !

!ArchiveViewer methodsFor: 'initialization' stamp: 'nk 6/10/2004 08:59'!
buttonOffColor
	^self defaultBackgroundColor darker! !

!ArchiveViewer methodsFor: 'initialization' stamp: 'nk 6/10/2004 08:59'!
buttonOnColor
	^self defaultBackgroundColor! !

!ArchiveViewer methodsFor: 'member operations' stamp: 'nk 2/24/2001 14:50'!
canAddMember
	^archive notNil! !

!ArchiveViewer methodsFor: 'archive operations' stamp: 'nk 2/24/2001 13:54'!
canCreateNewArchive
	^true! !

!ArchiveViewer methodsFor: 'member operations' stamp: 'nk 2/24/2001 11:02'!
canDeleteMember
	^memberIndex > 0! !

!ArchiveViewer methodsFor: 'archive operations' stamp: 'dgd 2/21/2003 22:36'!
canExtractAll
	^self members notEmpty! !

!ArchiveViewer methodsFor: 'member operations' stamp: 'nk 2/24/2001 11:02'!
canExtractMember
	^memberIndex > 0! !

!ArchiveViewer methodsFor: 'archive operations' stamp: 'nk 2/24/2001 11:12'!
canOpenNewArchive
	^true! !

!ArchiveViewer methodsFor: 'member operations' stamp: 'nk 2/24/2001 11:02'!
canRenameMember
	^memberIndex > 0! !

!ArchiveViewer methodsFor: 'archive operations' stamp: 'nk 2/24/2001 13:55'!
canSaveArchive
	^archive notNil! !

!ArchiveViewer methodsFor: 'member operations' stamp: 'nk 2/24/2001 23:50'!
canViewAllContents
	^memberIndex > 0 and: [ viewAllContents not ]! !

!ArchiveViewer methodsFor: 'member operations' stamp: 'sd 11/20/2005 21:26'!
changeViewAllContents

	(viewAllContents not and: [ self selectedMember notNil and: [ self selectedMember uncompressedSize > 50000 ]])
		ifTrue: [ (self confirm: 'This member''s size is ',
			(self selectedMember uncompressedSize asString),
			'; do you really want to see all that data?')
				ifFalse: [ ^self ]
		].

	viewAllContents := viewAllContents not.
	self changed: #contents! !

!ArchiveViewer methodsFor: 'archive operations' stamp: 'ar 9/2/2010 08:50'!
commentArchive
	| newName |
	archive ifNil: [ ^self ].
	newName := UIManager default
			request: 'New comment for archive:'
			initialAnswer: archive zipFileComment.
	archive zipFileComment: newName.! !

!ArchiveViewer methodsFor: 'member operations' stamp: 'ar 9/2/2010 08:50'!
commentMember
	| newName |
	newName := UIManager default
			request: 'New comment for member:'
			initialAnswer: self selectedMember fileComment.
	self selectedMember fileComment: newName.! !

!ArchiveViewer methodsFor: 'initialization' stamp: 'nk 3/7/2004 16:45'!
contents
	| contents errorMessage |
	self selectedMember ifNil: [^ ''].
	viewAllContents ifFalse: [^ self briefContents].

 	[ contents := self selectedMember contents ]
		on: CRCError
		do: [ :ex | errorMessage := String streamContents: [ :stream |
			stream nextPutAll: '********** WARNING!! Member is corrupt!! [ ';
			nextPutAll: (ex messageText copyUpToLast: $( );
			nextPutAll: '] **********'; cr ].
			ex proceed ].

	^self selectedMember isCorrupt
		ifFalse: [ contents ]
		ifTrue: [ errorMessage, contents ]! !

!ArchiveViewer methodsFor: 'initialization' stamp: 'nk 2/25/2001 00:04'!
contents: aText
	self shouldNotImplement.! !

!ArchiveViewer methodsFor: 'initialization' stamp: 'kfr 4/26/2015 15:30'!
createButtonBar
	| bar |
	
	bar := AlignmentMorph newRow.
	bar
		color: self defaultBackgroundColor;
		rubberBandCells: false;
		vResizing: #shrinkWrap;
		cellInset: 6 @ 0.
	#(#('new archive' #canCreateNewArchive #createNewArchive 'Create a new, empty archive and discard this one') #('load archive' #canOpenNewArchive #openNewArchive 'Open another archive and discard this one') #('save archive' #canSaveArchive #saveArchive 'Save this archive under a new name') #('extract all' #canExtractAll #extractAll 'Extract all this archive''s members into a directory') #('add file' #canAddMember #addMember 'Add a file to this archive') #('add clipboard' #canAddMember #addMemberFromClipboard 'Add the contents of the clipboard as a new file') #('add dir ' #canAddMember #addDirectory 'Add the entire contents of a directory, with all of its subdirectories') #('extract member' #canExtractMember #extractMember 'Extract the selected member to a file') #('delete member' #canDeleteMember #deleteMember 'Remove the selected member from this archive') #('rename member' #canRenameMember #renameMember 'Rename the selected member') #('view all' #canViewAllContents #changeViewAllContents 'Toggle the view of all the selected member''s contents')) 
		do: 
			[:arr | 
			| button |
			(button := PluggableButtonMorph 
						on: self
						getState: arr second
						action: arr third)
				vResizing: #spaceFill;
				hResizing: #spaceFill;
				onColor: self buttonOnColor offColor: self buttonOffColor;
				label: arr first withCRs;
				setBalloonText: arr fourth.
			bar addMorphBack: button].
	^bar! !

!ArchiveViewer methodsFor: 'initialization' stamp: 'edc 11/23/2007 09:32'!
createListHeadingUsingFont: font
	| sm |
	sm := StringMorph contents: ' order  uncomp   comp   CRC-32       date     time     file name'.
	font ifNotNil: [ sm font: font ].
	^(AlignmentMorph newColumn)
		color: self defaultBackgroundColor;
		addMorph: sm;
		yourself.! !

!ArchiveViewer methodsFor: 'archive operations' stamp: 'nk 2/24/2001 23:29'!
createNewArchive
	self setLabel: '(new archive)'.
	archive := ZipArchive new.
	self memberIndex: 0.
	self changed: #memberList.! !

!ArchiveViewer methodsFor: 'initialization' stamp: 'kfr 4/26/2015 15:42'!
createWindow
	| list heading font text buttonBar |

	font := (TextStyle named: #DefaultFixedTextStyle)
		ifNotNil: [ :ts | ts fontArray first].

	buttonBar := self createButtonBar.
	self addMorph: buttonBar
		fullFrame: (LayoutFrame fractions: (0@0 corner: 1.0@0.0) offsets: (0@0 corner: 0@34)).

	self minimumExtent: (buttonBar fullBounds width + 20) @ 230.
	self extent: self minimumExtent.

	heading := self createListHeadingUsingFont: font.
	self addMorph: heading
		fullFrame: (LayoutFrame fractions: (0@0 corner: 1.0@0.0) offsets: (0@34 corner: 0@44)).

	(list := PluggableListMorph new)
		on: self list: #memberList
		selected: #memberIndex changeSelected: #memberIndex:
		menu: #memberMenu:shifted: keystroke: nil.
	list color: self defaultBackgroundColor.

	font ifNotNil: [list font: font].
	self addMorph: list
		fullFrame: (LayoutFrame fractions: (0@0 corner: 1.0@0.8) offsets: (0@50 corner: 0@0)).

	text := PluggableTextMorph on: self 
			text: #contents accept: nil
			readSelection: nil menu: nil.
	self addMorph: text
		frame: (0@0.8 corner: 1.0@1.0).
	text lock.

	self setLabel: 'Ned''s Zip Viewer'! !

!ArchiveViewer methodsFor: 'accessing' stamp: 'mt 7/2/2016 11:36'!
defaultBackgroundColor

	^ Color veryVeryLightGray! !

!ArchiveViewer methodsFor: 'accessing' stamp: 'mt 7/2/2016 11:51'!
defaultColor

	^ Color veryVeryLightGray! !

!ArchiveViewer methodsFor: 'member operations' stamp: 'nk 2/24/2001 23:29'!
deleteMember
	self canDeleteMember ifFalse: [ ^self ].
	archive removeMember: self selectedMember.
	self memberIndex:  0.
	self changed: #memberList.
! !

!ArchiveViewer methodsFor: 'accessing' stamp: 'nk 4/29/2004 10:36'!
directory
	"For compatibility with file list."
	^self error: 'should use readOnlyStream instead!!'! !

!ArchiveViewer methodsFor: 'member list' stamp: 'edc 11/23/2007 09:32'!
displayLineFor: aMember
	| stream dateTime index |
	index := self archive members indexOf: aMember.
	stream := WriteStream on: (String new: 60).
	dateTime := Time dateAndTimeFromSeconds: aMember lastModTime. 
	stream
	nextPutAll: (index printString padded: #left to: 4 with: $  );
	space;
		nextPutAll: (aMember uncompressedSize printString padded: #left to: 8 with: $  );
		space; space;
		nextPutAll: (aMember compressedSize printString padded: #left to: 8 with: $  );
		space; space;
		nextPutAll: (aMember crc32String );
		space; space.
	dateTime first printOn: stream format: #(3 2 1 $- 2 1 2).
	stream space; space.
	dateTime second print24: true showSeconds: false on: stream.
	stream space; space;
		nextPutAll: (aMember fileName ).
	^stream contents! !

!ArchiveViewer methodsFor: 'member order' stamp: 'edc 11/23/2007 09:18'!
downMember
| temp |
	temp := (self archive members) at: memberIndex.
	self archive members at: memberIndex put: (self archive members at: memberIndex  + 1).
	self archive members at: (memberIndex  +1) put: temp.
	self memberIndex:  0.
	self changed: #memberList.! !

!ArchiveViewer methodsFor: 'archive operations' stamp: 'fbs 6/28/2013 08:34'!
extractAll
	| directory |

	self canExtractAll ifFalse: [^ self].
	directory := FileList2 modalFolderSelector ifNil: [^ self].
	UIManager default informUserDuring: [:bar| archive extractAllTo: directory informing: bar].! !

!ArchiveViewer methodsFor: 'archive operations' stamp: 'sd 11/20/2005 21:26'!
extractAllPossibleInDirectory: directory
	"Answer true if I can extract all the files in the given directory safely.
	Inform the user as to problems."
	| conflicts |
	self canExtractAll ifFalse: [ ^false ].
	conflicts := Set new.
	self members do: [ :ea | | fullName |
		fullName := directory fullNameFor: ea localFileName.
		(ea usesFileNamed: fullName) ifTrue: [ conflicts add: fullName ].
	].
	conflicts notEmpty ifTrue: [ | str |
		str := WriteStream on: (String new: 200).
		str nextPutAll: 'The following file(s) are needed by archive members and cannot be overwritten:';
			cr.
		conflicts do: [ :ea | str nextPutAll: ea ] separatedBy: [ str cr ].
		self inform: str contents.
		^false.
	].
	conflicts := Set new.
	self members do: [ :ea | | fullName  |
		fullName := directory relativeNameFor: ea localFileName.
		(directory fileExists: fullName)
			ifTrue: [ conflicts add: fullName ].
	].
	conflicts notEmpty ifTrue: [ | str |
		str := WriteStream on: (String new: 200).
		str nextPutAll: 'The following file(s) will be overwritten:'; cr.
		conflicts do: [ :ea | str nextPutAll: ea ] separatedBy: [ str cr ].
		str cr; nextPutAll: 'Is this OK?'.
		^self confirm: str contents.
	].
	^true.
! !

!ArchiveViewer methodsFor: 'archive operations' stamp: 'nk 11/11/2002 22:14'!
extractDirectoriesIntoDirectory: directory 
	(self members select: [:ea | ea isDirectory]) 
		do: [:ea | ea extractInDirectory: directory]! !

!ArchiveViewer methodsFor: 'archive operations' stamp: 'nk 11/11/2002 22:13'!
extractFilesIntoDirectory: directory 
	(self members reject: [:ea | ea isDirectory]) 
		do: [:ea | ea extractInDirectory: directory]! !

!ArchiveViewer methodsFor: 'member operations' stamp: 'nk 4/29/2004 10:46'!
extractMember
	"Extract the member after prompting for a filename.
	Answer the filename, or nil if error."

	| result name |
	self canExtractMember ifFalse: [ ^nil ].
	result := StandardFileMenu newFile.
	result ifNil: [ ^nil ].
	name := (result directory fullNameFor: result name).
	(archive canWriteToFileNamed: name)
		ifFalse: [ self inform: name, ' is used by one or more members
in your archive, and cannot be overwritten.
Try extracting to another file name'.
			^nil ].
	self selectedMember extractToFileNamed: name.
	^name! !

!ArchiveViewer methodsFor: 'accessing' stamp: 'nk 2/24/2001 11:03'!
fileName
	^fileName! !

!ArchiveViewer methodsFor: 'initialization' stamp: 'nk 2/24/2001 23:28'!
fileName: aString
	archive := ZipArchive new readFrom: aString.
	self setLabel: aString.
	self memberIndex:  0.
	self changed: #memberList! !

!ArchiveViewer methodsFor: 'accessing' stamp: 'nk 4/29/2004 10:53'!
fullName
	"For compatibility with FileList services.
	If this is called, it means that a service that requires a real filename has been requested.
	So extract the selected member to a temporary file and return that name."

	| fullName dir |
	self canExtractMember ifFalse: [ ^nil ].
	dir := FileDirectory default directoryNamed: '.archiveViewerTemp'.
	fullName := dir fullNameFor: self selectedMember localFileName.
	self selectedMember extractInDirectory: dir.
	^fullName! !

!ArchiveViewer methodsFor: 'member list' stamp: 'nk 2/23/2001 22:48'!
highlightMemberList: list with: morphList
	(morphList at: self memberIndex) color: Color red! !

!ArchiveViewer methodsFor: 'initialization' stamp: 'nk 6/10/2004 08:16'!
initialize
	super initialize.
	memberIndex := 0.
	viewAllContents := false.
! !

!ArchiveViewer methodsFor: 'parts bin' stamp: 'dls 10/22/2001 07:40'!
initializeToStandAlone
	self initialize createWindow.! !

!ArchiveViewer methodsFor: 'member operations' stamp: 'nk 2/24/2001 13:01'!
inspectMember
	self selectedMember inspect! !

!ArchiveViewer methodsFor: 'member list' stamp: 'nk 2/24/2001 09:40'!
memberIndex
	^memberIndex! !

!ArchiveViewer methodsFor: 'member list' stamp: 'nk 2/24/2001 23:46'!
memberIndex: n
	memberIndex := n.
	viewAllContents := false.
	self changed: #memberIndex.
	self changed: #contents.! !

!ArchiveViewer methodsFor: 'member list' stamp: 'nk 2/24/2001 11:51'!
memberList
	^ self members collect: [ :ea | self displayLineFor: ea ]! !

!ArchiveViewer methodsFor: 'member list' stamp: 'ul 12/12/2009 14:12'!
memberMenu: menu shifted: shifted
	| services |

	menu
		add: 'Comment archive' target: self selector: #commentArchive;
		balloonTextForLastItem: 'Add a comment for the entire archive'.

	self selectedMember ifNotNil: [ :member |
		menu
			addLine;
			add: 'Inspect member' target: self selector: #inspectMember;
			balloonTextForLastItem: 'Inspect the selected member';
			add: 'Comment member' target: self selector: #commentMember;
			balloonTextForLastItem: 'Add a comment for the selected member';
			addLine;
			add: 'member go up in order ' target: self selector: #upMember;
			add: 'member go down in order ' target: self selector: #downMember;
			add: 'select member order ' target: self selector: #toIndexPlace;
			addLine.
		services := FileList itemsForFile: member fileName.
		menu addServices2: services for: self extraLines: #().
	].


	^menu! !

!ArchiveViewer methodsFor: 'accessing' stamp: 'nk 2/24/2001 14:56'!
members
	^archive ifNil: [ #() asOrderedCollection ]
		ifNotNil: [ archive members asOrderedCollection ]! !

!ArchiveViewer methodsFor: 'archive operations' stamp: 'edc 5/7/2007 11:34'!
openNewArchive
	|  result |
	result := FileList2 modalFileSelector .
	result ifNil: [ ^self ].
	self fileName: (result directory fullNameFor: result name).
! !

!ArchiveViewer methodsFor: 'message handling' stamp: 'nk 2/24/2001 13:16'!
perform: selector orSendTo: otherTarget
	^ self perform: selector! !

!ArchiveViewer methodsFor: 'accessing' stamp: 'ul 12/12/2009 14:12'!
readOnlyStream
	"Answer a read-only stream on the selected member.
	For the various stream-reading services."

	^self selectedMember ifNotNil: [ :mem | mem contentStream ascii ]! !

!ArchiveViewer methodsFor: 'member operations' stamp: 'ar 9/2/2010 08:50'!
renameMember
	| newName |
	self canRenameMember ifFalse: [ ^self ].
	newName := UIManager default
		request: 'New name for member:'
		initialAnswer: self selectedMember fileName.
	newName notEmpty ifTrue: [
		self selectedMember fileName: newName.
		self changed: #memberList
	]! !

!ArchiveViewer methodsFor: 'archive operations' stamp: 'ar 8/6/2009 20:38'!
saveArchive
	| result name |
	
	name := FileDirectory  localNameFor: labelString .
	self canSaveArchive ifFalse: [ ^self ].
	result := UIManager default
		request: 'Name this zip '
		initialAnswer:  name.
	result ifNil: [ ^self ].
	
	(archive canWriteToFileNamed: result)
		ifFalse: [ self inform: name, ' is used by one or more members
in your archive, and cannot be overwritten.
Try writing to another file name'.
			^self ].
	[ archive writeToFileNamed: result ] on: Error do: [ :ex | self inform: ex description. ].
	self setLabel: name asString.
	self changed: #memberList	"in case CRC's and compressed sizes got set"! !

!ArchiveViewer methodsFor: 'accessing' stamp: 'nk 2/24/2001 11:17'!
selectedMember
	^memberIndex
		ifNil: [ nil ]
		ifNotNil: [ self members at: memberIndex ifAbsent: [ ] ]! !

!ArchiveViewer methodsFor: 'initialization' stamp: 'nk 6/10/2004 08:29'!
stream: aStream
	archive := ZipArchive new readFrom: aStream.
	self setLabel: aStream fullName.
	self memberIndex:  0.
	self changed: #memberList! !

!ArchiveViewer methodsFor: 'member order' stamp: 'ar 8/6/2009 20:38'!
toIndexPlace
| index max temp |
max := self archive members size.
index :=0.
[index := (UIManager default
		request: 'To which index '
		initialAnswer:  '1') asInteger.
		index between: 1 and: max] whileFalse.
	temp := (self archive members) at: memberIndex.
	self archive members at: memberIndex put: (self archive members at: index).
	self archive members at: index put: temp.
	self memberIndex:  0.
	self changed: #memberList.! !

!ArchiveViewer methodsFor: 'member order' stamp: 'edc 11/23/2007 09:57'!
upMember
| temp |
	temp := (self archive members) at: memberIndex.
	self archive members at: memberIndex put: (self archive members at: memberIndex  -1).
	self archive members at: (memberIndex  -1) put: temp.
	self memberIndex:  0.
	self changed: #memberList.! !

!ArchiveViewer methodsFor: 'initialization' stamp: 'nk 6/10/2004 08:15'!
windowIsClosing
	archive ifNotNil: [ archive close ].! !

!ArchiveViewer methodsFor: 'archive operations' stamp: 'KR 5/10/2008 14:56'!
writePrependingFile
	| result name prependedName |
	self canSaveArchive ifFalse: [ ^self ].
	result _ (StandardFileMenu newFileMenu: FileDirectory default)
		startUpWithCaption: 'Destination Zip File Name:' translated.
	result ifNil: [ ^self ].
	name _ result directory fullNameFor: result name.
	(archive canWriteToFileNamed: name)
		ifFalse: [ self inform: name, ' is used by one or more members
in your archive, and cannot be overwritten.
Try writing to another file name' translated.
			^self ].
	result _ (StandardFileMenu oldFileMenu: FileDirectory default)
		startUpWithCaption: 'Prepended File:' translated.
	result ifNil: [ ^self ].
	prependedName _ result directory fullNameFor: result name.
	[ archive writeToFileNamed: name prependingFileNamed: prependedName ]
		on: Error
		do: [ :ex | self inform: ex description. ].
	self changed: #memberList	"in case CRC's and compressed sizes got set"! !

!Exception class methodsFor: '*Tools-icons' stamp: 'topa 3/26/2015 16:08'!
toolIcon

	^ #exception! !

!ContextPart methodsFor: '*Tools-debugger access' stamp: 'nice 9/25/2013 22:47'!
errorReportOn: strm
	"Write a detailed error report on the stack (above me) on a stream.  For both the error file, and emailing a bug report.  Suppress any errors while getting printStrings.  Limit the length."

	| cnt aContext startPos |
 	strm print: Date today; space; print: Time now; cr.
	strm cr.
	strm nextPutAll: 'VM: ';
		nextPutAll:  Smalltalk platformName asString;
		nextPutAll: ' - ';
		nextPutAll: Smalltalk asString;
		cr.
	strm nextPutAll: 'Image: ';
		nextPutAll:  SystemVersion current version asString;
		nextPutAll: ' [';
		nextPutAll: Smalltalk lastUpdateString asString;
		nextPutAll: ']';
		cr.
	strm cr.
	SecurityManager default printStateOn: strm.
	
	"Note: The following is an open-coded version of ContextPart>>stackOfSize: since this method may be called during a low space condition and we might run out of space for allocating the full stack."
	cnt := 0.  startPos := strm position.
	aContext := self.
	[aContext notNil and: [(cnt := cnt + 1) < 20]] whileTrue:
		[aContext printDetails: strm.	"variable values"
		strm cr.
		aContext := aContext sender].

	strm cr; nextPutAll: '--- The full stack ---'; cr.
	aContext := self.
	cnt := 0.
	[aContext == nil] whileFalse:
		[cnt := cnt + 1.
		cnt = 20 ifTrue: [strm nextPutAll: ' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -'; cr].
		strm print: aContext; cr.  "just class>>selector"	

		"exit early if too long..."
		strm position > (startPos+ self class maxLengthForASingleDebugLogReport) ifTrue: [strm nextPutAll: '...etc...'.	^ self]. 		cnt > self class maxStackDepthForASingleDebugLogReport ifTrue: [strm nextPutAll: '-- and more not shown --'.	^ self].
		aContext := aContext sender].
! !

!PseudoClass methodsFor: '*Tools-icons' stamp: 'topa 3/26/2015 16:46'!
toolIcon
	"For polymorphy with behaviors"
	^ self realClass ifNil: [#blank] ifNotNil: [:real | real toolIcon]! !

!PseudoClass methodsFor: '*Tools-icons' stamp: 'topa 3/26/2015 16:46'!
toolIconSelector: aSymbol
	"For polymorphy with behaviors"
	^ self realClass ifNil: [#blank] ifNotNil: [:real | real toolIconSelector: aSymbol]! !

!StandardFileStream methodsFor: '*Tools-FileList' stamp: 'ul 11/23/2010 10:58'!
dirAndFileName
	"A temporary solution for supporting project loading via drag and drop."

	^{ self directory. self localName }! !

!Magnitude class methodsFor: '*Tools-icons' stamp: 'topa 3/26/2015 16:06'!
toolIcon

	^ #magnitude! !

!Behavior methodsFor: '*Tools-Browsing' stamp: 'nice 11/3/2013 00:33'!
browse
	^ToolSet browseClass: self! !

!Behavior methodsFor: '*Tools-accessing instances and variables' stamp: 'ul 11/15/2010 10:09'!
inspectAllInstances 
	"Inpsect all instances of the receiver.  1/26/96 sw"

	| all allSize prefix |
	all := self allInstances.
	(allSize := all size) = 0 ifTrue: [^ self inform: 'There are no 
instances of ', self name].
	prefix := allSize = 1
		ifTrue: 	['The lone instance']
		ifFalse:	['The ', allSize printString, ' instances'].
	
	all asArray inspectWithLabel: (prefix, ' of ', self name)! !

!Behavior methodsFor: '*Tools-accessing instances and variables' stamp: 'ul 11/15/2010 10:10'!
inspectSubInstances 
	"Inspect all instances of the receiver and all its subclasses.  CAUTION - don't do this for something as generic as Object!!  1/26/96 sw"

	| all allSize prefix |
	all := self allSubInstances.
	(allSize := all size) = 0 ifTrue: [^ self inform: 'There are no 
instances of ', self name, '
or any of its subclasses'].
	prefix := allSize = 1
		ifTrue: 	['The lone instance']
		ifFalse:	['The ', allSize printString, ' instances'].
	
	all asArray inspectWithLabel: (prefix, ' of ', self name, ' & its subclasses')! !

!Behavior methodsFor: '*Tools-Browser-accessors' stamp: 'eem 8/12/2009 11:45'!
settersReturnValue
	"Determine whether the browser's createInstVarAccessors code will generate
	 setters that answer self (the default here) or the value set.  Classes that want
	 to answer the value set (e.g. VMStructType) override."
	^false! !

!Behavior methodsFor: '*Tools-icons' stamp: 'topa 3/26/2015 16:05'!
toolIcon
	"Tools can annotate me with an icon identified by the symbol this method returns.
	Defaults to no icon"
	^ #blank! !

!Behavior methodsFor: '*Tools-icons' stamp: 'topa 3/26/2015 22:25'!
toolIconSelector: aSymbol
	"Tools can annotate the method identified by aSymbol with an icon identified by the symbol this method returns.
	We customize this for, eg, overriden methods or methods with breaks.
	Defaults to no icon"

	self methodDictionary at: aSymbol ifPresent: [ :method |
		method hasBreakpoint ifTrue: [^ #breakpoint].
		method literalsDo: [:literal |
			(#(halt halt: haltIfNil haltIf: haltOnce haltOnce: haltOnCount: halt:onCount: break) includes: literal)
				ifTrue: [^ #breakpoint].
			(#(flag: needsWork notYetImplemented) includes: literal)
				ifTrue: [^ #flag].
			(#(shouldBeImplemented subclassResponsibility) includes: literal)
				ifTrue: [
					(self isSelectorOverridden: aSymbol)
						ifTrue: [^ #abstract]
						ifFalse: [^ #notOverridden]].
			literal == #shouldNotImplement
				ifTrue: [^ #no]].
		method hasReportableSlip ifTrue: [^ #breakpoint]].
		
	(self isSelectorOverride: aSymbol)
		ifTrue: [
			(self isSelectorOverridden: aSymbol)
				ifTrue: [ ^ #arrowUpAndDown ]
				ifFalse: [ ^ #arrowUp ] ]
		ifFalse: [
			(self isSelectorOverridden: aSymbol)
				ifTrue: [^ #arrowDown ]].

	self methodDictionary at: aSymbol ifPresent: [ :method |
		(method primitive ~= 0 and: [method isQuick not])
			ifTrue: [^ #primitive]].
	

	^ #blank! !

AbstractHierarchicalList subclass: #ObjectExplorer
	instanceVariableNames: 'root currentParent inspector monitorList expression'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Explorer'!

!ObjectExplorer commentStamp: '<historical>' prior: 0!
ObjectExplorer provides a hierarchical alternative to #inspect. Simply evaluate an expression like:

World explore

and enjoy.!

!ObjectExplorer class methodsFor: 'defaults' stamp: 'RAA 6/21/1999 15:55'!
about

	StringHolder new textContents: self comment; openLabel: 'about ',self asString! !

!ObjectExplorer class methodsFor: 'instance creation' stamp: 'mt 4/2/2015 11:00'!
explore: anObject

	^ self new explore: anObject! !

!ObjectExplorer class methodsFor: 'defaults' stamp: 'mt 3/6/2015 17:05'!
nodeClass

	^ ObjectExplorerWrapper! !

!ObjectExplorer class methodsFor: 'instance creation' stamp: 'mt 4/2/2015 11:42'!
openOn: anObject

	^ ToolBuilder open: (self explore: anObject)! !

!ObjectExplorer class methodsFor: 'instance creation' stamp: 'mt 4/2/2015 11:01'!
openOn: anObject withLabel: label 
     "ObjectExplorer openOn: Smalltalk withLabel: 'Smalltalk'"

	^ ToolBuilder open: (self explore: anObject) label: label! !

!ObjectExplorer methodsFor: 'styling' stamp: 'ul 5/5/2015 01:04'!
aboutToStyle: aStyler

	aStyler
		classOrMetaClass: self object class;
		parseAMethod: false.
	^true
	! !

!ObjectExplorer methodsFor: 'menus - actions' stamp: 'mt 2/3/2016 20:16'!
browseClassRefs

	(self selectedClass notNil and: [self selectedClass isTrait not])
		ifTrue: [self systemNavigation browseAllCallsOnClass: self selectedClass theNonMetaClass].! !

!ObjectExplorer methodsFor: 'menus - actions' stamp: 'mt 2/3/2016 20:08'!
browseFull

	^ self selectedClass ifNotNil: [:cls | ToolSet browseClass: cls]! !

!ObjectExplorer methodsFor: 'menus - actions' stamp: 'mt 2/3/2016 20:56'!
browseVariableAssignments

	(self selectedClass notNil and: [self selectedClass isTrait not])
		ifTrue: [self systemNavigation browseVariableAssignments: self selectedClass].! !

!ObjectExplorer methodsFor: 'menus - actions' stamp: 'mt 2/3/2016 20:55'!
browseVariableReferences

	(self selectedClass notNil and: [self selectedClass isTrait not])
		ifTrue: [self systemNavigation browseVariableReferences: self selectedClass].! !

!ObjectExplorer methodsFor: 'toolbuilder' stamp: 'topa 5/5/2015 00:13'!
buildWith: builder

	| windowSpec treeSpec textSpec buttonSpec buttonOffset tool |
	windowSpec := builder pluggableWindowSpec new.
	windowSpec
		model: self;
		children: OrderedCollection new;
		label: #label;
		extent: self initialExtent.
	
	treeSpec := builder pluggableTreeSpec new.
	treeSpec
		model: self;
		nodeClass: self class nodeClass;
		roots: #getList;
		keyPress: #explorerKey:from:event:;
		getSelected: #currentSelection;
		setSelected: #currentSelection:;
		setSelectedParent: #currentParent:;
		menu: #genericMenu:;
		autoDeselect: false;
		columns: (ObjectExplorerWrapper showContentsInColumns
			ifTrue: [{
				[:listMorph | (listMorph filteredItems collect: [:item |
					item preferredWidthOfColumn: 1]) max].
				nil "take all the space"}]);
		frame: (0@0 corner: 1@0.71).
	windowSpec children add: treeSpec.

	buttonOffset := (Preferences standardButtonFont widthOfString: 'inspect') * 3/2.

	textSpec := builder pluggableCodePaneSpec new.
	textSpec
		model: self;
		getText: #expression;
		editText: #expression:;
		menu: #codePaneMenu:shifted:;
		help: 'Evaluate expressions for the current tree selection...' translated;
		frame: (LayoutFrame fractions: (0@0.71 corner: 1@1) offsets: (0@0 corner: buttonOffset negated@0)).
	windowSpec children add: textSpec.

	buttonSpec := builder pluggableButtonSpec new
		model: self;
		label: 'inspect';
		action: #inspectObject;
		help: 'Switch to an inspector tool';
		frame: (LayoutFrame fractions: (1@0.71 corner: 1@1) offsets: (buttonOffset negated@0 corner: 0 @ 0)).
	windowSpec children add: buttonSpec.
	
	tool := builder build: windowSpec.
	self changed: #expandRootsRequested.
	^ tool! !

!ObjectExplorer methodsFor: 'menus - actions' stamp: 'mt 2/3/2016 20:45'!
chasePointersForSelection
	
	self flag: #tooMany. "mt: Note that we might want to ignore references caused by this tool."
	self object chasePointers.! !

!ObjectExplorer methodsFor: 'menus - actions' stamp: 'mt 2/3/2016 20:18'!
classHierarchy
	"Create and schedule a class list browser on the receiver's hierarchy."

	self systemNavigation
		spawnHierarchyForClass: self selectedClass
		selector: nil
! !

!ObjectExplorer methodsFor: 'menus' stamp: 'topa 3/18/2015 09:10'!
codePaneMenu: aMenu shifted: shifted
	"Note that unless we override perform:orSendTo:, PluggableTextController will respond to all menu items"
	^ StringHolder codePaneMenu: aMenu shifted: shifted
! !

!ObjectExplorer methodsFor: 'menus - actions' stamp: 'mt 2/3/2016 20:58'!
copyName
	"Copy the name of the current variable, so the user can paste it into the window below and work with is."
	
	Clipboard clipboardText: self currentSelection key.	! !

!ObjectExplorer methodsFor: 'accessing' stamp: 'mt 3/7/2015 09:32'!
currentParent
	^ currentParent! !

!ObjectExplorer methodsFor: 'accessing' stamp: 'mt 3/7/2015 09:33'!
currentParent: anObject
	currentParent := anObject.! !

!ObjectExplorer methodsFor: 'accessing' stamp: 'mt 3/7/2015 08:26'!
currentSelection

	^ currentSelection! !

!ObjectExplorer methodsFor: 'accessing' stamp: 'mt 2/1/2016 10:30'!
currentSelection: anObject

	self currentSelection == anObject ifTrue: [
		self changed: #objectChanged with: anObject.
		^ self].
	
	currentSelection := anObject.
	
	self changed: #currentSelection.
	self changed: #style.! !

!ObjectExplorer methodsFor: 'menus - actions' stamp: 'mt 2/3/2016 20:40'!
defsOfSelection
	"Open a browser on all defining references to the selected instance variable, if that's what currently selected. "
	
	self selectedParentClass ifNil: [^ self changed: #flash].
	self selectedParentClass isVariable ifTrue: [^ self changed: #flash].
	self selectedInstVarName ifNil: [^ self changed: #flash].

	self systemNavigation browseAllStoresInto: self selectedInstVarName from: self selectedParentClass.! !

!ObjectExplorer methodsFor: 'accessing' stamp: 'RAA 9/23/1999 13:15'!
doItContext
	"Answer the context in which a text selection can be evaluated."

	^nil! !

!ObjectExplorer methodsFor: 'accessing' stamp: 'mt 3/7/2015 10:21'!
doItReceiver
	"Answer the object that should be informed of the result of evaluating a
	text selection."

	^ self object! !

!ObjectExplorer methodsFor: 'menus - actions' stamp: 'mt 2/3/2016 20:46'!
explorePointersForSelection
	
	self flag: #tooMany. "mt: Note that we might want to ignore references caused by this tool."
	self object explorePointers.! !

!ObjectExplorer methodsFor: 'menus' stamp: 'mt 2/3/2016 20:57'!
explorerKey: aChar from: view event: event

	event anyModifierKeyPressed ifFalse: [^ false].

	self object ifNotNil: [
		aChar == $i ifTrue: [self inspectSelection. ^ true].
		aChar == $I ifTrue: [self exploreSelection. ^ true].

		aChar == $b ifTrue:	[self browseFull. ^ true].
		aChar == $h ifTrue:	[self classHierarchy. ^ true].
		aChar == $c ifTrue: [self copyName. ^ true].
		aChar == $p ifTrue: [self browseFullProtocol. ^ true].
		aChar == $N ifTrue: [self browseClassRefs. ^ true].
		aChar == $v ifTrue: [self viewerForValue. ^ true]].

	^ false! !

!ObjectExplorer methodsFor: 'menus - actions' stamp: 'mt 2/3/2016 20:19'!
exploreSelection

	self object explore.! !

!ObjectExplorer methodsFor: 'user interface' stamp: 'mt 4/2/2015 11:00'!
explore: anObject

	self rootObject: anObject.! !

!ObjectExplorer methodsFor: 'accessing' stamp: 'mt 4/12/2015 16:46'!
expression

	^ expression! !

!ObjectExplorer methodsFor: 'accessing' stamp: 'mt 4/12/2015 16:46'!
expression: aString

	expression := aString.! !

!ObjectExplorer methodsFor: 'menus' stamp: 'mt 2/3/2016 20:29'!
genericMenu: aMenu 

	^ self menu: aMenu for: #(treeListMenu treeListMenuShifted:)! !

!ObjectExplorer methodsFor: 'accessing' stamp: 'mt 3/7/2015 12:01'!
getList
	"This explorer shows bindings in a tree structure. Create the root binding here."
	
	^ {self root}! !

!ObjectExplorer methodsFor: 'user interface' stamp: 'mt 4/12/2015 16:09'!
initialExtent

	^350@250! !

!ObjectExplorer methodsFor: 'menus - actions' stamp: 'mt 2/3/2016 20:58'!
inspectBasic
	"Bring up a non-special inspector"

	self object basicInspect.! !

!ObjectExplorer methodsFor: 'toolbuilder' stamp: 'mt 5/4/2016 13:24'!
inspectObject
	"Switch to an inspector tool."
	
	| window currentBounds |
	currentBounds := ToolBuilder findDefault getBoundsForWindow: self containingWindow.

	"Close first because MVC fiddles around with processes."
	self changed: #close. 
		
	window := self rootObject inspect.
	
	"---- In MVC, the lines after this will not be executed ---"

	window model setExpression: self expression.
	ToolBuilder findDefault setBoundsForWindow: window to: currentBounds.! !

!ObjectExplorer methodsFor: 'accessing - other' stamp: 'mt 3/30/2015 10:54'!
inspector
	
	^ inspector ifNil: [inspector := Inspector new]! !

!ObjectExplorer methodsFor: 'menus - actions' stamp: 'mt 2/3/2016 20:19'!
inspectSelection

	self object inspect.! !

!ObjectExplorer methodsFor: 'accessing' stamp: 'mt 11/13/2015 10:35'!
label

	^ self rootObject printStringLimitedTo: 40! !

!ObjectExplorer methodsFor: 'menus' stamp: 'mt 2/3/2016 20:47'!
mainTreeListMenu: aMenu
	<treeListMenu>

	aMenu addStayUpItemSpecial.

	aMenu addList: #(
		('inspect (i)'						inspectSelection)
		('explore (I)'						exploreSelection)
		-
		('method refs to this inst var'		referencesToSelection)
		('methods storing into this inst var'	defsOfSelection)
		('objects pointing to this value'		objectReferencesToSelection)
		('chase pointers'					chasePointersForSelection)
		('explore pointers'				explorePointersForSelection)
		-
		('browse full (b)'					browseFull)
		('browse class'						browseClass)
		('browse hierarchy (h)'					classHierarchy)
		('browse protocol (p)'				browseFullProtocol)
		-
		('references... (r)'					browseVariableReferences)
		('assignments... (a)'					browseVariableAssignments)
		('class refs (N)'						browseClassRefs)
		-
		('copy name (c)'					copyName)		
		('basic inspect'						inspectBasic)).

	Smalltalk isMorphic ifTrue: [
		aMenu addList: #(
			-
			('viewer for this value (v)'		viewerForValue))].

	aMenu addList: #(
		-
		('monitor changes'		monitorSelection)).
				
	monitorList isEmptyOrNil ifFalse: [
		aMenu addList: #(
			('stop monitoring all'		stopMonitoring))].
				
	^ aMenu! !

!ObjectExplorer methodsFor: 'monitoring' stamp: 'mt 3/7/2015 10:48'!
monitorList
	^monitorList ifNil: [ monitorList := WeakOrderedCollection new ].! !

!ObjectExplorer methodsFor: 'menus - actions' stamp: 'mt 2/3/2016 20:31'!
monitorSelection

	self monitor: self currentSelection.! !

!ObjectExplorer methodsFor: 'monitoring' stamp: 'mt 3/7/2015 11:22'!
monitor: aBinding
	"Start stepping and watching the given binding for changes."

	aBinding ifNil: [ ^self ].
	self world ifNil: [ ^self ].
	self monitorList add: aBinding.
	
	self world startStepping: self at: Time millisecondClockValue selector: #step arguments: #() stepTime: 2000.! !

!ObjectExplorer methodsFor: 'accessing - objects' stamp: 'mt 3/7/2015 10:21'!
object

	^ self currentSelection value! !

!ObjectExplorer methodsFor: 'menus - actions' stamp: 'mt 2/3/2016 20:42'!
objectReferencesToSelection
	"Open a list inspector on all the objects that point to the value of the selected instance variable, if any.  "

	self object ifNil: [^ self changed: #flash].

	self systemNavigation
		browseAllObjectReferencesTo: self object
		except: {self currentSelection. self currentParent. self parentObject}
		ifNone: [:obj | self changed: #flash].! !

!ObjectExplorer methodsFor: 'accessing - objects' stamp: 'mt 3/7/2015 10:21'!
parentObject

	^ self currentParent value! !

!ObjectExplorer methodsFor: 'menus - actions' stamp: 'mt 2/3/2016 20:39'!
referencesToSelection
	"Open a browser on all references to the selected instance variable, if that's what currently selected."
	
	self selectedParentClass ifNil: [^ self changed: #flash].
	self selectedParentClass isVariable ifTrue: [^ self changed: #flash].
	self selectedInstVarName ifNil: [^ self changed: #flash].

	self systemNavigation browseAllAccessesTo: self selectedInstVarName from: self selectedParentClass.! !

!ObjectExplorer methodsFor: 'monitoring' stamp: 'nk 7/31/2004 15:01'!
release
	self world ifNotNil: [ self world stopStepping: self selector: #step ].
	super release.! !

!ObjectExplorer methodsFor: 'user interface' stamp: 'cmm 2/13/2010 19:55'!
representsSameBrowseeAs: anotherObjectExplorer
	^ self rootObject == anotherObjectExplorer rootObject! !

!ObjectExplorer methodsFor: 'accessing' stamp: 'mt 3/7/2015 12:00'!
root

	^ root ifNil: [root := 'root' -> nil]! !

!ObjectExplorer methodsFor: 'accessing - objects' stamp: 'mt 3/7/2015 12:01'!
rootObject

	^ self root value! !

!ObjectExplorer methodsFor: 'accessing - objects' stamp: 'mt 3/7/2015 12:01'!
rootObject: anObject

	self root value: anObject.

	self changed: #label.
	self changed: #getList.
	self changed: #expandRootsRequested.

	self currentSelection: self getList first.! !

!ObjectExplorer methodsFor: 'accessing - other' stamp: 'RAA 9/23/1999 13:19'!
selectedClass
	"Answer the class of the receiver's current selection"

	^self doItReceiver class
! !

!ObjectExplorer methodsFor: 'accessing - other' stamp: 'mt 2/3/2016 20:37'!
selectedInstVarName

	self parentObject ifNil: [^ nil].
	(self parentObject class allInstVarNames includes: self currentSelection key)
		ifTrue: [^ self currentSelection key asSymbol].
	^ nil! !

!ObjectExplorer methodsFor: 'accessing - other' stamp: 'mt 2/3/2016 20:35'!
selectedParentClass

	^ self parentObject ifNotNil: [:o | o class]! !

!ObjectExplorer methodsFor: 'accessing - other' stamp: 'mt 4/12/2015 16:51'!
setExpression: aString

	self expression: aString.
	self changed: #expression.! !

!ObjectExplorer methodsFor: 'monitoring' stamp: 'nk 7/12/2003 17:55'!
shouldGetStepsFrom: aWorld
	^self monitorList notEmpty! !

!ObjectExplorer methodsFor: 'monitoring' stamp: 'mt 3/7/2015 10:52'!
step
	"Let all views know that some of my objects need to be updated."

	self monitorList do: [ :object |
		object ifNotNil: [self changed: #objectChanged with: object]].
	self monitorList ifEmpty: [ 
		ActiveWorld stopStepping: self selector: #step ].! !

!ObjectExplorer methodsFor: 'monitoring' stamp: 'nk 7/31/2004 15:01'!
stopMonitoring
	monitorList := nil.
	self world stopStepping: self selector: #step! !

!ObjectExplorer methodsFor: 'menus - actions' stamp: 'mt 2/3/2016 20:17'!
viewerForValue

	self object beViewed.! !

!ObjectExplorer methodsFor: 'monitoring' stamp: 'nk 7/31/2004 15:01'!
world
	^ActiveWorld! !

ObjectExplorer subclass: #PointerExplorer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Explorer'!

!PointerExplorer commentStamp: 'avi 8/21/2004 20:01' prior: 0!
A variant on the ObjectExlorer that works "backwards": like the ObjectExplorer, it shows a tree of objects, but expanding a node won't show the objects which that node references, but rather the objects that reference that node.  Its main use is to track down memory leaks: if you want to know why a particular object is still alive, open a PointerExplorer on it and drill down until you find the root object that's referencing it.  For example, find all the references to the symbol #zot with:

PointerExplorer new openExplorerFor: #zot

For the "name" of the object, the PointerExplorer shows each object's identityHash, to allow the user to identify when two similar objects are identical and notice cycles.!

!PointerExplorer class methodsFor: 'defaults' stamp: 'mt 3/6/2015 17:05'!
nodeClass

	^ PointerExplorerWrapper! !

!PointerExplorer methodsFor: 'accessing' stamp: 'mt 3/7/2015 12:02'!
rootObject: anObject

	self root key: anObject identityHash asString.
	super rootObject: anObject.! !

StringHolder subclass: #CodeHolder
	instanceVariableNames: 'currentCompiledMethod contentsSymbol multiWindowState'
	classVariableNames: 'ContentsSymbolQuints EditContentsOptions MultiWindowBrowsers'
	poolDictionaries: ''
	category: 'Tools-Base'!

!CodeHolder commentStamp: '<historical>' prior: 0!
An ancestor class for all models which can show code.  Eventually, much of the code that currently resides in StringHolder which only applies to code-holding StringHolders might get moved down here.!

CodeHolder subclass: #Browser
	instanceVariableNames: 'environment systemOrganizer classOrganizer metaClassOrganizer editSelection metaClassIndicated selectedSystemCategory selectedClassName selectedMessageName selectedMessageCategoryName classDefinition metaClassDefinition'
	classVariableNames: 'ListClassesHierarchically RecentClasses ShowClassIcons ShowMessageIcons SortMessageCategoriesAlphabetically'
	poolDictionaries: ''
	category: 'Tools-Browser'!

!Browser commentStamp: 'cwp 12/27/2012 11:09' prior: 0!
I represent a query path into the class descriptions, the software of the system.!

!Browser class methodsFor: 'preferences' stamp: 'eem 7/1/2010 09:24'!
canUseMultiWindowBrowsers
	^true! !

!Browser class methodsFor: 'class initialization' stamp: 'ar 2/26/2010 23:02'!
cleanUp
	"Flush caches"

	self initialize.! !

!Browser class methodsFor: 'instance creation' stamp: 'mt 1/22/2016 14:05'!
fullOnClass: aClass 
	"Open a new full browser set to class."

	^ self
		openBrowserView: (self new
			setClass: aClass;
			openEditString: nil)
		label: nil! !

!Browser class methodsFor: 'instance creation' stamp: 'mt 1/22/2016 14:06'!
fullOnClass: aClass category: category

	^ self 
		openBrowserView: (self new
			setClass: aClass;
			selectMessageCategoryNamed: category;
			openEditString: nil)
		label: nil! !

!Browser class methodsFor: 'instance creation' stamp: 'mt 1/22/2016 14:06'!
fullOnClass: aClass selector: aSelector
	"Open a new full browser set to class."

	^ self 
		openBrowserView: (self new
			setClass: aClass selector: aSelector;
			openEditString: nil)
		label: nil! !

!Browser class methodsFor: 'class initialization' stamp: 'hpt 8/5/2004 19:41'!
initialize
	"Browser initialize"

	RecentClasses := OrderedCollection new.
	self 
		registerInFlapsRegistry;
		registerInAppRegistry	! !

!Browser class methodsFor: 'preferences' stamp: 'rkrk 8/23/2009 18:27'!
listClassesHierarchically
	<preference: 'List classes hierarchically'
		category: 'browsing'
		description: 'When enabled, the class list in the browser is arranged and indented with regard to the class hierarchy.'
		type: #Boolean>
	^ListClassesHierarchically ifNil: [false]
! !

!Browser class methodsFor: 'preferences' stamp: 'rkrk 8/23/2009 18:30'!
listClassesHierarchically: aBool

	ListClassesHierarchically := aBool! !

!Browser class methodsFor: 'instance creation' stamp: 'di 10/18/1999 22:03'!
new

	^super new systemOrganizer: SystemOrganization! !

!Browser class methodsFor: 'instance creation' stamp: 'fbs 3/5/2011 20:55'!
newOnCategory: aCategory
	"Browse the system category of the given name.  7/13/96 sw"

	"Browser newOnCategory: 'Interface-Browser'"

	| newBrowser newCat |
	newBrowser := self new..
	newCat := aCategory asSymbol.
	(newBrowser systemCategoryList includes: newCat)
		ifTrue: [ newBrowser selectSystemCategory: newCat ]
		ifFalse: [ ^ self inform: 'No such category' ].
	
	^ self 
		openBrowserView: (newBrowser openSystemCatEditString: nil)
		label: 'Classes in category ', aCategory
! !

!Browser class methodsFor: 'instance creation' stamp: 'tk 4/18/1998 16:28'!
newOnClass: aClass 
	"Open a new class browser on this class."
	^ self newOnClass: aClass label: 'Class Browser: ', aClass name! !

!Browser class methodsFor: 'instance creation' stamp: 'mt 1/18/2016 08:43'!
newOnClass: aClass label: aLabel
	"Open a new class browser on this class."
	| newBrowser |

	newBrowser := self new.
	newBrowser setClass: aClass.
	^ self 
		openBrowserView: (newBrowser openOnClassWithEditString: nil)
		label: aLabel
! !

!Browser class methodsFor: 'instance creation' stamp: 'jcg 10/29/2003 23:12'!
newOnClass: aClass selector: aSymbol
	"Open a new class browser on this class."
	| newBrowser |

	newBrowser := self new.
	newBrowser setClass: aClass selector: aSymbol.
	^ self 
		openBrowserView: (newBrowser openOnClassWithEditString: nil)
		label: 'Class Browser: ', aClass name
! !

!Browser class methodsFor: 'instance creation' stamp: 'mt 1/22/2016 13:46'!
newOnMessageCategory: aCategory inClass: aClass

	^ self
		openBrowserView: (self new
			setClass: aClass;
			selectMessageCategoryNamed: aCategory;
			openMessageCatEditString: nil)
		label: 'Message Category Browser (' , aClass name, ')'.! !

!Browser class methodsFor: 'instance creation' stamp: 'md 3/10/2006 21:46'!
open
	^self openBrowser

! !

!Browser class methodsFor: 'instance creation' stamp: 'jcg 10/29/2003 23:11'!
openBrowser
	"Create and schedule a BrowserView with default browser label. The
	view consists of five subviews, starting with the list view of system
	categories of SystemOrganization. The initial text view part is empty."

	| br |
	br := self new.
	^ self
		openBrowserView: (br openEditString: nil)
		label: br defaultBrowserTitle.

! !

!Browser class methodsFor: 'instance creation' stamp: 'mt 1/22/2016 14:04'!
openBrowserView: aBrowserView label: aString 
	"Schedule aBrowserView, labelling the view aString."
	
	(aBrowserView isKindOf: ToolBuilderSpec)
		ifTrue:[
			(self canUseMultiWindowBrowsers
				 and: [self useMultiWindowBrowsers])
					ifTrue: [aBrowserView multiWindowStyle: #labelButton].
			aString
				ifNil: [ToolBuilder open: aBrowserView]
				ifNotNil: [ToolBuilder open: aBrowserView label: aString]]
		ifFalse:[
			aBrowserView isMorph
				ifTrue: [
					aString ifNotNil: [aBrowserView setLabel: aString].
					aBrowserView openInWorld]
				ifFalse: [
					aString ifNotNil: [aBrowserView label: aString].
					aBrowserView minimumSize: 300 @ 200.
					aBrowserView subViews do: [:each | each controller].
					aBrowserView controller open]].

	^ aBrowserView model
! !

!Browser class methodsFor: 'instance creation' stamp: 'ar 8/7/2009 21:03'!
prototypicalToolWindow
	"Answer an example of myself seen in a tool window, for the benefit of parts-launching tools"

	| aWindow |
	aWindow := self new openEditString: nil.
	^ ToolBuilder build: aWindow! !

!Browser class methodsFor: 'class initialization' stamp: 'hpt 8/5/2004 19:41'!
registerInAppRegistry
	"Register the receiver in the SystemBrowser AppRegistry"
	SystemBrowser register: self.! !

!Browser class methodsFor: 'class initialization' stamp: 'KR 8/12/2007 10:45'!
registerInFlapsRegistry
	"Register the receiver in the system's flaps registry"
	self environment
		at: #Flaps
		ifPresent: [:cl | cl registerQuad: {#Browser. #prototypicalToolWindow. 'Browser' translatedNoop. 'A Browser is a tool that allows you to view all the code of all the classes in the system' translatedNoop}
						forFlapNamed: 'Tools']! !

!Browser class methodsFor: 'preferences' stamp: 'mt 3/5/2015 11:30'!
showClassIcons
	<preference: 'Show class icons'
		category: 'browsing'
		description: 'When enabled, the class list will show icons for some standard types such as Morph and Magnitude.'
		type: #Boolean>
	^ShowClassIcons ifNil: [true]
! !

!Browser class methodsFor: 'preferences' stamp: 'mt 3/5/2015 11:30'!
showClassIcons: aBoolean
	ShowClassIcons := aBoolean.! !

!Browser class methodsFor: 'preferences' stamp: 'mt 3/5/2015 10:15'!
showMessageIcons
	<preference: 'Show message icons'
		category: 'browsing'
		description: 'When enabled, the message list will show icons for flags, overrides, etc.'
		type: #Boolean>
	^ShowMessageIcons ifNil: [true]
! !

!Browser class methodsFor: 'preferences' stamp: 'mt 3/5/2015 10:15'!
showMessageIcons: aBoolean

	ShowMessageIcons := aBoolean.! !

!Browser class methodsFor: 'preferences' stamp: 'mt 3/1/2015 10:33'!
sortMessageCategoriesAlphabetically

	<preference: 'Sort Message Categories Alphabetically'
		categoryList: #(browsing Tools)
		description: 'When enabled, the message category list in the browser is ordered by alphabet. No need to ''alphabetize'' this list manually via the context menu.'
		type: #Boolean>
		
	^ SortMessageCategoriesAlphabetically ifNil: [false]
! !

!Browser class methodsFor: 'preferences' stamp: 'mt 3/1/2015 10:33'!
sortMessageCategoriesAlphabetically: aBoolean

	SortMessageCategoriesAlphabetically := aBoolean.
! !

!Browser class methodsFor: 'instance creation' stamp: 'nk 6/2/2004 12:55'!
systemOrganizer: anOrganizer

	^(super new)
		systemOrganizer: anOrganizer;
		yourself! !

!Browser class methodsFor: 'class initialization' stamp: 'hpt 8/5/2004 19:42'!
unload
	"Unload the receiver from global registries"

	self environment at: #Flaps ifPresent: [:cl |
	cl unregisterQuadsWithReceiver: self].
	SystemBrowser unregister: self.! !

!Browser methodsFor: 'code pane' stamp: 'mt 3/16/2016 14:18'!
aboutToStyle: aStyler
	"This is a notification that aStyler is about to re-style its text.
	Set the classOrMetaClass in aStyler, so that identifiers
	will be resolved correctly.
	Answer true to allow styling to proceed, or false to veto the styling"
	| type |
	
	self isModeStyleable ifFalse: [^false].
	type := self editSelection.
	(#(newMessage editMessage editClass newClass) includes: type) ifFalse:[^false].
	aStyler classOrMetaClass: ((type = #editClass or: [type = #newClass]) ifFalse:[self selectedClassOrMetaClass]).
	^true! !

!Browser methodsFor: 'class functions' stamp: 'nice 10/20/2009 21:19'!
addAllMethodsToCurrentChangeSet
	"Add all the methods in the selected class or metaclass to the current change set.  You ought to know what you're doing before you invoke this!!"

	| aClass |
	(aClass := self selectedClassOrMetaClass) ifNotNil:
		[aClass selectorsDo:
			[:sel |
				ChangeSet current adoptSelector: sel forClass: aClass].
		self changed: #annotation]
! !

!Browser methodsFor: 'message category functions' stamp: 'mt 7/19/2016 14:27'!
addCategory
	"Present a choice of categories or prompt for a new category name and add it before the current selection, or at the end if no current selection"
	| labels reject lines newName oldCategory |
	self okToChange ifFalse: [^ self].
	self hasClassSelected ifFalse: [^ self].
	labels := OrderedCollection new.
	reject := Set new.
	reject
		addAll: self selectedClassOrMetaClass organization categories;
		add: ClassOrganizer nullCategory;
		add: ClassOrganizer default.
	lines := OrderedCollection new.
	self selectedClassOrMetaClass allSuperclasses do: [:cls | | cats |
		cls = Object ifFalse: [
			cats := cls organization categories reject:
				 [:cat | reject includes: cat].
			cats isEmpty ifFalse: [
				lines add: labels size.
				labels addAll: cats asSortedCollection.
				reject addAll: cats]]].
	(newName := UIManager default
		chooseFromOrAddTo: labels
		lines: lines
		title: 'Add Category') ifNil: [^ self].
	oldCategory := self selectedMessageCategoryName.
	newName isEmpty
		ifTrue: [^ self]
		ifFalse: [newName := newName asSymbol].
	self classOrMetaClassOrganizer
		addCategory: newName
		before: (self hasMessageCategorySelected
				ifFalse: [nil]
				ifTrue: [self selectedMessageCategoryName]).
	self changed: #messageCategoryList.
	self selectMessageCategoryNamed:
		(oldCategory isNil
			ifTrue: [self classOrMetaClassOrganizer categories last]
			ifFalse: [oldCategory]).
	self changed: #messageCategoryList.
! !

!Browser methodsFor: 'message functions' stamp: 'sw 1/11/2001 07:22'!
addExtraShiftedItemsTo: aMenu
	"The shifted selector-list menu is being built; some menu items are appropriate only for certain kinds of browsers, and this gives a hook for them to be added as approrpiate.  If any is added here, a line should be added first -- browse reimplementors of this message for examples."
! !

!Browser methodsFor: 'user interface' stamp: 'hpt 9/30/2004 20:51'!
addModelItemsToWindowMenu: aMenu
	"Add model-related items to the window menu"
	super addModelItemsToWindowMenu: aMenu.
	SystemBrowser addRegistryMenuItemsTo: aMenu inAccountOf: self.! !

!Browser methodsFor: 'traits' stamp: 'al 1/9/2006 18:29'!
addSpecialMenu: aMenu
	aMenu addList: #(
		-
		('new class'				newClass)
		('new trait'				newTrait)
		-).
	self selectedClass notNil ifTrue: [
		aMenu addList: #(
			('add trait' addTrait)
			-) ].
	aMenu addList: #(-).
	^ aMenu! !

!Browser methodsFor: 'system category functions' stamp: 'fbs 3/5/2011 20:54'!
addSystemCategory
	"Prompt for a new category name and add it before the
	current selection, or at the end if no current selection"
	| oldSelection newName |
	self okToChange ifFalse: [^ self].
	oldSelection := self selectedSystemCategory.
	newName := self
		request: 'Please type new category name'
		initialAnswer: 'Category-Name'.
	newName isEmpty
		ifTrue: [^ self]
		ifFalse: [newName := newName asSymbol].
	systemOrganizer
		addCategory: newName
		before: self selectedSystemCategory.
	self selectSystemCategory:
		(oldSelection isNil
			ifTrue: [ self systemCategoryList last ]
			ifFalse: [ oldSelection ]).
	self changed: #systemCategoryList.! !

!Browser methodsFor: 'traits' stamp: 'ar 12/27/2009 21:58'!
addTrait
	| input trait |
	input := UIManager default request: 'add trait'.
	input isEmptyOrNil ifFalse: [
		trait := Smalltalk classNamed: input.
		(trait isNil or: [trait isTrait not]) ifTrue: [
			^self inform: 'Input invalid. ' , input , ' does not exist or is not a trait'].
		self selectedClass setTraitComposition: self selectedClass traitComposition asTraitComposition +  trait.
		self contentsChanged].
! !

!Browser methodsFor: 'message category functions' stamp: 'fbs 5/1/2011 08:47'!
alphabetizeMessageCategories
	| oldMessageCategory oldMethod |
	oldMessageCategory := selectedMessageCategoryName.
	oldMethod := selectedMessageName.
	self hasClassSelected ifFalse: [^ false].
	self okToChange ifFalse: [^ false].
	self classOrMetaClassOrganizer sortCategories.
	self clearUserEditFlag.
	self editClass.
	self selectClassNamed: selectedClassName.
	self selectMessageCategoryNamed: oldMessageCategory.
	self selectMessageNamed: oldMethod.
	^ true! !

!Browser methodsFor: 'system category functions' stamp: 'fbs 3/5/2011 20:55'!
alphabetizeSystemCategories

	self okToChange ifFalse: [^ false].
	systemOrganizer sortCategories.
	self selectSystemCategory: nil.
	self changed: #systemCategoryList.
! !

!Browser methodsFor: 'annotation' stamp: 'ar 8/8/2009 17:04'!
annotation
	"Provide a line of content for an annotation pane, representing information about the method associated with the selected class and selector in the receiver."

	|  aSelector aClass |
	(aClass := self selectedClassOrMetaClass) == nil ifTrue: [^ ''].
	self editSelection == #editComment ifTrue:
		[^ self annotationForSelector: #Comment ofClass: aClass].

	self editSelection == #editClass ifTrue:
		[^ self annotationForSelector: #Definition ofClass: aClass].
	(aSelector := self selectedMessageName) ifNil: [^ ''].
	^ self annotationForSelector: aSelector ofClass: aClass! !

!Browser methodsFor: 'class comment pane' stamp: 'fbs 1/3/2013 22:48'!
annotationForClassCommentFor: aClass
	"Provide a line of content for an annotation pane, given that the receiver is pointing at the class comment of the given class."
	| aStamp |
	aStamp :=  classOrganizer commentStamp.
	^ aStamp
		ifNil:
			[self selectedClassName, ' has no class comment']
		ifNotNil:
			['class comment for ', self selectedClassName,
				(aStamp = '<historical>'
					ifFalse:
						[' - ', aStamp]
					ifTrue:
						[''])]! !

!Browser methodsFor: 'class comment pane' stamp: 'ar 8/8/2009 17:21'!
annotationForClassDefinitionFor: aClass
	"Provide a line of content for an annotation pane, given that the receiver is pointing at the class definition of the given class."
	^self classCommentText! !

!Browser methodsFor: 'annotation' stamp: 'bf 4/12/2011 15:27'!
annotation: aText 
	"The user accepted aText in our annotation pane"
	| theClass |
	(theClass := self selectedClass) ifNil: [^false].
	self editSelection == #editClass
		ifTrue: [
			self stripNaggingAttributeFromComment: aText.
			theClass comment: aText stamp: Utilities changeStamp.
			self changed: #classCommentText.
			^ true].
	^ false
! !

!Browser methodsFor: 'multi-window support' stamp: 'mt 5/1/2015 14:07'!
arrowKey: aChar from: view
	"Intercept Apple-Digit to select panes"
	| index |
	(aChar isDigit
	 and: [self multiWindowState notNil]) ifTrue:
	 	[index := aChar asciiValue - $0 asciiValue.
		index = 0 ifTrue: [index := 10].
		^index <= self multiWindowState models size
			ifTrue: [self multiWindowState selectWindowIndex: index]
			ifFalse: [self changed: #flash]].
	^super arrowKey: aChar from: view
! !

!Browser methodsFor: 'system category functions' stamp: 'sd 1/5/2002 21:11'!
browseAllClasses
	"Create and schedule a new browser on all classes alphabetically."
	| newBrowser |
	newBrowser := HierarchyBrowser new initAlphabeticListing.
	self class openBrowserView: (newBrowser openSystemCatEditString: nil)
		label: 'All Classes Alphabetically'! !

!Browser methodsFor: 'message functions' stamp: 'ul 4/7/2016 22:48'!
browseAllCommentsForClass
	"Opens a HelpBrowser on the class"

	| myClass |
	myClass := self selectedClassOrMetaClass ifNil: [ ^self ].
	myClass isTrait ifTrue: [ ^self ].
	(Smalltalk classNamed: #HelpBrowser)
		ifNil: [ ^self inform: 'HelpBrowser is not available.' ]
		ifNotNil: [ :HelpBrowser |
			HelpBrowser openOn: myClass theNonMetaClass ]
! !

!Browser methodsFor: 'toolbuilder' stamp: 'mt 4/24/2015 11:03'!
buildClassListSingletonWith: builder
	| listSpec |
	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #classListSingleton; 
		getIndex: #indexIsOne; 
		setIndex: #indexIsOne:; 
		menu: #classListMenu:shifted:;
		keyPress: #classListKey:from:;
		hScrollBarPolicy: #never;
		vScrollBarPolicy: #never;
		minimumHeight: 0.
	^listSpec
! !

!Browser methodsFor: 'toolbuilder' stamp: 'mt 3/5/2015 11:29'!
buildClassListWith: builder
	| listSpec |
	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #classList;
		getIndex: #classListIndex; 
		setIndex: #classListIndex:;
		icon: #classIconAt:; 
		menu: #classListMenu:shifted:; 
		keyPress: #classListKey:from:.
	SystemBrowser browseWithDragNDrop 
		ifTrue:[listSpec dragItem: #dragFromClassList:].

	^listSpec
! !

!Browser methodsFor: 'message category functions' stamp: ''!
buildMessageCategoryBrowser
	"Create and schedule a message category browser for the currently 
	selected message category."

	self buildMessageCategoryBrowserEditString: nil! !

!Browser methodsFor: 'message category functions' stamp: 'mt 1/22/2016 13:49'!
buildMessageCategoryBrowserEditString: aString 
	"Create and schedule a message category browser for the currently 
	selected	 message category. The initial text view contains the characters 
	in aString."

	^ self hasMessageCategorySelected ifTrue: [
		(self class
			newOnMessageCategory: self selectedMessageCategoryName
			inClass: self selectedClassOrMetaClass)
				"Select my message."
				selectMessageNamed: self selectedMessageName;
				yourself]! !

!Browser methodsFor: 'toolbuilder' stamp: 'ul 4/7/2016 22:57'!
buildMessageCategoryListWith: builder
	| listSpec |
	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #messageCategoryList; 
		getIndex: #messageCategoryListIndex; 
		setIndex: #messageCategoryListIndex:; 
		menu: #messageCategoryMenu:; 
		keyPress: #messageCategoryListKey:from:.
	SystemBrowser browseWithDragNDrop ifTrue:[
		listSpec
			dropAccept: #wantsMessageCategoriesDrop:;
			dropItem: #dropOnMessageCategories:at:].
	^listSpec
! !

!Browser methodsFor: 'toolbuilder' stamp: 'mt 4/24/2015 11:04'!
buildMessageListCatSingletonWith: builder
	| listSpec |
	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #messageCatListSingleton; 
		getIndex: #indexIsOne; 
		setIndex: #indexIsOne:; 
		menu: #messageCategoryMenu:;
		hScrollBarPolicy: #never;
		vScrollBarPolicy: #never;
		minimumHeight: 0.
	^listSpec
! !

!Browser methodsFor: 'toolbuilder' stamp: 'mt 4/5/2016 10:31'!
buildMessageListWith: builder
	| listSpec |
	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #messageList; 
		getIndex: #messageListIndex; 
		setIndex: #messageListIndex:; 
		icon: #messageIconAt:;
		helpItem: #messageHelpAt:;
		menu: #messageListMenu:shifted:; 
		keyPress: #messageListKey:from:.
	SystemBrowser browseWithDragNDrop 
		ifTrue:[listSpec dragItem: #dragFromMessageList:].
	^listSpec
! !

!Browser methodsFor: 'toolbuilder' stamp: 'mt 5/3/2015 15:36'!
buildSwitchesWith: builder
	"Build the instance/comment/class switch"

	| panelSpec i q c |
	panelSpec := builder pluggablePanelSpec new
		layout: #horizontal;
		spacing: -1;
		children: OrderedCollection new;
		yourself.

	i := builder pluggableButtonSpec new.
	i 
			model: self;
			label: 'instance';
			help: 'Show instance-side methods' translated;
			state: #instanceMessagesIndicated; 
			action: #indicateInstanceMessages.

	q := builder pluggableButtonSpec new.
	q 
			model: self;
			horizontalResizing: #shrinkWrap;
			label: '?';
			help: 'Cycle between definition, comment, and hierarchy view' translated; 
			state: #classCommentIndicated; 
			action: #plusButtonHit.

	c := builder pluggableButtonSpec new.
	c 
			model: self;
			label: 'class';
			help: 'Show class-side methods' translated;
			state: #classMessagesIndicated; 
			action: #indicateClassMessages.
			
	panelSpec children addAll: {
		i. c.
		builder pluggableSpacerSpec new.
		q
	}.

	^panelSpec! !

!Browser methodsFor: 'system category functions' stamp: ''!
buildSystemCategoryBrowser
	"Create and schedule a new system category browser."

	self buildSystemCategoryBrowserEditString: nil! !

!Browser methodsFor: 'system category functions' stamp: 'fbs 3/5/2011 20:55'!
buildSystemCategoryBrowserEditString: aString 
	"Create and schedule a new system category browser with initial textual 
	contents set to aString."

	| newBrowser |
	self hasSystemCategorySelected
		ifTrue: 
			[newBrowser := self class new.
			newBrowser selectSystemCategory: self selectedSystemCategory.
			newBrowser setClass: self selectedClassOrMetaClass selector: self selectedMessageName.
			self class openBrowserView: (newBrowser openSystemCatEditString: aString)
				label: 'Classes in category ', newBrowser selectedSystemCategory]! !

!Browser methodsFor: 'toolbuilder' stamp: 'fbs 12/29/2013 17:35'!
buildSystemCategoryListWith: builder
	| listSpec |
	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #systemCategoryList; 
		getIndex: #systemCategoryListIndex; 
		setIndex: #systemCategoryListIndex:; 
		menu: #systemCategoryMenu:; 
		keyPress: #systemCatListKey:from:.
	SystemBrowser browseWithDragNDrop ifTrue:[
		listSpec
			dropAccept: #wantsSystemCategoriesDrop:;
			dropItem: #dropOnSystemCategories:at:].
	^listSpec! !

!Browser methodsFor: 'toolbuilder' stamp: 'mt 4/24/2015 11:04'!
buildSystemCatListSingletonWith: builder
	| listSpec |
	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #systemCategorySingleton; 
		getIndex: #indexIsOne; 
		setIndex: #indexIsOne:; 
		menu: #systemCategoryMenu:; 
		keyPress: #systemCatSingletonKey:from:;
		hScrollBarPolicy: #never;
		vScrollBarPolicy: #never;
		minimumHeight: 0.
	^listSpec! !

!Browser methodsFor: 'toolbuilder' stamp: 'mt 4/30/2015 11:33'!
buildWith: builder
	"Create the ui for the browser"
	| windowSpec max |
	max := self wantsOptionalButtons ifTrue:[0.42] ifFalse:[0.5].
	windowSpec := self buildWindowWith: builder specs: {
		(0@0 corner: 0.25@max) -> [self buildSystemCategoryListWith: builder].
		(self classListFrame: max fromLeft: 0.25 width: 0.25) -> [self buildClassListWith: builder].
		(self switchesFrame: max fromLeft: 0.25 width: 0.25) -> [self buildSwitchesWith: builder].
		(0.5@0 corner: 0.75@max) -> [self buildMessageCategoryListWith: builder].
		(0.75@0 corner: 1@max) -> [self buildMessageListWith: builder].
		(0@max corner: 1@1) -> [self buildCodePaneWith: builder].
	}.
	^builder build: windowSpec! !

!Browser methodsFor: 'message category functions' stamp: 'sw 10/8/2001 14:10'!
canShowMultipleMessageCategories
	"Answer whether the receiver is capable of showing multiple message categories"

	^ true! !

!Browser methodsFor: 'message category list' stamp: 'nk 11/30/2002 08:20'!
categorizeAllUncategorizedMethods
	"Categorize methods by looking in parent classes for a method category."

	| organizer organizers |
	organizer := self classOrMetaClassOrganizer.
	organizers := self selectedClassOrMetaClass withAllSuperclasses collect: [:ea | ea organization].
	(organizer listAtCategoryNamed: ClassOrganizer default) do: [:sel | | found |
		found := (organizers collect: [ :org | org categoryOfElement: sel])
			detect: [:ea | ea ~= ClassOrganizer default and: [ ea ~= nil]]
			ifNone: [].
		found ifNotNil: [organizer classify: sel under: found]].

	self changed: #messageCategoryList! !

!Browser methodsFor: 'message category functions' stamp: 'sw 2/22/2001 06:54'!
categoryOfCurrentMethod
	"Determine the method category associated with the receiver at the current moment, or nil if none"

	| aCategory |
	^ super categoryOfCurrentMethod ifNil:
		[(aCategory := self messageCategoryListSelection) == ClassOrganizer allCategory
					ifTrue:
						[nil]
					ifFalse:
						[aCategory]]! !

!Browser methodsFor: 'message category functions' stamp: 'fbs 3/14/2011 17:43'!
changeMessageCategories: aString 
	"The characters in aString represent an edited version of the the message 
	categories for the selected class. Update this information in the system 
	and inform any dependents that the categories have been changed. This 
	message is invoked because the user had issued the categories command 
	and edited the message categories. Then the user issued the accept 
	command."

	self classOrMetaClassOrganizer changeFromString: aString.
	self clearUserEditFlag.
	self editClass.
	self selectClassNamed: selectedClassName.
	^ true! !

!Browser methodsFor: 'system category functions' stamp: 'di 4/12/98 13:21'!
changeSystemCategories: aString 
	"Update the class categories by parsing the argument aString."

	systemOrganizer changeFromString: aString.
	self changed: #systemCategoryList.
	^ true! !

!Browser methodsFor: 'metaclass' stamp: 'di 1/14/98 12:25'!
classCommentIndicated
	"Answer true iff we're viewing the class comment."

	^ editSelection == #editComment 
! !

!Browser methodsFor: 'class functions' stamp: 'bvs 7/20/2004 15:40'!
classCommentText
	"return the text to display for the comment of the currently selected class"
	| theClass |
	theClass := self selectedClassOrMetaClass.
	theClass ifNil: [ ^''].

	^ theClass hasComment
		ifTrue: [  theClass comment  ]
		ifFalse: [ self noCommentNagString ]! !

!Browser methodsFor: 'metaclass' stamp: 'mt 4/14/2015 13:15'!
classDefinitionIndicated

	^ editSelection == #editClass 
! !

!Browser methodsFor: 'class functions' stamp: 'eem 5/7/2008 12:04'!
classDefinitionText
	"return the text to display for the definition of the currently selected class"
	| theClass |
	^(theClass := self selectedClassOrMetaClass) ifNil: [''] ifNotNil: [theClass definition]! !

!Browser methodsFor: 'multi-window support' stamp: 'mt 5/1/2015 14:07'!
classHierarchy
	| behavior newBrowser |
	(behavior := self selectedClassOrMetaClass) isNil ifTrue:
		[^self].

	(self isPackage "PackageBrowser pains can't support a hierarchy browser; not sure why."
	 or: [self multiWindowState isNil]) ifTrue:
		[^super classHierarchy].

	(newBrowser := HierarchyBrowser new initHierarchyForClass: behavior)
		selectMessageCategoryNamed: self selectedMessageCategoryName;
		selectMessageNamed: self selectedMessageName;
		editSelection: editSelection.

	self multiWindowState addWindow: newBrowser
! !

!Browser methodsFor: 'class list' stamp: 'mt 3/5/2015 11:33'!
classIconAt: anIndex

	self class showClassIcons
		ifFalse: [^ nil].

	^ ToolIcons iconNamed: (ToolIcons iconForClass: (self classList at: anIndex) withBlanksTrimmed asSymbol)! !

!Browser methodsFor: 'class list' stamp: 'rkrk 8/24/2009 03:02'!
classList

	^ self class listClassesHierarchically
		ifTrue: [self hierarchicalClassList]
		ifFalse: [self defaultClassList].! !

!Browser methodsFor: 'initialize-release' stamp: 'fbs 4/5/2010 11:38'!
classListFrame: bottomFraction
	^self
		classListFrame: bottomFraction
		fromTop: 0
		fromLeft: 0.25
		width: 0.25.! !

!Browser methodsFor: 'initialize-release' stamp: 'fbs 4/5/2010 11:38'!
classListFrame: bottomFraction fromLeft: leftFraction width: rightFraction
	^self
		classListFrame: bottomFraction
		fromTop: 0
		fromLeft: leftFraction
		width: rightFraction.! !

!Browser methodsFor: 'initialize-release' stamp: 'mt 4/30/2015 11:29'!
classListFrame: bottomFraction fromTop: topFraction fromLeft: leftFraction width: rightFraction
	^LayoutFrame new
		leftFraction: leftFraction offset: 0;
		topFraction: topFraction offset: 0;
		rightFraction: (leftFraction + rightFraction) offset: 0;
		bottomFraction: bottomFraction offset: self buttonHeight negated! !

!Browser methodsFor: 'class list' stamp: 'fbs 3/16/2011 10:43'!
classListIndex
	"Answer the index of the current class selection."

	^ self classListIndexOf: self selectedClassName.! !

!Browser methodsFor: 'class list' stamp: 'fbs 3/14/2011 17:27'!
classListIndexOf: className 

	| classList |
	classList := self classList.
	classList := classList collect: [:ea | ea withoutLeadingBlanks asSymbol].
	^ classList indexOf: className.! !

!Browser methodsFor: 'class list' stamp: 'fbs 3/16/2011 10:54'!
classListIndex: anInteger 
	| newClassName |
	newClassName := self classList at: anInteger ifAbsent: [ nil ].
	newClassName := newClassName ifNotNil: [newClassName withoutLeadingBlanks asSymbol].
	self selectClassNamed: newClassName.! !

!Browser methodsFor: 'pluggable menus - hooks' stamp: 'topa 3/27/2015 01:20'!
classListMenuHook: aMenu shifted: aBoolean
	<classListMenu>
	<menuPriority: 400>
	^ self menuHook: aMenu named: #classListMenu shifted: aBoolean.
! !

!Browser methodsFor: 'class functions' stamp: 'topa 3/26/2015 21:39'!
classListMenuMore: aMenu 
	" The 'more..' link that toggles between shifted and unshifted menus
	in class lists "
	<classListMenuShifted: false>
	<menuPriority: 1000>
	^ aMenu addList: #(- ('more...'	offerShiftedClassListMenu)); yourself
! !

!Browser methodsFor: 'class functions' stamp: 'kfr 11/23/2015 23:26'!
classListMenu: aMenu 
	"Conveniently fit for backward compatibility with old browers stored in image segments"
	<classListMenuShifted: false>
	aMenu addList: #(
		-
		('browse full (b)'			browseMethodFull)
		('browse hierarchy (h)'		spawnHierarchy)
		('browse protocol (p)'		browseFullProtocol)
		-
		('printOut'					printOutClass)
		('fileOut'					fileOutClass)
		-
		('show hierarchy'			hierarchy)
		('show definition'			editClass)
		('show comment'			editComment)
		('show all comments'		browseAllCommentsForClass)
		-
		('references... (r)'			browseVariableReferences)
		('assignments... (a)'			browseVariableAssignments)
		('class refs (N)'				browseClassRefs)
		-
		('rename class ...'			renameClass)
		('copy class'				copyClass)
		('remove class (x)'			removeClass)
		-
		('find method...'				findMethod)).
	^ aMenu
! !

!Browser methodsFor: 'class functions' stamp: 'topa 3/13/2015 18:50'!
classListMenu: aMenu shifted: shifted
	"Set up the menu to apply to the receiver's class list, honoring the #shifted boolean"
	^ self menu: aMenu for: #(classListMenu classListMenuShifted:) shifted: shifted.
! !

!Browser methodsFor: 'class list' stamp: 'tk 4/5/98 12:25'!
classListSingleton

	| name |
	name := self selectedClassName.
	^ name ifNil: [Array new]
		ifNotNil: [Array with: name]! !

!Browser methodsFor: 'metaclass' stamp: 'ak 11/24/2000 21:46'!
classMessagesIndicated
	"Answer whether the messages to be presented should come from the 
	metaclass."

	^ self metaClassIndicated and: [self classCommentIndicated not]! !

!Browser methodsFor: 'system category functions' stamp: 'tk 4/2/98 13:43'!
classNotFound

	self changed: #flash.! !

!Browser methodsFor: 'metaclass' stamp: ''!
classOrMetaClassOrganizer
	"Answer the class organizer for the metaclass or class, depending on 
	which (instance or class) is indicated."

	self metaClassIndicated
		ifTrue: [^metaClassOrganizer]
		ifFalse: [^classOrganizer]! !

!Browser methodsFor: 'code pane' stamp: 'cmm 9/24/2015 15:04'!
compileMessage: aText notifying: aController
	"Compile the code that was accepted by the user, placing the compiled method into an appropriate message category.  Return true if the compilation succeeded, else false."
	| fallBackCategoryName originalSelectorName result fallBackMethodName |
	self selectedMessageCategoryName = ClassOrganizer allCategory
		ifTrue:
			[ "User tried to save a method while the ALL category was selected"
			fallBackCategoryName := selectedMessageCategoryName.
			fallBackMethodName := selectedMessageName.
			editSelection == #newMessage
				ifTrue:
					[ "Select the 'as yet unclassified' category"
					selectedMessageCategoryName := nil.
					(result := self defineMessageFrom: aText notifying: aController)
						ifNil:
							["Compilation failure:  reselect the original category & method"
							selectedMessageCategoryName := fallBackCategoryName.
							selectedMessageName := fallBackMethodName]
						ifNotNil:
							[self setSelector: result]]
				ifFalse:
					[originalSelectorName := self selectedMessageName.
					self setOriginalCategoryIndexForCurrentMethod.
					selectedMessageName := fallBackMethodName := originalSelectorName.			
					(result := self defineMessageFrom: aText notifying: aController)
						ifNotNil:
							[self setSelector: result]
						ifNil:
							[ "Compilation failure:  reselect the original category & method"
							selectedMessageCategoryName := fallBackCategoryName.
							selectedMessageName := fallBackMethodName.
							^ result notNil]].
			self changed: #messageCategoryList.
			^ result notNil]
		ifFalse:
			[ "User tried to save a method while the ALL category was NOT selected"
			^ (self defineMessageFrom: aText notifying: aController) notNil]! !

!Browser methodsFor: 'accessing' stamp: 'fbs 1/3/2013 23:10'!
contents
	"Depending on the current selection, different information is retrieved.
	Answer a string description of that information. This information is the
	method of the currently selected class and message."

	| comment theClass latestCompiledMethod |
	latestCompiledMethod := currentCompiledMethod.
	currentCompiledMethod := nil.

	editSelection == #newTrait
		ifTrue: [^ClassDescription newTraitTemplateIn: self selectedSystemCategory].
	editSelection == #none ifTrue: [^ ''].
	editSelection == #editSystemCategories 
		ifTrue: [^ systemOrganizer printString].
	editSelection == #newClass 
		ifTrue: [^ self newClassContents].
	editSelection == #editClass 
		ifTrue: [^self classDefinitionText].
	editSelection == #editComment 
		ifTrue:
			[(theClass := self selectedClass) ifNil: [^ ''].
			comment := theClass comment.
			currentCompiledMethod := classOrganizer commentRemoteStr.
			^ comment size = 0
				ifTrue: ['This class has not yet been commented.']
				ifFalse: [comment]].
	editSelection == #hierarchy 
		ifTrue: [^self selectedClassOrMetaClass printHierarchy].
	editSelection == #editMessageCategories 
		ifTrue: [^ self classOrMetaClassOrganizer printString].
	editSelection == #newMessage
		ifTrue:
			[^ (theClass := self selectedClassOrMetaClass) 
				ifNil: ['']
				ifNotNil: [theClass sourceCodeTemplate]].
	editSelection == #editMessage
		ifTrue:
			[^ self editContentsWithDefault:
				[currentCompiledMethod := latestCompiledMethod.
				self selectedMessage]].

	self error: 'Browser internal error: unknown edit selection.'! !

!Browser methodsFor: 'accessing' stamp: 'fbs 3/17/2011 07:56'!
contentsSelection
	"Return the interval of text in the code pane to select when I set the pane's contents"

	self hasMessageCategorySelected & (self hasMessageSelected not)
		ifTrue: [^ 1 to: 500]	"entire empty method template"
		ifFalse: [^ 1 to: 0]  "null selection"! !

!Browser methodsFor: 'accessing' stamp: 'al 4/24/2004 12:01'!
contents: input notifying: aController 
	"The retrieved information has changed and its source must now be
	 updated. The information can be a variety of things, depending on
	 the list selections (such as templates for class or message definition,
	 methods) or the user menu commands (such as definition, comment,
	 hierarchy).  Answer the result of updating the source."

	| aString aText theClass |
	self changed: #annotation.
	aString := input asString.
	aText := input asText.
	editSelection == #newTrait ifTrue: [^self defineTrait: input asString notifying: aController].
	editSelection == #editSystemCategories ifTrue: [^ self changeSystemCategories: aString].
	editSelection == #editClass | (editSelection == #newClass) ifTrue: [^ self defineClass: aString notifying: aController].
	editSelection == #editComment
		ifTrue: 
			[theClass := self selectedClass.
			theClass
				ifNil: 
					[self inform: 'You must select a class
before giving it a comment.'.
					^ false].
			theClass comment: aText stamp: Utilities changeStamp.
			self changed: #classCommentText.
			^ true].
	editSelection == #hierarchy ifTrue: [^ true].
	editSelection == #editMessageCategories ifTrue: [^ self changeMessageCategories: aString].
	editSelection == #editMessage | (editSelection == #newMessage)
		ifTrue:
			[^ self okayToAccept
				ifFalse:
					[false]
				ifTrue:
					[self compileMessage: aText notifying: aController]].
	editSelection == #none
		ifTrue: 
			[self inform: 'This text cannot be accepted
in this part of the browser.'.
			^ false].
	self error: 'unacceptable accept'! !

!Browser methodsFor: 'class functions' stamp: 'fbs 3/4/2011 20:56'!
copyClass
	| originalClass originalName copysName |
	self hasClassSelected ifFalse: [^ self].
	self okToChange ifFalse: [^ self].
	originalClass := self selectedClass.
	originalName := originalClass name.
	copysName := self request: 'Please type new class name' initialAnswer: originalName.
	copysName = '' ifTrue: [^ self].  " Cancel returns '' "
	copysName := copysName asSymbol.
	copysName = originalName ifTrue: [^ self].
	(Smalltalk hasClassNamed: copysName)
		ifTrue: [^ self error: copysName , ' already exists'].
	Cursor wait showWhile: [
		| newDefinition newMetaDefinition newClass |
		newDefinition := originalClass definition
			copyReplaceAll: originalName printString
			with: copysName printString.
		newClass := Compiler evaluate: newDefinition logged: true.
		newMetaDefinition := originalClass class definition
			copyReplaceAll: originalClass class name
			with: newClass class name.
		Compiler evaluate: newMetaDefinition logged: true.
		newClass copyAllCategoriesFrom: originalClass.
		newClass class copyAllCategoriesFrom: originalClass class.
		originalClass hasComment ifTrue: [
			newClass comment: originalClass comment ] ].
	self classListIndex: 0.
	self changed: #classList! !

!Browser methodsFor: 'accessing' stamp: 'nice 5/8/2011 10:11'!
couldBrowseAnyClass
	"Answer whether the receiver is equipped to browse any class. This is in support of the system-brower feature that allows the browser to be redirected at the selected class name.  This implementation is clearly ugly, but the feature it enables is handsome enough.  3/1/96 sw"

	^self dependents
		anySatisfy: [:d | (d respondsTo: #getListSelector)
				and: [d getListSelector == #systemCategoryList]]! !

!Browser methodsFor: 'class list' stamp: 'nice 12/26/2009 00:28'!
createHierarchyTreeOf: col

	"Create a tree from a flat collection of classes"
	| transformed |
	transformed := col collect: [:ea | 
		| childs indexes |
		childs := col select: [:class | class isTrait not and: [class superclass = ea]].
		indexes := childs collect: [:child | col indexOf: child].
		ea -> indexes].
	transformed copy do: [:ea |
		ea value: (ea value collect: [:idx | 
			| val |
			val := transformed at: idx.
			transformed at: idx put: nil.
			val])].
	^ transformed select: [:ea | ea notNil].
! !

!Browser methodsFor: 'class functions' stamp: 'cmm 5/2/2012 18:05'!
createInstVarAccessors
	"Create getters and setters for all inst vars defined at the level of the current class selection,
	 except do NOT clobber or override any selectors already understood by the instances of the selected class"

	self selectedClassOrMetaClass ifNotNil:
		[:aClass| | cr |
		cr := String with: Character cr.
		aClass instVarNames do: 
			[:aName | | newMessage setter |
			(aClass canUnderstand: aName asSymbol) ifFalse:
				[newMessage :=
					aName, cr, cr,
					'	^ ', aName.
				aClass compile: newMessage classified: #accessing notifying: nil].
			(aClass canUnderstand: (setter := aName, ':') asSymbol) ifFalse:
				[newMessage :=
					setter, ' anObject', cr, cr,
						(aClass settersReturnValue ifTrue: ['	^'] ifFalse: ['	']),
						aName, ' := anObject'.
				aClass compile: newMessage classified: #accessing notifying: nil]]]! !

!Browser methodsFor: 'controls' stamp: 'mt 7/23/2016 11:07'!
decorateButtons

	super decorateButtons.
	
	self changed: #classCommentIndicated.
	self changed: #instanceMessagesIndicated.
	self changed: #classMessagesIndicated.! !

!Browser methodsFor: 'initialize-release' stamp: 'sw 1/13/2000 16:45'!
defaultBrowserTitle
	^ 'System Browser'! !

!Browser methodsFor: 'class list' stamp: 'mt 4/12/2015 15:15'!
defaultClassList
	"Answer an array of the class names of the selected category. Answer an 
	empty array if no selection exists."
		
	^ self hasSystemCategorySelected
		ifFalse: [Array new]
		ifTrue: [self selectedSystemCategory = SystemOrganizer allCategory
			ifTrue: [systemOrganizer allElements sort]
			ifFalse: [systemOrganizer listAtCategoryNamed: self selectedSystemCategory]]
! !

!Browser methodsFor: 'user interface' stamp: 'mt 7/9/2016 08:52'!
defaultWindowColor
	^ (Color r: 0.764 g: 0.9 b: 0.63)! !

!Browser methodsFor: 'class functions' stamp: 'ul 12/9/2011 02:33'!
defineClass: defString notifying: aController  
	"The receiver's textual content is a request to define a new class. The
	source code is defString. If any errors occur in compilation, notify
	aController."
	| oldClass class newClassName defTokens keywdIx envt |
	oldClass := self selectedClassOrMetaClass.
	defTokens := defString findTokens: Character separators.
	
	((defTokens first = 'Trait' and: [defTokens second = 'named:'])
		or: [defTokens second = 'classTrait'])
		ifTrue: [^self defineTrait: defString notifying: aController].
		
	keywdIx := defTokens findFirst: [:x | x beginsWith: 'category'].
	envt := Smalltalk.
	keywdIx := defTokens findFirst: [:x | '*subclass*' match: x].
	newClassName := (defTokens at: keywdIx+1) copyWithoutAll: '#()'.
	((oldClass isNil or: [oldClass theNonMetaClass name asString ~= newClassName])
		and: [envt includesKey: newClassName asSymbol]) ifTrue:
			["Attempting to define new class over existing one when
				not looking at the original one in this browser..."
			(self confirm: ((newClassName , ' is an existing class in this system.
Redefining it might cause serious problems.
Is this really what you want to do?') asText makeBoldFrom: 1 to: newClassName size))
				ifFalse: [^ false]].
	"ar 8/29/1999: Use oldClass superclass for defining oldClass
	since oldClass superclass knows the definerClass of oldClass."
	oldClass ifNotNil:[oldClass := oldClass superclass].
	class := oldClass subclassDefinerClass
				evaluate: defString
				notifying: aController
				logged: false.
	(class isKindOf: Behavior)
		ifTrue: [self changed: #systemCategoryList.
				self changed: #classList.
				self clearUserEditFlag.
				self setClass: class selector: nil.
				"self clearUserEditFlag; editClass."
				^ true]
		ifFalse: [^ false]! !

!Browser methodsFor: 'message functions' stamp: 'cmm 11/12/2015 14:00'!
defineMessageFrom: aString notifying: aController
	"Compile the expressions in aString. Notify aController if a syntax error occurs. Install the compiled method in the selected class classified under  the currently selected message category name. Answer the selector obtained if compilation succeeds, nil otherwise."
	| selectedMessageName selector category oldMessageList selectedClassOrMetaClass |
	selectedMessageName := self selectedMessageName.
	oldMessageList := self messageList.
	selectedClassOrMetaClass := self selectedClassOrMetaClass.
	contents := nil.
	selector := (selectedClassOrMetaClass newParser parseSelector: aString).
	(self metaClassIndicated
		and: [(selectedClassOrMetaClass includesSelector: selector) not
		and: [Metaclass isScarySelector: selector]])
		ifTrue: ["A frist-time definition overlaps the protocol of Metaclasses"
				(self confirm: ((selector , ' is used in the existing class system.
Overriding it could cause serious problems.
Is this really what you want to do?') asText makeBoldFrom: 1 to: selector size))
				ifFalse: [^nil]].
	category := selectedMessageName
		ifNil: [ self selectedMessageCategoryName ]
		ifNotNil: [ (selectedClassOrMetaClass >> selectedMessageName) methodReference ifNotNil: [ : ref | ref category ]].
	selector := selectedClassOrMetaClass
				compile: aString
				classified: category
				notifying: aController.
	selector == nil ifTrue: [^ nil].
	contents := aString copy.
	selector ~~ selectedMessageName
		ifTrue: 
			[category = ClassOrganizer nullCategory
				ifTrue: [self changed: #classSelectionChanged.
						self changed: #classList.
						self messageCategoryListIndex: 1].
			self setClassOrganizer.  "In case organization not cached"
			(oldMessageList includes: selector)
				ifFalse: [self changed: #messageList].
			self messageListIndex: (self messageList indexOf: selector)].
	^ selector! !

!Browser methodsFor: 'traits' stamp: 'ar 12/27/2009 21:46'!
defineTrait: defString notifying: aController  

	| defTokens keywdIx envt oldTrait newTraitName trait |
	oldTrait := self selectedClassOrMetaClass.
	defTokens := defString findTokens: Character separators.
	keywdIx := defTokens findFirst: [:x | x = 'category'].
	envt := self selectedEnvironment.
	keywdIx := defTokens findFirst: [:x | x = 'named:'].
	newTraitName := (defTokens at: keywdIx+1) copyWithoutAll: '#()'.
	((oldTrait isNil or: [oldTrait baseTrait name asString ~= newTraitName])
		and: [envt includesKey: newTraitName asSymbol]) ifTrue:
			["Attempting to define new class/trait over existing one when
				not looking at the original one in this browser..."
			(self confirm: ((newTraitName , ' is an existing class/trait in this system.
Redefining it might cause serious problems.
Is this really what you want to do?') asText makeBoldFrom: 1 to: newTraitName size))
				ifFalse: [^ false]].

	trait := Compiler evaluate: defString notifying: aController logged: true.
	^(trait isTrait)
		ifTrue: [
			self changed: #classList.
			self classListIndex: (self classListIndexOf: trait baseTrait name).
			self clearUserEditFlag; editClass.
			true]
		ifFalse: [ false ]
! !

!Browser methodsFor: 'self-updating' stamp: 'mt 4/14/2015 13:47'!
didCodeChangeElsewhere

	^ super didCodeChangeElsewhere or: [self classDefinitionIndicated
		and: [self metaClassIndicated
			ifFalse: [classDefinition ~= self selectedClass definition]
			ifTrue: [metaClassDefinition ~= self selectedClass theMetaClass definition]]]! !

!Browser methodsFor: 'accessing' stamp: 'nice 3/31/2011 21:51'!
doItReceiver
	"This class's classPool has been jimmied to be the classPool of the class 
	being browsed. A doIt in the code pane will let the user see the value of 
	the class variables."

	^ self selectedClass! !

!Browser methodsFor: 'drag and drop' stamp: 'cmm 5/27/2012 21:36'!
dragFromClassList: index 
	"Drag a class from the browser"
	| name envt |
	(name := self classList at: index) ifNil: [ ^ nil ].
	(envt := self selectedEnvironment) ifNil: [ ^ nil ].
	^ envt
		at: name withBlanksTrimmed asSymbol
		ifAbsent: [  ]! !

!Browser methodsFor: 'drag and drop' stamp: 'ar 8/8/2009 10:12'!
dragFromMessageList: index
	"Drag a method from the browser"
	^self selectedClassOrMetaClass compiledMethodAt: (self messageList at: index) ifAbsent:[nil]! !

!Browser methodsFor: 'drag and drop' stamp: 'ul 1/10/2016 22:35'!
dropOnMessageCategories: method at: index

	| sourceClass destinationClass category copy |
	copy := Sensor shiftPressed.
	(method isKindOf: CompiledMethod) 
		ifFalse:[^self inform: 'Can only drop methods'].
	sourceClass := method methodClass.
	destinationClass := self selectedClassOrMetaClass.
	sourceClass == destinationClass ifTrue:[
		category := self messageCategoryList at: index.
		category = ClassOrganizer allCategory ifTrue: [^false].
		destinationClass organization classify: method selector  under: category suppressIfDefault: false logged: true.
		^true ].
	(copy
		or: [ (destinationClass inheritsFrom: sourceClass)
		or: [ (sourceClass inheritsFrom: destinationClass)
		or: [ sourceClass theNonMetaClass == destinationClass theNonMetaClass ] ] ])
		ifFalse: [
			(self confirm: (
				'Classes "{1}" and "{2}" are unrelated.{3}Are you sure you want to move this method?'
					format: { sourceClass. destinationClass. Character cr })) 
						ifFalse: [ ^false ] ].
	destinationClass
		compile: method getSource
		classified: (self messageCategoryList at: index)
		withStamp: method timeStamp
		notifying: nil.
	copy ifFalse: [
		sourceClass removeSelector: method selector ].
	^true! !

!Browser methodsFor: 'drag and drop' stamp: 'ar 8/8/2009 10:16'!
dropOnSystemCategories: aClass at: index
	| category |
	(aClass isBehavior) ifFalse:[^self inform: 'Can only drop classes'].
	category := self systemCategoryList at: index.
	self selectedEnvironment organization classify: aClass instanceSide name  under: category.
	^true! !

!Browser methodsFor: 'class functions' stamp: 'fbs 3/4/2011 20:53'!
editClass
	"Retrieve the description of the class definition."

	self hasClassSelected ifFalse: [^ self].
	self okToChange ifFalse: [^ self].
	self messageCategoryListIndex: 0.
	self editSelection: #editClass.
	self changed: #contents.
	self changed: #classCommentText.
! !

!Browser methodsFor: 'class functions' stamp: 'fbs 3/4/2011 20:53'!
editComment
	"Retrieve the description of the class comment."

	self hasClassSelected ifFalse: [^ self].
	self okToChange ifFalse: [^ self].
	self messageCategoryListIndex: 0.
	metaClassIndicated := false.
	self editSelection: #editComment.
	self changed: #classSelectionChanged.
	self changed: #messageCategoryList.
	self changed: #messageList.
	self decorateButtons.
	self contentsChanged
! !

!Browser methodsFor: 'message category functions' stamp: 'fbs 3/4/2011 20:53'!
editMessageCategories
	"Indicate to the receiver and its dependents that the message categories of 
	the selected class have been changed."

	self okToChange ifFalse: [^ self].
	self hasClassSelected
		ifTrue: 
			[self messageCategoryListIndex: 0.
			self editSelection: #editMessageCategories.
			self changed: #editMessageCategories.
			self contentsChanged]! !

!Browser methodsFor: 'accessing' stamp: ''!
editSelection
	^editSelection! !

!Browser methodsFor: 'accessing' stamp: 'nk 2/15/2004 13:27'!
editSelection: aSelection
	"Set the editSelection as requested."

	editSelection := aSelection.
	self changed: #editSelection.! !

!Browser methodsFor: 'system category functions' stamp: 'fbs 3/5/2011 20:55'!
editSystemCategories
	"Retrieve the description of the class categories of the system organizer."

	self okToChange ifFalse: [^ self].
	self selectSystemCategory: nil.
	self editSelection: #editSystemCategories.
	self changed: #editSystemCategories.
	self contentsChanged! !

!Browser methodsFor: 'class functions' stamp: 'nice 5/8/2011 10:13'!
explainSpecial: string 
	"Answer a string explaining the code pane selection if it is displaying 
	one of the special edit functions."

	| classes whole lits reply |
	(editSelection == #editClass or: [editSelection == #newClass])
		ifTrue: 
			["Selector parts in class definition"
			string last == $: ifFalse: [^nil].
			lits := Array with:
				#subclass:instanceVariableNames:classVariableNames:poolDictionaries:category:.
			(whole := lits
					detect: [:each | each keywords anySatisfy: [:frag | frag = string] ]
					ifNone: []) ~~ nil
				ifTrue: [reply := '"' , string , ' is one part of the message selector ' , whole , '.']
				ifFalse: [^nil].
			classes := self systemNavigation allClassesImplementing: whole.
			classes := 'these classes ' , classes printString.
			^reply , '  It is defined in ' , classes , '."
Smalltalk browseAllImplementorsOf: #' , whole].

	editSelection == #hierarchy
		ifTrue: 
			["Instance variables in subclasses"
			classes := self selectedClassOrMetaClass allSubclasses.
			classes := classes
					detect: [:each | each instVarNames anySatisfy: [:name | name = string] ]
					ifNone: [^nil].
			classes := classes printString.
			^'"is an instance variable in class ' , classes , '."
' , classes , ' browseAllAccessesTo: ''' , string , '''.'].
	editSelection == #editSystemCategories ifTrue: [^nil].
	editSelection == #editMessageCategories ifTrue: [^nil].
	^nil! !

!Browser methodsFor: 'class functions' stamp: 'fbs 3/4/2011 20:56'!
fileOutClass
	"Print a description of the selected class onto a file whose name is the 
	category name followed by .st."

Cursor write showWhile:
		[self hasClassSelected ifTrue: [self selectedClass fileOut]]! !

!Browser methodsFor: 'message category functions' stamp: 'fbs 3/30/2011 07:50'!
fileOutMessageCategories
	"Print a description of the selected message category of the selected class 
	onto an external file."

Cursor write showWhile:
	[self hasMessageCategorySelected
		ifTrue: 
			[self selectedClassOrMetaClass fileOutCategory: self selectedMessageCategoryName]]! !

!Browser methodsFor: 'system category functions' stamp: 'fbs 3/3/2011 17:53'!
fileOutSystemCategory
	"Print a description of each class in the selected category onto a file 
	whose name is the category name followed by .st."

	self hasSystemCategorySelected
		ifTrue: [systemOrganizer fileOutCategory: self selectedSystemCategory]! !

!Browser methodsFor: 'system category functions' stamp: 'mt 5/1/2015 14:08'!
findClass
	"Search for a class by name."

	| foundClass |
	(self multiWindowState notNil
	 or: [self okToChange]) ifFalse:
		[^self classNotFound].
	foundClass := UIManager default chooseClassOrTrait.
	foundClass ifNil: [^self classNotFound].
	(self selectedClass notNil
	 and: [self multiWindowState notNil
	 "Can only support multi-window if original window has all the right panes."
	 and: [self multiWindowState prototype isHierarchy not]]) ifTrue:
		[(self classList includes: foundClass name)
			ifTrue: [self multiWindowState copyWindow]
			ifFalse: [self multiWindowState addNewWindow]].
 	self selectCategoryForClass: foundClass.
	self selectClass: foundClass! !

!Browser methodsFor: 'class functions' stamp: 'bf 3/18/2010 14:03'!
findMethod
	"Pop up a list of the current class's methods, and select the one chosen by the user"
	| aClass selectors reply cat messageCatIndex messageIndex |
	self classListIndex = 0 ifTrue: [^ self].
	self okToChange ifFalse: [^ self].
	aClass := self selectedClassOrMetaClass.
	selectors := aClass selectors asSortedArray.
	selectors isEmpty ifTrue: [self inform: aClass name, ' has no methods.'. ^ self].
	reply := UIManager default 
		chooseFrom: selectors
		values: selectors
		lines: #(1).
	reply == nil ifTrue: [^ self].
	cat := aClass whichCategoryIncludesSelector: reply.
	messageCatIndex := self messageCategoryList indexOf: cat.
	self messageCategoryListIndex: messageCatIndex.
	messageIndex := (self messageList indexOf: reply).
	self messageListIndex: messageIndex! !

!Browser methodsFor: 'class list' stamp: 'mt 11/19/2015 09:12'!
flattenHierarchyTree: classHierarchy on: col indent: indent
	^ self
		flattenHierarchyTree: classHierarchy
		on: col
		indent: indent
		by: '  '.! !

!Browser methodsFor: 'class list' stamp: 'fbs 3/13/2011 21:20'!
flattenHierarchyTree: classHierarchy on: col indent: indent by: indentChars
	"Recursively add to col the names in classHierarchy indenting to show the hierarchical relationship. Use indentChars to do the indenting: spaces, tabs, etc."
	| plusIndent |
	plusIndent := indentChars.
	classHierarchy do: [:assoc |
		| class childs |
		class := assoc key.
		col add: indent , class name.
		childs := assoc value.
		self
			flattenHierarchyTree: childs
			on: col
			indent: indent , plusIndent
			by: indentChars].
	^ col! !

!Browser methodsFor: 'initialize-release' stamp: 'fbs 4/22/2010 14:53'!
frameOffsetFromTop: height fromLeft: leftFraction width: rightFraction bottomFraction: bottomFraction
	^LayoutFrame new
		topFraction: 0 offset: height;
		leftFraction: leftFraction offset: 0;
		rightFraction: (leftFraction + rightFraction) offset: 0;
		bottomFraction: bottomFraction offset: 0;
		yourself.! !

!Browser methodsFor: 'class list' stamp: 'fbs 3/14/2011 17:57'!
hasClassSelected
	^ selectedClassName notNil.! !

!Browser methodsFor: 'message category list' stamp: 'fbs 3/4/2011 17:40'!
hasMessageCategorySelected
	^ self selectedMessageCategoryName notNil.! !

!Browser methodsFor: 'message list' stamp: 'fbs 3/17/2011 07:39'!
hasMessageSelected
	^ self selectedMessageName notNil.! !

!Browser methodsFor: 'system category list' stamp: 'fbs 3/3/2011 17:27'!
hasSystemCategorySelected
	^ self selectedSystemCategory notNil.! !

!Browser methodsFor: 'class list' stamp: 'mt 4/12/2015 15:19'!
hierarchicalClassList

	"classNames are an arbitrary collection of classNames of the system.
	Reorder those class names so that they are sorted and indended by inheritance"
	| classes |
	"Creating the hierarchy is *really slow* for the full class list. Skip it for now."
	self selectedSystemCategory = SystemOrganizer allCategory
		ifTrue: [^ self defaultClassList].
		
	classes := self defaultClassList collect: [:sym | Smalltalk classNamed: sym].
	^ self
		flattenHierarchyTree: (self createHierarchyTreeOf: classes)
		on: OrderedCollection new
		indent: ''.! !

!Browser methodsFor: 'class functions' stamp: 'fbs 3/4/2011 20:52'!
hierarchy
	"Display the inheritance hierarchy of the receiver's selected class."

	self hasClassSelected ifFalse: [^ self].
	self okToChange ifFalse: [^ self].
	self messageCategoryListIndex: 0.
	self editSelection: #hierarchy.
	self changed: #editComment.
	self contentsChanged.
	^ self! !

!Browser methodsFor: 'message category functions' stamp: 'emm 5/30/2002 09:20'!
highlightMessageList: list with: morphList
	"Changed by emm to add emphasis in case of breakpoint"

	morphList do:[:each | 
		| classOrNil methodOrNil |
		classOrNil := self selectedClassOrMetaClass.
		methodOrNil := classOrNil isNil
			ifTrue:[nil]
			ifFalse:[classOrNil methodDictionary at: each contents ifAbsent:[]].
		(methodOrNil notNil and:[methodOrNil hasBreakpoint])
			ifTrue:[each contents: ((each contents ,' [break]') asText allBold)]]! !

!Browser methodsFor: 'system category list' stamp: 'tk 5/4/1998 15:46'!
indexIsOne
	"When used as a singleton list, index is always one"
	^ 1! !

!Browser methodsFor: 'system category list' stamp: 'tk 5/4/1998 15:46'!
indexIsOne: value
	"When used as a singleton list, can't change it"

	^ self! !

!Browser methodsFor: 'metaclass' stamp: 'laza 3/9/2010 15:54'!
indicateClassMessages
	"Indicate that the message selection should come from the metaclass 
	messages."

	self okToChange ifTrue: [
		self metaClassIndicated: true]! !

!Browser methodsFor: 'metaclass' stamp: 'laza 3/9/2010 15:54'!
indicateInstanceMessages
	"Indicate that the message selection should come from the class (instance) 
	messages."

	self okToChange ifTrue: [
		self metaClassIndicated: false]! !

!Browser methodsFor: 'message functions' stamp: 'al 4/24/2004 12:48'!
inspectInstances
	"Inspect all instances of the selected class.  1/26/96 sw"

	| myClass |
	((myClass := self selectedClassOrMetaClass) isNil or: [myClass isTrait])
		ifFalse: [myClass theNonMetaClass inspectAllInstances]
! !

!Browser methodsFor: 'message functions' stamp: 'al 4/24/2004 12:48'!
inspectSubInstances
	"Inspect all instances of the selected class and all its subclasses  1/26/96 sw"

	| aClass |
	((aClass := self selectedClassOrMetaClass) isNil or: [aClass isTrait])
		ifFalse: [
			aClass := aClass theNonMetaClass.
			aClass inspectSubInstances].
! !

!Browser methodsFor: 'metaclass' stamp: 'di 1/14/98 13:20'!
instanceMessagesIndicated
	"Answer whether the messages to be presented should come from the 
	class."

	^metaClassIndicated not and: [self classCommentIndicated not]! !

!Browser methodsFor: 'multi-window support' stamp: 'eem 7/1/2010 10:30'!
isHierarchy
	^false! !

!Browser methodsFor: 'multi-window support' stamp: 'eem 7/1/2010 10:31'!
isPackage
	^false! !

!Browser methodsFor: 'initialize-release' stamp: 'mt 5/1/2015 14:08'!
labelString
	| label |
	label := self selectedClassName
				ifNil: [ self defaultBrowserTitle ]
				ifNotNil: [ self defaultBrowserTitle, ': ', self selectedClassName asString ].
	(self multiWindowState notNil
	 and: [self multiWindowState models size > 1]) ifTrue:
		[label := (self multiWindowState models indexOf: self) printString, '. ', label].
	^label! !

!Browser methodsFor: 'message list' stamp: 'fbs 3/22/2011 21:28'!
lastMessageName
	^ self messageList last.! !

!Browser methodsFor: 'message category functions' stamp: 'ul 4/7/2016 22:55'!
mainMessageCategoryMenu: aMenu
	<messageCategoryMenu>
	^ aMenu addList: #(
			('browse'						buildMessageCategoryBrowser)
			('print out'						printOutMessageCategories)
			('file out'						fileOutMessageCategories)
			-
			('reorganize'					editMessageCategories)
			('alphabetize'					alphabetizeMessageCategories)
			('remove empty categories'	removeEmptyCategories)
			('categorize all uncategorized'	categorizeAllUncategorizedMethods)
			('new category...'				addCategory)
			-
			('rename...'						renameCategory)
			('remove (x)'					removeMessageCategory));
		yourself
! !

!Browser methodsFor: 'message functions' stamp: 'cmm 10/16/2015 14:14'!
mainMessageListMenu: aMenu
	<messageListMenuShifted: false>
	^ aMenu addList: #(
			('what to show...'			offerWhatToShowMenu)
			('toggle break on entry'		toggleBreakOnEntry)
			-
			('browse full (b)' 			browseMethodFull)
			('browse hierarchy (h)'			classHierarchy)
			('browse protocol (p)'			browseFullProtocol)
			-
			('fileOut'					fileOutMessage)
			('printOut'					printOutMessage)
			('copy selector (c)'			copySelector)
			('copy reference (C)'		copyReference)
			-
			('senders of... (n)'			browseSendersOfMessages)
			('implementors of... (m)'		browseMessages)
			('inheritance (i)'			methodHierarchy)
			('versions (v)'				browseVersions)
			-
			('references... (r)'			browseVariableReferences)
			('assignments... (a)'			browseVariableAssignments)
			('class refs (N)'			browseClassRefs)
			-
			('remove method (x)'			removeMessage)
			('explore method'			exploreMethod)
			('inspect method'			inspectMethod));
		yourself
! !

!Browser methodsFor: 'system category functions' stamp: 'ul 4/7/2016 22:52'!
mainSystemCategoryMenu: aMenu
	<systemCategoryMenu>
	^ aMenu addList: #(
			('find class... (f)'			findClass)
			('back... (b)'				recent)
			-
			('browse all'			browseAllClasses)
			('browse'				buildSystemCategoryBrowser)
			-
			('printOut'				printOutSystemCategory)
			('fileOut'				fileOutSystemCategory)
			-
			('reorganize'			editSystemCategories)
			('alphabetize'			alphabetizeSystemCategories)
			-
			('update'				updateSystemCategories)
			('add item...'				addSystemCategory)
			('rename...'				renameSystemCategory)
			('remove (x)'			removeSystemCategory));
		yourself
! !

!Browser methodsFor: 'class functions' stamp: 'nk 2/14/2004 15:07'!
makeNewSubclass

	self selectedClassOrMetaClass ifNil: [^ self].
	self okToChange ifFalse: [^ self].
	self editSelection: #newClass.
	self contentsChanged! !

!Browser methodsFor: 'message category list' stamp: 'mt 3/1/2015 10:37'!
messageCategoryList
	"Answer the selected category of messages."

	^ self hasClassSelected
		ifFalse: [Array new]
		ifTrue: [ {ClassOrganizer allCategory},
			(self class sortMessageCategoriesAlphabetically
				ifTrue: [self rawMessageCategoryList sorted]
				ifFalse: [self rawMessageCategoryList]) ]! !

!Browser methodsFor: 'message category list' stamp: 'fbs 3/30/2011 21:37'!
messageCategoryListIndex
	"Answer the index of the selected message category."

	^self messageCategoryList indexOf: selectedMessageCategoryName ifAbsent: [0].! !

!Browser methodsFor: 'message category list' stamp: 'fbs 3/30/2011 21:43'!
messageCategoryListIndex: anInteger
	"Set the selected message category to be the one indexed by anInteger."

	selectedMessageCategoryName := nil.
	self selectMessageCategoryNamed: (self messageCategoryList at: anInteger ifAbsent: [nil]).! !

!Browser methodsFor: 'message category list' stamp: 'ul 4/7/2016 22:58'!
messageCategoryListKey: aCharacter from: view

	aCharacter == $x ifTrue: [ ^self removeMessageCategory ].
	^self arrowKey: aCharacter from: view
	! !

!Browser methodsFor: 'message category list' stamp: 'ccn 3/24/1999 11:02'!
messageCategoryListSelection
	"Return the selected category name or nil."

	^ ((self messageCategoryList size = 0 
		or: [self messageCategoryListIndex = 0]) 
		or: [self messageCategoryList size < self messageCategoryListIndex])
			ifTrue: [nil]
			ifFalse: [self messageCategoryList at: (self messageCategoryListIndex max: 1)]! !

!Browser methodsFor: 'pluggable menus - hooks' stamp: 'topa 3/27/2015 01:20'!
messageCategoryMenuHook: aMenu shifted: aBoolean
	<messageCategoryMenu>
	<menuPriority: 400>
	^ self menuHook: aMenu named: #messageCategoryMenu shifted: aBoolean.
! !

!Browser methodsFor: 'message category functions' stamp: 'topa 3/13/2015 18:51'!
messageCategoryMenu: aMenu
	^ self menu: aMenu for: #(messageCategoryMenu messageCategoryMenuShifted:)
! !

!Browser methodsFor: 'message category list' stamp: 'tk 4/5/98 12:25'!
messageCatListSingleton

	| name |
	name := self selectedMessageCategoryName.
	^ name ifNil: [Array new]
		ifNotNil: [Array with: name]! !

!Browser methodsFor: 'message list' stamp: 'mt 6/11/2016 09:41'!
messageHelpAt: anIndex
	"Show the first n lines of the sources code of the selected message."
	Preferences balloonHelpInMessageLists ifFalse: [^ nil].
	self messageList size < anIndex ifTrue: [^ nil].
	^ self messageHelpFor: (self messageList at: anIndex)
! !

!Browser methodsFor: 'message list' stamp: 'mt 8/12/2016 10:01'!
messageHelpFor: aSelector 
	"Show the first n lines of the sources code of the message behind aSelector."
	
	| source formatted iconHelp |
	Preferences balloonHelpInMessageLists ifFalse: [^ nil].

	source := self selectedClassOrMetaClass sourceCodeAt: aSelector ifAbsent: [^ nil].
	source lineCount > 5 ifTrue: [
		| sourceLines |
		sourceLines := (source asString lines copyFrom: 1 to: 5) asOrderedCollection.
		sourceLines add: ' [...]'.
		source := sourceLines joinSeparatedBy: Character cr].

	
	formatted := (Smalltalk classNamed: #SHTextStylerST80)
		ifNil: [ source asText ]
		ifNotNil: [ :SHTextStylerST80 |
			SHTextStylerST80 new
				classOrMetaClass: self selectedClassOrMetaClass;
				styledTextFor: source asText ].
	iconHelp := (self messageIconHelpFor: aSelector) ifNotEmpty: [:t | 
		t , Character cr, Character cr].
	^ iconHelp asText
		append: formatted;
		yourself! !

!Browser methodsFor: 'message list' stamp: 'mt 6/11/2016 09:41'!
messageIconAt: anIndex

	self class showMessageIcons ifFalse: [^ nil].
	^ self messageIconFor: (self messageList at: anIndex ifAbsent: [^nil])! !

!Browser methodsFor: 'message list' stamp: 'topa 3/1/2016 16:39'!
messageIconFor: aSelector 

	self class showMessageIcons ifFalse: [^ nil].

	^ ToolIcons iconNamed: (ToolIcons
		iconForClass: self selectedClassOrMetaClass
		selector: aSelector)! !

!Browser methodsFor: 'message list' stamp: 'topa 3/1/2016 16:32'!
messageIconHelpFor: aSelector

	self class showMessageIcons ifFalse: [^ String empty].

	^ ToolIconHelp iconHelpNamed: (ToolIcons
		iconForClass: self selectedClassOrMetaClass
		selector: aSelector)! !

!Browser methodsFor: 'message list' stamp: 'fbs 3/30/2011 21:44'!
messageList
	"Answer an Array of the message selectors of the currently selected message category, provided that the messageCategoryListIndex is in proper range.  Otherwise, answer an empty Array  If messageCategoryListIndex is found to be larger than the number of categories (it happens!!), it is reset to zero."
	| sel |
	(sel := self messageCategoryListSelection) ifNil: 
		[
			^ self classOrMetaClassOrganizer
				ifNil:		[Array new]
				ifNotNil:	[self classOrMetaClassOrganizer allMethodSelectors]
			"^ Array new"
		].

	^ sel = ClassOrganizer allCategory
		ifTrue: 
			[self classOrMetaClassOrganizer
				ifNil:		[Array new]
				ifNotNil:	[self classOrMetaClassOrganizer allMethodSelectors]]
		ifFalse:
			[(self classOrMetaClassOrganizer listAtCategoryNamed: self selectedMessageCategoryName )
				ifNil: [selectedMessageCategoryName := nil. Array new]]! !

!Browser methodsFor: 'message list' stamp: 'fbs 3/29/2011 17:03'!
messageListIndex
	"Answer the index of the selected message selector into the currently 
	selected message category."

	^ self messageListIndexOf: self selectedMessageName! !

!Browser methodsFor: 'message list' stamp: 'fbs 3/22/2011 22:17'!
messageListIndexOf: aString
	^ self messageList indexOf: aString.! !

!Browser methodsFor: 'message list' stamp: 'fbs 3/28/2011 07:47'!
messageListIndex: anInteger
	"Set the selected message selector to be the one indexed by anInteger."

	self selectMessageNamed: (self messageList at: anInteger ifAbsent: [nil])! !

!Browser methodsFor: 'pluggable menus - hooks' stamp: 'topa 3/27/2015 01:20'!
messageListMenuHook: aMenu shifted: aBoolean
	<messageListMenu>
	<menuPriority: 400>
	^ self menuHook: aMenu named: #messageListMenu shifted: aBoolean.
! !

!Browser methodsFor: 'message functions' stamp: 'topa 3/13/2015 18:50'!
messageListMenu: aMenu shifted: shifted 
	"Answer the message-list menu"
	^ self menu: aMenu for: #(messageListMenu messageListMenuShifted:) shifted: shifted
! !

!Browser methodsFor: 'metaclass' stamp: 'sr 6/21/2000 17:23'!
metaClassIndicated
	"Answer the boolean flag that indicates which of the method dictionaries, 
	class or metaclass."

	^ metaClassIndicated! !

!Browser methodsFor: 'metaclass' stamp: 'fbs 3/30/2011 21:48'!
metaClassIndicated: trueOrFalse 
	"Indicate whether browsing instance or class messages."

	metaClassIndicated := trueOrFalse.
	self setClassOrganizer.
	self hasSystemCategorySelected ifTrue:
		[self editSelection: (self hasClassSelected
			ifFalse: [metaClassIndicated
				ifTrue: [#none]
				ifFalse: [#newClass]]
			ifTrue: [#editClass])].
	selectedMessageCategoryName := nil.
	selectedMessageName := nil.
	contents := nil.
	self changed: #classSelectionChanged.
	self changed: #messageCategoryList.
	self changed: #messageList.
	self changed: #contents.
	self changed: #annotation.
	self decorateButtons
! !

!Browser methodsFor: 'initialize-release' stamp: 'sw 9/22/1999 17:13'!
methodCategoryChanged
	self changed: #messageCategoryList.
	self changed: #messageList.
	self changed: #annotation.
	self messageListIndex: 0! !

!Browser methodsFor: 'multi-window support' stamp: 'eem 7/1/2010 10:30'!
multiWindowName
	"Answer the string to display for the receiver in a multi-window."
	^String streamContents:
		[:s| | str |
		self selectedClass
			ifNil: [s nextPut: $a; space; nextPutAll: self defaultBrowserTitle]
			ifNotNil:
				[s print: self selectedClass.
				 self metaClassIndicated ifTrue:
					[s nextPutAll: ' class'].
				  self isHierarchy ifTrue:
					[s space; nextPutAll: ' Hierarchy']].
		(str := self selectedMessageName) notNil
			ifTrue: [s nextPutAll: '>>'; nextPutAll: str]
			ifFalse:
				[(str := self selectedMessageCategoryName) notNil
					ifTrue: [s space; nextPut: ${; nextPutAll: str; nextPut: $}]]]! !

!Browser methodsFor: 'multi-window support' stamp: 'eem 8/11/2010 13:23'!
multiWindowNameForState: savedStateMessage
	"Answer the string to display for the receiver in a multi-window."
	| getarg |
	getarg := [:keyword| savedStateMessage arguments at: (savedStateMessage selector keywords indexOf: keyword)].
	^String streamContents:
		[:s|
		(getarg value: 'className:')
			ifNil: [(getarg value: 'restoreToCategory:')
					ifNil: [s nextPut: $a; space; nextPutAll: self defaultBrowserTitle]
					ifNotNil: [:categoryName| s nextPutAll: categoryName]]
			ifNotNil:
				[:className|
				s nextPutAll: className.
				(getarg value: 'meta:') ifTrue:
					[s nextPutAll: ' class'].
				  self isHierarchy ifTrue:
					[s space; nextPutAll: ' Hierarchy'].
				(getarg value: 'selector:')
					ifNil: [(getarg value: 'protocol:') ifNotNil:
							[:protocol| s space; nextPut: ${; nextPutAll: protocol; nextPut: $}]]
					ifNotNil: [:selector| s nextPutAll: '>>'; nextPutAll: selector]]]! !

!Browser methodsFor: 'traits' stamp: 'al 4/24/2004 11:48'!
newClass
	(self selectedClassOrMetaClass notNil and: 
		[self selectedClassOrMetaClass isTrait]) ifTrue: [self classListIndex: 0].
	self editClass.
	editSelection := #newClass.
	self contentsChanged! !

!Browser methodsFor: 'accessing' stamp: 'fbs 1/3/2013 21:19'!
newClassContents
	| theClassName |
	^ (theClassName := self selectedClassName)
		ifNil:
			[Class template: self selectedSystemCategory]
		ifNotNil:
			[Class templateForSubclassOf: theClassName asString category: self selectedSystemCategory]! !

!Browser methodsFor: 'traits' stamp: 'al 4/24/2004 11:48'!
newTrait
	self classListIndex: 0.
	self editClass.
	editSelection := #newTrait.
	self contentsChanged! !

!Browser methodsFor: 'class comment pane' stamp: 'md 2/24/2006 15:23'!
noCommentNagString

	^ Text string: 'THIS CLASS HAS NO COMMENT!!' translated attribute: TextColor red.
		! !

!Browser methodsFor: 'accessing' stamp: 'fbs 3/4/2011 20:57'!
noteSelectionIndex: anInteger for: aSymbol
	aSymbol == #systemCategoryList
		ifTrue:
			[self systemCategoryListIndex: anInteger].
	aSymbol == #classList
		ifTrue:
			[self classListIndex: anInteger].
	aSymbol == #messageCategoryList
		ifTrue:
			[self messageCategoryListIndex: anInteger].
	aSymbol == #messageList
		ifTrue:
			[self messageListIndex: anInteger].! !

!Browser methodsFor: 'multi-window support' stamp: 'mt 5/1/2015 14:08'!
okToClose
	^super okToClose
	  and: [self multiWindowState isNil or: [self multiWindowState okToClose]]! !

!Browser methodsFor: 'initialize-release' stamp: 'bp 4/2/2010 23:26'!
openEditString: aString
        "Create a pluggable version of all the views for a Browser, including views and controllers."
	"Example: 
		Browser fullOnClass: Browser.
	"
	| builder max |
	builder := ToolBuilder default.
	max := self wantsOptionalButtons ifTrue:[0.42] ifFalse:[0.5].
	^self buildWindowWith: builder specs: {
		(0@0 corner: 0.25@max) -> [self buildSystemCategoryListWith: builder].
		(self classListFrame: max) -> [self buildClassListWith: builder].
		(self switchesFrame: max) -> [self buildSwitchesWith: builder].
		(0.5@0 corner: 0.75@max) -> [self buildMessageCategoryListWith: builder].
		(0.75@0 corner: 1@max) -> [self buildMessageListWith: builder].
		"(0@max corner: 1@0.5) -> [self buildOptionalButtonsWith: builder]."
		(0@max corner: 1@1) -> [self buildCodePaneWith: builder].
	}! !

!Browser methodsFor: 'initialize-release' stamp: 'ar 1/19/2010 20:39'!
openMessageCatEditString: aString
        "Create a pluggable version of the views for a Browser that just shows one message category."
	"Example: 
		Preferences browseThemes.
	"
 	| builder max |
	aString ifNotNil:[
		"Note: The views aren't actually built yet after we've called buildWindowWith:.
		Since we can't send changed: #editString before the views have been built
		we just shoot the #changed: message into the #future. This is the easiest
		way to solve this issue locally."
		self future changed: #editString with: aString].
	builder := ToolBuilder default.
	max := self wantsOptionalButtons ifTrue:[0.32] ifFalse:[0.4].
	^self buildWindowWith: builder specs: {
		(0@0 corner: 1.0@0.08) -> [self buildMessageListCatSingletonWith: builder].
		(0.0@0.08 corner: 1.0@max) -> [self buildMessageListWith: builder].
		(0@max corner: 1@1) -> [self buildCodePaneWith: builder].
	}.! !

!Browser methodsFor: 'initialize-release' stamp: 'mt 4/30/2015 11:39'!
openOnClassWithEditString: aString
	"Create a pluggable version of all the views for a Browser, including views and controllers."
	"Example:
		Browser newOnClass: Browser.
	"
	| builder max |
	builder := ToolBuilder default.
	max := self wantsOptionalButtons ifTrue:[0.32] ifFalse:[0.4].
	^self buildWindowWith: builder specs: {
		(self topConstantHeightFrame: self buttonHeight fromLeft: 0 width: 0.5) -> [self buildClassListSingletonWith: builder].
		(self frameOffsetFromTop: self buttonHeight fromLeft: 0 width: 0.5 bottomFraction: max) -> [self buildMessageCategoryListWith: builder].
		(self topConstantHeightFrame: self buttonHeight fromLeft: 0.5 width: 0.5) -> [self buildSwitchesWith: builder].
		(self frameOffsetFromTop: self buttonHeight fromLeft: 0.5 width: 0.5 bottomFraction: max) -> [self buildMessageListWith: builder].
		(0@max corner: 1@1) -> [self buildCodePaneWith: builder].
	}
! !

!Browser methodsFor: 'initialize-release' stamp: 'cmm 4/21/2015 16:45'!
openSystemCatEditString: aString
	"Create a pluggable version of all the views for a Browser, including views and controllers.  The top list view is of the currently selected system class category--a single item list."
	"Example:
		Browser new browseAllClasses.
	"
	| builder catPaneHeight max |
	catPaneHeight := Preferences standardListFont height + 5 "top margin/border" + 5 "bottom margin/border".
	builder := ToolBuilder default.
	max := self wantsOptionalButtons ifTrue:[0.32] ifFalse:[0.4].
	^self buildWindowWith: builder specs: {
		(LayoutFrame fractions: (0@0 corner: 1.0@0) offsets: (0@0 corner: 0@catPaneHeight)) -> [self buildSystemCatListSingletonWith: builder].
		((self classListFrame: max fromTop: 0 fromLeft: 0 width: 0.333)
			topOffset: catPaneHeight) -> [self buildClassListWith: builder].
		(self switchesFrame: max fromLeft: 0 width: 0.333) -> [self buildSwitchesWith: builder].
		(LayoutFrame fractions: (0.333@0 corner: 0.666@max) offsets: (0@catPaneHeight corner: 0@0)) -> [self buildMessageCategoryListWith: builder].
		(LayoutFrame fractions: (0.666@0 corner: 1@max) offsets: (0@catPaneHeight corner: 0@0)) -> [self buildMessageListWith: builder].
		(0@max corner: 1@1) -> [self buildCodePaneWith: builder].
	}! !

!Browser methodsFor: 'class functions' stamp: 'mt 7/23/2016 11:09'!
plusButtonHit
	"Cycle among definition, comment, and hierachy"

	editSelection == #editComment ifTrue: [
		self hierarchy.
		self decorateButtons.
		^ self].
	
	editSelection == #hierarchy ifTrue: [
		self editSelection: #editClass.
		(self hasClassSelected and: [self okToChange]) ifTrue: [
			self changed: #editComment.
			self contentsChanged].
		self decorateButtons.
		^ self].
	
	self editComment.
	self decorateButtons.! !

!Browser methodsFor: 'class functions' stamp: 'fbs 3/4/2011 20:52'!
printOutClass
	"Print a description of the selected class onto a file whose name is the 
	category name followed by .html."

Cursor write showWhile:
		[self hasClassSelected ifTrue: [self selectedClass fileOutAsHtml: true]]! !

!Browser methodsFor: 'message category functions' stamp: 'fbs 3/30/2011 07:55'!
printOutMessageCategories
	"Print a description of the selected message category of the selected class 
	onto an external file in Html format."

Cursor write showWhile:
	[self hasMessageCategorySelected
		ifTrue: 
			[self selectedClassOrMetaClass fileOutCategory: self selectedMessageCategoryName
										asHtml: true]]! !

!Browser methodsFor: 'system category functions' stamp: 'fbs 3/3/2011 17:40'!
printOutSystemCategory
	"Print a description of each class in the selected category as Html."

Cursor write showWhile:
	[self hasSystemCategorySelected
		ifTrue: [systemOrganizer fileOutCategory: self selectedSystemCategory
								asHtml: true ]]
! !

!Browser methodsFor: 'message category list' stamp: 'fbs 3/30/2011 21:58'!
rawMessageCategoryList
	^ self hasClassSelected
		ifTrue: [self classOrMetaClassOrganizer categories]
		ifFalse: [Array new]! !

!Browser methodsFor: 'message category list' stamp: 'bf 4/12/2012 16:15'!
recategorizeMethodSelector: sel 
	"Categorize method named sel by looking in parent classes for a 
	method category. 
	Answer true if recategorized."
	self selectedClassOrMetaClass allSuperclasses
		do: [:ea | 
			| thisCat |
			thisCat := ea organization categoryOfElement: sel.
			(thisCat ~= ClassOrganizer default
					and: [thisCat notNil])
				ifTrue: [self classOrMetaClassOrganizer classify: sel under: thisCat logged: true.
					self changed: #messageCategoryList.
					^ true]].
	^ false! !

!Browser methodsFor: 'class list' stamp: 'TestRunner 2/8/2013 12:57'!
recent
	"Let the user select from a list of recently visited classes.  11/96 stp.
	 12/96 di:  use class name, not classes themselves.
	 : dont fall into debugger in empty case"

	| className class recentList |
	recentList := (RecentClasses reject: [:s | s = self selectedClassName]) select: [:n | Smalltalk hasClassNamed: n].
	recentList size = 0 ifTrue: [^ Beeper beep].
	className := UIManager default chooseFrom: recentList values: recentList.
	className == nil ifTrue: [^ self].
	
	self okToChange ifFalse: [^ self].

	class := Smalltalk at: className.
	self selectCategoryForClass: class.
	self classListIndex: (self classListIndexOf: class name)! !

!Browser methodsFor: 'message list' stamp: 'fbs 4/26/2011 08:41'!
reformulateList
	"If the receiver has a way of reformulating its message list, here is a chance for it to do so"
	super reformulateList.
	(self messageList includes: self selectedMessageName)
		ifFalse: [ self messageList
					ifEmpty: [ self selectMessageNamed: nil ]
					ifNotEmpty: [ self selectMessageNamed: self lastMessageName ]].! !

!Browser methodsFor: 'class functions' stamp: 'sw 3/5/2001 18:04'!
removeClass
	"If the user confirms the wish to delete the class, do so"

	super removeClass ifTrue:
		[self classListIndex: 0]! !

!Browser methodsFor: 'message category functions' stamp: 'nk 4/23/2004 09:18'!
removeEmptyCategories
	self okToChange ifFalse: [^ self].
	self selectedClassOrMetaClass organization removeEmptyCategories.
	self changed: #messageCategoryList
! !

!Browser methodsFor: 'message functions' stamp: 'fbs 3/17/2011 17:26'!
removeMessage
	"If a message is selected, create a Confirmer so the user can verify that  
	the currently selected message should be removed from the system. If 
	so,  
	remove it. If the Preference 'confirmMethodRemoves' is set to false, the 
	confirmer is bypassed."
	| messageName confirmation |
	self hasMessageSelected not
		ifTrue: [^ self].
	self okToChange
		ifFalse: [^ self].
	messageName := self selectedMessageName.
	confirmation := self systemNavigation   confirmRemovalOf: messageName on: self selectedClassOrMetaClass.
	confirmation = 3
		ifTrue: [^ self].
	self selectedClassOrMetaClass removeSelector: messageName.
	self selectMessageNamed: nil.
	self changed: #messageList.
	self setClassOrganizer.
	"In case organization not cached"
	confirmation = 2
		ifTrue: [self systemNavigation browseAllCallsOn: messageName]! !

!Browser methodsFor: 'message category functions' stamp: 'fbs 3/30/2011 08:01'!
removeMessageCategory
	"If a message category is selected, create a Confirmer so the user can 
	verify that the currently selected message category should be removed
 	from the system. If so, remove it."

	| messageCategoryName |
	self hasMessageCategorySelected ifFalse: [^ self].
	self okToChange ifFalse: [^ self].
	messageCategoryName := self selectedMessageCategoryName.
	(self messageList size = 0
		or: [self confirm: 'Are you sure you want to
remove this method category 
and all its methods?'])
		ifTrue: 
			[self selectedClassOrMetaClass removeCategory: messageCategoryName.
			self selectMessageCategoryNamed: nil.
			self changed: #classSelectionChanged].
	self changed: #messageCategoryList.
! !

!Browser methodsFor: 'message functions' stamp: 'tk 4/2/98 17:03'!
removeMessageFromBrowser
	"Our list speaks the truth and can't have arbitrary things removed"

	^ self changed: #flash! !

!Browser methodsFor: 'system category functions' stamp: 'fbs 7/20/2012 07:54'!
removeSystemCategory
	"If a class category is selected, create a Confirmer so the user can 
	verify that the currently selected class category and all of its classes
 	should be removed from the system. If so, remove it."

	self hasSystemCategorySelected ifFalse: [^ self].
	self okToChange ifFalse: [^ self].
	(self classList size = 0
		or: [self confirm: 'Are you sure you want to
remove this system category 
and all its classes?'])
		ifTrue: 
		[| nextSelectedCat |
		nextSelectedCat := self systemCategoryList before: self selectedSystemCategory ifAbsent: [nil].
		systemOrganizer removeSystemCategory: self selectedSystemCategory.
		self selectSystemCategory: nextSelectedCat.
		self changed: #systemCategoryList]! !

!Browser methodsFor: 'message category functions' stamp: 'fbs 6/28/2013 20:54'!
renameCategory
	"Prompt for a new category name and add it before the
	current selection, or at the end if no current selection"
	| oldName newName |
	self hasClassSelected ifFalse: [^ self].
	self okToChange ifFalse: [^ self].
	self hasMessageCategorySelected ifFalse: [^ self].
		
	oldName := self selectedMessageCategoryName.
	newName := self
		request: 'Please type new category name'
		initialAnswer: oldName.
	newName isEmpty
		ifTrue: [^ self]
		ifFalse: [newName := newName asSymbol].
	newName = oldName ifTrue: [^ self].
	self classOrMetaClassOrganizer
		renameCategory: oldName
		toBe: newName.
	self selectClassNamed: selectedClassName.
	self selectMessageCategoryNamed: newName.
	self changed: #messageCategoryList.
! !

!Browser methodsFor: 'class functions' stamp: 'topa 1/21/2015 11:12'!
renameClass
	| oldName newName obs oldBinding |
	self hasClassSelected ifFalse: [^ self].
	self okToChange
		ifFalse: [^ self].
	oldName := self selectedClass name.
	newName := self request: 'Please type new class name' initialAnswer: oldName.
	newName = ''
		ifTrue: [^ self].
	"Cancel returns ''"
	newName := newName asSymbol.
	newName = oldName
		ifTrue: [^ self].
	(self selectedClass environment includesKey: newName)
		ifTrue: [^ self error: newName , ' already exists'].
	oldBinding := self selectedClass environment declarationOf: oldName.
	self selectedClass rename: newName.
	selectedClassName := newName.
	self changed: #classList.
	obs := self systemNavigation allCallsOn: oldBinding.
	obs isEmpty
		ifFalse: [self systemNavigation
				browseMessageList: obs
				name: 'Obsolete References to ' , oldName
				autoSelect: oldName].
	self selectClassNamed: newName.! !

!Browser methodsFor: 'system category functions' stamp: 'fbs 8/3/2011 07:30'!
renameSystemCategory
	"Prompt for a new category name and add it before the
	current selection, or at the end if no current selection"
	| oldSelection newName |
	oldSelection := self selectedSystemCategory.
	oldSelection isNil
		ifTrue: [^ self].  "no selection"
	self okToChange ifFalse: [^ self].
	
	newName := self
		request: 'Please type new category name'
		initialAnswer: oldSelection.
	newName isEmpty
		ifTrue: [^ self]
		ifFalse: [newName := newName asSymbol].
	oldSelection = newName ifTrue: [^ self].
	systemOrganizer
		renameCategory: oldSelection
		toBe: newName.
	self selectSystemCategory: newName.
	self changed: #systemCategoryList.! !

!Browser methodsFor: 'accessing' stamp: 'rbb 3/1/2005 10:26'!
request: prompt initialAnswer: initialAnswer

	^ UIManager default
		request: prompt
		initialAnswer: initialAnswer
! !

!Browser methodsFor: 'multi-window support' stamp: 'eem 6/30/2010 20:20'!
restoreMultiWindowState: aMessage
	"Restore the state after a multi-window switch.."
	aMessage sentTo: self! !

!Browser methodsFor: 'multi-window support' stamp: 'eem 6/14/2013 13:52'!
restoreToCategory: category className: className protocol: protocol selector: selector mode: editMode meta: metaBool
	selectedSystemCategory := nil. "forces recomputation in selectSystemCategory:"
	self selectSystemCategory: category.
	self selectClassNamed: className.
	self metaClassIndicated: metaBool.
	self selectMessageCategoryNamed: protocol.
	self selectMessageNamed: selector.
	editSelection := editMode.
	self
		contentsChanged;
		decorateButtons! !

!Browser methodsFor: 'multi-window support' stamp: 'fbs 3/3/2011 17:53'!
saveMultiWindowState
	^Message
		selector: #restoreToCategory:className:protocol:selector:mode:meta:
		arguments: {	self selectedSystemCategory.
						self selectedClassName.
						self selectedMessageCategoryName.
						self selectedMessageName.
						self editSelection.
						self metaClassIndicated }! !

!Browser methodsFor: 'system category list' stamp: 'fbs 3/5/2011 20:55'!
selectCategoryForClass: theClass
	self selectSystemCategory: theClass category.! !

!Browser methodsFor: 'class list' stamp: 'cmm 4/21/2015 19:15'!
selectClassNamed: aSymbolOrString
	| className currentMessageCategoryName currentMessageName |

	currentMessageCategoryName := [self selectedMessageCategoryName]
										on: Error
										do: [:ex| ex return: nil].
	currentMessageName := [self selectedMessageName]
								on: Error
								do: [:ex| ex return: nil].
								
	selectedClassName := aSymbolOrString ifNotNil: [ aSymbolOrString asSymbol ].
	self setClassOrganizer.
	self setClassDefinition.

	"Try to reselect the category and/or selector if the new class has them."
	selectedMessageCategoryName :=(self messageCategoryList includes: currentMessageCategoryName)
		ifTrue: [currentMessageCategoryName]
		ifFalse: [nil].
	selectedMessageName := (self messageList includes: currentMessageName)
		ifTrue: [currentMessageName]
		ifFalse: [nil].

	self hasMessageSelected ifTrue:
		[self editSelection: #editMessage] ifFalse:
	[self hasMessageCategorySelected ifTrue:
		[self editSelection: #newMessage] ifFalse:
	[self classCommentIndicated
		ifTrue: [self editSelection: #editComment]
		ifFalse: [self editSelection: (self hasClassSelected not
					ifTrue: [(metaClassIndicated or: [ self hasSystemCategorySelected not ])
						ifTrue: [#none]
						ifFalse: [#newClass]]
					ifFalse: [#editClass])]]].
	contents := nil.
	self selectedClass isNil
		ifFalse: [className := self selectedClass name.
					(RecentClasses includes: className)
				ifTrue: [RecentClasses remove: className].
			RecentClasses addFirst: className.
			RecentClasses size > 16
				ifTrue: [RecentClasses removeLast]].
	self changed: #classSelectionChanged.
	self changed: #classCommentText.
	self changed: #classListIndex.	"update my selection"
	self changed: #messageCategoryList.
	self changed: #messageList.
	self changed: #relabel.
	self changed: #selectedSystemCategoryName.
	self contentsChanged! !

!Browser methodsFor: 'class list' stamp: 'fbs 3/16/2011 07:57'!
selectClass: classNotMeta
	^ self selectClassNamed:
		(classNotMeta
			ifNil: [ nil ]
			ifNotNil: [ classNotMeta name ]).! !

!Browser methodsFor: 'accessing' stamp: 'cwp 6/24/2012 23:20'!
selectEnvironment: anEnvironment 
	environment := anEnvironment.
	systemOrganizer := environment organization! !

!Browser methodsFor: 'class list' stamp: 'fbs 4/25/2011 19:27'!
selectedClass
	"Answer the class that is currently selected. Answer nil if no selection 
	exists."

	| name envt |
	(name := self selectedClassName) ifNil: [^ nil].
	(envt := self selectedEnvironment) ifNil: [^ nil].
	^ envt at: name ifAbsent: [nil]! !

!Browser methodsFor: 'class list' stamp: 'fbs 3/16/2011 10:43'!
selectedClassName
	^ selectedClassName.! !

!Browser methodsFor: 'metaclass' stamp: 'al 4/24/2004 11:47'!
selectedClassOrMetaClass
	"Answer the selected class/trait or metaclass/classTrait."

	| cls |
	^self metaClassIndicated
		ifTrue: [(cls := self selectedClass) ifNil: [nil] ifNotNil: [cls classSide]]
		ifFalse: [self selectedClass]! !

!Browser methodsFor: 'metaclass' stamp: ''!
selectedClassOrMetaClassName
	"Answer the selected class name or metaclass name."

	^self selectedClassOrMetaClass name! !

!Browser methodsFor: 'system category list' stamp: 'cwp 6/24/2012 23:06'!
selectedEnvironment
	"Answer the name of the selected system category or nil."

	self hasSystemCategorySelected ifFalse: [^nil].
	^ environment ifNil: [Smalltalk]! !

!Browser methodsFor: 'message list' stamp: 'md 2/20/2006 15:01'!
selectedMessage
	"Answer a copy of the source code for the selected message."

	| class selector method |
	contents == nil ifFalse: [^ contents copy].

	self showingDecompile ifTrue:
		[^ self decompiledSourceIntoContents].

	class := self selectedClassOrMetaClass.
	selector := self selectedMessageName.
	method := class compiledMethodAt: selector ifAbsent: [^ ''].	"method deleted while in another project"
	currentCompiledMethod := method.

	^ contents := (self showingDocumentation
		ifFalse: [ self sourceStringPrettifiedAndDiffed ]
		ifTrue: [ self commentContents ])
			copy asText makeSelectorBoldIn: class! !

!Browser methodsFor: 'message category list' stamp: 'fbs 3/30/2011 21:50'!
selectedMessageCategoryName
	"Answer the name of the selected message category, if any. Answer nil 
	otherwise."

	^ selectedMessageCategoryName! !

!Browser methodsFor: 'message list' stamp: 'fbs 3/29/2011 17:03'!
selectedMessageName
	"Answer the message selector of the currently selected message, if any. 
	Answer nil otherwise."

	^ selectedMessageName.! !

!Browser methodsFor: 'message list' stamp: 'topa 3/1/2016 16:42'!
selectedMessageName: aSelector
	"Make the given selector be the selected message name"

	| anIndex |
	anIndex := self messageList indexOf: aSelector.
	anIndex > 0 ifTrue:
		[self messageListIndex: anIndex].
	self changed: #selectedMessageName. "inform interested parties"! !

!Browser methodsFor: 'system category list' stamp: 'fbs 3/3/2011 07:55'!
selectedSystemCategory
	^ selectedSystemCategory! !

!Browser methodsFor: 'system category list' stamp: 'fbs 3/3/2011 13:10'!
selectedSystemCategoryName
	"Answer the name of the selected system category or nil."

	^ self selectedSystemCategory.! !

!Browser methodsFor: 'message category list' stamp: 'fbs 3/30/2011 21:55'!
selectMessageCategoryNamed: aSymbol 
	"Given aSymbol, select the category with that name.  Do nothing if 
	aSymbol doesn't exist."
	selectedMessageCategoryName := aSymbol.
	
	selectedMessageName := nil.
	self changed: #messageCategorySelectionChanged.
	self changed: #messageCategoryListIndex. "update my selection"
	self changed: #messageList.
	self editSelection: (aSymbol notNil
		ifTrue: [#newMessage]
		ifFalse: [self hasClassSelected
			ifTrue: [#editClass]
			ifFalse: [#newClass]]).
	contents := nil.
	self contentsChanged.! !

!Browser methodsFor: 'message list' stamp: 'fbs 3/29/2011 17:04'!
selectMessageNamed: aSymbolOrString
	| name |
	name := aSymbolOrString ifNotNil: [ aSymbolOrString asSymbol ].
	selectedMessageName := name.

	self editSelection: (name notNil
		ifTrue: [#editMessage]
		ifFalse: [self messageCategoryListIndex > 0
			ifTrue: [#newMessage]
			ifFalse: [self hasClassSelected
				ifTrue: [#editClass]
				ifFalse: [#newClass]]]).
	contents := nil.
	self changed: #messageListIndex. "update my selection"
	self contentsChanged.
	self decorateButtons.! !

!Browser methodsFor: 'system category list' stamp: 'fbs 3/1/2013 08:58'!
selectSystemCategory: aSymbol
	"Set the selected system category. Update all other selections to be deselected."
	aSymbol = selectedSystemCategory ifTrue: [^ self].
	selectedSystemCategory := aSymbol.
	selectedClassName := nil.
	selectedMessageCategoryName := nil.
	selectedMessageName := nil.
	self editSelection: ( aSymbol isNil ifTrue: [#none] ifFalse: [#newClass]).
	metaClassIndicated := false.
	self setClassOrganizer.
	contents := nil.
	self changed: #systemCategorySelectionChanged.
	self changed: #systemCategoryListIndex.	"update my selection"
	self changed: #classList.
	self changed: #messageCategoryList.
	self changed: #messageList.
	self changed: #relabel.
	self contentsChanged! !

!Browser methodsFor: 'metaclass' stamp: 'mt 4/14/2015 13:47'!
setClassDefinition
	"Remember the current class definition."
	
	| theClass |
	classDefinition := nil.
	metaClassDefinition := nil.
	self hasClassSelected ifFalse: [^ self].
	theClass := self selectedClass ifNil: [ ^self ].
	classDefinition := theClass definition.
	metaClassDefinition := theClass theMetaClass definition.! !

!Browser methodsFor: 'metaclass' stamp: 'fbs 3/4/2011 20:56'!
setClassOrganizer
	"Install whatever organization is appropriate"
	| theClass |
	classOrganizer := nil.
	metaClassOrganizer := nil.
	self hasClassSelected ifFalse: [^ self].
	theClass := self selectedClass ifNil: [ ^self ].
	classOrganizer := theClass organization.
	metaClassOrganizer := theClass classSide organization.! !

!Browser methodsFor: 'initialize-release' stamp: 'mt 1/18/2016 08:39'!
setClass: aBehavior
	"Set the state of a new, uninitialized Browser."

	| isMeta aClass |
	aBehavior ifNil: [^ self].
	(aBehavior isKindOf: Metaclass)
		ifTrue: [
			isMeta := true.
			aClass := aBehavior soleInstance]
		ifFalse: [
			isMeta := false.
			aClass := aBehavior].
		
	self
		selectEnvironment: aClass environment;
		selectCategoryForClass: aClass;
		classListIndex: (self classListIndexOf: aClass name);
		metaClassIndicated: isMeta.! !

!Browser methodsFor: 'initialize-release' stamp: 'mt 1/18/2016 08:40'!
setClass: aBehavior selector: aSymbol
	"Set the state of a new, uninitialized Browser."

	aBehavior ifNil: [^ self].

	self
		setClass: aBehavior;	
		setSelector: aSymbol.! !

!Browser methodsFor: 'message category list' stamp: 'fbs 3/30/2011 21:50'!
setOriginalCategoryIndexForCurrentMethod
	"private - Set the message category index for the currently selected method. 
	 
	 Note:  This should only be called when somebody tries to save  
	 a method that they are modifying while ALL is selected."

	selectedMessageCategoryName := self categoryOfCurrentMethod.! !

!Browser methodsFor: 'initialize-release' stamp: 'mt 3/2/2015 10:10'!
setSelector: aSymbol
	"Make the receiver point at the given selector, in the currently chosen class"

	| aClass |
	aSymbol ifNil: [^ self].
	(aClass := self selectedClassOrMetaClass) ifNil: [^ self].
	(aClass organization categoryOfElement: aSymbol)
		ifNil: [^ self]
		ifNotNil: [:category |
			self
				selectMessageCategoryNamed: category;
				selectMessageNamed: aSymbol].! !

!Browser methodsFor: 'class functions' stamp: 'topa 3/26/2015 21:40'!
shiftedClassListMenuMore: aMenu
	" The 'more..' link that toggles between unshifted and shifted menus
	in class lists "
	<classListMenuShifted: true>
	<menuPriority: 1000>
	^ aMenu addList: #(-
			('more...' offerUnshiftedClassListMenu 'return to the standard class-list menu'));
		yourself! !

!Browser methodsFor: 'class functions' stamp: 'eem 12/2/2015 15:37'!
shiftedClassListMenu: aMenu
	<classListMenuShifted: true>
	"Set up the menu to apply to the receiver's class list when the shift key is down"
	^ aMenu
		addList: #(
			-
			('local senders...'			browseLocalSenders	'browse senders local to this class')
			('unsent methods'			browseUnusedMethods	'browse all methods defined by this class that have no senders')
			('unreferenced inst vars'	showUnreferencedInstVars	'show a list of all instance variables that are not referenced in methods')
			('unreferenced class vars'	showUnreferencedClassVars	'show a list of all class variables that are not referenced in methods')
			('subclass template'			makeNewSubclass		'put a template into the code pane for defining of a subclass of this class')
			-
			('sample instance'			makeSampleInstance		'give me a sample instance of this class, if possible')
			('inspect instances'			inspectInstances			'open an inspector on all the extant instances of this class')
			('inspect subinstances'		inspectSubInstances		'open an inspector on all the extant instances of this class and of all of its subclasses')
			-
			('add all meths to current chgs'		addAllMethodsToCurrentChangeSet
																'place all the methods defined by this class into the current change set')
			('create inst var accessors'	createInstVarAccessors	'compile instance-variable access methods for any instance variables that do not yet have them'));
		yourself! !

!Browser methodsFor: 'message functions' stamp: 'cmm 4/30/2016 20:29'!
shiftedMessageListMenu: aMenu
	<messageListMenuShifted: true>
	"Fill aMenu with the items appropriate when the shift key is held down"

	aMenu addStayUpItem.
	aMenu addList: #(
		('toggle diffing (D)'						toggleDiffing)
		('implementors of sent messages'			browseAllMessages)
		-
		('local senders of...'						browseLocalSendersOfMessages)
		('local implementors of...'				browseLocalImplementors)
		-
		('spawn sub-protocol'					spawnProtocol)
		('spawn full protocol'					spawnFullProtocol)
		-
		('sample instance'						makeSampleInstance)
		('inspect instances'						inspectInstances)
		('inspect subinstances'					inspectSubInstances)).

	self addExtraShiftedItemsTo: aMenu.
	aMenu addList: #(
		-
		('change category...'					changeCategory)).

	self canShowMultipleMessageCategories ifTrue: [aMenu addList:
		 #(('toggle category selection (Y)'						showHomeCategory))].
	aMenu addList: #(
		-
		('change sets with this method'			findMethodInChangeSets)
		('revert to previous version'				revertToPreviousVersion)
		('remove from current change set'		removeFromCurrentChanges)
		('revert & remove from changes'		revertAndForget)
		('add to current change set'				adoptMessageInCurrentChangeset)
		('copy up or copy down...'				copyUpOrCopyDown)).
	^ aMenu
! !

!Browser methodsFor: 'code pane' stamp: 'sw 5/18/2001 20:55'!
showBytecodes
	"Show or hide the bytecodes of the selected method -- an older protocol now mostly not relevant."

	self toggleShowingByteCodes! !

!Browser methodsFor: 'message category functions' stamp: 'cmm 6/2/2016 17:21'!
showHomeCategory
	"Show the home category of the selected method.  This is only really useful if one is in a tool that supports the showing of categories.  Thus, it's good in browsers and hierarchy browsers but not in message-list browsers"

	| aSelector |
	self okToChange ifTrue:
		[(aSelector := self selectedMessageName) ifNotNil:
			[self toggleCategorySelectionForCurrentMethod.
			self selectedMessageName: aSelector]]! !

!Browser methodsFor: 'accessing' stamp: 'fbs 3/3/2011 22:58'!
spawn: aString 
	"Create and schedule a fresh browser and place aString in its code pane.  This method is called when the user issues the #spawn command (cmd-o) in any code pane.  Whatever text was in the original code pane comes in to this method as the aString argument; the changes in the original code pane have already been cancelled by the time this method is called, so aString is the only copy of what the user had in his code pane."

	self selectedClassOrMetaClass ifNotNil: [^ super spawn: aString].

	self hasSystemCategorySelected ifTrue:
		["This choice is slightly useless but is the historical implementation"
		^ self buildSystemCategoryBrowserEditString: aString].
		
	^ super spawn: aString  
	"This bail-out at least saves the text being spawned, which would otherwise be lost"! !

!Browser methodsFor: 'class comment pane' stamp: 'bf 4/12/2011 14:58'!
stripNaggingAttributeFromComment: aText
	^aText removeAttribute: TextColor red from: 1 to: aText size! !

!Browser methodsFor: 'accessing' stamp: 'sw 9/26/2002 17:56'!
suggestCategoryToSpawnedBrowser: aBrowser
	"aBrowser is a message-category browser being spawned from the receiver.  Tell it what it needs to know to get its category info properly set up."

	(self isMemberOf: Browser) "yecch, but I didn't invent the browser hierarchy"
		ifTrue:
			[aBrowser messageCategoryListIndex: (self messageCategoryList indexOf: self categoryOfCurrentMethod ifAbsent: [2])]
		ifFalse:
			[aBrowser setOriginalCategoryIndexForCurrentMethod]! !

!Browser methodsFor: 'initialize-release' stamp: 'fbs 4/3/2010 20:50'!
switchesFrame: bottomFraction
	^self switchesFrame: bottomFraction fromLeft: 0.25 width: 0.25.! !

!Browser methodsFor: 'initialize-release' stamp: 'mt 4/30/2015 11:28'!
switchesFrame: bottomFraction fromLeft: leftFraction width: rightFraction
	^LayoutFrame new
		leftFraction: leftFraction offset: 0;
		topFraction: bottomFraction offset: self buttonHeight negated;
		rightFraction: (leftFraction + rightFraction) offset: 0;
		bottomFraction: bottomFraction offset: 0! !

!Browser methodsFor: 'system category list' stamp: 'mt 4/12/2015 15:05'!
systemCategoryList
	"Answer the class categories modelled by the receiver."

	^ {SystemOrganizer allCategory},  systemOrganizer categories! !

!Browser methodsFor: 'system category list' stamp: 'fbs 3/3/2011 17:45'!
systemCategoryListIndex
	"Answer the index of the selected class category."

	^ self systemCategoryList indexOf: self selectedSystemCategory.! !

!Browser methodsFor: 'system category list' stamp: 'fbs 3/5/2011 20:55'!
systemCategoryListIndex: anInteger 
	"Set the selected system category index to be anInteger. Update all other 
	selections to be deselected."
	
	self selectSystemCategory: (self systemCategoryList at: anInteger ifAbsent: [ nil ])! !

!Browser methodsFor: 'pluggable menus - hooks' stamp: 'topa 3/27/2015 01:20'!
systemCategoryMenuHook: aMenu shifted: aBoolean
	<systemCategoryMenu>
	<menuPriority: 400>
	^ self menuHook: aMenu named: #systemCategoryMenu shifted: aBoolean.
! !

!Browser methodsFor: 'system category functions' stamp: 'topa 3/13/2015 18:51'!
systemCategoryMenu: aMenu
	^ self menu: aMenu for: #(systemCategoryMenu systemCategoryMenuShifted:)
! !

!Browser methodsFor: 'system category list' stamp: 'fbs 3/3/2011 17:53'!
systemCategorySingleton

	| cat |
	cat := self selectedSystemCategory.
	^ cat ifNil: [Array new]
		ifNotNil: [Array with: cat]! !

!Browser methodsFor: 'system category list' stamp: 'fbs 12/11/2012 22:25'!
systemCatListKey: aChar from: view
	"Respond to a Command key.  I am a model with a code pane, and I also have a listView that has a list of methods.  The view knows how to get the list and selection."

	aChar == $f ifTrue: [^ self findClass].
	aChar == $x ifTrue: [^ self removeSystemCategory].
	aChar == $b ifTrue: [^ self recent].
	^ self classListKey: aChar from: view! !

!Browser methodsFor: 'initialize-release' stamp: 'sw 11/8/1999 13:36'!
systemCatSingletonKey: aChar from: aView
	^ self messageListKey: aChar from: aView! !

!Browser methodsFor: 'system category functions' stamp: 'sw 11/8/1999 14:07'!
systemCatSingletonMenu: aMenu

	^ aMenu labels:
'browse all
browse
printOut
fileOut
update
rename...
remove' 
	lines: #(2 4)
	selections:
		#(browseAllClasses buildSystemCategoryBrowser
		printOutSystemCategory fileOutSystemCategory updateSystemCategories
		renameSystemCategory removeSystemCategory)
! !

!Browser methodsFor: 'initialize-release' stamp: 'fbs 3/30/2011 21:48'!
systemOrganizer: aSystemOrganizer
	"Initialize the receiver as a perspective on the system organizer, 
	aSystemOrganizer. Typically there is only one--the system variable 
	SystemOrganization."
	
	contents := nil.
	systemOrganizer := aSystemOrganizer.
	selectedSystemCategory := nil.
	selectedMessageCategoryName := nil.
	selectedClassName := nil.
	selectedMessageName := nil.
	metaClassIndicated := false.
	self setClassOrganizer.
	self editSelection: #none.! !

!Browser methodsFor: 'message category list' stamp: 'cmm 6/2/2016 17:24'!
toggleCategorySelectionForCurrentMethod
	| methodCategory selectorName |
	methodCategory := self categoryOfCurrentMethod.
	selectorName := self selectedMessageName.
	(methodCategory notNil and:
		[ methodCategory ~= ClassOrganizer allCategory and: [ methodCategory ~= selectedMessageCategoryName ] ])
		ifTrue:
			[ selectedMessageCategoryName := methodCategory.
			selectedMessageName := selectorName.
			self changed: #messageCategorySelectionChanged.
			self changed: #messageCategoryListIndex.
			"update my selection"
			self changed: #messageList.
			self changed: #messageListIndex ]
		ifFalse:
			[ methodCategory = selectedMessageCategoryName ifTrue:
				[ selectedMessageCategoryName := nil.
				self
					 changed: #messageCategorySelectionChanged ;
					 changed: #messageCategoryListIndex ;
					 changed: #messageList ] ]! !

!Browser methodsFor: 'initialize-release' stamp: 'fbs 4/22/2010 14:52'!
topConstantHeightFrame: height fromLeft: leftFraction width: rightFraction
	^LayoutFrame new
		topFraction: 0 offset: 0;
		leftFraction: leftFraction offset: 0;
		rightFraction: (leftFraction + rightFraction) offset: 0;
		bottomFraction: 0 offset: height;
		yourself.! !

!Browser methodsFor: 'system category functions' stamp: 'di 4/12/98 13:17'!
updateSystemCategories
	"The class categories were changed in another browser. The receiver must 
	reorganize its lists based on these changes."

	self okToChange ifFalse: [^ self].
	self changed: #systemCategoryList! !

!Browser methodsFor: 'copying' stamp: 'fbs 3/30/2011 21:51'!
veryDeepInner: deepCopier
	"Copy all of my instance variables.  Some need to be not copied at all, but shared.  See DeepCopier class comment."

super veryDeepInner: deepCopier.
"systemOrganizer := systemOrganizer. 	clone has the old value. we share it"
"classOrganizer := classOrganizer		clone has the old value. we share it"
"metaClassOrganizer 	:= metaClassOrganizer	clone has the old value. we share it"
selectedSystemCategory := selectedSystemCategory veryDeepCopyWith: deepCopier.
selectedClassName := selectedClassName veryDeepCopyWith: deepCopier.
selectedMessageCategoryName := selectedMessageCategoryName veryDeepCopyWith: deepCopier.
selectedMessageName := selectedMessageName veryDeepCopyWith: deepCopier.
editSelection := editSelection veryDeepCopyWith: deepCopier.
metaClassIndicated := metaClassIndicated veryDeepCopyWith: deepCopier.
! !

!Browser methodsFor: 'drag and drop' stamp: 'ar 8/8/2009 10:06'!
wantsMessageCategoriesDrop: anObject
	"Only accept drops of compiled methods on system categories"
	^anObject isKindOf: CompiledMethod! !

!Browser methodsFor: 'drag and drop' stamp: 'ar 8/8/2009 10:06'!
wantsSystemCategoriesDrop: anObject
	"Only accept drops of behaviors on system categories"
	^anObject isBehavior! !

Browser subclass: #FileContentsBrowser
	instanceVariableNames: 'packages infoString'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-File Contents Browser'!

!FileContentsBrowser commentStamp: '<historical>' prior: 0!
I am a class browser view on a fileout (either a source file (.st) or change set (.cs)). I do not actually load the code into to the system, nor do I alter the classes in the image. Use me to vet code in a comfortable way before loading it into your image.

From a FileList, I can be invoked by selecting a source file and selecting the "browse code" menu item from the yellow button menu.

I use PseudoClass, PseudoClassOrganizers, and PseudoMetaclass to model the class structure of the source file.!

!FileContentsBrowser class methodsFor: 'instance creation' stamp: 'nice 8/5/2011 13:30'!
browseCompressedCodeStream: aStandardFileStream 
	"Browse the selected file in fileIn format."
	| unzipped |
	unzipped :=  [MultiByteBinaryOrTextStream
			with:  (GZipReadStream on: aStandardFileStream) contents asString]
		ensure: [aStandardFileStream close].
	unzipped reset.
	self browseStream: unzipped named: aStandardFileStream name! !

!FileContentsBrowser class methodsFor: 'instance creation' stamp: 'nice 12/27/2009 03:11'!
browseFiles: fileList

	| browser |
	Cursor wait showWhile: [ | organizer packageDict |
		packageDict := Dictionary new.
		organizer := SystemOrganizer defaultList: Array new.
		fileList do: [:fileName | | package |
			package := FilePackage fromFileNamed: fileName.
			packageDict 
				at: package packageName 
				put: package.
			organizer 
				classifyAll: package classes keys 
				under: package packageName].
		(browser := self systemOrganizer: organizer)
			packages: packageDict].
	self
		openBrowserView: browser createViews
		label: 'File Contents Browser'.
! !

!FileContentsBrowser class methodsFor: 'instance creation' stamp: 'nb 6/17/2003 12:25'!
browseFile: aFilename
	"Open a file contents browser on a file of the given name"

	aFilename ifNil: [^ Beeper beep].
	self browseFiles: (Array with: aFilename)! !

!FileContentsBrowser class methodsFor: 'instance creation' stamp: 'edc 5/12/2006 07:09'!
browseStream: aStream
aStream setConverterForCode.
	self browseStream: aStream named: aStream name! !

!FileContentsBrowser class methodsFor: 'instance creation' stamp: 'nice 12/27/2009 03:11'!
browseStream: aStream named: aString

	| browser |
	Cursor wait showWhile: [ | package packageDict organizer |
		packageDict := Dictionary new.
		browser := self new.
		organizer := SystemOrganizer defaultList: Array new.
		package := (FilePackage new fullName: aString; fileInFrom: aStream).
		packageDict 
			at: package packageName 
			put: package.
		organizer 
			classifyAll: package classes keys 
			under: package packageName.
		(browser := self systemOrganizer: organizer)
			packages: packageDict].
	self
		openBrowserView: browser createViews
		label: 'File Contents Browser'.
! !

!FileContentsBrowser class methodsFor: 'file list services' stamp: 'nk 6/12/2004 11:41'!
fileReaderServicesForDirectory: aDirectory
	^{ self serviceBrowseCodeFiles }! !

!FileContentsBrowser class methodsFor: 'file list services' stamp: 'nk 2/17/2004 19:18'!
fileReaderServicesForFile: fullName suffix: suffix

	((FileStream isSourceFileSuffix: suffix) or: [ suffix = '*' ])
		ifTrue: [ ^Array with: self serviceBrowseCode].

	^(fullName endsWith: 'cs.gz')
		ifTrue: [ Array with: self serviceBrowseCompressedCode ]
		ifFalse: [#()]
! !

!FileContentsBrowser class methodsFor: 'class initialization' stamp: 'hg 8/3/2000 18:17'!
initialize

	FileList registerFileReader: self! !

!FileContentsBrowser class methodsFor: 'file list services' stamp: 'rbb 3/1/2005 10:53'!
selectAndBrowseFile: aFileList
	"When no file are selected you can ask to browse several of them"

	| selectionPattern files |
	selectionPattern := UIManager default request:'What files?' initialAnswer: '*.cs;*.st'.
	files := (aFileList directory fileNamesMatching: selectionPattern) 
				collect: [:each | aFileList directory fullNameFor: each].
	self browseFiles: files.


! !

!FileContentsBrowser class methodsFor: 'file list services' stamp: 'KR 3/21/2008 21:07'!
serviceBrowseCode
	"Answer the service of opening a file-contents browser"

	^ (SimpleServiceEntry
		provider: self 
		label: 'code-file browser' translatedNoop
		selector: #browseStream:
		description: 'open a "file-contents browser" on this file, allowing you to view and selectively load its code' translatedNoop
		buttonLabel: 'code' translatedNoop)
		argumentGetter: [ :fileList | fileList readOnlyStream ]! !

!FileContentsBrowser class methodsFor: 'file list services' stamp: 'KR 3/21/2008 21:06'!
serviceBrowseCodeFiles

	^  (SimpleServiceEntry 
		provider: self
		label: 'browse code files' translatedNoop
		selector: #selectAndBrowseFile:)
		argumentGetter: [ :fileList | fileList ];
		yourself! !

!FileContentsBrowser class methodsFor: 'file list services' stamp: 'KR 3/21/2008 21:06'!
serviceBrowseCompressedCode
	"Answer a service for opening a changelist browser on a file"

	^ (SimpleServiceEntry 
		provider: self 
		label: 'code-file browser' translatedNoop
		selector: #browseCompressedCodeStream:
		description: 'open a "file-contents browser" on this file, allowing you to view and selectively load its code' translatedNoop
		buttonLabel: 'code' translatedNoop)
		argumentGetter: [ :fileList | fileList readOnlyStream ]! !

!FileContentsBrowser class methodsFor: 'file list services' stamp: 'md 11/23/2004 13:34'!
services
	"Answer potential file services associated with this class"

	^ {self serviceBrowseCode}.! !

!FileContentsBrowser class methodsFor: 'preferences' stamp: 'ul 3/12/2015 00:59'!
showMessageIcons
	"Remove this method once PseudoClass implements the methods of the class API required to show the icons."

	^false! !

!FileContentsBrowser class methodsFor: 'class initialization' stamp: 'SD 11/15/2001 22:21'!
unload

	FileList unregisterFileReader: self ! !

!FileContentsBrowser methodsFor: 'edit pane' stamp: 'ar 8/17/2009 23:18'!
aboutToStyle: aStyler
	"This is a notification that aStyler is about to re-style its text.
	Set the classOrMetaClass in aStyler, so that identifiers
	will be resolved correctly.
	Answer true to allow styling to proceed, or false to veto the styling"

	self isModeStyleable ifFalse: [^false].
	aStyler classOrMetaClass: self selectedClassOrMetaClass.
	^true! !

!FileContentsBrowser methodsFor: 'other' stamp: 'tpr 9/2/2013 15:53'!
browseMessages
	"Open a message set browser of all implementors of the currently selected message"

	self systemNavigation browseAllImplementorsOf: (self selectedMessageName ifNil: [ ^nil ])! !

!FileContentsBrowser methodsFor: 'class list' stamp: 'ar 9/27/2005 20:27'!
browseMethodFull
	| myClass |
	(myClass := self selectedClassOrMetaClass) ifNotNil:
		[ToolSet browse: myClass realClass selector: self selectedMessageName]! !

!FileContentsBrowser methodsFor: 'other' stamp: 'fbs 3/17/2011 17:32'!
browseSenders
	"Create and schedule a message set browser on all senders of the 
	currently selected message selector. Do nothing if no message is selected."

	self hasMessageSelected 
		ifTrue: [self systemNavigation browseAllCallsOn: self selectedMessageName]! !

!FileContentsBrowser methodsFor: 'other' stamp: 'tpr 9/2/2013 15:56'!
browseSendersOfMessages
	"delegate to plain browseSenders since we can't (easily) find all the enst messages within the not-real method"

	^self browseSenders! !

!FileContentsBrowser methodsFor: 'other' stamp: 'eem 1/7/2016 11:19'!
browseVersions
	"Create and schedule a message set browser on all versions of the currently selected message selector."
	(ToolSet
		browseVersionsOf: self selectedClassOrMetaClass
		selector: self selectedMessageName) ifNil: [self changed: #flash]! !

!FileContentsBrowser methodsFor: 'toolbuilder' stamp: 'mt 4/19/2015 08:57'!
buildAnnotationPaneWith: builder

	| textSpec |
	textSpec := builder pluggableInputFieldSpec new.
	textSpec 
		model: self;
		getText: #infoViewContents.
	^textSpec! !

!FileContentsBrowser methodsFor: 'toolbuilder' stamp: 'topa 3/16/2015 10:23'!
buildSystemCategoryListWith: builder
	| listSpec |
	listSpec := builder pluggableListSpec new.
	listSpec 
			model: self;
			list: #systemCategoryList; 
			getIndex: #systemCategoryListIndex; 
			setIndex: #systemCategoryListIndex:; 
			menu: #packageListMenu:shifted:; 
			keyPress: #packageListKey:from:.
	^listSpec! !

!FileContentsBrowser methodsFor: 'toolbuilder' stamp: 'cmm 4/21/2015 21:19'!
buildSystemCatListSingletonWith: aToolBuilder
	"Overwritten to change callbacks for menu and keyboard interaction."
	^ aToolBuilder pluggableInputFieldSpec new
		model: self ;
		getText: #selectedSystemCategoryName ;
		setText: nil ;
"		keyPress: #systemCatSingletonKey:from: ;
"		menu: #packageListMenu:shifted:; 
"		keyPress: #packageListKey:from:;
"		yourself! !

!FileContentsBrowser methodsFor: 'toolbuilder' stamp: 'mt 4/19/2015 08:53'!
buildWith: builder

	^ self packages size = 1
		ifTrue:[
			self systemCategoryListIndex: 1.
			self openSystemCatEditString: '']
		ifFalse: [super buildWith: builder]! !

!FileContentsBrowser methodsFor: 'other' stamp: 'fbs 3/15/2011 22:41'!
changeMessageCategories: aString 
	"The characters in aString represent an edited version of the the message 
	categories for the selected class. Update this information in the system 
	and inform any dependents that the categories have been changed. This 
	message is invoked because the user had issued the categories command 
	and edited the message categories. Then the user issued the accept 
	command."

	self classOrMetaClassOrganizer changeFromString: aString.
	self unlock.
	self editClass.
	self selectClassNamed: selectedClassName.
	^ true! !

!FileContentsBrowser methodsFor: 'class list' stamp: 'fbs 3/3/2011 23:02'!
classList
	"Answer an array of the class names of the selected category. Answer an 
	empty array if no selection exists."

	(self hasSystemCategorySelected not or:[self selectedPackage isNil])
		ifTrue: [^Array new]
		ifFalse: [^self selectedPackage classes keys asArray sort].! !

!FileContentsBrowser methodsFor: 'keys' stamp: 'sma 5/6/2000 18:48'!
classListKey: aChar from: view
	aChar == $b ifTrue: [^ self browseMethodFull].
	aChar == $N ifTrue: [^ self browseClassRefs].
	self packageListKey: aChar from: view! !

!FileContentsBrowser methodsFor: 'menus' stamp: 'topa 3/16/2015 11:15'!
classListMenu: aMenu shifted: aBool

	^ self menu: aMenu for: #(fileClassListMenu fileClassListMenuShifted:) shifted: aBool
! !

!FileContentsBrowser methodsFor: 'accessing' stamp: ''!
contents
	self updateInfoView.
	(editSelection == #newClass and:[self selectedPackage notNil])
		ifTrue: [^self selectedPackage packageInfo].
	editSelection == #editClass
		ifTrue:[^self modifiedClassDefinition].
	^super contents! !

!FileContentsBrowser methodsFor: 'menus' stamp: 'sw 11/13/2001 09:12'!
contentsSymbolQuints
	"Answer a list of quintuplets representing information on the alternative views available in the code pane.  For the file-contents browser, the choices are restricted to source and the two diffing options"

	^ self sourceAndDiffsQuintsOnly! !

!FileContentsBrowser methodsFor: 'accessing' stamp: 'sw 5/23/2001 14:28'!
contents: input notifying: aController 
	"The retrieved information has changed and its source must now be 
	updated. The information can be a variety of things, depending on the 
	list selections (such as templates for class or message definition, methods) 
	or the user menu commands (such as definition, comment, hierarchy). 
	Answer the result of updating the source."

	| aString aText theClass |
	aString := input asString.
	aText := input asText.

	editSelection == #editComment 
		ifTrue: [theClass := self selectedClass.
				theClass ifNil: [self inform: 'You must select a class
before giving it a comment.'.
				^ false].
				theClass comment: aText. ^ true].
	editSelection == #editMessageCategories 
		ifTrue: [^ self changeMessageCategories: aString].

	self inform:'You cannot change the current selection'.
	^false
! !

!FileContentsBrowser methodsFor: 'creation' stamp: 'ar 8/7/2009 21:24'!
createViews
	"Create a pluggable version of all the views for a Browser, including views and controllers."
	contentsSymbol := self defaultDiffsSymbol.  "#showDiffs or #prettyDiffs"
	^self buildWith: ToolBuilder default! !

!FileContentsBrowser methodsFor: 'initialize-release' stamp: 'dew 9/15/2001 16:19'!
defaultBrowserTitle
	^ 'File Contents Browser'! !

!FileContentsBrowser methodsFor: 'user interface' stamp: 'mt 7/9/2016 08:52'!
defaultWindowColor
	^ (Color r: 0.7 g: 0.7 b: 0.508)! !

!FileContentsBrowser methodsFor: 'other' stamp: 'asm 5/30/2003 18:11'!
didCodeChangeElsewhere
	"Determine whether the code for the currently selected method and class has been changed somewhere else."

	| aClass |
	(aClass := self selectedClassOrMetaClass) ifNil: [^ false].

	(aClass isKindOf: PseudoClass) ifTrue: [^ false]. "class not installed"
	^super didCodeChangeElsewhere! !

!FileContentsBrowser methodsFor: 'infoView' stamp: 'sma 5/6/2000 19:19'!
extraInfo
	^ (self
		methodDiffFor: (self selectedClassOrMetaClass sourceCodeAt: self selectedMessageName)
		class: self selectedClass
		selector: self selectedMessageName
		meta: self metaClassIndicated) unembellished
			ifTrue: [' - identical']
			ifFalse: [' - modified']! !

!FileContentsBrowser methodsFor: 'menus' stamp: 'topa 3/16/2015 11:16'!
fileClassListMenu: aMenu
	"Answer the class list menu, ignoring the state of the shift key in this case"
	<fileClassListMenu>
	aMenu addList: #(
			('definition'		editClass)
			('comment'			editComment)
			-
			('browse full (b)'	browseMethodFull)
			('class refs (N)'	browseClassRefs)
			-
			('fileIn'			fileInClass)
			('fileOut'			fileOutClass)
			-
			('rename...'			renameClass)
			('remove'			removeClass)
			('remove existing'	removeUnmodifiedCategories)).
	^ aMenu
! !

!FileContentsBrowser methodsFor: 'fileIn/fileOut' stamp: ''!
fileInClass
	Cursor read showWhile:[
		self selectedClass fileIn.
	].! !

!FileContentsBrowser methodsFor: 'fileIn/fileOut' stamp: 'wod 6/16/1998 17:14'!
fileInMessage
	
	self selectedMessageName ifNil: [^self].
	Cursor read showWhile: [
		self selectedClassOrMetaClass fileInMethod: self selectedMessageName.
	].! !

!FileContentsBrowser methodsFor: 'fileIn/fileOut' stamp: 'wod 2/3/1999 18:46'!
fileInMessageCategories
	Cursor read showWhile:[
		self selectedClassOrMetaClass fileInCategory: self selectedMessageCategoryName.
	].! !

!FileContentsBrowser methodsFor: 'fileIn/fileOut' stamp: 'wod 5/13/1998 12:50'!
fileInPackage
	Cursor read showWhile:[
		self selectedPackage fileIn.
	].! !

!FileContentsBrowser methodsFor: 'fileIn/fileOut' stamp: 'ar 9/27/2005 20:07'!
fileIntoNewChangeSet
	| p ff |
	(p := self selectedPackage) ifNil: [^ Beeper beep].
	ff := FileStream readOnlyFileNamed: p fullPackageName.
	ChangeSet newChangesFromStream: ff named: p packageName! !

!FileContentsBrowser methodsFor: 'menus' stamp: 'topa 3/16/2015 11:15'!
fileMessageCategoryMenu: aMenu
	<fileMessageCategoryMenu>
	^ aMenu addList: #(
		('file in'				fileInMessageCategories)
		('file out'			fileOutMessageCategories)
		-
		('reorganize'		editMessageCategories)
		-
		('add item...'			addCategory)
		('rename...'			renameCategory)
		('remove'			removeMessageCategory)
		-
		('remove existing'	removeUnmodifiedMethods));
		yourself
! !

!FileContentsBrowser methodsFor: 'menus' stamp: 'topa 3/15/2015 23:28'!
fileMessageListMenu: aMenu
	<fileMessageListMenu>
	aMenu addList: #(
		('fileIn'						fileInMessage)
		('fileOut'					fileOutMessage)
		-
		('senders (n)'				browseSenders)
		('implementors (m)'			browseImplementors)
		('method inheritance (h)'	methodHierarchy)
		('versions (v)'				browseVersions)
		-
		('remove'					removeMessage)).
	^ aMenu
! !

!FileContentsBrowser methodsFor: 'fileIn/fileOut' stamp: ''!
fileOutClass
	Cursor write showWhile:[
		self selectedClass fileOut.
	].! !

!FileContentsBrowser methodsFor: 'fileIn/fileOut' stamp: 'wod 2/3/1999 18:46'!
fileOutMessageCategories
	Cursor write showWhile:[
		self selectedClassOrMetaClass fileOutCategory: self selectedMessageCategoryName.
	].! !

!FileContentsBrowser methodsFor: 'fileIn/fileOut' stamp: 'wod 5/13/1998 14:19'!
fileOutPackage
	Cursor write showWhile:[
		self selectedPackage fileOut.
	].! !

!FileContentsBrowser methodsFor: 'menus' stamp: 'topa 3/16/2015 11:16'!
filePackageListMenu: aMenu
	<filePackageListMenu>
	^ aMenu addList: #(
			('find class... (f)'		findClass)
			-
			('file in'			fileInPackage)
			('file into new changeset'	fileIntoNewChangeSet)
			('file out'			fileOutPackage)
			-
			('remove'			removePackage)
			-
			('remove existing'		removeUnmodifiedClasses));
		yourself
! !

!FileContentsBrowser methodsFor: 'class list' stamp: 'fbs 3/5/2011 20:57'!
findClass
	| pattern foundClass classNames index foundPackage |
	self okToChange ifFalse: [^ self classNotFound].
	pattern := (UIManager default request: 'Class Name?') asLowercase.
	pattern isEmpty ifTrue: [^ self].
	classNames := Set new.
	self packages do:[:p| classNames addAll: p classes keys].
	classNames := classNames asArray select: 
		[:n | (n asLowercase indexOfSubCollection: pattern startingAt: 1) > 0].
	classNames isEmpty ifTrue: [^ self].
	index := classNames size = 1
				ifTrue:	[1]
				ifFalse:	[(UIManager default chooseFrom: classNames lines: #())].
	index = 0 ifTrue: [^ self].
	foundPackage := nil.
	foundClass := nil.
	self packages do:[:p| 
		(p classes includesKey: (classNames at: index)) ifTrue:[
			foundClass := p classes at: (classNames at: index).
			foundPackage := p]].
	foundClass isNil ifTrue:[^self].
 	self selectSystemCategory: foundPackage packageName asSymbol.
	self classListIndex: (self classList indexOf: foundClass name). ! !

!FileContentsBrowser methodsFor: 'infoView' stamp: 'nice 8/27/2010 20:31'!
infoString
	^infoString ifNil: [infoString := StringHolder new]! !

!FileContentsBrowser methodsFor: 'infoView' stamp: 'sd 11/20/2005 21:27'!
infoViewContents
	"Answer the string to show in the info view"

	| theClass stamp exists |
	editSelection == #newClass ifTrue: [^ self packageInfo: self selectedPackage].
	self selectedClass isNil ifTrue: [^ ''].
	theClass := Smalltalk at: self selectedClass name asSymbol ifAbsent: [].
	editSelection == #editClass ifTrue:
		[^ theClass notNil
			ifTrue: ['Class exists already in the system' translated]
			ifFalse: ['New class' translated]].
	editSelection == #editMessage ifFalse: [^ ''].
	(theClass notNil and: [self metaClassIndicated])
		ifTrue: [theClass := theClass class].

	stamp := self selectedClassOrMetaClass stampAt: self selectedMessageName.
	exists := theClass notNil and: [theClass includesSelector: self selectedMessageName].
	^ stamp = 'methodWasRemoved'
		ifTrue:
			[exists
				ifTrue:
					['Existing method removed  by this change-set' translated]
				ifFalse:
					['Removal request for a method that is not present in this image' translated]]
		ifFalse:
			[stamp, ' · ',
				(exists 
					ifTrue: ['Method already exists' translated , self extraInfo]
					ifFalse: ['New method' translated])]! !

!FileContentsBrowser methodsFor: 'other' stamp: 'fbs 3/3/2011 17:52'!
labelString
	"Answer the string for the window title"

	^ 'File Contents Browser ', (self selectedSystemCategory ifNil: [''])! !

!FileContentsBrowser methodsFor: 'menus' stamp: 'topa 3/16/2015 11:14'!
messageCategoryMenu: aMenu
	^ self menu: aMenu for: #(fileMessageCategoryMenu fileMessageCategoryMenuShifted:)
! !

!FileContentsBrowser methodsFor: 'keys' stamp: 'sma 5/6/2000 18:50'!
messageListKey: aChar from: view
	aChar == $b ifTrue: [^ self browseMethodFull].
	super messageListKey: aChar from: view! !

!FileContentsBrowser methodsFor: 'menus' stamp: 'topa 3/15/2015 23:29'!
messageListMenu: aMenu shifted: aBool

	^ self menu: aMenu for: #(fileMessageListMenu fileMessageListMenuShifted:) shifted: aBool
! !

!FileContentsBrowser methodsFor: 'diffs' stamp: 'sw 5/20/2001 21:03'!
methodDiffFor: aString class: aPseudoClass selector: selector meta: meta 
	"Answer the diff between the current copy of the given class/selector/meta for the string provided"

	| theClass source |
	theClass := Smalltalk
				at: aPseudoClass name
				ifAbsent: [^ aString copy].
	meta
		ifTrue: [theClass := theClass class].
	(theClass includesSelector: selector)
		ifFalse: [^ aString copy].
	source := theClass sourceCodeAt: selector.
	^ Cursor wait
		showWhile: [TextDiffBuilder buildDisplayPatchFrom: source to: aString inClass: theClass prettyDiffs: self showingPrettyDiffs]! !

!FileContentsBrowser methodsFor: 'other' stamp: 'sma 2/6/2000 12:27'!
methodHierarchy
	(self selectedClassOrMetaClass isNil or:
		[self selectedClassOrMetaClass hasDefinition])
			ifFalse: [super methodHierarchy]! !

!FileContentsBrowser methodsFor: 'diffs' stamp: 'nice 12/26/2009 10:05'!
modifiedClassDefinition
	| pClass rClass old new diff |
	pClass := self selectedClassOrMetaClass.
	pClass hasDefinition ifFalse:[^pClass definition].
	rClass := Smalltalk at: self selectedClass name asSymbol ifAbsent:[nil].
	rClass isNil ifTrue:[^pClass definition].
	self metaClassIndicated ifTrue:[ rClass := rClass class].
	old := rClass definition.
	new := pClass definition.
	diff := Cursor wait showWhile:[
		ClassDiffBuilder buildDisplayPatchFrom: old to: new
	].
	^diff! !

!FileContentsBrowser methodsFor: 'infoView' stamp: 'ar 3/5/2010 21:09'!
packageInfo: p
	| nClasses newClasses oldClasses |
	p isNil ifTrue:[^''].
	nClasses := newClasses := oldClasses := 0.
	p classes do:[:cls|
		nClasses := nClasses + 1.
		(Smalltalk hasClassNamed: cls name)
			ifTrue:[oldClasses := oldClasses + 1]
			ifFalse:[newClasses := newClasses + 1]].
	^nClasses printString,' classes (', newClasses printString, ' new / ', oldClasses printString, ' modified)'! !

!FileContentsBrowser methodsFor: 'keys' stamp: 'sma 2/6/2000 12:05'!
packageListKey: aChar from: view
	aChar == $f ifTrue: [^ self findClass].
	self arrowKey: aChar from: view! !

!FileContentsBrowser methodsFor: 'menus' stamp: 'topa 3/16/2015 11:16'!
packageListMenu: aMenu shifted: shifted

	^ self menu: aMenu for: #(filePackageListMenu filePackageListMenuShifted:) shifted: shifted
! !

!FileContentsBrowser methodsFor: 'accessing' stamp: ''!
packages
	^packages! !

!FileContentsBrowser methodsFor: 'accessing' stamp: ''!
packages: aDictionary
	packages := aDictionary.! !

!FileContentsBrowser methodsFor: 'removing' stamp: 'fbs 3/4/2011 20:55'!
removeClass
	| class |
	self hasClassSelected ifFalse: [^ self].
	class := self selectedClass.
	(self confirm:'Are you certain that you
want to delete the class ', class name, '?') ifFalse:[^self].
	self selectedPackage removeClass: class.
	self classListIndex: 0.
	self changed: #classList.! !

!FileContentsBrowser methodsFor: 'removing' stamp: 'fbs 3/17/2011 17:34'!
removeMessage
	| messageName |
	self hasMessageSelected
		ifFalse: [^ self].
	self okToChange
		ifFalse: [^ self].
	messageName := self selectedMessageName.
	(self selectedClass confirmRemovalOf: messageName)
		ifFalse: [^ false].
	self selectedClassOrMetaClass removeMethod: self selectedMessageName.
	self selectMessageNamed: nil.
	self setClassOrganizer.
	"In case organization not cached"
	self changed: #messageList! !

!FileContentsBrowser methodsFor: 'removing' stamp: 'fbs 3/30/2011 07:50'!
removeMessageCategory
	"If a message category is selected, create a Confirmer so the user can 
	verify that the currently selected message category should be removed
 	from the system. If so, remove it."

	| messageCategoryName |
	self hasMessageCategorySelected ifFalse: [^ self].
	self okToChange ifFalse: [^ self].
	messageCategoryName := self selectedMessageCategoryName.
	(self messageList size = 0
		or: [self confirm: 'Are you sure you want to
remove this method category 
and all its methods?']) ifFalse: [^ self].
	self selectedClassOrMetaClass removeCategory: messageCategoryName.
	self messageCategoryListIndex: 0.
	self changed: #messageCategoryList.! !

!FileContentsBrowser methodsFor: 'removing' stamp: 'fbs 3/5/2011 20:57'!
removePackage
	self hasSystemCategorySelected ifTrue: [^ self].
	self okToChange ifFalse: [^ self].
	(self confirm: 'Are you sure you want to
remove this package 
and all its classes?') ifFalse:[^self].
	(systemOrganizer listAtCategoryNamed: self selectedSystemCategory) do:[:el|
		systemOrganizer removeElement: el].
	self packages removeKey: self selectedPackage packageName.
	systemOrganizer removeCategory: self selectedSystemCategory.
	self selectSystemCategory: nil.
	self changed: #systemCategoryList! !

!FileContentsBrowser methodsFor: 'removing' stamp: 'wod 2/3/1999 18:47'!
removeUnmodifiedCategories
	| theClass |
	self okToChange ifFalse: [^self].
	theClass := self selectedClass.
	theClass isNil ifTrue: [^self].
	Cursor wait showWhile:
		[theClass removeUnmodifiedMethods: theClass selectors.
		theClass metaClass removeUnmodifiedMethods: theClass metaClass selectors].
	self messageCategoryListIndex: 0.
	self changed: #messageCategoryList.! !

!FileContentsBrowser methodsFor: 'removing' stamp: 'jmv 3/2/2010 15:59'!
removeUnmodifiedClasses
	| packageList |
	self okToChange ifFalse:[^self].
	packageList := self selectedPackage
						ifNil: [ self packages] 
						ifNotNil: [ Array with: self selectedPackage].
	packageList do:[:package|
		package classes copy do:[:theClass|
			Cursor wait showWhile:[
				theClass removeAllUnmodified.
			].
			theClass hasChanges ifFalse:[
				package removeClass: theClass.
			].
		]].
	self classListIndex: 0.
	self changed: #classList! !

!FileContentsBrowser methodsFor: 'removing' stamp: 'jmv 3/2/2010 15:59'!
removeUnmodifiedMethods
	| theClass cat |
	self okToChange ifFalse:[^self].
	theClass := self selectedClassOrMetaClass.
	theClass ifNil: [ ^self].
	cat := self selectedMessageCategoryName.
	cat ifNil: [ ^self].
	Cursor wait showWhile:[
		theClass removeUnmodifiedMethods: (theClass organization listAtCategoryNamed: cat).
	].
	self messageListIndex: 0.
	self changed: #messageList.! !

!FileContentsBrowser methodsFor: 'class list' stamp: 'fbs 3/4/2011 20:55'!
renameClass
	| oldName newName |
	self hasClassSelected ifFalse: [^ self].
	self okToChange ifFalse: [^ self].
	oldName := self selectedClass name.
	newName := (self request: 'Please type new class name'
						initialAnswer: oldName) asSymbol.
	(newName isEmpty or:[newName = oldName]) ifTrue: [^ self].
	(self selectedPackage classes includesKey: newName)
		ifTrue: [^ self error: newName , ' already exists in the package'].
	systemOrganizer classify: newName under: self selectedSystemCategory.
	systemOrganizer removeElement: oldName.
	self selectedPackage renameClass: self selectedClass to: newName.
	self changed: #classList.
	self classListIndex: ((systemOrganizer listAtCategoryNamed: self selectedSystemCategory) indexOf: newName).
! !

!FileContentsBrowser methodsFor: 'edit pane' stamp: 'md 2/13/2006 14:36'!
selectedBytecodes
	"Compile the source code for the selected message selector and extract and return
	the bytecode listing."
	| class selector |
	class := self selectedClassOrMetaClass.
	selector := self selectedMessageName.
	contents := class sourceCodeAt: selector.
	contents := Compiler new
					parse: contents
					in: class
					notifying: nil.
	contents := contents generate.
	^ contents symbolic asText! !

!FileContentsBrowser methodsFor: 'class list' stamp: ''!
selectedClass
	"Answer the class that is currently selected. Answer nil if no selection 
	exists."

	self selectedClassName == nil ifTrue: [^nil].
	^self selectedPackage classAt: self selectedClassName! !

!FileContentsBrowser methodsFor: 'metaclass' stamp: 'asm 10/6/2003 11:29'!
selectedClassOrMetaClass
	"Answer the selected class or metaclass."

	| cls |
	self metaClassIndicated
		ifTrue: [^ (cls := self selectedClass) ifNotNil: [cls metaClass]]
		ifFalse: [^ self selectedClass]! !

!FileContentsBrowser methodsFor: 'edit pane' stamp: 'fbs 12/29/2013 17:09'!
selectedMessage
	"Answer a copy of the source code for the selected message selector."

	| class selector |
	class := self selectedClassOrMetaClass.
	selector := self selectedMessageName.
	contents := class sourceCodeAt: selector.
	SystemBrowser browseWithPrettyPrint 
		ifTrue: 
			[contents := class prettyPrinterClass 
						format: contents
						in: class
						notifying: nil
						decorated: false].
	self showingAnyKindOfDiffs 
		ifTrue: 
			[contents := self 
						methodDiffFor: contents
						class: self selectedClass
						selector: self selectedMessageName
						meta: self metaClassIndicated].
	^contents asText makeSelectorBoldIn: class! !

!FileContentsBrowser methodsFor: 'accessing' stamp: 'fbs 3/3/2011 17:51'!
selectedPackage
	| cat |
	cat := self selectedSystemCategory.
	cat isNil ifTrue:[^nil].
	^self packages at: cat asString ifAbsent:[nil]! !

!FileContentsBrowser methodsFor: 'metaclass' stamp: 'fbs 3/4/2011 20:55'!
setClassOrganizer
	"Install whatever organization is appropriate"
	| theClass |
	classOrganizer := nil.
	metaClassOrganizer := nil.
	self hasClassSelected ifFalse: [^ self].
	classOrganizer := (theClass := self selectedClass) organization.
	metaClassOrganizer := theClass metaClass organization.
! !

!FileContentsBrowser methodsFor: 'system category list' stamp: 'mt 4/14/2015 09:14'!
systemCategoryList

	^ self packages size = 1
		ifTrue: [super systemCategoryList allButFirst "without all category"]
		ifFalse: [super systemCategoryList]! !

!FileContentsBrowser methodsFor: 'infoView' stamp: 'wod 5/19/1998 17:34'!
updateInfoView

	Smalltalk isMorphic 
		ifTrue: [self changed: #infoViewContents]
		ifFalse: [
			self infoString contents: self infoViewContents.
			self infoString changed].! !

Browser subclass: #HierarchyBrowser
	instanceVariableNames: 'classDisplayList centralClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Browser'!

!HierarchyBrowser commentStamp: 'fbs 3/9/2011 12:02' prior: 0!
I provide facilities to explore classes in the context of their subclass hierarchy.

My classDisplayList instvar uses indentation to show the subclassing relationship between the displayed classes.
!

HierarchyBrowser subclass: #ClassListBrowser
	instanceVariableNames: 'defaultTitle'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Browser'!

!ClassListBrowser commentStamp: '<historical>' prior: 0!
A ClassListBrowser displays the code for an arbitrary list of classes.

ClassListBrowser example1.  "all classes that have the string 'Pluggable' in their names"
ClassListBrowser example2.  "all classes whose names start with the letter S"
ClassListBrowser example3.  "all variable classes"
ClassListBrowser example4.  "all classes with more than 100 methods"
ClassListBrowser example5.  "all classes that lack class comments"
ClassListBrowser example6.  "all classes that have class instance variables"

ClassListBrowser new initForClassesNamed: #(Browser Boolean) title: 'Browser and Boolean!!'.
!

!ClassListBrowser class methodsFor: 'instance creation' stamp: 'sd 4/17/2003 21:21'!
browseClassesSatisfying: classBlock title: aTitle
	"Put up a ClassListBrowser showing all classes that satisfy the classBlock."

	self new
		initForClassesNamed:
			(self systemNavigation allClasses select:
					[:c | (classBlock value: c) == true]
				thenCollect:
					[:c | c name])
		title:
			aTitle! !

!ClassListBrowser class methodsFor: 'examples' stamp: 'ul 8/14/2015 18:01'!
example1
	"Put up a ClassListBrowser that shows all classes that have the string 'Pluggable' in their names"

	self browseClassesSatisfying: [:cl | cl name includesSubstring: 'Pluggable'] title: 'Pluggables'

"ClassListBrowser example1"
	! !

!ClassListBrowser class methodsFor: 'examples' stamp: 'sd 4/17/2003 21:21'!
example2
	"Put up a ClassListBrowser that shows all classes whose names start with 
	the letter S"

	self new
		initForClassesNamed: (self systemNavigation allClasses
				collect: [:c | c name]
				thenSelect: [:aName | aName first == $S])
		title: 'All classes starting with S'
	"ClassListBrowser example2"! !

!ClassListBrowser class methodsFor: 'examples' stamp: 'sw 7/27/2002 16:03'!
example3
	"Put up a ClassListBrowser that shows all Variable classes"

	self browseClassesSatisfying:  [:c | c isVariable] title: 'All Variable classes'

"ClassListBrowser example3"
	! !

!ClassListBrowser class methodsFor: 'examples' stamp: 'sw 7/27/2002 16:04'!
example4
	"Put up a ClassListBrowser that shows all classes implementing more than 100 methods"

	self browseClassesSatisfying:
		[:c | (c selectors size + c class selectors size) > 100] title: 'Classes with more than 100 methods'

"ClassListBrowser example4"
	! !

!ClassListBrowser class methodsFor: 'examples' stamp: 'sw 7/27/2002 14:32'!
example5
	"Put up a ClassListBrowser that shows all classes that lack class comments"

	self
		browseClassesSatisfying: 
			[:c | c organization classComment isEmptyOrNil] 
		title: 'Classes lacking class comments'

"ClassListBrowser example5"
	! !

!ClassListBrowser class methodsFor: 'examples' stamp: 'sw 7/27/2002 14:33'!
example6
	"Put up a ClassListBrowser that shows all classes that have class instance variables"

	self
		browseClassesSatisfying: 
			[:c | c class instVarNames size > 0]
		title:
			'Classes that define class-side instance variables'

"ClassListBrowser example6"! !

!ClassListBrowser methodsFor: 'title' stamp: 'sw 7/18/2002 22:42'!
defaultTitle: aTitle
	"Set the browser's default title"

	defaultTitle := aTitle! !

!ClassListBrowser methodsFor: 'initialization' stamp: 'fbs 3/9/2011 11:42'!
initForClassesNamed: nameList title: aTitle
	"Initialize the receiver for the class-name-list and title provided"

	self systemOrganizer: SystemOrganization.
	metaClassIndicated := false.
	defaultTitle := aTitle.
	classDisplayList := nameList copy.
	self class openBrowserView:  (self openSystemCatEditString: nil)
		label: aTitle

	"ClassListBrowser new initForClassesNamed: #(Browser CategoryViewer) title: 'Frogs'"! !

!ClassListBrowser methodsFor: 'title' stamp: 'sw 7/18/2002 22:43'!
labelString
	"Answer the label strilng to use on the browser"

	^ defaultTitle ifNil: [super labelString]! !

!HierarchyBrowser methodsFor: 'class list' stamp: 'fbs 3/30/2011 08:54'!
assureSelectionsShow
	"This is a workaround for the fact that a hierarchy browser, when launched, often does not show the selected class"

	| saveMsgName saveCatName |
	saveCatName := self selectedMessageCategoryName.
	saveMsgName := self selectedMessageName.
	self selectClassNamed: selectedClassName.
	self selectMessageCategoryNamed: saveCatName.
	self selectMessageNamed: saveMsgName! !

!HierarchyBrowser methodsFor: 'menu messages' stamp: 'tk 4/7/98 13:53'!
buildClassBrowserEditString: aString 
	"Create and schedule a new class browser for the current selection, if one 
	exists, with initial textual contents set to aString."

	self spawnHierarchy! !

!HierarchyBrowser methodsFor: 'toolbuilder' stamp: 'ar 8/10/2009 23:28'!
buildWith: builder
	^builder build: (self openSystemCatEditString: '')! !

!HierarchyBrowser methodsFor: 'class list' stamp: 'fbs 3/9/2011 11:42'!
classList
	classDisplayList := classDisplayList select: [:each | Smalltalk includesKey: each withBlanksTrimmed asSymbol].
	^ classDisplayList! !

!HierarchyBrowser methodsFor: 'initialization' stamp: 'fbs 3/14/2011 17:31'!
classListIndex: newIndex
	"Cause system organization to reflect appropriate category"
	| newClassName ind |
	newIndex ~= 0 ifTrue:
		[newClassName := (classDisplayList at: newIndex) copyWithout: $ .
		selectedSystemCategory := (systemOrganizer categories at:
			(systemOrganizer numberOfCategoryOfElement: newClassName))].
	ind := super classListIndex: newIndex.

	"What I'd like to write:"
	"self selectedClassName ifNotNil:
		[ selectedSystemCategory := self selectedClass category ]."
	self changed: #systemCategorySingleton.
	^ ind! !

!HierarchyBrowser methodsFor: 'class functions' stamp: 'mt 3/16/2016 13:54'!
copyClass

	super copyClass.
	self updateAfterClassChange.! !

!HierarchyBrowser methodsFor: 'initialization' stamp: 'cmm 4/26/2015 17:17'!
defaultBrowserTitle
	^ self selectedSystemCategoryName ifNil: [ 'Hierarchy Browser' ]! !

!HierarchyBrowser methodsFor: 'class functions' stamp: 'mt 3/16/2016 13:54'!
defineClass: defString notifying: aController  

	super defineClass: defString notifying: aController.
	self updateAfterClassChange.! !

!HierarchyBrowser methodsFor: 'traits' stamp: 'mt 3/16/2016 13:53'!
defineTrait: defString notifying: aController  

	super defineTrait: defString notifying: aController.
	self updateAfterClassChange.! !

!HierarchyBrowser methodsFor: 'initialization' stamp: 'fbs 3/9/2011 11:42'!
initAlphabeticListing
	| tab stab index |
	self systemOrganizer: SystemOrganization.
	metaClassIndicated := false.
	classDisplayList := Smalltalk classNames.! !

!HierarchyBrowser methodsFor: 'initialization' stamp: 'mt 3/16/2016 13:55'!
initHierarchyForClass: aClassOrMetaClass 
	| nonMetaClass superclasses |
	centralClass := aClassOrMetaClass.
	nonMetaClass := aClassOrMetaClass theNonMetaClass.
	self systemOrganizer: SystemOrganization.
	metaClassIndicated := aClassOrMetaClass isMeta.
	classDisplayList := OrderedCollection new.
	(superclasses := nonMetaClass allSuperclasses reversed) withIndexDo:
		[ : each : indent | classDisplayList add:
			(String streamContents:
				[ : stream | indent - 1 timesRepeat: [ stream nextPutAll: '  ' ].
				stream nextPutAll: each name ]) ].
	nonMetaClass
		allSubclassesWithLevelDo:
			[ : eachClass : lvl | classDisplayList add:
				(String streamContents:
					[ : stream | lvl timesRepeat: [ stream nextPutAll: '  ' ].
					stream nextPutAll: eachClass name ]) ]
		startingLevel: superclasses size.
		
	self changed: #classList.
	self selectClass: nonMetaClass! !

!HierarchyBrowser methodsFor: 'multi-window support' stamp: 'eem 7/1/2010 10:30'!
isHierarchy
	^true! !

!HierarchyBrowser methodsFor: 'initialization' stamp: 'tk 4/5/98 10:29'!
openEditString: aString
	"Create a pluggable version of all the views for a HierarchyBrowser, including views and controllers.  The top list view is of the currently selected system class category--a single item list."

	^ self openSystemCatEditString: aString! !

!HierarchyBrowser methodsFor: 'toolbuilder' stamp: 'cmm 4/26/2015 17:25'!
openSystemCatEditString: aString
	"Create a pluggable version of all the views for a Browser, including views and controllers.  The top list view is of the currently selected system class category--a single item list."
	"Example:
		Browser new browseAllClasses.
	"
	| builder catPaneHeight max |
	catPaneHeight := Preferences standardListFont height + 5 "top margin/border" + 5 "bottom margin/border".
	builder := ToolBuilder default.
	max := self wantsOptionalButtons ifTrue:[0.32] ifFalse:[0.4].
	^self buildWindowWith: builder specs: {
		(self classListFrame: max fromTop: 0 fromLeft: 0 width: 0.333) -> [self buildClassListWith: builder].
		(self switchesFrame: max fromLeft: 0 width: 0.333) -> [self buildSwitchesWith: builder].
		(LayoutFrame fractions: (0.333@0 corner: 0.666@max) offsets: (0@0 corner: 0@0)) -> [self buildMessageCategoryListWith: builder].
		(LayoutFrame fractions: (0.666@0 corner: 1@max) offsets: (0@0 corner: 0@0)) -> [self buildMessageListWith: builder].
		(0@max corner: 1@1) -> [self buildCodePaneWith: builder].
	}! !

!HierarchyBrowser methodsFor: 'morphic ui' stamp: 'cmm 5/12/2011 18:03'!
postAcceptBrowseFor: aHierarchyBrowser 
	(aHierarchyBrowser selectedClass ~= self selectedClass or: [ aHierarchyBrowser selectedMessageName notNil ]) ifTrue: [ self selectMessageCategoryNamed: nil ].
	aHierarchyBrowser instanceMessagesIndicated
		ifTrue: [ self indicateInstanceMessages ]
		ifFalse: [ self indicateClassMessages ].
	self
		 selectClass: aHierarchyBrowser selectedClass ;
		 selectedMessageName: aHierarchyBrowser selectedMessageName ;
		 showHomeCategory! !

!HierarchyBrowser methodsFor: 'menu messages' stamp: 'tk 4/3/98 11:22'!
removeSystemCategory
	"If a class category is selected, create a Confirmer so the user can 
	verify that the currently selected class category and all of its classes
 	should be removed from the system. If so, remove it."

	self inform: 'Use a normal Browser, in which you can see 
the entire category you are trying to remove.'! !

!HierarchyBrowser methodsFor: 'class functions' stamp: 'mt 3/16/2016 13:52'!
renameClass

	super renameClass.
	self updateAfterClassChange.! !

!HierarchyBrowser methodsFor: 'morphic ui' stamp: 'fbs 3/9/2011 11:43'!
representsSameBrowseeAs: anotherModel
	^ self hasUnacceptedEdits not
		and: [ classDisplayList size = anotherModel classList size
		and: [ classDisplayList includesAllOf: anotherModel classList ] ]! !

!HierarchyBrowser methodsFor: 'initialization' stamp: 'fbs 3/16/2011 10:51'!
selectClassNamed: aSymbolOrString
	| newClassName |
	newClassName := aSymbolOrString ifNotNil: [ aSymbolOrString asSymbol ].
	selectedSystemCategory := (systemOrganizer categories at:
			(systemOrganizer numberOfCategoryOfElement: newClassName) ifAbsent: [ nil ]).
			
	super selectClassNamed: newClassName.
	self changed: #systemCategorySingleton.	
	
	^ newClassName.! !

!HierarchyBrowser methodsFor: 'initialization' stamp: 'fbs 3/5/2011 22:31'!
selectedClassName
	"Answer the name of the class currently selected.   di
	  bug fix for the case where name cannot be found -- return nil rather than halt"

	| aName |
	aName := super selectedClassName.
	aName ifNil: [ ^ nil ].
	^ (aName copyWithout: Character space) asSymbol! !

!HierarchyBrowser methodsFor: 'initialization' stamp: 'mt 1/22/2016 13:38'!
setClass: aClass

	self initHierarchyForClass: aClass.
	super setClass: aClass.! !

!HierarchyBrowser methodsFor: 'initialization' stamp: 'di 4/26/2000 20:20'!
systemCategorySingleton

	| cls |
	cls := self selectedClass.
	^ cls ifNil: [Array new]
		ifNotNil: [Array with: cls category]! !

!HierarchyBrowser methodsFor: 'menu messages' stamp: 'sw 11/8/1999 13:35'!
systemCatSingletonKey: aChar from: aView
	^ self systemCatListKey: aChar from: aView! !

!HierarchyBrowser methodsFor: 'menu messages' stamp: 'sw 11/8/1999 14:08'!
systemCatSingletonMenu: aMenu

	^ aMenu labels:
'find class... (f)
browse
printOut
fileOut
update
rename...
remove' 
	lines: #(1 4)
	selections:
		#(findClass buildSystemCategoryBrowser
		printOutSystemCategory fileOutSystemCategory updateSystemCategories
		 renameSystemCategory removeSystemCategory )
! !

!HierarchyBrowser methodsFor: 'initialization' stamp: 'rhi 12/2/2001 21:32'!
updateAfterClassChange
	"It is possible that some the classes comprising the hierarchy have changed, so reinitialize the entire browser."

	(centralClass notNil and: [centralClass isObsolete not])
		ifTrue: [self initHierarchyForClass: centralClass]! !

Browser subclass: #PackagePaneBrowser
	instanceVariableNames: 'package packageListIndex packageList'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Browser'!

!PackagePaneBrowser commentStamp: '<historical>' prior: 0!
A package browser represents a hierarchical query path through an organization of class and method information.   It parses class categories into a two-level hierarchy on the first '-' character, giving "packages" (e.g.,  Magnitude, Collections, Graphics, etc.), and "categories" (e.g., Magnitude-General and Magnitude-Number).

Instance Variables:
	package  <Symbol> the "category header," e.g., #Magnitudes or #Collections
	packageListIndex <Integer> The index in the package list
	packageList  <OrderedCollection of String> the list of package names
!

!PackagePaneBrowser class methodsFor: 'instance creation' stamp: 'ar 8/7/2009 21:04'!
prototypicalToolWindow
	"Answer an example of myself seen in a tool window, for the benefit of parts-launching tools"

	| aWindow |
	aWindow := self new openEditString: nil.
	^ ToolBuilder build: aWindow! !

!PackagePaneBrowser class methodsFor: 'class initialization' stamp: 'hpt 8/5/2004 20:12'!
registerInAppRegistry
	"Register the receiver in the SystemBrowser AppRegistry"
	SystemBrowser register: self.! !

!PackagePaneBrowser methodsFor: 'initialize-release' stamp: 'ar 8/8/2009 17:10'!
buildPackageListWith: builder
	| listSpec |
	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #packageList; 
		getIndex: #packageListIndex; 
		setIndex: #packageListIndex:; 
		menu: #packageMenu:; 
		keyPress: #packageListKey:from:.
	^listSpec
! !

!PackagePaneBrowser methodsFor: 'package list' stamp: 'JF 7/30/2003 12:35'!
categoryExistsForPackage
	^ self hasPackageSelected
		and: [(systemOrganizer categories indexOf: self package asSymbol) ~= 0]
! !

!PackagePaneBrowser methodsFor: 'dragNDrop' stamp: 'mir 5/29/2000 11:54'!
changeCategoryForClass: class srcSystemCategory: srcSystemCategorySel atListMorph: dstListMorph internal: internal copy: copyFlag 
	"only move semantic"
	| newClassCategory success |
	self flag: #stringSymbolProblem.
	success := copyFlag not ifFalse: [^ false].
	newClassCategory := self dstCategoryDstListMorph: dstListMorph internal: internal.
	(success := newClassCategory notNil & (newClassCategory ~= class category))
		ifTrue: 
			[class category: newClassCategory.
			self changed: #classList.
			internal ifFalse: [self selectClass: class]].
	^ success! !

!PackagePaneBrowser methodsFor: 'class list' stamp: 'fbs 3/3/2011 17:51'!
classList
	"Answer an array of the class names of the selected category. Answer an 
	empty array if no selection exists."

	^ self hasSystemCategorySelected 
		ifFalse: [self packageClasses]
		ifTrue: [systemOrganizer listAtCategoryNamed: self selectedSystemCategory]! !

!PackagePaneBrowser methodsFor: 'initialize-release' stamp: 'sw 1/13/2000 16:45'!
defaultBrowserTitle
	^ 'Package Browser'! !

!PackagePaneBrowser methodsFor: 'user interface' stamp: 'mt 7/9/2016 08:52'!
defaultWindowColor
	^ (Color r: 0.9 g: 0.9 b: 0.63)! !

!PackagePaneBrowser methodsFor: 'dragNDrop util' stamp: 'fbs 3/3/2011 17:50'!
dstCategoryDstListMorph: dstListMorph internal: internal 
	| dropItem |
	^ internal & (dstListMorph getListSelector == #systemCategoryList)
		ifTrue: [(dropItem := dstListMorph potentialDropItem) ifNotNil: [(self package , '-' , dropItem) asSymbol]]
		ifFalse: [self selectedSystemCategory]! !

!PackagePaneBrowser methodsFor: 'package list' stamp: 'JF 7/30/2003 12:24'!
hasPackageSelected

	^ packageListIndex ~= 0! !

!PackagePaneBrowser methodsFor: 'system category list' stamp: 'cao 12/12/2011 19:20'!
hasSystemCategorySelected
	^ super selectedSystemCategory notNil.! !

!PackagePaneBrowser methodsFor: 'multi-window support' stamp: 'eem 7/1/2010 10:31'!
isPackage
	^true! !

!PackagePaneBrowser methodsFor: 'initialize-release' stamp: 'mt 5/1/2015 14:08'!
labelString
	^self package
		ifNil: [super labelString]
		ifNotNil:
			[:pkg| | label |
			label := self defaultBrowserTitle, ': ', pkg, (self selectedClass
														ifNil: ['']
														ifNotNil: [' ', self selectedClass printString]).
			(self multiWindowState notNil
			 and: [self multiWindowState models size > 1]) ifTrue:
				[label := (self multiWindowState models indexOf: self) printString, '. ', label].
			label]! !

!PackagePaneBrowser methodsFor: 'package list' stamp: 'topa 3/16/2015 11:20'!
mainPackageMenu: aMenu
	"Answer a Menu of operations on class packages to be 
	displayed when the operate menu button is pressed."
	<packageListMenu>
	^aMenu addList: #(
			('find class...'		findClass)
			('recent classes...'	recent)
			-
			('reorganize'		editSystemCategories)
			('update'			updatePackages));
		yourself.
! !

!PackagePaneBrowser methodsFor: 'multi-window support' stamp: 'fbs 3/3/2011 17:50'!
multiWindowName
	"Answer the string to display for the receiver in a multi-window."
	^String streamContents:
		[:s| | str |
		self package
			ifNil: [s nextPut: $a; space; nextPutAll: self defaultBrowserTitle]
			ifNotNil:
				[:pkg|
				 self selectedClass
					ifNil: [self selectedSystemCategory
							ifNil: [s nextPutAll: pkg]
							ifNotNil: [:cat| s nextPutAll: cat]]
					ifNotNil:
						[:class|
						 s nextPutAll: pkg; space; print: class.
						 self metaClassIndicated ifTrue:
							[s nextPutAll: ' class']]].
		(str := self selectedMessageName) notNil
			ifTrue: [s nextPutAll: '>>'; nextPutAll: str]
			ifFalse:
				[(str := self selectedMessageCategoryName) notNil
					ifTrue: [s space; nextPut: ${; nextPutAll: str; nextPut: $}]]]! !

!PackagePaneBrowser methodsFor: 'multi-window support' stamp: 'eem 8/11/2010 13:31'!
multiWindowNameForState: savedStateMessage
	"Answer the string to display for the receiver in a multi-window."
	| getarg |
	getarg := [:keyword| savedStateMessage arguments at: (savedStateMessage selector keywords indexOf: keyword)].
	^String streamContents:
		[:s|
		(getarg value: 'restoreToPackage:')
			ifNil: [s nextPut: $a; space; nextPutAll: self defaultBrowserTitle]
			ifNotNil:
				[:pkg|
				 s nextPutAll: pkg.
				(getarg value: 'className:')
					ifNil: [(getarg value: 'category:') ifNotNil:
							[:categoryName| s nextPut: $-; nextPutAll: categoryName]]
					ifNotNil:
						[:className|
						s space; nextPutAll: className.
						(getarg value: 'meta:') ifTrue:
							[s nextPutAll: ' class'].
						  self isHierarchy ifTrue:
							[s space; nextPutAll: ' Hierarchy'].
						(getarg value: 'selector:')
							ifNil: [(getarg value: 'protocol:') ifNotNil:
									[:protocol| s space; nextPut: ${; nextPutAll: protocol; nextPut: $}]]
							ifNotNil: [:selector| s nextPutAll: '>>'; nextPutAll: selector]]]]! !

!PackagePaneBrowser methodsFor: 'initialize-release' stamp: 'mt 4/30/2015 11:41'!
openEditString: aString
        "Create a pluggable version of all the views for a Browser, including views and controllers."
	"Example: 
		PackagePaneBrowser fullOnClass: Browser.
	"
	| builder max |
	builder := ToolBuilder default.
	max := self wantsOptionalButtons ifTrue:[0.42] ifFalse:[0.5].
	^self buildWindowWith: builder specs: {
		(0@0 corner: 0.15@max) -> [self buildPackageListWith: builder].
		(0.15@0 corner: 0.35@max) -> [self buildSystemCategoryListWith: builder].
		(self classListFrame: max fromLeft: 0.35 width: 0.25) -> [self buildClassListWith: builder].
		(self switchesFrame: max fromLeft: 0.35 width: 0.25) -> [self buildSwitchesWith: builder].
		(0.6@0 corner: 0.75@max) -> [self buildMessageCategoryListWith: builder].
		(0.75@0 corner: 1@max) -> [self buildMessageListWith: builder].
		(0@max corner: 1@1) -> [self buildCodePaneWith: builder].
	}! !

!PackagePaneBrowser methodsFor: 'package list' stamp: 'JF 7/30/2003 12:25'!
package
	"Answer the receiver's 'package'."

	^ self hasPackageSelected
		ifFalse: [nil]
		ifTrue: [self packageList at: packageListIndex]
! !

!PackagePaneBrowser methodsFor: 'class list' stamp: 'JF 7/30/2003 12:36'!
packageClasses
	^ self categoryExistsForPackage
		ifFalse: [Array new]
		ifTrue:
			[systemOrganizer listAtCategoryNumber:
				(systemOrganizer categories indexOf: self package asSymbol)]! !

!PackagePaneBrowser methodsFor: 'package list' stamp: 'nice 12/27/2009 03:10'!
packageList
	"Answer a list of the packages in the current system organization."

	| str stream |
	str := Set new: 100.
	stream := WriteStream on: (Array new: 100).
	systemOrganizer categories do:
		[ :categ | | cats | 
		cats := categ asString copyUpTo: $-.
		(str includes: cats) ifFalse: 
			[str add: cats.
			stream nextPut: cats]].
	^stream contents! !

!PackagePaneBrowser methodsFor: 'package list' stamp: 'stp 10/05/1998 19:48'!
packageListIndex
	"Answer the index of the current package selection."

	^packageListIndex! !

!PackagePaneBrowser methodsFor: 'package list' stamp: 'fbs 3/30/2011 21:52'!
packageListIndex: anInteger 
	"Set anInteger to be the index of the current package selection."

	packageListIndex := anInteger.
	anInteger = 0
		ifFalse: [package := self packageList at: packageListIndex].
	selectedMessageCategoryName := nil.
	self selectSystemCategory: nil.
	selectedMessageName := nil.
	selectedClassName := nil.
	self setClassOrganizer.
	self changed: #packageSelectionChanged.
	self changed: #packageListIndex.	"update my selection"
	self changed: #systemCategoryList.	"update the category list"
	self selectSystemCategory: nil.	"update category list selection"
! !

!PackagePaneBrowser methodsFor: 'package list' stamp: 'fbs 12/12/2012 15:43'!
packageListKey: aChar from: view
	aChar == $f ifTrue: [^ self findClass].
	^ self classListKey: aChar from: view! !

!PackagePaneBrowser methodsFor: 'package list' stamp: 'topa 3/16/2015 11:20'!
packageMenu: aMenu

	^ self menu: aMenu for: #(packageListMenu packageListMenuShifted:)! !

!PackagePaneBrowser methodsFor: 'multi-window support' stamp: 'eem 7/6/2010 12:24'!
restoreToPackage: packageNameOrNil category: category className: className protocol: protocol selector: selector mode: editMode meta: metaBool
	self packageListIndex: (self packageList indexOf: packageNameOrNil).
	super restoreToCategory: category className: className protocol: protocol selector: selector mode: editMode meta: metaBool! !

!PackagePaneBrowser methodsFor: 'multi-window support' stamp: 'fbs 3/3/2011 17:33'!
saveMultiWindowState
	^Message
		selector: #restoreToPackage:category:className:protocol:selector:mode:meta:
		arguments: {	self package.
						self selectedSystemCategory.
						self selectedClassName.
						self selectedMessageCategoryName.
						self selectedMessageName.
						self editSelection.
						self metaClassIndicated }! !

!PackagePaneBrowser methodsFor: 'system category list' stamp: 'fbs 3/5/2011 20:57'!
selectCategoryForClass: theClass
	"Set the package and category lists to display the given class."

	| cat catName |
	cat := theClass category.
	self packageListIndex: (self packageList indexOf: (cat copyUpTo: $-)).
	catName := (cat copyFrom: ((cat indexOf: $- ifAbsent: [0]) + 1) to: cat size).
	self selectSystemCategory: catName.! !

!PackagePaneBrowser methodsFor: 'class list' stamp: 'md 3/3/2006 11:04'!
selectedClass
	"Answer the class that is currently selected. Answer nil if no selection 
	exists."

	| name envt |
	(name := self selectedClassName) ifNil: [^ nil].
	"(envt := self selectedEnvironment) ifNil: [^ nil]."
	envt:= Smalltalk.
	^ envt at: name! !

!PackagePaneBrowser methodsFor: 'system category list' stamp: 'cao 12/12/2011 19:21'!
selectedSystemCategory
	"Answer the name of the selected system category or nil."

	self hasSystemCategorySelected
		ifFalse: [^nil].
	packageListIndex = 0
		ifTrue: [^ super selectedSystemCategory ].
	^ self package , '-' , super selectedSystemCategory! !

!PackagePaneBrowser methodsFor: 'system category list' stamp: 'di 12/16/1999 16:14'!
systemCategoryList
	"Answer the sequenceable collection containing the class categories that 
	the receiver accesses."

	| prefix |
	packageListIndex = 0 ifTrue: [^ systemOrganizer categories].
	prefix := self package, '-'.
	^ Array streamContents:
		[:strm |
		systemOrganizer categories do: 
			[ :cat | (cat beginsWith: prefix) ifTrue:
				[strm nextPut: (cat copyFrom: prefix size + 1 to: cat size)]]]! !

!PackagePaneBrowser methodsFor: 'system category list' stamp: 'cao 12/12/2011 19:51'!
systemCategoryListIndex
	"Answer the index of the selected class category."

	^ self systemCategoryList indexOf: super selectedSystemCategory.! !

!PackagePaneBrowser methodsFor: 'initialize-release' stamp: 'stp 10/06/1998 22:02'!
systemOrganizer: aSystemOrganizer 
	"Initialize the receiver as a perspective on the system organizer, 
	aSystemOrganizer. Typically there is only one--the system variable 
	SystemOrganization."

	super systemOrganizer: aSystemOrganizer .
	packageListIndex := 0! !

!PackagePaneBrowser methodsFor: 'package list' stamp: 'nk 2/14/2004 15:09'!
updatePackages
	"Update the contents of the package list."

	self editSelection: #none.
	self changed: #packageList.
	self changed: #package.
	self packageListIndex: 0 ! !

CodeHolder subclass: #ChangeList
	instanceVariableNames: 'changeList list listIndex listSelections file lostMethodPointer showsVersions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Changes'!

!ChangeList commentStamp: '<historical>' prior: 0!
A ChangeList represents a list of changed methods that reside on a file in fileOut format.  The classes and methods in my list are not necessarily in this image!!  Used as the model for both Version Lists and Changed Methods (in Screen Menu, Changes...).  Note that the two kinds of window have different controller classes!!!!

It holds three lists:
	changeList - a list of ChangeRecords
	list - a list of one-line printable headers
	listSelections - a list of Booleans (true = selected, false = not selected) multiple OK.
	listIndex 
Items that are removed (removeDoits, remove an item) are removed from all three lists.
Most recently clicked item is the one showing in the bottom pane.!

ChangeList subclass: #ChangeListForProjects
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Changes'!

!ChangeListForProjects commentStamp: '<historical>' prior: 0!
A ChangeList that looks at the changes in a revokable project.  This class has no users at present.!

!ChangeListForProjects methodsFor: 'contents' stamp: 'sw 9/5/2001 15:25'!
contents
	^ self showingAnyKindOfDiffs
		ifFalse: [self undiffedContents]
		ifTrue: [self currentDiffedFromContents]
			"Current is writing over one in list.  Show how I would change it"! !

!ChangeListForProjects methodsFor: 'contents' stamp: 'sw 5/19/2001 11:06'!
currentDiffedFromContents
	"Answer the current in-memory method diffed from the current contents"

	| aChange aClass |
	listIndex = 0
		ifTrue: [^ ''].
	aChange := changeList at: listIndex.
	^ ((aChange type == #method
				and: [(aClass := aChange methodClass) notNil])
			and: [aClass includesSelector: aChange methodSelector])
		ifTrue: [TextDiffBuilder
				buildDisplayPatchFrom: aChange text
				to: (aClass sourceCodeAt: aChange methodSelector)
				inClass: aClass
				prettyDiffs: self showingPrettyDiffs]
		ifFalse: [(changeList at: listIndex) text]! !

!ChangeList class methodsFor: 'fileIn/Out' stamp: 'md 10/22/2003 16:13'!
browseChangesFile: fullName
	"Browse the selected file in fileIn format."

	fullName
		ifNotNil:
			[ChangeList browseStream: (FileStream readOnlyFileNamed:  fullName)]
		ifNil:
			[Beeper beep]! !

!ChangeList class methodsFor: 'fileIn/Out' stamp: 'nice 12/27/2009 03:11'!
browseCompressedChangesFile: fullName 
	"Browse the selected file in fileIn format."

	| unzipped stream |
	fullName ifNil: [^Beeper beep].
	stream := FileStream readOnlyFileNamed: fullName.
	[ | zipped |stream converter: Latin1TextConverter new.
	zipped := GZipReadStream on: stream.
	unzipped := zipped contents asString]
		ensure: [stream close].
	stream := (MultiByteBinaryOrTextStream with: unzipped) reset.
	ChangeList browseStream: stream! !

!ChangeList class methodsFor: 'public access' stamp: 'di 1/18/2001 15:30'!
browseFile: fileName    "ChangeList browseFile: 'AutoDeclareFix.st'"
	"Opens a changeList on the file named fileName"

	^ self browseStream: (FileStream readOnlyFileNamed: fileName)! !

!ChangeList class methodsFor: 'public access' stamp: 'fbs 7/25/2013 07:07'!
browseRecentLog
	"ChangeList browseRecentLog"
	"Prompt with a menu of how far back to go to browse the current image's changes log file"
	^ self
		browseRecentLogOn: (SourceFiles at: 2)
		startingFrom: Smalltalk lastQuitLogPosition! !

!ChangeList class methodsFor: 'public access' stamp: 'nb 6/17/2003 12:25'!
browseRecentLogOnPath: fullName 
	"figure out where the last snapshot or quit was, then browse the recent  entries."

	fullName
		ifNotNil:
			[self browseRecentLogOn: (FileStream readOnlyFileNamed: fullName)]
		ifNil:
			[Beeper beep]
	! !

!ChangeList class methodsFor: 'public access' stamp: 'nk 7/8/2003 13:56'!
browseRecentLogOn: origChangesFile 
	"figure out where the last snapshot or quit was, then browse the recent entries."

	| end done block pos chunk changesFile positions prevBlock |
	changesFile := origChangesFile readOnlyCopy.
	positions := SortedCollection new.
	end := changesFile size.
	prevBlock := end.
	block := end - 1024 max: 0.
	done := false.
	[done
		or: [positions size > 0]]
		whileFalse: [changesFile position: block.
			"ignore first fragment"
			changesFile nextChunk.
			[changesFile position < prevBlock]
				whileTrue: [pos := changesFile position.
					chunk := changesFile nextChunk.
					((chunk indexOfSubCollection: '----' startingAt: 1) = 1) ifTrue: [
						({ '----QUIT'. '----SNAPSHOT' } anySatisfy: [ :str |
							chunk beginsWith: str ])
								ifTrue: [positions add: pos]]].
			block = 0
				ifTrue: [done := true]
				ifFalse: [prevBlock := block.
					block := block - 1024 max: 0]].
	changesFile close.
	positions isEmpty
		ifTrue: [self inform: 'File ' , changesFile name , ' does not appear to be a changes file']
		ifFalse: [self browseRecentLogOn: origChangesFile startingFrom: positions last]! !

!ChangeList class methodsFor: 'public access' stamp: 'cmm 11/7/2013 15:25'!
browseRecentLogOn: origChangesFile startingFrom: initialPos 
	"Prompt with a menu of how far back to go when browsing a changes file."

	| end banners positions pos chunk i changesFile |
	changesFile := origChangesFile readOnlyCopy.
	banners := OrderedCollection new.
	positions := OrderedCollection new.
	end := changesFile size.
	changesFile setConverterForCode.
	pos := initialPos.
	[pos = 0
		or: [banners size > 500]]
		whileFalse: [changesFile position: pos.
			chunk := changesFile nextChunk.
			i := chunk indexOfSubCollection: 'priorSource: ' startingAt: 1.
			i > 0
				ifTrue: [positions addLast: pos.
					banners
						addLast: (chunk copyFrom: 5 to: i - 2).
					pos := Number
								readFrom: (chunk copyFrom: i + 13 to: chunk size)]
				ifFalse: [pos := 0]].
	changesFile close.
	banners size = 0 ifTrue: [^ self inform: 
'this image has never been saved
since changes were compressed' translated].
	pos := UIManager default chooseFrom:  banners values: positions title: 'Browse as far back as...' translated.
	pos ifNil: [^ self].
	self browseRecent: end - pos on: origChangesFile! !

!ChangeList class methodsFor: 'public access' stamp: 'HK 4/18/2002 15:02'!
browseRecent: charCount 
	"ChangeList browseRecent: 5000"
	"Opens a changeList on the end of the changes log file"
	^ self browseRecent: charCount on: (SourceFiles at: 2) ! !

!ChangeList class methodsFor: 'public access' stamp: 'nice 1/19/2010 01:52'!
browseRecent: charCount on: origChangesFile 
	"Opens a changeList on the end of the specified changes log file"
	| changeList end changesFile |
	changesFile := origChangesFile readOnlyCopy.
	changesFile setConverterForCode.
	end := changesFile size.
	changeList := Cursor read
		showWhile: [self new
						scanFile: changesFile
						from: (0 max: end - charCount)
						to: end].
	changesFile close.
	self
		open: changeList
		name: 'Recent changes'
		multiSelect: true! !

!ChangeList class methodsFor: 'public access' stamp: 'nice 1/19/2010 01:52'!
browseStream: changesFile
	"Opens a changeList on a fileStream"
	| changeList charCount |
	changesFile readOnly.
	changesFile setConverterForCode.
	charCount := changesFile size.
	charCount > 1000000 ifTrue:
		[(self confirm: 'The file ', changesFile name , '
is really long (' , charCount printString , ' characters).
Would you prefer to view only the last million characters?')
			ifTrue: [charCount := 1000000]].
	"changesFile setEncoderForSourceCodeNamed: changesFile name."
	changeList := Cursor read showWhile:
		[self new
			scanFile: changesFile from: changesFile size-charCount to: changesFile size].
	changesFile close.
	self open: changeList name: changesFile localName , ' log' multiSelect: true! !

!ChangeList class methodsFor: 'fileIn/Out' stamp: 'nk 7/16/2003 15:48'!
fileReaderServicesForFile: fullName suffix: suffix
	| services |
	services := OrderedCollection new.
	(FileStream isSourceFileSuffix: suffix) | (suffix = '*')
		ifTrue: [ services add: self serviceBrowseChangeFile ].
	(suffix = 'changes') | (suffix = '*')
		ifTrue: [ services add: self serviceBrowseDotChangesFile ].
	(fullName asLowercase endsWith: '.cs.gz') | (suffix = '*')
		ifTrue: [ services add: self serviceBrowseCompressedChangeFile ].
	^services! !

!ChangeList class methodsFor: 'public access' stamp: 'fbs 7/25/2013 07:07'!
getRecentLocatorWithPrompt: aPrompt
	"Prompt with a menu of how far back to go.  Return nil if user backs out.  Otherwise return the number of characters back from the end of the .changes file the user wishes to include"
	 "ChangeList getRecentPosition"
	| end changesFile banners positions pos chunk i |
	changesFile := (SourceFiles at: 2) readOnlyCopy.
	banners := OrderedCollection new.
	positions := OrderedCollection new.
	end := changesFile size.
	pos := Smalltalk lastQuitLogPosition.
	[pos = 0 or: [banners size > 20]] whileFalse:
		[changesFile position: pos.
		chunk := changesFile nextChunk.
		i := chunk indexOfSubCollection: 'priorSource: ' startingAt: 1.
		i > 0 ifTrue: [positions addLast: pos.
					banners addLast: (chunk copyFrom: 5 to: i-2).
					pos := Number readFrom: (chunk copyFrom: i+13 to: chunk size)]
			ifFalse: [pos := 0]].
	changesFile close.
	pos := UIManager default chooseFrom: banners values: positions title: aPrompt.
	pos == nil ifTrue: [^ nil].
	^ end - pos! !

!ChangeList class methodsFor: 'initialize-release' stamp: 'hg 8/3/2000 18:14'!
initialize

	FileList registerFileReader: self! !

!ChangeList class methodsFor: 'instance creation' stamp: 'ar 8/7/2009 22:17'!
open: aChangeList name: aString multiSelect: multiSelect
	"Create a standard system view for the messageSet, whose label is aString.
	The listView may be either single or multiple selection type"
	^ToolBuilder default open: aChangeList label: aString! !

!ChangeList class methodsFor: 'fileIn/Out' stamp: 'KR 3/21/2008 21:07'!
serviceBrowseChangeFile
	"Answer a service for opening a changelist browser on a file"

	^ (SimpleServiceEntry 
		provider: self 
		label: 'changelist browser' translatedNoop
		selector: #browseStream:
		description: 'open a changelist tool on this file' translatedNoop
		buttonLabel: 'changes' translatedNoop)
		argumentGetter: [ :fileList | fileList readOnlyStream ]! !

!ChangeList class methodsFor: 'fileIn/Out' stamp: 'KR 3/21/2008 21:06'!
serviceBrowseCompressedChangeFile
	"Answer a service for opening a changelist browser on a file"

	^ SimpleServiceEntry 
		provider: self 
		label: 'changelist browser' translatedNoop
		selector: #browseCompressedChangesFile:
		description: 'open a changelist tool on this file' translatedNoop
		buttonLabel: 'changes' translatedNoop! !

!ChangeList class methodsFor: 'fileIn/Out' stamp: 'KR 3/21/2008 21:05'!
serviceBrowseDotChangesFile
	"Answer a service for opening a changelist browser on the tail end of a .changes file"

	^ SimpleServiceEntry 
		provider: self 
		label: 'recent changes in file' translatedNoop
		selector: #browseRecentLogOnPath:
		description: 'open a changelist tool on recent changes in file' translatedNoop
		buttonLabel: 'recent changes' translatedNoop! !

!ChangeList class methodsFor: 'fileIn/Out' stamp: 'nk 12/13/2002 12:04'!
services
	"Answer potential file services associated with this class"

	^ { self serviceBrowseChangeFile. 
		self serviceBrowseDotChangesFile.
		self serviceBrowseCompressedChangeFile }! !

!ChangeList class methodsFor: 'class initialization' stamp: 'SD 11/15/2001 22:21'!
unload

	FileList unregisterFileReader: self ! !

!ChangeList methodsFor: 'menu actions' stamp: 'jm 5/3/1998 19:15'!
acceptFrom: aView

	aView controller text = aView controller initialText ifFalse: [
		aView flash.
		^ self inform: 'You can only accept this version as-is.
If you want to edit, copy the text to a browser'].
	(aView setText: aView controller text from: self) ifTrue:
		[aView ifNotNil: [aView controller accept]].	"initialText"
! !

!ChangeList methodsFor: 'initialization-release' stamp: 'sd 11/20/2005 21:26'!
addItem: item text: text
	| cr |
	cr := Character cr.
	changeList addLast: item.
	list addLast: (text collect: [:x | x = cr ifTrue: [$/] ifFalse: [x]])! !

!ChangeList methodsFor: 'viewing access' stamp: 'sw 6/18/2001 10:44'!
annotation
	"Answer the string to be shown in an annotation pane.  Make plain that the annotation is associated with the current in-image version of the code, not of the selected disk-based version, and if the corresponding method is missing from the in-image version, mention that fact."

	| annot aChange aClass |

	annot := super annotation.
	annot asString = '------' ifTrue: [^ annot].

	^ ((aChange := self currentChange) notNil and: [aChange methodSelector notNil])
		ifFalse:
			[annot]
		ifTrue:
			[((aClass := aChange methodClass) isNil or: [(aClass includesSelector: aChange methodSelector) not])
				ifTrue:
					[aChange methodClassName, ' >> ', aChange methodSelector, ' is not present in the current image.']
				ifFalse:
					['current version: ', annot]]! !

!ChangeList methodsFor: 'menu actions' stamp: 'ul 11/15/2010 11:31'!
browseAllVersionsOfSelections
	"Opens a Versions browser on all the currently selected methods, showing each alongside all of their historical versions."
	|  oldSelection aList |
	oldSelection := self listIndex.
	aList := OrderedCollection new.
	Cursor read showWhile: [
		1 to: changeList size do: [:i |
			(listSelections at: i) ifTrue: [
				listIndex := i.
				self browseVersions.
				aList add: i.
				]]].
	listIndex := oldSelection.

	aList size = 0 ifTrue: [^ self inform: 'no selected methods have in-memory counterparts'].
! !

!ChangeList methodsFor: 'menu actions' stamp: 'fbs 5/21/2013 22:27'!
browseCurrentVersionsOfSelections
	"Opens a message-list browser on the current in-memory versions of all methods that are currently seleted"
	| aList |
	aList := OrderedCollection new.
	Cursor read showWhile: [
		1 to: changeList size do: [:i |
			(listSelections at: i) ifTrue: [
				| aClass aChange |
				aChange := changeList at: i.
				(aChange type = #method
					and: [(aClass := aChange methodClass) notNil
					and: [aClass includesSelector: aChange methodSelector]])
						ifTrue: [
							aList add: (
								MethodReference class: aClass  
									selector: aChange methodSelector
							)
						]]]].

	aList size = 0 ifTrue: [^ self inform: 'no selected methods have in-memory counterparts'].
	ToolSet
		browseMessageSet: aList
		name: 'Current versions of selected methods in ', file localName
		autoSelect: nil! !

!ChangeList methodsFor: 'menu actions' stamp: 'nk 1/7/2004 10:23'!
browseVersions
	| change class browser |
	listIndex = 0
		ifTrue: [^ nil ].
	change := changeList at: listIndex.
	((class := change methodClass) notNil
			and: [class includesSelector: change methodSelector])
		ifFalse: [ ^nil ].
	browser := super browseVersions.
	browser ifNotNil: [ browser addedChangeRecord: change ].
	^browser! !

!ChangeList methodsFor: 'toolbuilder' stamp: 'ar 8/8/2009 11:03'!
buildChangeListWith: builder multiSelect: multiSelect

	| listSpec |
	multiSelect ifTrue:[
		listSpec := builder pluggableMultiSelectionListSpec new.
		listSpec getSelectionList: #listSelectionAt:.
		listSpec setSelectionList: #listSelectionAt:put:.
	] ifFalse:[
		listSpec := builder pluggableListSpec new.
	].

	listSpec 
		model: self;
		list: #list; 
		getIndex: #listIndex; 
		setIndex: #toggleListIndex:; 
		menu: (self showsVersions ifTrue: [#versionsMenu:] ifFalse: [#changeListMenu:]); 
		keyPress: #changeListKey:from:.

	^listSpec! !

!ChangeList methodsFor: 'menu actions' stamp: 'nk 6/2/2006 08:03'!
buildMorphicCodePaneWith: editString

	| codePane |

	codePane := AcceptableCleanTextMorph
		on: self
		text: #contents 
		accept: #contents:
		readSelection: #contentsSelection 
		menu: #codePaneMenu:shifted:.
	codePane font: Preferences standardCodeFont.
	editString ifNotNil: [
		codePane editString: editString.
		codePane hasUnacceptedEdits: true
	].
	^codePane
! !

!ChangeList methodsFor: 'toolbuilder' stamp: 'rkrk 8/24/2009 07:02'!
buildWith: builder
	^self buildWith: builder multiSelect: self showsVersions not! !

!ChangeList methodsFor: 'toolbuilder' stamp: 'ar 8/8/2009 17:06'!
buildWith: builder multiSelect: multiSelect 
	"Open a morphic view for the messageSet, whose label is labelString. 
	The listView may be either single or multiple selection type"
	| windowSpec max |
	max := self wantsOptionalButtons ifTrue:[0.33] ifFalse:[0.4].
	windowSpec := self buildWindowWith: builder specs: {
		(0@0 corner: 1@max) -> [self buildChangeListWith: builder multiSelect: multiSelect].
		(0@max corner: 1@1) -> [self buildCodePaneWith: builder].
	}.
	^builder build: windowSpec! !

!ChangeList methodsFor: 'accessing' stamp: ''!
changeList
	^ changeList! !

!ChangeList methodsFor: 'initialization-release' stamp: 'sw 1/7/2000 12:42'!
changeListButtonSpecs

	^#(
		('select all' 			selectAll				'select all entries')
		('deselect all'		deselectAll			'deselect all entries')
		('select conflicts'	selectAllConflicts	'select all methods that occur in any change set')
		('file in selections' 	fileInSelections		'file in all selected entries')
		)! !

!ChangeList methodsFor: 'menu actions' stamp: 'sw 1/25/2001 07:22'!
changeListKey: aChar from: view
	"Respond to a Command key in the list pane."

	aChar == $D ifTrue: [^ self toggleDiffing].
	aChar == $a ifTrue: [^ self selectAll].

	^ self arrowKey: aChar from: view! !

!ChangeList methodsFor: 'menu actions' stamp: 'topa 3/16/2015 11:24'!
changeListMenu: aMenu
	^ self menu: aMenu for: #(changeListMenu changeListMenuShifted)
! !

!ChangeList methodsFor: 'accessing' stamp: 'sd 11/20/2005 21:26'!
changes: changeRecords file: aFile
	file := aFile.
	changeList := OrderedCollection new.
	list := OrderedCollection new.
	listIndex := 0.
	changeRecords do: [:each |
		(each respondsTo: #methodClass)
			ifFalse: [self addItem: ChangeRecord new text: each asString]
			ifTrue:
				[self addItem: each text: ('method: ' , each methodClass name , (each isMetaClassChange ifTrue: [' class '] ifFalse: [' '])
					, each methodSelector
					, '; ' , each stamp)]].
	listSelections := Array new: list size withAll: false! !

!ChangeList methodsFor: 'menu actions' stamp: 'sw 5/20/2001 21:18'!
compareToCurrentVersion
	"If the current selection corresponds to a method in the system, then spawn a window showing the diffs as text"

	| change class s1 s2 |
	listIndex = 0
		ifTrue: [^ self].
	change := changeList at: listIndex.
	((class := change methodClass) notNil
			and: [class includesSelector: change methodSelector])
		ifTrue: [s1 := (class sourceCodeAt: change methodSelector) asString.
			s2 := change string.
			s1 = s2
				ifTrue: [^ self inform: 'Exact Match'].
			(StringHolder new
				textContents: (TextDiffBuilder buildDisplayPatchFrom: s1 to: s2 inClass: class  prettyDiffs: self showingPrettyDiffs))
				openLabel: 'Comparison to Current Version']
		ifFalse: [self flash]! !

!ChangeList methodsFor: 'viewing access' stamp: 'sw 9/5/2001 13:52'!
contents
	"Answer the contents string, obeying diffing directives if needed"

	^ self showingAnyKindOfDiffs
		ifFalse:
			[self undiffedContents]
		ifTrue:
			[self showsVersions
				ifTrue:
					[self diffedVersionContents]
				ifFalse:
					[self contentsDiffedFromCurrent]]! !

!ChangeList methodsFor: 'viewing access' stamp: 'fbs 7/15/2013 09:44'!
contentsDiffedFromCurrent
	"Answer the contents diffed forward from current (in-memory) method version"

	| aChange aClass |
	listIndex = 0
		ifTrue: [^ ''].
	aChange := changeList at: listIndex.
	 (aChange type == #method
	 and: [(aClass := aChange methodClass) notNil
	 and: [aClass includesSelector: aChange methodSelector]]) ifTrue:
		[^self
			methodDiffFor: aChange text
			class: aClass
			selector: aChange methodSelector
			prettyDiffs: self showingPrettyDiffs].

	aChange type == #doIt ifTrue:
		[| tokens |
		 tokens := Scanner new scanTokens: aChange string.
		 ((tokens select:
				[:substr| #(subclass: variableByteSubclass: variableWordSubclass:
							instanceVariableNames: classVariableNames: ) includes: substr])
					asSet size >= 3
		  and: [(aClass := Smalltalk at: tokens third ifAbsent: []) notNil
		  and: [aClass isBehavior]]) ifTrue:
			[^ClassDiffBuilder buildDisplayPatchFrom: aClass definition to: aChange string].

		(tokens size = 4
		 and: [tokens second == #class
		 and: [tokens third == #instanceVariableNames:
		 and: [(aClass := Smalltalk at: tokens first ifAbsent: []) notNil
		 and: [aClass isBehavior]]]]) ifTrue:
			[^ClassDiffBuilder buildDisplayPatchFrom: aClass class definition to: aChange string]].

	(aChange type == #classComment
	and: [(aClass := aChange commentClass) notNil]) ifTrue:
		[^ClassDiffBuilder buildDisplayPatchFrom: aClass comment asString to: aChange string].

	^(changeList at: listIndex) text! !

!ChangeList methodsFor: 'viewing access' stamp: 'sw 11/13/2001 09:12'!
contentsSymbolQuints
	"Answer a list of quintuplets representing information on the alternative views available in the code pane"

	^ self sourceAndDiffsQuintsOnly! !

!ChangeList methodsFor: 'viewing access' stamp: 'tk 4/10/1998 09:25'!
contents: aString
	listIndex = 0 ifTrue: [self changed: #flash. ^ false].
	lostMethodPointer ifNotNil: [^ self restoreDeletedMethod].
	self okToChange "means not dirty" ifFalse: ["is dirty"
		self inform: 'This is a view of a method on a file.\Please cancel your changes.  You may\accept, but only when the method is untouched.' withCRs.  ^ false].
		"Can't accept changes here.  Method text must be unchanged!!"
	(changeList at: listIndex) fileIn.
	^ true! !

!ChangeList methodsFor: 'accessing' stamp: 'ls 5/12/1999 07:55'!
currentChange
	"return the current change being viewed, or nil if none"
	listIndex = 0 ifTrue: [ ^nil ].
	^changeList at: listIndex! !

!ChangeList methodsFor: 'user interface' stamp: 'mt 7/9/2016 08:52'!
defaultWindowColor
	^ (Color r: 0.719 g: 0.9 b: 0.9)! !

!ChangeList methodsFor: 'menu actions' stamp: 'sw 1/25/2001 08:38'!
deselectAll 
	"Deselect all items in the list pane, and clear the code pane"

	listIndex := 0.
	listSelections atAllPut: false.
	self changed: #allSelections.
	self contentsChanged! !

!ChangeList methodsFor: 'menu actions' stamp: 'sw 1/25/2001 09:04'!
destroyCurrentCodeOfSelections
	"Actually remove from the system any in-memory methods with class and selector identical to items current selected.  This may seem rather arcane but believe me it has its great uses, when trying to split out code.  To use effectively, first file out a change set that you wish to split off.  Then open a ChangeList browser on that fileout.  Now look through the methods, and select any of them which you want to remove completely from the system, then issue this command.  For those methods where you have made changes to pre-existing versions, of course, you won't want to remove them from the system, so use this mechanism with care!!"

	|  aClass aChange aList |
	aList := OrderedCollection new.
	1 to: changeList size do:
		[:index |
			(listSelections at: index) ifTrue:
				[aChange := changeList at: index.
				(aChange type = #method
					and: [(aClass := aChange methodClass) notNil
					and: [aClass includesSelector: aChange methodSelector]])
						ifTrue:
							[aList add: {aClass. aChange methodSelector}]]].

	aList size > 0 ifTrue:
		[(self confirm: 'Warning!! This will actually remove ', aList size printString,  ' method(s) from the system!!') ifFalse: [^ self]].
	aList do:
		[:aPair | Transcript cr; show: 'Removed: ', aPair first printString, '.', aPair second.
			aPair first removeSelector: aPair second]! !

!ChangeList methodsFor: 'viewing access' stamp: 'mt 10/21/2015 09:10:18'!
diffedVersionContents
	"Answer diffed version contents, maybe pretty maybe not"

	| change class earlier later |
	(listIndex = 0
			or: [changeList size < listIndex])
		ifTrue: [^ ''].
	change := changeList at: listIndex.
	later := change text.
	class := change methodClass.
	(listIndex == changeList size or: [class == nil])
		ifTrue: [^ (self showingPrettyDiffs and: [class notNil])
			ifTrue: [class prettyPrinterClass format: later in: class notifying: nil]
			ifFalse: [later]].

	earlier := (changeList at: listIndex + 1) text.

	^ TextDiffBuilder buildDisplayPatchFrom: earlier to: later inClass: class prettyDiffs: self showingPrettyDiffs! !

!ChangeList methodsFor: 'accessing' stamp: ''!
file
	^file! !

!ChangeList methodsFor: 'menu actions' stamp: 'sw 10/11/1999 17:10'!
fileInSelections 
	| any |
	any := false.
	listSelections with: changeList do: 
		[:selected :item | selected ifTrue: [any := true. item fileIn]].
	any ifFalse:
		[self inform: 'nothing selected, so nothing done']! !

!ChangeList methodsFor: 'menu actions' stamp: 'rbb 3/1/2005 10:27'!
fileOutSelections 
	| fileName internalStream |
	fileName := UIManager default request: 'Enter the base of file name' initialAnswer: 'Filename'.
	internalStream := WriteStream on: (String new: 1000).
	internalStream header; timeStamp.
	listSelections with: changeList do: 
		[:selected :item | selected ifTrue: [item fileOutOn: internalStream]].

	FileStream writeSourceCodeFrom: internalStream baseName: fileName isSt: true useHtml: false.
! !

!ChangeList methodsFor: 'initialization-release' stamp: 'sw 9/5/2001 13:53'!
initialize
	"Initialize a blank ChangeList.  Set the contentsSymbol to reflect whether diffs will initally be shown or not"

	contentsSymbol := Preferences diffsInChangeList
		ifTrue:
			[self defaultDiffsSymbol]
		ifFalse:
			[#source].
	changeList := OrderedCollection new.
	list := OrderedCollection new.
	listIndex := 0.
	super initialize! !

!ChangeList methodsFor: 'menu actions' stamp: 'sw 1/25/2001 08:35'!
invertSelections
	"Invert the selectedness of each item in the changelist"

	listSelections := listSelections collect: [ :ea | ea not].
	listIndex := 0.
	self changed: #allSelections.
	self contentsChanged! !

!ChangeList methodsFor: 'viewing access' stamp: ''!
list
	^ list! !

!ChangeList methodsFor: 'accessing' stamp: 'TPR 11/28/1998 17:38'!
listHasSingleEntry
	"does the list of changes have only a single item?"
	^list size = 1! !

!ChangeList methodsFor: 'viewing access' stamp: ''!
listIndex
	^ listIndex! !

!ChangeList methodsFor: 'viewing access' stamp: ''!
listSelectionAt: index
	^ listSelections at: index! !

!ChangeList methodsFor: 'viewing access' stamp: 'di 1/13/1999 14:59'!
listSelectionAt: index put: value

	^ listSelections at: index put: value! !

!ChangeList methodsFor: 'accessing' stamp: 'tk 6/21/1999 20:43'!
listSelections
	listSelections ifNil: [
		list ifNotNil: [
			listSelections := Array new: list size withAll: false]].
	^ listSelections! !

!ChangeList methodsFor: 'menu actions' stamp: 'topa 3/16/2015 11:23'!
mainChangeListMenu: aMenu
	"Fill aMenu up so that it comprises the primary changelist-browser menu"
	<changeListMenu>

	aMenu addTitle: 'change list'.
	aMenu addStayUpItemSpecial.

	aMenu addList: #(

	('fileIn selections'							fileInSelections						'import the selected items into the image')
	('fileOut selections...'						fileOutSelections						'create a new file containing the selected items')
	-
	('compare to current'						compareToCurrentVersion			'open a separate window which shows the text differences between the on-file version and the in-image version.' )
	('toggle diffing (D)'							toggleDiffing						'start or stop showing diffs in the code pane.')
	-
	('select conflicts with any changeset'		selectAllConflicts					'select methods in the file which also occur in any change-set in the system')
	('select conflicts with current changeset'	selectConflicts						'select methods in the file which also occur in the current change-set')
	('select conflicts with...'						selectConflictsWith					'allows you to designate a file or change-set against which to check for code conflicts.')
	-
	('select unchanged definitions'				selectUnchangedDefinitions			'select class definitions, class comments and methods in the file whose in-image versions are the same as their in-file counterparts' )
	('select unchanged methods'					selectUnchangedMethods				'select methods in the file whose in-image versions are the same as their in-file counterparts' )
	('select new methods'						selectNewMethods					'select methods in the file that do not current occur in the image')
	('select methods for this class'				selectMethodsForThisClass			'select all methods in the file that belong to the currently-selected class')
	('select methods for extant classes'			selectMethodsForExtantClasses		'select all methods in the file that belong to a class that exists in the image')
	('select changes with contents matching'		selectContentsMatching				'select all changes in the file whose text includes a pattern')

	-
	('select all (a)'								selectAll								'select all the items in the list')
	('deselect all'								deselectAll							'deselect all the items in the list')
	('invert selections'							invertSelections						'select every item that is not currently selected, and deselect every item that *is* currently selected')
	-
	('browse all versions of single selection'			browseVersions		'open a version browser showing the versions of the currently selected method')
	('browse all versions of selections'			browseAllVersionsOfSelections		'open a version browser showing all the versions of all the selected methods')
	('browse current versions of selections'		browseCurrentVersionsOfSelections	'open a message-list browser showing the current (in-image) counterparts of the selected methods')
	('destroy current methods of selections'		destroyCurrentCodeOfSelections		'remove (*destroy*) the in-image counterparts of all selected methods')
	-
	('remove doIts'								removeDoIts							'remove all items that are doIts rather than methods')
	('remove older versions'						removeOlderMethodVersions			'remove all but the most recent versions of methods in the list')
	('remove up-to-date versions'				removeExistingMethodVersions		'remove all items whose code is the same as the counterpart in-image code')
	('remove selected items'						removeSelections					'remove the selected items from the change-list')
	('remove unselected items'					removeNonSelections					'remove all the items not currently selected from the change-list')).

	^ aMenu

! !

!ChangeList methodsFor: 'viewing access' stamp: 'md 11/14/2003 18:02'!
methodDiffFor: aString class: aClass selector: aSelector prettyDiffs: prettyDiffBoolean
	"Return a string comprising a source-code diff between an existing method and the source-code in aString.  DO prettyDiff if prettyDiffBoolean is true."

	^ (aClass notNil and: [aClass includesSelector: aSelector])
		ifTrue:
			[TextDiffBuilder
				buildDisplayPatchFrom: (aClass sourceCodeAt: aSelector)
				to: aString
				inClass: aClass
				prettyDiffs: prettyDiffBoolean]
		ifFalse:
			[aString copy]! !

!ChangeList methodsFor: 'initialization-release' stamp: 'sbw 12/30/1999 11:02'!
optionalButtonHeight

	^ 15! !

!ChangeList methodsFor: 'menu actions' stamp: 'sw 10/11/1999 17:18'!
perform: selector orSendTo: otherTarget
	"Selector was just chosen from a menu by a user.  If I can respond, then perform it on myself.  If not, send it to otherTarget, presumably the editPane from which the menu was invoked." 

	(#accept == selector) ifTrue:
		[otherTarget isMorph ifFalse: [^ self acceptFrom: otherTarget view]].
			"weird special case just for mvc changlist"

	^ super perform: selector orSendTo: otherTarget! !

!ChangeList methodsFor: 'menu actions' stamp: 'tk 4/8/98 12:38'!
removeDoIts
	"Remove doits from the receiver, other than initializes. 1/26/96 sw"

	| newChangeList newList |

	newChangeList := OrderedCollection new.
	newList := OrderedCollection new.

	changeList with: list do:
		[:chRec :str |
			(chRec type ~~ #doIt or:
				[str endsWith: 'initialize'])
					ifTrue:
						[newChangeList add: chRec.
						newList add: str]].
	newChangeList size < changeList size
		ifTrue:
			[changeList := newChangeList.
			list := newList.
			listIndex := 0.
			listSelections := Array new: list size withAll: false].
	self changed: #list.

	! !

!ChangeList methodsFor: 'menu actions' stamp: 'nice 2/23/2011 21:46'!
removeExistingMethodVersions
	"Remove all up to date version of entries from the receiver"
	| newChangeList newList |
	newChangeList := OrderedCollection new.
	newList := OrderedCollection new.

	changeList with: list do:[:chRec :strNstamp | 
			| str keep cls sel |
			keep := true.
			(cls := chRec methodClass) ifNotNil:[
				str := chRec string.
				sel := cls newParser parseSelector: str.
				keep := (cls sourceCodeAt: sel ifAbsent:['']) asString ~= str.
			].
			keep ifTrue:[
					newChangeList add: chRec.
					newList add: strNstamp]].
	newChangeList size < changeList size
		ifTrue:
			[changeList := newChangeList.
			list := newList.
			listIndex := 0.
			listSelections := Array new: list size withAll: false].
	self changed: #list! !

!ChangeList methodsFor: 'menu actions' stamp: 'ul 11/15/2010 11:34'!
removeNonSelections
	"Remove the unselected items from the receiver."

	| newChangeList newList |

	newChangeList := OrderedCollection new.
	newList := OrderedCollection new.

	1 to: changeList size do:
		[:i | (listSelections at: i) ifTrue:
			[newChangeList add: (changeList at: i).
			newList add: (list at: i)]].
	newChangeList size = 0 ifTrue:
		[^ self inform: 'That would remove everything.
Why would you want to do that?'].

	newChangeList size < changeList size
		ifTrue:
			[changeList := newChangeList.
			list := newList.
			listIndex := 0.
			listSelections := Array new: list size withAll: false].
	self changed: #list

	! !

!ChangeList methodsFor: 'menu actions' stamp: 'nice 12/27/2009 03:10'!
removeOlderMethodVersions
	"Remove older versions of entries from the receiver."
	| newChangeList newList found |
	newChangeList := OrderedCollection new.
	newList := OrderedCollection new.
	found := OrderedCollection new.

	changeList reverseWith: list do:
		[:chRec :strNstamp | | str | str := strNstamp copyUpTo: $;.
			(found includes: str)
				ifFalse:
					[found add: str.
					newChangeList add: chRec.
					newList add: strNstamp]].
	newChangeList size < changeList size
		ifTrue:
			[changeList := newChangeList reversed.
			list := newList reversed.
			listIndex := 0.
			listSelections := Array new: list size withAll: false].
	self changed: #list! !

!ChangeList methodsFor: 'menu actions' stamp: 'sd 11/20/2005 21:26'!
removeSelections
	"Remove the selected items from the receiver.  9/18/96 sw"

	| newChangeList newList |

	newChangeList := OrderedCollection new.
	newList := OrderedCollection new.

	1 to: changeList size do:
		[:i | (listSelections at: i) ifFalse:
			[newChangeList add: (changeList at: i).
			newList add: (list at: i)]].
	newChangeList size < changeList size
		ifTrue:
			[changeList := newChangeList.
			list := newList.
			listIndex := 0.
			listSelections := Array new: list size withAll: false].
	self changed: #list

	! !

!ChangeList methodsFor: 'viewing access' stamp: 'Igor.Stasenko 12/20/2009 18:54'!
restoreDeletedMethod
	"If lostMethodPointer is not nil, then this is a version browser for a method that has been removed.  In this case we want to establish a sourceCode link to prior versions.  We do this by installing a dummy method with the correct source code pointer prior to installing this version."
	| dummyMethod class selector |
	dummyMethod := CompiledMethod toReturnSelfTrailerBytes: 
		(CompiledMethodTrailer new sourcePointer: lostMethodPointer).
	class := (changeList at: listIndex) methodClass.
	selector := (changeList at: listIndex) methodSelector.
	class addSelectorSilently: selector withMethod: dummyMethod.
	(changeList at: listIndex) fileIn.
	"IF for some reason, the dummy remains, remove it, but (N.B.!!) we might not get control back if the compile (fileIn above) fails."
	(class compiledMethodAt: selector) == dummyMethod
		ifTrue: [class basicRemoveSelector: selector].
	^ true! !

!ChangeList methodsFor: 'scanning' stamp: 'ul 8/14/2015 18:00'!
scanCategory  
	"Scan anything that involves more than one chunk; method name is historical only"

	| itemPosition item tokens stamp anIndex |
	itemPosition := file position.
	item := file nextChunk.

	((item includesSubstring: 'commentStamp:')
	or: [(item includesSubstring: 'methodsFor:')
	or: [item endsWith: 'reorganize']]) ifFalse:
		["Maybe a preamble, but not one we recognize; bail out with the preamble trick"
		^ self addItem: (ChangeRecord new file: file position: itemPosition type: #preamble)
				 text: ('preamble: ' , item contractTo: 50)].

	tokens := Scanner new scanTokens: item.
	tokens size >= 3 ifTrue:
		[stamp := ''.
		anIndex := tokens indexOf: #stamp: ifAbsent: [nil].
		anIndex ifNotNil: [stamp := tokens at: (anIndex + 1)].

		tokens second == #methodsFor:
			ifTrue: [^ self scanCategory: tokens third class: tokens first
							meta: false stamp: stamp].
		tokens third == #methodsFor:
			ifTrue: [^ self scanCategory: tokens fourth class: tokens first
							meta: true stamp: stamp]].

	tokens second == #commentStamp:
		ifTrue:
			[stamp := tokens third.
			self addItem:
					(ChangeRecord new file: file position: file position type: #classComment
									class: tokens first category: nil meta: false stamp: stamp)
					text: 'class comment for ' , tokens first, 
						  (stamp isEmpty ifTrue: [''] ifFalse: ['; ' , stamp]).
			file nextChunk.
			^ file skipStyleChunk].

	self assert: tokens last == #reorganize.
	self addItem:
		(ChangeRecord new
			file: file position: file position type: #reorganize
			class: tokens first category: nil meta: false stamp: stamp)
		text: 'organization for ' , tokens first, (tokens second == #class ifTrue: [' class'] ifFalse: ['']).
	file nextChunk! !

!ChangeList methodsFor: 'scanning' stamp: 'eem 12/6/2014 12:38'!
scanCategory: category class: class meta: meta stamp: stamp
	| itemPosition method selector |
	[itemPosition := file position.
	method := file nextChunk.
	file skipStyleChunk.
	method size > 0]						"done when double terminators"
		whileTrue:
		[self addItem: (ChangeRecord new file: file position: itemPosition type: #method
							class: class category: category meta: meta stamp: stamp)
			text: 'method: ' , class , (meta ifTrue: [' class '] ifFalse: [' '])
				, ((selector := ((Smalltalk classNamed: class) ifNil: [Object]) newParser parseSelector: method) isNil
					ifTrue: ['unparsableSelector']
					ifFalse: [selector])
				, (stamp isEmpty ifTrue: [''] ifFalse: ['; ' , stamp])]! !

!ChangeList methodsFor: 'scanning' stamp: 'ul 6/17/2011 12:32'!
scanFile: aFile from: startPosition to: stopPosition
	
	file := aFile.
	changeList := OrderedCollection new.
	list := OrderedCollection new.
	listIndex := 0.
	file position: startPosition.
'Scanning ', aFile localName, '...'
	displayProgressFrom: startPosition to: stopPosition
	during: [:bar | | prevChar itemPosition item |
	[file position < stopPosition]
		whileTrue:
		[bar value: file position.
		[file atEnd not and: [file peek isSeparator]]
				whileTrue: [prevChar := file next].
		(file peekFor: $!!)
		ifTrue:
			[(prevChar = Character cr or: [prevChar = Character lf])
				ifTrue: [self scanCategory]]
		ifFalse:
			[itemPosition := file position.
			item := file nextChunk.
			file skipStyleChunk.
			item size > 0 ifTrue:
				[self addItem: (ChangeRecord new file: file position: itemPosition type: #doIt)
					text: 'do it: ' , (item contractTo: 50)]]]].
	listSelections := Array new: list size withAll: false! !

!ChangeList methodsFor: 'menu actions' stamp: 'sd 11/20/2005 21:26'!
selectAll
	listIndex := 0.
	listSelections atAllPut: true.
	self changed: #allSelections! !

!ChangeList methodsFor: 'menu actions' stamp: 'nice 1/19/2010 01:51'!
selectAllConflicts
	"Selects all method definitions in the receiver which are also in any existing change set in the system.  This makes no statement about whether the content of the methods differ, only whether there is a change represented."
	Cursor read showWhile: 
		[ | aClass aChange |
		1 to: changeList size do:
			[:i | aChange := changeList at: i.
			listSelections at: i put:
				(aChange type = #method
				and: [(aClass := aChange methodClass) notNil
				and: [ChangesOrganizer doesAnyChangeSetHaveClass: aClass andSelector:  aChange methodSelector]])]].
	self changed: #allSelections! !

!ChangeList methodsFor: 'menu actions' stamp: 'nice 1/19/2010 01:51'!
selectConflicts
	"Selects all method definitions for which there is ALSO an entry in changes"
	Cursor read showWhile: 
	[ | change class |
	1 to: changeList size do:
		[:i | change := changeList at: i.
		listSelections at: i put:
			(change type = #method
			and: [(class := change methodClass) notNil
			and: [(ChangeSet current atSelector: change methodSelector
						class: class) ~~ #none]])]].
	self changed: #allSelections! !

!ChangeList methodsFor: 'menu actions' stamp: 'pk 10/17/2006 09:34'!
selectConflictsWith
	"Selects all method definitions for which there is ALSO an entry in the specified changeSet or changList chosen by the user. 4/11/96 tk"
	| aStream all index |
	aStream := WriteStream on: (String new: 200).
	(all := ChangesOrganizer allChangeSets copy) do:
		[:sel | aStream nextPutAll: (sel name contractTo: 40); cr].
	ChangeList allSubInstancesDo:
		[:sel | aStream nextPutAll: (sel file name); cr.
			all addLast: sel].
	aStream skip: -1.
	index := (UIManager default chooseFrom: (aStream contents substrings)).
	index > 0 ifTrue: [
		self selectConflicts: (all at: index)].
! !

!ChangeList methodsFor: 'menu actions' stamp: 'nice 1/19/2010 01:51'!
selectConflicts: changeSetOrList
	"Selects all method definitions for which there is ALSO an entry in the specified changeSet or changList"
	Cursor read showWhile: 
	[ | change class |
	(changeSetOrList isKindOf: ChangeSet) ifTrue: [
	1 to: changeList size do:
		[:i | change := changeList at: i.
		listSelections at: i put:
			(change type = #method
			and: [(class := change methodClass) notNil
			and: [(changeSetOrList atSelector: change methodSelector
						class: class) ~~ #none]])]]
	ifFalse: ["a ChangeList"
	1 to: changeList size do:
		[:i | change := changeList at: i.
		listSelections at: i put:
			(change type = #method
			and: [(class := change methodClass) notNil
			and: [changeSetOrList list includes: (list at: i)]])]]
	].
	self changed: #allSelections! !

!ChangeList methodsFor: 'menu actions' stamp: 'ul 8/14/2015 18:00'!
selectContentsMatching
	| pattern |
	pattern := UIManager default request: 'pattern to match'.
	pattern isEmpty ifTrue: [^self].
	^Cursor execute showWhile:
		[self selectSuchThat: ((pattern includesAnyOf: '?*')
								ifTrue: [[ :change | pattern match: change string]]
								ifFalse: [[ :change | change string includesSubstring: pattern]])]! !

!ChangeList methodsFor: 'viewing access' stamp: 'nk 2/26/2004 13:50'!
selectedClass
	^(self selectedClassOrMetaClass ifNil: [ ^nil ]) theNonMetaClass ! !

!ChangeList methodsFor: 'viewing access' stamp: 'sma 2/5/2000 19:09'!
selectedClassOrMetaClass
	| c |
	^ (c := self currentChange) ifNotNil: [c methodClass]! !

!ChangeList methodsFor: 'viewing access' stamp: 'sma 2/5/2000 19:10'!
selectedMessageName
	| c |
	^ (c := self currentChange) ifNotNil: [c methodSelector]! !

!ChangeList methodsFor: 'menu actions' stamp: 'eem 8/18/2009 13:41'!
selectMethodsForExtantClasses
	^self selectSuchThat:
		[ :change |
		Smalltalk hasClassNamed: change methodClassName]! !

!ChangeList methodsFor: 'menu actions' stamp: 'eem 7/30/2014 19:26'!
selectMethodsForThisClass
	self currentChange ifNil: [ ^self ].
	self currentChange methodClassName ifNotNil:
		[:name|
		self selectSuchThat:
			(Sensor leftShiftDown
				ifTrue: [[:change :index| (listSelections at: index) or: [change methodClassName = name]]]
				ifFalse: [[:change| change methodClassName = name]])]! !

!ChangeList methodsFor: 'menu actions' stamp: 'nice 1/19/2010 01:51'!
selectNewMethods
	"Selects all method definitions for which there is no counterpart method in the current image"
	Cursor read showWhile: 
		[ | change class |
		1 to: changeList size do:
			[:i | change := changeList at: i.
			listSelections at: i put:
				((change type = #method and:
					[((class := change methodClass) isNil) or:
						[(class includesSelector: change methodSelector) not]]))]].
	self changed: #allSelections! !

!ChangeList methodsFor: 'menu actions' stamp: 'rbb 3/1/2005 10:27'!
selectSuchThat
	"query the user for a selection criterio.  By Lex Spoon.  NB: the UI for invoking this from a changelist browser is currently commented out; to reenfranchise it, you'll need to mild editing to ChangeList method #changeListMenu:"
	| code block |
	code := UIManager default request: 'selection criteria for a change named aChangeRecord?\For instance, ''aChangeRecord category = ''System-Network''''' withCRs.

	code isEmpty ifTrue: [^ self ].

	block := Compiler evaluate: '[:aChangeRecord | ', code, ']'.

	self selectSuchThat: block! !

!ChangeList methodsFor: 'menu actions' stamp: 'eem 7/30/2014 19:24'!
selectSuchThat: aBlock
	"select all changes for which block returns true"
	listSelections := aBlock numArgs = 2
						ifTrue: [changeList withIndexCollect: aBlock]
						ifFalse: [changeList collect: aBlock].
	self changed: #allSelections! !

!ChangeList methodsFor: 'menu actions' stamp: 'eem 7/22/2010 13:40'!
selectUnchangedDefinitions
	"Selects all recognizable definitions for which there is already a definition in the current image, whose source is exactly the same."
	| change class tokens |
	Cursor read showWhile: 
	[1 to: changeList size do:
		[:i | change := changeList at: i.
		listSelections at: i put: false.

		(change type = #method
		 and: [(class := change methodClass) notNil
		 and: [class includesSelector: change methodSelector]]) ifTrue:
			[listSelections
				at: i
				put: change string withBlanksCondensed
					= (class sourceCodeAt: change methodSelector) asString withBlanksCondensed].

		(change type == #classComment
		and: [(class := change commentClass) notNil]) ifTrue:
			[listSelections at: i put: change string = class comment asString].

		change type == #doIt ifTrue:
			[tokens := Scanner new scanTokens: change string.

			 ((tokens select:
				[:substr| #(subclass: variableSubclass: variableByteSubclass: variableWordSubclass:
							instanceVariableNames: classVariableNames: ) includes: substr])
					asSet size >= 3
			 and: [(class := Smalltalk at: tokens third ifAbsent: []) notNil
			 and: [class isBehavior]]) ifTrue:
				[listSelections
					at: i
					put: change string withBlanksCondensed
						= class definition withBlanksCondensed].

			(tokens size = 4
			 and: [tokens second == #class
			 and: [tokens third == #instanceVariableNames:
			 and: [(class := Smalltalk at: tokens first ifAbsent: []) notNil
			 and: [class isBehavior]]]]) ifTrue:
				[listSelections
					at: i
					put: change string withBlanksCondensed
						= class class definition withBlanksCondensed].

			(tokens size = 3
			 and: [tokens second == #removeSelector:
			 and: [(class := Smalltalk at: tokens first ifAbsent: []) isNil
			 	or: [class isBehavior and: [(class includesSelector: tokens third) not]]]]) ifTrue:
				[listSelections at: i put: true].

			(tokens size = 4
			 and: [tokens second == #class
			 and: [tokens third == #removeSelector:
			 and: [(class := Smalltalk at: tokens first ifAbsent: []) isNil
			 	or: [class isBehavior and: [(class class includesSelector: tokens fourth) not]]]]]) ifTrue:
				[listSelections at: i put: true]]]].

	self changed: #allSelections! !

!ChangeList methodsFor: 'menu actions' stamp: 'nice 1/19/2010 01:52'!
selectUnchangedMethods
	"Selects all method definitions for which there is already a method in the current image, whose source is exactly the same.  9/18/96 sw"
	Cursor read showWhile: 
	[ | class change |
	1 to: changeList size do:
		[:i | change := changeList at: i.
		listSelections at: i put:
			((change type = #method and:
				[(class := change methodClass) notNil]) and:
					[(class includesSelector: change methodSelector) and:
						[change string withBlanksCondensed = (class sourceCodeAt: change methodSelector) asString withBlanksCondensed ]])]].
	self changed: #allSelections! !

!ChangeList methodsFor: 'accessing' stamp: 'di 6/15/97 15:13'!
setLostMethodPointer: sourcePointer
	lostMethodPointer := sourcePointer! !

!ChangeList methodsFor: 'accessing' stamp: 'sw 10/19/1999 15:11'!
showsVersions
	^ false! !

!ChangeList methodsFor: 'viewing access' stamp: 'sw 1/28/1999 12:30'!
toggleListIndex: newListIndex

	listIndex ~= 0 ifTrue: [listSelections at: listIndex put: false].
	newListIndex ~= 0 ifTrue: [listSelections at: newListIndex put: true].
	listIndex := newListIndex.
	self changed: #listIndex.
	self contentsChanged! !

!ChangeList methodsFor: 'viewing access' stamp: 'sw 1/25/1999 14:45'!
undiffedContents
	^ listIndex = 0
		ifTrue: ['']
		ifFalse: [(changeList at: listIndex) text]! !

!ChangeList methodsFor: 'initialization-release' stamp: 'sw 8/15/2002 22:34'!
wantsPrettyDiffOption
	"Answer whether pretty-diffs are meaningful for this tool"

	^ true! !

ChangeList subclass: #VersionsBrowser
	instanceVariableNames: 'classOfMethod selectorOfMethod addedChangeRecord'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Changes'!

!VersionsBrowser commentStamp: 'nk 11/25/2003 10:04' prior: 0!
VersionsBrowser shows all the versions of a particular method, and lets you compare them, revert to selected versions, and so on.!

VersionsBrowser subclass: #ClassCommentVersionsBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Changes'!

!ClassCommentVersionsBrowser commentStamp: 'asm 8/13/2002 23:20' prior: 0!
A class-comment-versions-browser tool!

!ClassCommentVersionsBrowser class methodsFor: 'instance creation' stamp: 'nice 12/27/2009 02:54'!
browseCommentOf: class
	Cursor read showWhile:
		[| changeList |
		changeList := self new scanVersionsOf: class.
	 	 changeList ifNil: [^ self inform: 'No versions available'].
		 self open: changeList name: 'Recent versions of ',class name,'''s comments' multiSelect: false ]
! !

!ClassCommentVersionsBrowser class methodsFor: 'utilities' stamp: 'asm 8/13/2002 22:09'!
commentRecordsOf: aClass
	"Return a list of ChangeRecords for all versions of the method at selector. Source code can be retrieved by sending string to any one.  Return nil if the method is absent."

	| aList |
	aList := self new
			scanVersionsOf: aClass.
	^ aList ifNotNil: [aList changeList]! !

!ClassCommentVersionsBrowser class methodsFor: 'utilities' stamp: 'asm 8/13/2002 20:54'!
timeStampFor: aSelector class: aClass reverseOrdinal: anInteger
	"Answer the time stamp corresponding to some version of the given method, nil if none.  The reverseOrdinal parameter is interpreted as:  1 = current version; 2 = last-but-one version, etc."
	
	| aChangeList |
	aChangeList :=  self new scanVersionsOf: aClass.
	^ aChangeList ifNil: [nil] ifNotNil:
		[aChangeList list size >= anInteger
			ifTrue:
				[(aChangeList changeList at: anInteger) stamp]
			ifFalse:
				[nil]]! !

!ClassCommentVersionsBrowser methodsFor: 'contents' stamp: 'ul 1/9/2016 21:51'!
aboutToStyle: aStyler

	^false! !

!ClassCommentVersionsBrowser methodsFor: 'misc' stamp: 'sw 8/17/2002 21:57'!
classCommentIndicated
	"Answer whether the receiver is pointed at a class comment"

	^ true! !

!ClassCommentVersionsBrowser methodsFor: 'menu' stamp: 'asm 8/13/2002 21:33'!
compareToCurrentVersion
	"If the current selection corresponds to a method in the system, then spawn a window showing the diffs as text"

	| change s1 s2 |
	listIndex = 0
		ifTrue: [^ self].
	change := changeList at: listIndex.
	s1 := classOfMethod organization classComment.
	s2 := change string.
	s1 = s2
		ifTrue: [^ self inform: 'Exact Match'].
			(StringHolder new
				textContents: (TextDiffBuilder buildDisplayPatchFrom: s1 to: s2 inClass: classOfMethod  prettyDiffs: self showingPrettyDiffs))
				openLabel: 'Comparison to Current Version'! !

!ClassCommentVersionsBrowser methodsFor: 'misc' stamp: 'sw 8/15/2002 22:38'!
contentsSymbolQuints
	"Answer a list of quintuplets representing information on the alternative views available in the code pane"

	^ #(
(source			togglePlainSource 		showingPlainSourceString	'source'			'the textual source code as writen')
(showDiffs		toggleRegularDiffing	showingRegularDiffsString	'showDiffs'		'the textual source diffed from its prior version'))! !

!ClassCommentVersionsBrowser methodsFor: 'user interface' stamp: 'mt 7/9/2016 08:52'!
defaultWindowColor
	^ (Color r: 0.753 g: 0.677 b: 0.9)! !

!ClassCommentVersionsBrowser methodsFor: 'basic function' stamp: 'asm 8/13/2002 22:26'!
diffedVersionContents
	"Answer diffed version contents, maybe pretty maybe not"

	| change class earlier later |
	(listIndex = 0
			or: [changeList size < listIndex])
		ifTrue: [^ ''].
	change := changeList at: listIndex.
	later := change text.
	class := self selectedClass.
	(listIndex == changeList size or: [class == nil])
		ifTrue: [^ later].

	earlier := (changeList at: listIndex + 1) text.

	^ TextDiffBuilder buildDisplayPatchFrom: earlier to: later inClass: class prettyDiffs: self showingPrettyDiffs! !

!ClassCommentVersionsBrowser methodsFor: 'menu' stamp: 'topa 3/18/2015 15:39'!
listSelectionVersionsMenu: aMenu

	^ aMenu addTranslatedList: #(
		('compare to current'		compareToCurrentVersion		'compare selected version to the current version')
		('revert to selected version'	fileInSelections					'resubmit the selected version, so that it becomes the current version'));
	yourself
! !

!ClassCommentVersionsBrowser methodsFor: 'menu' stamp: 'asm 8/13/2002 21:02'!
offerVersionsHelp
	(StringHolder new contents: self versionsHelpString)
		openLabel: 'Class Comment Versions Browsers'! !

!ClassCommentVersionsBrowser methodsFor: 'misc' stamp: 'asm 8/13/2002 22:14'!
priorSourceOrNil
	"If the currently-selected method has a previous version, return its source, else return nil"
	| aClass aSelector  changeRecords |
	(aClass := self selectedClass) ifNil: [^ nil].
	(aSelector := self selectedMessageName) ifNil: [^ nil].
	changeRecords :=  self class commentRecordsOf: self selectedClass.
	(changeRecords == nil or: [changeRecords size <= 1]) ifTrue: [^ nil].
	^ (changeRecords at: 2) string 
! !

!ClassCommentVersionsBrowser methodsFor: 'basic function' stamp: 'asm 8/13/2002 21:28'!
reformulateList

     classOfMethod organization classComment ifNil: [^ self].

	self scanVersionsOf: classOfMethod.
	self changed: #list. "for benefit of mvc"
	listIndex := 1.
	self changed: #listIndex.
	self contentsChanged! !

!ClassCommentVersionsBrowser methodsFor: 'basic function' stamp: 'fbs 5/6/2011 21:08'!
scanVersionsOf: class 
	"Scan for all past versions of the class comment of the given class"

	| oldCommentRemoteStr sourceFilesCopy position prevPos stamp preamble tokens prevFileIndex |

	classOfMethod := class.
	oldCommentRemoteStr := class  organization commentRemoteStr.
	currentCompiledMethod := oldCommentRemoteStr.
	selectorOfMethod := #Comment.
	changeList := OrderedCollection new.
	list := OrderedCollection new.
	listIndex := 0.
	oldCommentRemoteStr ifNil:[^ nil] ifNotNil: [oldCommentRemoteStr sourcePointer].

	sourceFilesCopy := SourceFiles collect:
		[:x | x isNil ifTrue: [ nil ]
				ifFalse: [x readOnlyCopy]].
	position := oldCommentRemoteStr position.
	file := sourceFilesCopy at: oldCommentRemoteStr sourceFileNumber.
	[position notNil & file notNil]
		whileTrue:
		[file position: (0 max: position-150).  " Skip back to before the preamble"
		[file position < (position-1)]  "then pick it up from the front"
			whileTrue: [preamble := file nextChunk].

		prevPos := nil.
		stamp := ''.
		(preamble findString: 'commentStamp:' startingAt: 1) > 0
			ifTrue: [tokens := Scanner new scanTokens: preamble.
				(tokens at: tokens size-3) = #commentStamp:
				ifTrue: ["New format gives change stamp and unified prior pointer"
						stamp := tokens at: tokens size-2.
						prevPos := tokens last.
						prevFileIndex := sourceFilesCopy fileIndexFromSourcePointer: prevPos.
						prevPos := sourceFilesCopy filePositionFromSourcePointer: prevPos]]
			ifFalse: ["The stamp get lost, maybe after a condenseChanges"
					stamp := '<historical>'].

 		self addItem:
				(ChangeRecord new file: file position: position type: #classComment
						class: class name category: nil meta: class isMeta stamp: stamp)
			text: stamp , ' ' , class name , ' class comment'. 
		prevPos = 0 ifTrue:[prevPos := nil].
		position := prevPos.
		prevPos notNil 
					ifTrue:[file := sourceFilesCopy at: prevFileIndex]].
	sourceFilesCopy do: [:x | x notNil ifTrue: [x close]].
	listSelections := Array new: list size withAll: false! !

!ClassCommentVersionsBrowser methodsFor: 'misc' stamp: 'asm 8/13/2002 20:59'!
selectedClass
	"Answer the class currently selected in the browser.  In the case of a VersionsBrowser, the class and selector are always the same, regardless of which version is selected and indeed whether or not any entry is selected in the list pane"

	^ classOfMethod! !

!ClassCommentVersionsBrowser methodsFor: 'basic function' stamp: 'asm 8/13/2002 21:33'!
updateListsAndCodeIn: aWindow
	| aComment |
	aComment := classOfMethod organization commentRemoteStr.
	aComment == currentCompiledMethod
		ifFalse:
			["Do not attempt to formulate if there is no source pointer.
			It probably means it has been recompiled, but the source hasn't been written
			(as during a display of the 'save text simply?' confirmation)."
			aComment last ~= 0 ifTrue: [self reformulateList]].
	^ true
! !

!ClassCommentVersionsBrowser methodsFor: 'misc' stamp: 'sw 8/15/2002 22:35'!
wantsPrettyDiffOption
	"Answer whether pretty-diffs are meaningful for this tool"

	^ false! !

!VersionsBrowser class methodsFor: 'instance creation' stamp: 'sd 3/30/2005 21:53'!
browseVersionsForClass: aClass selector: aSelector
	self
		browseVersionsOf: (aClass compiledMethodAt: aSelector)
		class: aClass
		meta: aClass isMeta
		category: (aClass organization categoryOfElement: aSelector)
		selector: aSelector! !

!VersionsBrowser class methodsFor: 'instance creation' stamp: 'di 1/11/2000 12:45'!
browseVersionsOf: method class: class meta: meta category: msgCategory selector: selector 
	^ self browseVersionsOf: method class: class meta: meta category: msgCategory selector: selector lostMethodPointer: nil! !

!VersionsBrowser class methodsFor: 'instance creation' stamp: 'nk 1/7/2004 10:19'!
browseVersionsOf: method class: class meta: meta category: msgCategory selector: selector lostMethodPointer: sourcePointer 
	| changeList browser |
	Cursor read showWhile:
		[changeList := (browser := self new)
			scanVersionsOf: method class: class meta: meta
			category: msgCategory selector: selector].
	changeList ifNil: [ self inform: 'No versions available'. ^nil ].

	sourcePointer ifNotNil:
		[changeList setLostMethodPointer: sourcePointer].

	self open: changeList name: 'Recent versions of ' ,
selector multiSelect: false.

	^browser! !

!VersionsBrowser class methodsFor: 'instance creation' stamp: 'sw 10/21/1999 17:21'!
timeStampFor: aSelector class: aClass reverseOrdinal: anInteger
	"Answer the time stamp corresponding to some version of the given method, nil if none.  The reverseOrdinal parameter is interpreted as:  1 = current version; 2 = last-but-one version, etc."
	
	| method aChangeList |
	method := aClass compiledMethodAt: aSelector ifAbsent: [^ nil].
	aChangeList := self new
			scanVersionsOf: method class: aClass meta: aClass isMeta
			category: nil selector: aSelector.
	^ aChangeList ifNil: [nil] ifNotNil:
		[aChangeList list size >= anInteger
			ifTrue:
				[(aChangeList changeList at: anInteger) stamp]
			ifFalse:
				[nil]]! !

!VersionsBrowser class methodsFor: 'instance creation' stamp: 'sw 10/19/1999 15:01'!
versionCountForSelector: aSelector class: aClass
	"Answer the number of versions known to the system for the given class and method, including the current version.  A result of greater than one means that there is at least one superseded version.  Answer zero if no logged version can be obtained."
	
	| method aChangeList |
	method := aClass compiledMethodAt: aSelector ifAbsent: [^ 0].
	aChangeList := self new
			scanVersionsOf: method class: aClass meta: aClass isMeta
			category: nil selector: aSelector.
	^ aChangeList ifNil: [0] ifNotNil: [aChangeList list size]! !

!VersionsBrowser methodsFor: 'contents' stamp: 'ul 1/9/2016 22:03'!
aboutToStyle: aStyler

	"Style in source view, or when there's nothing to diff with."
	(self isModeStyleable or: [ listIndex = list size ]) ifFalse: [ ^false ].
	aStyler
		classOrMetaClass: classOfMethod;
		parseAMethod: true.
	^true
	! !

!VersionsBrowser methodsFor: 'init & update' stamp: 'nk 1/7/2004 10:10'!
addedChangeRecord
	^addedChangeRecord! !

!VersionsBrowser methodsFor: 'init & update' stamp: 'nk 1/7/2004 10:23'!
addedChangeRecord: aChangeRecord
	addedChangeRecord := aChangeRecord.
	self reformulateList.! !

!VersionsBrowser methodsFor: 'misc' stamp: 'ul 11/15/2010 12:10'!
addPriorVersionsCountForSelector: aSelector ofClass: aClass to: aStream
	"Add an annotation detailing the prior versions count.  Specially handled here for the case of a selector no longer in the system, whose prior versions are seen in a versions browser -- in this case, the inherited version of this method will not work."

	(aClass includesSelector: aSelector) ifTrue:
		[^ super addPriorVersionsCountForSelector: aSelector ofClass: aClass to: aStream].

	aStream nextPutAll: 
		((changeList size > 0
			ifTrue:
				[changeList size = 1
					ifTrue:
						['Deleted - one prior version']
					ifFalse:
						['Deleted - ', changeList size printString, ' prior versions']]
			ifFalse:
				['surprisingly, no prior versions']), self annotationSeparator)! !

!VersionsBrowser methodsFor: 'init & update' stamp: 'sw 5/6/2000 01:16'!
changeListButtonSpecs

	^#(
		('compare to current'
		compareToCurrentVersion
		'opens a separate window which shows the text differences between the selected version and the current version')

		('revert'
		fileInSelections
		'reverts the method to the version selected')

		('remove from changes'
		removeMethodFromChanges
		'remove this method from the current change set')

		('help'
		offerVersionsHelp
		'further explanation about use of Versions browsers')
		)! !

!VersionsBrowser methodsFor: 'menu' stamp: 'sw 2/27/2001 08:46'!
changeListKey: aChar from: view
	"Respond to a Command key in the list pane. of the versions browser"

	^ self messageListKey: aChar from: view! !

!VersionsBrowser methodsFor: 'misc' stamp: 'sw 8/17/2002 21:57'!
classCommentIndicated
	"Answer whether the receiver is pointed at a class comment"

	^ false! !

!VersionsBrowser methodsFor: 'menu' stamp: 'ar 8/6/2009 19:23'!
compareToOtherVersion
	"Prompt the user for a reference version, then spawn a window 
	showing the diffs between the older and the newer of the current 
	version and the reference version as text."

	| change1 change2 s1 s2 |
	change1 := changeList at: listIndex ifAbsent: [ ^self ].

	change2 := ((UIManager default
				chooseFrom: (list copyWithoutIndex: listIndex)
				values: (changeList copyWithoutIndex: listIndex))) ifNil: [ ^self ].
	
	"compare earlier -> later"
	"change1 timeStamp < change2 timeStamp
		ifFalse: [ | temp | temp := change1. change1 := change2. change2 := temp ]."

	s1 := change1 string.
	s2 := change2 string.
	s1 = s2
		ifTrue: [^ self inform: 'Exact Match' translated].

	(StringHolder new
		textContents: (TextDiffBuilder
				buildDisplayPatchFrom: s1
				to: s2
				inClass: classOfMethod
				prettyDiffs: self showingPrettyDiffs))
		openLabel: (('Comparison from {1} to {2}' translated) format: { change1 stamp. change2 stamp })! !

!VersionsBrowser methodsFor: 'user interface' stamp: 'mt 7/9/2016 08:52'!
defaultWindowColor
	^ (Color r: 0.782 g: 0.677 b: 0.9)! !

!VersionsBrowser methodsFor: 'menu' stamp: 'sw 10/12/1999 17:51'!
fileInSelections 
	super fileInSelections.
	self reformulateList! !

!VersionsBrowser methodsFor: 'menu' stamp: 'eem 7/14/2014 14:45'!
fileOutSelection
	| them it file |
	them := OrderedCollection new.
	listSelections with: changeList do: 
		[:selected :item | selected ifTrue: [them add: item]].
	them size ~= 1
		ifTrue: [self inform: 'single version not selected, so nothing done']
		ifFalse:
			[it := them first.
			 file := FileStream newFileNamed: it methodClassName, (it isMetaClassChange ifTrue: [' class'] ifFalse: ['']), '-' , (it methodSelector copyReplaceAll: ':' with: '').
			 [file header; timeStamp.
			  it fileOutOn: file] ensure: [file close]]! !

!VersionsBrowser methodsFor: 'menu' stamp: 'dew 9/7/2001 00:29'!
findOriginalChangeSet
	| changeSet |
	self currentChange ifNil: [^ self].
	changeSet := self currentChange originalChangeSetForSelector: self selectedMessageName.
	changeSet = #sources ifTrue:
		[^ self inform: 'This version is in the .sources file.'].
	changeSet ifNil:
		[^ self inform: 'This version was not found in any changeset nor in the .sources file.'].
	(ChangeSorter new myChangeSet: changeSet) open! !

!VersionsBrowser methodsFor: 'user interface' stamp: 'rkrk 8/24/2009 07:17'!
initialExtent

	"adopt the extent so that the revert button fits in"
	^650@400! !

!VersionsBrowser methodsFor: 'menu' stamp: 'topa 3/18/2015 15:35'!
listSelectionVersionsMenu: aMenu
	"Fill aMenu with menu items appropriate to the receiver"
	<versionsMenu>

	listIndex > 0 ifTrue:[
		(list size > 1 ) ifTrue: [ aMenu addTranslatedList: #(
			('compare to current'		compareToCurrentVersion		'compare selected version to the current version')
			('compare to version...'	compareToOtherVersion		'compare selected version to another selected version'))].
		"Note: Revert to selected should be visible for lists of length one for having the ability to revert to an accidentally deleted method"
		 aMenu addTranslatedList: #(
			('revert to selected version'	fileInSelections					'resubmit the selected version, so that it becomes the current version') 
			('fileOut selected version'	fileOutSelection					'file out the selected version') )].
	^ aMenu! !

!VersionsBrowser methodsFor: 'menu' stamp: 'cmm 10/16/2015 14:29'!
mainVersionsMenu: aMenu
	"Fill aMenu with menu items appropriate to the receiver"
	<versionsMenu>

	aMenu addTranslatedList: #(
		('remove from changes'		removeMethodFromChanges	'remove this method from the current change set, if present')
		('find original change set'	findOriginalChangeSet			'locate the changeset which originally contained this version')
		-
		('toggle diffing (D)'			toggleDiffing					'toggle whether or not diffs should be shown here')
		('update list'				reformulateList					'reformulate the list of versions, in case it somehow got out of synch with reality')
		-
		('senders (n)'				browseSenders					'browse all senders of this selector')
		('implementors (m)'			browseImplementors			'browse all implementors of this selector')
		('inheritance (i)'		methodHierarchy			'browse method inheritance')
		-
		('help...'					offerVersionsHelp				'provide an explanation of the use of this tool')).
											
	^aMenu! !

!VersionsBrowser methodsFor: 'menu' stamp: 'sw 10/12/1999 22:49'!
offerVersionsHelp
	(StringHolder new contents: self versionsHelpString)
		openLabel: 'Versions Browsers'! !

!VersionsBrowser methodsFor: 'toolbuilder' stamp: 'rkrk 8/24/2009 07:08'!
optionalButtonPairs

	^ #(('revert' fileInSelections 'resubmit the selected version, so that it becomes the current version')) , super optionalButtonPairs! !

!VersionsBrowser methodsFor: 'init & update' stamp: 'sw 11/28/1999 22:50'!
reformulateList
	| aMethod |
	"Some uncertainty about how to deal with lost methods here"
	aMethod := classOfMethod compiledMethodAt: selectorOfMethod ifAbsent: [^ self].
	
	self scanVersionsOf: aMethod class: classOfMethod theNonMetaClass meta: classOfMethod isMeta category: (classOfMethod whichCategoryIncludesSelector: selectorOfMethod) selector: selectorOfMethod.
	self changed: #list. "for benefit of mvc"
	listIndex := 1.
	self changed: #listIndex.
	self contentsChanged
! !

!VersionsBrowser methodsFor: 'menu' stamp: 'sd 5/23/2003 14:50'!
removeMethodFromChanges
	"Remove my method from the current change set"

	ChangeSet current removeSelectorChanges: selectorOfMethod class: classOfMethod.
	self changed: #annotation
! !

!VersionsBrowser methodsFor: 'init & update' stamp: 'bf 10/11/2011 16:07'!
scanVersionsOf: method class: class meta: meta category: category selector: selector
	| position prevPos prevFileIndex preamble tokens sourceFilesCopy stamp cat |
	selectorOfMethod := selector.
	currentCompiledMethod := method.
	classOfMethod := meta ifTrue: [class class] ifFalse: [class].
	cat := category ifNil: [''].
	changeList := OrderedCollection new.
	list := OrderedCollection new.
	self addedChangeRecord ifNotNil: [ :change |
		self addItem: change text: ('{1} (in {2})' translated format: { change stamp. change fileName }) ].
	listIndex := 0.
	position := method filePosition.
	sourceFilesCopy := SourceFiles collect:
		[:x | x isNil ifTrue: [ nil ]
				ifFalse: [x readOnlyCopy]].
	method fileIndex = 0 ifTrue: [^ nil].
	file := sourceFilesCopy at: method fileIndex.

	[position notNil & file notNil]
		whileTrue:
		[preamble := method getPreambleFrom: file at: (0 max: position - 3).

		"Preamble is likely a linked method preamble, if we're in
			a changes file (not the sources file).  Try to parse it
			for prior source position and file index"
		prevPos := nil.
		stamp := ''.
		(preamble findString: 'methodsFor:' startingAt: 1) > 0
			ifTrue: [tokens := Scanner new scanTokens: preamble]
			ifFalse: [tokens := Array new  "ie cant be back ref"].
		((tokens size between: 7 and: 8)
			and: [(tokens at: tokens size - 5) = #methodsFor:])
			ifTrue:
				[(tokens at: tokens size - 3) = #stamp:
				ifTrue: ["New format gives change stamp and unified prior pointer"
						stamp := tokens at: tokens size - 2.
						prevPos := tokens last.
						prevFileIndex := sourceFilesCopy fileIndexFromSourcePointer: prevPos.
						prevPos := sourceFilesCopy filePositionFromSourcePointer: prevPos]
				ifFalse: ["Old format gives no stamp; prior pointer in two parts"
						prevPos := tokens at: tokens size - 2.
						prevFileIndex := tokens last].
				cat := tokens at: tokens size - 4.
				(prevPos = 0 or: [prevFileIndex = 0]) ifTrue: [prevPos := nil]].
		((tokens size between: 5 and: 6)
			and: [(tokens at: tokens size - 3) = #methodsFor:])
			ifTrue:
				[(tokens at: tokens size - 1) = #stamp:
				ifTrue: ["New format gives change stamp and unified prior pointer"
						stamp := tokens at: tokens size].
				cat := tokens at: tokens size - 2].
 		self addItem:
				(ChangeRecord new file: file position: position type: #method
						class: class name category: cat meta: meta stamp: stamp)
			text: stamp , ' ' , class name , (meta ifTrue: [' class '] ifFalse: [' ']) , selector, ' {', cat, '}'.
		position := prevPos.
		prevPos notNil ifTrue:
			[file := sourceFilesCopy at: prevFileIndex]].
	sourceFilesCopy do: [:x | x notNil ifTrue: [x close]].
	listSelections := Array new: list size withAll: false! !

!VersionsBrowser methodsFor: 'misc' stamp: 'sw 1/25/2001 07:03'!
selectedClass
	"Answer the class currently selected in the browser.  In the case of a VersionsBrowser, the class and selector are always the same, regardless of which version is selected and indeed whether or not any entry is selected in the list pane"

	^ classOfMethod theNonMetaClass! !

!VersionsBrowser methodsFor: 'misc' stamp: 'sw 1/25/2001 06:26'!
selectedClassOrMetaClass
	"Answer the class or metaclass currently selected in the browser.  In the case of a VersionsBrowser, the class and selector are always the same, regardless of which version is selected and indeed whether or not any entry is selected in the list pane"

	^ classOfMethod! !

!VersionsBrowser methodsFor: 'misc' stamp: 'sw 1/25/2001 06:10'!
selectedMessageName
	"Answer the message name currently selected in the browser.  In the case of a VersionsBrowser, the class and selector are always the same, regardless of which version is selected and indeed whether or not any entry is selected in the list pane"

	^ selectorOfMethod! !

!VersionsBrowser methodsFor: 'misc' stamp: 'sw 10/19/1999 15:04'!
showsVersions
	^ true! !

!VersionsBrowser methodsFor: 'menu' stamp: 'topa 3/18/2015 15:40'!
titleVersionsMenu: aMenu
	"Add a menu title atop aMenu"
	<versionsMenu>
	<menuPriority: 100>
	aMenu title: 'Versions' translated.
	aMenu addStayUpItemSpecial.
	^ aMenu! !

!VersionsBrowser methodsFor: 'init & update' stamp: 'di 11/28/2000 01:25'!
updateListsAndCodeIn: aWindow
	| aMethod |
	aMethod := classOfMethod compiledMethodAt: selectorOfMethod ifAbsent: [^ false].
	aMethod == currentCompiledMethod
		ifFalse:
			["Do not attempt to formulate if there is no source pointer.
			It probably means it has been recompiled, but the source hasn't been written
			(as during a display of the 'save text simply?' confirmation)."
			aMethod last ~= 0 ifTrue: [self reformulateList]].
	^ true
! !

!VersionsBrowser methodsFor: 'menu' stamp: 'nice 12/27/2009 03:10'!
versionFrom: secsSince1901
	
	"Return changeRecord of the version in effect at that time.  Accept in the VersionsBrowser does not use this code."

	changeList do: [:cngRec | | vTime strings |
		(strings := cngRec stamp findTokens: ' ') size > 2 ifTrue: [
				vTime := strings second asDate asSeconds + 
							strings third asTime asSeconds.
				vTime <= secsSince1901 ifTrue: ["this one"
					^ cngRec == changeList first ifTrue: [nil] ifFalse: [cngRec]]]].
	"was not defined that early.  Don't delete the method."
	^ changeList last	"earliest one may be OK"	! !

!VersionsBrowser methodsFor: 'menu' stamp: 'nk 11/25/2003 10:19'!
versionsHelpString
	^ 'Each entry in the list pane represents a version of the source code for the same method; the topmost entry is the current version, the next entry is the next most recent, etc.

To revert to an earlier version, select it (in the list pane) and then do any of the following:
  *  Choose "revert to this version" from the list pane menu.
  *  Hit the "revert" button,
  *  Type ENTER in the code pane
  *  Type cmd-s (alt-s) in the code pane.

The code pane shows the source for the selected version.  If "diffing" is in effect, then differences betwen the selected version and the version before it are pointed out in the pane.  Turn diffing on and off by choosing "toggle diffing" from the list pane menu, or hitting the "diffs" button, or hitting cmd-D when the cursor is over the list pane.

To get a comparison between the selected version and the current version, choose "compare to current" from the list pane menu or hit the "compare to current" button.  (This is meaningless if the current version is selected, and is unnecessary if you''re interested in diffs from between the current version and the next-most-recent version, since the standard in-pane "diff" feature will give you that.)

You can also compare the selected version with any other version using the "compare to version..." menu choice.

If further versions of the method in question have been submitted elsewhere since you launched a particular Versions Browser, it will still stay nicely up-to-date if you''re in Morphic and have asked that smart updating be maintained; if you''re in mvc or in morphic but with smart-updating turned off, a versions browser is only brought up to date when you activate its window (and when you issue "revert" from within it, of course,) and you can also use the "update list" command to make certain the versions list is up to date.

Hit the "remove from changes" button, or choose the corresponding command in the list pane menu, to have the method in question deleted from the current change set.  This is useful if you''ve put debugging code into a method, and now want to strip it out and cleanse your current change set of all memory of the excursion.

Note:  the annotation pane in versions browsers shows information about the *current* version of the method in the image, not about the selected version.'! !

!VersionsBrowser methodsFor: 'menu' stamp: 'topa 3/16/2015 11:30'!
versionsMenu: aMenu

	^ self menu: aMenu for: #(versionsMenu versionsMenuShifted:)
! !

CodeHolder subclass: #ChangeSorter
	instanceVariableNames: 'parent myChangeSet currentClassName currentSelector priorChangeSetList'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Changes'!

!ChangeSorter commentStamp: 'wiz 2/3/2010 23:38' prior: 0!
I display a ChangeSet.  Two of me are in a DualChangeSorter.

aStringOrNil
Instance Variables
	currentClassName:		<aStringOrNil>
	currentSelector:		<aStringOrNil>
	myChangeSet:		<aChangeSet>
	parent:		<aDualChangeSorterOrNil>
	priorChangeSetList:		<aCollection>


currentClassName
	- string parseable into class-name [class] [class trait]
	needs to be fitlered by (self withoutItemAnnotation: currentClassName) to remove pakaging note


currentSelector
	- string parseable into selector-name 
	needs to be fitlered by (self withoutItemAnnotation: currentSelector) to remove pakaging note

myChangeSet
	- name of current changeset
parent
	-the dual changesorter that contains this one. Used for dealing with the other half.
priorChangeSetList
	- holds the current change set list. Used to detect changes in list when a newly generated list no long match the prior list.
	!

ChangeSorter subclass: #ChangeSetBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Changes'!

!ChangeSetBrowser commentStamp: '<historical>' prior: 0!
A tool allowing you to browse the methods of a single change set.!

!ChangeSetBrowser methodsFor: 'initialization' stamp: 'sw 7/27/2001 20:38'!
addModelItemsToWindowMenu: aMenu
	"Add model-related items to the given window menu"

	| oldTarget |
	oldTarget := aMenu defaultTarget.
	aMenu defaultTarget: self.
	aMenu addLine.
	aMenu add: 'rename change set' action: #rename.
	aMenu add: 'make changes go to me' action: #newCurrent.
	aMenu addLine.
	aMenu add: 'file out' action: #fileOut.
	aMenu add: 'browse methods' action: #browseChangeSet.
	aMenu addLine.
	myChangeSet hasPreamble
		ifTrue:
			[aMenu add: 'edit preamble' action: #addPreamble.
			aMenu add: 'remove preamble' action: #removePreamble]
		ifFalse:
			[aMenu add: 'add preamble' action: #addPreamble].

	myChangeSet hasPostscript
		ifTrue:
			[aMenu add: 'edit postscript...' action: #editPostscript.
			aMenu add: 'remove postscript' action: #removePostscript]
		ifFalse:
			[aMenu add: 'add postscript...' action: #editPostscript].
	aMenu addLine.
	
	aMenu add: 'destroy change set' action: #remove.
	aMenu addLine.
	Smalltalk isMorphic ifTrue:
		[aMenu addLine.
		aMenu add: 'what to show...' target: self action: #offerWhatToShowMenu].
	aMenu addLine.
	aMenu add: 'more...' action: #offerShiftedChangeSetMenu.
	aMenu defaultTarget: oldTarget.

	^ aMenu! !

!ChangeSetBrowser methodsFor: 'menu' stamp: 'topa 3/15/2015 23:00'!
changeSetMenuForDropInClassCats: aMenu

	^ aMenu! !

!ChangeSetBrowser methodsFor: 'menu' stamp: 'topa 3/15/2015 22:54'!
changeSetMenuForModification: aMenu

	^ aMenu! !

!ChangeSetBrowser methodsFor: 'menu' stamp: 'topa 3/15/2015 22:50'!
changeSetMenuForOpposite: aMenu

	^ aMenu! !

!ChangeSetBrowser methodsFor: 'menu' stamp: 'topa 3/15/2015 22:52'!
changeSetMenuForPromote: aMenu

	^ aMenu! !

!ChangeSetBrowser methodsFor: 'menu' stamp: 'sw 3/12/2001 14:07'!
offerUnshiftedChangeSetMenu
	"The user chose 'more' from the shifted window menu; go back to the regular window menu"

	self containingWindow ifNotNil: [self containingWindow offerWindowMenu] ! !

!ChangeSetBrowser methodsFor: 'initialization' stamp: 'sw 3/14/2001 10:03'!
wantsAnnotationPane
	"This kind of browser always wants annotation panes, so answer true"

	^ true! !

!ChangeSetBrowser methodsFor: 'initialization' stamp: 'sw 3/9/2001 15:02'!
wantsOptionalButtons
	"Sure, why not?"

	^ true! !

!ChangeSorter class methodsFor: 'deprecated' stamp: 'pk 10/17/2006 08:59'!
allChangeSetNames

	^ ChangesOrganizer allChangeSetNames! !

!ChangeSorter class methodsFor: 'deprecated' stamp: 'pk 10/17/2006 09:00'!
allChangeSets

	^ ChangesOrganizer allChangeSets! !

!ChangeSorter class methodsFor: 'deprecated' stamp: 'pk 10/17/2006 09:12'!
allChangeSetsWithClass: class selector: selector

	^ ChangesOrganizer allChangeSetsWithClass: class selector: selector! !

!ChangeSorter class methodsFor: 'deprecated' stamp: 'pk 10/17/2006 09:19'!
assuredChangeSetNamed: aName

	^ ChangesOrganizer assuredChangeSetNamed: aName! !

!ChangeSorter class methodsFor: 'deprecated' stamp: 'pk 10/17/2006 09:01'!
basicNewChangeSet: newName

	^ ChangesOrganizer basicNewChangeSet: newName! !

!ChangeSorter class methodsFor: 'deprecated' stamp: 'pk 10/17/2006 09:03'!
belongsInAdditions: aChangeSet

	^ ChangesOrganizer belongsInAdditions: aChangeSet! !

!ChangeSorter class methodsFor: 'deprecated' stamp: 'pk 10/17/2006 09:03'!
belongsInAll: aChangeSet

	^ ChangesOrganizer belongsInAll: aChangeSet! !

!ChangeSorter class methodsFor: 'deprecated' stamp: 'pk 10/17/2006 09:03'!
belongsInMyInitials: aChangeSet

	^ ChangesOrganizer belongsInMyInitials: aChangeSet! !

!ChangeSorter class methodsFor: 'deprecated' stamp: 'pk 10/17/2006 09:04'!
belongsInNumbered: aChangeSet

	^ ChangesOrganizer belongsInNumbered: aChangeSet! !

!ChangeSorter class methodsFor: 'deprecated' stamp: 'pk 10/17/2006 09:04'!
belongsInProjectChangeSets: aChangeSet

	^ ChangesOrganizer belongsInProjectChangeSets: aChangeSet! !

!ChangeSorter class methodsFor: 'deprecated' stamp: 'pk 10/17/2006 09:06'!
belongsInProjectsInRelease:  aChangeSet

	^ ChangesOrganizer belongsInProjectsInRelease:  aChangeSet! !

!ChangeSorter class methodsFor: 'deprecated' stamp: 'pk 10/17/2006 09:06'!
belongsInRecentUpdates: aChangeSet

	^ ChangesOrganizer belongsInRecentUpdates: aChangeSet! !

!ChangeSorter class methodsFor: 'browse' stamp: 'ul 11/15/2010 11:35'!
browseChangeSetsWithClass: class selector: selector
	"Put up a menu comprising a list of change sets that hold changes for the given class and selector.  If the user selects one, open a single change-sorter onto it"

	| hits index |
	hits := self allChangeSets select: 
		[:cs | (cs atSelector: selector class: class) ~~ #none].
	hits isEmpty ifTrue: [^ self inform: class name, '.', selector , '
is not in any change set'].
	index := hits size = 1
		ifTrue:	[1]
		ifFalse:	[(UIManager default chooseFrom: (hits collect: [:cs | cs name])
					lines: #())].
	index = 0 ifTrue: [^ self].
	(ChangeSorter new myChangeSet: (hits at: index)) open.
! !

!ChangeSorter class methodsFor: 'browse' stamp: 'ul 11/15/2010 11:35'!
browseChangeSetsWithSelector: aSelector
	"Put up a list of all change sets that contain an addition, deletion, or change of any method with the given selector"

	| hits index |
	hits := self allChangeSets select: 
		[:cs | cs hasAnyChangeForSelector: aSelector].
	hits isEmpty ifTrue: [^ self inform: aSelector , '
is not in any change set'].
	index := hits size = 1
		ifTrue:	[1]
		ifFalse:	[(UIManager default chooseFrom: (hits collect: [:cs | cs name])
					lines: #())].
	index = 0 ifTrue: [^ self].
	(ChangeSetBrowser new myChangeSet: (hits at: index)) open

"ChangeSorter browseChangeSetsWithSelector: #clearPenTrails"
! !

!ChangeSorter class methodsFor: 'deprecated' stamp: 'pk 10/17/2006 09:20'!
buildAggregateChangeSet

	^ ChangesOrganizer buildAggregateChangeSet

	! !

!ChangeSorter class methodsFor: 'deprecated' stamp: 'pk 10/17/2006 09:12'!
changeSetNamed: aName

	^ ChangesOrganizer changeSetNamed: aName! !

!ChangeSorter class methodsFor: 'deprecated' stamp: 'pk 10/17/2006 09:07'!
changeSetNamesInReleaseImage

	^ ChangesOrganizer changeSetNamesInReleaseImage! !

!ChangeSorter class methodsFor: 'deprecated' stamp: 'pk 10/17/2006 09:08'!
changeSetNamesInThreeOh

	^ ChangesOrganizer changeSetNamesInThreeOh! !

!ChangeSorter class methodsFor: 'deprecated' stamp: 'pk 10/17/2006 09:12'!
changeSetsNamedSuchThat: nameBlock

	^ ChangesOrganizer changeSetsNamedSuchThat: nameBlock! !

!ChangeSorter class methodsFor: 'deprecated' stamp: 'pk 10/17/2006 09:12'!
changeSet: aChangeSet containsClass: aClass

	^ ChangesOrganizer changeSet: aChangeSet containsClass: aClass! !

!ChangeSorter class methodsFor: 'deprecated' stamp: 'pk 10/17/2006 09:20'!
countOfChangeSetsWithClass: aClass andSelector: aSelector

	^ ChangesOrganizer countOfChangeSetsWithClass: aClass andSelector: aSelector! !

!ChangeSorter class methodsFor: 'deprecated' stamp: 'pk 10/17/2006 09:16'!
deleteChangeSetsNumberedLowerThan: anInteger

	^ ChangesOrganizer deleteChangeSetsNumberedLowerThan: anInteger! !

!ChangeSorter class methodsFor: 'deprecated' stamp: 'pk 10/17/2006 09:21'!
doesAnyChangeSetHaveClass: aClass andSelector: aSelector

	^ ChangesOrganizer doesAnyChangeSetHaveClass: aClass andSelector: aSelector! !

!ChangeSorter class methodsFor: 'deprecated' stamp: 'pk 10/17/2006 09:13'!
existingOrNewChangeSetNamed: aName

	^ ChangesOrganizer existingOrNewChangeSetNamed: aName! !

!ChangeSorter class methodsFor: 'deprecated' stamp: 'pk 10/17/2006 09:23'!
fileOutChangeSetsNamed: nameList

	^ ChangesOrganizer fileOutChangeSetsNamed: nameList! !

!ChangeSorter class methodsFor: 'deprecated' stamp: 'pk 10/17/2006 09:13'!
gatherChangeSets

	^ ChangesOrganizer gatherChangeSets! !

!ChangeSorter class methodsFor: 'deprecated' stamp: 'pk 10/17/2006 09:13'!
highestNumberedChangeSet

	^ ChangesOrganizer highestNumberedChangeSet
! !

!ChangeSorter class methodsFor: 'class initialization' stamp: 'pk 10/17/2006 09:10'!
initialize
	"ChangeSorter initialize"

	FileList registerFileReader: self.

	self registerInFlapsRegistry.
! !

!ChangeSorter class methodsFor: 'deprecated' stamp: 'pk 10/17/2006 09:13'!
mostRecentChangeSetWithChangeForClass: class selector: selector

	^ ChangesOrganizer mostRecentChangeSetWithChangeForClass: class selector: selector! !

!ChangeSorter class methodsFor: 'deprecated' stamp: 'pk 10/17/2006 09:01'!
newChangeSet

	^ ChangesOrganizer newChangeSet! !

!ChangeSorter class methodsFor: 'deprecated' stamp: 'pk 10/17/2006 09:02'!
newChangeSet: aName

	^ ChangesOrganizer newChangeSet: aName! !

!ChangeSorter class methodsFor: 'deprecated' stamp: 'pk 10/17/2006 09:02'!
newChangesFromStream: aStream named: aName

	^ ChangesOrganizer newChangesFromStream: aStream named: aName! !

!ChangeSorter class methodsFor: 'deprecated' stamp: 'pk 10/17/2006 09:11'!
noteChangeSetsInRelease

	^ ChangesOrganizer noteChangeSetsInRelease! !

!ChangeSorter class methodsFor: 'instance creation' stamp: 'dtl 9/6/2009 19:36'!
open
	"Open a new instance of the receiver's class"

	self new open! !

!ChangeSorter class methodsFor: 'deprecated' stamp: 'pk 10/17/2006 09:14'!
promoteToTop: aChangeSet

	^ ChangesOrganizer promoteToTop: aChangeSet! !

!ChangeSorter class methodsFor: 'browse' stamp: 'ar 8/7/2009 22:19'!
prototypicalToolWindow
	"Answer a window representing a prototypical instance of the receiver"

	^ToolBuilder build: self new! !

!ChangeSorter class methodsFor: 'deprecated' stamp: 'pk 10/17/2006 09:21'!
recentUpdateMarker

	^ ChangesOrganizer recentUpdateMarker! !

!ChangeSorter class methodsFor: 'deprecated' stamp: 'pk 10/17/2006 09:22'!
recentUpdateMarker: aNumber

	^ ChangesOrganizer recentUpdateMarker: aNumber! !

!ChangeSorter class methodsFor: 'deprecated' stamp: 'KR 8/12/2007 10:45'!
registerInFlapsRegistry
	"Register the receiver in the system's flaps registry"
	self environment
		at: #Flaps
		ifPresent: [:cl | cl registerQuad: {#ChangeSorter.		#prototypicalToolWindow.		'Change Set'	 translatedNoop. 'A tool that allows you to view and manipulate all the code changes in a single change set' translatedNoop}
						forFlapNamed: 'Tools']! !

!ChangeSorter class methodsFor: 'deprecated' stamp: 'pk 10/17/2006 09:17'!
removeChangeSetsNamedSuchThat: nameBlock

	^ ChangesOrganizer removeChangeSetsNamedSuchThat: nameBlock! !

!ChangeSorter class methodsFor: 'deprecated' stamp: 'pk 10/17/2006 09:16'!
removeChangeSet: aChangeSet

	^ ChangesOrganizer removeChangeSet: aChangeSet! !

!ChangeSorter class methodsFor: 'deprecated' stamp: 'pk 10/17/2006 09:17'!
removeEmptyUnnamedChangeSets

	^ ChangesOrganizer removeEmptyUnnamedChangeSets! !

!ChangeSorter class methodsFor: 'deprecated' stamp: 'pk 10/17/2006 09:22'!
reorderChangeSets

	^ ChangesOrganizer reorderChangeSets! !

!ChangeSorter class methodsFor: 'deprecated' stamp: 'pk 10/17/2006 09:22'!
secondaryChangeSet

	^ ChangesOrganizer secondaryChangeSet! !

!ChangeSorter class methodsFor: 'deprecated' stamp: 'asm 4/11/2003 12:32'!
unload
	"Unload the receiver from global registries"

	self environment at: #FileList ifPresent: [:cl |
	cl unregisterFileReader: self].
	self environment at: #Flaps ifPresent: [:cl |
	cl unregisterQuadsWithReceiver: self] ! !

!ChangeSorter methodsFor: 'code pane' stamp: 'ar 8/17/2009 23:14'!
aboutToStyle: aStyler
	"This is a notification that aStyler is about to re-style its text.
	Set the classOrMetaClass in aStyler, so that identifiers
	will be resolved correctly.
	Answer true to allow styling to proceed, or false to veto the styling"

	self isModeStyleable ifFalse: [^false].
	self currentSelector ifNil: [^false].
	aStyler classOrMetaClass: self selectedClassOrMetaClass.
	^true! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'sw 3/6/1999 23:22'!
addPreamble
	myChangeSet assurePreambleExists.
	self okToChange ifTrue:
		[currentClassName := nil.
		currentSelector := nil.
		self showChangeSet: myChangeSet]! !

!ChangeSorter methodsFor: 'annotation' stamp: 'sw 2/22/2001 10:35'!
addPriorVersionsCountForSelector: aSelector ofClass: aClass to: aStream
	"Add an annotation detailing the prior versions count.  Specially handled here for the case of a selector no longer in the system, whose prior version is pointed to by the lost-method pointer in the change held on to by the changeset"

	(aClass includesSelector: aSelector) ifTrue:
		[^ super addPriorVersionsCountForSelector: aSelector ofClass: aClass to: aStream].
	aStream nextPutAll:
		((myChangeSet methodInfoFromRemoval: {aClass name. aSelector})
			ifNil:
				['no prior versions']
			ifNotNil:
				['version(s) retrievable here']), self annotationSeparator! !

!ChangeSorter methodsFor: 'annotation' stamp: 'nice 2/15/2011 23:34'!
annotationForPackageforSelector: aSelector ofClass: aClass 
	"Provide a line of content for an annotation pane, representing 
	information about the given selector and class"
	"requestList"
	| aCategory |
	aClass ifNil: [ ^nil].
	aSelector ifNotNil:
		[ aCategory := aClass organization categoryOfElement: aSelector.
		(aCategory notNil and: [ aCategory first = $* ]) 
			ifTrue: [^ aCategory asString]] .
	
	"Ok. So the selector category does not indicate our package. We defer to the class category"
	^ aClass category asString.
	! !

!ChangeSorter methodsFor: 'class list' stamp: 'wiz 2/3/2007 01:14'!
basicClassList
	"Computed.  View should try to preserve selections, even though index changes"

	^ myChangeSet ifNotNil: [myChangeSet changedClassNames] ifNil: [OrderedCollection new]
! !

!ChangeSorter methodsFor: 'message list' stamp: 'wiz 2/3/2007 01:13'!
basicMessageList 

	| probe newSelectors className |
	currentClassName ifNil: [^ #()].
	className := (self withoutItemAnnotation: currentClassName) .
	probe := (className endsWith: ' class')
		ifTrue: [className]
		ifFalse: [className asSymbol].
	newSelectors := myChangeSet selectorsInClass: probe.
	(newSelectors includes: (self selectedMessageName)) 
		ifFalse: [currentSelector := nil].
	^ newSelectors asSortedCollection
! !

!ChangeSorter methodsFor: 'annotation' stamp: 'wiz 2/1/2007 16:56'!
beginNote
"return the string at the beginning of item annotation"
^' {'
! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'tk 4/24/1998 13:27'!
browseChangeSet
	"Open a message list browser on the new and changed methods in the current change set"

	ChangedMessageSet openFor: myChangeSet

! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'cmm 2/17/2010 22:02'!
browseMethodConflicts
	"Check to see if any other change set also holds changes to any methods in the selected change set; if so, open a browser on all such."

	| aList |

	aList := myChangeSet 
		messageListForChangesWhich: [ :aClass :aSelector |
			(ChangesOrganizer allChangeSetsWithClass: aClass selector: aSelector) size > 1
		]
		ifNone: [^ self inform: 'No other change set has changes
for any method in this change set.'].
	
	ToolSet
		browseMessageSet: aList 
		name: 'Methods in "', myChangeSet name, '" that are also in other change sets (', aList size printString, ')'
		autoSelect: nil! !

!ChangeSorter methodsFor: 'message list' stamp: 'Igor.Stasenko 12/20/2009 18:54'!
browseVersions
	"Create and schedule a changelist browser on the versions of the 
	selected message."
	| class selector method category pair sourcePointer |

	(selector := self selectedMessageName) ifNil: [^ self].
	class := self selectedClassOrMetaClass.
	(class includesSelector: selector)
		ifTrue: [method := class compiledMethodAt: selector.
				category := class whichCategoryIncludesSelector: selector.
				sourcePointer := nil]
		ifFalse: [pair := myChangeSet methodInfoFromRemoval: {class name. selector}.
				pair ifNil: [^ nil].
				sourcePointer := pair first.
				method := CompiledMethod toReturnSelfTrailerBytes:
					(CompiledMethodTrailer new sourcePointer: sourcePointer).
				category := pair last].
	VersionsBrowser
		browseVersionsOf: method
		class: self selectedClass meta: class isMeta
		category: category selector: selector
		lostMethodPointer: sourcePointer.
! !

!ChangeSorter methodsFor: 'toolbuilder' stamp: 'ar 9/6/2009 16:01'!
buildWith: builder
	"
		ToolBuilder open: ChangeSorter.
	"
	|  windowSpec |
	windowSpec := builder pluggableWindowSpec new.
	windowSpec label: 'Change Sorter'.
	windowSpec model: self.
	windowSpec children: OrderedCollection new.
	windowSpec label: #labelString.
	self buildWith: builder in: windowSpec rect: (0@0 extent: 1@1).
	^builder build: windowSpec! !

!ChangeSorter methodsFor: 'toolbuilder' stamp: 'sd 11/20/2005 21:26'!
buildWith: builder in: window rect: rect
	| csListHeight msgListHeight csMsgListHeight listSpec textSpec |
	contents := ''.
	csListHeight := 0.25.
	msgListHeight := 0.25.
	csMsgListHeight := csListHeight + msgListHeight.

	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #changeSetList; 
		getSelected: #currentCngSet; 
		setSelected: #showChangeSetNamed:; 
		menu: #changeSetMenu:shifted:; 
		keyPress: #changeSetListKey:from:;
		autoDeselect: false;
		frame: (((0@0 extent: 0.5@csListHeight)
			scaleBy: rect extent) translateBy: rect origin).
	window children add: listSpec.

	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #classList; 
		getSelected: #currentClassName; 
		setSelected: #currentClassName:; 
		menu: #classListMenu:shifted:; 
		keyPress: #classListKey:from:;
		frame: (((0.5@0 extent: 0.5@csListHeight)
			scaleBy: rect extent) translateBy: rect origin).
	window children add: listSpec.

	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #messageList; 
		getSelected: #currentSelector;
		setSelected: #currentSelector:; 
		menu: #messageMenu:shifted:; 
		keyPress: #messageListKey:from:;
		frame: (((0@csListHeight extent: 1@msgListHeight)
			scaleBy: rect extent) translateBy: rect origin).
	window children add: listSpec.

	textSpec := builder pluggableTextSpec new.
	textSpec 
		model: self;
		getText: #contents; 
		setText: #contents:notifying:; 
		selection: #contentsSelection; 
		menu: #codePaneMenu:shifted:;
		frame: (((0@csMsgListHeight corner: 1@1) scaleBy: rect extent) translateBy: rect origin).
	window children add: textSpec.
	^window! !

!ChangeSorter methodsFor: 'access' stamp: 'tk 4/29/1998 08:22'!
changeSet
	^ myChangeSet! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'pk 10/17/2006 09:48'!
changeSetCategories

	^ ChangesOrganizer changeSetCategories! !

!ChangeSorter methodsFor: 'access' stamp: 'sw 1/27/2000 11:19'!
changeSetCurrentlyDisplayed
	^ myChangeSet! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'bf 5/10/2012 20:44'!
changeSetList
	"Answer a list of ChangeSet names to be shown in the change sorter."

	^ChangeSet allChangeSets reverse collect: [:cs | cs name]! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'ar 1/4/2010 01:55'!
changeSetListKey: aChar from: view
	"Respond to a Command key.  I am a model with a listView that has a list of changeSets."

	aChar == $b ifTrue: [^ self browseChangeSet].
	aChar == $B ifTrue: [^ self openChangeSetBrowser].
	aChar == $c ifTrue: [^ self copyAllToOther].
	aChar == $D ifTrue: [^ self toggleDiffing]. 
	aChar == $f ifTrue: [^ self findCngSet].
	aChar == $m ifTrue: [^ self newCurrent].
	aChar == $n ifTrue: [^ self newSet].
	aChar == $o ifTrue: [^ self fileOut].
	aChar == $p ifTrue: [^ self addPreamble].
	aChar == $r ifTrue: [^ self rename].
	aChar == $x ifTrue: [^ self remove].
	aChar == $- ifTrue: [^ self subtractOtherSide].

	^ self messageListKey: aChar from: view! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'topa 3/15/2015 23:00'!
changeSetMenuForDropInClassCats: aMenu
	
	aMenu add: 'remove contained in class categories...' action: #removeContainedInClassCategories.
	aMenu balloonTextForLastItem: ' Drops any changes in given class categories'.

	^ aMenu
! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'topa 3/15/2015 22:54'!
changeSetMenuForModification: aMenu

	aMenu addLine.

	aMenu add: 'file into new...' action: #fileIntoNewChangeSet.
	aMenu balloonTextForLastItem: 
'Load a fileout from disk and place its changes into a new change set (seldom needed -- much better to do this from a file-list browser these days.)'.

	aMenu add: 'reorder all change sets' action: #reorderChangeSets.
	aMenu balloonTextForLastItem:
'Applies a standard reordering of all change-sets in the system -- at the bottom will come the sets that come with the release; next will come all the numbered updates; finally, at the top, will come all other change sets'.


	^ aMenu! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'topa 3/15/2015 22:49'!
changeSetMenuForOpposite: aMenu

	parent ifNotNil:
		[aMenu add: 'conflicts with change set opposite' action: #methodConflictsWithOtherSide.
			aMenu balloonTextForLastItem: 
'Browse all methods that occur both in this change set and in the one on the opposite side of the change sorter.'.].
	^ aMenu! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'topa 3/15/2015 22:52'!
changeSetMenuForPromote: aMenu

	aMenu add: 'promote to top of list' action: #promoteToTopChangeSet.
	aMenu balloonTextForLastItem:
'Make this change set appear first in change-set lists in all change sorters.'.

	^ aMenu! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'topa 3/15/2015 22:56'!
changeSetMenu: aMenu shifted: isShifted 

	^ self menu: aMenu for: #( changeSetMenu changeSetMenuShifted: ) shifted: isShifted
! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'sw 10/30/2000 10:48'!
checkForAlienAuthorship
	"Open a message list browser on all uncommented methods in the current change set that have alien authorship"

	myChangeSet checkForAlienAuthorship

! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'nk 3/30/2002 08:56'!
checkForAnyAlienAuthorship
	"Open a message list browser on all uncommented methods in the current change set that have alien authorship, even historically"

	myChangeSet checkForAnyAlienAuthorship

! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'sw 3/29/2001 12:47'!
checkForUnclassifiedMethods
	"Open a message list browser on all methods in the current change set that have not been categorized"

	myChangeSet checkForUnclassifiedMethods

! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'sw 7/18/2002 17:58'!
checkForUncommentedClasses
	"Open a class list browser on classes in the change set that lack class comments"

	myChangeSet checkForUncommentedClasses! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'sw 10/30/2000 10:39'!
checkForUncommentedMethods
	"Open a message list browser on all uncommented methods in the current change set"

	myChangeSet checkForUncommentedMethods

! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'sw 10/27/1999 14:20'!
checkForUnsentMessages
	"Open a message list browser on all unsent messages in the current change set"

	myChangeSet checkForUnsentMessages

! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'sw 9/28/2006 04:32'!
checkThatSidesDiffer: escapeBlock
	"If the change sets on both sides of the dual sorter are the same, put up an error message and escape via escapeBlock, else proceed happily"

	parent ifNil: [^ escapeBlock value].  "Not relevant unless in dual change sorter."

	(myChangeSet == (parent other: self) changeSet)
		ifTrue:
			[self inform: 
'This command requires that the
change sets selected on the two
sides of the change sorter *not*
be the same.' translated.
			^ escapeBlock value]
! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'ar 8/6/2009 18:47'!
chooseCngSet
	"Present the user with an alphabetical list of change set names, and let her choose one"

	| changeSetsSortedAlphabetically chosen |
	self okToChange ifFalse: [^ self].

	changeSetsSortedAlphabetically := self changeSetList asSortedCollection:
		[:a :b | a asLowercase withoutLeadingDigits < b asLowercase withoutLeadingDigits].

	chosen := UIManager default chooseFrom: changeSetsSortedAlphabetically values: changeSetsSortedAlphabetically.
	chosen ifNil: [^ self].
	self showChangeSet: (ChangesOrganizer changeSetNamed: chosen)! !

!ChangeSorter methodsFor: 'class list' stamp: 'wiz 2/3/2007 01:35'!
classList
	"return the classlist with package note appended."
	
	^ self basicClassList collect: [: each | 
		each asString, (self packageNoteForClass: (Smalltalk classNamed: each) selector: nil) ] .! !

!ChangeSorter methodsFor: 'class list' stamp: 'sw 3/5/2001 18:24'!
classListKey: aChar from: view
	"Respond to a Command key in the class-list pane."

	aChar == $x ifTrue: [^ self removeClass].
	aChar == $d ifTrue: [^ self forgetClass]. 

	^ self messageListKey: aChar from: view "picks up b,h,p"! !

!ChangeSorter methodsFor: 'class list' stamp: 'topa 3/15/2015 23:13'!
classListMenu: aMenu shifted: shifted
	"Fill aMenu with items appropriate for the class list"
	^ self menu: aMenu for: #(classListMenu classListMenuShifted:) shifted: shifted
! !

!ChangeSorter methodsFor: 'class list' stamp: 'sw 2/26/2001 12:00'!
classMenu: aMenu
	"Set up aMenu for the class-list.  Retained for backward compatibility with old change sorters in image segments"

	^ self classListMenu: aMenu shifted: false! !

!ChangeSorter methodsFor: 'class list' stamp: 'sw 3/6/2001 12:40'!
classMenu: aMenu shifted: shifted
	"Fill aMenu with items appropriate for the class list.  Retained for bkwd compatibility"

	^ self classListMenu: aMenu shifted: shifted! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'sw 1/28/1999 12:30'!
clearChangeSet
	"Clear out the current change set, after getting a confirmation."
	| message |

	self okToChange ifFalse: [^ self].
	myChangeSet isEmpty ifFalse:
		[message := 'Are you certain that you want to\forget all the changes in this set?' withCRs.
		(self confirm: message) ifFalse: [^ self]].
	myChangeSet clear.
	self changed: #classList.
	self changed: #messageList.
	self setContents.
	self contentsChanged.
! !

!ChangeSorter methodsFor: 'code pane' stamp: 'ul 11/15/2010 11:34'!
contents: aString notifying: aController 
	"Compile the code in aString. Notify aController of any syntax errors. 
	Create an error if the category of the selected message is unknown. 
	Answer false if the compilation fails. Otherwise, if the compilation 
	created a new method, deselect the current selection. Then answer true."
	| category selector class oldSelector |

	(class := self selectedClassOrMetaClass) ifNil:
		[(myChangeSet preambleString == nil or: [aString size = 0]) ifTrue: [ ^ false].
		(aString count: [:char | char == $"]) odd 
			ifTrue: [self inform: 'unmatched double quotes in preamble']
			ifFalse: [(Scanner new scanTokens: aString) size > 0 ifTrue: [
				self inform: 'Part of the preamble is not within double-quotes.
To put a double-quote inside a comment, type two double-quotes in a row.
(Ignore this warning if you are including a doIt in the preamble.)']].
		myChangeSet preambleString: aString.
		self currentSelector: nil.  "forces update with no 'unsubmitted chgs' feedback"
		^ true].
	oldSelector := self selectedMessageName.
	category := class organization categoryOfElement: oldSelector.
	selector := class compile: aString
				classified: category
				notifying: aController.
	selector ifNil: [^ false].
	(self messageList includes: selector)
		ifTrue: [self currentSelector: selector]
		ifFalse: [self currentSelector: oldSelector].
	self update.
	^ true! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'sw 1/27/2000 11:21'!
copyAllToOther
	"Copy this entire change set into the one on the other side"
	| companionSorter |
	self checkThatSidesDiffer: [^ self].
	(companionSorter := parent other: self) changeSetCurrentlyDisplayed assimilateAllChangesFoundIn: myChangeSet.
	companionSorter changed: #classList.	"Later the changeSet itself will notice..."
	companionSorter changed: #messageList! !

!ChangeSorter methodsFor: 'class list' stamp: 'nb 6/17/2003 12:25'!
copyClassToOther
	"Place these changes in the other changeSet also"

	| otherSorter otherChangeSet |
	self checkThatSidesDiffer: [^ self].
	self okToChange ifFalse: [^ Beeper beep].
	currentClassName ifNil: [^ Beeper beep].
	otherSorter := parent other: self.
	otherChangeSet := otherSorter changeSet.

	otherChangeSet absorbClass: self selectedClassOrMetaClass name from: myChangeSet.
	otherSorter showChangeSet: otherChangeSet.! !

!ChangeSorter methodsFor: 'message list' stamp: 'wiz 2/3/2007 01:32'!
copyMethodToOther
	"Place this change in the other changeSet also"
	| other cls sel |
	self checkThatSidesDiffer: [^ self].
	currentSelector ifNotNil:
		[other := (parent other: self) changeSet.
		cls := self selectedClassOrMetaClass.
		sel := self selectedMessageName.

		other absorbMethod: sel class: cls from: myChangeSet.
		(parent other: self) showChangeSet: other]
! !

!ChangeSorter methodsFor: 'class list' stamp: 'tk 4/24/1998 09:14'!
currentClassName

	^ currentClassName! !

!ChangeSorter methodsFor: 'class list' stamp: 'sw 1/28/1999 12:30'!
currentClassName: aString

	currentClassName := aString.
	currentSelector := nil.	"fix by wod"
	self changed: #currentClassName.
	self changed: #messageList.
	self setContents.
	self contentsChanged.! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'tk 6/5/1998 06:47'!
currentCngSet
	^ myChangeSet name! !

!ChangeSorter methodsFor: 'message list' stamp: 'tk 4/24/1998 09:15'!
currentSelector

	^ currentSelector! !

!ChangeSorter methodsFor: 'message list' stamp: 'sw 1/28/1999 12:31'!
currentSelector: messageName

	currentSelector := messageName.
	self changed: #currentSelector.
	self setContents.
	self contentsChanged.! !

!ChangeSorter methodsFor: 'user interface' stamp: 'mt 7/9/2016 08:52'!
defaultWindowColor
	^ (Color r: 0.719 g: 0.9 b: 0.9)! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'tk 4/28/1998 08:06'!
editPostscript
	"Allow the user to edit the receiver's change-set's postscript -- in a separate window"

	myChangeSet editPostscript! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'tk 4/28/1998 08:06'!
editPreamble
	"Allow the user to edit the receiver's change-set's preamble -- in a separate window."

	myChangeSet editPreamble! !

!ChangeSorter methodsFor: 'annotation' stamp: 'wiz 2/1/2007 16:56'!
endNote
"return the string at the beginning of item annotation"
^'}'
! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'sw 4/19/2000 16:18'!
expungeUniclasses
	"remove all memory of uniclasses in the receiver"

	self okToChange ifFalse: [^ self].
	myChangeSet expungeUniclasses.
	self changed: #classList.
	self changed: #messageList.

! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'nk 1/4/2004 17:07'!
fileIntoNewChangeSet
	"Obtain a file designation from the user, and file its contents into a  
	new change set whose name is a function of the filename. Show the  
	new set and leave the current changeSet unaltered."
	| aNewChangeSet stream |
	self okToChange
		ifFalse: [^ self].
	ChangeSet promptForDefaultChangeSetDirectoryIfNecessary.
	stream := StandardFileMenu oldFileStreamFrom: ChangeSet defaultChangeSetDirectory.
	stream
		ifNil: [^ self].
	aNewChangeSet := self class
				newChangesFromStream: stream
				named: (FileDirectory localNameFor: stream name).
	aNewChangeSet
		ifNotNil: [self showChangeSet: aNewChangeSet]! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'tk 6/10/1999 12:44'!
fileOut
	"File out the current change set."

	myChangeSet fileOut.
	parent modelWakeUp.	"notice object conversion methods created"
! !

!ChangeSorter methodsFor: 'class list' stamp: 'wiz 2/3/2007 01:37'!
fileOutClass
	"this is a hack!!!! makes a new change set, called the class name, adds author initials to try to make a unique change set name, files it out and removes it. kfr 16 june 2000" 
	| aSet |
	"File out the selected class set."
     aSet := self class newChangeSet: (self withoutItemAnnotation: currentClassName).
	aSet absorbClass: self selectedClassOrMetaClass name from: myChangeSet.
	aSet fileOut.
	self class removeChangeSet: aSet.
	parent modelWakeUp.	"notice object conversion methods created"

! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'laza 3/3/2010 11:16'!
findCngSet 
	"Search for a changeSet by name.  Pop up a menu of all changeSets whose name contains the string entered by the user.  If only one matches, then the pop-up menu is bypassed"
	| index pattern candidates nameList |
	self okToChange ifFalse: [^ self].
	pattern := UIManager default request: 'ChangeSet name or fragment?'.
	pattern isEmpty ifTrue: [^ self].
	nameList := self changeSetList asSet.
	candidates := ChangeSet allChangeSets select:
			[:c | (nameList includes: c name) and: 
				[c name includesSubstring: pattern caseSensitive: false]].
	candidates size = 0 ifTrue: [^ Beeper beep].
	candidates size = 1 ifTrue:
		[^ self showChangeSet: candidates first].
	index := UIManager default chooseFrom: (candidates collect: [:each | each name]).
	index = 0 ifFalse: [self showChangeSet: (candidates at: index)].
! !

!ChangeSorter methodsFor: 'message list' stamp: 'di 6/22/1998 02:08'!
forget
	"Drop this method from the changeSet"

	self okToChange ifFalse: [^ self].
	currentSelector ifNotNil: [
		myChangeSet removeSelectorChanges: self selectedMessageName 
			class: self selectedClassOrMetaClass.
		currentSelector := nil.
		self showChangeSet: myChangeSet]! !

!ChangeSorter methodsFor: 'class list' stamp: 'eem 3/20/2013 12:34'!
forgetClass
	"Remove all mention of this class from the changeSet.
	 After forgetting, select at the same point in the class
	 list, which helps if e.g. doing something repetitive such
	 as moving a number of classes to the other side."
	| index classList |
	self okToChange ifFalse: [^ self].
	currentClassName ifNil: [^self].
	index := self classList indexOf: currentClassName.
	myChangeSet removeClassChanges: (self withoutItemAnnotation: currentClassName).
	currentSelector := nil.
	classList := self classList.
	currentClassName := classList isEmpty ifFalse:
								[classList at: (index min: classList size)].
	self showChangeSet: myChangeSet! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'tk 10/26/1999 14:24'!
goToChangeSetsProject
	"Transport the user to a project which bears the selected changeSet as its current changeSet"

	| aProject |
	(aProject := myChangeSet correspondingProject) 
		ifNotNil:
			[aProject enter: false revert: false saveForRevert: false]
		ifNil:
			[self inform: 'Has no project']! !

!ChangeSorter methodsFor: 'initialize' stamp: 'ar 9/6/2009 16:02'!
initialize
	super initialize.
	myChangeSet := ChangeSet current.! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'sw 1/10/1999 01:01'!
inspectChangeSet
	"Open a message list browser on the new and changed methods in the current change set"

	myChangeSet inspectWithLabel: 'Change set: ', myChangeSet name

! !

!ChangeSorter methodsFor: 'access' stamp: 'tk 4/30/1998 13:37'!
label
	^ self labelString! !

!ChangeSorter methodsFor: 'access' stamp: 'ar 1/4/2010 01:58'!
labelString
	"The label for my entire window.  The large button that displays my name is gotten via mainButtonName"

	^ String streamContents:
		[:aStream |
			aStream nextPutAll: (ChangeSet current == myChangeSet
				ifTrue: ['Changes go to "', myChangeSet name, '"']
				ifFalse: ['ChangeSet: ', myChangeSet name])]! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'sw 2/17/1999 11:05'!
lookForSlips
	"Open a message list browser on the new and changed methods in the current change set"

	myChangeSet lookForSlips

! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'dvf 5/13/2000 05:08'!
mailOut
	"Create a mail with a gzipped attachment holding out the current change 
	set. "
	myChangeSet mailOut.
	parent modelWakeUp! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'tk 4/24/1998 13:10'!
mainButtonName

	^ myChangeSet name! !

!ChangeSorter methodsFor: 'class list' stamp: 'topa 3/16/2015 10:32'!
mainClassListMenu: aMenu
	"Fill aMenu with items appropriate for the class list"

	<classListMenuShifted: false>
	aMenu title: 'class list'.
	aMenu addStayUpItemSpecial.
	parent ifNotNil: [
		aMenu addList: #( "These two only apply to dual change sorters"
			('copy class chgs to other side'			copyClassToOther)	
			('move class chgs to other side'			moveClassToOther))].

	aMenu addList: #(
			-
			('delete class from change set (d)'		forgetClass)
			('remove class from system (x)'			removeClass)
			-
			('browse full (b)'						browseMethodFull)
			('browse hierarchy (h)'					spawnHierarchy)
			('browse protocol (p)'					browseFullProtocol)
			-
			('printOut'								printOutClass)
			('fileOut'								fileOutClass)
			-
			('references... (r)'						browseVariableReferences)
			('assignments... (a)'						browseVariableAssignments)
			('class refs (N)'							browseClassRefs)
			-
			('more...'								offerShiftedClassListMenu)).

	^ aMenu! !

!ChangeSorter methodsFor: 'message list' stamp: 'yo 6/6/2012 15:58'!
messageList 
	| thisClass organization |
	(thisClass := self selectedClassOrMetaClass) ifNil: [^ #() ] .
	organization := thisClass organization.
	^self basicMessageList collect: [ :each |
		each asString , (self noteString: (organization categoryOfElement: each))].
! !

!ChangeSorter methodsFor: 'class list' stamp: 'sw 3/5/2001 18:30'!
messageListKey: aChar from: view
	"Respond to a Command key in the message-list pane."

	aChar == $d ifTrue: [^ self forget].
	super messageListKey: aChar from: view! !

!ChangeSorter methodsFor: 'message list' stamp: 'sw 3/9/2001 14:27'!
messageListMenu: aMenu shifted: shifted
	"Fill aMenu with items appropriate for the message list; could be for a single or double changeSorter"

	^ self messageMenu: aMenu shifted: shifted! !

!ChangeSorter methodsFor: 'message list' stamp: 'topa 3/15/2015 21:04'!
messageMenu: aMenu shifted: shifted
	"Fill aMenu with items appropriate for the message list; could be for a single or double changeSorter"

	^ self menu: aMenu for: #(messageListMenu messageListMenuShifted:) shifted: shifted! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'cmm 2/17/2010 22:02'!
methodConflictsWithOtherSide
	"Check to see if the change set on the other side shares any methods with the selected change set; if so, open a browser on all such."

	| aList other |

	self checkThatSidesDiffer: [^ self].
	other := (parent other: self) changeSet.
	aList := myChangeSet 
		messageListForChangesWhich: [ :aClass :aSelector |
			aClass notNil and: [(other methodChangesAtClass: aClass name) includesKey: aSelector]
		]
		ifNone:  [^ self inform: 'There are no methods that appear
both in this change set and
in the one on the other side.'].
	
	ToolSet 
		browseMessageSet: aList 
		name: 'Methods in "', myChangeSet name, '" that are also in ', other name,' (', aList size printString, ')'
		autoSelect: nil! !

!ChangeSorter methodsFor: 'access' stamp: 'sma 11/11/2000 23:28'!
modelWakeUp
	"A window with me as model is being entered.
	Make sure I am up-to-date with the changeSets."

	self canDiscardEdits ifTrue: [self update]! !

!ChangeSorter methodsFor: 'class list' stamp: 'nb 6/17/2003 12:25'!
moveClassToOther
	"Place class changes in the other changeSet and remove them from this one"

	self checkThatSidesDiffer: [^ self].
	(self okToChange and: [currentClassName notNil]) ifFalse: [^ Beeper beep].

	self copyClassToOther.
	self forgetClass! !

!ChangeSorter methodsFor: 'message list' stamp: 'wiz 7/22/2007 22:13'!
moveMethodToOther
	"Place this change in the other changeSet and remove it from this side"

	| other cls sel |
	self checkThatSidesDiffer: [^self].
	self okToChange ifFalse: [^Beeper beep].
	currentSelector ifNotNil: 
			[other := (parent other: self) changeSet.
			other == myChangeSet ifTrue: [^Beeper  beep].
			cls := self selectedClassOrMetaClass.
			sel := self selectedMessageName.
			other 
				absorbMethod: sel
				class: cls
				from: myChangeSet.
			(parent other: self) showChangeSet: other.
			self forget	"removes the method from this side"]! !

!ChangeSorter methodsFor: 'access' stamp: 'tk 4/24/1998 08:43'!
myChangeSet: anObject
	myChangeSet := anObject! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'sd 5/23/2003 15:15'!
newCurrent
	"make my change set be the current one that changes go into"

	ChangeSet  newChanges: myChangeSet.
	self update.  "Because list of changes in a category may thus have changed"
	self changed: #relabel.! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'ar 1/4/2010 01:59'!
newSet
	"Create a new changeSet and show it., making it the current one.  Reject name if already in use."

	| aSet |
	self okToChange ifFalse: [^ self].
	aSet := self class newChangeSet.
	aSet ifNotNil:[
		self update.
		self showChangeSet: aSet.
		self changed: #relabel]! !

!ChangeSorter methodsFor: 'annotation' stamp: 'wiz 2/1/2007 17:21'!
noteString: aString
^ self beginNote , aString asString , self endNote! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'sw 2/27/2001 21:55'!
offerShiftedChangeSetMenu
	"Offer the shifted version of the change set menu"

	self offerMenuFrom: #changeSetMenu:shifted: shifted: true! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'sw 3/6/2001 14:41'!
offerUnshiftedChangeSetMenu
	"Offer the unshifted version of the change set menu"

	self offerMenuFrom: #changeSetMenu:shifted: shifted: false! !

!ChangeSorter methodsFor: 'creation' stamp: 'ar 8/7/2009 22:23'!
open
	"ChangeSorterPluggable new open"
	^ToolBuilder open: self! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'ar 8/7/2009 22:23'!
openChangeSetBrowser
	"Open a ChangeSet browser on the current change set"
	ToolBuilder open: (ChangeSetBrowser new myChangeSet: myChangeSet)! !

!ChangeSorter methodsFor: 'annotation' stamp: 'wiz 2/3/2007 15:39'!
packageNoteForClass: aClass selector: aSelector 
"return the category name that represents the package name for aClass>>aSelector.
when selector is nil or in a normal catagory return "
| package |
package := (self annotationForPackageforSelector: aSelector
			ofClass: aClass) ifNil: ['<class was deleted???>'] .

^ self noteString: package






! !

!ChangeSorter methodsFor: 'access' stamp: 'tk 4/24/1998 08:42'!
parent
	^ parent! !

!ChangeSorter methodsFor: 'access' stamp: 'tk 4/24/1998 08:42'!
parent: anObject
	parent := anObject! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'sw 12/13/2003 18:14'!
promoteToTopChangeSet
	"Move the selected change-set to the top of the list"

	self class promoteToTop: myChangeSet.
	(parent ifNil: [self]) modelWakeUp! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'di 6/14/1998 12:00'!
remove
	"Completely destroy my change set.  Check if it's OK first"

	self okToChange ifFalse: [^ self].
	self removePrompting: true.
	self update! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'rbb 3/1/2005 10:29'!
removeContainedInClassCategories
	| matchExpression |
	myChangeSet removePreamble.
	matchExpression :=  UIManager default request: 'Enter class category name (wildcard is ok)' initialAnswer: 'System-*'. 
	(Smalltalk organization categories
		select: [:each | matchExpression match: each])
		do: [:eachCat | 
			| classNames | 
			classNames := Smalltalk organization listAtCategoryNamed: eachCat.
			classNames
				do: [:eachClassName | 
					myChangeSet removeClassChanges: eachClassName.
					myChangeSet removeClassChanges: eachClassName , ' class'].
			self showChangeSet: myChangeSet]! !

!ChangeSorter methodsFor: 'message list' stamp: 'di 6/21/1998 23:13'!
removeFromCurrentChanges
	"Redisplay after removal in case we are viewing the current changeSet"

	super removeFromCurrentChanges.
	currentSelector := nil.
	self showChangeSet: myChangeSet! !

!ChangeSorter methodsFor: 'message list' stamp: 'ul 11/15/2010 11:34'!
removeMessage
	"Remove the selected msg from the system. Real work done by the 
	parent, a ChangeSorter"
	| confirmation sel |
	self okToChange
		ifFalse: [^ self].
	currentSelector
		ifNotNil: [confirmation := self systemNavigation   confirmRemovalOf: (sel := self selectedMessageName) on: self selectedClassOrMetaClass.
			confirmation = 3
				ifTrue: [^ self].
			self selectedClassOrMetaClass removeSelector: sel.
			self update.
			confirmation = 2
				ifTrue: [self systemNavigation browseAllCallsOn: sel]]! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'sw 6/29/1999 20:53'!
removePostscript
	(myChangeSet hasPostscript and: [myChangeSet postscriptHasDependents]) ifTrue:
		[^ self inform:
'Cannot remove the postscript right
now because there is at least one
window open on that postscript.
Close that window and try again.'].

	myChangeSet removePostscript.
	self showChangeSet: myChangeSet! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'sw 3/5/1999 19:32'!
removePreamble
	myChangeSet removePreamble.
	self showChangeSet: myChangeSet! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'bf 5/8/2013 17:40'!
removePrompting: doPrompt
	"Completely destroy my change set.  Check if it's OK first, and if doPrompt is true, get the user to confirm his intentions first."

	| message aName changeSetNumber msg |
	aName := myChangeSet name.
	myChangeSet okayToRemove ifFalse: [^ self]. "forms current changes for some project"
	(myChangeSet isEmpty or: [doPrompt not]) ifFalse:
		[message := 'Are you certain that you want to 
remove (destroy) the change set
named  "', aName, '" ?'.
		(self confirm: message) ifFalse: [^ self]].

	doPrompt ifTrue:
		[msg := myChangeSet hasPreamble
			ifTrue:
				[myChangeSet hasPostscript
					ifTrue:
						['a preamble and a postscript']
					ifFalse:
						['a preamble']]
			ifFalse:
				[myChangeSet hasPostscript
					ifTrue:
						['a postscript']
					ifFalse:
						['']].
		msg isEmpty ifFalse:
			[(self confirm: 
'Caution!!  This change set has
', msg, ' which will be
lost if you destroy the change set.
Do you really want to go ahead with this?') ifFalse: [^ self]]].

	"Go ahead and remove the change set"
	changeSetNumber := myChangeSet name initialIntegerOrNil.
	changeSetNumber ifNotNil: [SystemVersion current unregisterUpdate: changeSetNumber].
	ChangesOrganizer removeChangeSet: myChangeSet.
	self changed: #changeSetList.
	self showChangeSet: ChangeSet current.! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'nice 6/18/2013 22:19'!
rename
	"Store a new name string into the selected ChangeSet.  reject duplicate name; allow user to back out"

	| newName |
	newName := UIManager default request: 'New name for this change set'
						initialAnswer: myChangeSet name.
	(newName = myChangeSet name or: [newName size = 0]) ifTrue:
			[^ Beeper beep].

	(self class changeSetNamed: newName) ifNotNil:
			[^ UIManager default inform: 'Sorry that name is already used'].

	myChangeSet name: newName.
	self update.
	self changed: #mainButtonName.
	self changed: #relabel.! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'sw 3/5/2001 11:03'!
reorderChangeSets
	"apply a standard reordering -- let the class handle this"

	^ self class reorderChangeSets! !

!ChangeSorter methodsFor: 'class list' stamp: 'sw 12/7/1998 09:43'!
selectedClass
	"Answer the currently-selected class.  If there is no selection, or if the selection refers to a class no longer extant, return nil"
	| c |
	^ currentClassName ifNotNil: [(c := self selectedClassOrMetaClass)
		ifNotNil: [c theNonMetaClass]]! !

!ChangeSorter methodsFor: 'traits' stamp: 'wiz 2/3/2007 01:34'!
selectedClassOrMetaClass
	"Careful, the class may have been removed!!"

	| cName tName className |
	currentClassName ifNil: [^ nil].
	className := (self withoutItemAnnotation: currentClassName) .
	(className endsWith: ' class')
		ifTrue: [cName := (className copyFrom: 1 to: className size-6) asSymbol.
				^ (Smalltalk at: cName ifAbsent: [^nil]) class].
	(currentClassName endsWith: ' classTrait')
		ifTrue: [tName := (className copyFrom: 1 to: className size-11) asSymbol.
				^ (Smalltalk at: tName ifAbsent: [^nil]) classTrait].
	cName := className asSymbol.
	^ Smalltalk at: cName ifAbsent: [nil]! !

!ChangeSorter methodsFor: 'message list' stamp: 'wiz 2/3/2007 01:33'!
selectedMessageName

	currentSelector ifNil: [^ nil].
	^ (self withoutItemAnnotation: currentSelector) asSymbol! !

!ChangeSorter methodsFor: 'code pane' stamp: 'nice 3/29/2010 21:06'!
setContents
	"return the source code that shows in the bottom pane"

	| sel class strm changeType | 
	self clearUserEditFlag.
	currentClassName ifNil: [^ contents := myChangeSet preambleString ifNil: ['']].
	class := self selectedClassOrMetaClass.
	(sel := self selectedMessageName) == nil
		ifFalse: [changeType := (myChangeSet atSelector: (sel := sel asSymbol) class: class).
			changeType == #remove
				ifTrue: [^ contents := 'Method has been removed (see versions)'].
			changeType == #addedThenRemoved
				ifTrue: [^ contents := 'Added then removed (see versions)'].
			class ifNil: [^ contents := 'Method was added, but cannot be found!!'].
			(class includesSelector: sel)
				ifFalse: [^ contents := 'Method was added, but cannot be found!!'].
			contents := class sourceCodeAt: sel.
			(#(prettyPrint prettyDiffs) includes: contentsSymbol) ifTrue:
				[contents :=  class prettyPrinterClass
					format: contents in: class notifying: nil].
			self showingAnyKindOfDiffs
				ifTrue: [contents := self diffFromPriorSourceFor: contents].
			^ contents := contents asText makeSelectorBoldIn: class]
		ifTrue: [strm := WriteStream on: (String new: 100).
			(myChangeSet classChangeAt: (self withoutItemAnnotation: currentClassName)) do:
				[:each |
				each = #remove ifTrue: [strm nextPutAll: 'Entire class was removed.'; cr].
				each = #addedThenRemoved ifTrue: [strm nextPutAll: 'Class was added then removed.'].
				each = #rename ifTrue: [strm nextPutAll: 'Class name was changed.'; cr].
				each = #add ifTrue: [strm nextPutAll: 'Class definition was added.'; cr].
				each = #change ifTrue: [strm nextPutAll: 'Class definition was changed.'; cr].
				each = #reorganize ifTrue: [strm nextPutAll: 'Class organization was changed.'; cr].
				each = #comment ifTrue: [strm nextPutAll: 'New class comment.'; cr.
				]].
			^ contents := strm contents].! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'ar 8/6/2009 20:39'!
setRecentUpdatesMarker
	"Allow the user to change the recent-updates marker"

	| result |
	result := UIManager default request: 
('Enter the lowest change-set number
that you wish to consider "recent"?
(note: highest change-set number
in this image at this time is ', ChangeSet highestNumberedChangeSet asString, ')') initialAnswer: self class recentUpdateMarker asString.
	(result notNil and: [result startsWithDigit]) ifTrue:
		[self class recentUpdateMarker: result asInteger.
		Smalltalk isMorphic ifTrue: [SystemWindow wakeUpTopWindowUponStartup]]! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'topa 3/15/2015 23:00'!
shiftedChangeSetMenu: aMenu
	"Set up aMenu to hold items relating to the change-set-list pane when the shift key is down"

	<changeSetMenuShifted: true>

	aMenu title: 'Change set (shifted)'.
	aMenu addStayUpItemSpecial.

	"CONFLICTS SECTION"
	aMenu add: 'conflicts with other change sets' action: #browseMethodConflicts.
	aMenu balloonTextForLastItem: 
'Browse all methods that occur both in this change set and in at least one other change set.'.
	self changeSetMenuForOpposite: aMenu.
	aMenu addLine.

	"CHECKS SECTION"
	aMenu add: 'check for slips' action: #lookForSlips.
	aMenu balloonTextForLastItem: 
'Check this change set for halts and references to Transcript.'.

	aMenu add: 'check for unsent messages' action: #checkForUnsentMessages.
	aMenu balloonTextForLastItem:
'Check this change set for messages that are not sent anywhere in the system'.

	aMenu add: 'check for uncommented methods' action: #checkForUncommentedMethods.
	aMenu balloonTextForLastItem:
'Check this change set for methods that do not have comments'.

	aMenu add: 'check for uncommented classes' action: #checkForUncommentedClasses.
	aMenu balloonTextForLastItem:
'Check for classes with code in this changeset which lack class comments'.

	Utilities authorInitialsPerSe isEmptyOrNil ifFalse:
		[aMenu add: 'check for other authors' action: #checkForAlienAuthorship.
		aMenu balloonTextForLastItem:
'Check this change set for methods whose current authoring stamp does not start with "', Utilities authorInitials, '"'.

	aMenu add: 'check for any other authors' action: #checkForAnyAlienAuthorship.
	aMenu balloonTextForLastItem:
'Check this change set for methods any of whose authoring stamps do not start with "', Utilities authorInitials, '"'].

	aMenu add: 'check for uncategorized methods' action: #checkForUnclassifiedMethods.
	aMenu balloonTextForLastItem:
'Check to see if any methods in the selected change set have not yet been assigned to a category.  If any are found, open a browser on them.'.
	aMenu addLine.

	aMenu add: 'inspect change set' action: #inspectChangeSet.
	aMenu balloonTextForLastItem: 
'Open an inspector on this change set. (There are some details in a change set which you don''t see in a change sorter.)'.

	aMenu add: 'update' action: #update.
	aMenu balloonTextForLastItem: 
'Update the display for this change set.  (This is done automatically when you activate this window, so is seldom needed.)'.

	aMenu add: 'go to change set''s project' action: #goToChangeSetsProject.
	aMenu balloonTextForLastItem: 
'If this change set is currently associated with a Project, go to that project right now.'.

	self changeSetMenuForPromote: aMenu.

	aMenu add: 'trim history' action: #trimHistory.
	aMenu balloonTextForLastItem: 
' Drops any methods added and then removed, as well as renaming and reorganization of newly-added classes.  NOTE: can cause confusion if later filed in over an earlier version of these changes'.

	self changeSetMenuForDropInClassCats: aMenu.
	
	aMenu add: 'clear this change set' action: #clearChangeSet.
	aMenu balloonTextForLastItem: 
'Reset this change set to a pristine state where it holds no information. CAUTION: this is destructive and irreversible!!'.
	aMenu add: 'expunge uniclasses' action: #expungeUniclasses.
	aMenu balloonTextForLastItem:
'Remove from the change set all memory of uniclasses, e.g. classes added on behalf of etoys, fabrik, etc., whose classnames end with a digit.'.

	aMenu add: 'uninstall this change set' action: #uninstallChangeSet.
	aMenu balloonTextForLastItem: 
'Attempt to uninstall this change set. CAUTION: this may not work completely and is irreversible!!'.

	self changeSetMenuForModification: aMenu.

	aMenu addLine.

	aMenu add: 'more...' action: #offerUnshiftedChangeSetMenu.
	aMenu balloonTextForLastItem: 
'Takes you back to the primary change-set menu.'.

	^ aMenu! !

!ChangeSorter methodsFor: 'class list' stamp: 'topa 3/15/2015 23:11'!
shiftedClassListMenu: aMenu
	"Fill aMenu with items appropriate for the class list"
	<classListMenuShifted: true>

	aMenu title: 'class list'.
	aMenu addStayUpItemSpecial.

	aMenu addList: #(
			-
			('unsent methods'						browseUnusedMethods)
			('unreferenced inst vars'				showUnreferencedInstVars)
			('unreferenced class vars'				showUnreferencedClassVars)
			-
			('sample instance'						makeSampleInstance)
			('inspect instances'						inspectInstances)
			('inspect subinstances'					inspectSubInstances)
			-
			('more...'								offerUnshiftedClassListMenu )).

	^ aMenu! !

!ChangeSorter methodsFor: 'message list' stamp: 'topa 3/16/2015 15:18'!
shiftedMessageMenu: aMenu
	"Arm the menu so that it holds items appropriate to the message-list while the shift key is down.  Answer the menu."
	<messageListMenuShifted: true>
	^ aMenu addList: #(
		-
		('toggle diffing (D)'					toggleDiffing)
		('implementors of sent messages'		browseAllMessages)
		('change category...'				changeCategory)
			-
		('sample instance'					makeSampleInstance)
		('inspect instances'					inspectInstances)
		('inspect subinstances'				inspectSubInstances)
		-
		('change sets with this method'		findMethodInChangeSets)
		('revert to previous version'			revertToPreviousVersion)
		('revert & remove from changes'	revertAndForget));
	yourself
! !

!ChangeSorter methodsFor: 'access' stamp: 'pk 10/17/2006 09:37'!
showChangeSetNamed: aName

	self showChangeSet: (ChangesOrganizer changeSetNamed: aName) ! !

!ChangeSorter methodsFor: 'access' stamp: 'sw 1/28/1999 12:31'!
showChangeSet: chgSet

	myChangeSet == chgSet ifFalse: [
		myChangeSet := chgSet.
		currentClassName := nil.
		currentSelector := nil].
	self changed: #relabel.
	self changed: #currentCngSet.	"new -- list of sets"
	self changed: #mainButtonName.	"old, button"
	self changed: #classList.
	self changed: #messageList.
	self setContents.
	self contentsChanged.! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'sd 11/20/2005 21:26'!
submergeIntoOtherSide
	"Copy the contents of the receiver to the other side, then remove the receiver -- all after checking that all is well."
	| other message nextToView i all |
	self checkThatSidesDiffer: [^ self].
	self okToChange ifFalse: [^ self].
	other := (parent other: self) changeSet.
	other == myChangeSet ifTrue: [^ self inform: 'Both sides are the same!!'].
	myChangeSet isEmpty ifTrue: [^ self inform: 'Nothing to copy.  To remove,
simply choose "remove".'].

	myChangeSet okayToRemove ifFalse: [^ self].
	message := 'Please confirm:  copy all changes
in "', myChangeSet name, '" into "', other name, '"
and then destroy the change set
named "', myChangeSet name, '"?'.
 
	(self confirm: message) ifFalse: [^ self].

	(myChangeSet hasPreamble or: [myChangeSet hasPostscript]) ifTrue:
		[(self confirm: 
'Caution!!  This change set has a preamble or
a postscript or both.  If you submerge it into
the other side, these will be lost.
Do you really want to go ahead with this?') ifFalse: [^ self]].

	other assimilateAllChangesFoundIn: myChangeSet.
	all := ChangeSet allChangeSets.
	nextToView := ((all includes: myChangeSet)
		and: [(i := all indexOf: myChangeSet) < all size])
		ifTrue: [all at: i+1]
		ifFalse: [other].

	self removePrompting: false.
	self showChangeSet: nextToView.
	parent modelWakeUp.
! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'sw 7/8/1999 12:32'!
subtractOtherSide
	"Subtract the changes found on the other side from the requesting side."
	self checkThatSidesDiffer: [^ self].
	myChangeSet forgetAllChangesFoundIn: ((parent other: self) changeSet).
	self showChangeSet: myChangeSet! !

!ChangeSorter methodsFor: 'code pane' stamp: 'sw 11/13/2001 07:34'!
toggleDiffing
	"Toggle whether diffs should be shown in the code pane"

	self okToChange ifTrue:
		[super toggleDiffing.
		self changed: #contents.
		self update]

! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'di 5/12/2000 15:03'!
trimHistory
	"Drop non-essential history (rename, reorg, method removals) from newly-added classes."

	myChangeSet trimHistory

! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'di 3/8/2000 14:18'!
uninstallChangeSet
	"Attempt to uninstall the current change set, after confirmation."

	self okToChange ifFalse: [^ self].
	(self confirm: 'Uninstalling a changeSet is unreliable at best.
It will only work if the changeSet consists only of single
changes, additions and removals of methods, and if
no subsequent changes have been to any of them.
No changes to classes will be undone.
The changeSet will be cleared after uninstallation.
Do you still wish to attempt to uninstall this changeSet?')
	ifFalse: [^ self].

	myChangeSet uninstall.
	self changed: #relabel.
	self changed: #classList.
	self changed: #messageList.
	self setContents.
	self contentsChanged.
! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'topa 3/15/2015 21:47'!
unshiftedChangeSetMenu: aMenu
	"Set up aMenu to hold commands for the change-set-list pane.  This could be for a single or double changeSorter"
	<changeSetMenuShifted: false>
	Smalltalk isMorphic
		ifTrue:
			[aMenu title: 'Change Set']
		ifFalse:
			[aMenu title: 'Change Set:
' , myChangeSet name].
	aMenu addStayUpItemSpecial.

	aMenu add: 'make changes go to me (m)' action: #newCurrent.
	aMenu addLine.
	aMenu add: 'new change set... (n)' action: #newSet.
	aMenu add: 'find...(f)' action: #findCngSet.
	aMenu add: 'select change set...' action: #chooseCngSet.
	aMenu addLine.
	aMenu add: 'rename change set (r)' action: #rename.
	aMenu add: 'file out (o)' action: #fileOut.
	aMenu add: 'mail to list' action: #mailOut.
	aMenu add: 'browse methods (b)' action: #browseChangeSet.
	aMenu add: 'browse change set (B)' action: #openChangeSetBrowser.
	aMenu addLine.
	parent
		ifNotNil: 
			[aMenu add: 'copy all to other side (c)' action: #copyAllToOther.
			aMenu add: 'submerge into other side' action: #submergeIntoOtherSide.
			aMenu add: 'subtract other side (-)' action: #subtractOtherSide.
			aMenu addLine].
	myChangeSet hasPreamble
		ifTrue: 
			[aMenu add: 'edit preamble (p)' action: #addPreamble.
			aMenu add: 'remove preamble' action: #removePreamble]
		ifFalse: [aMenu add: 'add preamble (p)' action: #addPreamble].
	myChangeSet hasPostscript
		ifTrue: 
			[aMenu add: 'edit postscript...' action: #editPostscript.
			aMenu add: 'remove postscript' action: #removePostscript]
		ifFalse: [aMenu add: 'add postscript...' action: #editPostscript].
	aMenu addLine.

	aMenu add: 'destroy change set (x)' action: #remove.
	aMenu addLine.
	aMenu add: 'more...' action: #offerShiftedChangeSetMenu.
	^ aMenu! !

!ChangeSorter methodsFor: 'message list' stamp: 'cmm 10/16/2015 14:14'!
unshiftedMessageMenu: aMenu
	"Fill aMenu with items appropriate for the message list; could be for a single or double changeSorter"

	<messageListMenuShifted: false>

	aMenu title: 'message list'.
	aMenu addStayUpItemSpecial.

	parent ifNotNil:
		[aMenu addList: #(
			('copy method to other side'			copyMethodToOther)
			('move method to other side'			moveMethodToOther))].

	aMenu addList: #(
			('delete method from changeSet (d)'	forget)
			-
			('remove method from system (x)'	removeMessage)
				-
			('browse full (b)'					browseMethodFull)
			('browse hierarchy (h)'				spawnHierarchy)
			('browse protocol (p)'				browseFullProtocol)
			-
			('fileOut'							fileOutMessage)
			('printOut'							printOutMessage)
			-
			('senders of... (n)'					browseSendersOfMessages)
			('implementors of... (m)'				browseMessages)
			('inheritance (i)'					methodHierarchy)
			('versions (v)'						browseVersions)).
	^ aMenu
! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'di 6/21/1998 13:02'!
update
	"recompute all of my panes"

	self updateIfNecessary.
	parent ifNotNil: [(parent other: self) updateIfNecessary]! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'ar 1/4/2010 01:59'!
updateIfNecessary
	"Recompute all of my panes."

	| newList |
	self okToChange ifFalse: [^ self].

	myChangeSet ifNil: [^ self].  "Has been known to happen though shouldn't"
	(myChangeSet isMoribund) ifTrue:
		[self changed: #changeSetList.
		^ self showChangeSet: ChangeSet current].

	newList := self changeSetList.

	(priorChangeSetList == nil or: [priorChangeSetList ~= newList])
		ifTrue:
			[priorChangeSetList := newList.
			self changed: #changeSetList].
	self showChangeSet: myChangeSet! !

!ChangeSorter methodsFor: 'creation' stamp: 'tk 12/7/1999 12:53'!
veryDeepFixupWith: deepCopier

	super veryDeepFixupWith: deepCopier.
	parent := deepCopier references at: parent ifAbsent: [parent].
	self updateIfNecessary! !

!ChangeSorter methodsFor: 'creation' stamp: 'tk 12/7/1999 12:51'!
veryDeepInner: deepCopier
	"Copy all of my instance variables.  Some need to be not copied at all, but shared."

super veryDeepInner: deepCopier.
"parent := parent.		Weakly copied"
"myChangeSet := myChangeSet.		Weakly copied"
currentClassName := currentClassName veryDeepCopyWith: deepCopier.
"currentSelector := currentSelector.		Symbol"
priorChangeSetList := priorChangeSetList veryDeepCopyWith: deepCopier.


! !

!ChangeSorter methodsFor: 'code pane' stamp: 'JW 2/2/2001 21:41'!
wantsOptionalButtons
	"No optional buttons for ChangeSorter"
	^false! !

!ChangeSorter methodsFor: 'annotation' stamp: 'wiz 2/1/2007 17:17'!
withoutItemAnnotation: aStringOrNil
"return the current item without the package annotation we added on"
| endItemIndex |
aStringOrNil ifNil: [^nil] .
( endItemIndex := aStringOrNil findString: self beginNote) = 0
	ifTrue: [^ aStringOrNil ] .
^ aStringOrNil first: endItemIndex - 1! !

!CodeHolder class methodsFor: 'controls' stamp: 'nice 5/8/2011 10:16'!
addContentsSymbolQuint: quint afterEntry: aSymbol 
	"Register a menu selection item in the position after the entry with
	selection symbol aSymbol."

	"CodeHolder
		addContentsSymbolQuint: #(#altSyntax #toggleAltSyntax #showingAltSyntaxString 'altSyntax' 'alternative syntax')
		afterEntry: #colorPrint"

	(ContentsSymbolQuints
		anySatisfy: [:e | (e isKindOf: Collection) and: [e first = quint first]])
		ifFalse: [
			| entry |
			entry := ContentsSymbolQuints
						detect: [:e | (e isKindOf: Collection) and: [e first = aSymbol]].
			ContentsSymbolQuints add: quint after: entry.
			^ self].
	self notify: 'entry already exists for ', quint first! !

!CodeHolder class methodsFor: 'controls' stamp: 'dtl 12/9/2009 13:59'!
addContentsSymbolQuint: quint afterPosition: index
	"Register a menu selection item in the position after index."

	"CodeHolder
		addContentsSymbolQuint: #(#altSyntax #toggleAltSyntax #showingAltSyntaxString 'altSyntax' 'alternative syntax')
		afterPosition: 1"

	| entry |
	entry := ContentsSymbolQuints at: index.
	self contentsSymbolQuints add: quint after: entry! !

!CodeHolder class methodsFor: 'controls' stamp: 'dtl 12/9/2009 14:02'!
addEditContentsOption: anAssociation

	"CodeHolder addEditContentsOption: #translateToC -> #selectedTranslateToC"

	EditContentsOptions add: anAssociation! !

!CodeHolder class methodsFor: 'preferences' stamp: 'eem 7/1/2010 09:24'!
canUseMultiWindowBrowsers
	"This is a hook to allow browsers to control whether to respond to the muli-window preference.
	 Currenty CodeHolder cannot usefully use multi-windows, but may be able to do so in the future."
	^false! !

!CodeHolder class methodsFor: 'controls' stamp: 'ar 1/4/2010 00:27'!
defaultContentsSymbolQuints
	"Default list of quintuplets representing information on the alternative views available in the code pane
		first element:	the contentsSymbol used
		second element:	the selector to call when this item is chosen.
		third element:	the selector to call to obtain the wording of the menu item.
		fourth element:	the wording to represent this view
		fifth element:	balloon help
	A hypen indicates a need for a seperator line in a menu of such choices"

	^ {
		{#source
			. #togglePlainSource 
			. #showingPlainSourceString 
			. 'source'
			. 'the textual source code as written' translated} .
		{#documentation
			. #toggleShowDocumentation
			. #showingDocumentationString
			. 'documentation'
			. 'the first comment in the method' translated} .

		#- .
		{#prettyPrint
			. #togglePrettyPrint
			. #prettyPrintString
			. 'prettyPrint'
			. 'the method source presented in a standard text format' translated} .

		#- .
		{#showDiffs
			. #toggleRegularDiffing
			. #showingRegularDiffsString
			. 'showDiffs'
			. 'the textual source diffed from its prior version' translated} .

		#- .
		{#decompile
			. #toggleDecompile
			. #showingDecompileString
			. 'decompile'
			. 'source code decompiled from byteCodes' translated} .
		{#byteCodes
			. #toggleShowingByteCodes
			. #showingByteCodesString
			. 'byteCodes'	
			. 'the bytecodes that comprise the compiled method' translated} .
	}! !

!CodeHolder class methodsFor: 'controls' stamp: 'dtl 2/4/2007 09:59'!
defaultEditContentsOptions
	"An array of associations of current display mode symbol to selector
	that creates the edit contents for that display mode. The default selector
	is #selectedMessage; this is a list of alternative to the default."

	^ {
		#byteCodes -> #selectedBytecodes
	}! !

!CodeHolder class methodsFor: 'class initialization' stamp: 'dtl 12/9/2009 14:15'!
initialize

	"CodeHolder initialize"

	ContentsSymbolQuints := self defaultContentsSymbolQuints asOrderedCollection.
	EditContentsOptions := Dictionary new.
	self defaultEditContentsOptions
		do: [:opt | EditContentsOptions at: opt key put: opt value]! !

!CodeHolder class methodsFor: 'controls' stamp: 'dtl 12/9/2009 14:00'!
removeContentsSymbol: aSymbol 
	"Unregister the menu selection item with selection symbol aSymbol."

	"CodeHolder removeContentsSymbol: #altSyntax"

	| entries |
	entries := ContentsSymbolQuints
		select: [:e | (e isKindOf: Collection) and: [e first = aSymbol]].
	ContentsSymbolQuints removeAll: entries.
	^ entries
! !

!CodeHolder class methodsFor: 'preferences' stamp: 'tpr 8/12/2016 16:57'!
useMultiWindowBrowsers
	<preference: 'Multi-window browsers'
		category: 'browsing'
		description: 'When enabled, the browser uses a multi-window to conserve real-estate.  e.g. enable then click in the window label of a browser to see the items being browsed.'
		type: #Boolean>
	^MultiWindowBrowsers ifNil: [false]! !

!CodeHolder class methodsFor: 'preferences' stamp: 'eem 6/30/2010 21:37'!
useMultiWindowBrowsers: aBoolean
	MultiWindowBrowsers := aBoolean! !

!CodeHolder methodsFor: 'contents' stamp: 'ar 8/17/2009 23:13'!
aboutToStyle: aStyler
	"This is a notification that aStyler is about to re-style its text.
	The default is to answer false to veto the styling"

	^false! !

!CodeHolder methodsFor: 'toolbuilder' stamp: 'laza 3/2/2010 09:20'!
addCodeProvenanceButtonTo: panelSpec using: builder
	panelSpec children add: (self buildCodeProvenanceButtonWith: builder)! !

!CodeHolder methodsFor: 'misc' stamp: 'yo 2/17/2005 18:14'!
addModelItemsToWindowMenu: aMenu
	"Add model-related item to the window menu"

	super addModelItemsToWindowMenu: aMenu. 
	Smalltalk isMorphic ifTrue:
		[aMenu addLine.
		aMenu add: 'what to show...' translated target: self action: #offerWhatToShowMenu]! !

!CodeHolder methodsFor: 'annotation' stamp: 'ul 11/15/2010 11:35'!
addPriorVersionsCountForSelector: aSelector ofClass: aClass to: aStream
	"add an annotation detailing the prior versions count"
	| versionsCount |

	versionsCount := VersionsBrowser versionCountForSelector: aSelector class: aClass.
	aStream nextPutAll: 
				((versionsCount > 1
					ifTrue:
						[versionsCount = 2 ifTrue:
							['1 prior version']
							ifFalse:
								[versionsCount printString, ' prior versions']]
					ifFalse:
						['no prior versions']), self annotationSeparator)! !

!CodeHolder methodsFor: 'commands' stamp: 'sd 5/23/2003 14:35'!
adoptMessageInCurrentChangeset
	"Add the receiver's method to the current change set if not already there"

	self setClassAndSelectorIn: [:cl :sel |
		cl ifNotNil:
			[ChangeSet current adoptSelector: sel forClass: cl.
			self changed: #annotation]]
! !

!CodeHolder methodsFor: 'annotation' stamp: 'ar 8/8/2009 17:20'!
annotation
	"Provide a line of content for an annotation pane, representing information about the method associated with the selected class and selector in the receiver."

	|  aSelector aClass |

	((aSelector := self selectedMessageName) == nil or: [(aClass := self selectedClassOrMetaClass) == nil]) ifTrue: [^ ''].
	^ self annotationForSelector: aSelector ofClass: aClass! !

!CodeHolder methodsFor: 'annotation' stamp: 'sw 9/11/2002 21:30'!
annotationForClassCommentFor: aClass
	"Provide a line of content for an annotation pane, given that the receiver is pointing at the clas comment of the given class."

	| aStamp nonMeta |
	aStamp :=  (nonMeta := aClass theNonMetaClass) organization commentStamp.
	^ aStamp
		ifNil:
			[nonMeta name, ' has no class comment']
		ifNotNil:
			['class comment for ', nonMeta name,
				(aStamp = '<historical>'
					ifFalse:
						[' - ', aStamp]
					ifTrue:
						[''])]! !

!CodeHolder methodsFor: 'annotation' stamp: 'sw 8/26/2002 10:19'!
annotationForClassDefinitionFor: aClass
	"Provide a line of content for an annotation pane, given that the receiver is pointing at the class definition of the given class."

	^ 'Class definition for ', aClass name! !

!CodeHolder methodsFor: 'annotation' stamp: 'sw 8/26/2002 10:19'!
annotationForHierarchyFor: aClass
	"Provide a line of content for an annotation pane, given that the receiver is pointing at the hierarchy of the given class."

	^ 'Hierarchy for ', aClass name! !

!CodeHolder methodsFor: 'annotation' stamp: 'nice 2/24/2013 17:48'!
annotationForSelector: aSelector ofClass: aClass 
	"Provide a line of content for an annotation pane, representing  
	information about the given selector and class"
	| separator aStream requestList |
	aSelector == #Comment
		ifTrue: [^ self annotationForClassCommentFor: aClass].
	aSelector == #Definition
		ifTrue: [^ self annotationForClassDefinitionFor: aClass].
	aSelector == #Hierarchy
		ifTrue: [^ self annotationForHierarchyFor: aClass].
	aStream := (String new: 512) writeStream.
	requestList := self annotationRequests.
	separator := requestList size > 1
				ifTrue: [self annotationSeparator]
				ifFalse: [''].
	requestList
		do: [:aRequest | | aString sendersCount aComment aCategory implementorsCount aList stamp | 
			aRequest == #firstComment
				ifTrue: [aComment := aClass firstCommentAt: aSelector.
					aComment isEmptyOrNil
						ifFalse: [aStream nextPutAll: aComment , separator]].
			aRequest == #masterComment
				ifTrue: [aComment := aClass supermostPrecodeCommentFor: aSelector.
					aComment isEmptyOrNil
						ifFalse: [aStream nextPutAll: aComment , separator]].
			aRequest == #documentation
				ifTrue: [aComment := aClass precodeCommentOrInheritedCommentFor: aSelector.
					aComment isEmptyOrNil
						ifFalse: [aStream nextPutAll: aComment , separator]].
			aRequest == #timeStamp
				ifTrue: [stamp := self timeStamp.
					aStream
						nextPutAll: (stamp size > 0
								ifTrue: [stamp , separator]
								ifFalse: ['no timeStamp' , separator])].
			aRequest == #messageCategory
				ifTrue: [aCategory := aClass organization categoryOfElement: aSelector.
					aCategory
						ifNotNil: ["woud be nil for a method no longer present,  
							e.g. in a recent-submissions browser"
							aStream nextPutAll: aCategory , separator]].
			aRequest == #sendersCount
				ifTrue: [sendersCount := (self systemNavigation allCallsOn: aSelector) size.
					sendersCount := sendersCount = 1
								ifTrue: ['1 sender']
								ifFalse: [sendersCount printString , ' senders'].
					aStream nextPutAll: sendersCount , separator].
			aRequest == #implementorsCount
				ifTrue: [implementorsCount := self systemNavigation numberOfImplementorsOf: aSelector.
					implementorsCount := implementorsCount = 1
								ifTrue: ['1 implementor']
								ifFalse: [implementorsCount printString , ' implementors'].
					aStream nextPutAll: implementorsCount , separator].
			aRequest == #priorVersionsCount
				ifTrue: [self
						addPriorVersionsCountForSelector: aSelector
						ofClass: aClass
						to: aStream].
			aRequest == #priorTimeStamp
				ifTrue: [stamp := VersionsBrowser
								timeStampFor: aSelector
								class: aClass
								reverseOrdinal: 2.
					stamp
						ifNotNil: [aStream nextPutAll: 'prior time stamp: ' , stamp , separator]].
			aRequest == #recentChangeSet
				ifTrue: [aString := ChangesOrganizer mostRecentChangeSetWithChangeForClass: aClass selector: aSelector.
					aString size > 0
						ifTrue: [aStream nextPutAll: aString , separator]].
			aRequest == #allChangeSets
				ifTrue: [aList := ChangesOrganizer allChangeSetsWithClass: aClass selector: aSelector.
					aList size > 0
						ifTrue: [aList size = 1
								ifTrue: [aStream nextPutAll: 'only in change set ']
								ifFalse: [aStream nextPutAll: 'in change sets: '].
							aList
								do: [:aChangeSet | aStream nextPutAll: aChangeSet name , ' ']]
						ifFalse: [aStream nextPutAll: 'in no change set'].
					aStream nextPutAll: separator]].
	^ aStream contents! !

!CodeHolder methodsFor: 'toolbuilder' stamp: 'mt 4/30/2015 12:02'!
annotationFrame
	^LayoutFrame new
		leftFraction: 0 offset: 0;
		topFraction: 1 offset: self annotationPaneHeight negated;
		rightFraction: 1 offset: 0;
		bottomFraction: 1 offset: 0! !

!CodeHolder methodsFor: 'toolbuilder' stamp: 'mt 4/30/2015 12:03'!
annotationPaneHeight
	^ (Preferences standardCodeFont height * 3/2) rounded! !

!CodeHolder methodsFor: 'annotation' stamp: 'RAA 1/13/2001 07:20'!
annotationPaneMenu: aMenu shifted: shifted

	^ aMenu 
		labels: 'change pane size'
		lines: #()
		selections: #(toggleAnnotationPaneSize)! !

!CodeHolder methodsFor: 'annotation' stamp: 'sw 9/27/1999 14:13'!
annotationRequests
	^ Preferences defaultAnnotationRequests! !

!CodeHolder methodsFor: 'annotation' stamp: 'sw 2/22/2001 10:02'!
annotationSeparator
	"Answer the separator to be used between annotations"

	^ ' · '! !

!CodeHolder methodsFor: 'annotation' stamp: 'bf 4/12/2011 14:11'!
annotation: aString 
	"The user accepted aString in our annotation pane.  Return false because by default we cannot edit annotations"
	^ false
! !

!CodeHolder methodsFor: 'commands' stamp: 'sd 4/16/2003 09:33'!
browseImplementors
	"Create and schedule a message set browser on all implementors of the currently selected message selector. Do nothing if no message is selected."

	| aMessageName |
	(aMessageName := self selectedMessageName) ifNotNil: 
		[self systemNavigation browseAllImplementorsOf: aMessageName]! !

!CodeHolder methodsFor: 'commands' stamp: 'nk 6/26/2003 21:43'!
browseSenders
	"Create and schedule a message set browser on all senders of the currently selected message selector.  Of there is no message currently selected, offer a type-in"

	self sendQuery: #browseAllCallsOn: to: self systemNavigation! !

!CodeHolder methodsFor: 'toolbuilder' stamp: 'mt 4/19/2015 08:56'!
buildAnnotationPaneWith: builder
	
	| annoSpec |
	annoSpec := builder pluggableInputFieldSpec new.
	annoSpec
		model: self;
		getText: #annotation;
		setText: #annotation:. 
	^ annoSpec! !

!CodeHolder methodsFor: 'construction' stamp: 'sw 1/4/2001 12:34'!
buildClassBrowserEditString: aString 
	"Create and schedule a new class browser for the current selection, with initial textual contents set to aString.  This is used specifically in spawning where a class is established but a method-category is not."

	| newBrowser  |
	newBrowser := Browser new.
	newBrowser setClass: self selectedClassOrMetaClass selector: nil.
	newBrowser editSelection: #newMessage.
	Browser openBrowserView: (newBrowser openOnClassWithEditString: aString)
			label: 'Class Browser: ', self selectedClassOrMetaClass name
! !

!CodeHolder methodsFor: 'toolbuilder' stamp: 'mt 4/19/2015 08:57'!
buildCodePaneWith: builder
	| textSpec top buttonSpec annoSpec |
	self wantsOptionalButtons ifTrue: [
		top := builder pluggablePanelSpec new.
		top children: OrderedCollection new.
		buttonSpec := self buildOptionalButtonsWith: builder.
		buttonSpec frame: self optionalButtonsFrame.
		top children add: buttonSpec].
	textSpec := builder pluggableCodePaneSpec new.
	textSpec 
		model: self;
		getText: #contents; 
		setText: #contents:notifying:; 
		selection: #contentsSelection; 
		menu: #codePaneMenu:shifted:.
	self wantsAnnotationPane ifTrue: [
		top ifNil: [
			top := builder pluggablePanelSpec new.
			top children: OrderedCollection new].
		annoSpec := self buildAnnotationPaneWith: builder.
		annoSpec frame: self annotationFrame.
		top children add: annoSpec].
	top ifNotNil: [
		textSpec frame: self textFrame.
		top children add: textSpec].
	^top ifNil: [textSpec]! !

!CodeHolder methodsFor: 'toolbuilder' stamp: 'mt 4/30/2015 14:25'!
buildCodeProvenanceButtonWith: builder
	| buttonSpec |
	buttonSpec := builder pluggableActionButtonSpec new.
	buttonSpec
		model: self;
		label: #codePaneProvenanceString;
		changeLabelWhen: #contents;
		style: #menuButton;
		action: #offerWhatToShowMenu;
		help: 'Governs what view is shown in the code pane.  Click here to change the view';
		margin: (5@0 corner: 0@0).
	^buttonSpec! !

!CodeHolder methodsFor: 'construction' stamp: 'mt 2/15/2016 19:31'!
buildMorphicCodePaneWith: editString
	"Construct the pane that shows the code.
	Respect the Preference for standardCodeFont."

	| codePane |
	codePane := PluggableTextMorph
				on: self
				text: #contents
				accept: #contents:notifying:
				readSelection: #contentsSelection
				menu: #codePaneMenu:shifted:.
	codePane font: Preferences standardCodeFont.
	editString
		ifNotNil: [codePane editString: editString.
			codePane hasUnacceptedEdits: true].
	^ codePane! !

!CodeHolder methodsFor: 'toolbuilder' stamp: 'topa 6/23/2016 09:47'!
buildOptionalButtonsWith: builder

	| panelSpec |
	panelSpec := builder pluggablePanelSpec new.
	panelSpec children: OrderedCollection new.
	self optionalButtonPairs do:[:spec|
		| buttonSpec |
		buttonSpec := builder pluggableActionButtonSpec new.
		buttonSpec model: self.
		buttonSpec label: spec first.
		buttonSpec action: spec second.
		spec second == #methodHierarchy
			ifTrue:[buttonSpec
				enabled: #inheritanceButtonEnabled;
				color: #inheritanceButtonColor].
		spec second == #browseVersions
			ifTrue:[buttonSpec enabled: #versionsButtonEnabled].
		spec size > 2 ifTrue:[buttonSpec help: spec third].
		panelSpec children add: buttonSpec].

	"What to show"
	panelSpec children add: builder pluggableSpacerSpec new.
	self addCodeProvenanceButtonTo: panelSpec using: builder.

	panelSpec layout: #horizontal. "buttons"
	^panelSpec! !

!CodeHolder methodsFor: 'toolbuilder' stamp: 'topa 4/26/2015 23:33'!
buttonHeight
	^ Preferences standardButtonFont height * 2! !

!CodeHolder methodsFor: 'message category functions' stamp: 'sw 10/8/2001 14:19'!
canShowMultipleMessageCategories
	"Answer whether the receiver is capable of showing multiple message categories"

	^ false! !

!CodeHolder methodsFor: 'categories' stamp: 'nice 12/26/2009 09:46'!
categoryFromUserWithPrompt: aPrompt for: aClass
	"self new categoryFromUserWithPrompt: 'testing' for: SystemDictionary"

	|  labels myCategories reject lines newName menuIndex | 
	labels := OrderedCollection with: 'new...'.
	labels addAll: (myCategories := aClass organization categories asSortedCollection:
		[:a :b | a asLowercase < b asLowercase]).
	reject := myCategories asSet.
	reject
		add: ClassOrganizer nullCategory;
		add: ClassOrganizer default.
	lines := OrderedCollection with: 1 with: (myCategories size + 1).

	aClass allSuperclasses do:
		[:cls |
			| cats |
			cats := cls organization categories reject:
				 [:cat | reject includes: cat].
			cats isEmpty ifFalse:
				[lines add: labels size.
				labels addAll: (cats asSortedCollection:
					[:a :b | a asLowercase < b asLowercase]).
				reject addAll: cats]].

	newName := (labels size = 1 or:
		[menuIndex := (UIManager default chooseFrom: labels lines: lines title: aPrompt).
		menuIndex = 0 ifTrue: [^ nil].
		menuIndex = 1])
			ifTrue:
				[UIManager default request: 'Please type new category name'
					initialAnswer: 'category name']
			ifFalse: 
				[labels at: menuIndex].
	^ newName ifNotNil: [newName asSymbol]! !

!CodeHolder methodsFor: 'categories' stamp: 'sd 2/1/2004 17:55'!
categoryOfCurrentMethod
	"Answer the category that owns the current method.  If unable to determine a category, answer nil."

	| aClass aSelector |
	^ (aClass := self selectedClassOrMetaClass) 
		ifNotNil: [(aSelector := self selectedMessageName) 
			            ifNotNil: [aClass whichCategoryIncludesSelector: aSelector]]! !

!CodeHolder methodsFor: 'categories' stamp: 'sd 2/1/2004 17:55'!
changeCategory
	"Present a menu of the categories of messages for the current class, 
	and let the user choose a new category for the current message"

	| aClass aSelector |
	(aClass := self selectedClassOrMetaClass) ifNotNil:
		[(aSelector := self selectedMessageName) ifNotNil:
			[(self letUserReclassify: aSelector in: aClass) ifTrue:
				["ChangeSet current reorganizeClass: aClass."
				"Decided on further review that the above, when present, could cause more
                    unexpected harm than good"
				self methodCategoryChanged]]]! !

!CodeHolder methodsFor: 'controls' stamp: 'sw 5/19/2001 01:12'!
codePaneProvenanceString
	"Answer a string that reports on code-pane-provenance"

	| symsAndWordings |
	(symsAndWordings := self contentsSymbolQuints) do:
		[:aQuad |
			contentsSymbol == aQuad first ifTrue: [^ aQuad fourth]].
	^ symsAndWordings first fourth "default to plain source, for example if nil as initially"! !

!CodeHolder methodsFor: 'contents' stamp: 'sd 11/20/2005 21:27'!
commentContents
	"documentation for the selected method"

	| poss aClass aSelector |
	^ (poss := (aClass := self selectedClassOrMetaClass)
						ifNil:
							['----']
						ifNotNil:
							[(aSelector := self selectedMessageName)
								ifNil:
									['---']
								ifNotNil:
									[(aClass precodeCommentOrInheritedCommentFor: aSelector)", String cr, String cr, self timeStamp"
"which however misses comments that are between the temps  declaration and the body of the method; those are picked up by ·aClass commentOrInheritedCommentFor: aSelector· but that method will get false positives from comments *anywhere* in the method source"]])
		isEmptyOrNil
			ifTrue:
				[aSelector
					ifNotNil:
						[((aClass methodHeaderFor: aSelector), '

Has no comment') asText makeSelectorBoldIn: aClass]
					ifNil:
						['Hamna']]
			ifFalse:	[aSelector
				ifNotNil: [((aClass methodHeaderFor: aSelector), '

', poss) asText makeSelectorBoldIn: aClass]
				ifNil: [poss]]! !

!CodeHolder methodsFor: 'contents' stamp: 'di 10/1/2001 22:25'!
contents
	"Answer the source code or documentation for the selected method"

	self showingByteCodes ifTrue:
		[^ self selectedBytecodes].

	self showingDocumentation ifTrue:
		[^ self commentContents].

	^ self selectedMessage! !

!CodeHolder methodsFor: 'contents' stamp: 'rhi 12/3/2001 22:25'!
contentsChanged

	super contentsChanged.
	self changed: #annotation! !

!CodeHolder methodsFor: 'contents' stamp: 'fbs 12/29/2013 17:08'!
contentsSymbol
	"Answer a symbol indicating what kind of content should be shown for the method; for normal showing of source code, this symbol is #source.  A nil value in the contentsSymbol slot will be set to #source by this method"

	^ contentsSymbol ifNil:
		[contentsSymbol := SystemBrowser browseWithPrettyPrint
								ifTrue:
									[#prettyPrint]
								ifFalse:
									[#source]]! !

!CodeHolder methodsFor: 'controls' stamp: 'dtl 12/9/2009 13:58'!
contentsSymbolQuints
	"Answer a list of quintuplets representing information on the alternative views available in the code pane
		first element:	the contentsSymbol used
		second element:	the selector to call when this item is chosen.
		third element:	the selector to call to obtain the wording of the menu item.
		fourth element:	the wording to represent this view
		fifth element:	balloon help
	A hypen indicates a need for a seperator line in a menu of such choices"

	^ ContentsSymbolQuints! !

!CodeHolder methodsFor: 'contents' stamp: 'sw 11/29/2000 09:51'!
contentsSymbol: aSymbol
	"Set the contentsSymbol as indicated.  #source means to show source code, #comment means to show the first comment found in the source code"

	contentsSymbol := aSymbol! !

!CodeHolder methodsFor: 'commands' stamp: 'nice 12/26/2009 09:47'!
copyUpOrCopyDown
	"Used to copy down code from a superclass to a subclass or vice-versa in one easy step, if you know what you're doing.  Prompt the user for which class to copy down or copy up to, then spawn a fresh browser for that class, with the existing code planted in it, and with the existing method category also established."

	| aClass aSelector allClasses implementors aMenu |
	Smalltalk isMorphic ifFalse: [^ self inform: 
'Sorry, for the moment you have to be in
Morphic to use this feature.'].

	((aClass := self selectedClassOrMetaClass) isNil or: [(aSelector := self selectedMessageName) == nil]) 
		ifTrue:	[^ Beeper beep].

	allClasses := self systemNavigation hierarchyOfClassesSurrounding: aClass.
	implementors := self systemNavigation hierarchyOfImplementorsOf: aSelector forClass: aClass.
	aMenu := MenuMorph new defaultTarget: self.
	aMenu title: 
aClass name, '.', aSelector, '
Choose where to insert a copy of this method
(blue = current, black = available, red = other implementors'.
	allClasses do:
		[:cl |
			| aColor |
			aColor := cl == aClass
				ifTrue:	[#blue]
				ifFalse:
					[(implementors includes: cl)
						ifTrue:	[#red]
						ifFalse:	[#black]].
			(aColor == #red)
				ifFalse:
					[aMenu add: cl name selector: #spawnToClass: argument: cl]
				ifTrue:
					[aMenu add: cl name selector: #spawnToCollidingClass: argument: cl].
			aMenu lastItem color: (Color colorFrom: aColor)].
	aMenu popUpInWorld! !

!CodeHolder methodsFor: 'message list' stamp: 'nk 6/19/2004 16:50'!
decompiledSourceIntoContents
	"For backwards compatibility."

	^self  decompiledSourceIntoContentsWithTempNames: (Sensor leftShiftDown not) 
! !

!CodeHolder methodsFor: 'message list' stamp: 'bf 3/19/2012 18:41'!
decompiledSourceIntoContentsWithTempNames: showTempNames 
	"Obtain a source string by decompiling the method's code, and place 
	that source string into my contents.
	Also return the string.
	Get temps from source file if showTempNames is true."

	| class selector method |
	class := self selectedClassOrMetaClass.
	selector := self selectedMessageName.
	"Was method deleted while in another project?"
	method := class compiledMethodAt: selector ifAbsent: [^ ''].

	currentCompiledMethod := method.
	contents := (showTempNames
		ifTrue: [method decompileWithTemps]
		ifFalse: [method decompile]) decompileString.
	contents := contents asText makeSelectorBoldIn: class.
	^ contents copy! !

!CodeHolder methodsFor: 'controls' stamp: 'mt 7/23/2016 10:59'!
decorateButtons
	"Change screen feedback for any buttons in the UI of the receiver that may wish it.  Initially, it is only the Inheritance button that is decorated, but one can imagine others."
	self changed: #inheritanceButtonColor.
	self changed: #inheritanceButtonEnabled.
	self changed: #versionsButtonEnabled.! !

!CodeHolder methodsFor: 'annotation' stamp: 'sw 9/28/2001 08:43'!
defaultAnnotationPaneHeight
	"Answer the receiver's preferred default height for new annotation panes."

	^ Preferences parameterAt: #defaultAnnotationPaneHeight ifAbsentPut: [25]! !

!CodeHolder methodsFor: 'annotation' stamp: 'sw 9/28/2001 08:44'!
defaultButtonPaneHeight
	"Answer the user's preferred default height for new button panes."

	^ Preferences parameterAt: #defaultButtonPaneHeight ifAbsentPut: [25]! !

!CodeHolder methodsFor: 'diffs' stamp: 'sw 9/5/2001 13:36'!
defaultDiffsSymbol
	"Answer the code symbol to use when generically switching to diffing"

	^ Preferences diffsWithPrettyPrint 
		ifTrue:
			[#prettyDiffs]
		ifFalse:
			[#showDiffs]! !

!CodeHolder methodsFor: 'self-updating' stamp: 'nk 4/29/2004 12:25'!
didCodeChangeElsewhere
	"Determine whether the code for the currently selected method and class has been changed somewhere else."
	| aClass aSelector aCompiledMethod |
	currentCompiledMethod ifNil: [^ false].

	(aClass := self selectedClassOrMetaClass) ifNil: [^ false].

	(aSelector := self selectedMessageName) ifNil: [^ false].

	self classCommentIndicated
		ifTrue: [^ currentCompiledMethod ~~ aClass organization commentRemoteStr].

	^ (aCompiledMethod := aClass compiledMethodAt: aSelector ifAbsent: [^ false]) ~~ currentCompiledMethod
		and: [aCompiledMethod last ~= 0 "either not yet installed"
				or: [ currentCompiledMethod last = 0 "or these methods don't have source pointers"]]! !

!CodeHolder methodsFor: 'diffs' stamp: 'sw 5/20/2001 21:14'!
diffFromPriorSourceFor: sourceCode 
	"If there is a prior version of source for the selected method, return a diff, else just return the source code"

	| prior |
	^ (prior := self priorSourceOrNil)
		ifNil: [sourceCode]
		ifNotNil: [TextDiffBuilder buildDisplayPatchFrom: prior to: sourceCode inClass: self selectedClass prettyDiffs: self showingPrettyDiffs]! !

!CodeHolder methodsFor: 'contents' stamp: 'dtl 2/3/2007 20:14'!
editContents

	^ self editContentsWithDefault: [self selectedMessage]
! !

!CodeHolder methodsFor: 'contents' stamp: 'dtl 12/9/2009 14:03'!
editContentsWithDefault: aBlock

	| selector |
	selector := EditContentsOptions
		at: self contentsSymbol
		ifAbsent: [^ aBlock value].
	^ self perform: selector
! !

!CodeHolder methodsFor: 'message functions' stamp: 'eem 6/2/2014 07:26'!
exploreMethod
	(self selectedClassOrMetaClass
		compiledMethodAt: self selectedMessageName
		ifAbsent: []) ifNotNil:
			[:method| method explore]! !

!CodeHolder methodsFor: 'misc' stamp: 'nk 4/10/2001 07:52'!
getSelectorAndSendQuery: querySelector to: queryPerformer
	"Obtain a selector relevant to the current context, and then send the querySelector to the queryPerformer with the selector obtained as its argument.  If no message is currently selected, then obtain a method name from a user type-in"

	self getSelectorAndSendQuery: querySelector to: queryPerformer with: { }.
! !

!CodeHolder methodsFor: 'misc' stamp: 'md 3/3/2006 09:21'!
getSelectorAndSendQuery: querySelector to: queryPerformer with: queryArgs
	"Obtain a selector relevant to the current context, and then send the querySelector to the queryPerformer with the selector obtained and queryArgs as its arguments.  If no message is currently selected, then obtain a method name from a user type-in"

	| strm array |
	strm := WriteStream on: (array := Array new: queryArgs size + 1).
	strm nextPut: nil.
	strm nextPutAll: queryArgs.

	self selectedMessageName ifNil: [ | selector |
		selector := UIManager default request: 'Type selector:' initialAnswer: 'flag:'.
		selector := selector copyWithout: Character space.
		^ selector isEmptyOrNil ifFalse: [
			(Symbol hasInterned: selector
				ifTrue: [ :aSymbol |
					array at: 1 put: aSymbol.
					queryPerformer perform: querySelector withArguments: array])
				ifFalse: [ self inform: 'no such selector']
		]
	].

	self selectMessageAndEvaluate: [:selector |
		array at: 1 put: selector.
		queryPerformer perform: querySelector withArguments: array
	]! !

!CodeHolder methodsFor: 'misc' stamp: 'fbs 7/25/2013 07:08'!
informPossiblyCorruptSource

	| sourcesName |
	sourcesName := FileDirectory localNameFor: Smalltalk sourcesName.
	self inform: 'There may be a problem with your sources file!!

The source code for every method should (usually) start with the
method selector but this is not the case with this method!! You may
proceed with caution but it is recommended that you get a new source file.

This can happen if you download the "' , sourcesName  , '" file, 
or the ".changes" file you use, as TEXT. It must be transfered 
in BINARY mode, even if it looks like a text file, 
to preserve the CR line ends.

Mac users: This may have been caused by Stuffit Expander. 
To prevent the files above to be converted to Mac line ends 
when they are expanded, do this: Start the program, then 
from Preferences... in the File menu, choose the Cross 
Platform panel, then select "Never" and press OK. 
Then expand the compressed archive again.

(Occasionally, the source code for a method may legitimately
start with a non-alphabetic character -- for example, Behavior
method #formalHeaderPartsFor:.  In such rare cases, you can
happily disregard this warning.)'! !

!CodeHolder methodsFor: 'toolbuilder' stamp: 'nice 2/9/2010 15:16'!
inheritanceButtonColor
	"Check to see if the currently-viewed method has a super send or an override, and if so, change screen feedback, unless the #decorateBrowserButtons says not to."

	| flags aColor cm |
	cm := currentCompiledMethod.
	((cm isKindOf: CompiledMethod) and: [Preferences decorateBrowserButtons])
		ifFalse: [^Color transparent].

	"This table duplicates the old logic, but adds two new colors for the cases where there is a superclass definition, but this method doesn't call it."

	flags := 0.
	self isThisAnOverride ifTrue: [ flags := flags bitOr: 4 ].
	cm sendsToSuper ifTrue: [ flags := flags bitOr: 2 ].
	self isThereAnOverride ifTrue: [ flags := flags bitOr: 1 ].
	aColor := {
		Color transparent.
		Color tan lighter.
		Color green muchLighter.
		Color blue muchLighter.
		Color red muchLighter.	"has super but doesn't call it"
		(Color r: 0.94 g: 0.823 b: 0.673).	"has sub; has super but doesn't call it"
		Color green muchLighter.
		Color blue muchLighter.
	} at: flags + 1.

	^aColor! !

!CodeHolder methodsFor: 'toolbuilder' stamp: 'topa 6/23/2016 09:49'!
inheritanceButtonEnabled
	"The inheritance button is only enabled when a method is selected"

	^  self selectedMessageName notNil
! !

!CodeHolder methodsFor: 'message functions' stamp: 'eem 6/2/2014 07:26'!
inspectMethod
	(self selectedClassOrMetaClass
		compiledMethodAt: self selectedMessageName
		ifAbsent: []) ifNotNil:
			[:method| method inspect]! !

!CodeHolder methodsFor: 'diffs' stamp: 'topa 3/16/2015 15:54'!
installTextualCodingPane
	"Install text into the code pane"

	| aWindow codePane aPane boundsToUse |
	(aWindow := self containingWindow) ifNil: [self error: 'where''s that window?'].
	codePane := self codeTextMorph ifNil: [self error: 'no code pane'].
	aPane := self buildMorphicCodePaneWith: nil.
	boundsToUse := (codePane bounds origin- (1@1)) corner: (codePane owner bounds corner " (1@1").
	aWindow replacePane: codePane with: aPane.
	aPane vResizing: #spaceFill; hResizing: #spaceFill; borderWidth: 0.
	aPane bounds: boundsToUse.
	aPane owner clipSubmorphs: false.

	self contentsChanged! !

!CodeHolder methodsFor: 'contents' stamp: 'ar 8/17/2009 23:13'!
isModeStyleable
	"determine the current mode can be styled"
	^ self showingSource or: [self showingPrettyPrint]! !

!CodeHolder methodsFor: 'misc' stamp: 'ul 3/9/2015 23:49'!
isThereAnOverride
	"Answer whether any subclass of my selected class implements my selected selector"

	| aName aClass |
	aName := self selectedMessageName ifNil: [^ false].
	aClass := self selectedClassOrMetaClass ifNil: [^ false].
	^aClass isSelectorOverridden: aName! !

!CodeHolder methodsFor: 'misc' stamp: 'ul 3/9/2015 23:48'!
isThisAnOverride
	"Answer whether any superclass of my selected class implements my selected selector"

	| aName aClass |
	aName := self selectedMessageName ifNil: [^ false].
	aClass := self selectedClassOrMetaClass ifNil: [^ false].
	^aClass isSelectorOverride: aName! !

!CodeHolder methodsFor: 'categories' stamp: 'bf 4/12/2012 16:13'!
letUserReclassify: anElement in: aClass
	"Put up a list of categories and solicit one from the user.  
	Answer true if user indeed made a change, else false"
	

	| currentCat newCat |
	currentCat := aClass organization categoryOfElement: anElement.
	newCat := self 
				categoryFromUserWithPrompt: 'choose category (currently "', currentCat, '")' 
				for: aClass.
	(newCat ~~ nil and: [newCat ~= currentCat])
		ifTrue:
			[aClass organization classify: anElement under: newCat suppressIfDefault: false logged: true.
			^ true]
		ifFalse:
			[^ false]! !

!CodeHolder methodsFor: 'categories & search pane' stamp: 'mt 4/12/2015 19:45'!
listPaneWithSelector: aSelector
	"If, among my window's paneMorphs, there is a list pane defined with aSelector as its retriever, answer it, else answer nil"

	| aWindow |
	Smalltalk isMorphic ifFalse: [^ nil].
	^ (aWindow := self containingWindow) ifNotNil:
		[aWindow paneMorphSatisfying:
			[:aMorph | (aMorph isKindOf: PluggableListMorph) and:
				[aMorph getListSelector == aSelector]]]! !

!CodeHolder methodsFor: 'traits' stamp: 'al 4/24/2004 12:46'!
makeSampleInstance
	| aClass nonMetaClass anInstance |
	((aClass := self selectedClassOrMetaClass) isNil or: [aClass isTrait]) ifTrue: [^ self].
	nonMetaClass := aClass theNonMetaClass.
	anInstance := self sampleInstanceOfSelectedClass.
	(anInstance isNil and: [nonMetaClass ~~ UndefinedObject]) ifTrue: 
		[^ self inform: 'Sorry, cannot make an instance of ', nonMetaClass name].

	(Smalltalk isMorphic and: [anInstance isMorph])
		ifTrue:
			[self currentHand attachMorph: anInstance]
		ifFalse:
			[anInstance inspectWithLabel: 'An instance of ', nonMetaClass name]! !

!CodeHolder methodsFor: 'message list menu' stamp: 'mt 1/22/2016 13:10'!
messageListKey: aChar from: view
	"Respond to a Command key.  I am a model with a code pane, and I also
	have a listView that has a list of methods.  The view knows how to get
	the list and selection."
	| sel class |
	aChar == $D ifTrue: [^ self toggleDiffing].
	sel := self selectedMessageName.
	aChar == $m ifTrue:  "These next two put up a type in if no message selected"
		[^ self useSelector: sel orGetSelectorAndSendQuery: #browseAllImplementorsOf: to: self ].
	aChar == $n ifTrue: 
		[^ self useSelector: sel orGetSelectorAndSendQuery: #browseAllCallsOn: to: self ].
	aChar == $d ifTrue: [^ self removeMessageFromBrowser].
	"The following require a class selection"
	(class := self selectedClassOrMetaClass) ifNil: [^ self arrowKey: aChar from: view].
	aChar == $b ifTrue: [^ ToolSet browse: class selector: sel].
	aChar == $N ifTrue: [^ self browseClassRefs].
	aChar == $i ifTrue: [^ self methodHierarchy].
	aChar == $h ifTrue: [^ self classHierarchy].
	aChar == $p ifTrue: [^ self browseFullProtocol].
	aChar == $r ifTrue: [^ self browseVariableReferences].
	aChar == $a ifTrue: [^ self browseVariableAssignments].
	"The following require a method selection"
	sel ifNotNil: 
		[aChar == $o ifTrue: [^ self fileOutMessage].
		aChar == $c ifTrue: [^ self copySelector].
		aChar == $v ifTrue: [^ self browseVersions].
		aChar == $x ifTrue: [^ self removeMessage].
		aChar == $C ifTrue: [ self copyReference ].
		(aChar == $Y and: [self canShowMultipleMessageCategories])
			ifTrue: [^ self showHomeCategory]].
	^ self arrowKey: aChar from: view! !

!CodeHolder methodsFor: 'message list menu' stamp: 'topa 3/16/2015 16:33'!
messageListMenuMore: aMenu
	" The 'more..' link that toggles between shifted and unshifted menus
	in message lists and context stacks "
	<messageListMenuShifted: false>
	<contextStackMenuShifted: false>
	<menuPriority: 1000>
	^ aMenu addList: #(
			-
			('more...'				shiftedYellowButtonActivity));
		yourself
! !

!CodeHolder methodsFor: 'categories' stamp: 'sw 9/27/1999 14:11'!
methodCategoryChanged
	self changed: #annotation! !

!CodeHolder methodsFor: 'misc' stamp: 'sw 9/27/2001 01:26'!
modelWakeUpIn: aWindow
	"The window has been activated.  Respond to possible changes that may have taken place while it was inactive"

	self updateListsAndCodeIn: aWindow.
	self decorateButtons.
	self refreshAnnotation.

	super modelWakeUpIn: aWindow! !

!CodeHolder methodsFor: 'multi-window support' stamp: 'eem 10/17/2010 14:24'!
multiWindowState
	^multiWindowState! !

!CodeHolder methodsFor: 'multi-window support' stamp: 'eem 6/30/2010 19:02'!
multiWindowState: aSavedMultiWindowState
	multiWindowState := aSavedMultiWindowState! !

!CodeHolder methodsFor: 'categories & search pane' stamp: 'sw 12/1/2000 20:44'!
newSearchPane
	"Answer a new search pane for the receiver"

	| aTextMorph |
	aTextMorph := PluggableTextMorph on: self
					text: #lastSearchString accept: #lastSearchString:
					readSelection: nil menu: nil.
	aTextMorph setProperty: #alwaysAccept toValue: true.
	aTextMorph askBeforeDiscardingEdits: false.
	aTextMorph acceptOnCR: true.
	aTextMorph setBalloonText: 'Type here and hit ENTER, and all methods whose selectors match what you typed will appear in the list pane below.'.
	^ aTextMorph! !

!CodeHolder methodsFor: 'commands' stamp: 'sw 5/18/2001 17:51'!
offerMenu
	"Offer a menu to the user from the bar of tool buttons"

	self offerDurableMenuFrom: #messageListMenu:shifted: shifted: false! !

!CodeHolder methodsFor: 'commands' stamp: 'sw 2/27/2001 12:14'!
offerShiftedClassListMenu
	"Offer the shifted class-list menu."

	^ self offerMenuFrom: #classListMenu:shifted: shifted: true! !

!CodeHolder methodsFor: 'commands' stamp: 'sw 2/27/2001 12:15'!
offerUnshiftedClassListMenu
	"Offer the shifted class-list menu."

	^ self offerMenuFrom: #classListMenu:shifted: shifted: false! !

!CodeHolder methodsFor: 'what to show' stamp: 'nice 12/26/2009 09:47'!
offerWhatToShowMenu
	"Offer a menu governing what to show"
	| builder menuSpec |
	builder := ToolBuilder default.
	menuSpec := builder pluggableMenuSpec new.
	self contentsSymbolQuints do: [:aQuint | aQuint == #-
		ifTrue: [menuSpec addSeparator]
		ifFalse: [
			| item |
			item := menuSpec add: (self perform: aQuint third) 
					target: self selector: aQuint second argumentList: #().
			item help: aQuint fifth.
		].
	].
	builder runModal: (builder open: menuSpec).! !

!CodeHolder methodsFor: 'misc' stamp: 'dtl 12/9/2009 16:25'!
okayToAccept
	"Answer whether it is okay to accept the receiver's input"

	self showingDocumentation ifTrue:
		[self inform: 
'Sorry, for the moment you can
only submit changes here when
you are showing source.  Later, you
will be able to edit the isolated comment
here and save it back, but only if YOU
implement it!!.'.
		^ false].

	self showingEditContentsOption ifTrue:
		[self inform: 'Cannot accept ', self contentsSymbol, ' input'.
		^ false].

	self showingAnyKindOfDiffs ifFalse:
		[^ true]. 
	^ (UIManager default chooseFrom: {
		'accept anyway -- I''ll take my chances'.
		'um, let me reconsider'.
	} title:
'Caution!!  You are "showing diffs" here, so 
there is a danger that some of the text in the
code pane is contaminated by the "diff" display') = 1! !

!CodeHolder methodsFor: 'controls' stamp: 'cmm 5/2/2011 09:46'!
optionalButtonPairs
	"Answer a tuple (formerly pairs) defining buttons, in the format:
			button label
			selector to send
			help message"

	| aList |

	aList := #(
	('browse'			browseMethodFull			'view this method in a browser')
	('senders' 			browseSendersOfMessages	'browse senders of...')
	('implementors'		browseMessages				'browse implementors of...')
	('versions'			browseVersions				'browse versions')), 

	(Preferences decorateBrowserButtons
		ifTrue:
			[{#('inheritance'		methodHierarchy 'browse method inheritance
green: sends to super
tan: has override(s)
mauve: both of the above
pink: is an override but doesn''t call super
pinkish tan: has override(s), also is an override but doesn''t call super' )}]
		ifFalse:
			[{#('inheritance'		methodHierarchy			'browse method inheritance')}]),

	#(
	('hierarchy'		classHierarchy				'browse class hierarchy')
	('vars'			browseVariableReferences			'references...')).

	^ aList! !

!CodeHolder methodsFor: 'toolbuilder' stamp: 'bp 4/2/2010 22:39'!
optionalButtonsFrame
	^LayoutFrame new
		leftFraction: 0 offset: 0;
		topFraction: 0 offset: 0;
		rightFraction: 1 offset: 0;
		bottomFraction: 0 offset: self buttonHeight! !

!CodeHolder methodsFor: 'what to show' stamp: 'sw 5/22/2001 16:36'!
prettyPrintString
	"Answer whether the receiver is showing pretty-print"

	^ ((contentsSymbol == #prettyPrint)
		ifTrue:
			['<yes>']
		ifFalse:
			['<no>']), 'prettyPrint'! !

!CodeHolder methodsFor: 'misc' stamp: 'sw 9/27/1999 14:09'!
priorSourceOrNil
	"If the currently-selected method has a previous version, return its source, else return nil"
	| aClass aSelector  changeRecords |
	(aClass := self selectedClassOrMetaClass) ifNil: [^ nil].
	(aSelector := self selectedMessageName) ifNil: [^ nil].
	changeRecords := aClass changeRecordsAt: aSelector.
	(changeRecords == nil or: [changeRecords size <= 1]) ifTrue: [^ nil].
	^ (changeRecords at: 2) string 
! !

!CodeHolder methodsFor: 'toolbuilder' stamp: 'cmm 5/1/2011 22:13'!
receiverClass
	^ self selectedClassOrMetaClass ! !

!CodeHolder methodsFor: 'misc' stamp: 'ul 12/12/2009 14:11'!
refreshAnnotation
	"If the receiver has an annotation pane that does not bear unaccepted edits, refresh it"

	(self dependents detect: [:m | (m inheritsFromAnyIn: #('PluggableTextView' 'PluggableTextMorph')) and: [m getTextSelector == #annotation]] ifNone: [nil]) ifNotNil:
		[:aPane | aPane hasUnacceptedEdits ifFalse:
			[aPane update: #annotation]]! !

!CodeHolder methodsFor: 'misc' stamp: 'md 8/14/2005 17:49'!
refusesToAcceptCode
	"Answer whether receiver, given its current contentsSymbol, could accept code happily if asked to"

	^ (#(byteCodes documentation tiles) includes: self contentsSymbol)! !

!CodeHolder methodsFor: 'misc' stamp: 'tk 9/9/2000 21:08'!
releaseCachedState
	"Can always be found again.  Don't write on a file."
	currentCompiledMethod := nil.! !

!CodeHolder methodsFor: 'commands' stamp: 'nb 6/17/2003 12:25'!
removeClass
	"Remove the selected class from the system, at interactive user request.  Make certain the user really wants to do this, since it is not reversible.  Answer true if removal actually happened."

	| message  className classToRemove result |
	self okToChange ifFalse: [^ false].
	classToRemove := self selectedClassOrMetaClass ifNil: [Beeper beep. ^ false].
	classToRemove := classToRemove theNonMetaClass.
	className := classToRemove name.
	message := 'Are you certain that you
want to REMOVE the class ', className, '
from the system?'.
	(result := self confirm: message)
		ifTrue: 
			[classToRemove subclasses size > 0
				ifTrue: [(self confirm: 'class has subclasses: ' , message)
					ifFalse: [^ false]].
			classToRemove removeFromSystem.
			self changed: #classList.
			true].
	^ result! !

!CodeHolder methodsFor: 'diffs' stamp: 'mt 5/1/2015 14:17'!
restoreTextualCodingPane
	"If the receiver is showing tiles, restore the textual coding pane"

	contentsSymbol == #tiles ifTrue:
		[contentsSymbol := #source.
		self installTextualCodingPane]! !

!CodeHolder methodsFor: 'misc' stamp: 'sw 5/8/2000 12:34'!
sampleInstanceOfSelectedClass
	| aClass |
	"Return a sample instance of the class currently being pointed at"
	(aClass := self selectedClassOrMetaClass) ifNil: [^ nil].
	^ aClass theNonMetaClass initializedInstance! !

!CodeHolder methodsFor: 'categories & search pane' stamp: 'sw 3/7/2001 12:22'!
searchPane
	"Answer the search pane associated with the receiver in its window, or nil if none.  Morphic only"

	^ self textPaneWithSelector: #lastSearchString! !

!CodeHolder methodsFor: 'message list' stamp: 'sw 8/16/2002 23:23'!
selectedBytecodes
	"Answer text to show in a code pane when in showing-byte-codes mode"

	^ (self selectedClassOrMetaClass compiledMethodAt: self selectedMessageName ifAbsent: [^ '' asText]) symbolic asText! !

!CodeHolder methodsFor: 'message list' stamp: 'fbs 6/7/2012 21:06'!
selectedMessage
	"Answer a copy of the source code for the selected message.  This generic version is probably actually never reached, since every subclass probably reimplements and does not send to super.  In time, ideally, most, or all, reimplementors would vanish and all would defer instead to a universal version right here.  Everything in good time."

	| class selector method |
	contents ifNotNil: [^ contents copy].

	self showingDecompile ifTrue:[^ self decompiledSourceIntoContents].

	class := self selectedClassOrMetaClass.
	(class isNil or: [(selector := self selectedMessageName) isNil]) ifTrue: [^ ''].
	method := class compiledMethodAt: selector ifAbsent: [^ ''].	"method deleted while in another project"
	currentCompiledMethod := method.

	^ contents := (self showingDocumentation
		ifFalse: [self sourceStringPrettifiedAndDiffed]
		ifTrue:	[ self commentContents])
			copy asText makeSelectorBoldIn: class! !

!CodeHolder methodsFor: 'categories' stamp: 'sw 3/22/2000 23:04'!
selectedMessageCategoryName
	"Answer the name of the message category of the message of the currently selected context."

	^ self selectedClass organization categoryOfElement: self selectedMessageName! !

!CodeHolder methodsFor: 'misc' stamp: 'rbb 3/1/2005 10:31'!
sendQuery: querySelector to: queryPerformer
	"Apply a query to the primary selector associated with the current context.  If no such selection exists, obtain one by user type-in. Then send querySelector to queryPerformer with the selector as its argument."

	| aSelector aString |
	aSelector := self selectedMessageName ifNil:
		[aString :=UIManager default request: 'Type selector:' initialAnswer: 'flag:'.
		^ aString isEmptyOrNil ifFalse:
			[(Symbol hasInterned: aString ifTrue:
				[:aSymbol | queryPerformer perform: querySelector with: aSymbol])
				ifFalse:
					[self inform: 'no such selector']]].

	queryPerformer perform: querySelector with: aSelector! !

!CodeHolder methodsFor: 'misc' stamp: 'sw 12/28/2000 15:32'!
setClassAndSelectorIn: csBlock
	"Evaluate csBlock with my selected class and and selector as its arguments; provide nil arguments if I don't have a method currently selected"

	| aName |
	(aName := self selectedMessageName)
		ifNil:
			[csBlock value: nil value: nil]
		ifNotNil:
			[csBlock value: self selectedClassOrMetaClass value: aName]
! !

!CodeHolder methodsFor: 'what to show' stamp: 'sw 2/14/2001 15:25'!
setContentsToForceRefetch
	"Set the receiver's contents such that on the next update the contents will be formulated afresh.  This is a critical and obscure difference between Browsers on the one hand and MessageSets on the other, and has over the years been the source of much confusion and much difficulty.  By centralizing the different handling here, we don't need so many idiosyncratic overrides in MessageSet any more"

	contents := nil! !

!CodeHolder methodsFor: 'message list menu' stamp: 'topa 3/16/2015 16:33'!
shiftedMessageListMore: aMenu
	" The 'more..' link that toggles between shifted and unshifted menus
	in message lists and context stacks "
	<messageListMenuShifted: true>
	<contextStackMenuShifted: true>
	<menuPriority: 1000>
	^ aMenu addList: #(
			-
			('more...'				unshiftedYellowButtonActivity));
		yourself
! !

!CodeHolder methodsFor: 'commands' stamp: 'sw 3/6/2001 15:18'!
shiftedYellowButtonActivity
	"Offer the shifted selector-list menu"

	^ self offerMenuFrom: #messageListMenu:shifted: shifted: true! !

!CodeHolder methodsFor: 'what to show' stamp: 'sw 5/20/2001 21:13'!
showByteCodes: aBoolean
	"Get into or out of bytecode-showoing mode"

	self okToChange ifFalse: [^ self changed: #flash].
	aBoolean
		ifTrue:
			[contentsSymbol := #byteCodes]
		ifFalse:
			[contentsSymbol == #byteCodes ifTrue: [contentsSymbol := #source]].
	self contentsChanged! !

!CodeHolder methodsFor: 'what to show' stamp: 'sw 5/20/2001 09:14'!
showDecompile: aBoolean
	"Set the decompile toggle as indicated"

	self contentsSymbol: (aBoolean ifFalse: [#source] ifTrue: [#decompile])! !

!CodeHolder methodsFor: 'diffs' stamp: 'sw 11/13/2001 07:49'!
showDiffs
	"Answer whether the receiver is showing diffs of source code.  The preferred protocol here is #showingRegularDiffs, but this message is still sent by some preexisting buttons so is retained."

	^ contentsSymbol == #showDiffs
! !

!CodeHolder methodsFor: 'diffs' stamp: 'sw 9/5/2001 13:36'!
showDiffs: aBoolean
	"Set whether I'm showing diffs as indicated; use the global preference to determine which kind of diffs to institute."

	self showingAnyKindOfDiffs
		ifFalse:
			[aBoolean ifTrue:
				[contentsSymbol := self defaultDiffsSymbol]]
		ifTrue:
			[aBoolean ifFalse:
				[contentsSymbol := #source]].
	self setContentsToForceRefetch.
	self contentsChanged! !

!CodeHolder methodsFor: 'what to show' stamp: 'sw 12/5/2000 12:25'!
showDocumentation: aBoolean
	"Set the showDocumentation toggle as indicated"

	self contentsSymbol: (aBoolean ifFalse: [#source] ifTrue: [#documentation])! !

!CodeHolder methodsFor: 'diffs' stamp: 'sw 9/5/2001 13:32'!
showingAnyKindOfDiffs
	"Answer whether the receiver is currently set to show any kind of diffs"

	^ #(showDiffs prettyDiffs) includes: contentsSymbol! !

!CodeHolder methodsFor: 'what to show' stamp: 'sw 5/18/2001 18:05'!
showingByteCodes
	"Answer whether the receiver is showing bytecodes"

	^ contentsSymbol == #byteCodes! !

!CodeHolder methodsFor: 'what to show' stamp: 'sw 5/22/2001 18:28'!
showingByteCodesString
	"Answer whether the receiver is showing bytecodes"

	^ (self showingByteCodes
		ifTrue:
			['<yes>']
		ifFalse:
			['<no>']), 'byteCodes'! !

!CodeHolder methodsFor: 'what to show' stamp: 'sw 5/20/2001 06:52'!
showingDecompile
	"Answer whether the receiver should show decompile rather than, say, source code"

	^ self contentsSymbol == #decompile
! !

!CodeHolder methodsFor: 'what to show' stamp: 'sw 5/20/2001 06:50'!
showingDecompileString
	"Answer a string characerizing whether decompilation is showing"

	^ (self showingDecompile
		ifTrue:
			['<yes>']
		ifFalse:
			['<no>']), 'decompile'! !

!CodeHolder methodsFor: 'what to show' stamp: 'sw 12/5/2000 12:12'!
showingDocumentation
	"Answer whether the receiver should show documentation rather than, say, source code"

	^ self contentsSymbol == #documentation
! !

!CodeHolder methodsFor: 'what to show' stamp: 'sw 5/18/2001 20:05'!
showingDocumentationString
	"Answer a string characerizing whether documentation is showing"

	^ (self showingDocumentation
		ifTrue:
			['<yes>']
		ifFalse:
			['<no>']), 'documentation'! !

!CodeHolder methodsFor: 'what to show' stamp: 'dtl 12/9/2009 16:21'!
showingEditContentsOption
	"True if any of the optional EditContentsOptions modes is in effect. This
	includes bytecode display and possibly other display modes."

	^ EditContentsOptions includesKey: self contentsSymbol
! !

!CodeHolder methodsFor: 'what to show' stamp: 'sw 5/18/2001 19:43'!
showingPlainSource
	"Answer whether the receiver is showing plain source"

	^ contentsSymbol == #source! !

!CodeHolder methodsFor: 'what to show' stamp: 'sw 5/22/2001 09:31'!
showingPlainSourceString
	"Answer a string telling whether the receiver is showing plain source"

	^ (self showingPlainSource
		ifTrue:
			['<yes>']
		ifFalse:
			['<no>']), 'source'! !

!CodeHolder methodsFor: 'diffs' stamp: 'sw 5/19/2001 00:07'!
showingPrettyDiffs
	"Answer whether the receiver is showing pretty diffs of source code"

	^ contentsSymbol == #prettyDiffs
! !

!CodeHolder methodsFor: 'diffs' stamp: 'sw 5/22/2001 16:41'!
showingPrettyDiffsString
	"Answer a string representing whether I'm showing pretty diffs"

	^ (self showingPrettyDiffs
		ifTrue:
			['<yes>']
		ifFalse:
			['<no>']), 'prettyDiffs'! !

!CodeHolder methodsFor: 'what to show' stamp: 'sw 5/18/2001 18:36'!
showingPrettyPrint
	"Answer whether the receiver is showing pretty-print"

	^ contentsSymbol == #prettyPrint! !

!CodeHolder methodsFor: 'diffs' stamp: 'sw 11/13/2001 07:07'!
showingRegularDiffs
	"Answer whether the receiver is showing regular diffs of source code"

	^ contentsSymbol == #showDiffs
! !

!CodeHolder methodsFor: 'diffs' stamp: 'sw 11/13/2001 07:43'!
showingRegularDiffsString
	"Answer a string representing whether I'm showing regular diffs"

	^ (self showingRegularDiffs
		ifTrue:
			['<yes>']
		ifFalse:
			['<no>']), 'showDiffs'! !

!CodeHolder methodsFor: 'what to show' stamp: 'sw 12/5/2000 11:48'!
showingSource
	"Answer whether the receiver is currently showing source code"

	^ self contentsSymbol == #source
! !

!CodeHolder methodsFor: 'diffs' stamp: 'sw 5/22/2001 18:25'!
showPrettyDiffs: aBoolean
	"Set whether I'm showing pretty diffs as indicated"

	self showingPrettyDiffs
		ifFalse:
			[aBoolean ifTrue:
				[contentsSymbol := #prettyDiffs]]
		ifTrue:
			[aBoolean ifFalse:
				[contentsSymbol := #source]].
	self setContentsToForceRefetch.
	self contentsChanged! !

!CodeHolder methodsFor: 'diffs' stamp: 'sw 11/13/2001 07:50'!
showRegularDiffs: aBoolean
	"Set whether I'm showing regular diffs as indicated"

	self showingRegularDiffs
		ifFalse:
			[aBoolean ifTrue:
				[contentsSymbol := #showDiffs]]
		ifTrue:
			[aBoolean ifFalse:
				[contentsSymbol := #source]].
	self setContentsToForceRefetch.
	self contentsChanged! !

!CodeHolder methodsFor: 'traits' stamp: 'ul 11/15/2010 11:35'!
showUnreferencedClassVars
	"Search for all class variables known to the selected class, and put up a 
	list of those that have no references anywhere in the system. The 
	search includes superclasses, so that you don't need to navigate your 
	way to the class that defines each class variable in order to determine 
	whether it is unreferenced"
	| cls aList aReport |
	((cls := self selectedClass) isNil or: [cls isTrait]) ifTrue: [^ self].
	aList := self systemNavigation allUnreferencedClassVariablesOf: cls.
	aList size = 0
		ifTrue: [^ self inform: 'There are no unreferenced
class variables in
' , cls name].
	aReport := String
				streamContents: [:aStream | 
					aStream nextPutAll: 'Unreferenced class variable(s) in ' , cls name;
						 cr.
					aList
						do: [:el | aStream tab; nextPutAll: el; cr]].
	Transcript cr; show: aReport.
	UIManager default chooseFrom: aList values: aList 
		title: 'Unreferenced
class variables in 
' , cls name! !

!CodeHolder methodsFor: 'traits' stamp: 'ul 11/15/2010 11:35'!
showUnreferencedInstVars
	"Search for all instance variables known to the selected class, and put up a list of those that have no references anywhere in the system.  The search includes superclasses, so that you don't need to navigate your way to the class that defines each inst variable in order to determine whether it is unreferenced"

	| cls aList aReport |
	((cls := self selectedClassOrMetaClass) isNil or: [cls isTrait]) ifTrue: [^ self].
	aList := cls allUnreferencedInstanceVariables.
	aList size = 0 ifTrue: [^ self inform: 'There are no unreferenced
instance variables in
', cls name].
	aReport := String streamContents:
		[:aStream |
			aStream nextPutAll: 'Unreferenced instance variable(s) in ', cls name; cr.
			aList do: [:el | aStream tab; nextPutAll: el; cr]].
	Transcript cr; show: aReport.
	UIManager default chooseFrom: aList values: aList title: 'Unreferenced
instance variables in 
', cls name! !

!CodeHolder methodsFor: 'controls' stamp: 'sw 11/13/2001 09:12'!
sourceAndDiffsQuintsOnly
	"Answer a list of quintuplets representing information on the alternative views available in the code pane for the case where the only plausible choices are showing source or either of the two kinds of diffs"

	^ #(
(source			togglePlainSource 		showingPlainSourceString	'source'			'the textual source code as writen')
(showDiffs		toggleRegularDiffing	showingRegularDiffsString	'showDiffs'		'the textual source diffed from its prior version')
(prettyDiffs		togglePrettyDiffing		showingPrettyDiffsString	'prettyDiffs'		'formatted textual source diffed from formatted form of prior version'))! !

!CodeHolder methodsFor: 'message list' stamp: 'topa 7/14/2010 16:47'!
sourceStringPrettifiedAndDiffed
	"Answer a copy of the source code for the selected message, transformed by diffing and pretty-printing exigencies"

	| class selector sourceString |
	class := self selectedClassOrMetaClass.
	selector := self selectedMessageName.
	(class isNil or: [selector isNil]) ifTrue: [^'missing'].
	sourceString := class ultimateSourceCodeAt: selector ifAbsent: [^'error'].
	self validateMessageSource: sourceString forSelector: selector inClass: class.
	(#(#prettyPrint #prettyDiffs) 
		includes: contentsSymbol) 
			ifTrue: 
				[sourceString := class prettyPrinterClass 
							format: sourceString
							in: class
							notifying: nil].
	self showingAnyKindOfDiffs 
		ifTrue: [sourceString := self diffFromPriorSourceFor: sourceString].
	^sourceString! !

!CodeHolder methodsFor: 'traits' stamp: 'mt 1/22/2016 14:15'!
spawnHierarchy
	"Create and schedule a new hierarchy browser on the currently selected class or meta."

	^ ToolSet
		browseHierarchy: self selectedClassOrMetaClass
		selector: self selectedMessageName! !

!CodeHolder methodsFor: 'commands' stamp: 'sw 3/20/2001 15:10'!
spawnToClass: aClass
	"Used to copy down code from a superclass to a subclass in one easy step, if you know what you're doing.  Spawns a new message-category browser for the indicated class, populating it with the source code seen in the current tool."

	| aCategory newBrowser org |	
	(aCategory := self categoryOfCurrentMethod)
		ifNil:
			[self buildClassBrowserEditString: self contents]
		ifNotNil:
			[((org := aClass organization) categories includes: aCategory)
				ifFalse:	[org addCategory: aCategory].
			newBrowser := Browser new setClass: aClass selector: nil.
			newBrowser selectMessageCategoryNamed: aCategory.
			Browser openBrowserView: (newBrowser openMessageCatEditString: self contents)
		label: 'category "', aCategory, '" in ', 
				newBrowser selectedClassOrMetaClassName]! !

!CodeHolder methodsFor: 'commands' stamp: 'sw 3/20/2001 15:11'!
spawnToCollidingClass: aClass
	"Potentially used to copy down code from a superclass to a subclass in one easy step, in the case where the given class already has its own version of code, which would consequently be clobbered if the spawned code were accepted."

	self inform: 'That would be destructive of
some pre-existing code already in that
class for this selector.  For the moment,
we will not let you do this to yourself.'! !

!CodeHolder methodsFor: 'commands' stamp: 'fbs 3/3/2011 13:26'!
spawn: aString 
	"Create and schedule a spawned message category browser for the currently selected message category.  The initial text view contains the characters in aString.  In the spawned browser, preselect the current selector (if any) as the going-in assumption, though upon acceptance this will often change"

	| newBrowser aCategory aClass |
	(aClass := self selectedClassOrMetaClass) isNil ifTrue:
		[^ aString isEmptyOrNil ifFalse: [(Workspace new contents: aString) openLabel: 'spawned workspace']].

	(aCategory := self categoryOfCurrentMethod)
		ifNil:
			[self buildClassBrowserEditString: aString]
		ifNotNil:
			[newBrowser := Browser new setClass: aClass selector: self selectedMessageName.
			self suggestCategoryToSpawnedBrowser: newBrowser.
			^ Browser openBrowserView: (newBrowser openMessageCatEditString: aString)
		label: 'category "', aCategory, '" in ', 
				newBrowser selectedClassOrMetaClassName]! !

!CodeHolder methodsFor: 'self-updating' stamp: 'sw 10/19/1999 08:37'!
stepIn: aSystemWindow
	self updateListsAndCodeIn: aSystemWindow! !

!CodeHolder methodsFor: 'misc' stamp: 'sw 2/22/2001 06:37'!
suggestCategoryToSpawnedBrowser: aBrowser
	"aBrowser is a message-category browser being spawned from the receiver.  Tell it what it needs to know to get its category info properly set up."

	aBrowser setOriginalCategoryIndexForCurrentMethod! !

!CodeHolder methodsFor: 'toolbuilder' stamp: 'mt 4/29/2015 17:43'!
textFrame
	| topOffset bottomOffset |
	topOffset := self wantsOptionalButtons
		ifTrue: [self buttonHeight]
		ifFalse: [0].
	bottomOffset := self wantsAnnotationPane
		ifTrue: [self annotationPaneHeight negated]
		ifFalse: [0].
	^LayoutFrame new
		leftFraction: 0 offset: 0;
		topFraction: 0 offset: topOffset;
		rightFraction: 1 offset: 0;
		bottomFraction: 1 offset: bottomOffset! !

!CodeHolder methodsFor: 'breakpoints' stamp: 'topa 3/2/2016 01:11'!
toggleBreakOnEntry
	"Install or uninstall a halt-on-entry breakpoint"

	| selectedMethod |
	self selectedClassOrMetaClass ifNil: [ ^self].
	selectedMethod := self selectedClassOrMetaClass
		compiledMethodAt: self selectedMessageName
		ifAbsent: [^ self].
	selectedMethod hasBreakpoint
		ifTrue:
			[BreakpointManager unInstall: selectedMethod]
		ifFalse:
			[BreakpointManager 
				installInClass: self selectedClassOrMetaClass
				selector: self selectedMessageName].
	self changed: #messageList! !

!CodeHolder methodsFor: 'what to show' stamp: 'sw 5/20/2001 06:48'!
toggleDecompile
	"Toggle the setting of the showingDecompile flag, unless there are unsubmitted edits that the user declines to discard"

	| wasShowing |
	self okToChange ifTrue:
		[wasShowing := self showingDecompile.
		self restoreTextualCodingPane.
		self showDecompile: wasShowing not.
		self setContentsToForceRefetch.
		self contentsChanged]

! !

!CodeHolder methodsFor: 'diffs' stamp: 'sw 11/13/2001 07:30'!
toggleDiffing
	"Toggle whether diffs should be shown in the code pane.  If any kind of diffs were being shown, stop showing diffs.  If no kind of diffs were being shown, start showing whatever kind of diffs are called for by default."

	| wasShowingDiffs |
	self okToChange ifTrue:
		[wasShowingDiffs := self showingAnyKindOfDiffs.
		self restoreTextualCodingPane.
		self showDiffs: wasShowingDiffs not.
		self setContentsToForceRefetch.
		self contentsChanged]

! !

!CodeHolder methodsFor: 'diffs' stamp: 'sw 5/18/2001 19:57'!
togglePlainSource
	"Toggle whether plain source shown in the code pane"
	
	| wasShowingPlainSource |
	self okToChange ifTrue:
		[wasShowingPlainSource := self showingPlainSource.
		self restoreTextualCodingPane.
		wasShowingPlainSource
			ifTrue:
				[self showDocumentation: true]
			ifFalse:
				[contentsSymbol := #source].
		self setContentsToForceRefetch.
		self changed: #contents]

! !

!CodeHolder methodsFor: 'diffs' stamp: 'sw 5/19/2001 00:02'!
togglePrettyDiffing
	"Toggle whether pretty-diffing should be shown in the code pane"

	| wasShowingDiffs |
	self okToChange ifTrue:
		[wasShowingDiffs := self showingPrettyDiffs.
		self restoreTextualCodingPane.
		self showPrettyDiffs: wasShowingDiffs not.
		self setContentsToForceRefetch.
		self contentsChanged]

! !

!CodeHolder methodsFor: 'diffs' stamp: 'sw 5/18/2001 19:54'!
togglePrettyPrint
	"Toggle whether pretty-print is in effectin the code pane"

	self restoreTextualCodingPane.
	self okToChange ifTrue:
		[self showingPrettyPrint
			ifTrue:
				[contentsSymbol := #source]
			ifFalse:
				[contentsSymbol := #prettyPrint].
		self setContentsToForceRefetch.
		self contentsChanged]

! !

!CodeHolder methodsFor: 'diffs' stamp: 'sw 11/13/2001 07:27'!
toggleRegularDiffing
	"Toggle whether regular-diffing should be shown in the code pane"

	| wasShowingDiffs |
	self okToChange ifTrue:
		[wasShowingDiffs := self showingRegularDiffs.
		self restoreTextualCodingPane.
		self showRegularDiffs: wasShowingDiffs not.
		self setContentsToForceRefetch.
		self contentsChanged]

! !

!CodeHolder methodsFor: 'what to show' stamp: 'sw 5/18/2001 20:15'!
toggleShowDocumentation
	"Toggle the setting of the showingDocumentation flag, unless there are unsubmitted edits that the user declines to discard"

	| wasShowing |
	self okToChange ifTrue:
		[wasShowing := self showingDocumentation.
		self restoreTextualCodingPane.
		self showDocumentation: wasShowing not.
		self setContentsToForceRefetch.
		self contentsChanged]

! !

!CodeHolder methodsFor: 'what to show' stamp: 'sw 5/18/2001 20:09'!
toggleShowingByteCodes
	"Toggle whether the receiver is showing bytecodes"

	self restoreTextualCodingPane.
	self showByteCodes: self showingByteCodes not.
	self setContentsToForceRefetch.
	self contentsChanged! !

!CodeHolder methodsFor: 'commands' stamp: 'sw 3/6/2001 15:19'!
unshiftedYellowButtonActivity
	"Offer the unshifted shifted selector-list menu"

	^ self offerMenuFrom: #messageListMenu:shifted: shifted: false! !

!CodeHolder methodsFor: 'self-updating' stamp: 'sw 2/14/2001 15:34'!
updateCodePaneIfNeeded
	"If the code for the currently selected method has changed underneath me, then update the contents of my code pane unless it holds unaccepted edits"

	self didCodeChangeElsewhere
		ifTrue:
			[self hasUnacceptedEdits
				ifFalse:
					[self setContentsToForceRefetch.
					self contentsChanged]
				ifTrue:
					[self changed: #codeChangedElsewhere]]! !

!CodeHolder methodsFor: 'self-updating' stamp: 'sw 10/19/1999 14:14'!
updateListsAndCodeIn: aWindow
	super updateListsAndCodeIn: aWindow.
	self updateCodePaneIfNeeded! !

!CodeHolder methodsFor: 'misc' stamp: 'rbb 3/1/2005 10:31'!
useSelector: incomingSelector orGetSelectorAndSendQuery: querySelector to: queryPerformer
	"If incomingSelector is not nil, use it, else obtain a selector from user type-in.   Using the determined selector, send the query to the performer provided."

	| aSelector |
	incomingSelector
		ifNotNil:
			[queryPerformer perform: querySelector with: incomingSelector]
		ifNil:
			[aSelector :=UIManager default request: 'Type selector:' initialAnswer: 'flag:'.
			aSelector isEmptyOrNil ifFalse:
				[(Symbol hasInterned: aSelector ifTrue:
					[:aSymbol | queryPerformer perform: querySelector with: aSymbol])
					ifFalse:
						[self inform: 'no such selector']]]! !

!CodeHolder methodsFor: 'message list' stamp: 'nice 2/23/2011 21:47'!
validateMessageSource: sourceString forSelector: aSelector inClass: theClass
	"Check whether there is evidence that method source is invalid"

	(theClass newParser parseSelector: sourceString asString) = aSelector
		ifFalse: [self informPossiblyCorruptSource].! !

!CodeHolder methodsFor: 'toolbuilder' stamp: 'topa 6/23/2016 09:49'!
versionsButtonEnabled
	"The versions button is only enabled when a method is selected or we view the 
	class comment."

	^  self selectedMessageName notNil or: [self classCommentIndicated]! !

!CodeHolder methodsFor: 'diffs' stamp: 'sw 11/13/2001 07:24'!
wantsDiffFeedback
	"Answer whether the receiver is showing diffs of source code"

	^ self showingAnyKindOfDiffs! !

!CodeHolder methodsFor: 'self-updating' stamp: 'sw 10/20/1999 12:22'!
wantsStepsIn: aWindow
	^ Preferences smartUpdating! !

CodeHolder subclass: #Debugger
	instanceVariableNames: 'interruptedProcess interruptedController contextStack contextStackIndex contextStackList receiverInspector contextVariablesInspector externalInterrupt proceedValue selectingPC savedCursor isolationHead failedProject errorWasInUIProcess labelString message untilExpression'
	classVariableNames: 'ContextStackKeystrokes ErrorRecursion InterruptUIProcessIfBlockedOnErrorInBackgroundProcess WantsAnnotationPane'
	poolDictionaries: ''
	category: 'Tools-Debugger'!

!Debugger commentStamp: '<historical>' prior: 0!
I represent the machine state at the time of an interrupted process. I also represent a query path into the state of the process. The debugger is typically viewed through a window that views the stack of suspended contexts, the code for, and execution point in, the currently selected message, and inspectors on both the receiver of the currently selected message, and the variables in the current context.

Special note on recursive errors:
Some errors affect Squeak's ability to present a debugger.  This is normally an unrecoverable situation.  However, if such an error occurs in an isolation layer, Squeak will attempt to exit from the isolation layer and then present a debugger.  Here is the chain of events in such a recovery.

	* A recursive error is detected.
	* The current project is queried for an isolationHead
	* Changes in the isolationHead are revoked
	* The parent project of isolated project is returned to
	* The debugger is opened there and execution resumes.

If the user closes that debugger, execution continues in the outer project and layer.  If, after repairing some damage, the user proceeds from the debugger, then the isolationHead is re-invoked, the failed project is re-entered, and execution resumes in that world. !

!Debugger class methodsFor: 'instance creation' stamp: 'mt 2/15/2016 15:29'!
context: aContext
	"Answer an instance of me for debugging the active process starting with the given context."

	^ Project current
		dispatchTo: self
		addPrefixAndSend: #Context:
		withArguments: { aContext }! !

!Debugger class methodsFor: 'private' stamp: 'eem 7/26/2010 18:01'!
ifPreferredInterruptUIProcessIfBlocked: errorWasInUIProcess
	| sema |
	errorWasInUIProcess ifTrue:
		[^self].
	self interruptUIProcessIfBlockedOnErrorInBackgroundProcess ifFalse:
		[^self].
	"Only interrupt the UI if it is unresponsive (and so is doing something that needs
	 interrupting).  Test using addDeferredUIMessage: to see if it is running UI activities."
	sema := Semaphore new.
	Project current addDeferredUIMessage: [sema signal].
	(sema waitTimeoutMSecs: 100) ifTrue:
		[[Project current interruptName: 'Interrupt from Background Error'] fork]

	"| s |
	s := Semaphore new.
	[self assert: 0 > 1000. s signal] fork.
	s wait"! !

!Debugger class methodsFor: 'instance creation' stamp: 'hmm 8/3/2001 13:05'!
informExistingDebugger: aContext label: aString
	"Walking the context chain, we try to find out if we're in a debugger stepping situation.
	If we find the relevant contexts, we must rearrange them so they look just like they would
	if the methods were excuted outside of the debugger."
	| ctx quickStepMethod oldSender baseContext |
	ctx := thisContext.
	quickStepMethod := ContextPart compiledMethodAt: #quickSend:to:with:super:.
	[ctx sender == nil or: [ctx sender method == quickStepMethod]] whileFalse: [ctx := ctx sender].
	ctx sender == nil ifTrue: [^self].
	baseContext := ctx.
	"baseContext is now the context created by the #quickSend... method."
	oldSender := ctx := ctx sender home sender.
	"oldSender is the context which originally sent the #quickSend... method"
	[ctx == nil or: [ctx receiver isKindOf: self]] whileFalse: [ctx := ctx sender].
	ctx == nil ifTrue: [^self].
	"ctx is the context of the Debugger method #doStep"
	ctx receiver labelString: aString.
	ctx receiver externalInterrupt: false; proceedValue: aContext receiver.
	baseContext swapSender: baseContext sender sender sender.	"remove intervening contexts"
	thisContext swapSender: oldSender.	"make myself return to debugger"
	ErrorRecursion := false.
	^aContext! !

!Debugger class methodsFor: 'class initialization' stamp: 'hg 9/29/2001 20:24'!
initialize
	ErrorRecursion := false.
	ContextStackKeystrokes := Dictionary new
		at: $e put: #send;
		at: $t put: #doStep;
		at: $T put: #stepIntoBlock;
		at: $p put: #proceed;
		at: $r put: #restart;
		at: $f put: #fullStack;
		at: $w put: #where;
		yourself.

	"Debugger initialize"! !

!Debugger class methodsFor: 'preferences' stamp: 'eem 7/26/2010 15:52'!
interruptUIProcessIfBlockedOnErrorInBackgroundProcess
	<preference: 'Interrupt UI process on background error'
		category: 'debug'
		description: 'When enabled, the debugger will interrupt the UI process if an error occurs in a background process and the UI process is blocked.'
		type: #Boolean>
	^InterruptUIProcessIfBlockedOnErrorInBackgroundProcess ifNil: [false]! !

!Debugger class methodsFor: 'preferences' stamp: 'eem 7/26/2010 15:53'!
interruptUIProcessIfBlockedOnErrorInBackgroundProcess: aBoolean
	InterruptUIProcessIfBlockedOnErrorInBackgroundProcess := aBoolean! !

!Debugger class methodsFor: 'opening' stamp: 'mt 2/15/2016 15:23'!
openContext: aContext label: aString contents: contentsStringOrNil
	"Open a notifier in response to an error, halt, or notify. A notifier view just shows a short view of the sender stack and provides a menu that lets the user open a full debugger."

	^ Project current
		dispatchTo: self
		addPrefixAndSend: #OpenContext:label:contents:
		withArguments: { aContext . aString . contentsStringOrNil }! !

!Debugger class methodsFor: 'opening' stamp: 'mt 2/15/2016 15:26'!
openInterrupt: aString onProcess: interruptedProcess
	"Open a notifier in response to an interrupt. An interrupt occurs when the user types the interrupt key (cmd-. on Macs, ctrl-c or alt-. on other systems) or when the low-space watcher detects that memory is low."

	^ Project current
		dispatchTo: self
		addPrefixAndSend: #OpenInterrupt:onProcess:
		withArguments: { aString . interruptedProcess }! !

!Debugger class methodsFor: 'opening' stamp: 'dtl 2/27/2011 16:37'!
openOn: process context: context label: title contents: contentsStringOrNil fullView: bool
	"Open a notifier in response to an error, halt, or notify. A notifier view just shows a short
	view of the sender stack and provides a menu that lets the user open a full debugger.
	Dispatch the request to a method appropriate for the active project."

	^ Project current
		dispatchTo: self
		addPrefixAndSend: #OpenOn:context:label:contents:fullView:
		withArguments: { process . context . title . contentsStringOrNil . bool }
! !

!Debugger class methodsFor: 'preferences' stamp: 'tpr 8/9/2016 19:41'!
wantsAnnotationPane
	<preference: 'Show annotation pane in the debugger.'
		categoryList: #(debug tools)
		description: 'If true, a small horizontal annotation pane shows information about the selected method.'
		type: #Boolean>
	^ WantsAnnotationPane ifNil: [false]! !

!Debugger class methodsFor: 'preferences' stamp: 'mt 11/10/2015 13:35'!
wantsAnnotationPane: boolean

	WantsAnnotationPane := boolean.! !

!Debugger methodsFor: 'context stack menu' stamp: 'sw 12/28/1999 13:04'!
abandon
	"abandon the debugger from its pre-debug notifier"
	self abandon: self topView! !

!Debugger methodsFor: 'context stack menu' stamp: 'ar 8/7/2009 23:01'!
abandon: aTopView 
	"abandon the notifier represented by aTopView"
	ToolBuilder default close: aTopView! !

!Debugger methodsFor: 'code pane' stamp: 'ar 8/17/2009 23:18'!
aboutToStyle: aStyler
	"This is a notification that aStyler is about to re-style its text.
	Set the classOrMetaClass in aStyler, so that identifiers
	will be resolved correctly.
	Answer true to allow styling to proceed, or false to veto the styling"
	
	self isModeStyleable ifFalse: [^false].
	aStyler 
		classOrMetaClass: self selectedClassOrMetaClass;
		sourceMap: self debuggerMap.
	^true! !

!Debugger methodsFor: 'toolbuilder' stamp: 'laza 3/2/2010 09:22'!
addCodeProvenanceButtonTo: panelSpec using: builder
	"No thanks!!"! !

!Debugger methodsFor: 'context stack menu' stamp: 'rbb 3/1/2005 10:50'!
askForCategoryIn: aClass default: aString
	| categories index category |
	categories := OrderedCollection with: 'new ...'. 
	categories addAll: (aClass allMethodCategoriesIntegratedThrough: Object).	
	index := UIManager default  
				chooseFrom: categories
				title: 'Please provide a good category for the new method!!' translated.
	index = 0 ifTrue: [^ aString].
	category := index = 1 ifTrue: [UIManager default request: 'Enter category name:']
						ifFalse: [categories at: index].
	^ category isEmpty ifTrue: [^ aString] ifFalse: [category]! !

!Debugger methodsFor: 'private' stamp: 'sd 11/20/2005 21:27'!
askForSuperclassOf: aClass toImplement: aSelector ifCancel: cancelBlock
	| classes chosenClassIndex |
	classes := aClass withAllSuperclasses.
	chosenClassIndex := UIManager default 
		chooseFrom: (classes collect: [:c | c name])
		title: 'Define #', aSelector, ' in which class?'.
	chosenClassIndex = 0 ifTrue: [^ cancelBlock value].
	^ classes at: chosenClassIndex! !

!Debugger methodsFor: 'private' stamp: 'fbs 4/29/2013 20:41'!
askForSuperclassOf: aClass upTo: superclass toImplement: aSelector ifCancel: cancelBlock
	| classes chosenClassIndex |
	classes := aClass withAllSuperclasses reject: [:cls | aClass isKindOf: cls].
	chosenClassIndex := UIManager default 
		chooseFrom: (classes collect: [:c | c name])
		title: 'Define #', aSelector, ' in which class?'.
	chosenClassIndex = 0 ifTrue: [^ cancelBlock value].
	^ classes at: chosenClassIndex! !

!Debugger methodsFor: 'context stack menu' stamp: 'eem 9/7/2010 16:15'!
browseMessages
	"Present a menu of all messages sent by the currently selected message.
	Open a message set browser of all implementors of the message chosen.
	Do nothing if no message is chosen."

	contextStackIndex = 0 ifTrue: [^ self].
	self withSelectorAndMessagesIn: self selectedContext method
		evaluate: [:selector| self systemNavigation browseAllImplementorsOf: selector]! !

!Debugger methodsFor: 'context stack menu' stamp: 'eem 9/7/2010 16:14'!
browseSendersOfMessages
	"Present a menu of the currently selected message, as well as all
	messages sent by it.  Open a message set browser of all senders
	of the message chosen."

	contextStackIndex = 0 ifTrue: [^ self].
	self withSelectorAndMessagesIn: self selectedContext method
		evaluate: [:selector| self systemNavigation browseAllCallsOn: selector]! !

!Debugger methodsFor: 'context stack menu' stamp: 'eem 1/7/2016 11:19'!
browseVersions
	"Create and schedule a message set browser on all versions of the currently selected message selector."
	(ToolSet
		browseVersionsOf: self selectedClassOrMetaClass
		selector: self selectedMessageName) ifNil: [self changed: #flash]! !

!Debugger methodsFor: 'toolbuilder' stamp: 'mt 11/11/2015 11:19'!
buildCodePaneWith: builder
	
	| textSpec top controlButtons browseButtons annoSpec |
	top := builder pluggablePanelSpec new.
	top children: OrderedCollection new.
	
	controlButtons := self buildControlButtonsWith: builder.
	controlButtons frame: self controlButtonsFrame.
	top children add: controlButtons.
	
	self wantsOptionalButtons ifTrue: [
		browseButtons := self buildOptionalButtonsWith: builder.
		browseButtons frame: self optionalButtonsFrame.
		top children add: browseButtons].

	textSpec := builder pluggableCodePaneSpec new.
	textSpec 
		model: self;
		getText: #contents; 
		setText: #contents:notifying:; 
		selection: #contentsSelection; 
		menu: #codePaneMenu:shifted:;
		frame: self textFrame.
	top children add: textSpec.

	self wantsAnnotationPane ifTrue: [
		annoSpec := self buildAnnotationPaneWith: builder.
		annoSpec frame: self annotationFrame.
		top children add: annoSpec].
	.
	^ top! !

!Debugger methodsFor: 'toolbuilder' stamp: 'mt 11/11/2015 11:09'!
buildControlButtonsWith: builder

	| panelSpec |
	panelSpec := builder pluggablePanelSpec new.
	panelSpec children: OrderedCollection new.
	self customButtonSpecs do:[:spec|
		| buttonSpec |
		buttonSpec := builder pluggableActionButtonSpec new.
		buttonSpec model: self.
		buttonSpec label: spec first.
		buttonSpec action: spec second.
		spec second == #methodHierarchy ifTrue:[
			buttonSpec color: #inheritanceButtonColor.
		]. 
		spec size > 2 ifTrue:[buttonSpec help: spec third].
		panelSpec children add: buttonSpec.
	].

	panelSpec layout: #horizontal. "buttons"
	^panelSpec! !

!Debugger methodsFor: 'toolbuilder' stamp: 'mt 4/5/2016 14:20'!
buildFullWith: builder
	| windowSpec listSpec textSpec |
	windowSpec := builder pluggableWindowSpec new
		model: self;
		label: 'Debugger';
		children: OrderedCollection new.

	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #contextStackList; 
		getIndex: #contextStackIndex; 
		setIndex: #toggleContextStackIndex:; 
		menu: #contextStackMenu:shifted:; 
		icon: #messageIconAt:;
		helpItem: #messageHelpAt:;
		keyPress: #contextStackKey:from:;
		frame: (0@0 corner: 1@0.22).
	windowSpec children add: listSpec.


	textSpec := self buildCodePaneWith: builder.
	textSpec frame: (0@0.22corner: 1@0.8).
	windowSpec children add: textSpec.

	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self receiverInspector;
		list: #fieldList; 
		getIndex: #selectionIndex; 
		setIndex: #toggleIndex:; 
		menu: #fieldListMenu:; 
		keyPress: #inspectorKey:from:;
		frame: (0@0.8 corner: 0.2@1);
		help: 'Receiver''s\Instance\Variables' withCRs.
	windowSpec children add: listSpec.

	textSpec := builder pluggableTextSpec new.
	textSpec 
		model: self receiverInspector;
		getText: #contents; 
		setText: #accept:; 
		help: '<- Select receiver''s field' translated;
		selection: #contentsSelection; 
		menu: #codePaneMenu:shifted:;
		frame: (0.2@0.8 corner: 0.5@1).
	windowSpec children add: textSpec.

	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self contextVariablesInspector;
		list: #fieldList; 
		getIndex: #selectionIndex; 
		setIndex: #toggleIndex:; 
		menu: #fieldListMenu:; 
		keyPress: #inspectorKey:from:;
		frame: (0.5@0.8 corner: 0.7@1);
		help: 'Other\Context\Bindings' withCRs.
	windowSpec children add: listSpec.

	textSpec := builder pluggableTextSpec new.
	textSpec 
		model: self contextVariablesInspector;
		getText: #contents; 
		setText: #accept:; 
		help: '<- Select context''s field' translated;
		selection: #contentsSelection; 
		menu: #codePaneMenu:shifted:;
		frame: (0.7@0.8 corner: 1@1).
	windowSpec children add: textSpec.

	^builder build: windowSpec! !

!Debugger methodsFor: 'toolbuilder' stamp: 'mt 11/6/2015 11:57'!
buildNotifierWith: builder label: label message: messageString
	| windowSpec listSpec textSpec panelSpec quads |
	windowSpec := builder pluggableWindowSpec new
		model: self;
		extent: self initialExtentForNotifier;
		label: label;
		children: OrderedCollection new.

	panelSpec := builder pluggablePanelSpec new.
	panelSpec children: OrderedCollection new.
	quads := self preDebugButtonQuads.
	(self interruptedContext selector == #doesNotUnderstand:) ifTrue: [
		quads := quads copyWith: 
			{ 'Create'. #createMethod. #magenta. 'create the missing method' }
	].
	(#(#notYetImplemented #shouldBeImplemented #requirement) includes: self interruptedContext selector) ifTrue: [
		quads := quads copyWith: 
			{ 'Create'. #createImplementingMethod. #magenta. 'implement the marked method' }
	].
	(self interruptedContext selector == #subclassResponsibility) ifTrue: [
		quads := quads copyWith: 
			{ 'Create'. #createOverridingMethod. #magenta. 'create the missing overriding method' }
	].
	quads do:[:spec| | buttonSpec |
		buttonSpec := builder pluggableButtonSpec new.
		buttonSpec model: self.
		buttonSpec label: spec first.
		buttonSpec action: spec second.
		buttonSpec help: spec fourth.
		panelSpec children add: buttonSpec.
	].
	panelSpec layout: #horizontal. "buttons"
	panelSpec frame: self preDebugButtonQuadFrame.
	windowSpec children add: panelSpec.

	Preferences eToyFriendly | messageString notNil ifFalse:[
		listSpec := builder pluggableListSpec new.
		listSpec 
			model: self;
			list: #contextStackList; 
			getIndex: #contextStackIndex; 
			setIndex: #debugAt:; 
			icon: #messageIconAt:;
			help: #messageHelpAt:; 
			frame: self contextStackFrame.
		windowSpec children add: listSpec.
	] ifTrue:[
		message := messageString.
		textSpec := builder pluggableTextSpec new.
		textSpec 
			model: self;
			getText: #preDebugMessageString; 
			setText: nil; 
			selection: nil; 
			menu: #debugProceedMenu:;
			frame: self contextStackFrame.
		windowSpec children add: textSpec.
	].

	^windowSpec! !

!Debugger methodsFor: 'toolbuilder' stamp: 'ar 2/11/2005 16:24'!
buildWith: aBuilder
	^self buildFullWith: aBuilder! !

!Debugger methodsFor: 'private' stamp: 'yo 8/12/2003 16:34'!
checkContextSelection

	contextStackIndex = 0 ifTrue: [self contextStackIndex: 1 oldContextWas: nil].
! !

!Debugger methodsFor: 'toolbuilder' stamp: 'cmm 5/2/2011 18:21'!
classHierarchy
	"Create and schedule a class list browser on the receiver's hierarchy."
	(self selectedMessageName = #doesNotUnderstand: and: [ self selectedClassOrMetaClass = Object ])
		ifTrue:
			[ self systemNavigation
				spawnHierarchyForClass: self receiverClass
				selector: self selectedMessageName ]
		ifFalse: [ super classHierarchy ]! !

!Debugger methodsFor: 'context stack menu' stamp: ''!
close: aScheduledController 
	"The argument is a controller on a view of the receiver.
	That view is closed."

	aScheduledController close
! !

!Debugger methodsFor: 'code pane' stamp: 'topa 3/16/2015 15:47'!
codePaneSelectionInterval

	^ self codeTextMorph
		ifNotNil: [:cp | cp selectionInterval]
		ifNil: [Interval from: 0 to: 0]! !

!Debugger methodsFor: 'accessing' stamp: 'fbs 12/2/2010 17:38'!
contents 
	"Depending on the current selection, different information is retrieved.
	Answer a string description of that information.  This information is the
	method in the currently selected context."

	^contents ifNil:
		[self selectedContext
			ifNotNil: [self selectedMessage]
			ifNil: [String new]] ! !

!Debugger methodsFor: 'code pane' stamp: 'tk 4/15/1998 18:31'!
contentsSelection

	^ self pcRange! !

!Debugger methodsFor: 'accessing' stamp: 'eem 9/17/2015 12:54'!
contents: aText notifying: aController
	"The retrieved information has changed and its source must now be updated.
	 In this case, the retrieved information is the method of the selected context."
	| result selector classOfMethod category h ctxt newMethod |
	contextStackIndex = 0 ifTrue:
		[^false].
	self selectedContext isExecutingBlock ifTrue:
		[h := self selectedContext activeHome.
		 h ifNil:
			[self inform: 'Method for block not found on stack, can''t edit and continue'.
			 ^false].
		 (self confirm: 'I will have to revert to the method from\which this block originated.  Is that OK?' withCRs) ifFalse:
			[^false].
		self resetContext: h changeContents: false.
		"N.B. Only reset the contents if the compilation succeeds.  If contents are reset
		 when compilation fails both compiler error message and modifications are lost."
		(result := self contents: aText notifying: aController) ifTrue:
			[self contentsChanged].
		^result].

	classOfMethod := self selectedClass.
	category := self selectedMessageCategoryName.
	selector := self selectedClass newParser parseSelector: aText.
	(selector == self selectedMessageName
	 or: [(self selectedMessageName beginsWith: 'DoIt')
		and: [selector numArgs = self selectedMessageName numArgs]]) ifFalse:
		[self inform: 'can''t change selector'.
		 ^false].
	selector := classOfMethod
				compile: aText
				classified: category
				notifying: aController.
	selector ifNil: [^false]. "compile cancelled"
	contents := aText.
	newMethod := classOfMethod compiledMethodAt: selector.
	newMethod isQuick ifTrue:
		[self cutBackExecutionToSenderContext].
	ctxt := interruptedProcess popTo: self selectedContext.
	ctxt == self selectedContext
		ifFalse:
			[self inform: 'Method saved, but current context unchanged\because of unwind error. Click OK to see error' withCRs]
		ifTrue:
			[newMethod isQuick ifFalse:
				[interruptedProcess
					restartTopWith: newMethod;
				 	stepToSendOrReturn].
			contextVariablesInspector object: nil].
	self resetContext: ctxt.
	Smalltalk isMorphic ifTrue:
		[World
			addAlarm: #changed:
			withArguments: #(contentsSelection)
			for: self
			at: (Time millisecondClockValue + 200)].
	^true! !

!Debugger methodsFor: 'toolbuilder' stamp: 'mt 11/11/2015 11:32'!
contextStackFrame

	^LayoutFrame new
		leftFraction: 0 offset: 0;
		topFraction: 0 offset: self buttonHeight;
		rightFraction: 1 offset: 0;
		bottomFraction: 1 offset: 0! !

!Debugger methodsFor: 'context stack (message list)' stamp: ''!
contextStackIndex
	"Answer the index of the selected context."

	^contextStackIndex! !

!Debugger methodsFor: 'private' stamp: 'eem 3/12/2009 14:55'!
contextStackIndex: anInteger oldContextWas: oldContext 
	"Change the context stack index to anInteger, perhaps in response to user selection."

	| isNewMethod selectedContextSlotName index |
	contextStackIndex := anInteger.
	anInteger = 0 ifTrue:
		[currentCompiledMethod := contents := nil.
		 self changed: #contextStackIndex.
		 self decorateButtons.
		 self contentsChanged.
		 contextVariablesInspector object: nil.
		 receiverInspector object: self receiver.
		 ^self].
	selectedContextSlotName := contextVariablesInspector selectedSlotName.
	isNewMethod := oldContext == nil
					or: [oldContext method ~~ (currentCompiledMethod := self selectedContext method)].
	isNewMethod ifTrue:
		[contents := self selectedMessage.
		 self contentsChanged.
		 self pcRange].
	self changed: #contextStackIndex.
	self decorateButtons.
	contextVariablesInspector object: self selectedContext.
	((index := contextVariablesInspector fieldList indexOf: selectedContextSlotName) ~= 0
	 and: [index ~= contextVariablesInspector selectionIndex]) ifTrue:
		[contextVariablesInspector toggleIndex: index].
	receiverInspector object: self receiver.
	isNewMethod ifFalse:
		[self changed: #contentsSelection]! !

!Debugger methodsFor: 'context stack menu' stamp: 'tk 4/18/1998 09:24'!
contextStackKey: aChar from: view
	"Respond to a keystroke in the context list"

 	| selector |
	selector := ContextStackKeystrokes at: aChar ifAbsent: [nil].
	selector ifNil: [self messageListKey: aChar from: view]
		ifNotNil: [self perform: selector]! !

!Debugger methodsFor: 'context stack (message list)' stamp: ''!
contextStackList
	"Answer the array of contexts."

	^contextStackList! !

!Debugger methodsFor: 'context stack menu' stamp: 'topa 3/16/2015 16:35'!
contextStackMenu: aMenu shifted: shifted
	"Set up the menu appropriately for the context-stack-list, either shifted or unshifted as per the parameter provided"

	^ self menu: aMenu for: #(contextStackMenu contextStackMenuShifted:) shifted: shifted
! !

!Debugger methodsFor: 'accessing' stamp: 'fbs 12/10/2010 22:30'!
contextStackTop
	^ contextStack first.! !

!Debugger methodsFor: 'accessing' stamp: ''!
contextVariablesInspector
	"Answer the instance of Inspector that is providing a view of the 
	variables of the selected context."

	^contextVariablesInspector! !

!Debugger methodsFor: 'toolbuilder' stamp: 'mt 11/11/2015 11:18'!
controlButtonsFrame
	^LayoutFrame new
		leftFraction: 0 offset: 0;
		topFraction: 0 offset: 0;
		rightFraction: 1 offset: 0;
		bottomFraction: 0 offset: self buttonHeight! !

!Debugger methodsFor: 'private' stamp: 'fbs 4/29/2013 20:27'!
createImplementingMethod
	"Should only be called when this Debugger was created in response to a
	NotYetImplemented exception. All we need to do is pop the signalling context off the stack and open the #notYetImplemented method."
	| signallingContext |
	signallingContext := self selectedContext sender.
	"Pop the signalling context off the stack"
	self resetContext: signallingContext.
	self debug.! !

!Debugger methodsFor: 'private' stamp: 'fbs 4/30/2013 08:38'!
createMethod
	"Should only be called when this Debugger was created in response to a
	MessageNotUnderstood exception. Create a stub for the method that was
	missing and proceed into it."
	
	| msg chosenClass |
	msg := self contextStackTop exceptionMessage.
	chosenClass := self
		askForSuperclassOf: self contextStackTop receiver class
		toImplement: msg selector
		ifCancel: [^self].
	self implementMissingMethod: msg inClass: chosenClass.! !

!Debugger methodsFor: 'private' stamp: 'fbs 4/30/2013 08:38'!
createOverridingMethod
	"Should only be called when this Debugger was created in response to a
	SubclassResponsibility exception. Create a stub for the method that needs
	overriding and proceed into it. Let the user only select a class in the
	inheritance chain between the actual class and the class declaring the
	subclassResponsibility."
	| chosenClass msg category |
	msg := self contextStackTop exceptionMessage.
	chosenClass := self
		askForSuperclassOf: self contextStackTop receiver class
		upTo: self contextStackTop sender method methodClass
		toImplement: msg selector
		ifCancel: [^self].
	"Use the same category as the marker method."
	category := self contextStackTop sender selectorCategory.
	self implementOverridingMethod: msg inClass: chosenClass inCategory: category.! !

!Debugger methodsFor: 'initialize' stamp: 'ab 3/23/2005 14:03'!
customButtonSpecs
	"Answer an array of elements of the form wording, selector, help-message, that characterize the custom button row of a debugger."

	| list |
	list := #(('Proceed'	proceed				'close the debugger and proceed.')
		('Restart'		restart				'reset this context to its start.')
		('Into'			send				'step Into message sends')
		('Over'			doStep				'step Over message sends')
		('Through'		stepIntoBlock		'step into a block')
		('Full Stack'		fullStack			'show full stack')
		('Where'		where				'select current pc range')
		('Tally'			tally				'time in milliseconds to execute')).
	Preferences restartAlsoProceeds ifTrue:
		[list := list collect: [:each |
			each second == #restart
				ifTrue: [each copy at: 3 put: 'proceed from the beginning of this context.'; yourself]
				ifFalse: [each]]].
	^ list! !

!Debugger methodsFor: 'private' stamp: 'eem 9/17/2015 13:11'!
cutBackExecutionToSenderContext
	"When accepting a new version of a method which can't be simulated (i.e. a quick method) we
	 must cut back to the sender.  But this is non-trivial. If the quick method has been reached via
	 a perform: (as it is when one uses Create toi implement a method from an MNU) then the relevant
	 arguments won't be on the stack and we can't simply proceed without crashing the VM."
	| oldContext context sel |
	oldContext := self selectedContext.
	self down.
	context := self selectedContext.
	context jump: (context previousPc - context pc).
	sel := context selectorToSendOrSelf.
	sel numArgs = oldContext method numArgs
		ifTrue:
			[context push: oldContext receiver.
			 oldContext arguments do:
				[:arg| context push: arg]]
		ifFalse:
			[context privRefresh; stepToSendOrReturn]! !

!Debugger methodsFor: 'notifier menu' stamp: 'dtl 7/7/2013 17:54'!
debug
	"Open a full DebuggerView."
	| topView |
	topView := self topView.
	topView model: nil.  "so close won't release me."
	self breakDependents.
	self openFullNoSuspendFrom: topView.
! !

!Debugger methodsFor: 'initialize' stamp: 'kfr 10/4/2000 22:13'!
debugAt: anInteger
	self toggleContextStackIndex: anInteger. 
	 ^ self debug.! !

!Debugger methodsFor: 'code pane menu' stamp: 'topa 3/16/2015 18:38'!
debuggerCodePaneMenu: aMenu
	<codePaneMenu>
	<menuPriority: 200>
	^ aMenu
		add: 'run to here' target: self selector: #runToSelection: argument: self codePaneSelectionInterval;
		add: 'run until...' target: self selector: #runUntil;
		addLine;
		yourself
! !

!Debugger methodsFor: 'accessing' stamp: 'abc 2/12/2010 15:02'!
debuggerMap
	^self selectedContext debuggerMap! !

!Debugger methodsFor: 'context stack menu' stamp: 'tk 4/16/1998 12:19'!
debugProceedMenu: aMenu
	^ aMenu labels: 
'proceed
debug'
	lines: #()
	selections: #(proceed debug )
! !

!Debugger methodsFor: 'user interface' stamp: 'mt 7/9/2016 08:52'!
defaultWindowColor
	^ (Color r: 0.9 g: 0.719 b: 0.719)! !

!Debugger methodsFor: 'code pane' stamp: ''!
doItContext
	"Answer the context in which a text selection can be evaluated."

	contextStackIndex = 0
		ifTrue: [^super doItContext]
		ifFalse: [^self selectedContext]! !

!Debugger methodsFor: 'code pane' stamp: ''!
doItReceiver
	"Answer the object that should be informed of the result of evaluating a
	text selection."

	^self receiver! !

!Debugger methodsFor: 'accessing' stamp: 'tk 4/16/1998 12:16'!
doNothing: newText
	"Notifier window can't accept text"! !

!Debugger methodsFor: 'context stack menu' stamp: 'eem 7/1/2015 16:33'!
doStep
	"Send the selected message in the accessed method, and regain control 
	after the invoked method returns."
	
	| currentContext newContext |
	self okToChange ifFalse: [^ self].
	self checkContextSelection.
	currentContext := self selectedContext.
	newContext := self handleLabelUpdatesIn: [interruptedProcess completeStep: currentContext]
						whenExecuting: currentContext.
	newContext == currentContext ifTrue:
		[newContext := interruptedProcess stepToSendOrReturn].
	self contextStackIndex > 1
		ifTrue: [self resetContext: newContext]
		ifFalse:
			[newContext == currentContext
				ifTrue: [self changed: #contentsSelection.
						self updateInspectors]
				ifFalse: [self resetContext: newContext]].
! !

!Debugger methodsFor: 'context stack menu' stamp: 'tk 4/15/1998 16:55'!
down
	"move down the context stack to the previous (enclosing) context"

	self toggleContextStackIndex: contextStackIndex+1! !

!Debugger methodsFor: 'initialize' stamp: 'ajh 7/20/2003 23:41'!
errorWasInUIProcess: boolean

	errorWasInUIProcess := boolean! !

!Debugger methodsFor: 'context stack (message list)' stamp: 'fbs 12/10/2010 22:22'!
expandStack
	"A Notifier is being turned into a full debugger.  Show a substantial amount of stack in the context pane."

	self newStack: (self contextStackTop stackOfSize: 20).
	contextStackIndex := 0.
	receiverInspector := Inspector inspect: nil.
	contextVariablesInspector := ContextVariablesInspector inspect: nil.
	proceedValue := nil! !

!Debugger methodsFor: 'private' stamp: 'sd 11/20/2005 21:27'!
externalInterrupt: aBoolean

	externalInterrupt := aBoolean ! !

!Debugger methodsFor: 'context stack menu' stamp: 'nice 3/12/2010 22:57'!
fullStack
	"Change from displaying the minimal stack to a full one."

	contextStack last sender isNil
		ifTrue:
			["Already expanded"
			self changed: #flash]
		ifFalse:
			[| oldContextStackIndex |
			oldContextStackIndex := contextStackIndex.
			self fullyExpandStack.
			oldContextStackIndex = contextStackIndex
				ifFalse:
					[self toggleContextStackIndex: oldContextStackIndex]]! !

!Debugger methodsFor: 'context stack (message list)' stamp: 'fbs 12/10/2010 22:21'!
fullyExpandStack
	"Expand the stack to include all of it, rather than the first four or five contexts.
	Well, almost all of it, we better maintain sane limits too."

	self okToChange ifFalse: [^ self].
	self newStack: (self contextStackTop stackOfSize: contextStack size + 100000).
	self changed: #contextStackList! !

!Debugger methodsFor: 'tally support' stamp: 'topa 3/16/2015 15:49'!
getSelectedText
	| m interval text |
	m := self codeTextMorph ifNil: [^ ''].
	interval := m selectionInterval.
	text := m text.
	^ text copyFrom: interval first to: interval last
	! !

!Debugger methodsFor: 'context stack menu' stamp: 'eem 7/1/2015 16:32'!
handleLabelUpdatesIn: aBlock whenExecuting: aContext
	"Send the selected message in the accessed method, and regain control 
	after the invoked method returns."
	
	^aBlock
		on: Notification
		do: [:ex|
			(ex tag isArray
			 and: [ex tag size = 2
			 and: [(ex tag first == aContext or: [ex tag first hasSender: aContext])]])
				ifTrue:
					[self labelString: ex tag second description.
					 ex resume]
				ifFalse:
					[ex pass]]! !

!Debugger methodsFor: 'context stack menu' stamp: 'fbs 4/30/2013 08:38'!
implementMissingMethod: aMessage inClass: aClass
	^ self
		implementMissingMethod: aMessage
		inClass: aClass
		inCategory: (self askForCategoryIn: aClass default: 'as yet unclassified').! !

!Debugger methodsFor: 'context stack menu' stamp: 'fbs 4/30/2013 08:43'!
implementMissingMethod: aMessage inClass: aClass inCategory: aSymbol
	"Create a stub implementation of the missing message and sew it onto the top of the stack, ensuring the context's arguments are set correctly. Debug the new context."
	self pushStubMethodOnStack: aMessage inClass: aClass inCategory: aSymbol.

	"Cut out the sender context. This is the context that signalled the MessageNotUnderstood."
	self selectedContext privSender: self selectedContext sender.
	self resetContext: self selectedContext.
	self debug.! !

!Debugger methodsFor: 'context stack menu' stamp: 'fbs 4/30/2013 08:43'!
implementOverridingMethod: aMessage inClass: aClass inCategory: aSymbol
	"Create a stub implementation of the overriding message and sew it onto the top of the stack, ensuring the context's arguments are set correctly. Debug the new context."
	self pushStubMethodOnStack: aMessage inClass: aClass inCategory: aSymbol.

	"Cut out the sender context. This is the context that signalled the SubclassResponsibility."
	self selectedContext privSender: self selectedContext sender sender.
	self resetContext: self selectedContext.
	self debug.! !

!Debugger methodsFor: 'initialize' stamp: 'mt 4/19/2015 18:00'!
initialExtent
	"Initial extent for the full debugger. For the notifier's extent see #initialExtentForNotifier."
	
	^ 600@700! !

!Debugger methodsFor: 'initialize' stamp: 'mt 4/19/2015 17:49'!
initialExtentForNotifier

	^ 450@200! !

!Debugger methodsFor: 'accessing' stamp: 'fbs 12/10/2010 22:36'!
interruptedContext
	"Answer the suspended context of the interrupted process."

	^self contextStackTop.! !

!Debugger methodsFor: 'accessing' stamp: ''!
interruptedProcess
	"Answer the interrupted process."

	^interruptedProcess! !

!Debugger methodsFor: 'accessing' stamp: 'hmm 7/16/2001 21:54'!
labelString
	^labelString! !

!Debugger methodsFor: 'accessing' stamp: 'hmm 7/16/2001 21:54'!
labelString: aString
	labelString := aString.
	self changed: #relabel! !

!Debugger methodsFor: 'private' stamp: 'yo 12/3/2004 17:14'!
lowSpaceChoices
	"Return a notifier message string to be presented when space is running low."

	^ 'Warning!! Squeak is almost out of memory!!

Low space detection is now disabled. It will be restored when you close or proceed from this error notifier. Don''t panic, but do proceed with caution.

Here are some suggestions:

 If you suspect an infinite recursion (the same methods calling each other again and again), then close this debugger, and fix the problem.

 If you want this computation to finish, then make more space available (read on) and choose "proceed" in this debugger. Here are some ways to make more space available...
   > Close any windows that are not needed.
   > Get rid of some large objects (e.g., images).
   > Leave this window on the screen, choose "save as..." from the screen menu, quit, restart the Squeak VM with a larger memory allocation, then restart the image you just saved, and choose "proceed" in this window.

 If you want to investigate further, choose "debug" in this window.  Do not use the debugger "fullStack" command unless you are certain that the stack is not very deep. (Trying to show the full stack will definitely use up all remaining memory if the low-space problem is caused by an infinite recursion!!).

'
! !

!Debugger methodsFor: 'context stack menu' stamp: 'nice 12/26/2009 01:30'!
mailOutBugReport
	"Compose a useful bug report showing the state of the process as well as vital image statistics as suggested by Chris Norton - 
'Squeak could pre-fill the bug form with lots of vital, but
oft-repeated, information like what is the image version, last update
number, VM version, platform, available RAM, author...'

and address it to the list with the appropriate subject prefix."

	MailSender default ifNil: [^self].

	Cursor write
		showWhile: 
			["Prepare the message"
			| messageStrm |
			messageStrm := WriteStream on: (String new: 1500).
			messageStrm nextPutAll: 'From: ';
			 nextPutAll: MailSender userName;
			 cr;
			 nextPutAll: 'To: squeak-dev@lists.squeakfoundation.org';
			 cr;
			 nextPutAll: 'Subject: ';
			 nextPutAll: '[BUG]'; nextPutAll: self interruptedContext printString;
			 cr;cr;
			 nextPutAll: 'here insert explanation of what you were doing, suspect changes you''ve made and so forth.';cr;cr.
			self interruptedContext errorReportOn: messageStrm.

			MailSender sendMessage: (MailMessage from: messageStrm contents)].
! !

!Debugger methodsFor: 'context stack menu' stamp: 'topa 3/16/2015 16:30'!
mainContextStackMenu: aMenu
	"Set up the menu appropriately for the context-stack-list, unshifted"
	<contextStackMenuShifted: false>
	^ aMenu addList: #(
			('fullStack (f)' 				fullStack) 
			('restart (r)' 				restart) 
			('proceed (p)' 				proceed) 
			('step (t)' 					doStep) 
			('step through (T)'	 		stepIntoBlock) 
			('send (e)' 					send) 
			('where (w)' 				where) 
			('peel to first like this' 		peelToFirst) 
			- 
			('return entered value' 		returnValue) 
			- 
			('toggle break on entry'	toggleBreakOnEntry) 
			('senders of    (n)' 			browseSendersOfMessages) 
			('implementors of    (m)' 	browseMessages) 
			('inheritance (i)' 			methodHierarchy) 
			- 
			('versions (v)' 				browseVersions) 
			- 
			('references    (r)' 			browseVariableReferences) 
			('assignments    (a)' 		browseVariableAssignments) 
			- 
			('class refs (N)' 				browseClassRefs) 
			('browse full (b)' 			browseMethodFull) 
			('file out ' 			 		fileOutMessage) 
			- 
			('mail out bug report' 		mailOutBugReport));
		yourself
! !

!Debugger methodsFor: 'context stack (message list)' stamp: 'ul 4/3/2016 03:55'!
messageHelpAt: anIndex
	"Show the first n lines of the sources code of the selected message."
	
	| method source formatted lineCount |
	Preferences balloonHelpInMessageLists ifFalse: [^ nil].
	contextStack size < anIndex ifTrue: [^ nil].
	
	method := (contextStack at: anIndex) method.
	
	source := method getSource.
	formatted := (Smalltalk classNamed: #SHTextStylerST80)
		ifNil: [ source asText ]
		ifNotNil: [ :SHTextStylerST80 |
			SHTextStylerST80 new
				classOrMetaClass: method methodClass;
				styledTextFor: source asText ].
	
	lineCount := 0.
	source doWithIndex: [:char :index |
		char = Character cr ifTrue: [lineCount := lineCount + 1].
		lineCount > 10 ifTrue: [
			formatted := formatted copyFrom: 1 to: index-1.
			formatted append: ' [...]'.
			^ formatted]].

	^ formatted! !

!Debugger methodsFor: 'context stack (message list)' stamp: 'mt 11/6/2015 12:01'!
messageIconAt: anIndex

	Browser showMessageIcons
		ifFalse: [^ nil].

	^ ToolIcons iconNamed: (ToolIcons
		iconForClass: (contextStack at: anIndex) method methodClass
		selector: (contextStack at: anIndex) method selector)! !

!Debugger methodsFor: 'context stack (message list)' stamp: ''!
messageListIndex
	"Answer the index of the currently selected context."

	^contextStackIndex! !

!Debugger methodsFor: 'context stack menu' stamp: 'sw 3/16/2001 17:20'!
messageListMenu: aMenu shifted: shifted
	"The context-stack menu takes the place of the message-list menu in the debugger, so pass it on"

	^ self contextStackMenu: aMenu shifted: shifted! !

!Debugger methodsFor: 'private' stamp: 'sd 11/20/2005 21:27'!
newStack: stack
	| oldStack diff |
	oldStack := contextStack.
	contextStack := stack.
	(oldStack == nil or: [oldStack last ~~ stack last])
		ifTrue: [contextStackList := contextStack collect: [:ctx | ctx printString].
				^ self].
	"May be able to re-use some of previous list"
	diff := stack size - oldStack size.
	contextStackList := diff <= 0
		ifTrue: [contextStackList copyFrom: 1-diff to: oldStack size]
		ifFalse: [diff > 1
				ifTrue: [contextStack collect: [:ctx | ctx printString]]
				ifFalse: [(Array with: stack first printString) , contextStackList]]! !

!Debugger methodsFor: 'initialize' stamp: 'eem 10/6/2014 12:52'!
openFullNoSuspendFrom: topView
	"Create, schedule and answer a full debugger with the given label. Do not terminate the current active process."

	| oldContextStackIndex |
	oldContextStackIndex := contextStackIndex.
	self expandStack. "Sets contextStackIndex to zero."
	^[ToolBuilder default openDebugger: self label: topView label closing: topView] ensure:
		[self toggleContextStackIndex: oldContextStackIndex]! !

!Debugger methodsFor: 'initialize' stamp: 'eem 10/6/2014 12:52'!
openFullNoSuspendLabel: aString
	"Create, schedule and answer a full debugger with the given label. Do not terminate the current active process."

	| oldContextStackIndex |
	oldContextStackIndex := contextStackIndex.
	self expandStack. "Sets contextStackIndex to zero."
	^[ToolBuilder default openDebugger: self label: aString] ensure:
		[self toggleContextStackIndex: oldContextStackIndex]! !

!Debugger methodsFor: 'initialize' stamp: 'mt 8/25/2015 13:35'!
openNotifierContents: msgString label: label
	"Create, schedule and answer a notifier view with the given label and message. A notifier view shows just the message or the first several lines of the stack, with a menu that allows the user to open a full debugger if so desired."
	"NOTE: When this method returns, a new process has been scheduled to run the windows, and thus this notifier, but the previous active process has not been suspended.  The sender will do this."
	| msg builder spec |

	Sensor flushKeyboard.
	savedCursor := Cursor currentCursor.
	Cursor currentCursor: Cursor normal.
	msg := (label beginsWith: 'Space is low')
		ifTrue: [self lowSpaceChoices, (msgString ifNil: [String empty])]
		ifFalse: [msgString].
	builder := ToolBuilder default.
	spec := self buildNotifierWith: builder label: label message: msg.
	self expandStack.
	^[builder openDebugger: spec] ensure:
		[errorWasInUIProcess := Project current spawnNewProcessIfThisIsUI: interruptedProcess]
! !

!Debugger methodsFor: 'toolbuilder' stamp: 'mt 11/11/2015 11:20'!
optionalButtonsFrame
	^LayoutFrame new
		leftFraction: 0 offset: 0;
		topFraction: 0 offset: self buttonHeight;
		rightFraction: 1 offset: 0;
		bottomFraction: 0 offset: self buttonHeight*2! !

!Debugger methodsFor: 'code pane' stamp: 'tk 5/2/1998 10:04'!
pc

	^ self pcRange! !

!Debugger methodsFor: 'code pane' stamp: 'eem 3/12/2009 14:54'!
pcRange
	"Answer the indices in the source code for the method corresponding to 
	the selected context's program counter value."

	(selectingPC and: [contextStackIndex ~= 0]) ifFalse:
		[^1 to: 0].
	self selectedContext isDead ifTrue:
		[^1 to: 0].
	^self selectedContext debuggerMap
		rangeForPC: self selectedContext pc
		contextIsActiveContext: contextStackIndex = 1! !

!Debugger methodsFor: 'context stack menu' stamp: 'md 2/20/2006 20:23'!
peelToFirst
	"Peel the stack back to the second occurance of the currently selected message.  Very useful for an infinite recursion.  Gets back to the second call so you can see one complete recursion cycle, and how it was called at the beginning.  Also frees a lot of space!!"

	| ctxt |
	contextStackIndex = 0 ifTrue: [^ Beeper beep].
	"self okToChange ifFalse: [^ self]."
	ctxt := interruptedProcess popTo: self selectedContext findSecondToOldestSimilarSender.
	self resetContext: ctxt.
! !

!Debugger methodsFor: 'code pane menu' stamp: 'tk 4/17/1998 17:25'!
perform: selector orSendTo: otherTarget
	"Selector was just chosen from a menu by a user.  If can respond, then perform it on myself.  If not, send it to otherTarget, presumably the editPane from which the menu was invoked." 

	| result |
	(#(debug proceed) includes: selector)		"When I am a notifier window"
		ifTrue: [^ self perform: selector]
		ifFalse: [result := super perform: selector orSendTo: otherTarget.
				selector == #doIt ifTrue: [
					result ~~ #failedDoit ifTrue: [self proceedValue: result]].
				^ result]! !

!Debugger methodsFor: 'context stack menu' stamp: 'fbs 4/30/2013 08:39'!
populateImplementInMenu: aMenu

	| msg |
	msg := self selectedContext at: 1.
	self selectedContext receiver class withAllSuperclasses do:
		[:each |
		aMenu add: each name target: self selector: #implementMissingMethod:inClass: argumentList: (Array with: msg with: each)].
	^ aMenu

! !

!Debugger methodsFor: 'toolbuilder' stamp: 'fbs 4/4/2010 23:52'!
preDebugButtonQuadFrame
	^LayoutFrame new
		leftFraction: 0 offset: 0;
		topFraction: 0 offset: 0;
		rightFraction: 1 offset: 0;
		bottomFraction: 0 offset: self buttonHeight.! !

!Debugger methodsFor: 'initialize' stamp: 'yo 3/15/2005 13:18'!
preDebugButtonQuads

	^Preferences eToyFriendly
		ifTrue: [
	{
	{'Store log' translated.	#storeLog. 	#blue. 	'write a log of the encountered problem' translated}.
	{'Abandon' translated.	#abandon. 	#black.	'abandon this execution by closing this window' translated}.
	{'Debug'	 translated.		#debug. 	#red. 	'bring up a debugger' translated}}]
		ifFalse: [
	{
	{'Proceed' translated.	#proceed. 	#blue. 	'continue execution' translated}.
	{'Abandon' translated.	#abandon. 	#black.	'abandon this execution by closing this window' translated}.
	{'Debug'	 translated.		#debug.		#red. 	'bring up a debugger' translated}}]
! !

!Debugger methodsFor: 'toolbuilder' stamp: 'tfel 8/8/2016 09:25'!
preDebugMessageString
	^ message ifNil: ['An error has occurred; you should probably just hit ''abandon''.  Sorry!!' translated].! !

!Debugger methodsFor: 'context stack menu' stamp: 'di 5/5/1998 00:07'!
proceed
	"Proceed execution of the receiver's model, starting after the expression at 
	which an interruption occurred."

	Smalltalk okayToProceedEvenIfSpaceIsLow ifTrue: [
		self proceed: self topView].
! !

!Debugger methodsFor: 'accessing' stamp: ''!
proceedValue
	"Answer the value to return to the selected context when the interrupted 
	process proceeds."

	^proceedValue! !

!Debugger methodsFor: 'accessing' stamp: 'sd 11/20/2005 21:27'!
proceedValue: anObject 
	"Set the value to be returned to the selected context when the interrupted 
	process proceeds."

	proceedValue := anObject! !

!Debugger methodsFor: 'context stack menu' stamp: 'ajh 1/24/2003 12:29'!
proceed: aTopView 
	"Proceed from the interrupted state of the currently selected context. The 
	argument is the topView of the receiver. That view is closed."

	self okToChange ifFalse: [^ self].
	self checkContextSelection.
	self resumeProcess: aTopView! !

!Debugger methodsFor: 'private' stamp: 'fbs 12/10/2010 22:22'!
process: aProcess controller: aController context: aContext

	super initialize.
	Smalltalk at: #MessageTally ifPresentAndInMemory: [ :tally |
		tally terminateTimerProcess].
	contents := nil. 
	interruptedProcess := aProcess.
	interruptedController := aController.
	self newStack: (aContext stackOfSize: 1).
	contextStackIndex := 1.
	externalInterrupt := false.
	selectingPC := true.
	Smalltalk isMorphic ifTrue:
		[errorWasInUIProcess := false]! !

!Debugger methodsFor: 'private' stamp: 'fbs 4/30/2013 08:42'!
pushStubMethodOnStack: aMessage inClass: aClass inCategory: aSymbol
	"Create a stub implementation of the message and sew it onto the top of the stack, ensuring the context's arguments are set correctly."
	aClass
		compile: aMessage createStubMethod
		classified: aSymbol.
	self setContentsToForceRefetch.
	self selectedContext privRefreshWith: (aClass lookupSelector: aMessage selector).
	aMessage numArgs > 0 ifTrue:
		[aMessage arguments withIndexDo:
			[:arg :index|
			self selectedContext tempAt: index put: arg]].! !

!Debugger methodsFor: 'accessing' stamp: ''!
receiver
	"Answer the receiver of the selected context, if any. Answer nil 
	otherwise."

	contextStackIndex = 0
		ifTrue: [^nil]
		ifFalse: [^self selectedContext receiver]! !

!Debugger methodsFor: 'toolbuilder' stamp: 'cmm 5/1/2011 22:13'!
receiverClass
	^ self selectedContext receiver class! !

!Debugger methodsFor: 'accessing' stamp: ''!
receiverInspector
	"Answer the instance of Inspector that is providing a view of the 
	variables of the selected context's receiver."

	^receiverInspector! !

!Debugger methodsFor: 'accessing' stamp: 'md 2/20/2006 18:52'!
receiverInspectorObject: obj context: ctxt

	"set context before object so it can refer to context when building field list"
	receiverInspector context: ctxt.
	receiverInspector object: obj.
! !

!Debugger methodsFor: 'initialize' stamp: 'jm 8/20/1998 18:31'!
release

	self windowIsClosing.
	super release.
! !

!Debugger methodsFor: 'private' stamp: 'bf 11/16/2012 21:03'!
resetContext: aContext 
	^ self resetContext: aContext changeContents: true! !

!Debugger methodsFor: 'private' stamp: 'bf 11/16/2012 18:27'!
resetContext: aContext changeContents: aBoolean
	"Used when a new context becomes top-of-stack, for instance when the
	method of the selected context is re-compiled, or the simulator steps or
	returns to a new method. There is room for much optimization here, first
	to save recomputing the whole stack list (and text), and secondly to avoid
	recomposing all that text (by editing the paragraph instead of recreating it)."

	| oldContext |
	oldContext := self selectedContext.
	self newStack: aContext contextStack.
	self changed: #contextStackList.
	self contextStackIndex: 1 oldContextWas: oldContext.
	aBoolean ifTrue: [self contentsChanged].
! !

!Debugger methodsFor: 'context stack menu' stamp: 'ajh 3/4/2004 23:14'!
restart
	"Proceed from the initial state of the currently selected context. The 
	argument is a controller on a view of the receiver. That view is closed."
	"Closing now depends on a preference #restartAlsoProceeds - hmm 9/7/2001 16:46"

	| ctxt noUnwindError |
	self okToChange ifFalse: [^ self].
	self checkContextSelection.
	ctxt := interruptedProcess popTo: self selectedContext.
	noUnwindError := false.
	ctxt == self selectedContext ifTrue: [
		noUnwindError := true.
		interruptedProcess restartTop; stepToSendOrReturn].
	self resetContext: ctxt.
	(Preferences restartAlsoProceeds and: [noUnwindError]) ifTrue: [self proceed].
! !

!Debugger methodsFor: 'private' stamp: 'dtl 7/7/2013 18:45'!
resumeProcess: aTopView

	^ Project current
		dispatchTo: self
		addPrefixAndSend: #ResumeProcess:
		withArguments: { aTopView }
! !

!Debugger methodsFor: 'context stack menu' stamp: 'rbb 3/1/2005 10:50'!
returnValue
	"Force a return of a given value to the previous context!!"

	| previous selectedContext expression value |
	contextStackIndex = 0 ifTrue: [^Beeper beep].
	selectedContext := self selectedContext.
	expression := UIManager default request: 'Enter expression for return value:'.
	value := Compiler new 
				evaluate: expression
				in: selectedContext
				to: selectedContext receiver.
	previous := selectedContext sender.
	self resetContext: previous.
	interruptedProcess popTo: previous value: value! !

!Debugger methodsFor: 'code pane menu' stamp: 'nk 5/31/2003 07:38'!
runToSelection: selectionInterval
	| currentContext |
	self pc first >= selectionInterval first ifTrue: [ ^self ].
	currentContext := self selectedContext.
	[ currentContext == self selectedContext and: [ self pc first < selectionInterval first ] ] whileTrue: [ self doStep ].! !

!Debugger methodsFor: 'code pane menu' stamp: 'eem 10/21/2014 15:37'!
runUntil
	"Step until an expression evaluates to other than false, reporting an error if it doesn't evaluate to true.
	 Remember the expression in an inst var.  If shift is pressed when the expression is supplied, don't update the UI.
	 If shift is pressed while stepping, stop stepping.  Using a user interrupt to break out would be more natural
	 but Squeak currently doesn't provide a UserInterrupt expection.  It should do."
	| expression receiver context method value lastUpdate updateUI breakOnShift |
	expression := UIManager default 
					request: 'run until expression is true (shift to disable ui update; shift to break).'
					initialAnswer: (untilExpression ifNil: 'boolean expression').
	(expression isNil or: [expression isEmpty]) ifTrue:
		[^self].
	updateUI := breakOnShift := Sensor shiftPressed not.
	untilExpression := expression.
	context := self selectedContext.
	receiver := context receiver.
	method := receiver class evaluatorClass new 
				compiledMethodFor: untilExpression
				in: context
				to: receiver
				notifying: nil
				ifFail: [^ #failedDoit].

	lastUpdate := Time millisecondClockValue.
	Cursor execute showWhile:
		[[self selectedContext == context
		  and: [context willReturn not
		  and: [(value := receiver with: context executeMethod: method) == false]]] whileTrue:
			[interruptedProcess completeStep: self selectedContext.
			 self selectedContext == context ifTrue:
				[self resetContext: interruptedProcess stepToSendOrReturn changeContents: false].
			 Time millisecondClockValue - lastUpdate > 50 ifTrue:
				[updateUI ifTrue:
					[self changed: #contentsSelection.
					 World displayWorldSafely].
				 breakOnShift
					ifTrue: [Sensor shiftPressed ifTrue:
								[self changed: #contentsSelection.
								 self updateInspectors.
								 ^self]]
					ifFalse: [Sensor shiftPressed ifFalse: [breakOnShift := true]].
				 lastUpdate := Time millisecondClockValue]]].
	self changed: #contentsSelection.
	self updateInspectors.
	(value ~~ false and: [value ~~ true]) ifTrue:
		[UIManager default inform: 'expression ', (untilExpression contractTo: 40), ' answered ', (value printString contractTo: 20), '!!!!']! !

!Debugger methodsFor: 'class list' stamp: 'md 2/17/2006 09:32'!
selectedClass
	"Answer the class in which the currently selected context's method was 
	found."

	^self selectedContext methodClass! !

!Debugger methodsFor: 'class list' stamp: 'md 2/17/2006 09:29'!
selectedClassOrMetaClass
	"Answer the class in which the currently selected context's method was 
	found."

	^self selectedClass! !

!Debugger methodsFor: 'private' stamp: 'fbs 12/10/2010 22:24'!
selectedContext

	contextStackIndex = 0
		ifTrue: [^contextStack first]
		ifFalse: [^contextStack at: contextStackIndex]! !

!Debugger methodsFor: 'context stack (message list)' stamp: 'nice 1/10/2011 21:09'!
selectedMessage
	"Answer the source code of the currently selected context."
	| aContext |
	^contents := (aContext := self selectedContext) debuggerMap sourceText asText makeSelectorBoldIn: aContext home receiver class! !

!Debugger methodsFor: 'message category list' stamp: ''!
selectedMessageCategoryName
	"Answer the name of the message category of the message of the 
	currently selected context."

	^self selectedClass organization categoryOfElement: self selectedMessageName! !

!Debugger methodsFor: 'context stack (message list)' stamp: 'tfel 9/15/2009 23:07'!
selectedMessageName
	"Answer the message selector of the currently selected context.
	 If the method is unbound we can still usefully answer its old selector."

	| selector |
	selector := self selectedContext selector.
	^(selector ~~ self selectedContext method selector
	    and: [selector beginsWith: 'DoIt'])
		ifTrue: [self selectedContext method selector]
		ifFalse: [selector]! !

!Debugger methodsFor: 'context stack menu' stamp: 'sd 11/20/2005 21:27'!
selectPC
	"Toggle the flag telling whether to automatically select the expression 
	currently being executed by the selected context."

	selectingPC := selectingPC not! !

!Debugger methodsFor: 'context stack menu' stamp: 'ajh 1/24/2003 12:29'!
send
	"Send the selected message in the accessed method, and take control in 
	the method invoked to allow further step or send."

	self okToChange ifFalse: [^ self].
	self checkContextSelection.
	interruptedProcess step: self selectedContext.
	self resetContext: interruptedProcess stepToSendOrReturn.
! !

!Debugger methodsFor: 'context stack menu' stamp: 'cmm 10/16/2015 14:15'!
shiftedContextStackMenu: aMenu
	"Set up the menu appropriately for the context-stack-list, shifted"
	<contextStackMenuShifted: true>
	^ aMenu addList: #(
			('browse class hierarchy'				classHierarchy)
			('browse class'							browseClass)
			('implementors of sent messages'		browseAllMessages)
			('change sets with this method'			findMethodInChangeSets)
			-	
			('inspect instances'						inspectInstances)
			('inspect subinstances'					inspectSubInstances)
			-	
			('revert to previous version'			revertToPreviousVersion)
			('remove from current change set'		removeFromCurrentChanges)
			('revert & remove from changes'		revertAndForget));
		yourself
! !

!Debugger methodsFor: 'dependents access' stamp: 'di 1/14/1999 09:28'!
step 
	"Update the inspectors."

	receiverInspector ifNotNil: [receiverInspector step].
	contextVariablesInspector ifNotNil: [contextVariablesInspector step].
! !

!Debugger methodsFor: 'context stack menu' stamp: 'eem 7/1/2015 16:36'!
stepIntoBlock
	"Send messages until you return to the present method context.
	 Used to step into a block in the method."

	self handleLabelUpdatesIn:
			[interruptedProcess stepToHome: self selectedContext]
		whenExecuting: self selectedContext.
	self resetContext: interruptedProcess stepToSendOrReturn! !

!Debugger methodsFor: 'notifier menu' stamp: 'nice 2/13/2011 22:24'!
storeLog

	Smalltalk logSqueakError: labelString printString inContext: self contextStackTop
! !

!Debugger methodsFor: 'tally support' stamp: 'topa 3/16/2015 15:50'!
tally

	self codeTextMorph ifNotNil: [:o| o tallyIt] ifNil: [Beeper beep]
! !

!Debugger methodsFor: 'toolbuilder' stamp: 'mt 11/11/2015 11:24'!
textFrame
	
	^ super textFrame
		topOffset: (self wantsOptionalButtons ifTrue: [self buttonHeight * 2] ifFalse: [self buttonHeight]);
		yourself! !

!Debugger methodsFor: 'context stack (message list)' stamp: ''!
toggleContextStackIndex: anInteger 
	"If anInteger is the same as the index of the selected context, deselect it. 
	Otherwise, the context whose index is anInteger becomes the selected 
	context."

	self contextStackIndex: 
		(contextStackIndex = anInteger
			ifTrue: [0]
			ifFalse: [anInteger])
		oldContextWas:
		(contextStackIndex = 0
			ifTrue: [nil]
			ifFalse: [contextStack at: contextStackIndex])! !

!Debugger methodsFor: 'context stack menu' stamp: 'tk 4/15/1998 16:55'!
up
	"move up the context stack to the next (enclosed) context"

	contextStackIndex > 1 ifTrue: [self toggleContextStackIndex: contextStackIndex-1]! !

!Debugger methodsFor: 'self-updating' stamp: 'mt 3/10/2015 21:49'!
updateCodePaneIfNeeded
	"Ignore. We must not update code but stick to particular compiled methods on the stack."! !

!Debugger methodsFor: 'dependents access' stamp: 'hmm 7/15/2001 19:48'!
updateInspectors 
	"Update the inspectors on the receiver's variables."

	receiverInspector == nil ifFalse: [receiverInspector update].
	contextVariablesInspector == nil ifFalse: [contextVariablesInspector update]! !

!Debugger methodsFor: 'toolbuilder' stamp: 'mt 11/10/2015 13:35'!
wantsAnnotationPane

	^ self class wantsAnnotationPane! !

!Debugger methodsFor: 'toolbuilder' stamp: 'mt 11/11/2015 11:09'!
wantsOptionalButtons
	"The debugger benefits so majorly from the optional buttons that we put them up regardless of the global setting.  Some traditionalists will want to change this method manually!!"

	^ Preferences extraDebuggerButtons! !

!Debugger methodsFor: 'dependents access' stamp: 'di 1/14/1999 09:25'!
wantsSteps
 
	^ true! !

!Debugger methodsFor: 'context stack menu' stamp: 'nk 2/6/2001 19:34'!
where
	"Select the expression whose evaluation was interrupted."

	selectingPC := true.
	self contextStackIndex: contextStackIndex oldContextWas: self selectedContext
! !

!Debugger methodsFor: 'initialize' stamp: 'fbs 12/10/2010 22:21'!
windowIsClosing
	"My window is being closed; clean up. Restart the low space watcher."

	interruptedProcess == nil ifTrue: [^ self].
	interruptedProcess terminate.
	interruptedProcess := nil.
	interruptedController := nil.
	contextStack := nil.
	receiverInspector := nil.
	contextVariablesInspector := nil.
	Smalltalk installLowSpaceWatcher.  "restart low space handler"
! !

CodeHolder subclass: #DependencyBrowser
	instanceVariableNames: 'packageList packageDeps classDeps classList messageList packageListIndex packageDepsIndex classDepsIndex classListIndex messageListIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Browser'!

!DependencyBrowser commentStamp: 'fbs 5/6/2011 11:29' prior: 0!
A simple dependency browser showing five panes:
[1]: Packages: The list of available packages in the system.
[2]: Package Dependencies: The dependent packages of the currently selected package.
[3]: Class Dependencies: The classes causing the dependencies.
[4]: Class List: The classes introducing the dependencies.
[5]: Messages: The messages introducing the dependencies.!

!DependencyBrowser class methodsFor: 'class initialization' stamp: 'cmm 1/8/2011 15:40'!
initialize
	"Initializes the receiver class"
 
	 TheWorldMenu registerOpenCommand: {'Dependency Browser'. {self. #open}}. ! !

!DependencyBrowser class methodsFor: 'opening' stamp: 'ar 3/2/2010 22:19'!
open
	"DependencyBrowser open"
	^ToolBuilder open: self! !

!DependencyBrowser methodsFor: 'contents' stamp: 'laza 2/3/2011 18:33'!
aboutToStyle: aStyler
	"This is a notification that aStyler is about to re-style its text.
	Set the classOrMetaClass in aStyler, so that identifiers
	will be resolved correctly.
	Answer true to allow styling to proceed, or false to veto the styling"
	| selectedClass |
	selectedClass := self classListSelection ifNil:[^false].
	self messageListSelection ifNil:[^false].
	aStyler classOrMetaClass: ((self messageListSelection == #Definition) ifFalse:[Smalltalk classNamed: selectedClass]).
	^true! !

!DependencyBrowser methodsFor: 'toolbuilder' stamp: 'cmm 1/29/2011 17:10'!
buildClassDepsWith: builder
	| listSpec |
	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		name: 'Required Classes' ;
		list: #classDeps; 
		getIndex: #classDepsIndex; 
		setIndex: #classDepsIndex:; 
		menu: #classDepsMenu:; 
		keyPress: #classDepsKey:from:.
	^listSpec
! !

!DependencyBrowser methodsFor: 'toolbuilder' stamp: 'cmm 1/29/2011 17:13'!
buildClassListWith: builder
	| listSpec |
	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		name: 'Client classes' ;
		list: #classList; 
		getIndex: #classListIndex; 
		setIndex: #classListIndex:; 
		menu: #classListMenu:; 
		keyPress: #classListKey:from:.
	^listSpec
! !

!DependencyBrowser methodsFor: 'toolbuilder' stamp: 'cmm 1/29/2011 17:14'!
buildMessageListWith: builder
	| listSpec |
	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		name: 'Client methods' ;
		list: #messageList; 
		getIndex: #messageListIndex; 
		setIndex: #messageListIndex:; 
		menu: #messageListMenu:; 
		keyPress: #messageListKey:from:.
	^listSpec
! !

!DependencyBrowser methodsFor: 'toolbuilder' stamp: 'cmm 1/29/2011 17:10'!
buildPackageDepsWith: builder
	| listSpec |
	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		name: 'Required Packages' ;
		list: #packageDeps; 
		getIndex: #packageDepsIndex; 
		setIndex: #packageDepsIndex:; 
		menu: #packageDepsMenu:; 
		keyPress: #packageDepsKey:from:.
	^listSpec
! !

!DependencyBrowser methodsFor: 'toolbuilder' stamp: 'cmm 1/29/2011 17:08'!
buildPackageListWith: builder
	| listSpec |
	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		name: 'Packages' ;
		list: #packageList; 
		getIndex: #packageListIndex; 
		setIndex: #packageListIndex:; 
		menu: #packageListMenu:; 
		keyPress: #packageListKey:from:.
	^listSpec
! !

!DependencyBrowser methodsFor: 'toolbuilder' stamp: 'ar 3/2/2010 21:33'!
buildWith: builder
	"Create the ui for the browser"
	| windowSpec max |
	max := self wantsOptionalButtons ifTrue:[0.42] ifFalse:[0.5].
	windowSpec := self buildWindowWith: builder specs: {
		(0@0 corner: 0.2@max) -> [self buildPackageListWith: builder].
		(0.2@0 corner: 0.4@max) -> [self buildPackageDepsWith: builder].
		(0.4@0 corner: 0.6@max) -> [self buildClassDepsWith: builder].
		(0.6@0 corner: 0.8@max) -> [self buildClassListWith: builder].
		(0.8@0 corner: 1.0@max) -> [self buildMessageListWith: builder].
		(0@max corner: 1@1) -> [self buildCodePaneWith: builder].
	}.
	^builder build: windowSpec! !

!DependencyBrowser methodsFor: 'class deps' stamp: 'fbs 5/6/2011 07:38'!
classDeps
	"Class dependencies for the currently selected package"
	packageDeps ifNil: [^ #()].
	^ (packageDeps at: self packageDepsSelection ifAbsent:[#()]) sort.
! !

!DependencyBrowser methodsFor: 'class deps' stamp: 'ar 3/2/2010 22:10'!
classDepsIndex
	"Class dependency selection"
	^classDepsIndex ifNil:[0]! !

!DependencyBrowser methodsFor: 'class deps' stamp: 'ar 3/2/2010 22:10'!
classDepsIndex: idx
	"Class dependency selection"
	classDepsIndex := idx.
	self classListIndex: 0.
	self changed: #classDepsIndex.
	self changed: #classList.
! !

!DependencyBrowser methodsFor: 'class deps' stamp: 'fbs 7/24/2013 11:28'!
classDepsKey: aCharacter from: aPluggableListMorphPlus 
	aCharacter = $N ifTrue: [^ self referencesToIt: (self classDeps at: self classDepsIndex)].! !

!DependencyBrowser methodsFor: 'class deps' stamp: 'nice 1/24/2011 23:26'!
classDepsMenu: aMenu
	^aMenu! !

!DependencyBrowser methodsFor: 'class deps' stamp: 'ar 3/2/2010 22:10'!
classDepsSelection
	"Class dependency selection"
	^(self classDepsIndex between: 1 and: self classDeps size)
		ifTrue:[self classDeps at: self classDepsIndex].! !

!DependencyBrowser methodsFor: 'class list' stamp: 'fbs 5/6/2011 11:30'!
classList
	"List of classes that refer to dependencies"
	classDeps ifNil: [^ #()].
	
	"classList stores the actual classes displayed in my class list, corresponding to the collection of Strings returned by self classList. This allows us to unambiguously determine the class or metaclass currently being browsed simply by knowing the index of the selected class."
	classList := ((classDeps at: self classDepsSelection ifAbsent:[#()]) 
		collect:[:mref| mref actualClass] as: Set) asArray sort: [:a :b | a name < b name].
		
	^ classList collect: #name.! !

!DependencyBrowser methodsFor: 'class list' stamp: 'ar 3/2/2010 22:10'!
classListIndex
	"Class list selection"
	^classListIndex ifNil:[0]! !

!DependencyBrowser methodsFor: 'class list' stamp: 'ar 3/2/2010 22:10'!
classListIndex: idx
	"Class list selection"
	classListIndex := idx.
	self messageListIndex: 0.
	self changed: #classListIndex.
	self changed: #messageList.
! !

!DependencyBrowser methodsFor: 'class list' stamp: 'nice 1/24/2011 23:26'!
classListMenu: aMenu
	^aMenu! !

!DependencyBrowser methodsFor: 'class list' stamp: 'ar 3/2/2010 22:10'!
classListSelection
	"Class list selection"
	^(self classListIndex between: 1 and: self classList size)
		ifTrue:[self classList at: self classListIndex]! !

!DependencyBrowser methodsFor: 'package deps' stamp: 'yo 4/17/2012 23:27'!
computePackageDependencies: pkgName
	"Compute the dependencies for the given package"
	| pi |
	classDeps := Dictionary new.
	packageDeps := Dictionary new.
	pkgName ifNil:[^self].
	pi := PackageOrganizer default packageNamed: pkgName ifAbsent:[^self]. "unloaded"
	pi classes do:[:pkgClass| 
		(classDeps at: (pkgClass superclass ifNil:[ProtoObject]) name
			ifAbsentPut:[OrderedCollection new]) add: 
				(MethodReference class: pkgClass selector: #Definition)].

	pi methods do:[:mref| | cm |
		cm := mref compiledMethod.
		1 to: cm numLiterals do:[:i| | lit |
			((lit := cm literalAt: i) isVariableBinding and:[lit value isBehavior]) ifTrue:[
				(classDeps at: lit value name ifAbsentPut:[OrderedCollection new])
					add: (MethodReference class: cm methodClass selector: cm selector)]]].

	classDeps keys do:[:className| | aClass pkg |
		aClass := Smalltalk classNamed: className.
		pkg := aClass ifNil: [nil] ifNotNil: [PackageOrganizer default packageOfClass: aClass ifNone:[nil]].
		pkg ifNil:[
			Transcript cr; show: 'WARNING: No package for ', className.
			(classDeps removeKey: className) do:[:each| Transcript crtab; show: each].
		] ifNotNil:[
			(packageDeps at: pkg name ifAbsentPut:[OrderedCollection new]) add: className.
		].
	].

	(packageDeps removeKey: pkgName ifAbsent:[#()]) do:[:each|
		classDeps removeKey: each ifAbsent:[].
	].! !

!DependencyBrowser methodsFor: 'contents' stamp: 'yo 5/11/2012 11:41'!
contents: input notifying: aController 
	"The retrieved information has changed and its source must now be
	 updated. The information can be a variety of things, depending on
	 the list selections (such as templates for class or message definition,
	 methods) or the user menu commands (such as definition, comment,
	 hierarchy).  Answer the result of updating the source."

	| aString aText theClass theMethodName |
	self changed: #annotation.
	aString := input asString.
	aText := input asText.
	
	theClass := self selectedClassOrMetaClass.
	theMethodName := self selectedMessageName.
	(theClass notNil and: [theMethodName notNil]) ifTrue: [
		^ self okayToAccept
				ifFalse:
					[false]
				ifTrue:
					[self defineMessageFrom: aText notifying: aController]].
	self error: 'unacceptable accept'
! !

!DependencyBrowser methodsFor: 'contents' stamp: 'yo 5/11/2012 11:20'!
defineMessageFrom: aString notifying: aController
	"Compile the expressions in aString. Notify aController if a syntax error occurs. Install the compiled method in the selected class classified under  the currently selected message category name. Answer the selector obtained if compilation succeeds, nil otherwise."
	| selectedMessageName selector category oldMessageList |
	selectedMessageName := self selectedMessageName.
	oldMessageList := self messageList.
	contents := nil.
	selector := (self selectedClassOrMetaClass newParser parseSelector: aString).
	selector := self selectedClassOrMetaClass
				compile: aString
				classified: (category := self selectedMessageCategoryName)
				notifying: aController.
	selector == nil ifTrue: [^ false].
	contents := aString copy.
	^ true
! !

!DependencyBrowser methodsFor: 'package list' stamp: 'fbs 5/5/2011 17:47'!
hasPackageSelected
	^ packageListIndex > 0.! !

!DependencyBrowser methodsFor: 'initialize-release' stamp: 'fbs 5/5/2011 17:52'!
initialize
	super initialize.
	packageListIndex := 0.
	packageDepsIndex := 0.
	classDepsIndex := 0.
	classListIndex := 0.
	messageListIndex := 0.! !

!DependencyBrowser methodsFor: 'message list' stamp: 'fbs 5/6/2011 07:33'!
messageList
	"List of messages creating dependencies"
	| selectedClass |
	classDeps ifNil: [^ #()].
	
	selectedClass := self classListSelection.
	^((classDeps at: self classDepsSelection ifAbsent:[#()]) 
		select:[:each| each actualClass name = selectedClass]
		thenCollect:[:mref| mref methodSymbol]) asArray sort! !

!DependencyBrowser methodsFor: 'message list' stamp: 'ar 3/2/2010 22:10'!
messageListIndex
	"Message list selection"
	^messageListIndex ifNil:[0]! !

!DependencyBrowser methodsFor: 'message list' stamp: 'ar 3/2/2010 22:10'!
messageListIndex: idx
	"Message list selection"
	messageListIndex := idx.
	self changed: #messageListIndex.
	self changed: #contents.! !

!DependencyBrowser methodsFor: 'message list' stamp: 'nice 1/24/2011 23:26'!
messageListMenu: aMenu
	^aMenu! !

!DependencyBrowser methodsFor: 'message list' stamp: 'ar 3/2/2010 22:10'!
messageListSelection
	"Message list selection"
	^(self messageListIndex between: 1 and: self messageList size)
		ifTrue:[self messageList at: self messageListIndex]! !

!DependencyBrowser methodsFor: 'package deps' stamp: 'ar 3/2/2010 22:09'!
packageDeps
	"Package dependencies for the currently selected package"
	packageDeps ifNil:[
		packageDeps := Dictionary new.
		Cursor wait showWhile:[
			self computePackageDependencies: self packageListSelection.
		].
	].
	^packageDeps keys sort! !

!DependencyBrowser methodsFor: 'package deps' stamp: 'ar 3/2/2010 22:09'!
packageDepsIndex
	"Current package dependencies selection"
	^packageDepsIndex ifNil:[0]! !

!DependencyBrowser methodsFor: 'package deps' stamp: 'ar 3/2/2010 22:09'!
packageDepsIndex: aNumber
	"Current package dependencies selection"
	packageDepsIndex := aNumber.
	self classDepsIndex: 0.
	self changed: #packageDepsIndex.
	self changed: #classDeps.
! !

!DependencyBrowser methodsFor: 'package deps' stamp: 'nice 1/24/2011 23:26'!
packageDepsMenu: aMenu
	^aMenu! !

!DependencyBrowser methodsFor: 'package deps' stamp: 'ar 3/2/2010 22:09'!
packageDepsSelection
	"Current package dependencies selection"
	^(self packageDepsIndex between: 1 and: self packageDeps size)
		ifTrue:[self packageDeps at: self packageDepsIndex]! !

!DependencyBrowser methodsFor: 'package list' stamp: 'ul 3/8/2010 01:41'!
packageList
	"The base list of packages in the system"
	^packageList ifNil:[packageList := (PackageInfo allPackages collect: [ :each | each packageName]) sort]! !

!DependencyBrowser methodsFor: 'package list' stamp: 'ar 3/2/2010 22:08'!
packageListIndex
	"Current package list selection"
	^packageListIndex ifNil:[0]! !

!DependencyBrowser methodsFor: 'package list' stamp: 'ar 3/2/2010 22:08'!
packageListIndex: aNumber
	"Current package list selection"
	packageListIndex := aNumber.
	self changed: #packageListIndex.
	self packageDepsIndex: 0.
	packageDeps := nil.
	self changed: #packageDeps.
! !

!DependencyBrowser methodsFor: 'package list' stamp: 'fbs 12/12/2012 15:43'!
packageListKey: aChar from: view
	aChar == $f ifTrue: [^ self findClass].
	^ self classListKey: aChar from: view! !

!DependencyBrowser methodsFor: 'package list' stamp: 'nice 1/24/2011 23:26'!
packageListMenu: aMenu
	^aMenu! !

!DependencyBrowser methodsFor: 'package list' stamp: 'ar 3/2/2010 22:09'!
packageListSelection
	"Current package list selection"
	^(self packageListIndex between: 1 and: self packageList size)
		ifTrue:[self packageList at: self packageListIndex]! !

!DependencyBrowser methodsFor: 'accessing' stamp: 'fbs 7/24/2013 11:28'!
referencesToIt: aClassName 
	| binding environment |
	environment := self selectedEnvironment.
	binding := (environment bindingOf: aClassName) ifNil: [ ^ self ].
	self systemNavigation browseAllCallsOn: binding! !

!DependencyBrowser methodsFor: 'morphic ui' stamp: 'cmm 7/28/2015 15:26'!
representsSameBrowseeAs: anotherModel 
	^ self hasUnacceptedEdits not! !

!DependencyBrowser methodsFor: 'class list' stamp: 'yo 5/11/2012 11:38'!
selectedClass
	"Answer the class that is currently selected. Answer nil if no selection 
	exists."
	
	| name envt nonMetaClass nonMetaName |
	(name := self selectedClassName) ifNil: [^ nil].
	(envt := self selectedEnvironment) ifNil: [^ nil].
	nonMetaName := (name endsWith: ' class') ifTrue: [name allButLast: 6] ifFalse: [name].
	nonMetaClass := envt at: nonMetaName asSymbol ifAbsent: [^ nil].
	^ nonMetaName = name ifTrue: [nonMetaClass] ifFalse: [nonMetaClass class].
! !

!DependencyBrowser methodsFor: 'class list' stamp: 'fbs 5/5/2011 17:45'!
selectedClassName
	| idx |
	idx := classListIndex ifNil: [0].
	^ self classList ifNotNil: [ :l | l at: idx ifAbsent: [nil]]! !

!DependencyBrowser methodsFor: 'class list' stamp: 'fbs 5/6/2011 07:34'!
selectedClassOrMetaClass
	"Answer the class or metaclass that is currently selected. Answer nil if no selection 
	exists."
	classList ifNil: [^nil].
	^ classList at: classListIndex ifAbsent: [nil].! !

!DependencyBrowser methodsFor: 'accessing' stamp: 'fbs 5/5/2011 17:47'!
selectedEnvironment
	"Answer the name of the selected package or nil."

	self hasPackageSelected ifFalse: [^nil].
	^ Smalltalk! !

!DependencyBrowser methodsFor: 'contents' stamp: 'laza 5/12/2011 14:10'!
selectedMessage
	"Source code for currently selected message"
	| className methodName mref |
	className := self classListSelection.
	methodName := self messageListSelection.
	mref := (classDeps at: self classDepsSelection ifAbsent:[#()])
		detect:[:mr| mr actualClass name = className 
						and:[mr methodSymbol = methodName]]
		ifNone:[nil].
	mref ifNil:[^self class comment].
	mref methodSymbol == #Definition ifTrue:[^mref actualClass definition].
	^mref sourceCode! !

!DependencyBrowser methodsFor: 'message list' stamp: 'fbs 5/5/2011 17:59'!
selectedMessageName
	^ self messageList at: messageListIndex ifAbsent: [nil]! !

CodeHolder subclass: #MessageSet
	instanceVariableNames: 'growable messageList autoSelectString messageListIndex editSelection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Browser'!

!MessageSet commentStamp: '<historical>' prior: 0!
I represent a query path of the retrieval result of making a query about methods in the system. The result is a set of methods, denoted by a message selector and the class in which the method was found. As a StringHolder, the string I represent is the source code of the currently selected method. I am typically viewed in a Message Set Browser consisting of a MessageListView and a BrowserCodeView.!

MessageSet subclass: #ChangedMessageSet
	instanceVariableNames: 'changeSet'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Browser'!

!ChangedMessageSet commentStamp: '<historical>' prior: 0!
A ChangedMessageSet is a message set associated with a change-set; it bears an entry for every method added or changed in the change set, as well as for every class-comment of which the change-set bears a note.!

!ChangedMessageSet class methodsFor: 'as yet unclassified' stamp: 'RAA 5/29/2001 10:19'!
openFor: aChangeSet
	"Open up a ChangedMessageSet browser on the given change set; this is a conventional message-list browser whose message-list consists of all the methods in aChangeSet.  After any method submission, the message list is refigured, making it plausibly dynamic"

	| messageSet |

	messageSet := aChangeSet changedMessageListAugmented select: [ :each | each isValid].
	self 
		openMessageList: messageSet 
		name: 'Methods in Change Set ', aChangeSet name
		autoSelect: nil
		changeSet: aChangeSet! !

!ChangedMessageSet class methodsFor: 'as yet unclassified' stamp: 'ar 8/7/2009 21:42'!
openMessageList: messageList name: labelString autoSelect: autoSelectString changeSet: aChangeSet
	| messageSet |
	messageSet := self messageList: messageList.
	messageSet changeSet: aChangeSet.
	messageSet autoSelectString: autoSelectString.
	ToolBuilder open: messageSet label: labelString.! !

!ChangedMessageSet methodsFor: 'initialization' stamp: 'sd 11/20/2005 21:26'!
changeSet: aChangeSet
	changeSet := aChangeSet! !

!ChangedMessageSet methodsFor: 'acceptance' stamp: 'nice 2/23/2011 21:46'!
contents: aString notifying: aController
	"Accept the string as new source for the current method, and make certain the annotation pane gets invalidated"

	| existingSelector existingClass superResult newSelector |
	existingSelector := self selectedMessageName.
	existingClass := self selectedClassOrMetaClass.

	superResult := super contents: aString notifying: aController.
	superResult ifTrue:  "succeeded"
		[newSelector := existingClass newParser parseSelector: aString.
		newSelector ~= existingSelector
			ifTrue:   "Selector changed -- maybe an addition"
				[self reformulateList.
				self changed: #messageList.
				self messageList doWithIndex:
					[:aMethodReference :anIndex |
						(aMethodReference actualClass == existingClass and:
									[aMethodReference methodSymbol == newSelector])
							ifTrue:
								[self messageListIndex: anIndex]]]].
	^ superResult! !

!ChangedMessageSet methodsFor: 'message list' stamp: 'sw 1/28/2001 20:59'!
growable
	"Answer whether the receiver can be changed by manual additions & deletions"

	^ false! !

!ChangedMessageSet methodsFor: 'reformulation' stamp: 'sw 6/26/2001 11:20'!
reformulateList
	"Reformulate the message list of the receiver"

	self initializeMessageList: (changeSet changedMessageListAugmented select: 
		[:each | each isValid])
! !

MessageSet subclass: #MessageNames
	instanceVariableNames: 'searchString selectorList selectorListIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Browser'!

!MessageNames class methodsFor: 'instance creation' stamp: 'ar 8/7/2009 22:09'!
methodBrowserSearchingFor: searchString
	"Answer an method-browser window whose search-string is initially as indicated"

	| aWindow |
	aWindow := self new searchString: searchString.
	^ToolBuilder default build: aWindow! !

!MessageNames class methodsFor: 'instance creation' stamp: 'fbs 3/23/2011 21:40'!
openMessageNames
	"Open a new instance of the receiver in the active world"
	^(ToolBuilder open: self new label: 'Message Names') model

	"MessageNames openMessageNames"
! !

!MessageNames class methodsFor: 'instance creation' stamp: 'mt 5/3/2015 15:23'!
prototypicalToolWindow
	"Answer an example of myself seen in a tool window, for the benefit of parts-launching tools"

	^ self methodBrowserSearchingFor: ''! !

!MessageNames methodsFor: 'toolbuilder' stamp: 'mt 5/12/2015 21:03'!
buildSearchPaneWith: builder
	
	| panelSpec textSpec buttonSpec |
	panelSpec := builder pluggablePanelSpec new
		layout: #horizontal;
		children: OrderedCollection new;
		yourself.

	textSpec := builder pluggableInputFieldSpec new.
	textSpec 
		model: searchString;
		help: 'Type here, then hit Search.' translated;
		getText: #contents; 
		setText: #contents:;
		softLineWrap: true.
	panelSpec children add: textSpec.
		
	buttonSpec := builder pluggableActionButtonSpec new.
	buttonSpec 
		model: self;
		label: 'Search';
		action: #doSearch;
		horizontalResizing: #shrinkWrap.
	panelSpec children add: buttonSpec.
			
	^ panelSpec! !

!MessageNames methodsFor: 'toolbuilder' stamp: 'ar 8/8/2009 11:40'!
buildSelectorListWith: builder
	| listSpec |
	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #selectorList; 
		getIndex: #selectorListIndex; 
		setIndex: #selectorListIndex:; 
		menu: #selectorListMenu:; 
		keyPress: #selectorListKey:from:.
	^listSpec
! !

!MessageNames methodsFor: 'toolbuilder' stamp: 'mt 5/8/2015 10:59'!
buildWith: builder
	"ToolBuilder open: MessageNames new"

	| windowSpec max searchHeight |
	max := self wantsOptionalButtons ifTrue:[0.42] ifFalse:[0.5].
	searchHeight := Preferences standardDefaultTextFont height * 2.
	
	windowSpec := self buildWindowWith: builder specs: {
		(0@0 corner: 0.5@0.1) -> [self buildSearchPaneWith: builder].
		(0@0.1 corner: 0.5 @ max) -> [self buildSelectorListWith: builder].
		(0.5@0.0 corner: 1.0@max) -> [self buildMessageListWith: builder].
		(0@max corner: 1@1) -> [self buildCodePaneWith: builder].
	}.

	^ builder build: windowSpec! !

!MessageNames methodsFor: 'search' stamp: 'mt 5/3/2015 09:44'!
computeMessageList

	^ selectorListIndex = 0
		ifTrue: [#()]
		ifFalse: [self systemNavigation
			allImplementorsOf: (selectorList at: selectorListIndex)]! !

!MessageNames methodsFor: 'search' stamp: 'mt 5/3/2015 09:25'!
computeSelectorListFrom: searchString
	"Compute selector list from search string. The searchString is a list of expressions separated by ;. Each expression is matched individually. An expression can be a simple string (same as *expression*), a string with double quotes (exact match) or a match expression (see String >> #match:)."
	
	^ (Array streamContents: [ :stream |
			(searchString findBetweenSubStrs: ';') do: [ :selPat |
				(selPat first = $" and: [ selPat last = $" and: [ selPat size > 2 ] ])
					ifTrue: [
						Symbol 
							hasInterned: (selPat copyFrom: 2 to: selPat size - 1)
							ifTrue: [ :sym | stream nextPut: sym ] ]
					ifFalse: [
						| raw n m |
						n := selPat occurrencesOf: $*.
						m := selPat occurrencesOf:  $#.
						raw := ((n > 0 or: [ m > 0 ]) 	and: [ selPat size > (n + m) ])
							ifTrue: [ Symbol selectorsMatching: selPat ]
							ifFalse: [ Symbol selectorsContaining: selPat ].
						stream nextPutAll: raw ] ] ]) 
			sort: [ :x :y | x caseInsensitiveLessOrEqual: y ]! !

!MessageNames methodsFor: 'message list menu' stamp: 'sw 8/15/2002 17:24'!
copyName
	"Copy the current selector to the clipboard"

	| selector |
	(selector := self selectorList at: selectorListIndex ifAbsent: [nil]) ifNotNil:
		[Clipboard clipboardText: selector asString asText]! !

!MessageNames methodsFor: 'user interface' stamp: 'mt 7/9/2016 08:52'!
defaultWindowColor
	^ (Color r: 0.639 g: 0.9 b: 0.497)! !

!MessageNames methodsFor: 'search' stamp: 'mt 5/3/2015 10:42'!
doSearch
	"The user hit the Search button -- treat it as a synonym for the user having hit the Return or Enter (or cmd-s) in the type-in pane"

	searchString changed: #acceptChanges.! !

!MessageNames methodsFor: 'search' stamp: 'mt 5/3/2015 11:02'!
doSearch: aSearchString
	
	| normalizedSearchString |
	normalizedSearchString := aSearchString asString copyWithout: Character space.
		
	Cursor wait showWhile: [
		self selectorList: (self computeSelectorListFrom: normalizedSearchString)].
	
	^ true! !

!MessageNames methodsFor: 'toolbuilder' stamp: 'mt 5/3/2015 10:06'!
frameOffsetFromTop: height fromLeft: leftFraction width: rightFraction bottomFraction: bottomFraction
	^LayoutFrame new
		topFraction: 0 offset: height;
		leftFraction: leftFraction offset: 0;
		rightFraction: (leftFraction + rightFraction) offset: 0;
		bottomFraction: bottomFraction offset: 0;
		yourself.! !

!MessageNames methodsFor: 'initialization' stamp: 'mt 5/8/2015 11:26'!
initialize

	super initialize.
	
	searchString := ValueHolder new contents: ''.
	searchString addDependent: self.
	
	selectorList := #().
	selectorListIndex := 0.
	
	self messageListIndex: 0.! !

!MessageNames methodsFor: 'initialization' stamp: 'mt 5/3/2015 11:02'!
labelString

	^ self searchString
		ifEmpty: ['Message Names']
		ifNotEmpty: [:s | 'Message names containing "', s asString asLowercase, '"']! !

!MessageNames methodsFor: 'selector list' stamp: 'topa 3/16/2015 16:40'!
mainSelectorListMenu: aMenu
	"Answer the menu associated with the selectorList"
	<selectorListMenu>
	aMenu addList: #(
		('senders (n)'				browseSenders		'browse senders of the chosen selector')
		('copy selector to clipboard'	copyName			'copy the chosen selector to the clipboard, for subsequent pasting elsewhere')
		-
		('show only implemented selectors'	showOnlyImplementedSelectors		'remove from the selector-list all symbols that do not represent implemented methods')).

	^ aMenu! !

!MessageNames methodsFor: 'message list' stamp: 'mt 5/3/2015 09:54'!
messageList: someObjects

	messageList := someObjects.
	self changed: #messageList.
	
	self messageListIndex: (messageList size > 0
			ifTrue: [1]
			ifFalse: [0]).! !

!MessageNames methodsFor: 'search' stamp: 'mt 5/3/2015 10:39'!
searchString

	^ searchString contents! !

!MessageNames methodsFor: 'search' stamp: 'mt 5/3/2015 10:50'!
searchString: aString
	
	searchString contents: aString.! !

!MessageNames methodsFor: 'class list' stamp: 'fbs 3/24/2011 17:54'!
selectedClassOrMetaClass
	"Answer the currently selected class (or metaclass)."
	self hasMessageSelected ifTrue:
		[ ^ self setClassAndSelectorIn: [:c :s | ^c] ].
	
	(selectorListIndex isNil not and: [selectorListIndex > 0]) ifTrue: [^Smalltalk classNamed: (self selectorList at: selectorListIndex)].
	
	^ nil.
	! !

!MessageNames methodsFor: 'message list' stamp: 'eem 8/8/2014 07:06'!
selectedMessageName
	selectorList basicSize = 0 ifTrue: [^ nil]. "Deals with selectorList nil or empty"
	^selectorList at: (selectorListIndex max: 1) ifAbsent: [nil] "If no selection we can still find a selector"! !

!MessageNames methodsFor: 'selector list' stamp: 'mt 5/3/2015 09:57'!
selectorList

	^ selectorList! !

!MessageNames methodsFor: 'toolbuilder' stamp: 'fbs 4/22/2010 21:44'!
selectorListFrame: max fromTop: topOffset
	^LayoutFrame new
		leftFraction: 0 offset: 0;
		topFraction: 0 offset: topOffset;
		rightFraction: 0.5 offset: 0;
		bottomFraction: max offset: 0.! !

!MessageNames methodsFor: 'selector list' stamp: 'sw 7/24/2001 01:55'!
selectorListIndex
	"Answer the selectorListIndex"

	^ selectorListIndex! !

!MessageNames methodsFor: 'selector list' stamp: 'eem 12/6/2015 12:19'!
selectorListIndex: anInteger 
	"Set the selectorListIndex as specified, and propagate consequences"
	| methodClass index |
	methodClass := currentCompiledMethod ifNotNil: [currentCompiledMethod methodClass].
	selectorListIndex := anInteger.
	self changed: #selectorListIndex.

	messageList := self computeMessageList.
	self changed: #messageList.
	messageList size > 1 ifTrue:
		[methodClass ifNotNil: [index := messageList findFirst: [:methodRef| methodRef actualClass = methodClass]]].

	"If a method of the same class exists, select that, otherwise select the first message if any."
	self messageListIndex: (index ifNil: [1 min: messageList size])! !

!MessageNames methodsFor: 'selector list' stamp: 'sw 7/24/2001 01:35'!
selectorListKey: aChar from: view
	"Respond to a Command key in the message-list pane."

	aChar == $n ifTrue: [^ self browseSenders].
	aChar == $c ifTrue: [^ self copyName].
	aChar == $b ifTrue: [^ self browseMethodFull].
! !

!MessageNames methodsFor: 'selector list' stamp: 'sw 7/24/2001 01:47'!
selectorListMenuTitle
	"Answer the title to supply for the menu belonging to the selector-list pane"

	^ 'Click on any item in the list
to see all implementors of it'! !

!MessageNames methodsFor: 'selector list' stamp: 'topa 3/16/2015 16:40'!
selectorListMenu: aMenu
	"Answer the menu associated with the selectorList"
	^ self menu: aMenu for: #(selectorListMenu selectorListMenuShifted:)
! !

!MessageNames methodsFor: 'selector list' stamp: 'mt 5/3/2015 09:55'!
selectorList: someObjects
	"Answer the selectorList"

	selectorList := someObjects.
	self changed: #selectorList.
	
	"Select first result if any."
	self selectorListIndex: (selectorList size > 0
			ifTrue: [1]
			ifFalse: [0]).! !

!MessageNames methodsFor: 'search' stamp: 'mt 5/3/2015 10:00'!
showOnlyImplementedSelectors
	"Caution -- can be slow!! Filter my selector list down such that it only  
	shows selectors that are actually implemented somewhere in the system."
	self okToChange ifFalse: [^ self].
	
	Cursor wait showWhile: [
		self selectorList: (self systemNavigation allSelectorsWithAnyImplementorsIn: selectorList)].! !

!MessageNames methodsFor: 'toolbuilder' stamp: 'fbs 6/7/2011 17:40'!
topConstantHeightFrame: height fromLeft: leftFraction width: rightFraction
	^LayoutFrame new
		topFraction: 0 offset: 0;
		leftFraction: leftFraction offset: 0;
		rightFraction: (leftFraction + rightFraction) offset: 0;
		bottomFraction: 0 offset: height;
		yourself.! !

!MessageNames methodsFor: 'updating' stamp: 'mt 5/3/2015 11:03'!
update: aspect

	aspect == #contents
		ifTrue: [
			self changed: #labelString.
			self doSearch: self searchString].
		
	super update: aspect.! !

!MessageSet class methodsFor: 'utilities' stamp: 'RAA 5/29/2001 10:19'!
extantMethodsIn: aListOfMethodRefs
	"Answer the subset of the incoming list consisting only of those message markers that refer to methods actually in the current image"


	self flag: #mref.	"may be removed in second round"


	^ aListOfMethodRefs select: [:aToken |
		self 
			parse: aToken 
			toClassAndSelector: [ :aClass :aSelector |
				aClass notNil and: [aClass includesSelector: aSelector]
			]
	]! !

!MessageSet class methodsFor: 'utilities' stamp: 'sw 6/6/2001 15:09'!
isPseudoSelector: aSelector
	"Answer whether the given selector is a special marker"

	^ #(Comment Definition Hierarchy) includes: aSelector! !

!MessageSet class methodsFor: 'instance creation' stamp: ''!
messageList: anArray 
	"Answer an instance of me with message list anArray."

	^self new initializeMessageList: anArray! !

!MessageSet class methodsFor: 'instance creation' stamp: ''!
openMessageList: anArray name: aString 
	"Create a standard system view for the message set on the list, anArray. 
	The label of the view is aString."

	self open: (self messageList: anArray) name: aString! !

!MessageSet class methodsFor: 'instance creation' stamp: 'ar 8/7/2009 21:43'!
openMessageList: messageList name: labelString autoSelect: autoSelectString
	"Open a system view for a MessageSet on messageList. 
	 1/24/96 sw: the there-are-no msg now supplied by my sender"

	| messageSet |
	messageSet := self messageList: messageList.
	messageSet autoSelectString: autoSelectString.
	^ToolBuilder open: messageSet label: labelString! !

!MessageSet class methodsFor: 'instance creation' stamp: 'ar 8/7/2009 21:42'!
open: aMessageSet name: aString 
	"Create a standard system view for the messageSet, aMessageSet, whose label is aString."
	^ToolBuilder open: aMessageSet label: aString! !

!MessageSet class methodsFor: 'utilities' stamp: 'ul 8/14/2015 18:02'!
parse: methodRef toClassAndSelector: csBlock
	"Decode strings of the form <className> [class] <selectorName>."

	| tuple cl |


	self flag: #mref.	"compatibility with pre-MethodReference lists"

	methodRef ifNil: [^ csBlock value: nil value: nil].
	methodRef isString ifFalse:
		[^methodRef setClassAndSelectorIn: csBlock].
	methodRef isEmpty ifTrue:
		[^csBlock value: nil value: nil].
	tuple := (methodRef asString includesSubstring: '>>')
				ifTrue: [(methodRef findTokens: '>>') fold: [:a :b| (a findTokens: ' '), {b first = $# ifTrue: [b allButFirst] ifFalse: [b]}]]
				ifFalse: [methodRef asString findTokens: ' .'].
	cl := Smalltalk at: tuple first asSymbol ifAbsent: [^ csBlock value: nil value: nil].
	^(tuple size = 2 or: [tuple size > 2 and: [(tuple at: 2) ~= 'class']])
		ifTrue: [csBlock value: cl value: (tuple at: 2) asSymbol]
		ifFalse: [csBlock value: cl class value: (tuple at: 3) asSymbol]! !

!MessageSet methodsFor: 'code pane' stamp: 'eem 9/29/2010 16:23'!
aboutToStyle: aPluggableShoutMorphOrView
	"This is a notification that aPluggableShoutMorphOrView is about to re-style its text.
	Set the classOrMetaClass in aPluggableShoutMorphOrView, so that identifiers
	will be resolved correctly.
	Answer true to allow styling to proceed, or false to veto the styling"

	| selectedMessageName showingMethod |
	self showingSource ifFalse: [^false].
	selectedMessageName := self selectedMessageName.
	showingMethod := (#(Comment Definition Hierarchy) includes: selectedMessageName) not.
	"Hack!!  setting classOrMetaClass: to nil allows doit or class definition colouring."
	aPluggableShoutMorphOrView classOrMetaClass: (showingMethod ifTrue: [self selectedClassOrMetaClass]).
	^(#(Comment Hierarchy) includes: selectedMessageName) not! !

!MessageSet methodsFor: 'message list' stamp: 'cmm 4/7/2016 17:54'!
addExtraShiftedItemsTo: aMenu
	"The shifted selector-list menu is being built.  Add items specific to MessageSet"
	self growable ifTrue:
		[aMenu addList: #(
			-
			('remove from this browser (d)'		removeMessageFromBrowser)
			('filter message list...'			filterMessageList))].
	aMenu 
		add: 'sort by date'
		action: #sortByDate! !

!MessageSet methodsFor: 'message list' stamp: 'tk 5/1/2001 18:14'!
addItem: classAndMethod
	"Append a classAndMethod string to the list.  Select the new item."

	"Do some checks on the input?"
	self okToChange ifFalse: [^ self].
	messageList add: classAndMethod.
	self changed: #messageList.
	self messageListIndex: messageList size.! !

!MessageSet methodsFor: 'private' stamp: 'sw 1/11/2001 09:16'!
adjustWindowTitleAfterFiltering
	"Set the title of the receiver's window, if any, to reflect the just-completed filtering"

	| aWindow existingLabel newLabel |

	(aWindow := self containingWindow) ifNil: [^ self].
	(existingLabel := aWindow label) isEmptyOrNil ifTrue: [^ self].
	(((existingLabel size < 3) or: [existingLabel last ~~ $]]) or: [(existingLabel at: (existingLabel size - 1)) isDigit not]) ifTrue: [^ self].
	existingLabel size to: 1 by: -1 do:
		[:anIndex | ((existingLabel at: anIndex) == $[) ifTrue:
			[newLabel := (existingLabel copyFrom: 1 to: anIndex),
				'Filtered: ',
				messageList size printString,
				']'.
			^ aWindow setLabel: newLabel]]
			

! !

!MessageSet methodsFor: 'private' stamp: ''!
autoSelectString
	"Return the string to be highlighted when making new selections"
	^ autoSelectString! !

!MessageSet methodsFor: 'private' stamp: 'sd 11/20/2005 21:27'!
autoSelectString: aString
	"Set the string to be highlighted when making new selections"
	autoSelectString := aString! !

!MessageSet methodsFor: 'message functions' stamp: 'cmm 2/18/2010 12:27'!
browseAllImplementorsOf: selectorSymbol
	self systemNavigation browseAllImplementorsOf: selectorSymbol! !

!MessageSet methodsFor: 'toolbuilder' stamp: 'mt 11/6/2015 11:35'!
buildMessageListWith: builder
	| listSpec |
	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #messageList; 
		getIndex: #messageListIndex; 
		setIndex: #messageListIndex:;
		icon: #messageIconAt:;
		help: #messageHelpAt:; 
		menu: #messageListMenu:shifted:; 
		keyPress: #messageListKey:from:.
	SystemBrowser browseWithDragNDrop 
		ifTrue:[listSpec dragItem: #dragFromMessageList:].
	^listSpec
! !

!MessageSet methodsFor: 'toolbuilder' stamp: 'ar 8/25/2009 09:46'!
buildWith: builder
	| windowSpec max result |
	self wantsOptionalButtons ifTrue:[max := 0.3] ifFalse:[max := 0.3].
	windowSpec := self buildWindowWith: builder specs: {
		(0@0 corner: 1@max) -> [self buildMessageListWith: builder].
		(0@max corner: 1@1) -> [self buildCodePaneWith: builder].
	}.
	result := builder build: windowSpec.
	autoSelectString ifNotNil:[self changed: #autoSelect].
	^result! !

!MessageSet methodsFor: 'message category functions' stamp: 'sw 10/8/2001 14:10'!
canShowMultipleMessageCategories
	"Answer whether the receiver is capable of showing multiple message categories"

	^ false! !

!MessageSet methodsFor: 'metaclass' stamp: 'nk 4/29/2004 12:20'!
classCommentIndicated
	"Answer true iff we're viewing the class comment."

	^ editSelection == #editComment or: [ self selectedMessageName == #Comment ]! !

!MessageSet methodsFor: 'contents' stamp: 'fbs 3/28/2011 22:40'!
contents
	"Answer the contents of the receiver"

	^ contents == nil
		ifTrue: [currentCompiledMethod := nil. '']
		ifFalse: [self hasMessageSelected
			ifTrue: [self editContents]
			ifFalse: [currentCompiledMethod := nil. contents]]! !

!MessageSet methodsFor: 'private' stamp: 'sw 7/31/2002 12:58'!
contents: aString notifying: aController 
	"Compile the code in aString. Notify aController of any syntax errors. 
	Answer false if the compilation fails. Otherwise, if the compilation 
	created a new method, deselect the current selection. Then answer true."

	| category selector class oldSelector |
	self okayToAccept ifFalse: [^ false].
	self setClassAndSelectorIn: [:c :os | class := c.  oldSelector := os].
	class ifNil: [^ false].
	(oldSelector ~~ nil and: [oldSelector first isUppercase]) ifTrue:
		[oldSelector = #Comment ifTrue:
			[class comment: aString stamp: Utilities changeStamp.
			self changed: #annotation.
 			self clearUserEditFlag.
			^ false].
		oldSelector = #Definition ifTrue:
			["self defineClass: aString notifying: aController."
			class subclassDefinerClass
				evaluate: aString
				notifying: aController
				logged: true.
			self clearUserEditFlag.
 			^ false].
		oldSelector = #Hierarchy ifTrue:
			[self inform: 'To change the hierarchy, edit the class definitions'. 
			^ false]].
	"Normal method accept"
	category := class organization categoryOfElement: oldSelector.
	selector := class compile: aString
				classified: category
				notifying: aController.
	selector == nil ifTrue: [^ false].
	self noteAcceptanceOfCodeFor: selector.
	selector == oldSelector ifFalse:
		[self reformulateListNoting: selector].
	contents := aString copy.
	self changed: #annotation.
	^ true! !

!MessageSet methodsFor: 'user interface' stamp: 'mt 7/9/2016 08:52'!
defaultWindowColor
	^ (Color r: 0.719 g: 0.9 b: 0.9)! !

!MessageSet methodsFor: 'message functions' stamp: 'fbs 7/19/2012 14:33'!
deleteFromMessageList: aMessage
	"Delete the given message from the receiver's message list"
	| currIdx |
	currIdx := self messageListIndex.
	messageList := messageList copyWithout: aMessage.
	messageList ifNotEmpty: [self messageListIndex: {currIdx. messageList size.} min]! !

!MessageSet methodsFor: 'private' stamp: 'cmm 10/16/2015 14:30'!
deselectAll
	self messageListIndex: 0! !

!MessageSet methodsFor: 'accessing' stamp: 'bf 7/11/2011 11:40'!
doItReceiver
	^ self selectedClass! !

!MessageSet methodsFor: 'drag and drop' stamp: 'fbs 6/8/2011 12:03'!
dragFromMessageList: index
	"Drag a method from the browser"
	^self selectedClassOrMetaClass compiledMethodAt: (self messageList at: index) ifAbsent:[nil]! !

!MessageSet methodsFor: 'drag and drop' stamp: 'nk 6/13/2004 07:32'!
dragPassengerFor: item inMorph: dragSource 
	| transferType |
	transferType := self dragTransferTypeForMorph: dragSource.
	transferType == #messageList
		ifTrue: [^self selectedClassOrMetaClass->(item contents findTokens: ' ') second asSymbol].
	transferType == #classList
		ifTrue: [^self selectedClass].
	^nil! !

!MessageSet methodsFor: 'accessing' stamp: 'fbs 6/7/2011 17:45'!
editSelection: aSelection
	"Set the editSelection as requested."

	editSelection := aSelection.
	self changed: #editSelection.! !

!MessageSet methodsFor: 'message functions' stamp: 'eem 11/12/2012 09:41'!
fileOutMessage
	"Put a description of the selected method on a file, or all methods if none selected."

	| fileName |
	self selectedMessageName ifNotNil:
		[^super fileOutMessage].
	fileName := UIManager default request: 'File out on which file?' initialAnswer: 'methods'.
	Cursor write showWhile:
		[| internalStream |
		internalStream := WriteStream on: (String new: 1000).
		internalStream header; timeStamp.
		messageList do:
			[:methodRef|
			methodRef methodSymbol == #Comment
				ifTrue:
					[methodRef actualClass organization
						putCommentOnFile: internalStream
						numbered: 1
						moveSource: false
						forClass: methodRef actualClass]
				ifFalse:
					[methodRef actualClass
						printMethodChunk: methodRef methodSymbol
						withPreamble: true
						on: internalStream
						moveSource: false
						toFile: nil]].
		FileStream writeSourceCodeFrom: internalStream baseName: fileName isSt: true useHtml: false]! !

!MessageSet methodsFor: 'filtering' stamp: 'nk 9/7/2004 11:54'!
filterFrom: aBlock
	"Filter the receiver's list down to only those items that satisfy aBlock, which takes a class an a selector as its arguments."

	| newList |
	newList := messageList select:
		[:anElement |
			self class parse: anElement toClassAndSelector: [ :cls :sel | 
				(self class isPseudoSelector: sel) not and: [  aBlock value: cls value: sel ]]].
	self setFilteredList: newList! !

!MessageSet methodsFor: 'filtering' stamp: 'fbs 6/18/2013 17:23'!
filterMessageList
	"Allow the user to refine the list of messages."
	| builder menuSpec |
	builder := ToolBuilder default.
	menuSpec := builder pluggableMenuSpec new
		model: self;
		yourself.
	menuSpec addList:
		#(
		('unsent messages' filterToUnsentMessages 'filter to show only messages that have no senders')
		-
		('messages that send...' filterToSendersOf 'filter to show only messages that send a selector I specify')
		('messages that do not send...' filterToNotSendersOf 'filter to show only messages that do not send a selector I specify')
		-
		('messages whose selector is...' filterToImplementorsOf 'filter to show only messages with a given selector I specify')
		('messages whose selector is NOT...' filterToNotImplementorsOf 'filter to show only messages whose selector is NOT a seletor I specify')
		-
		('messages in current change set' filterToCurrentChangeSet 'filter to show only messages that are in the current change set')
		('messages not in current change set' filterToNotCurrentChangeSet 'filter to show only messages that are not in the current change set')
		-
		('messages in any change set' filterToAnyChangeSet 'filter to show only messages that occur in at least one change set')
		('messages not in any change set' filterToNotAnyChangeSet 'filter to show only messages that do not occur in any change set in the system')
		-
		('messages authored by me' filterToCurrentAuthor 'filter to show only messages whose authoring stamp has my initials')
		('messages not authored by me' filterToNotCurrentAuthor 'filter to show only messages whose authoring stamp does not have my initials')
		-
		('messages logged in .changes file' filterToMessagesInChangesFile 'filter to show only messages whose latest source code is logged in the .changes file')
		('messages only in .sources file' filterToMessagesInSourcesFile 'filter to show only messages whose latest source code is logged in the .sources file')
		-
		('messages with prior versions'	 filterToMessagesWithPriorVersions 'filter to show only messages that have at least one prior version')
		('messages without prior versions' filterToMessagesWithoutPriorVersions 'filter to show only messages that have no prior versions')
		-
		('uncommented messages' filterToUncommentedMethods 'filter to show only messages that do not have comments at the beginning')
		('commented messages' filterToCommentedMethods 'filter to show only messages that have comments at the beginning')
		-
		('messages in hardened classes' filterToMessagesWithHardenedClasses 'filter to show only messages of established classes (as opposed to Uniclasses such as Player23)')		-
		('methods in classes with matching names' filterToMatchingClassesNames 'filter to show only methods of classes with names that match the given criteria (wildcards are allowed)')
		('methods in package...' filterToPackage 'filter to show only methods of a given package')
		-
		('messages that...' filterToMessagesThat 'let me type in a block taking a class and a selector, which will specify yea or nay concerning which elements should remain in the list')).
	builder runModal: (builder open: menuSpec).! !

!MessageSet methodsFor: 'filtering' stamp: 'pk 10/17/2006 09:42'!
filterToAnyChangeSet
	"Filter down only to messages present in ANY change set"

	self filterFrom:
		[:aClass :aSelector |
			ChangesOrganizer doesAnyChangeSetHaveClass: aClass andSelector: aSelector]
! !

!MessageSet methodsFor: 'filtering' stamp: 'nice 10/20/2009 20:58'!
filterToCommentedMethods
	"Filter the receiver's list down to only those items which have comments"

	self filterFrom:
		[:aClass :aSelector |
			(aClass includesSelector: aSelector) and:
						[(aClass firstPrecodeCommentFor: aSelector) isEmptyOrNil not]]! !

!MessageSet methodsFor: 'filtering' stamp: 'fbs 3/4/2013 20:51'!
filterToCurrentAuthor
	"Filter down only to messages with my initials as most recent author"
	self filterFrom:
		[:aClass :aSelector | | aMethod aTimeStamp |
			(aClass notNil and: [aSelector notNil]) and:			
				[aMethod := aClass compiledMethodAt: aSelector ifAbsent: [nil].
				aMethod notNil and:
					[(aTimeStamp := aMethod timeStamp) notNil and:
						[aTimeStamp beginsWith: Utilities authorInitials]]]]! !

!MessageSet methodsFor: 'filtering' stamp: 'sd 5/23/2003 14:38'!
filterToCurrentChangeSet
	"Filter the receiver's list down to only those items in the current change set"

	self filterFrom:
		[:aClass :aSelector |
			(aClass notNil and: [aSelector notNil]) and:
				[(ChangeSet current atSelector: aSelector class: aClass) ~~ #none]]! !

!MessageSet methodsFor: 'filtering' stamp: 'fbs 6/8/2011 12:18'!
filterToImplementorsOf
	"Filter the receiver's list down to only those items with a given selector"

	| aFragment inputWithBlanksTrimmed |

	aFragment := self request: 'type selector:' initialAnswer: ''.
	aFragment  isEmptyOrNil ifTrue: [^ self].
	inputWithBlanksTrimmed := aFragment withBlanksTrimmed.
	Symbol hasInterned: inputWithBlanksTrimmed ifTrue:
		[:aSymbol | 
			self filterFrom:
				[:aClass :aSelector |
					aSelector == aSymbol]]! !

!MessageSet methodsFor: 'filtering' stamp: 'dtl 2/12/2010 19:55'!
filterToMatchingClassesNames

	| reply |

	reply := UIManager default
					request: 'Type the string to match'
					initialAnswer: 'ClassName*'
					centerAt: Sensor cursorPoint.
	reply isEmptyOrNil ifTrue: [^ self].

	self filterFrom: [:aClass :aSelector | reply match: aClass name asString]
! !

!MessageSet methodsFor: 'filtering' stamp: 'ul 11/15/2010 11:54'!
filterToMessagesInChangesFile
	"Filter down only to messages whose source code risides in the Changes file.  This allows one to ignore long-standing methods that live in the .sources file."

	
	self filterFrom:
		[:aClass :aSelector | | cm |
			aClass notNil and: [aSelector notNil and:
				[(self class isPseudoSelector: aSelector) not and:
					[(cm := aClass compiledMethodAt: aSelector ifAbsent: [nil]) notNil and:
					[cm fileIndex ~= 1]]]]]! !

!MessageSet methodsFor: 'filtering' stamp: 'ul 11/15/2010 11:54'!
filterToMessagesInSourcesFile
	"Filter down only to messages whose source code resides in the .sources file."

	
	self filterFrom: [:aClass :aSelector | | cm |
		(aClass notNil and: [aSelector notNil]) and:
			[(self class isPseudoSelector: aSelector) not and:
				[(cm := aClass compiledMethodAt: aSelector ifAbsent: [nil]) notNil and:
					[cm fileIndex = 1]]]]! !

!MessageSet methodsFor: 'filtering' stamp: 'rbb 3/1/2005 11:00'!
filterToMessagesThat
	"Allow the user to type in a block which will be"

	| reply |
	reply := UIManager default
		multiLineRequest: 'Type your block here'
		centerAt: Sensor cursorPoint
		initialAnswer: '[:aClass :aSelector |
	
	]'
		answerHeight: 200.
	reply isEmptyOrNil ifTrue: [^ self].
	self filterFrom: (Compiler evaluate: reply)
! !

!MessageSet methodsFor: 'filtering' stamp: 'sw 2/13/2001 12:02'!
filterToMessagesWithHardenedClasses
	"Filter the receiver's list down to only those items representing methods of hardened classes, as opposed to uniclasses"

	self filterFrom:
		[:aClass :aSelector |
			(aClass notNil and: [aSelector notNil]) and:
				[aClass isUniClass not]]! !

!MessageSet methodsFor: 'filtering' stamp: 'sw 6/6/2001 15:12'!
filterToMessagesWithoutPriorVersions
	"Filter down only to messages which have no prior version stored"

	self filterFrom:
		[:aClass :aSelector |
			(aClass notNil and: [aSelector notNil]) and:
				[(self class isPseudoSelector: aSelector) not and:
					[(VersionsBrowser versionCountForSelector: aSelector class: aClass) <= 1]]]! !

!MessageSet methodsFor: 'filtering' stamp: 'sw 8/12/2001 22:25'!
filterToMessagesWithPriorVersions
	"Filter down only to messages which have at least one prior version"

	self filterFrom:
		[:aClass :aSelector |
			(aClass notNil and: [aSelector notNil]) and:
				[(self class isPseudoSelector: aSelector) not and:
					[(VersionsBrowser versionCountForSelector: aSelector class: aClass) > 1]]]! !

!MessageSet methodsFor: 'filtering' stamp: 'pk 10/17/2006 09:42'!
filterToNotAnyChangeSet
	"Filter down only to messages present in NO change set"

	self filterFrom:
		[:aClass :aSelector |
			(ChangesOrganizer doesAnyChangeSetHaveClass: aClass andSelector: aSelector) not]
! !

!MessageSet methodsFor: 'filtering' stamp: 'fbs 3/4/2013 20:51'!
filterToNotCurrentAuthor
	"Filter down only to messages not stamped with my initials"
	self filterFrom:
		[:aClass :aSelector | | aTimeStamp aMethod |
			(aClass notNil and: [aSelector notNil]) and:			
				[aMethod := aClass compiledMethodAt: aSelector ifAbsent: [nil].
				aMethod notNil and:
					[(aTimeStamp := aMethod timeStamp) isNil or:
						[(aTimeStamp beginsWith: Utilities authorInitials) not]]]]! !

!MessageSet methodsFor: 'filtering' stamp: 'sd 5/23/2003 14:38'!
filterToNotCurrentChangeSet
	"Filter the receiver's list down to only those items not in the current change set"

	self filterFrom:
		[:aClass :aSelector |
			(aClass notNil and: [aSelector notNil]) and:
				[(ChangeSet current atSelector: aSelector class: aClass) == #none]]! !

!MessageSet methodsFor: 'filtering' stamp: 'fbs 6/8/2011 12:18'!
filterToNotImplementorsOf
	"Filter the receiver's list down to only those items whose selector is NOT one solicited from the user."

	| aFragment inputWithBlanksTrimmed |

	aFragment := self request: 'type selector: ' initialAnswer: ''.
	aFragment  isEmptyOrNil ifTrue: [^ self].
	inputWithBlanksTrimmed := aFragment withBlanksTrimmed.
	Symbol hasInterned: inputWithBlanksTrimmed ifTrue:
		[:aSymbol | 
			self filterFrom:
				[:aClass :aSelector |
					aSelector ~~ aSymbol]]! !

!MessageSet methodsFor: 'filtering' stamp: 'fbs 6/8/2011 12:18'!
filterToNotSendersOf
	"Filter the receiver's list down to only those items which do not send a given selector"

	| aFragment inputWithBlanksTrimmed |

	aFragment := self request: 'type selector:' initialAnswer: ''.
	aFragment  isEmptyOrNil ifTrue: [^ self].
	inputWithBlanksTrimmed := aFragment withBlanksTrimmed.
	Symbol hasInterned: inputWithBlanksTrimmed ifTrue:
		[:aSymbol | 
			self filterFrom:
				[:aClass :aSelector | | aMethod |
					(aMethod := aClass compiledMethodAt: aSelector) isNil or:
						[(aMethod hasLiteralThorough: aSymbol) not]]]! !

!MessageSet methodsFor: 'filtering' stamp: 'fbs 3/6/2013 17:17'!
filterToPackage

	| packages selectedIndex selectedPackage |

	packages := (PackageOrganizer default packages sort: [ :a :b |
		a packageName <= b packageName ]).

	selectedIndex := UIManager default chooseFrom: (packages collect:[:each | each packageName]) lines: #() title: 'Select a package...'.
	selectedIndex isZero ifTrue:[^ self].

	selectedPackage := packages at: selectedIndex.
	self filterFrom: [:aClass :aSelector | selectedPackage includesMethod: aSelector ofClass: aClass ]
! !

!MessageSet methodsFor: 'filtering' stamp: 'fbs 6/8/2011 12:18'!
filterToSendersOf
	"Filter the receiver's list down to only those items which send a given selector"

	| aFragment inputWithBlanksTrimmed |

	aFragment := self request: 'type selector:' initialAnswer: ''.
	aFragment  isEmptyOrNil ifTrue: [^ self].
	inputWithBlanksTrimmed := aFragment withBlanksTrimmed.
	Symbol hasInterned: inputWithBlanksTrimmed ifTrue:
		[:aSymbol | 
			self filterFrom:
				[:aClass :aSelector | | aMethod |
					(aMethod := aClass compiledMethodAt: aSelector) notNil and:
						[aMethod hasLiteralThorough: aSymbol]]]

! !

!MessageSet methodsFor: 'filtering' stamp: 'nice 10/20/2009 20:59'!
filterToUncommentedMethods
	"Filter the receiver's list down to only those items which lack comments"

	self filterFrom:
		[:aClass :aSelector |
			(aClass includesSelector: aSelector) and:
						[(aClass firstPrecodeCommentFor: aSelector) isEmptyOrNil]]! !

!MessageSet methodsFor: 'filtering' stamp: 'sd 4/29/2003 12:24'!
filterToUnsentMessages
	"Filter the receiver's list down to only those items which have no  
	senders"
	self
		filterFrom: [:aClass :aSelector | (self systemNavigation allCallsOn: aSelector) isEmpty]! !

!MessageSet methodsFor: 'message list' stamp: 'sw 1/28/2001 20:56'!
growable
	"Answer whether the receiver is capable of growing/shrinking dynamically"

	^ growable ~~ false! !

!MessageSet methodsFor: 'message list' stamp: 'sw 12/6/2000 07:12'!
growable: aBoolean
	"Give or take away the growable trait; when a message set is growable, methods submitted within it will be added to its message list"

	growable := aBoolean! !

!MessageSet methodsFor: 'message list' stamp: 'fbs 3/28/2011 22:31'!
hasMessageSelected
	^ messageListIndex ~= 0.! !

!MessageSet methodsFor: 'initialize-release' stamp: 'fbs 6/19/2011 16:03'!
initialize
	super initialize.
	messageList := OrderedCollection new.! !

!MessageSet methodsFor: 'private' stamp: 'mt 4/6/2016 10:49'!
initializeMessageList: anArray
	"Initialize my messageList from the given list of MethodReference or string objects.  NB: special handling for uniclasses."

	
	messageList := OrderedCollection new.
	anArray do: [ :each |
		MessageSet 
			parse: each  
			toClassAndSelector: [ :class :sel | | s |
				class ifNotNil:
					[class isUniClass
						ifTrue:
							[s := class typicalInstanceName, ' ', sel]
						ifFalse:
							[s := class name , ' ' , sel , ' {' , 
								((class organization categoryOfElement: sel) ifNil: ['']) , '}' ,
								' {', class category, '}'].
					messageList add: (
						MethodReference new
							setClass: class  
							methodSymbol: sel 
							stringVersion: s)]]].
	messageListIndex := messageList isEmpty ifTrue: [0] ifFalse: [1].
	contents := ''! !

!MessageSet methodsFor: 'message list' stamp: 'fbs 3/22/2011 21:29'!
lastMessageName
	^ self messageList last selector.! !

!MessageSet methodsFor: 'message functions' stamp: 'cmm 10/16/2015 14:16'!
mainMessageListMenu: aMenu
	"Answer the message-list menu"
	<messageListMenuShifted: false>
	aMenu addList: #(
			('what to show...'			offerWhatToShowMenu)
			('toggle break on entry'		toggleBreakOnEntry)
			-
			('browse full (b)' 			browseMethodFull)
			('browse hierarchy (h)'			classHierarchy)
			('browse protocol (p)'			browseFullProtocol)
			-
			('fileOut'				fileOutMessage)
			('printOut'				printOutMessage)
			('copy selector (c)'		copySelector)
			('copy reference (C)'	copyReference)
			-
			('senders of... (n)'			browseSendersOfMessages)
			('implementors of... (m)'		browseMessages)
			('inheritance (i)'			methodHierarchy)
			('versions (v)'				browseVersions)
			-
			('references... (r)'			browseVariableReferences)
			('assignments... (a)'			browseVariableAssignments)
			('class refs (N)'			browseClassRefs)
			-
			('remove method (x)'			removeMessage)
			('explore method'			exploreMethod)
			('inspect method'			inspectMethod)).
	^ aMenu! !

!MessageSet methodsFor: 'message list' stamp: 'ul 4/3/2016 03:56'!
messageHelpAt: anIndex
	"Show the first n lines of the sources code of the selected message."
	
	| reference source formatted lineCount |
	Preferences balloonHelpInMessageLists ifFalse: [^ nil].
	self messageList size < anIndex ifTrue: [^ nil].
	
	reference := self messageList at: anIndex.
	reference isValid ifFalse: [^ nil].
	
	source := reference compiledMethod getSource.
	formatted := (Smalltalk classNamed: #SHTextStylerST80)
		ifNil: [ source asText ]
		ifNotNil: [ :SHTextStylerST80 |
			SHTextStylerST80 new
				classOrMetaClass: reference actualClass;
				styledTextFor: source asText ].
	
	lineCount := 0.
	source doWithIndex: [:char :index |
		char = Character cr ifTrue: [lineCount := lineCount + 1].
		lineCount > 10 ifTrue: [
			formatted := formatted copyFrom: 1 to: index-1.
			formatted append: ' [...]'.
			^ formatted]].

	^ formatted! !

!MessageSet methodsFor: 'message list' stamp: 'mt 11/6/2015 11:47'!
messageIconAt: anIndex

	Browser showMessageIcons
		ifFalse: [^ nil].

	^ ToolIcons iconNamed: (ToolIcons
		iconForClass: (self messageList at: anIndex) actualClass
		selector: (self messageList at: anIndex) selector)! !

!MessageSet methodsFor: 'message list' stamp: ''!
messageList
	"Answer the current list of messages."

	^messageList! !

!MessageSet methodsFor: 'message list' stamp: 'fbs 4/11/2011 11:47'!
messageListIndex
	^messageListIndex ifNil: [0]! !

!MessageSet methodsFor: 'message list' stamp: 'fbs 3/22/2011 22:17'!
messageListIndexOf: aString
	^ (self messageList collect: [:each | each selector]) indexOf: aString.! !

!MessageSet methodsFor: 'message list' stamp: 'cmm 2/18/2010 22:48'!
messageListIndex: anInteger 
	"Set the index of the selected item to be anInteger."

	messageListIndex := anInteger.
	contents := 
		messageListIndex ~= 0
			ifTrue: [self selectedMessage]
			ifFalse: [''].
	self changed: #messageListIndex.	 "update my selection"
	self editSelection: #editMessage.
	self contentsChanged.
	(messageListIndex ~= 0 and: [ autoSelectString notNil and: [ self contents notEmpty ] ]) ifTrue: [ self changed: #autoSelect ].
	self decorateButtons
! !

!MessageSet methodsFor: 'pluggable menus - hooks' stamp: 'topa 3/16/2015 13:45'!
messageListMenuHook: aMenu shifted: aBoolean
	<messageListMenu>
	^ self menuHook: aMenu named: #messageListMenu shifted: aBoolean.
! !

!MessageSet methodsFor: 'message functions' stamp: 'topa 3/16/2015 13:46'!
messageListMenu: aMenu shifted: shifted 
	"Answer the message-list menu"
	^ self menu: aMenu for: #(messageListMenu messageListMenuShifted:) shifted: shifted
! !

!MessageSet methodsFor: 'class list' stamp: ''!
metaClassIndicated
	"Answer the boolean flag that indicates whether
	this is a class method."

	^ self selectedClassOrMetaClass isMeta! !

!MessageSet methodsFor: 'message functions' stamp: 'sw 2/24/1999 18:31'!
methodCategoryChanged
	self changed: #annotation! !

!MessageSet methodsFor: 'user interface' stamp: 'mt 4/12/2015 19:48'!
modelWakeUp
	"A window with me as model has been activated."

	super modelWakeUp.
	(self canDiscardEdits and: [autoSelectString notNil]) ifTrue: [
		self codeTextMorph ifNotNil: [:tm |
			(tm hasProperty: #launched) ifFalse: [
				tm scrollSelectionIntoView.
				tm setProperty: #launched toValue: true] ] ]! !

!MessageSet methodsFor: 'message list' stamp: 'sbw 12/30/1999 17:19'!
optionalButtonHeight

	^ 15! !

!MessageSet methodsFor: 'message functions' stamp: 'cmm 6/6/2011 09:55'!
reformulateList
	"The receiver's messageList has been changed; rebuild it"
	super reformulateList.
	self
		 changed: #messageList ;
		 changed: #messageListIndex.
	self contentsChanged.
	autoSelectString ifNotNil: [ self changed: #autoSelect ]! !

!MessageSet methodsFor: 'message functions' stamp: 'fbs 3/17/2011 17:36'!
removeMessage
	"Remove the selected message from the system. 1/15/96 sw"
	| messageName confirmation |
	self hasMessageSelected
		ifFalse: [^ self].
	self okToChange
		ifFalse: [^ self].
	messageName := self selectedMessageName.
	confirmation := self systemNavigation  confirmRemovalOf: messageName on: self selectedClassOrMetaClass.
	confirmation = 3
		ifTrue: [^ self].
	self selectedClassOrMetaClass removeSelector: messageName.
	self deleteFromMessageList: self selection.
	self reformulateList.
	confirmation = 2
		ifTrue: [self systemNavigation browseAllCallsOn: messageName]! !

!MessageSet methodsFor: 'message functions' stamp: 'fbs 3/22/2011 22:11'!
removeMessageFromBrowser
	"Remove the selected message from the browser."

	self hasMessageSelected ifFalse: [^ self].
	self deleteFromMessageList: self selection.
	self reformulateList.
	self adjustWindowTitleAfterFiltering
! !

!MessageSet methodsFor: 'morphic ui' stamp: 'cmm 2/19/2010 13:32'!
representsSameBrowseeAs: anotherModel
	^ self hasUnacceptedEdits not
	and: [ messageList = anotherModel messageList ]! !

!MessageSet methodsFor: 'user interface' stamp: 'fbs 6/8/2011 12:17'!
request: prompt initialAnswer: initialAnswer

	^ UIManager default
		request: prompt
		initialAnswer: initialAnswer! !

!MessageSet methodsFor: 'class list' stamp: 'sd 11/20/2005 21:27'!
selectedClass 
	"Return the base class for the current selection.  1/17/96 sw fixed up so that it doesn't fall into a debugger in a msg browser that has no message selected"

	| aClass |
	^ (aClass := self selectedClassOrMetaClass) == nil
		ifTrue:
			[nil]
		ifFalse:
			[aClass theNonMetaClass]! !

!MessageSet methodsFor: 'class list' stamp: 'tk 4/4/98 18:50'!
selectedClassName
	"Answer the name of class of the currently selected message. Answer nil if no selection 
	exists."

	| cls |
	(cls := self selectedClass) ifNil: [^ nil].
	^ cls name! !

!MessageSet methodsFor: 'class list' stamp: 'fbs 3/22/2011 22:19'!
selectedClassOrMetaClass
	"Answer the currently selected class (or metaclass)."
	self hasMessageSelected ifFalse: [^nil].
	self setClassAndSelectorIn: [:c :s | ^c]! !

!MessageSet methodsFor: 'message list' stamp: 'ar 1/2/2010 03:41'!
selectedMessage
	"Answer the source method for the currently selected message."

	
	self setClassAndSelectorIn: [:class :selector | | source | 
		class ifNil: [^ 'Class vanished'].
		selector first isUppercase ifTrue:
			[selector == #Comment ifTrue:
				[currentCompiledMethod := class organization commentRemoteStr.
				^ class comment].
			selector == #Definition ifTrue:
				[^ class definition].
			selector == #Hierarchy ifTrue: [^ class printHierarchy]].
		source := class sourceMethodAt: selector ifAbsent:
			[currentCompiledMethod := nil.
			^ 'Missing'].

		self showingDecompile ifTrue: [^ self decompiledSourceIntoContents].

		currentCompiledMethod := class compiledMethodAt: selector ifAbsent: [nil].
		self showingDocumentation ifTrue: [^ self commentContents].

	source := self sourceStringPrettifiedAndDiffed.
	^ source asText makeSelectorBoldIn: class]! !

!MessageSet methodsFor: 'class list' stamp: 'fbs 3/24/2011 07:43'!
selectedMessageCategoryName 
	"Answer the name of the selected message category or nil."
	| cls |
	self hasMessageSelected ifFalse: [^ nil].
	cls := self selectedClassOrMetaClass.
	
	cls ifNil: [^ nil].
	
	^ cls organization categoryOfElement: self selectedMessageName! !

!MessageSet methodsFor: 'message list' stamp: 'sma 3/3/2000 11:17'!
selectedMessageName
	"Answer the name of the currently selected message."
	"wod 6/16/1998: answer nil if none are selected."

	messageListIndex = 0 ifTrue: [^ nil].
	^ self setClassAndSelectorIn: [:class :selector | ^ selector]! !

!MessageSet methodsFor: 'private' stamp: 'fbs 3/24/2011 17:40'!
selection
	"Answer the item in the list that is currently selected, or nil if no selection is present"

	^ self messageList at: (self messageListIndex) ifAbsent: [nil]! !

!MessageSet methodsFor: 'private' stamp: 'RAA 5/29/2001 10:12'!
setClassAndSelectorIn: csBlock
	| sel |
	"Decode strings of the form <className> [class] <selectorName>."

	self flag: #mref.	"compatibility with pre-MethodReference lists"

	sel := self selection.
	^(sel isKindOf: MethodReference) ifTrue: [
		sel setClassAndSelectorIn: csBlock
	] ifFalse: [
		MessageSet parse: sel toClassAndSelector: csBlock
	]! !

!MessageSet methodsFor: 'contents' stamp: 'sw 2/14/2001 15:25'!
setContentsToForceRefetch
	"Set the receiver's contents such that on the next update the contents will be formulated afresh.  This is a critical and obscure difference between Browsers on the one hand and MessageSets on the other, and has over the years been the source of much confusion and much difficulty.  By centralizing the different handling here, we don't need so many idiosyncratic overrides in MessageSet any more"

	contents := ''! !

!MessageSet methodsFor: 'private' stamp: 'ul 11/15/2010 11:54'!
setFilteredList: newList 
	"Establish newList as the new list if appropriate, and adjust the window title accordingly; if the new list is of the same size as the old, warn and do nothing"
	newList size = 0 ifTrue:
		[ self inform: 'Nothing would be left in the list if you did that'.
		^false ].
	newList size = messageList size ifTrue:
		[ self inform: 'That leaves the list unchanged'.
		^false ].
	self
		 initializeMessageList: newList ;
		 adjustWindowTitleAfterFiltering.
	^true! !

!MessageSet methodsFor: 'message functions' stamp: 'topa 3/16/2015 15:17'!
shiftedMessageListMenu: aMenu
	"Fill aMenu with the items appropriate when the shift key is held down"
	<messageListMenuShifted: true>

	aMenu addStayUpItem.
	aMenu addList: #(
		('toggle diffing (D)'						toggleDiffing)
		('implementors of sent messages'			browseAllMessages)
		-
		('local senders of...'						browseLocalSendersOfMessages)
		('local implementors of...'				browseLocalImplementors)
		-
		('spawn sub-protocol'					spawnProtocol)
		('spawn full protocol'					spawnFullProtocol)
		-
		('sample instance'						makeSampleInstance)
		('inspect instances'						inspectInstances)
		('inspect subinstances'					inspectSubInstances)).

	self addExtraShiftedItemsTo: aMenu.
	aMenu addList: #(
		-
		('change category...'					changeCategory)).

	self canShowMultipleMessageCategories ifTrue: [aMenu addList:
		 #(('show category (C)'						showHomeCategory))].
	aMenu addList: #(
		-
		('change sets with this method'			findMethodInChangeSets)
		('revert to previous version'				revertToPreviousVersion)
		('remove from current change set'		removeFromCurrentChanges)
		('revert & remove from changes'		revertAndForget)
		('add to current change set'				adoptMessageInCurrentChangeset)
		('copy up or copy down...'				copyUpOrCopyDown)).
	^ aMenu
! !

!MessageSet methodsFor: 'message list' stamp: 'nice 8/19/2010 16:33'!
sortByDate
	"Sort the message-list by date of time-stamp"

	| assocs inOrder |
	assocs := messageList collect:
		[:aRef | | aDate aCompiledMethod |
			aDate := aRef methodSymbol == #Comment
				ifTrue:
					[aRef actualClass organization dateCommentLastSubmitted]
				ifFalse:
					[aCompiledMethod := aRef actualClass compiledMethodAt: aRef methodSymbol ifAbsent: [nil].
					aCompiledMethod ifNotNil: [aCompiledMethod dateMethodLastSubmitted]].
			aRef -> (aDate ifNil: [Date fromString: '01/01/1996'])].  "The dawn of Squeak history"
	inOrder := assocs asArray sort: [:a :b | a value < b value].

	messageList := inOrder collect: [:assoc | assoc key].
	self changed: #messageList! !

!MessageSet methodsFor: 'initialize-release' stamp: 'fbs 3/29/2011 17:10'!
systemOrganizer: aSystemOrganizer

	messageListIndex := 0.
	^ super systemOrganizer: aSystemOrganizer.! !

!MessageSet methodsFor: 'copying' stamp: 'cmm 2/18/2014 14:16'!
veryDeepInner: deepCopier
	super veryDeepInner: deepCopier.
	messageList := messageList veryDeepCopyWith: deepCopier.
	editSelection := editSelection veryDeepCopyWith: deepCopier.! !

MessageSet subclass: #MessageTrace
	instanceVariableNames: 'autoSelectStrings messageSelections anchorIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Browser'!

!MessageTrace commentStamp: 'cmm 3/2/2010 20:26' prior: 0!
A MessageTrace is a MessageSet allowing efficient sender/implementor message following.  With implementors indented below, and senders outdended above, message flow is succinctly expressed, hierarchically.

My autoSelectStrings and messageSelections are Arrays of Booleans, parallel to my messageList.  Each boolean indicates whether that message is selected.  Each autoSelectStrings indicates which string should be highlighted in the code for each method in my messageList.!

!MessageTrace class methodsFor: 'initializing' stamp: 'cmm 1/28/2003 22:20'!
initialize

	self setUpPreferencesPanel! !

!MessageTrace class methodsFor: 'initializing' stamp: 'cmm 3/2/2010 20:39'!
setUpPreferencesPanel
	Preferences
		addPreference: #traceMessages
		category: #browsing
		default: false
		balloonHelp: 'If true, browsing senders or implementors in a methods browser will add to the methods in that browser instead of opening a new browser.'
! !

!MessageTrace methodsFor: 'building' stamp: 'cmm 4/18/2016 18:15'!
addChildMessages: methodReferences autoSelectString: aString 
	| currentIndentionLevel addables selectables selectableString newAnchor |
	selectableString := aString keywords 
		ifEmpty: [ String empty ] 
		ifNotEmptyDo: [ : keywords | keywords first ].
	[ (messageListIndex between: 1 and: autoSelectStrings size) ]
		whileFalse:
			[ autoSelectStrings add: selectableString ].
	autoSelectStrings 
		at: messageListIndex
		put: selectableString.
	currentIndentionLevel := self currentIndentionLevel.
	"Don't add mulitple copies of the same method, if a method is already in the list we will merely select it."
	addables := methodReferences reject: [ : each | messageList includes: each ].
	addables do: 
		[ : each | 
		each stringVersion: (self indentionPrefixOfSize: currentIndentionLevel + 1) , each asStringOrText.
		messageList 
			add: each
			afterIndex: self messageListIndex.
		autoSelectStrings 
			add: nil
			afterIndex: self messageListIndex.
		messageSelections 
			add: false
			afterIndex: self messageListIndex ].
	selectables := 
		addables copy
			addAll: (methodReferences select: [ : each | messageList includes: each ]) ;
			yourself.
	self deselectAll.
	anchorIndex := nil.
	selectables do:
		[ : each |
		self
			messageAt: (newAnchor := messageList indexOf: each) 
			beSelected: true.
		anchorIndex ifNil: [ anchorIndex := newAnchor ] ].
	self changed: #messageList.
	"Select the first child method."
	self messageListIndex: 
		(selectables size > 0
			ifTrue: [ messageList indexOf: selectables last ]
			ifFalse: [ messageList ifEmpty: [ 0 ] ifNotEmpty: [ 1 ] ])! !

!MessageTrace methodsFor: 'building' stamp: 'cmm 2/22/2002 01:22'!
addChildMethodsNamed: selectorSymbol

	| methodsReferences |

	messageListIndex = 0 
		ifTrue:
			[ ^(PopUpMenu labels: ' OK ')
				startUpWithCaption: 'Please reselect a method.' ].
	(methodsReferences := self filteredSelectorsNamed: selectorSymbol) isEmpty
		ifTrue:
			[ ^(PopUpMenu labels: ' OK ')
				startUpWithCaption: 'There are no methods named ', selectorSymbol ]
		ifFalse:
			[ self
				addChildMessages: methodsReferences
				autoSelectString: selectorSymbol ]
! !

!MessageTrace methodsFor: 'message list' stamp: 'cmm 4/29/2013 23:30'!
addExtraShiftedItemsTo: aMenu 
	"The shifted selector-list menu is being built.  Add items specific to MessageTrace."
	super addExtraShiftedItemsTo: aMenu.
	aMenu addList: #(#('invert level selection (I)' #invertLevelSelection) )! !

!MessageTrace methodsFor: 'building' stamp: 'cmm 4/29/2013 23:39'!
addParentMessages: methodReferences autoSelectString: aString 
	| currentIndentionLevel addables selectables |
	addables := methodReferences reject: [ : each | messageList includes: each ].
	selectables := addables copy
		addAll: (methodReferences select: [ : each | messageList includes: each ]) ;
		yourself.
	currentIndentionLevel := self currentIndentionLevel.
	(currentIndentionLevel = 0 and: [ addables notEmpty ]) ifTrue: 
		[ self indentEverything.
		currentIndentionLevel := 1 ].
	addables do: 
		[ : each | 
		each stringVersion: (self indentionPrefixOfSize: currentIndentionLevel - 1) , each asStringOrText.
		messageList 
			add: each
			afterIndex: self messageListIndex - 1.
		autoSelectStrings 
			add: aString
			afterIndex: self messageListIndex - 1.
		messageSelections 
			add: false
			afterIndex: self messageListIndex - 1 ].
	self deselectAll.
	selectables do:
		[ : each | | messageIndex |
		messageIndex := messageList indexOf: each.
		self
			messageAt: messageIndex 
			beSelected: true.
		autoSelectStrings
			at: messageIndex
			put: aString ].
	self changed: #messageList.
	selectables size > 0 ifTrue:
		[ self messageListIndex: (messageList indexOf: selectables first) ]! !

!MessageTrace methodsFor: 'building' stamp: 'cmm 4/29/2013 23:22'!
addParentMethodsSending: selectorSymbol

	| methodsList |
	(methodsList := self systemNavigation allCallsOn: selectorSymbol) isEmpty
		ifTrue:
			[ ^(PopUpMenu labels: ' OK ')
				startUpWithCaption: 'There are no methods that send ', selectorSymbol ]
		ifFalse:
			[ self
				addParentMessages: methodsList
				autoSelectString: selectorSymbol ]
! !

!MessageTrace methodsFor: 'building' stamp: 'cm 1/9/2002 14:27'!
autoSelectString: aString

	super autoSelectString: aString.
	autoSelectStrings := messageList collect: [ :each | aString ]
! !

!MessageTrace methodsFor: 'actions' stamp: 'cmm 2/19/2010 16:08'!
browseAllCallsOn: selectorSymbol
	(self hasUnacceptedEdits or: [ Preferences traceMessages not ])
		ifTrue: [ super browseAllCallsOn: selectorSymbol ]
		ifFalse: [ self addParentMethodsSending: selectorSymbol ]! !

!MessageTrace methodsFor: 'actions' stamp: 'cmm 2/19/2010 16:08'!
browseAllImplementorsOf: selectorSymbol
	| selectorToBrowse |
	selectorToBrowse := self selection 
		ifNil: [ selectorSymbol ] 
		ifNotNil: [ self getImplementorNamed: selectorSymbol ].
	(self hasUnacceptedEdits or: [ Preferences traceMessages not ])
		ifTrue: [ super browseAllImplementorsOf: selectorToBrowse ]
		ifFalse: [ self addChildMethodsNamed: selectorToBrowse ]
! !

!MessageTrace methodsFor: 'private initializing' stamp: 'mt 11/6/2015 11:38'!
buildMessageListWith: builder
	| listSpec |
	listSpec := builder pluggableAlternateMultiSelectionListSpec new.
	listSpec 
		model: self ;
		list: #messageList ;
		getIndex: #messageListIndex ;
		setIndex: #toggleSelectionAt:shifted:controlled: ;
		icon: #messageIconAt:;
		help: #messageHelpAt:; 
		menu: #messageListMenu:shifted: ; 
		getSelectionList: #isMessageSelectedAt: ;
		setSelectionList: #messageAt:beSelected: ;
		keyPress: #messageListKey:from:.
	SystemBrowser browseWithDragNDrop 
		ifTrue: [ listSpec dragItem: #dragFromMessageList: ].
	^ listSpec! !

!MessageTrace methodsFor: '*Tools' stamp: 'cmm 6/20/2012 17:34'!
copySelector
	Clipboard clipboardText:
		(String streamContents:
			[ : stream | self selectedMessages
				do: [ : each | stream nextPutAll: each selector asString ]
				separatedBy: [ stream space ] ])! !

!MessageTrace methodsFor: 'private accessing' stamp: 'cmm 4/30/2013 00:07'!
currentIndentionLevel
	^ messageListIndex isZero ifFalse: [ self indentionLevelAt: messageListIndex ]! !

!MessageTrace methodsFor: 'building' stamp: 'cmm 1/30/2002 00:53'!
deleteFromMessageList: aMethodReference
	"Delete the given message from the receiver's message list"

	| index |
	autoSelectStrings removeAt: (index := messageList indexOf: aMethodReference).
	messageSelections removeAt: index.
	super deleteFromMessageList: aMethodReference.
	anchorIndex ifNotNil:
		[ anchorIndex := anchorIndex min: messageList size ]
! !

!MessageTrace methodsFor: 'actions' stamp: 'cmm 10/16/2015 14:12'!
deselectAll
	super deselectAll.
	1 to: messageSelections size do: [ :index | messageSelections at: index put: false ]
! !

!MessageTrace methodsFor: 'private actions' stamp: 'cmm 1/18/2002 15:17'!
deselectAllBetweenLastSelectionAnd: indexPosition

	| lower higher |
	lower := messageListIndex min: indexPosition.
	higher := messageListIndex max: indexPosition.
	lower = 0 ifTrue: [ ^nil ].
	lower to: higher do:
		[ :index | messageSelections at: index put: false ]
! !

!MessageTrace methodsFor: 'filtering' stamp: 'cmm 7/3/2003 12:33'!
filteredSelectorsNamed: selectorSymbol

	^SystemNavigation new allImplementorsOf: selectorSymbol
! !

!MessageTrace methodsFor: 'filtering' stamp: 'cmm 4/26/2015 17:42'!
filterFrom: aBlock 
	"Filter the receiver's list down to only those items that satisfy aBlock, which takes a class an a selector as its arguments."
	| newList newAutoSelectStrings newMessageSelections oldList |
	oldList := Dictionary new.
	messageList do: [ : each | oldList at: each put: each ].
	newList := messageList class new.
	newAutoSelectStrings := autoSelectStrings class new.
	newMessageSelections := messageSelections class new.
	messageList withIndexDo:
		[ : each : index | (self class
			parse: each
			toClassAndSelector: aBlock) ifTrue:
				[ newList add: each.
				newAutoSelectStrings add: (autoSelectStrings at: index).
				newMessageSelections add: (messageSelections at: index) ] ].
	(self setFilteredList: newList) ifTrue:
		[ autoSelectStrings := newAutoSelectStrings.
		messageList do:
			[ : each | each stringVersion: (oldList at: each) stringVersion ] ]! !

!MessageTrace methodsFor: 'private accessing' stamp: 'cmm 12/14/2007 13:03'!
getImplementorNamed: selectorSymbol
	 | allPossibilities |
	allPossibilities := (((self selection compiledMethod messages 
		select: [ :eachSelector | eachSelector beginsWith: selectorSymbol ])
		copyWith: selectorSymbol)
		select: [ :each | Symbol hasInterned: each ifTrue: [ :s | ] ])
		asSet asSortedCollection asOrderedCollection.
	(allPossibilities includes: selectorSymbol) ifTrue:
		[ allPossibilities addFirst: (allPossibilities remove: selectorSymbol) ].
	^allPossibilities size > 1
		ifTrue: 
			[ | selectionIndex |
			selectionIndex := (PopUpMenu labelArray: allPossibilities lines: #(1))
				startUpWithCaption: 
					'Browse implementors of
					which message?'.
			selectionIndex = 0 ifTrue: [ selectorSymbol ] ifFalse: [ allPossibilities at: selectionIndex ] ]
		ifFalse: [ allPossibilities isEmpty
			ifTrue: [ selectorSymbol ]
			ifFalse: [ allPossibilities first ] ]
! !

!MessageTrace methodsFor: 'indenting' stamp: 'cmm 1/13/2002 00:02'!
indentEverything

	messageList do: [ :each | each stringVersion: (self indentionPrefixOfSize: 1), each stringVersion ]
! !

!MessageTrace methodsFor: 'private accessing' stamp: 'cmm 4/29/2013 23:52'!
indentionLevelAt: msgListIndex
	^ self indentionsIn: (messageList at: msgListIndex) stringVersion! !

!MessageTrace methodsFor: 'indenting' stamp: 'cm 1/8/2002 17:50'!
indentionPrefixOfSize: levelInteger

	| answer |
	answer := String new: levelInteger * self indentionSize.
	answer atAllPut: $ . "space"
	^answer
! !

!MessageTrace methodsFor: 'indenting' stamp: 'cm 1/8/2002 17:34'!
indentionSize

	^2  "that is, 2 spaces.."
! !

!MessageTrace methodsFor: 'indenting' stamp: 'cm 1/8/2002 17:50'!
indentionsIn: aString

	aString
		withIndexDo: 
			[ :eachChar :index | 
			eachChar = $  "space" ifFalse: [ ^(index-1) / self indentionSize ] ].
	^0
! !

!MessageTrace methodsFor: 'private initializing' stamp: 'cmm 1/17/2002 22:33'!
initialize

	super initialize.
	messageSelections := OrderedCollection new.
	autoSelectStrings := OrderedCollection new
! !

!MessageTrace methodsFor: 'private initializing' stamp: 'cmm 3/2/2010 14:43'!
initializeMessageList: anArray
	messageSelections := (Array new: anArray size withAll: false) asOrderedCollection.
	super initializeMessageList: anArray.
	self 
		messageAt: messageListIndex
		beSelected: true.
	"autoSelectStrings is initialized right after this method, in autoSelectString:"
! !

!MessageTrace methodsFor: 'actions' stamp: 'cmm 7/25/2014 20:07'!
invertLevelSelection
	"The user wants to be able to quickly and easily assemble message-traces.  When browsing senders or implementors of a heavily overloaded selector, typically just one or two messages are relevant out of several.  In that situation, the user need merely select the range of relevant methods to arrive within two gestures of trimming the irrelevant methods:  Command+I (capital eye) to invert the selection within the local group followed by Command+d to remove that selection from the browser."
	| index currentLevel start |
	currentLevel := self currentIndentionLevel ifNil:
		[ | possibilities |
		possibilities := ((1 to: messageSelections size) select:
			[ : idx | messageSelections at: idx ]) collect:
			[ : eachSelectedIndex | self indentionLevelAt: eachSelectedIndex ].
		possibilities asSet size > 1
			ifTrue: [ ^ self ]
			ifFalse:
				[ possibilities ifEmpty: [ ^ self ].
				start := messageSelections indexOf: true.
				possibilities anyOne ] ].
	"First invert at messageListIndex then crawl up and down from there until a different level is reached, inverting as we go."
	messageListIndex > 0 ifTrue: [ start := messageListIndex ].
	self toggleMessageSelectionAt: start.
	index := start.
	[ index := index - 1.
	index > 0 and: [ (self indentionLevelAt: index) = currentLevel ] ] whileTrue: [ self toggleMessageSelectionAt: index ].
	index := start.
	[ index := index + 1.
	index <= messageList size and: [ (self indentionLevelAt: index) = currentLevel ] ] whileTrue: [ self toggleMessageSelectionAt: index ].
	self
		 messageListIndex: 0 ;
		 changed: #messageList! !

!MessageTrace methodsFor: 'testing' stamp: 'cmm 3/2/2004 23:40'!
isMessageSelectedAt: anInteger

	^messageSelections at: anInteger ifAbsent: [ false ]! !

!MessageTrace methodsFor: 'actions' stamp: 'cmm 2/19/2010 16:21'!
messageAt: indexInteger beSelected: aBoolean
	^ indexInteger isZero ifFalse:
		[ messageSelections
			at: indexInteger
			put: aBoolean ]! !

!MessageTrace methodsFor: 'actions' stamp: 'cmm 2/19/2010 17:45'!
messageListIndex: anInteger
	autoSelectStrings notEmpty ifTrue: 
		[ autoSelectString :=
			anInteger = 0
				ifTrue: [ '' ]
				ifFalse:
					[ messageListIndex := anInteger.  "setting the window label, below, can't wait for this.."
					self containingWindow setLabel: (self windowLabelAt: anInteger).
					(autoSelectStrings at: anInteger) ifNotNilDo: [ : fullSelector | fullSelector keywords first ] ] ].
	anInteger > 0 ifTrue:
		[ self
			messageAt: anInteger
			beSelected: true ].
	super messageListIndex: anInteger
! !

!MessageTrace methodsFor: 'message list' stamp: 'cmm 5/7/2013 19:49'!
messageListKey: aChar from: view 
	aChar == $I ifTrue: [ ^ self invertLevelSelection ].
	^ super messageListKey: aChar from: view! !

!MessageTrace methodsFor: 'accessing' stamp: 'cmm 3/2/2010 15:46'!
noteSelectionIndex: anInteger for: aSymbol
	aSymbol == #messageList
		ifTrue:
			[ messageListIndex := anInteger.
			self 
				messageAt: messageListIndex
				beSelected: true ].
	super
		noteSelectionIndex: anInteger
		for: aSymbol! !

!MessageTrace methodsFor: 'actions' stamp: 'cmm 3/2/2010 17:01'!
removeMessage
	"Remove the selected messages from the system."
	self selectedMessages size = 0 ifTrue: [ ^self ].
	self selectedMessages size = 1 ifTrue: [ ^super removeMessage ].
	(self confirm: 'Are you certain you want to remove all of the selected methods from the image?') ifFalse: [ ^self ].
	self selectedMessages do:
		[ :each | 
		each actualClass removeSelector: each methodSymbol.
		self deleteFromMessageList: each ]! !

!MessageTrace methodsFor: 'building' stamp: 'cmm 3/11/2010 14:04'!
removeMessageFromBrowser
	| indexToSelect |
	"Try to keep the same selection index."
	indexToSelect := (messageSelections indexOf: true) max: 1.
	self selectedMessages do: [ :eachMethodReference | self deleteFromMessageList: eachMethodReference ].
	self deselectAll.
	messageSelections ifNotEmpty:
		[ messageSelections 
			at: (indexToSelect min: messageSelections size)  "safety"
			put: true ].
	anchorIndex := indexToSelect min: messageSelections size.
	self 
		messageListIndex: anchorIndex ; 
		reformulateList! !

!MessageTrace methodsFor: 'private actions' stamp: 'cmm 1/22/2002 16:04'!
selectAllBetweenAnchorAnd: indexPosition

	| lower higher |
	self deselectAllBetweenLastSelectionAnd: indexPosition.
	anchorIndex ifNil: [ anchorIndex := indexPosition ].
	lower := anchorIndex min: indexPosition.
	higher := anchorIndex max: indexPosition.
	lower to: higher do:
		[ :index | messageSelections at: index put: true ]
! !

!MessageTrace methodsFor: 'private actions' stamp: 'cmm 1/18/2002 11:28'!
selectedMessages

	| answer |
	answer := OrderedCollection new.
	messageSelections withIndexDo:
		[ :eachSelection :index |
		eachSelection ifTrue: [ answer add: (messageList at: index) ] ].
	^answer
! !

!MessageTrace methodsFor: 'private actions' stamp: 'cmm 1/18/2002 15:19'!
toggleMessageSelectionAt: anInteger

	messageSelections 
		at: anInteger 
		put: (messageSelections at: anInteger) not
! !

!MessageTrace methodsFor: 'actions' stamp: 'mt 4/12/2015 18:27'!
toggleSelectionAt: currentPosition
	"Compatibility with MVC"

	^ self
		toggleSelectionAt: currentPosition
		shifted: false
		controlled: false! !

!MessageTrace methodsFor: 'actions' stamp: 'cmm 4/18/2016 18:22'!
toggleSelectionAt: anInteger shifted: isShifted controlled: isMouseDragging 
	anInteger = 0 ifTrue: [ ^ nil ].
	(isMouseDragging not and: [anInteger = anchorIndex and: [ (messageSelections occurrencesOf: true) = 1 ]])
		ifTrue:
			[ self toggleMessageSelectionAt: anInteger.
			anchorIndex := nil.
			self messageListIndex: 0 ]
		ifFalse:
			[ self deselectAll.
			isShifted
				ifTrue: [ self selectAllBetweenAnchorAnd: anInteger ]
				ifFalse:
					[ self
						messageAt: anInteger
						beSelected: true.
					anchorIndex := anInteger ].
			self messageListIndex: anInteger ]! !

!MessageTrace methodsFor: 'copying' stamp: 'cmm 8/13/2011 19:30'!
veryDeepInner: aDeepCopier 
	super veryDeepInner: aDeepCopier.
	autoSelectStrings := autoSelectStrings veryDeepCopyWith: aDeepCopier.
	messageSelections := messageSelections veryDeepCopyWith: aDeepCopier! !

!MessageTrace methodsFor: 'private accessing' stamp: 'cm 1/9/2002 17:26'!
windowLabelAt: anInteger

	| str |
	^(str := autoSelectStrings at: anInteger)
		ifNil:
			[ 'Implementors of ',
				(self class
					parse: self selection
					toClassAndSelector: [ :class :selector | selector ]) ]
		ifNotNil:
			[ 'Senders of ', str ]
! !

MessageSet subclass: #RecentMessageSet
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Browser'!

!RecentMessageSet commentStamp: 'sw 8/1/2002 17:40' prior: 0!
RecentMessageSet is a message set that shows the most recently-submitted methods, in chronological order.!

!RecentMessageSet methodsFor: 'message list' stamp: 'fbs 9/3/2013 09:06'!
addExtraShiftedItemsTo: aMenu
	"The shifted selector-list menu is being built.  Overridden here to defeat the presence of the items that add or change order, since RecentMessageSet defines methods & order explicitly based on external criteria"

	aMenu add: 'set size of recent history...' target: RecentMessages action: #setRecentHistorySize! !

!RecentMessageSet methodsFor: 'contents' stamp: 'sw 10/19/1999 17:33'!
contents: c notifying: n
	| result |
	result := super contents: c notifying: n.
	result == true ifTrue:
		[self reformulateList].
	^ result! !

!RecentMessageSet methodsFor: 'update' stamp: 'sw 1/28/2001 20:59'!
growable
	"Answer whether the receiver can be changed by manual additions & deletions"

	^ false! !

!RecentMessageSet methodsFor: 'message functions' stamp: 'topa 3/16/2015 15:16'!
messageListMenuForRecentSubmission: aMenu
	<messageListMenuShifted: false>
	aMenu addList:#(
			('remove from RecentSubmissions'		removeFromRecentSubmissions)).
	^ aMenu! !

!RecentMessageSet methodsFor: 'update' stamp: 'mt 3/24/2016 08:11'!
reformulateList
	"Reformulate the receiver's list.  Exclude methods now deleted"
	
	self
		initializeMessageList: RecentMessages default methodReferences;
		messageListIndex: (self messageListIndex min: messageList size);
		changed: #messageList;
		changed: #messageListIndex.! !

!RecentMessageSet methodsFor: 'message functions' stamp: 'fbs 9/9/2013 20:58'!
removeFromRecentSubmissions
	"Remove the currently-selected method from the RecentMessages list"

	| aClass methodSym |
	((aClass := self selectedClassOrMetaClass) notNil and: [(methodSym := self selectedMessageName) notNil])
		ifTrue: 
			[RecentMessages default purgeFromRecentSubmissions: (MethodReference class: aClass selector: methodSym).
			self reformulateList]! !

!RecentMessageSet methodsFor: 'message list' stamp: 'fbs 9/3/2013 08:50'!
setRecentHistorySize
	"Let the user specify the recent history size"

	| aReply aNumber |
	aReply := UIManager default request: 'How many recent methods
should be maintained?' initialAnswer: RecentMessages numberOfRecentSubmissionsToStore asString.
	aReply isEmptyOrNil ifFalse:
		[aNumber := aReply asNumber rounded.
		(aNumber > 1 and: [aNumber <= 1000])
			ifTrue:
				[RecentMessages numberOfRecentSubmissionsToStore: aNumber.
				self inform: 'Okay, ', aNumber asString, ' is the new size of the recent method history']
			ifFalse:
				[self inform: 'Sorry, must be a number between 2 & 1000']]! !

!RecentMessageSet methodsFor: 'update' stamp: 'cmm 11/30/2013 18:26'!
updateListsAndCodeIn: aWindow 
	| recentFromUtilities |
	self canDiscardEdits ifFalse: [ ^ self ].
	RecentMessages default isEmpty ifTrue: [ ^ self ].
	recentFromUtilities := RecentMessages default mostRecent asString , ' '.
	(messageList isEmpty or: [ messageList first asStringOrText asString beginsWith: recentFromUtilities ])
		ifTrue: [ self updateCodePaneIfNeeded ]
		ifFalse: [ self reformulateList ]! !

MessageSet subclass: #TimeProfileBrowser
	instanceVariableNames: 'block tally methodReferences'
	classVariableNames: 'TextMenu'
	poolDictionaries: ''
	category: 'Tools-Debugger'!

!TimeProfileBrowser commentStamp: '<historical>' prior: 0!
A TimeProfileBrowser is a browser visualizing the runtime profile of an executed Smalltalk block.  It is useful for finding performance bottlenecks in code. When optimizing code it can
be hard to know what methods actually constitute the bulk of the execution time. Is it a few
methods that take very long time to execute or is it perhaps a single method that gets executed a thousand times?

The block is first spied on using a MessageTally instance (which has even more funtionality than used by the TimeProfileBrowser) which samples the block during it's execution and collects the amount of time approximately spent in the methods executed. Then the methods are shown in the browser with their relative execution time in percent.

Example:
TimeProfileBrowser onBlock: [20 timesRepeat:  [Transcript show: 100 factorial printString]]
!

!TimeProfileBrowser class methodsFor: 'instance creation' stamp: 'sd 11/20/2005 21:28'!
onBlock: block
	"Open a profile browser on the given block, thereby running the block and 
	 collecting the message tally."
	"TimeProfileBrowser onBlock: [20 timesRepeat: 
			[Transcript show: 100 factorial printString]]"

	| inst result |
	inst := self new.
	result := inst runBlock: block.
	self open: inst name: 'Time Profile'.
	^ result! !

!TimeProfileBrowser class methodsFor: 'instance creation' stamp: 'sd 11/20/2005 21:28'!
spyOnProcess: aProcess forMilliseconds: msecDuration 
	"Run aProcess for msecDuration milliseconds, then open a TimeProfileBrowser on the results."

	"| p |  
	p := [100000 timesRepeat: [3.14159 printString]] fork.  
	(Delay forMilliseconds: 100) wait.  
	TimeProfileBrowser spyOnProcess: p forMilliseconds: 1000"

	| inst |
	inst := self new.
	inst runProcess: aProcess forMilliseconds: msecDuration pollingEvery: MessageTally defaultPollPeriod.
	self open: inst name: (String streamContents: [ :s | s nextPutAll: 'Time Profile for '; print: msecDuration; nextPutAll: ' msec' ]).
	^ inst! !

!TimeProfileBrowser class methodsFor: 'instance creation' stamp: 'nk 3/8/2004 12:46'!
spyOn: block
	"Open a profile browser on the given block, thereby running the block and 
	 collecting the message tally."
	"TimeProfileBrowser spyOn:  [20 timesRepeat: 
			[Transcript show: 100 factorial printString]]"

	^self onBlock: block! !

!TimeProfileBrowser methodsFor: 'message list' stamp: 'ul 4/7/2016 17:11'!
hasMessageSelected

	^super hasMessageSelected and: [
		(self methodReferences at: self messageListIndex) notNil ]! !

!TimeProfileBrowser methodsFor: 'private' stamp: 'dvf 7/5/2000 18:48'!
initializeMessageList: anArray
	messageList := anArray.
	messageListIndex := 0.
	contents := ''! !

!TimeProfileBrowser methodsFor: 'contents' stamp: 'ul 11/2/2011 02:45'!
isModeStyleable
	"Ensure that only methods are styled."

	^self selectedMessageName notNil and: [ super isModeStyleable ]! !

!TimeProfileBrowser methodsFor: 'message list' stamp: 'ul 4/7/2016 17:26'!
messageHelpAt: anIndex
	"Show the first n lines of the sources code of the selected message."
	
	| reference source formatted lineCount |
	Preferences balloonHelpInMessageLists ifFalse: [^ nil].
	self messageList size < anIndex ifTrue: [^ nil].
	
	reference := (self methodReferences at: anIndex) ifNil: [ ^nil ].
	reference isValid ifFalse: [ ^nil ].
	
	source := reference compiledMethod getSource.
	formatted := (Smalltalk classNamed: #SHTextStylerST80)
		ifNil: [ source asText ]
		ifNotNil: [ :SHTextStylerST80 |
			SHTextStylerST80 new
				classOrMetaClass: reference actualClass;
				styledTextFor: source asText ].
	
	lineCount := 0.
	source doWithIndex: [:char :index |
		char = Character cr ifTrue: [lineCount := lineCount + 1].
		lineCount > 10 ifTrue: [
			formatted := formatted copyFrom: 1 to: index-1.
			formatted append: ' [...]'.
			^ formatted]].

	^ formatted! !

!TimeProfileBrowser methodsFor: 'message list' stamp: 'ul 11/18/2015 13:08'!
messageIconAt: index

	^nil! !

!TimeProfileBrowser methodsFor: 'private' stamp: 'ul 8/3/2011 01:54'!
messageListKey: aChar from: view 
	"Respond to a Command key. Cmd-D means re-run block."

	aChar == $d ifTrue: [ ^Cursor execute showWhile: block ].
	^super messageListKey: aChar from: view! !

!TimeProfileBrowser methodsFor: 'private' stamp: 'ul 4/7/2016 19:28'!
methodReferenceFrom: aString
	"Try to create a MethodReference from a line returned by MessageTally. Return nil if the string doesn't have the given format."

	| stream className selector |
	stream := aString readStream.
	"Skip percentages and timing data."
	stream
		skipTo: $};
		skipSeparators.
	(stream peekFor: $[) ifTrue: [ "Skip block markers."
		stream upToAnyOf: CharacterSet separators ].
	className := stream upToAnyOf: '(>'.
	stream atEnd ifTrue: [ ^nil ].
	stream last == $( ifTrue: [ "Method is defined in a super class"
		className := stream upTo: $).
		(stream peekFor: $>) ifFalse: [ ^nil ] ].
	(stream peekFor: $>) ifFalse: [ ^nil ].
	selector := stream upToEnd.
	^MethodReference
		class: ((Smalltalk classNamed: className) ifNil: [ ^nil ])
		selector: ((Symbol lookup: selector) ifNil: [ ^nil ])! !

!TimeProfileBrowser methodsFor: 'accessing' stamp: 'ul 4/7/2016 17:09'!
methodReferences

	^methodReferences ifNil: [
		methodReferences := messageList collect: [ :each | self methodReferenceFrom: each ] ]
! !

!TimeProfileBrowser methodsFor: 'private' stamp: 'nk 3/8/2004 12:51'!
runBlock: aBlock
	^self runBlock: aBlock pollingEvery: MessageTally defaultPollPeriod! !

!TimeProfileBrowser methodsFor: 'private' stamp: 'nice 2/24/2013 18:10'!
runBlock: aBlock pollingEvery: pollPeriod 
	| list result |
	block := MessageSend 
				receiver: self
				selector: #runBlock:pollingEvery:
				arguments: { 
						aBlock.
						pollPeriod}.	"so we can re-run it"
	tally := MessageTally new.
	tally
		reportOtherProcesses: false;
		maxClassNameSize: 1000;
		maxClassPlusSelectorSize: 1000;
		maxTabs: 100.
	result := tally spyEvery: pollPeriod on: aBlock.
	list := OrderedCollection withAll: (String streamContents: [ :s | tally report: s]) lines.
	self initializeMessageList: list.
	self changed: #messageList.
	self changed: #messageListIndex.
	^result! !

!TimeProfileBrowser methodsFor: 'private' stamp: 'nice 2/24/2013 18:11'!
runProcess: aProcess forMilliseconds: msecDuration pollingEvery: pollPeriod 
	| list result |
	block := MessageSend 
				receiver: self
				selector: #runProcess:forMilliseconds:pollingEvery: 
				arguments: { 
						aProcess.
						msecDuration.
						pollPeriod}.	"so we can re-run it"
	tally := MessageTally new.
	tally
		reportOtherProcesses: false;
		maxClassNameSize: 1000;
		maxClassPlusSelectorSize: 1000;
		maxTabs: 100.
	result := tally 
				spyEvery: pollPeriod
				onProcess: aProcess
				forMilliseconds: msecDuration.
	list := OrderedCollection withAll: (String streamContents: [ :s | tally report: s]) lines.
	self initializeMessageList: list.
	self changed: #messageList.
	self changed: #messageListIndex.
	^result! !

!TimeProfileBrowser methodsFor: 'message list' stamp: 'ul 4/7/2016 19:37'!
selectedMessage
	"Answer the source method for the currently selected message."

	self setClassAndSelectorIn: [ :class :selector |
		| source | 
		source := (class == UndefinedObject and: [ selector == #DoIt ])
			ifTrue: [ 'DoIt', String cr, String cr, block arguments first decompile decompileString ]
			ifFalse: [ class sourceMethodAt: selector ifAbsent: [ ^'Missing' ] ].
		SystemBrowser browseWithPrettyPrint ifTrue: [
			source := class prettyPrinterClass 
				format: source
				in: class
				notifying: nil
				decorated: false ].
		^source asText makeSelectorBoldIn: class ].
	^''! !

!TimeProfileBrowser methodsFor: 'private' stamp: 'ul 4/7/2016 19:46'!
setClassAndSelectorIn: csBlock
	"Evaluate csBlock with the selected methodReference, or return nil if there's nothing selected."

	^((self methodReferences at: self messageListIndex) ifNil: [ ^nil ])
		setClassAndSelectorIn: csBlock! !

!TimeProfileBrowser methodsFor: 'accessing' stamp: 'stp 05/08/1999 11:37'!
tally
	"Answer the receiver's 'tally'."

	^tally! !

!TimeProfileBrowser methodsFor: 'accessing' stamp: 'stp 05/08/1999 11:37'!
tally: anObject
	"Set the receiver's instance variable 'tally' to be anObject."

	tally := anObject! !

CodeHolder subclass: #MethodHolder
	instanceVariableNames: 'methodClass methodSelector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Base'!

!MethodHolder methodsFor: 'menu' stamp: 'dgd 8/30/2003 21:54'!
addModelMenuItemsTo: aCustomMenu forMorph: aMorph hand: aHandMorph
	aCustomMenu addLine.
	aCustomMenu add: 'whose script is this?' translated target: self action: #identifyScript
	! !

!MethodHolder methodsFor: 'miscellaneous' stamp: 'sw 3/28/2002 00:36'!
changeMethodSelectorTo: aSelector
	"Change my method selector as noted.  Reset currentCompiledMethod"

	methodSelector := aSelector.
	currentCompiledMethod := methodClass compiledMethodAt: aSelector ifAbsent: [nil]! !

!MethodHolder methodsFor: 'miscellaneous' stamp: 'tk 8/30/2000 13:07'!
compiledMethod

	^ methodClass compiledMethodAt: methodSelector! !

!MethodHolder methodsFor: 'contents' stamp: 'md 2/20/2006 15:05'!
contents
	"Answer the contents, with due respect for my contentsSymbol"

	contents := methodClass sourceCodeAt: methodSelector ifAbsent: [''].
	currentCompiledMethod := methodClass compiledMethodAt: methodSelector ifAbsent: [nil].

	self showingDecompile ifTrue: [^ self decompiledSourceIntoContents].
	self showingDocumentation ifTrue: [^ self commentContents].
	^ contents := self sourceStringPrettifiedAndDiffed asText makeSelectorBoldIn: methodClass! !

!MethodHolder methodsFor: 'contents' stamp: 'nice 2/23/2011 21:51'!
contents: input notifying: aController 
	| selector |
	(selector := methodClass newParser parseSelector: input asText) ifNil:
		[self inform: 'Sorry - invalid format for the 
method name and arguments -- cannot accept.'.
		^ false].

	selector == methodSelector ifFalse:
		[self inform:
'You cannot change the name of
the method here -- it must continue
to be ', methodSelector.
		^ false].

	selector := methodClass
				compile: input asText
				classified: self selectedMessageCategoryName
				notifying: aController.
	selector == nil ifTrue: [^ false].
	contents := input asString copy.
	currentCompiledMethod := methodClass compiledMethodAt: methodSelector.
	^ true! !

!MethodHolder methodsFor: 'menu' stamp: 'nice 3/31/2011 21:51'!
doItReceiver
	"If there is an instance associated with me, answer it, for true mapping of self.  If not, then do what other code-bearing tools do, viz. give access to the class vars."
	^ self selectedClass! !

!MethodHolder methodsFor: 'miscellaneous' stamp: 'sw 10/23/1999 23:01'!
identifyScript
	| msg aPlayer |
	msg := methodClass isUniClass
		ifTrue:
			[aPlayer := methodClass someInstance.
			aPlayer costume
				ifNotNil:
					['This holds code for a script
named ', methodSelector, ' belonging
to an object named ', aPlayer externalName]
				ifNil:
					['This formerly held code for a script
named ', methodSelector, ' for a Player
who once existed but now is moribund.']]
		ifFalse:
			['This holds code for the method
named ', methodSelector, '
for class ', methodClass name].
	self inform: msg! !

!MethodHolder methodsFor: 'miscellaneous' stamp: 'sw 10/21/1999 13:05'!
methodClass: aClass methodSelector: aSelector
	methodClass := aClass.
	methodSelector := aSelector.
	currentCompiledMethod := aClass compiledMethodAt: aSelector ifAbsent: [nil]! !

!MethodHolder methodsFor: 'selection' stamp: 'sw 10/21/1999 09:40'!
selectedClass
	^ methodClass theNonMetaClass! !

!MethodHolder methodsFor: 'selection' stamp: 'sw 10/27/1999 14:30'!
selectedClassOrMetaClass
	^ methodClass! !

!MethodHolder methodsFor: 'selection' stamp: 'sw 10/21/1999 10:16'!
selectedMessageCategoryName
	^ methodClass organization categoryOfElement: methodSelector! !

!MethodHolder methodsFor: 'selection' stamp: 'sw 10/21/1999 09:39'!
selectedMessageName
	^ methodSelector! !

!MethodHolder methodsFor: 'miscellaneous' stamp: 'tk 8/30/2000 13:08'!
versions
	"Return a VersionsBrowser (containing a list of ChangeRecords) of older versions of this method."

	^ VersionsBrowser new scanVersionsOf: self compiledMethod
			class: self selectedClass 
			meta: methodClass isMeta 
			category: self selectedMessageCategoryName
				"(classOfMethod whichCategoryIncludesSelector: selectorOfMethod)"
			selector: methodSelector! !

StringHolder subclass: #FileList
	instanceVariableNames: 'fileName directory volList volListIndex list listIndex pattern sortMode brevityState directoryCache lastGraphicsExtent'
	classVariableNames: 'FileReaderRegistry RecentDirs'
	poolDictionaries: ''
	category: 'Tools-FileList'!

!FileList commentStamp: 'nk 11/26/2002 11:52' prior: 0!
I am model that can be used to navigate the host file system. By omitting the volume list, file list, and template panes from the view, I can also be used as the model for an editor on an individual file.

The FileList now provides a registration mechanism to which any tools the filelist uses ***MUST*** register.  This way it is possible to dynamically load or unload a new tool and have the FileList automatically updated.  This change supports a decomposition of Squeak and removes a problem with dead reference to classes after a major shrink.

Tools should implement the following methods (look for implementors in the image):

#fileReaderServicesForFile:suffix: (appropriate services for given file, takes a file name and a lowercased suffix)

#services (all provided services, to be displayed in full list)

These methods both return a collection of SimpleServiceEntry instances.  These contain a class, a menu label and a method selector having one argument.  They may also provide separate button labels and description.

The argument to the specified method will be a string representing the full name of a file when one is selected or the file list itself when there is no selected file.

Tools must register with the FileList calling the class method #registerFileReader: when they load. They also must call #unregisterFileReader: when they unload.

There is a testSuite called FileListTest that presents some examples. 

Stef (I do not like really this distinction passing always a file list could be better)


Old Comments: 


FileLists can now see FTP servers anywhere on the net.  In the volume list menu: 
fill in server info...		Gives you a form to register a new ftp server you want to use.
open server...		Choose a server to connect to.
local disk			Go back to looking at your local volume.


Still undone (you can contribute code):
[ ] Using a Proxy server to get out through a firewall.  What is the convention for proxy servers with FTP?
[ ] Fill in the date and size info in the list of remote files.  Allow sorting by it.  New smarts needed in (ServerDirectory fileNameFormattedFrom:sizePad:sortMode:).
[ ] Currently the FileList has no way to delete a directory.  Since you can't select a directory without going into it, it would have to be deleting the current directory.  Which would usually be empty.!

FileList subclass: #FileList2
	instanceVariableNames: 'showDirsInFileList currentDirectorySelected fileSelectionBlock dirSelectionBlock optionalButtonSpecs modalView directoryChangeBlock ok'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-FileList'!

FileList2 class
	instanceVariableNames: 'lastSelDir'!

!FileList2 commentStamp: 'BJP 11/19/2003 21:13' prior: 0!
Some variations on FileList that
- use a hierarchical pane to show folder structure
- use different pane combinations, button layouts and prefiltering for specific uses

FileList2 morphicView openInWorld				"an alternative to the standard FileList"
FileList2 morphicViewNoFile openInWorld			"useful for selecting, but not viewing"
FileList2 morphicViewProjectLoader openInWorld	"useful for finding and loading projects"
FileList2 modalFolderSelector						"allows the user to select a folder"



!

FileList2 class
	instanceVariableNames: 'lastSelDir'!

FileList2 subclass: #FileChooser
	instanceVariableNames: 'view caption captionMorph captionBox cancelButton okButton buttonPane captionPane directoryPane filePane showShortFileNames'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-FileList'!

!FileChooser commentStamp: 'miki 8/15/2005 12:07' prior: 0!
This class provides a simple "modal" dialog box to choose a file, with a directory tree, a file list, and open and cancel buttons. It is meant to be an improvement of FileList2 modalFileSelector. 

An applictaion can customize the user interface in a straightforward way. Creation of the file choser is done in several steps by calling various helper methods.. 

The order of the method calls when creating a customized file chooser are important. The UI must be created before methods that change the attributes of the UI can be called. You can either start by creating the default UI, and them modify the morphs in the file chooser (there are methods to access the buttons and the panes).

You can also build a completely custom UI, by writing your own methods for creating the layout etc. One way to do this is to subclass FileChooser and override the methods you want to change, andother way is to supply your own morphic view to the file chooser. This must be an instance of MorphicModel or a subclass of it, because the file chooser uses the model functionality.

There are two varieties of the UI, one that is supposed to be like a dialog box (uses colors from the menu preferences in class Preference), and one is using a system window. The way a system window works turns out to be somehat different from how a plain Morphic Model works, and this is why there are separate methods for creating the dialog box UI and the system window UI.

On the class side, there are examples that shows differents ways to use this class.

On the to do list is adding support for a file save dialog box, with a directory tree and a text input field for typing a file name.

(Mikael Kindborg, 050815)
!

!FileChooser class methodsFor: 'examples' stamp: 'dtl 2/1/2010 19:46'!
example1
	"Open file chooser with the standard dialog box UI."
	"FileChooser example1"
	| fc stream |
	fc := FileChooser new.
	fc initializeAsDialogBox.
	stream := fc open.
	stream inspect.! !

!FileChooser class methodsFor: 'examples' stamp: 'dtl 2/1/2010 19:46'!
example2
	"Open file chooser with a system window UI."
	"FileChooser example2"
	| fc stream |
	fc := FileChooser new.
	fc initializeAsSystemWindow.
	stream := fc open.
	stream inspect.! !

!FileChooser class methodsFor: 'examples' stamp: 'dtl 2/1/2010 19:47'!
example3
	"Open file chooser with a system window UI that has a caption pane and shows only picture files."
	"FileChooser example3"
	| fc stream |
	fc := FileChooser new.
	fc initializeAsSystemWindowWithCaptionPane.
	fc setCaption: 'Select a picture file' translated.
	fc setSuffixes: {'png' . 'gif' . 'bmp' . 'jpg' . 'jpeg' }.
	stream := fc open.
	stream ifNotNil: [(Form fromBinaryStream: stream) asMorph openInHand].! !

!FileChooser class methodsFor: 'examples' stamp: 'dtl 2/1/2010 19:46'!
example4
	"Open file chooser with a customized dialog box UI. The order of the messages is important. In general, call the initialize method first, then modify things, and finally call open."
	"FileChooser example4"
	| fc stream |
	fc := FileChooser new.
	fc initializeAsDialogBox.
	fc setDirectory: FileDirectory root.
	fc setSuffixes: {'png' . 'gif' . 'bmp' . 'jpg' . 'jpeg' }.
	fc setCaption: 'Select a picture file' translated.
	fc morphicView 
		borderColor: Color black; 
		borderWidth: 2;
		color: Color white.
	fc setPaneColor: Color gray muchLighter.
	fc captionPane color: Color orange muchLighter.
	fc okButton color: Color green muchLighter.
	fc cancelButton color: Color blue muchLighter.
	fc morphicView position: 20@20.
	stream := fc open.
	stream ifNotNil: [(Form fromBinaryStream: stream) asMorph openInHand].! !

!FileChooser class methodsFor: 'instance creation' stamp: 'dtl 2/1/2010 19:49'!
new
	"Superclass may attempt to initialize to a specific default directory. Bypass
	that behavior."

	^ self basicNew initialize! !

!FileChooser methodsFor: 'ui creation' stamp: 'nice 12/26/2009 10:05'!
addFullPanesTo: aMorph from: aCollection
	aCollection do: [ :each |
		| frame |
		frame := LayoutFrame 
			fractions: each second 
			offsets: each third.
		aMorph addMorph: each first fullFrame: frame.
	]! !

!FileChooser methodsFor: 'accessing' stamp: 'miki 8/15/2005 11:26'!
buttonPane
	^buttonPane! !

!FileChooser methodsFor: 'accessing' stamp: 'miki 8/14/2005 22:48'!
cancelButton
	^cancelButton! !

!FileChooser methodsFor: 'accessing' stamp: 'miki 8/13/2005 17:52'!
caption
	^caption! !

!FileChooser methodsFor: 'accessing' stamp: 'miki 8/15/2005 11:26'!
captionPane
	^captionPane! !

!FileChooser methodsFor: 'ui creation' stamp: 'miki 8/15/2005 18:54'!
centerMorphicView
	self morphicView
		fullBounds;
		position: Display extent - self morphicView extent // 2.
! !

!FileChooser methodsFor: 'ui creation' stamp: 'sd 11/20/2005 21:27'!
createCancelButton
	cancelButton := SimpleButtonMorph new.
	cancelButton
		label: 'Cancel' translated;
		color: Color transparent;
		borderColor: Color black;
		borderWidth: 1.
	cancelButton 
		on: #mouseUp 
		send: #cancelHit
		to: self.
	^cancelButton
! !

!FileChooser methodsFor: 'ui creation' stamp: 'miki 8/15/2005 15:05'!
createDialogBoxButtonPane
	"Create buttons suitable for a MorphicModel file chooser."

	buttonPane := AlignmentMorph new.
	buttonPane
		layoutPolicy: ProportionalLayout new;
		color: Color transparent;
		borderWidth: 0.
	self createOkButton.
	self createCancelButton.
	buttonPane addMorph: self cancelButton
		fullFrame: (LayoutFrame fractions: (0 @ 0 corner: 0.49 @ 1.0)
				offsets: (0 @ 0 corner: 0 @ 0)).
	buttonPane addMorph: self okButton
		fullFrame: (LayoutFrame fractions: (0.51 @ 0 corner: 1.0 @ 1.0)
				offsets: (0 @ 0 corner: 0 @ 0)).
	^buttonPane! !

!FileChooser methodsFor: 'ui creation' stamp: 'miki 8/15/2005 18:45'!
createDialogBoxCaptionPane
	"Create a morph to hold the caption string. The caption is created in setCaption:"

	| icon frame |
	captionPane := AlignmentMorph new.
	captionPane
		color: Color transparent;
		layoutPolicy: ProportionalLayout new.

	"You can change the caption later by calling setCaption:"
	self setCaption: 'Please select a file' translated.
	self setCaptionFont: Preferences windowTitleFont.
	icon := SketchMorph new.
	icon form: MenuIcons openIcon.
	captionPane addMorph: icon.
	frame := LayoutFrame new.
	frame
		leftFraction: 0;
		topFraction: 0.5;
		leftOffset: icon form width // 2;
		topOffset: (icon form width // 2) negated.
	icon layoutFrame: frame.
	^captionPane! !

!FileChooser methodsFor: 'ui creation' stamp: 'miki 8/15/2005 12:13'!
createDialogBoxLayout
	"Create a layout suitable for a MorphicModel file chooser."

	| inset insetNeg captionTop captionBottom buttonsBottom buttonsTop contentTop contentBottom |

	inset := 6.
	insetNeg := inset negated.
	captionTop := 0.
	captionBottom := 33.
	contentTop := captionBottom + inset.
	contentBottom := -30 - inset - inset.
	buttonsTop := contentBottom + inset.
	buttonsBottom := insetNeg.

	self addFullPanesTo: self morphicView
		from: {
				{
					(self captionPane).
					(0 @ 0 corner: 1 @ 0).
					(0 @ captionTop corner: 0 @ captionBottom)
				}.
				{
					(self buttonPane).
					(0 @ 1 corner: 1 @ 1).
					(inset @ buttonsTop corner: insetNeg @ buttonsBottom)
				}.
				{
					(self directoryPane).
					(0 @ 0 corner: 0.5 @ 1).
					(inset @ contentTop corner: insetNeg @ contentBottom)
				}.
				{
					(self filePane).
					(0.5 @ 0 corner: 1 @ 1).
					(inset @ contentTop corner: insetNeg @ contentBottom)
				}
			}! !

!FileChooser methodsFor: 'ui creation' stamp: 'sd 11/20/2005 21:27'!
createDialogBoxMorphicView
	| m |
	m := MorphicModel new
		layoutPolicy: ProportionalLayout new;
		color: Preferences menuColor;
		borderColor: Preferences menuBorderColor;
		borderWidth: Preferences menuBorderWidth;
		layoutInset: 0;
		extent: 600@400.
	self setMorphicView: m.
	^m! !

!FileChooser methodsFor: 'ui creation' stamp: 'miki 8/15/2005 19:02'!
createDialogBoxUI
	"This method creates UI components and a layout that are suitable for a MorphicModel. Also centers the morphic view in the world. Note that the order of the method calls are important if you modify this."

	self
		createDialogBoxMorphicView;
		createDialogBoxCaptionPane;
		createDialogBoxButtonPane;
		createDirectoryPane;
		createFilePane;
		createDialogBoxLayout;
		centerMorphicView.
	^self morphicView! !

!FileChooser methodsFor: 'ui creation' stamp: 'miki 8/15/2005 11:27'!
createDirectoryPane
	directoryPane := self morphicDirectoryTreePane.
	directoryPane borderWidth: 0.
	^directoryPane! !

!FileChooser methodsFor: 'ui creation' stamp: 'miki 8/15/2005 11:27'!
createFilePane
	filePane := self morphicFileListPane.
	filePane borderWidth: 0.
	^filePane! !

!FileChooser methodsFor: 'ui creation' stamp: 'sd 11/20/2005 21:27'!
createOkButton
	okButton := SimpleButtonMorph new.
	okButton 
		label: 'Open' translated;
		color: Color transparent;
		borderColor: Color black;
		borderWidth: 1.
	okButton 
		on: #mouseUp 
		send: #okHit
		to: self.
	^okButton! !

!FileChooser methodsFor: 'ui creation' stamp: 'miki 8/15/2005 11:26'!
createSystemWindowButtonPane
	"Create buttons suitable for a SystemWindow file chooser."

	self optionalButtonSpecs: self okayAndCancelServices.
	buttonPane := self optionalButtonRow.
	okButton := buttonPane firstSubmorph.
	cancelButton := buttonPane firstSubmorph.
	^buttonPane! !

!FileChooser methodsFor: 'ui creation' stamp: 'miki 8/15/2005 11:26'!
createSystemWindowCaptionPane
	"Create a morph to hold the caption string. The caption is created in setCaption:"

	captionPane := AlignmentMorph new.
	captionPane
		color: Color transparent;
		layoutPolicy: ProportionalLayout new.
	"You can change the caption later by calling setCaption:"
	self setCaption: 'Please select a file' translated.
	^captionPane! !

!FileChooser methodsFor: 'ui creation' stamp: 'miki 8/15/2005 12:12'!
createSystemWindowLayout
	"Create a layout suitable for a SystemWindow file chooser."

	| buttonsHeight |

	buttonsHeight := 33.

	self addFullPanesTo: self morphicView
		from: {
				{
					(self buttonPane).
					(0 @ 0 corner: 1 @ 0).
					(0 @ 0 corner: 0 @ buttonsHeight)
				}.
				{
					(self directoryPane).
					(0 @ 0 corner: 0.5 @ 1).
					(0 @ buttonsHeight corner: 0 @ 0)
				}.
				{
					(self filePane).
					(0.5 @ 0 corner: 1 @ 1).
					(0 @ buttonsHeight corner: 0 @ 0)
				}
			}! !

!FileChooser methodsFor: 'ui creation' stamp: 'miki 8/15/2005 12:18'!
createSystemWindowLayoutWithCaptionPane

	| buttonsHeight captionHeight |

	buttonsHeight := 33.
	captionHeight := 28.

	self addFullPanesTo: self morphicView
		from: {
				{
					(self captionPane). 
					(0 @ 0 corner: 1 @ 0). 
					(0 @ 0 corner: 0 @ captionHeight)
				}.
				{
					(self buttonPane).
					(0 @ 0 corner: 1 @ 0).
					(0 @ captionHeight corner: 0 @ (captionHeight + buttonsHeight))
				}.
				{
					(self directoryPane).
					(0 @ 0 corner: 0.5 @ 1).
					(0 @ (captionHeight + buttonsHeight) corner: 0 @ 0)
				}.
				{
					(self filePane).
					(0.5 @ 0 corner: 1 @ 1).
					(0 @ (captionHeight + buttonsHeight) corner: 0 @ 0)
				}
			}! !

!FileChooser methodsFor: 'ui creation' stamp: 'sd 11/20/2005 21:27'!
createSystemWindowMorphicView
	| m |
	m := SystemWindow labelled: 'Please select a file' translated. "self directory pathName."
	"m deleteCloseBox."
	self setMorphicView: m.! !

!FileChooser methodsFor: 'ui creation' stamp: 'miki 8/15/2005 11:40'!
createSystemWindowUI
	"This method creates UI components and a layout that are suitable for a SystemWindow. Note that the order of the method calls are important."

	self
		createSystemWindowMorphicView;
		createSystemWindowButtonPane;
		createDirectoryPane;
		createFilePane;
		createSystemWindowLayout.
	^self morphicView! !

!FileChooser methodsFor: 'ui creation' stamp: 'miki 8/15/2005 11:40'!
createSystemWindowUIWithCaptionPane
	self
		createSystemWindowMorphicView;
		createSystemWindowCaptionPane;
		createSystemWindowButtonPane;
		createDirectoryPane;
		createFilePane;
		createSystemWindowLayoutWithCaptionPane.
	^self morphicView! !

!FileChooser methodsFor: 'accessing' stamp: 'miki 8/14/2005 22:55'!
directory
	^super directory! !

!FileChooser methodsFor: 'accessing' stamp: 'miki 8/15/2005 11:27'!
directoryPane
	^directoryPane! !

!FileChooser methodsFor: 'updating' stamp: 'laza 3/8/2010 12:02'!
fileNameFormattedFrom: entry sizePad: sizePad 
	"If the short file list flag is false, we send this on to the superclass."

	| nameStr |
	showShortFileNames 
		ifFalse: [^super fileNameFormattedFrom: entry sizePad: sizePad].

	"Otherwise, just show the name of the file in the file list."
	nameStr := (entry isDirectory)
					ifTrue: [entry name, self folderString]
					ifFalse: [entry name].
	^nameStr! !

!FileChooser methodsFor: 'accessing' stamp: 'miki 8/15/2005 11:27'!
filePane
	^filePane! !

!FileChooser methodsFor: 'initialization' stamp: 'dtl 2/1/2010 19:47'!
initializeAsDialogBox
	self initializeBasicParameters.
	self createDialogBoxUI.
	self morphicView
		useRoundedCorners;
		color: Preferences menuColor;
		adoptPaneColor: Preferences menuLineColor.
	self 
		setCaptionColor: Preferences menuTitleColor;
		setButtonColor: Preferences menuColor.! !

!FileChooser methodsFor: 'initialization' stamp: 'dtl 2/1/2010 19:47'!
initializeAsSystemWindow
	self initializeBasicParameters.
	self createSystemWindowUI.! !

!FileChooser methodsFor: 'initialization' stamp: 'dtl 2/1/2010 19:47'!
initializeAsSystemWindowWithCaptionPane
	self initializeBasicParameters.
	self createSystemWindowUIWithCaptionPane.! !

!FileChooser methodsFor: 'initialization' stamp: 'dtl 2/1/2010 19:45'!
initializeBasicParameters
	self showShortFileNames: true.
	self setDirectory: FileDirectory default.! !

!FileChooser methodsFor: 'accessing' stamp: 'miki 8/13/2005 17:57'!
morphicView
	^view! !

!FileChooser methodsFor: 'accessing' stamp: 'miki 8/14/2005 22:48'!
okButton
	^okButton! !

!FileChooser methodsFor: 'open' stamp: 'mt 8/11/2016 11:55'!
open
	| model |
	self postOpen. "Funny name in this context, should be renamed, but whatever..."
	self morphicView openInWorld.
	UserInterfaceTheme current applyTo: self morphicView allMorphs.
	model := self morphicView model.
	FileChooser modalLoopOn: self morphicView.
	^ model getSelectedFile.
! !

!FileChooser methodsFor: 'ui creation' stamp: 'miki 8/15/2005 01:12'!
setButtonColor: aColor
	self okButton  color: aColor.
	self cancelButton  color: aColor.
! !

!FileChooser methodsFor: 'ui creation' stamp: 'miki 8/15/2005 11:23'!
setCaptionColor: aColor 
	self captionPane color: aColor! !

!FileChooser methodsFor: 'ui creation' stamp: 'miki 8/15/2005 14:55'!
setCaptionFont: aFont
	self caption font: aFont.
	self setCaption: self caption contents asString.

! !

!FileChooser methodsFor: 'ui creation' stamp: 'miki 8/15/2005 11:23'!
setCaption: aString 
	| frame |
	caption ifNil: 
			[caption := StringMorph new.
			self captionPane addMorph: caption].
	caption contents: aString.
	frame := LayoutFrame new.
	frame
		leftFraction: 0.5;
		topFraction: 0.5;
		leftOffset: caption width negated // 2;
		topOffset: caption height negated // 2.
	caption layoutFrame: frame! !

!FileChooser methodsFor: 'initialization' stamp: 'miki 8/14/2005 23:12'!
setDirectory: aDir
	^super directory: aDir! !

!FileChooser methodsFor: 'initialization' stamp: 'sd 11/20/2005 21:27'!
setMorphicView: aMorphicModel
	view := aMorphicModel.
	self modalView: view.
	view model: self.! !

!FileChooser methodsFor: 'ui creation' stamp: 'miki 8/15/2005 00:41'!
setPaneColor: aColor
	self morphicView 
		color: aColor;
		adoptPaneColor: aColor.

! !

!FileChooser methodsFor: 'initialization' stamp: 'nice 1/11/2010 20:55'!
setSuffixes: aList
	self fileSelectionBlock:  [:entry :myPattern |
			entry isDirectory
				ifTrue:
					[false]
				ifFalse:
					[aList includes: (FileDirectory extensionFor: entry name asLowercase)]]! !

!FileChooser methodsFor: 'initialization' stamp: 'miki 8/15/2005 11:32'!
showShortFileNames: aBoolean 
	showShortFileNames := aBoolean! !

!FileChooser methodsFor: 'updating' stamp: 'miki 8/14/2005 23:04'!
updateButtonRow
	"Prevent updating of the the button row."! !

!FileList2 class methodsFor: 'utility' stamp: 'nice 12/27/2009 03:11'!
addFullPanesTo: window from: aCollection

	

	aCollection do: [ :each | | frame |
		frame := LayoutFrame 
			fractions: each second 
			offsets: each third.
		window addMorph: each first fullFrame: frame.
	]! !

!FileList2 class methodsFor: 'blue ui' stamp: 'dgd 11/3/2004 20:09'!
blueButtonText: aString textColor: textColor color: aColor inWindow: window 
	| result |
	result := window
				fancyText: aString translated
				font: Preferences standardEToysFont
				color: textColor.
	result setProperty: #buttonText toValue: aString;
		 hResizing: #rigid;
		 extent: 100 @ 20;
		 layoutInset: 4;
		 borderWidth: ColorTheme current dialogButtonBorderWidth;
		 useRoundedCorners.
	aColor isNil
		ifFalse: [""result color: aColor. result borderColor: aColor muchDarker].
	^ result! !

!FileList2 class methodsFor: 'blue ui' stamp: 'dgd 11/3/2004 20:13'!
blueButtonText: aString textColor: textColor color: aColor inWindow: window balloonText: balloonText selector: sel recipient: recip 
	| result |
	result := window
				fancyText: aString translated
font: Preferences standardEToysFont
				color: textColor.
	result setProperty: #buttonText toValue: aString;
		 hResizing: #rigid;
		 extent: 100 @ 20;
		 layoutInset: 4;
		 borderWidth: ColorTheme current dialogButtonBorderWidth;
		 useRoundedCorners;
		 setBalloonText: balloonText.
	result
		on: #mouseUp
		send: sel
		to: recip.
	aColor isNil
		ifFalse: [""
			result color: aColor.
			result borderColor: aColor muchDarker].
	^ result! !

!FileList2 class methodsFor: 'blue ui' stamp: 'dgd 11/2/2004 21:43'!
blueButtonText: aString textColor: textColor inWindow: window 
	^ self
		blueButtonText: aString
		textColor: textColor
		color: nil
		inWindow: window! !

!FileList2 class methodsFor: 'blue ui' stamp: 'dgd 11/3/2004 20:04'!
blueButtonText: aString textColor: textColor inWindow: window balloonText: balloonText selector: sel recipient: recip 
	^ self
		blueButtonText: aString
		textColor: textColor
		color: nil
		inWindow: window
		balloonText: balloonText
		selector: sel
		recipient: recip ! !

!FileList2 class methodsFor: 'blue ui' stamp: 'nice 12/27/2009 03:11'!
enableTypeButtons: typeButtons info: fileTypeInfo forDir: aDirectory

	| foundSuffixes firstEnabled |

	firstEnabled := nil.
	foundSuffixes := (aDirectory ifNil: [ #()] ifNotNil: [ aDirectory fileNames]) collect: [ :each | (each findTokens: '.') last asLowercase].
	foundSuffixes := foundSuffixes asSet.
	fileTypeInfo with: typeButtons do: [ :info :button | | enableIt fileSuffixes |
		fileSuffixes := info second.
		enableIt := fileSuffixes anySatisfy: [ :patt | foundSuffixes includes: patt].
		button 
			setProperty: #enabled 
			toValue: enableIt.
		enableIt ifTrue: [firstEnabled ifNil: [firstEnabled := button]].
	].
	firstEnabled ifNotNil: [^firstEnabled mouseUp: nil].
	typeButtons do: [ :each | each color: Color gray].

! !

!FileList2 class methodsFor: 'blue ui' stamp: 'tfel 7/28/2016 10:17'!
endingSpecs
	"Answer a collection of specs to build the selective 'find anything' tool called by the Navigator. This version uses the services registry to do so."
	"FileList2 morphicViewGeneralLoaderInWorld: World"
	| categories specs rejects |
	rejects := #(addFileToNewZip: compressFile: openInZipViewer: extractAllFrom: openOn:).
	categories := #(
		('Art' ('bmp' 'gif' 'jpg' 'jpeg' 'form' 'png' 'pcx' 'xbm' 'xpm' 'ppm' 'pbm'))
		('Morphs' ('morph' 'morphs' 'sp'))
		('Projects' ('extseg' 'project' 'pr'))
		('MIDI' ('mid' 'midi'))
		('Music' ('mp3'))
		('Movies' ('movie' 'mpg' 'mpeg' 'qt' 'mov'))
		('Flash' ('swf'))
	).

		"('Books' ('bo'))"
		"('Code' ('st' 'cs'))"
		"('TrueType' ('ttf'))"
		"('3ds' ('3ds'))"
		"('Tape' ('tape'))"
		"('Wonderland' ('wrl'))"
		"('HTML' ('htm' 'html'))"

	categories first at: 2 put: ImageReadWriter allTypicalFileExtensions.
	specs := OrderedCollection new.
	categories do: [ :cat | | catSpecs catServices okExtensions services |
		services := Dictionary new.
		catSpecs := Array new: 3.
		catServices := OrderedCollection new.
		okExtensions := Set new.

		cat second do: [ :ext | (FileList itemsForFile: (FileDirectory default fullPathFor: ('fred.',ext))) do: [ :i |
			(rejects includes: i selector) ifFalse: [
				okExtensions add: ext.
				services at: i label put: i ]]].
		services do: [ :svc | catServices add: svc ].
		services isEmpty ifFalse: [ 
			catSpecs at: 1 put: cat first;
				at: 2 put: okExtensions;
				at: 3 put: catServices.
			specs add: catSpecs ]
	].
	^specs
! !

!FileList2 class methodsFor: 'as yet unclassified' stamp: 'ar 10/10/2000 15:59'!
hideSqueakletDirectoryBlock
	^[:dirName| (dirName sameAs: 'Squeaklets') not]! !

!FileList2 class methodsFor: 'accessing' stamp: 'ul 2/22/2010 22:32'!
lastSelDir
	"Return the last selected directory or the default directory if no directory was selected so far."

	^lastSelDir ifNil: [ lastSelDir := FileDirectory default ]! !

!FileList2 class methodsFor: 'accessing' stamp: 'ul 2/22/2010 22:34'!
lastSelDir: aFileDirectory
	"Store the last selected directory. This will be selected as default in newly opened file or folder selectors"
	
	^lastSelDir := aFileDirectory! !

!FileList2 class methodsFor: 'modal dialogs' stamp: 'mt 8/11/2016 11:59'!
modalFileSelector

	| window |

	window := self morphicViewFileSelector.
	window openCenteredInWorld.
	UserInterfaceTheme current applyTo: window allMorphs.
	self modalLoopOn: window.
	^(window valueOfProperty: #fileListModel) getSelectedFile! !

!FileList2 class methodsFor: 'modal dialogs' stamp: 'tfel 8/10/2016 15:39'!
modalFileSelectorForSuffixes: aList

	| window aFileList |

	window := self morphicViewFileSelectorForSuffixes: aList.
	aFileList := window valueOfProperty: #fileListModel.
	aFileList resort: #name.
	window openCenteredInWorld.
	self modalLoopOn: window.
	^aFileList getSelectedFile! !

!FileList2 class methodsFor: 'modal dialogs' stamp: 'mt 8/11/2016 11:59'!
modalFileSelectorForSuffixes: aList directory: aDirectory

	| window aFileList |

	window := self morphicViewFileSelectorForSuffixes: aList directory: aDirectory.
	aFileList := window valueOfProperty: #fileListModel.
	window openCenteredInWorld.
	UserInterfaceTheme current applyTo: window allMorphs.
	self modalLoopOn: window.
	^aFileList getSelectedFile! !

!FileList2 class methodsFor: 'modal dialogs' stamp: 'ul 2/22/2010 22:21'!
modalFolderSelector

	^self modalFolderSelector: self lastSelDir
	! !

!FileList2 class methodsFor: 'modal dialogs' stamp: 'mt 8/11/2016 12:00'!
modalFolderSelectorForProjectLoad

	| window fileModel w |

	window := self morphicViewProjectLoader2InWorld: self currentWorld reallyLoad: false.
	fileModel := window valueOfProperty: #FileList.
	w := self currentWorld.
	window position: w topLeft + (w extent - window extent // 2).
	window openInWorld: w.
	UserInterfaceTheme current applyTo: window allMorphs.
	self modalLoopOn: window.
	^fileModel getSelectedDirectory withoutListWrapper! !

!FileList2 class methodsFor: 'modal dialogs' stamp: 'mt 8/11/2016 12:00'!
modalFolderSelectorForProject: aProject
"
FileList2 modalFolderSelectorForProject: Project current
"
	| window fileModel w |

	window := FileList2 morphicViewProjectSaverFor: aProject.
	fileModel := window valueOfProperty: #FileList.
	w := self currentWorld.
	window position: w topLeft + (w extent - window extent // 2).
	w addMorphInLayer: window.
	w startSteppingSubmorphsOf: window.
	UserInterfaceTheme current applyTo: window allMorphs.
	self modalLoopOn: window.
	^fileModel getSelectedDirectory withoutListWrapper! !

!FileList2 class methodsFor: 'modal dialogs' stamp: 'mt 8/11/2016 11:58'!
modalFolderSelector: aDir

	| window fileModel |
	window := self morphicViewFolderSelector: aDir.
	fileModel := window model.
	window openInWorld: self currentWorld extent: 300@400.
	UserInterfaceTheme current applyTo: window allMorphs.
	self modalLoopOn: window.
	^fileModel getSelectedDirectory withoutListWrapper! !

!FileList2 class methodsFor: 'utility' stamp: 'miki 8/15/2005 18:34'!
modalLoopOn: aMorph
	[aMorph world notNil] whileTrue: [
		aMorph outermostWorldMorph doOneCycle.
	].! !

!FileList2 class methodsFor: 'as yet unclassified' stamp: 'btr 1/30/2004 00:56'!
morphicView
	^ self morphicViewOnDirectory: FileDirectory default! !

!FileList2 class methodsFor: 'morphic ui' stamp: 'RAA 3/6/2001 12:47'!
morphicViewFileSelector

	^self morphicViewFileSelectorForSuffixes: nil
! !

!FileList2 class methodsFor: 'morphic ui' stamp: 'ul 2/22/2010 22:22'!
morphicViewFileSelectorForSuffixes: aList 
	"Answer a morphic file-selector tool for the given suffix list."
	
	^self morphicViewFileSelectorForSuffixes: aList directory: self lastSelDir! !

!FileList2 class methodsFor: 'morphic ui' stamp: 'nice 2/11/2010 14:37'!
morphicViewFileSelectorForSuffixes: aList directory: dir
	"Answer a morphic file-selector tool for the given suffix list and the given directory."

	| aFileList window fixedSize midLine gap |
	aFileList := self new directory: dir.
	aFileList optionalButtonSpecs: aFileList okayAndCancelServices.
	aList ifNotNil:
		[aFileList fileSelectionBlock: [:entry :myPattern |
			entry isDirectory
				ifTrue:
					[false]
				ifFalse:
					[aList includes: (FileDirectory extensionFor: entry name asLowercase)]]].
	window := BorderedMorph new
		layoutPolicy: ProportionalLayout new;
		color: Color lightBlue;
		borderColor: Color blue;
		borderWidth: 4;
		layoutInset: 4;
		extent: 600@400;
		useRoundedCorners.
	window setProperty: #fileListModel toValue: aFileList.
	aFileList modalView: window.
	midLine := 0.4.
	fixedSize := 25.
	gap := 5.
	self addFullPanesTo: window from: {
		{self textRow: 'Please select a file'. 0 @ 0 corner: 1 @ 0. 0@0 corner: 0@fixedSize}.
		{aFileList optionalButtonRow. 0 @ 0 corner: 1 @ 0. 0@fixedSize corner: 0@(fixedSize * 2)}.
		{aFileList morphicDirectoryTreePane. 0@0 corner: midLine@1. 
					gap @(fixedSize * 2) corner: gap negated@0}.
		{aFileList morphicFileListPane. midLine @ 0 corner: 1@1. 
					gap@(fixedSize * 2) corner: gap negated@0}.
	}.

	aFileList postOpen.

	^ window ! !

!FileList2 class methodsFor: 'morphic ui' stamp: 'gh 9/16/2002 10:30'!
morphicViewFolderSelector

	^self morphicViewFolderSelector: FileDirectory default! !

!FileList2 class methodsFor: 'morphic ui' stamp: 'bkv 11/12/2002 16:55'!
morphicViewFolderSelector: aDir
	"Answer a tool that allows the user to select a folder"

	| aFileList window fixedSize |
	aFileList := self new directory: aDir.
	aFileList optionalButtonSpecs: aFileList servicesForFolderSelector.
	window := (SystemWindow labelled: aDir pathName) model: aFileList.
	aFileList modalView: window.

	fixedSize := 25.
	self addFullPanesTo: window from: {
		{self textRow: 'Please select a folder'. 0 @ 0 corner: 1 @ 0. 
				0@0 corner: 0@fixedSize}.
		{aFileList optionalButtonRow. 0 @ 0 corner: 1 @ 0. 
				0@fixedSize corner: 0@(fixedSize * 2)}.
		{aFileList morphicDirectoryTreePane. 0@0 corner: 1@1.
				0@(fixedSize * 2) corner: 0@0}.
	}.
	aFileList postOpen.
	^ window ! !

!FileList2 class methodsFor: 'blue ui' stamp: 'nice 2/11/2010 14:37'!
morphicViewGeneralLoaderInWorld: aWorld
"
FileList2 morphicViewGeneralLoaderInWorld: self currentWorld
"
	| window aFileList buttons treePane textColor1 fileListPane pane2a pane2b fileTypeInfo fileTypeButtons fileTypeRow actionRow |

	fileTypeInfo := self endingSpecs.
	window := AlignmentMorphBob1 newColumn.
	window hResizing: #shrinkWrap; vResizing: #shrinkWrap.
	textColor1 := Color r: 0.742 g: 0.839 b: 1.0.
	aFileList := self new directory: FileDirectory default.
	aFileList 
		fileSelectionBlock: self projectOnlySelectionBlock;
		modalView: window.
	window
		setProperty: #FileList toValue: aFileList;
		wrapCentering: #center; cellPositioning: #topCenter;
		borderWidth: ColorTheme current dialogBorderWidth;
		borderColor: ColorTheme current dialogBorderColor;
		useRoundedCorners.

	fileTypeButtons := fileTypeInfo collect: [ :each |
		(self blueButtonText: each first textColor: Color gray inWindow: window)
			setProperty: #enabled toValue: true;
			hResizing: #shrinkWrap;
			useSquareCorners
	].
	buttons := {{'OK'. ColorTheme current okColor}. {'Cancel'. ColorTheme current cancelColor}} collect: [ :each |
		self blueButtonText: each first textColor: textColor1 color: each second inWindow: window
	].

	treePane := aFileList morphicDirectoryTreePane 
		extent: 250@300; 
		retractable: false;
		borderWidth: 0.
	fileListPane := aFileList morphicFileListPane 
		extent: 350@300; 
		retractable: false;
		borderWidth: 0.
	window addARow: {window fancyText: 'Find...' translated font: Preferences standardEToysTitleFont color: textColor1}.
	fileTypeRow := window addARowCentered: fileTypeButtons cellInset: 2.
	actionRow := window addARowCentered: {
		buttons first. 
		(Morph new extent: 30@5) color: Color transparent. 
		buttons second
	} cellInset: 2.
	window
		addARow: {
				(window inAColumn: {(pane2a := window inARow: {window inAColumn: {treePane}}) 
					useRoundedCorners;
					layoutInset: 0;
					borderWidth: ColorTheme current dialogPaneBorderWidth;
					borderColor: ColorTheme current dialogPaneBorderColor
				}) layoutInset: 10.
				(window inAColumn: {(pane2b := window inARow: {window inAColumn: {fileListPane}}) 
					useRoundedCorners;
					layoutInset: 0;
					borderWidth: ColorTheme current dialogPaneBorderWidth;
					borderColor: ColorTheme current dialogPaneBorderColor
				}) layoutInset: 10.
		}.
	window fullBounds.
	window fillWithRamp: ColorTheme current dialogRampOrColor oriented: 0.65.
	pane2a fillWithRamp: ColorTheme current dialogPaneRampOrColor oriented: (0.7 @ 0.35).
	pane2b fillWithRamp: ColorTheme current dialogPaneRampOrColor oriented: (0.7 @ 0.35).
"
	buttons do: [ :each |
		each fillWithRamp: ColorTheme current dialogButtonsRampOrColor oriented: (0.75 @ 0).
	].
"
	fileTypeButtons do: [ :each | 
		each 
			on: #mouseUp 
			send: #value:value: 
			to: [ :evt :morph | 
				self update: actionRow in: window fileTypeRow: fileTypeRow morphUp: morph.
			]
	].
	buttons first on: #mouseUp send: #okHit to: aFileList.
	buttons second on: #mouseUp send: #cancelHit to: aFileList.
	aFileList postOpen.
	window position: aWorld topLeft + (aWorld extent - window extent // 2).
	aFileList directoryChangeBlock: [ :newDir |
		self update: actionRow in: window fileTypeRow: fileTypeRow morphUp: nil.
		self enableTypeButtons: fileTypeButtons info: fileTypeInfo forDir: newDir.
	].
	aFileList directory: aFileList directory.
	window adoptPaneColor: (Color r: 0.548 g: 0.677 b: 1.0).
	window becomeModal.
	^ window openInWorld: aWorld.! !

!FileList2 class methodsFor: 'morphic ui' stamp: 'nice 2/11/2010 14:37'!
morphicViewImageViewer

	| dir aFileList window midLine fixedSize |

	dir := FileDirectory default.
	aFileList := self new directory: dir.
	aFileList optionalButtonSpecs: aFileList specsForImageViewer.
	aFileList fileSelectionBlock: [ :entry :myPattern |
		entry isDirectory ifTrue: [
			false
		] ifFalse: [
			#('bmp' 'gif' 'jpg' 'form' 'png') includes: 
					 (FileDirectory extensionFor: entry name asLowercase)
		]
	].
	window := (SystemWindow labelled: dir pathName) model: aFileList.

	fixedSize := 25.
	midLine := 0.4.
	self addFullPanesTo: window from: {
		{aFileList optionalButtonRow. 0 @ 0 corner: 1 @ 0.
				0@0 corner: 0@fixedSize}.
		{aFileList morphicDirectoryTreePane. 0@0 corner: midLine@1.
				0@fixedSize corner: 0@0}.
		{aFileList morphicFileListPane. midLine @ 0 corner: 1@1.
				0@fixedSize corner: 0@0}.
	}.
	aFileList postOpen.
	^ window ! !

!FileList2 class methodsFor: 'morphic ui' stamp: 'RAA 1/8/2001 21:39'!
morphicViewNoFile

	| dir aFileList window midLine fixedSize |

	dir := FileDirectory default.
	aFileList := self new directory: dir.
	window := (SystemWindow labelled: dir pathName) model: aFileList.

	fixedSize := 25.
	midLine := 0.4.
	self addFullPanesTo: window from: {
		{aFileList morphicPatternPane. 0@0 corner: 0.3@0. 0@0 corner: 0@fixedSize}.
		{aFileList optionalButtonRow. 0.3 @ 0 corner: 1@0. 0@0 corner: 0@fixedSize}.
		{aFileList morphicDirectoryTreePane. 0@0 corner: midLine@1. 0@fixedSize corner: 0@0}.
		{aFileList morphicFileListPane. midLine @ 0 corner: 1@1. 0@fixedSize corner: 0@0}.
	}.
	aFileList postOpen.
	^ window ! !

!FileList2 class methodsFor: 'morphic ui' stamp: 'sw 2/22/2002 02:02'!
morphicViewProjectLoader

	| dir aFileList window midLine fixedSize |

	dir := FileDirectory default.
	aFileList := self new directory: dir.
	aFileList optionalButtonSpecs: aFileList servicesForProjectLoader.
	aFileList fileSelectionBlock: self projectOnlySelectionBlock.
	window := (SystemWindow labelled: dir pathName) model: aFileList.

	fixedSize := 25.
	midLine := 0.4.
	self addFullPanesTo: window from: {
		{aFileList optionalButtonRow. 0 @ 0 corner: 1 @ 0. 0@0 corner: 0@fixedSize}.
		{aFileList morphicDirectoryTreePane. 0@0 corner: midLine@1. 0@fixedSize corner: 0@0}.
		{aFileList morphicFileListPane. midLine @ 0 corner: 1@1. 0@fixedSize corner: 0@0}.
	}.
	aFileList postOpen.
	^ window ! !

!FileList2 class methodsFor: 'blue ui' stamp: 'RAA 7/15/2000 19:21'!
morphicViewProjectLoader2InWorld: aWorld

	^self morphicViewProjectLoader2InWorld: aWorld reallyLoad: true! !

!FileList2 class methodsFor: 'blue ui' stamp: 'RAA 2/19/2001 10:14'!
morphicViewProjectLoader2InWorld: aWorld reallyLoad: aBoolean

	^self 
		morphicViewProjectLoader2InWorld: aWorld 
		reallyLoad: aBoolean
		dirFilterType: #initialDirectoryList
! !

!FileList2 class methodsFor: 'blue ui' stamp: 'tfel 8/10/2016 15:40'!
morphicViewProjectLoader2InWorld: aWorld reallyLoad: aBoolean dirFilterType: aSymbol

	| window aFileList buttons treePane textColor1 fileListPane pane2a pane2b treeExtent filesExtent |

	window := AlignmentMorphBob1 newColumn.
	window hResizing: #shrinkWrap; vResizing: #shrinkWrap.
	textColor1 := Color black.
	aFileList := self new directory: FileDirectory default.
	aFileList resort: #name.
	aFileList 
		optionalButtonSpecs: aFileList servicesForProjectLoader;
		fileSelectionBlock: (
			aSymbol == #limitedSuperSwikiDirectoryList ifTrue: [
				MessageSend receiver: self selector: #projectOnlySelectionMethod:
			] ifFalse: [
				self projectOnlySelectionBlock
			]
		);
		"dirSelectionBlock: self hideSqueakletDirectoryBlock;"
		modalView: window.
	window
		setProperty: #FileList toValue: aFileList;
		wrapCentering: #center; cellPositioning: #topCenter;
		borderWidth: ColorTheme current dialogBorderWidth;
		borderColor: ColorTheme current dialogBorderColor;
		useRoundedCorners.
	buttons := {{'OK'. ColorTheme current okColor}. {'Cancel'. ColorTheme current cancelColor}} collect: [ :each |
		self blueButtonText: each first textColor: textColor1 color: each second inWindow: window
	].

	aWorld width < 800 ifTrue: [
		treeExtent := 150@300.
		filesExtent := 350@300.
	] ifFalse: [
		treeExtent := 250@300.
		filesExtent := 350@300.
	].
	(treePane := aFileList morphicDirectoryTreePaneFiltered: aSymbol)
		extent: treeExtent; 
		retractable: false;
		borderWidth: 0.
	fileListPane := aFileList morphicFileListPane 
		extent: filesExtent; 
		retractable: false;
		borderWidth: 0.
	window
		addARow: {
			window fancyText: 'Load A Project' translated font: Preferences standardEToysTitleFont color: textColor1
		};
		addARowCentered: {
			buttons first. 
			(Morph new extent: 30@5) color: Color transparent. 
			buttons second
		};
		addARow: {
			window fancyText: 'Please select a project' translated  font: Preferences standardEToysFont color: textColor1
		};
		addARow: {
				(window inAColumn: {(pane2a := window inARow: {window inAColumn: {treePane}}) 
					useRoundedCorners;
					layoutInset: 0;
					borderWidth: ColorTheme current dialogPaneBorderWidth;
					borderColor: ColorTheme current dialogPaneBorderColor
				}) layoutInset: 10.
				(window inAColumn: {(pane2b := window inARow: {window inAColumn: {fileListPane}}) 
					useRoundedCorners;
					layoutInset: 0;
					borderWidth: ColorTheme current dialogPaneBorderWidth;
					borderColor: ColorTheme current dialogPaneBorderColor
				}) layoutInset: 10.
		}.
	window fullBounds.
	window fillWithRamp: ColorTheme current dialogRampOrColor oriented: 0.65.
	pane2a fillWithRamp: ColorTheme current dialogPaneRampOrColor oriented: (0.7 @ 0.35).
	pane2b fillWithRamp: ColorTheme current dialogPaneRampOrColor oriented: (0.7 @ 0.35).
"
	buttons do: [ :each |
		each fillWithRamp: ColorTheme current dialogButtonsRampOrColor oriented: (0.75 @ 0).
	].
"
	buttons first 
		on: #mouseUp 
		send: (aBoolean ifTrue: [#okHitForProjectLoader] ifFalse: [#okHit])
		to: aFileList.
	buttons second on: #mouseUp send: #cancelHit to: aFileList.
	aFileList postOpen.
	window position: aWorld topLeft + (aWorld extent - window extent // 2).
	window adoptPaneColor: (Color r: 0.548 g: 0.677 b: 1.0).
	window becomeModal.
	^ window openInWorld: aWorld.! !

!FileList2 class methodsFor: 'blue ui' stamp: 'tfel 8/7/2016 11:42'!
morphicViewProjectSaverFor: aProject
"
(FileList2 morphicViewProjectSaverFor: Project current) openInWorld
"
	| window aFileList buttons treePane pane2 textColor1 option treeExtent buttonData buttonRow |

	textColor1 := Color black.
	aFileList := self new directory: ServerDirectory projectDefaultDirectory.
	aFileList dirSelectionBlock: self hideSqueakletDirectoryBlock.
	window := AlignmentMorphBob1 newColumn.
	window hResizing: #shrinkWrap; vResizing: #shrinkWrap.
	aFileList modalView: window.
	window
		setProperty: #FileList toValue: aFileList;
		wrapCentering: #center; cellPositioning: #topCenter;
		borderWidth: ColorTheme current dialogBorderWidth;
		borderColor: ColorTheme current dialogBorderColor;
		useRoundedCorners.

	buttonData := Preferences enableLocalSave
				ifTrue: [{
							{'Save'. #okHit. 'Save in the place specified below, and in the Squeaklets folder on your local disk'. ColorTheme current okColor}.
							{'Save on local disk only'. #saveLocalOnlyHit. 'saves in the Squeaklets folder'. ColorTheme current okColor}.
							{'Cancel'. #cancelHit. 'return without saving'. ColorTheme current cancelColor}
						}]
				ifFalse: [{
							{'Save'. #okHit. 'Save in the place specified below, and in the Squeaklets folder on your local disk'. ColorTheme current okColor}.
							{'Cancel'. #cancelHit. 'return without saving'. ColorTheme current cancelColor}
						}].
	buttons := buttonData collect: [ :each |
		(self blueButtonText: each first textColor: textColor1 color: each fourth inWindow: window)
			setBalloonText: each third translated;
			hResizing: #shrinkWrap;
			on: #mouseUp send: each second to: aFileList
	].

	option := aProject world 
		valueOfProperty: #SuperSwikiPublishOptions 
		ifAbsent: [#initialDirectoryList].
	aProject world removeProperty: #SuperSwikiPublishOptions.

	treeExtent := World height < 500
						ifTrue: [ 350@150 ]
						ifFalse: [ 350@300 ].

	(treePane := aFileList morphicDirectoryTreePaneFiltered: option) 
		extent: treeExtent; 
		retractable: false;
		borderWidth: 0.
	window
		addARowCentered: {
			window fancyText: 'Publish This Project' translated font: Preferences standardEToysTitleFont color: textColor1
		}.
	buttonRow := OrderedCollection new.
	buttons do: [:button | buttonRow add: button] separatedBy: [buttonRow add: ((Morph new extent: 30@5) color: Color transparent)].

"	addARowCentered: {
			buttons first. 
			(Morph new extent: 30@5) color: Color transparent. 
			buttons second.
			(Morph new extent: 30@5) color: Color transparent. 
			buttons third
		};"
	window
		addARowCentered: buttonRow;
		addARowCentered: { (window inAColumn: {(ProjectViewMorph on: aProject) lock}) layoutInset: 4};
		addARowCentered: {
			window fancyText: 'Please select a folder' translated font: Preferences standardEToysFont color: textColor1
		};
		addARow: {
			(
				window inAColumn: {
					(pane2 := window inARow: {window inAColumn: {treePane}}) 
						useRoundedCorners;
						layoutInset: 0;
						borderWidth: ColorTheme current dialogPaneBorderWidth;
						borderColor: ColorTheme current dialogPaneBorderColor
				}
			) layoutInset: 10
		}.
	window fullBounds.
	window fillWithRamp: ColorTheme current dialogRampOrColor oriented: 0.65.
	pane2 fillWithRamp: ColorTheme current dialogPaneRampOrColor oriented: (0.7 @ 0.35).
"
	buttons do: [ :each |
		each fillWithRamp: ColorTheme current dialogButtonsRampOrColor oriented: (0.75 @ 0).
	].
"
	window setProperty: #morphicLayerNumber toValue: 11.
	aFileList postOpen.
	window adoptPaneColor: (Color r: 0.548 g: 0.677 b: 1.0).
	^ window ! !

!FileList2 class methodsFor: 'instance creation' stamp: 'nk 7/12/2000 11:03'!
openMorphicViewInWorld
	"FileList2 openMorphicViewInWorld"
	^self morphicView openInWorld! !

!FileList2 class methodsFor: 'as yet unclassified' stamp: 'nice 2/11/2010 14:37'!
projectOnlySelectionBlock

	^[ :entry :myPattern |
		entry isDirectory ifTrue: [
			false
		] ifFalse: [
			#('*.pr' '*.pr.gz' '*.project') anySatisfy: [ :each | each match: entry name]
		]
	]! !

!FileList2 class methodsFor: 'as yet unclassified' stamp: 'nice 12/27/2009 03:11'!
projectOnlySelectionMethod: incomingEntries

	| versionsAccepted |

	"this shows only the latest version of each project"
	versionsAccepted := Dictionary new.
	incomingEntries do: [ :entry | | basicInfoTuple basicVersion basicName |
		entry isDirectory ifFalse: [
			(#('*.pr' '*.pr.gz' '*.project') anySatisfy: [ :each | each match: entry name]) ifTrue: [
				basicInfoTuple := Project parseProjectFileName: entry name.
				basicName := basicInfoTuple first.
				basicVersion := basicInfoTuple second.
				((versionsAccepted includesKey: basicName) and: 
						[(versionsAccepted at: basicName) first > basicVersion]) ifFalse: [
					versionsAccepted at: basicName put: {basicVersion. entry}
				].
			]
		]
	].
	^versionsAccepted asArray collect: [ :each | each second]! !

!FileList2 class methodsFor: 'instance creation' stamp: 'nk 6/14/2004 08:41'!
prototypicalToolWindow
	"Answer an example of myself seen in a tool window, for the benefit of parts-launching tools"

	^ self morphicView applyModelExtent! !

!FileList2 class methodsFor: 'as yet unclassified' stamp: 'nice 2/11/2010 14:37'!
selectionBlockForSuffixes: anArray

	^[ :entry :myPattern |
		entry isDirectory ifTrue: [
			false
		] ifFalse: [
			anArray anySatisfy: [ :each | each match: entry name]
		]
	]! !

!FileList2 class methodsFor: 'utility' stamp: 'mt 8/11/2016 12:08'!
textRow: aString 

	^AlignmentMorph newRow 
		wrapCentering: #center; cellPositioning: #leftCenter;
		color: Color transparent;
		layoutInset: 0;
		addMorph: (
			AlignmentMorph newColumn
			wrapCentering: #center; cellPositioning: #topCenter;
			color: Color transparent;
			vResizing: #shrinkWrap;
			layoutInset: 0;
			addMorph: (
				AlignmentMorph newRow
				wrapCentering: #center; cellPositioning: #leftCenter;
				color: Color transparent;
				hResizing: #shrinkWrap;
				vResizing: #shrinkWrap;
				layoutInset: 0;
				addMorph: ((StringMorph contents: aString) color: ((UserInterfaceTheme current get: #textColor for: #PluggableTextMorph) ifNil: [Color black])) asMorph
			)
		)! !

!FileList2 class methodsFor: 'morphic ui' stamp: 'dgd 4/3/2006 14:04'!
update: actionRow in: window fileTypeRow: fileTypeRow morphUp: morph

	| fileTypeInfo info2 buttons textColor1 fileSuffixes fileActions aFileList fileTypeString |

	(morph notNil and:[(morph valueOfProperty: #enabled) not]) ifTrue: [^self].
	fileTypeRow submorphsDo: [ :sub |
		sub color: (
			sub == morph 
				ifTrue: [Color white] 
				ifFalse: [(sub valueOfProperty: #enabled) 
							ifTrue: [Color transparent] ifFalse: [Color gray]]
		).
	].
	fileTypeString := morph isNil ifTrue:['xxxx'] ifFalse:[morph valueOfProperty: #buttonText].

	aFileList := window valueOfProperty: #FileList.
	textColor1 := Color r: 0.742 g: 0.839 b: 1.0.
	actionRow removeAllMorphs.
	fileTypeInfo := self endingSpecs.
	info2 := fileTypeInfo detect: [ :each | each first = fileTypeString] ifNone: [ nil ].
	info2 isNil
		ifTrue:[
			buttons := OrderedCollection new
		]
		ifFalse:[
			fileSuffixes := info2 second.
			fileActions := info2 third.
			buttons := fileActions collect: [ :each | aFileList blueButtonForService: each textColor: textColor1 inWindow: window ].
			buttons do: [ :each |
				each fillWithRamp: ColorTheme current okColor oriented: (0.75 @ 0).
			].
		].
	buttons addLast: (self
								blueButtonText: 'Cancel'
								textColor: textColor1
								color: ColorTheme current cancelColor
								inWindow: window
								balloonText: 'Cancel this search' selector: #cancelHit recipient: aFileList).
	buttons do: [ :each | actionRow addMorphBack: each].
	window fullBounds.
	fileSuffixes isNil ifFalse:[
		aFileList fileSelectionBlock: (
			self selectionBlockForSuffixes: (fileSuffixes collect: [ :each | '*.',each])
		).
	].
	aFileList updateFileList.! !

!FileList2 methodsFor: 'own services' stamp: 'nk 6/14/2004 09:43'!
addNewDirectory
	super addNewDirectory.
	self updateDirectory.! !

!FileList2 methodsFor: 'user interface' stamp: 'nice 2/11/2010 14:37'!
blueButtonForService: aService textColor: textColor inWindow: window 
	| block result |
	block := [self fullName isNil
				ifTrue: [self inform: 'Please select a file' translated]
				ifFalse: [aService performServiceFor: self]].
	result := window
				fancyText: aService buttonLabel capitalized translated
				font: Preferences standardEToysFont
				color: textColor.
	result setProperty: #buttonText toValue: aService buttonLabel capitalized;
		 hResizing: #rigid;
		 extent: 100 @ 20;
		 layoutInset: 4;
		 borderWidth: ColorTheme current dialogButtonBorderWidth;
		 useRoundedCorners;
		 setBalloonText: aService label.
	result
		on: #mouseUp
		send: #value
		to: block.
	^ result! !

!FileList2 methodsFor: 'private' stamp: 'RAA 4/6/2001 12:45'!
cancelHit

	modalView delete.
	directory := fileName := currentDirectorySelected := nil.! !

!FileList2 methodsFor: 'volume list and pattern' stamp: 'nk 6/14/2004 09:45'!
changeDirectoryTo: aFileDirectory
	"Change directory as requested."

	self directory: aFileDirectory.
	self updateDirectory! !

!FileList2 methodsFor: 'private' stamp: 'LC 1/6/2002 06:50'!
currentDirectorySelected
	^ currentDirectorySelected
! !

!FileList2 methodsFor: 'own services' stamp: 'nk 6/14/2004 09:42'!
deleteDirectory
	super deleteDirectory.
	self updateDirectory.! !

!FileList2 methodsFor: 'volume list and pattern' stamp: 'RAA 8/17/2000 13:59'!
directory

	^directory! !

!FileList2 methodsFor: 'initialization' stamp: 'RAA 8/17/2000 13:22'!
directoryChangeBlock: aBlockOrNil

	directoryChangeBlock := aBlockOrNil.! !

!FileList2 methodsFor: 'private' stamp: 'ar 2/12/2001 16:20'!
directoryNamesFor: item
	"item may be file directory or server directory"
	| entries |
	entries := item directoryNames.
	dirSelectionBlock ifNotNil:[entries := entries select: dirSelectionBlock].
	^entries! !

!FileList2 methodsFor: 'initialization' stamp: 'tpr 12/1/2003 17:14'!
directory: dir
	"Set the path of the volume to be displayed."

	self okToChange ifFalse: [^ self].

	self modelSleep.
	directory := dir.
	self modelWakeUp.

	sortMode == nil ifTrue: [sortMode := #date].
	volList := Array with: '[]'.
	directory ifNotNil: [
		volList := volList, directory pathParts.  "Nesting suggestion from RvL"
	].
	volList := volList withIndexCollect: [:each :i | ( String new: i-1 withAll: $ ), each].
	self changed: #relabel.
	self changed: #volumeList.
	self pattern: pattern.
	directoryChangeBlock ifNotNil: [directoryChangeBlock value: directory].! !

!FileList2 methodsFor: 'initialization' stamp: 'ar 10/10/2000 16:00'!
dirSelectionBlock: aBlock
	dirSelectionBlock := aBlock! !

!FileList2 methodsFor: 'initialization' stamp: 'RAA 6/16/2000 13:08'!
fileSelectionBlock: aBlock

	fileSelectionBlock := aBlock! !

!FileList2 methodsFor: 'private' stamp: 'LC 1/6/2002 06:51'!
getSelectedDirectory
	ok == true ifFalse: [^ nil].
	^ currentDirectorySelected
! !

!FileList2 methodsFor: 'private' stamp: 'sw 9/12/2002 00:43'!
getSelectedFile
	"Answer a filestream on the selected file.  If it cannot be opened for read/write, try read-only before giving up; answer nil if unsuccessful"

	ok == true ifFalse: [^ nil].
	directory ifNil: [^ nil].
	fileName ifNil: [^ nil].
	^ (directory oldFileNamed: fileName) ifNil:
		[directory readOnlyFileNamed: fileName]! !

!FileList2 methodsFor: 'own services' stamp: 'sd 5/11/2003 22:15'!
importImage
	"Import the given image file and store the resulting Form in the default Imports"

	| fname image |
	fname := fileName sansPeriodSuffix.
	image := Form fromFileNamed: self fullName.
	Imports default importImage: image named: fname.
! !

!FileList2 methodsFor: 'initialization' stamp: 'nice 12/27/2009 03:10'!
initialDirectoryList

	| dirList |
	dirList := (FileDirectory on: '') directoryNames collect: [ :each |
		FileDirectoryWrapper with: (FileDirectory on: each) name: each model: self].
	dirList isEmpty ifTrue:[
		dirList := Array with: (FileDirectoryWrapper 
			with: FileDirectory default 
			name: FileDirectory default localName 
			model: self)].
	dirList := dirList,(
		ServerDirectory serverNames collect: [ :n | | nameToShow dir | 
			dir := ServerDirectory serverNamed: n.
			nameToShow := n.
			(dir directoryWrapperClass with: dir name: nameToShow model: self)
				balloonText: dir realUrl
		]
	).
	^dirList! !

!FileList2 methodsFor: 'initialize-release' stamp: 'ul 11/23/2010 10:46'!
initialize

	super initialize.
	showDirsInFileList := false.
	fileSelectionBlock := [ :entry :myPattern |
		entry isDirectory ifTrue: [
			showDirsInFileList
		] ifFalse: [
			myPattern = '*' or: [myPattern match: entry name]
		]
	].
	dirSelectionBlock := [ :dirName | true].! !

!FileList2 methodsFor: 'drag''n''drop' stamp: 'nk 6/15/2003 22:00'!
isDirectoryList: aMorph
	^aMorph isKindOf: SimpleHierarchicalListMorph! !

!FileList2 methodsFor: 'initialization' stamp: 'RAA 6/16/2000 10:40'!
labelString
	^ (directory ifNil: [^'[]']) pathName contractTo: 50! !

!FileList2 methodsFor: 'initialization' stamp: 'nice 12/27/2009 03:10'!
limitedSuperSwikiDirectoryList

	| dirList localDirName localDir |

	dirList := OrderedCollection new.
	ServerDirectory serverNames do: [ :n | | dir nameToShow | 
		dir := ServerDirectory serverNamed: n.
		dir isProjectSwiki ifTrue: [
			nameToShow := n.
			dirList add: ((dir directoryWrapperClass with: dir name: nameToShow model: self)
				balloonText: dir realUrl)
		].
	].
	ServerDirectory localProjectDirectories do: [ :each |
		dirList add: (FileDirectoryWrapper with: each name: each localName model: self)
	].
	"Make sure the following are always shown, but not twice"
	localDirName := SecurityManager default untrustedUserDirectory.
	localDir := FileDirectory on: localDirName.
	((ServerDirectory localProjectDirectories collect: [:each | each pathName]) includes: localDirName)
			ifFalse: [dirList add: (FileDirectoryWrapper with: localDir name: localDir localName model: self)].
	FileDirectory default pathName = localDirName
			ifFalse: [dirList add: (FileDirectoryWrapper with: FileDirectory default name: FileDirectory default localName model: self)].
	(dirList anySatisfy: [:each | each withoutListWrapper acceptsUploads])
		ifFalse: [dirList add: (FileDirectoryWrapper with: FileDirectory default name: FileDirectory default localName model: self)].
	^dirList! !

!FileList2 methodsFor: 'initialization' stamp: 'mir 2/6/2004 17:25'!
limitedSuperSwikiPublishDirectoryList

	| dirList localDirName localDir |

	dirList := self publishingServers.
	ServerDirectory localProjectDirectories do: [ :each |
		dirList add: (FileDirectoryWrapper with: each name: each localName model: self)].

	"Make sure the following are always shown, but not twice"
	localDirName := SecurityManager default untrustedUserDirectory.
	localDir := FileDirectory on: localDirName.
	((ServerDirectory localProjectDirectories collect: [:each | each pathName]) includes: localDirName)
			ifFalse: [dirList add: (FileDirectoryWrapper with: localDir name: localDir localName model: self)].
	FileDirectory default pathName = localDirName
			ifFalse: [dirList add: (FileDirectoryWrapper with: FileDirectory default name: FileDirectory default localName model: self)].
	^dirList! !

!FileList2 methodsFor: 'volume list and pattern' stamp: 'nice 3/21/2010 21:20'!
listForPatterns: anArray
	"Make the list be those file names which match the patterns."

	| sizePad newList |
	directory ifNil: [^#()].
	(fileSelectionBlock isKindOf: MessageSend) ifTrue: [
		fileSelectionBlock arguments: {directory entries}.
		newList := fileSelectionBlock value.
		fileSelectionBlock arguments: #().
	] ifFalse: [
		newList := Set new.
		anArray do: [ :pat |
			newList addAll: (directory entries select: [:entry | fileSelectionBlock value: entry value: pat]) ].
	].
	newList := newList asArray sort: self sortBlock.
	sizePad := (newList inject: 0 into: [:mx :entry | mx max: entry fileSize])
					asStringWithCommas size.
	^newList collect: [ :e | self fileNameFormattedFrom: e sizePad: sizePad ]! !

!FileList2 methodsFor: 'volume list and pattern' stamp: 'nice 3/21/2010 21:33'!
listForPattern: pat
	"Make the list be those file names which match the pattern."

	| sizePad newList entries |
	directory ifNil: [^#()].
	entries := (Preferences eToyLoginEnabled
		and: [Utilities authorNamePerSe notNil])
		ifTrue: [directory matchingEntries: {'submittedBy: ' , Utilities authorName.} ]
		ifFalse: [directory entries].
	(fileSelectionBlock isKindOf: MessageSend) ifTrue: [
		fileSelectionBlock arguments: {entries}.
		newList := fileSelectionBlock value.
		fileSelectionBlock arguments: #().
	] ifFalse: [
		newList := entries select: [:entry | fileSelectionBlock value: entry value: pat].
	].
	newList := newList asArray sort: self sortBlock.
	sizePad := (newList inject: 0 into: [:mx :entry | mx max: entry fileSize])
					asStringWithCommas size - 1.
	^newList collect: [ :e | self fileNameFormattedFrom: e sizePad: sizePad ]! !

!FileList2 methodsFor: 'private' stamp: 'RAA 6/21/2000 12:06'!
modalView: aSystemWindowOrSuch

	modalView := aSystemWindowOrSuch! !

!FileList2 methodsFor: 'user interface' stamp: 'RAA 2/17/2001 12:18'!
morphicDirectoryTreePane

	^self morphicDirectoryTreePaneFiltered: #initialDirectoryList
! !

!FileList2 methodsFor: 'user interface' stamp: 'rww 12/13/2003 13:07'!
morphicDirectoryTreePaneFiltered: aSymbol
	^(SimpleHierarchicalListMorph 
		on: self
		list: aSymbol
		selected: #currentDirectorySelected
		changeSelected: #setSelectedDirectoryTo:
		menu: #volumeMenu:
		keystroke: nil)
			autoDeselect: false;
			enableDrag: false;
			enableDrop: true;
			yourself
		
! !

!FileList2 methodsFor: 'user interface' stamp: 'RAA 6/16/2000 10:53'!
morphicFileContentsPane

	^PluggableTextMorph 
		on: self 
		text: #contents 
		accept: #put:
		readSelection: #contentsSelection 
		menu: #fileContentsMenu:shifted:
! !

!FileList2 methodsFor: 'user interface' stamp: 'nk 6/15/2003 13:05'!
morphicFileListPane

	^(PluggableListMorph 
		on: self 
		list: #fileList 
		selected: #fileListIndex
		changeSelected: #fileListIndex: 
		menu: #fileListMenu:)
			enableDrag: true;
			enableDrop: false;
			yourself

! !

!FileList2 methodsFor: 'user interface' stamp: 'BG 2/28/2004 22:58'!
morphicPatternPane
   | pane |
    pane := PluggableTextMorph 
		on: self 
		text: #pattern 
		accept: #pattern:.
    pane acceptOnCR: true.
   ^pane
		
! !

!FileList2 methodsFor: 'own services' stamp: 'sw 2/22/2002 02:35'!
okayAndCancelServices
	"Answer ok and cancel services"

	^ {self serviceOkay. self serviceCancel}! !

!FileList2 methodsFor: 'private' stamp: 'ul 2/22/2010 22:32'!
okHit

	ok := true.
	currentDirectorySelected
		ifNil: [ Beeper beep ]
		ifNotNil: [
			self class lastSelDir: directory.
			modalView delete ]! !

!FileList2 methodsFor: 'private' stamp: 'TN 4/13/2005 18:32'!
okHitForProjectLoader

	| areaOfProgress |
	fileName ifNil: [^ self].
	ok := true.
	areaOfProgress := modalView firstSubmorph.
	[
		areaOfProgress setProperty: #deleteOnProgressCompletion toValue: modalView.
		self openProjectFromFile.
		modalView delete.	"probably won't get here"
	]
		on: ProgressTargetRequestNotification
		do: [ :ex | ex resume: areaOfProgress].


! !

!FileList2 methodsFor: 'own services' stamp: 'dtl 2/10/2010 15:34'!
openImageInWindow
	"Handle five file formats: GIF, JPG, PNG, Form stoteOn: (run coded), and BMP.
	Fail if file format is not recognized."

	| image myStream |
	myStream := (directory readOnlyFileNamed: fileName) binary.
	[image := Form fromBinaryStream: myStream.
	Project current openImage: image name: fileName saveResource: false]
		ensure: [myStream close]
! !

!FileList2 methodsFor: 'own services' stamp: 'hg 8/3/2000 16:55'!
openProjectFromFile
	"Reconstitute a Morph from the selected file, presumed to be represent
	a Morph saved via the SmartRefStream mechanism, and open it in an
	appropriate Morphic world."

	Project canWeLoadAProjectNow ifFalse: [^ self].
	ProjectViewMorph 
		openFromDirectory: directory 
		andFileName: fileName
! !

!FileList2 methodsFor: 'initialization' stamp: 'ar 10/4/2009 21:31'!
optionalButtonRow
	"Answer the button row associated with a file list"

	| aRow |
	aRow := AlignmentMorph newRow beSticky.
	aRow color: Color transparent.
	aRow clipSubmorphs: true.
	aRow layoutInset: 5@1; cellInset: 6.
	self universalButtonServices do:  "just the three sort-by items"
			[:service |
				aRow addMorphBack: (service buttonToTriggerIn: self).
				(service selector  == #sortBySize)
					ifTrue:
						[aRow addTransparentSpacerOfSize: (4@0)]].
	aRow setNameTo: 'buttons'.
	aRow setProperty: #buttonRow toValue: true.  "Used for dynamic retrieval later on"
	^ aRow! !

!FileList2 methodsFor: 'initialization' stamp: 'RAA 6/16/2000 13:00'!
optionalButtonSpecs

	^optionalButtonSpecs ifNil: [super optionalButtonSpecs]! !

!FileList2 methodsFor: 'initialization' stamp: 'RAA 6/16/2000 13:01'!
optionalButtonSpecs: anArray

	optionalButtonSpecs := anArray! !

!FileList2 methodsFor: 'private' stamp: 'RAA 6/16/2000 10:48'!
postOpen

	directory ifNotNil: [
		self changed: #(openPath) , directory pathParts. 
	].
! !

!FileList2 methodsFor: 'initialization' stamp: 'nice 12/27/2009 03:10'!
publishingServers

	| dirList |

	dirList := OrderedCollection new.
	ServerDirectory serverNames do: [ :n | | dir nameToShow | 
		dir := ServerDirectory serverNamed: n.
		(dir isProjectSwiki and: [dir acceptsUploads])
			 ifTrue: [
				nameToShow := n.
				dirList add: ((dir directoryWrapperClass with: dir name: nameToShow model: self)
					balloonText: dir realUrl)]].
	^dirList! !

!FileList2 methodsFor: 'own services' stamp: 'yo 7/31/2004 18:08'!
removeLinefeeds
	"Remove any line feeds by converting to CRs instead.  This is a temporary implementation for 3.6 only... should be removed during 3.7alpha."
	| fileContents |
	fileContents := ((FileStream readOnlyFileNamed: self fullName) wantsLineEndConversion: true) contentsOfEntireFile.
	(FileStream newFileNamed: self fullName) 
		nextPutAll: fileContents;
		close.! !

!FileList2 methodsFor: 'private' stamp: 'LC 1/6/2002 07:12'!
saveLocalOnlyHit
	ok := true.
	modalView delete.
	directory := fileName := nil.
	currentDirectorySelected := #localOnly.! !

!FileList2 methodsFor: 'own services' stamp: 'KR 3/21/2008 21:05'!
serviceCancel
	"Answer a service for hitting the cancel button"

	^ (SimpleServiceEntry new
		provider: self 
		label: 'cancel' translatedNoop
		selector: #cancelHit 
		description: 'hit here to cancel ' translatedNoop)
		buttonLabel: 'cancel' translatedNoop! !

!FileList2 methodsFor: 'own services' stamp: 'KR 3/21/2008 20:56'!
serviceOkay
	"Answer a service for hitting the okay button"

	^ (SimpleServiceEntry new
		provider: self 
		label: 'okay' translatedNoop
		selector: #okHit 
		description: 'hit here to accept the current selection' translatedNoop)
		buttonLabel: 'ok' translatedNoop! !

!FileList2 methodsFor: 'own services' stamp: 'KR 3/21/2008 20:53'!
serviceOpenProjectFromFile
	"Answer a service for opening a .pr project file"

	^ SimpleServiceEntry 
		provider: self 
		label: 'load as project' translatedNoop
		selector: #openProjectFromFile
		description: 'open project from file' translatedNoop
		buttonLabel: 'load' translatedNoop! !

!FileList2 methodsFor: 'own services' stamp: 'sw 2/22/2002 02:36'!
servicesForFolderSelector
	"Answer the ok and cancel servies for the folder selector"

	^ self okayAndCancelServices! !

!FileList2 methodsFor: 'own services' stamp: 'sw 2/22/2002 02:36'!
servicesForProjectLoader
	"Answer the services to show in the button pane for the project loader"

	^ {self serviceSortByName. self serviceSortByDate. self serviceSortBySize. self serviceOpenProjectFromFile}! !

!FileList2 methodsFor: 'private' stamp: 'LC 1/6/2002 09:03'!
setSelectedDirectoryTo: aFileDirectoryWrapper
	currentDirectorySelected := aFileDirectoryWrapper.
	self directory: aFileDirectoryWrapper withoutListWrapper.
	brevityState := #FileList.
	"self addPath: path."
	self changed: #fileList.
	self changed: #contents.
	self changed: #currentDirectorySelected.! !

!FileList2 methodsFor: 'as yet unclassified' stamp: 'BG 2/29/2004 23:40'!
specsForImageViewer

	 ^{self serviceSortByName. self serviceSortByDate. self serviceSortBySize }! !

!FileList2 methodsFor: 'initialization' stamp: 'sw 2/22/2002 02:34'!
universalButtonServices
	"Answer the services to be reflected in the receiver's buttons"

	^ self optionalButtonSpecs! !

!FileList2 methodsFor: 'initialization' stamp: 'nk 6/14/2004 09:39'!
updateDirectory
	"directory has been changed externally, by calling directory:.
	Now change the view to reflect the change."
	self changed: #currentDirectorySelected.
	self postOpen.! !

!FileList class methodsFor: 'file reader registration' stamp: 'sd 2/1/2002 21:30'!
allRegisteredServices
	"self allRegisteredServices"

	| col |
	col := OrderedCollection new.
	self registeredFileReaderClasses do: [:each | col addAll: (each services)].
	^ col! !

!FileList class methodsFor: 'file reader registration' stamp: 'sd 1/31/2002 21:42'!
detectService: aBlock ifNone: anotherBlock
	"self detectService: [:each | each selector = #fileIn:] ifNone: [nil]"

	^ self allRegisteredServices
			detect: aBlock
			ifNone: anotherBlock! !

!FileList class methodsFor: 'class initialization' stamp: 'dvf 8/23/2003 12:17'!
initialize
	"FileList initialize"

	RecentDirs := OrderedCollection new.
	(self systemNavigation allClassesImplementing: #fileReaderServicesForFile:suffix:) do: 		[:providerMetaclass |
			self registerFileReader: providerMetaclass soleInstance]! !

!FileList class methodsFor: 'file reader registration' stamp: 'SD 11/11/2001 13:53'!
isReaderNamedRegistered: aSymbol
	"return if a given reader class has been registered. Note that this is on purpose that the argument is
	a symbol and not a class"

	 ^ (self registeredFileReaderClasses collect: [:each | each name]) includes: aSymbol
! !

!FileList class methodsFor: 'file reader registration' stamp: 'nk 6/12/2004 11:42'!
itemsForDirectory: aFileDirectory
	"Answer a list of services appropriate when no file is selected."

	| services |
	services := OrderedCollection new.
	self registeredFileReaderClasses do: [:reader |
		reader ifNotNil: [services addAll: (reader fileReaderServicesForDirectory: aFileDirectory) ]].
	^ services! !

!FileList class methodsFor: 'file reader registration' stamp: 'nk 12/7/2002 12:53'!
itemsForFile: fullName
	"Answer a list of services appropriate for a file of the given full name"

	| services suffix |
	suffix := self suffixOf: fullName.
	services := OrderedCollection new.
	self registeredFileReaderClasses do: [:reader |
		reader ifNotNil: [services addAll: (reader fileReaderServicesForFile: fullName suffix: suffix)]].
	^ services! !

!FileList class methodsFor: 'instance creation' stamp: 'ar 10/4/2009 22:10'!
new
	^self newOn: FileDirectory default! !

!FileList class methodsFor: 'instance creation' stamp: 'mt 10/22/2015 10:44:49'!
newOn: aDirectory

	^super new directory: aDirectory! !

!FileList class methodsFor: 'instance creation' stamp: 'ar 10/4/2009 20:55'!
open
	"Open a view of an instance of me on the default directory."
	^ToolBuilder open: self! !

!FileList class methodsFor: 'instance creation' stamp: 'ar 10/4/2009 22:45'!
openEditorOn: aFileStream editString: editString
	"Open an editor on the given FileStream."
	| fileModel topView builder |
	fileModel := FileList new setFileStream: aFileStream.	"closes the stream"
	builder := ToolBuilder default.
	topView := fileModel buildEditorWith: builder.
	^builder open: topView.! !

!FileList class methodsFor: 'instance creation' stamp: 'SD 11/8/2001 21:20'!
openFileDirectly

	| aResult |
	(aResult := StandardFileMenu oldFile) ifNotNil:
		[self openEditorOn: (aResult directory readOnlyFileNamed: aResult name) editString: nil]! !

!FileList class methodsFor: 'instance creation' stamp: 'mt 10/22/2015 10:44:44'!
openOn: directory

	^ToolBuilder open: (self newOn: directory)! !

!FileList class methodsFor: 'instance creation' stamp: 'ar 10/4/2009 20:55'!
prototypicalToolWindow
	"Answer an example of myself seen in a tool window, for the benefit of parts-launching tools"
	^ ToolBuilder build: self new! !

!FileList class methodsFor: 'file reader registration' stamp: 'ar 9/29/2005 12:27'!
registeredFileReaderClasses
	FileReaderRegistry := nil. "wipe it out"
	^FileServices registeredFileReaderClasses
	! !

!FileList class methodsFor: 'file reader registration' stamp: 'SD 11/8/2001 21:17'!
registerFileReader: aProviderClass
	"register the given class as providing services for reading files"

	| registeredReaders |
	registeredReaders := self registeredFileReaderClasses.
	(registeredReaders includes: aProviderClass) 
			ifFalse: [ registeredReaders addLast: aProviderClass ]! !

!FileList class methodsFor: 'class initialization' stamp: 'KR 8/12/2007 11:06'!
registerInFlapsRegistry
	"Register the receiver in the system's flaps registry"
	self environment
		at: #Flaps
		ifPresent: [:cl | cl registerQuad: {#FileList	. #prototypicalToolWindow, 'File List' translatedNoop.		'A File List is a tool for browsing folders and files on disks and on ftp types.' translatedNoop} 
						forFlapNamed: 'Tools']! !

!FileList class methodsFor: 'class initialization' stamp: 'ar 9/27/2005 21:48'!
removeObsolete
	"FileList removeObsolete"
	self registeredFileReaderClasses copy 
		do:[:cls| cls isObsolete ifTrue:[self unregisterFileReader: cls]]! !

!FileList class methodsFor: 'file reader registration' stamp: 'nk 12/7/2002 12:52'!
suffixOf: aName
	"Answer the file extension of the given file"
	^ aName
		ifNil:
			['']
		ifNotNil:
			[(FileDirectory extensionFor: aName) asLowercase]! !

!FileList class methodsFor: 'class initialization' stamp: 'asm 4/08/2003 12:15'!
unload
	"Unload the receiver from global registries"

	self environment at: #Flaps ifPresent: [:cl |
	cl unregisterQuadsWithReceiver: self] ! !

!FileList class methodsFor: 'file reader registration' stamp: 'SD 11/8/2001 21:18'!
unregisterFileReader: aProviderClass
	"unregister the given class as providing services for reading files"

	self registeredFileReaderClasses remove: aProviderClass ifAbsent: [nil]! !

!FileList methodsFor: 'file menu action' stamp: 'sge 11/28/1999 09:04'!
addNewDirectory
	self 
		addNew: 'Directory'
		byEvaluating: [:newName | directory createDirectory: newName]
! !

!FileList methodsFor: 'file menu action' stamp: 'sge 11/28/1999 09:04'!
addNewFile
	self 
		addNew: 'File'
		byEvaluating: [:newName | (directory newFileNamed: newName) close]
! !

!FileList methodsFor: 'file menu action' stamp: 'rbb 3/1/2005 10:53'!
addNew: aString byEvaluating: aBlock
	"A parameterization of earlier versions of #addNewDirectory and
	#addNewFile.  Fixes the bug in each that pushing the cancel button
	in the FillInTheBlank dialog gave a walkback."

	| response newName index ending |
	self okToChange ifFalse: [^ self].
	(response := UIManager default
						request: ('New {1} Name?' translated format: {aString translated})
						initialAnswer: ('{1}Name' translated format: {aString translated}))
		isEmpty ifTrue: [^ self].
	newName := response asFileName.
	Cursor wait showWhile: [
		aBlock value: newName].
	self updateFileList.
	index := list indexOf: newName.
	index = 0 ifTrue: [ending := ') ',newName.
		index := list findFirst: [:line | line endsWith: ending]].
	self fileListIndex: index.
! !

!FileList methodsFor: 'private' stamp: 'nice 5/8/2011 10:26'!
addPath: aString
	"Add the given string to the list of recently visited directories."

	| full |
	aString ifNil: [^self].
	full := String streamContents: 
		[ :strm | 2 to: volList size do: 
			[ :i | strm nextPutAll: (volList at: i) withBlanksTrimmed.
			strm nextPut: FileDirectory pathNameDelimiter]].
	full := full, aString.
"Remove and super-directories of aString from the collection."
	RecentDirs removeAllSuchThat: [ :aDir | ((aDir, '*') match: full)].

"If a sub-directory is in the list, do nothing."
	(RecentDirs anySatisfy: [ :aDir | ((full, '*') match: aDir)])
		ifTrue: [^self].

	[RecentDirs size >= 10]
		whileTrue: [RecentDirs removeFirst].
	RecentDirs addLast: full! !

!FileList methodsFor: 'server list' stamp: 'SD 11/10/2001 17:49'!
askServerInfo
	"Get the user to create a ServerDirectory for a new server.  Fill in and say Accept."
	| template |
	template := '"Please fill in the following info, then select all text and choose DoIt."

	| aa | 
	self flag: #ViolateNonReferenceToOtherClasses.
	aa := ServerDirectory new.
	aa server: ''st.cs.uiuc.edu''.    "host"
	aa user: ''anonymous''.
	aa password: ''yourEmail@school.edu''.
	aa directory: ''/Smalltalk/Squeak/Goodies''.
	aa url: ''''.    "<- this is optional.  Only used when *writing* update files."
	ServerDirectory addServer: aa named: ''UIUCArchive''.  "<- known by this name in Squeak"'.

	(StringHolder new contents: template) openLabel: 'FTP Server Form'
	! !

!FileList methodsFor: 'private' stamp: 'topa 3/16/2015 16:07'!
availableGraphicsExtent

	^ self codeTextMorph ifNotNil: [:m | m innerBounds extent - (6@22)]
! !

!FileList methodsFor: 'toolbuilder' stamp: 'ar 10/4/2009 20:40'!
buildButtonPaneWith: builder
	| panelSpec |
	panelSpec := builder pluggablePanelSpec new.
	panelSpec 
		model: self;
		children: #getButtonRow;
		layout: #horizontal.
	^panelSpec
! !

!FileList methodsFor: 'toolbuilder' stamp: 'ar 10/4/2009 22:44'!
buildCodePaneWith: builder
	| textSpec |
	textSpec := builder pluggableTextSpec new.
	textSpec 
		model: self;
		getText: #contents; 
		setText: #put:; 
		selection: #contentsSelection; 
		menu: #fileContentsMenu:shifted:.
	^textSpec! !

!FileList methodsFor: 'toolbuilder' stamp: 'ar 10/4/2009 20:33'!
buildContentPaneWith: builder
	| textSpec |
	textSpec := builder pluggableTextSpec new.
	textSpec 
		model: self;
		getText: #contents; 
		setText: #put:; 
		selection: #contentsSelection; 
		menu: #fileContentsMenu:shifted:.
	^textSpec
! !

!FileList methodsFor: 'toolbuilder' stamp: 'mt 1/21/2016 14:54'!
buildDirectoryTreeWith: builder 
	| treeSpec |
	treeSpec := builder pluggableTreeSpec new.
	treeSpec
		 model: self ;
		 roots: #rootDirectoryList ;
		 hasChildren: #hasMoreDirectories: ;
		 getChildren: #subDirectoriesOf: ;
		 getSelectedPath: #selectedPath ;
		 setSelected: #setDirectoryTo: ;
		 getSelected: #directory;
		 label: #directoryNameOf: ;
		 menu: #volumeMenu: ;
		 autoDeselect: false.
	SystemBrowser browseWithDragNDrop ifTrue:
		[ treeSpec
			dragItem: #dragFromDirectoryList: ;
			dropItem: #drop:ontoDirectory:shouldCopy: ;
			dropAccept: #wantsDraggedObject: ].
	^ treeSpec! !

!FileList methodsFor: 'toolbuilder' stamp: 'ar 10/4/2009 22:42'!
buildEditorWith: builder
	^super buildWith: builder! !

!FileList methodsFor: 'toolbuilder' stamp: 'mt 8/8/2016 09:54'!
buildFileListWith: builder 
	| buttons listSpec top |
	top := builder pluggablePanelSpec new.
	top children: OrderedCollection new.
	buttons := self buildButtonPaneWith: builder.
	buttons frame:
		(self
			topConstantHeightFrame: self buttonHeight
			fromLeft: 0
			width: 1).
	top children add: buttons.
	listSpec := builder pluggableListSpec new.
	listSpec
		 model: self ;
		 list: #fileList ;
		 getIndex: #fileListIndex ;
		 setIndex: #fileListIndex: ;
		 menu: #fileListMenu: ;
		 keyPress: nil ;
		 frame:
		(self
			frameOffsetFromTop: self buttonHeight * 1.1
			fromLeft: 0
			width: 1
			bottomFraction: 1) .
	SystemBrowser browseWithDragNDrop ifTrue: [ listSpec dragItem: #dragFromFileList: ].
	top children add: listSpec.
	^ top! !

!FileList methodsFor: 'toolbuilder' stamp: 'topa 4/26/2015 23:33'!
buildPatternInputWith: builder
	| textSpec |
	textSpec := builder pluggableInputFieldSpec new.
	textSpec 
		model: self;
		font: self pathAndPatternFont;
		getText: #pathAndPattern; 
		setText: #pathAndPattern:.
	^textSpec
! !

!FileList methodsFor: 'toolbuilder' stamp: 'topa 4/26/2015 22:55'!
buildWith: builder
	"FileList open"
	| windowSpec window |
	windowSpec := self buildWindowWith: builder specs: {
		(self topConstantHeightFrame: self pathAndPatternHeight
			fromLeft: 0
			width: 1) -> [self buildPatternInputWith: builder].
		(self frameOffsetFromTop: self pathAndPatternHeight
			fromLeft: 0.25
			width: 0.75
			bottomFraction: 0.5) -> [self buildFileListWith: builder].
		(self frameOffsetFromTop: self pathAndPatternHeight
			fromLeft: 0
			width: 0.25
			bottomFraction: 1) -> [self buildDirectoryTreeWith: builder].
		(0.25@0.5 corner: 1@1) -> [self buildContentPaneWith: builder].
	}.
	window := builder build: windowSpec.
	self changed: #selectedPath.
	^window! !

!FileList methodsFor: 'toolbuilder' stamp: 'topa 4/26/2015 23:34'!
buttonHeight
	^ Preferences standardButtonFont height * 2! !

!FileList methodsFor: 'initialization' stamp: 'sw 11/30/2002 00:05'!
buttonSelectorsToSuppress
	"Answer a list of action selectors whose corresponding services we would prefer *not* to have appear in the filelist's button pane; this can be hand-jimmied to suit personal taste."

	^ #(removeLineFeeds: addFileToNewZip: compressFile: putUpdate:)! !

!FileList methodsFor: 'file menu action' stamp: 'ka 8/3/2001 21:12'!
compressFile
	"Compress the currently selected file"

	| f |
	f := StandardFileStream
				readOnlyFileNamed: (directory fullNameFor: self fullName).
	f compressFile.
	self updateFileList! !

!FileList methodsFor: 'private' stamp: 'hmm 4/27/2010 12:47'!
contents
	"Answer the contents of the file, reading it first if needed."
	"Possible brevityState values:
		FileList,
		fullFile, briefFile, needToGetFull, needToGetBrief,
		fullHex, briefHex, needToGetFullHex, needToGetBriefHex"

	(listIndex = 0) | (brevityState == #FileList) ifTrue: [^ self defaultContents].  "no file selected"
	brevityState == #fullFile ifTrue: [^ contents].
	brevityState == #fullHex ifTrue: [^ contents].
	brevityState == #briefFile ifTrue: [^ contents].
	brevityState == #briefHex ifTrue: [^ contents].
	brevityState == #graphic ifTrue: [^ contents].

	brevityState == #needToGetGraphic ifTrue: [^self readGraphicContents].
	
	brevityState == #needToGetFullHex ifTrue: [^ self readContentsHex: false].
	brevityState == #needToGetBriefHex ifTrue: [^ self readContentsHex: true].

	brevityState == #needToGetFull ifTrue: [^ self readContentsBrief: false].
	brevityState == #needToGetBrief ifTrue: [^ self readContentsBrief: true].  "default"

	(TextConverter allEncodingNames includes: brevityState) 
		ifTrue: [ ^self readContentsAsEncoding: brevityState].

	self halt: 'unknown state ' , brevityState printString! !

!FileList methodsFor: 'menu messages' stamp: 'ar 1/15/2001 18:38'!
copyName

	listIndex = 0 ifTrue: [^ self].
	Clipboard clipboardText: self fullName asText.
! !

!FileList methodsFor: 'private' stamp: 'dgd 12/27/2003 12:22'!
defaultContents
	contents := list == nil
		ifTrue: [String new]
		ifFalse: [String streamContents:
					[:s | s nextPutAll: 'NO FILE SELECTED' translated; cr.
					s nextPutAll: '  -- Folder Summary --' translated; cr.
					list do: [:item | s nextPutAll: item; cr]]].
	brevityState := #FileList.
	^ contents! !

!FileList methodsFor: 'private' stamp: 'yo 7/6/2004 20:52'!
defaultEncoderFor: aFileName

	"This method just illustrates the stupidest possible implementation of encoder selection."
	| l |
	l := aFileName asLowercase.
"	((l endsWith: FileStream multiCs) or: [
		l endsWith: FileStream multiSt]) ifTrue: [
		^ UTF8TextConverter new.
	].
"
	((l endsWith: FileStream cs) or: [
		l endsWith: FileStream st]) ifTrue: [
		^ MacRomanTextConverter new.
	].

	^ Latin1TextConverter new.

	! !

!FileList methodsFor: 'user interface' stamp: 'mt 7/9/2016 08:52'!
defaultWindowColor
	^ (Color r: 0.65 g: 0.65 b: 0.65)! !

!FileList methodsFor: 'volume list and pattern' stamp: 'KR 5/1/2008 01:37'!
deleteDirectory
	"Remove the currently selected directory"
	| localDirName |
	directory entries size = 0 ifFalse:[^self inform:'Directory must be empty' translated].
	localDirName _ directory localName.
	(self confirm: ('Really delete {1}?' translated format: {localDirName})) ifFalse: [^ self].
	self volumeListIndex: self volumeListIndex-1.
	directory deleteDirectory: localDirName.
	self updateFileList.! !

!FileList methodsFor: 'file menu action' stamp: 'dgd 9/21/2003 17:37'!
deleteFile
	"Delete the currently selected file"
	listIndex = 0 ifTrue: [^ self].
	(self confirm: ('Really delete {1}?' translated format:{fileName})) ifFalse: [^ self].
	directory deleteFileNamed: fileName.
	self updateFileList.
	brevityState := #FileList.
	self get! !

!FileList methodsFor: 'file list menu' stamp: 'RAA 2/2/2002 08:18'!
dirAndFileName

	^{directory. fileName}! !

!FileList methodsFor: 'volume list and pattern' stamp: 'SD 11/11/2001 13:59'!
directory

	^ directory! !

!FileList methodsFor: 'directory tree' stamp: 'ar 4/10/2010 11:00'!
directoryNameOf: aDirectory
	"Attempt to find the name of ServerDirectories when used."

	^(aDirectory isRemoteDirectory and:[aDirectory isKindOf: ServerDirectory])
		ifTrue:[ServerDirectory servers keyAtIdentityValue: aDirectory]
		ifFalse:[aDirectory localName]! !

!FileList methodsFor: 'initialization' stamp: 'mt 4/1/2015 09:11'!
directory: aFileDirectory 
	"Set the path of the volume to be displayed."
	self okToChange ifFalse: [ ^ self ].
	self modelSleep.
	directory := aFileDirectory.
	self modelWakeUp.
	sortMode == nil ifTrue: [ sortMode := #date ].
	volList := (Array with: '[]') , directory pathParts withIndexCollect:
		[ : each : i | (String
			new: i - 1
			withAll: Character space) , each ].
	volListIndex := volList size.
	self changed: #relabel.
	self changed: #volumeList.
	self changed: #directory.
	self pattern: pattern! !

!FileList methodsFor: 'file list' stamp: 'cmm 5/27/2012 21:29'!
dragFromDirectoryList: anIndex
	^ self directory! !

!FileList methodsFor: 'file list' stamp: 'cmm 5/27/2012 17:12'!
dragFromFileList: anIndex 
	^ self directory entryAt: (self fileNameFromFormattedItem: (self fileList at: self fileListIndex))! !

!FileList methodsFor: 'drag''n''drop' stamp: 'nk 6/14/2003 12:58'!
dragPassengerFor: item inMorph: dragSource
	^self directory fullNameFor: ((self fileNameFromFormattedItem: item contents copy)
		copyReplaceAll: self folderString with: '').
! !

!FileList methodsFor: 'toolbuilder' stamp: 'cmm 5/27/2012 21:13'!
drop: aDirectoryEntryFile ontoDirectory: aFileDirectory shouldCopy: aBoolean 
	aDirectoryEntryFile containingDirectory = aFileDirectory ifTrue: [ ^ self ].
	aBoolean
		ifTrue: [ aFileDirectory copyHere: aDirectoryEntryFile ]
		ifFalse:
			[ directory
				rename: aDirectoryEntryFile fullName
				toBe: (aFileDirectory fullNameFor: aDirectoryEntryFile name).
			self setDirectoryTo: directory ]! !

!FileList methodsFor: 'initialization' stamp: 'sw 2/26/2002 00:37'!
dynamicButtonServices
	"Answer services for buttons that may come and go in the button pane, depending on selection"

	^ fileName isEmptyOrNil
		ifTrue:
			[#()]
		ifFalse:
			[ | toReject |
				toReject := self buttonSelectorsToSuppress.
				(self itemsForFile: self fullName) reject:
					[:svc | toReject includes: svc selector]]! !

!FileList methodsFor: 'private' stamp: 'mt 4/12/2015 19:10'!
entriesMatching: patternString
	"Answer a list of directory entries which match the patternString.
	The patternString may consist of multiple patterns separated by ';'.
	Each pattern can include a '*' or '#' as wildcards - see String>>match:"

	| entries patterns |
	entries := directory entries reject:[:e| Smalltalk isMorphic and: [e isDirectory]].
	patterns := patternString findTokens: ';'.
	(patterns anySatisfy: [:each | each = '*'])
		ifTrue: [^ entries].
	^ entries select: [:entry | patterns anySatisfy: [:each | each match: entry name]]! !

!FileList methodsFor: 'toolbuilder' stamp: 'ar 10/4/2009 20:51'!
executeService: aService
	aService performServiceFor: self.! !

!FileList methodsFor: 'file list menu' stamp: 'topa 3/16/2015 17:00'!
fileContentsMenu: aMenu shifted: shifted
	"Construct aMenu to have items appropriate for the file browser's code pane, given the shift state provided"
	^ self menu: aMenu for: #(fileContentsMenu fileContentsMenuShifted:) shifted: shifted! !

!FileList methodsFor: 'file list' stamp: ''!
fileList
	"Answer the list of files in the current volume."

	^ list! !

!FileList methodsFor: 'file list' stamp: ''!
fileListIndex
	"Answer the index of the currently selected file."

	^ listIndex! !

!FileList methodsFor: 'file list' stamp: 'cmm 6/20/2010 16:35'!
fileListIndex: anInteger
	"Select the file name having the given index, and display its contents."

	| item name |
	self okToChange ifFalse: [^ self].
	listIndex := anInteger.
	listIndex = 0 
		ifTrue: [fileName := nil]
		ifFalse:
			[item := self fileNameFromFormattedItem: (list at: anInteger).
			(item endsWith: self folderString)
				ifTrue:
					["remove [...] folder string and open the folder"
					name := item copyFrom: 1 to: item size - self folderString size.
					listIndex := 0.
					brevityState := #FileList.
					self addPath: name.
					name first = $^
						ifTrue: [self directory: (ServerDirectory serverNamed: name allButFirst)]
						ifFalse: [volListIndex = 1 ifTrue: [name := name, directory slash].
							self directory: (directory directoryNamed: name)]]
				ifFalse: [fileName := item]].  "open the file selected"

	brevityState := self isGraphicsFileSelected
		ifTrue: [#needToGetGraphic]
		ifFalse: [#needToGetBrief]..
	self 
		changed: #fileListIndex ;
		changed: #contents ;
		changed: #labelString.
	self updateButtonRow! !

!FileList methodsFor: 'file list menu' stamp: 'topa 3/16/2015 16:53'!
fileListMenu: aMenu
	^ self menu: aMenu for: #(fileListMenu fileListMenuShifted:)
! !

!FileList methodsFor: 'file list' stamp: 'sd 2/14/2002 16:58'!
fileName

	^ fileName! !

!FileList methodsFor: 'volume list and pattern' stamp: 'laza 3/8/2010 10:24'!
fileNameFormattedFrom: entry sizePad: sizePad
	"entry is a 5-element array of the form:
		(name creationTime modificationTime dirFlag fileSize)"
	| sizeStr nameStr dateStr |
	nameStr := entry isDirectory
		ifTrue: [entry name , self folderString]
		ifFalse: [entry name].
	dateStr := ((Date fromSeconds: entry modificationTime )
					printFormat: #(3 2 1 $. 1 1 2)) , ' ' ,
				(String streamContents: [:s |
					(Time fromSeconds: entry modificationTime \\ 86400)
						print24: true on: s]).
	sizeStr := entry fileSize asStringWithCommas.
	sortMode = #name ifTrue:
		[^ nameStr , '    (' , dateStr , ' ' , sizeStr , ')'].
	sortMode = #date ifTrue:
		[^ '(' , dateStr , ' ' , sizeStr , ') ' , nameStr].
	sortMode = #size ifTrue:
		[^ '(' , (String new: sizePad - sizeStr size withAll: Character space) , sizeStr , ' ' , dateStr , ') ' , nameStr].
! !

!FileList methodsFor: 'private' stamp: 'rhi 9/8/2001 02:17'!
fileNameFromFormattedItem: item
	"Extract fileName and folderString from a formatted fileList item string"

	| from to |
	self sortingByName
		ifTrue: [
			from := item lastIndexOf: $( ifAbsent: [0].
			to := item lastIndexOf: $) ifAbsent: [0]]
		ifFalse: [
			from := item indexOf: $( ifAbsent: [0].
			to := item indexOf: $) ifAbsent: [0]].
	^ (from * to = 0
		ifTrue: [item]
		ifFalse: [item copyReplaceFrom: from to: to with: '']) withBlanksTrimmed! !

!FileList methodsFor: 'file list menu' stamp: 'nk 11/16/2002 13:00'!
fileSelectedMenu: aMenu

	| firstItems secondItems thirdItems n1 n2 n3 services |
	firstItems := self itemsForFile: self fullName.
	secondItems := self itemsForAnyFile.
	thirdItems := self itemsForNoFile.
	n1 := firstItems size.
	n2 := n1 + secondItems size.
	n3 := n2 + thirdItems size.
	services := firstItems, secondItems, thirdItems, self serviceAllFileOptions.
	services do: [ :svc | svc addDependent: self ].
	^ aMenu 
		addServices2: services 
		for: self
		extraLines: (Array with: n1 with: n2 with: n3)
! !

!FileList methodsFor: 'private' stamp: ''!
folderString
	^ ' [...]'! !

!FileList methodsFor: 'toolbuilder' stamp: 'fbs 5/1/2010 23:00'!
frameOffsetFromTop: height fromLeft: leftFraction width: rightFraction bottomFraction: bottomFraction
	^LayoutFrame new
		topFraction: 0 offset: height;
		leftFraction: leftFraction offset: 0;
		rightFraction: (leftFraction + rightFraction) offset: 0;
		bottomFraction: bottomFraction offset: 0;
		yourself.! !

!FileList methodsFor: 'file list menu' stamp: 'kfr 1/21/2016 15:57'!
fullFileListMenu: aMenu shifted: aBoolean
	"Fill the menu with all possible items for the file list pane, regardless of selection."

	| lastProvider |
	aMenu title: 'all possible file operations' translated.
	aMenu addStayUpItemSpecial.

	lastProvider := nil.
	(self itemsForFile: self fullName) do: [ :svc |
		(lastProvider notNil and: [svc provider ~~ lastProvider])
			ifTrue: [ aMenu addLine ].
		svc addServiceFor: self toMenu: aMenu.
		Smalltalk isMorphic ifTrue: [aMenu submorphs last setBalloonText: svc description].
		lastProvider := svc provider.
		svc addDependent: self.
	].

	^aMenu! !

!FileList methodsFor: 'private' stamp: 'sw 1/7/2003 17:08'!
fullName
	"Answer the full name for the currently selected file; answer nil if no file is selected."

	^ fileName ifNotNil: [directory
		ifNil:
			[FileDirectory default fullNameFor: fileName]
		ifNotNil:
			[directory fullNameFor: fileName]]
! !

!FileList methodsFor: 'file menu action' stamp: 'jm 5/3/1998 18:03'!
get
	"Get contents of file again, it may have changed. Do this by making the cancel string be the contents, and doing a cancel."

	Cursor read showWhile: [
		self okToChange ifFalse: [^ nil].
		brevityState == #briefHex
			ifTrue: [brevityState := #needToGetFullHex]
			ifFalse: [brevityState := #needToGetFull].
		self changed: #contents].
! !

!FileList methodsFor: 'toolbuilder' stamp: 'ar 10/4/2009 20:40'!
getButtonRow
	"Answer the dynamic button row to use for the currently selected item."
	| builder svc |
	builder := ToolBuilder default.
	svc := self universalButtonServices.
	self fileListIndex = 0 ifFalse:[svc := svc, self dynamicButtonServices].
	^svc collect:[:service| service buildWith: builder in: self].! !

!FileList methodsFor: 'file menu action' stamp: 'yo 3/31/2003 11:25'!
getEncodedText

	Cursor read showWhile: [
		self selectEncoding.
		self changed: #contents].
! !

!FileList methodsFor: 'file menu action' stamp: 'jm 5/3/1998 18:04'!
getHex
	"Get contents of file again, and display in Hex. Do this by making the cancel string be the contents, and doing a cancel."

	Cursor read showWhile: [
		brevityState := #needToGetBriefHex.
		self changed: #contents].
! !

!FileList methodsFor: 'directory tree' stamp: 'ar 10/4/2009 21:34'!
getSelectedPath
	self halt.! !

!FileList methodsFor: 'directory tree' stamp: 'ar 10/4/2009 22:12'!
hasMoreDirectories: aDirectory
	(aDirectory isKindOf: FileDirectory) ifFalse:[^true]. "server directory; don't ask"
	^directoryCache at: aDirectory ifAbsentPut:[
		[aDirectory directoryNames notEmpty] on: Error do:[:ex| true].
	].! !

!FileList methodsFor: 'initialization' stamp: 'ar 10/4/2009 22:11'!
initialize
	super initialize.
	directoryCache := WeakIdentityKeyDictionary new.! !

!FileList methodsFor: 'private' stamp: 'jmv 3/2/2010 16:00'!
isFileSelected
	"return if a file is currently selected"

	^ fileName notNil! !

!FileList methodsFor: 'private' stamp: 'nice 5/8/2011 10:27'!
isGraphicsFileSelected
	^fileName notNil
		and: [(self itemsForFile: self fullName) anySatisfy: [:each | each provider == Form and: [each selector == #importImage:]]]! !

!FileList methodsFor: 'file list menu' stamp: 'sw 11/8/2003 13:32'!
itemsForAnyFile
	"Answer a list of universal services that could apply to any file"
	
	| services |
	services := OrderedCollection new: 4.
	services add: self serviceCopyName. 
	services add: self serviceRenameFile. 
	services add: self serviceDeleteFile.
	services add: self serviceViewContentsInWorkspace.
	^ services! !

!FileList methodsFor: 'file list menu' stamp: 'nk 6/12/2004 12:05'!
itemsForDirectory: dir 
	| services |
	services := OrderedCollection new.
	dir ifNotNil: [
		services
			addAll: (self class itemsForDirectory: dir).
		services last useLineAfter: true. ].
	services add: self serviceAddNewFile.
	services add: self serviceAddNewDirectory.
	^ services! !

!FileList methodsFor: 'file list menu' stamp: 'nk 12/7/2002 12:56'!
itemsForFile: fullName
	"Answer a list of services appropriate for a file of the given full name"
	| suffix |
	suffix := self class suffixOf: fullName.
	^ (self class itemsForFile: fullName) , (self myServicesForFile: fullName suffix: suffix)! !

!FileList methodsFor: 'file list menu' stamp: 'nk 6/12/2004 12:06'!
itemsForNoFile

	| services |
	services := OrderedCollection new.
	services add: self serviceSortByName.
	services add: self serviceSortBySize.
	services add: (self serviceSortByDate useLineAfter: true).
	services addAll: (self itemsForDirectory: (self isFileSelected ifFalse: [ self directory ] ifTrue: [])).
	^ services

		! !

!FileList methodsFor: 'initialization' stamp: 'cmm 6/20/2010 16:30'!
labelString
	^ directory
		ifNil: [ 'File List' ]
		ifNotNil:
			[ fileName
				ifNil: [ directory fullName ]
				ifNotNil: [ directory fullNameFor: fileName ] ]! !

!FileList methodsFor: 'private' stamp: 'nice 3/21/2010 21:19'!
listForPatterns: anArray
	"Make the list be those file names which match the pattern."

	| sizePad newList |
	newList := Set new.
	anArray do: [ :pat | newList addAll: (self entriesMatching: pat) ].
	newList := newList asArray sort: self sortBlock.
	sizePad := (newList inject: 0 into: [:mx :entry | mx max: entry fileSize])
					asStringWithCommas size.
	newList := newList collect: [ :e | self fileNameFormattedFrom: e sizePad: sizePad ].

	volList size = 1 ifTrue:
		["Include known servers along with other desktop volumes" 
		^ newList  ,
			(ServerDirectory serverNames collect: [:n | '^' , n , self folderString])].
	^ newList! !

!FileList methodsFor: 'volume list and pattern' stamp: 'nice 3/21/2010 21:32'!
listForPattern: pat
	"Make the list be those file names which match the pattern."

	| sizePad newList |
	newList := (self entriesMatching: pat) asArray sort: self sortBlock.
	sizePad := (newList inject: 0 into: [:mx :entry | mx max: (entry at: 5)])
					asStringWithCommas size - 1.
	newList := newList collect: [ :e | self fileNameFormattedFrom: e sizePad: sizePad ].

	volList size = 1 ifTrue:
		["Include known servers along with other desktop volumes" 
		^ newList ,
		(ServerDirectory serverNames collect: [:n | '^' , n , self folderString])].
	^ newList! !

!FileList methodsFor: 'file list menu' stamp: 'mt 11/13/2015 09:41'!
mainFileContentsMenu: aMenu
	"Construct aMenu to have items appropriate for the file browser's code pane, for the unshifted state"
	<fileContentsMenuShifted: false>
	^ aMenu addTranslatedList: #(
			('find... (f)' 				find)
			('find again (g)' 			findAgain)
			('find and replace... '		findReplace)
			('do/replace again (j)' 	again)
			-
			('undo (z)' 				undo)
			('redo (Z)' 				redo)
			-
			('copy (c)' 				copySelection)
			('cut (x)' 				cut)
			('paste (v)' 				paste)
			('paste...' 				pasteRecent)
			-
			('do it (d)' 				doIt)
			('print it (p)' 			printIt)
			('inspect it (i)' 			inspectIt)
			('fileIn selection (G)'	fileItIn)
			-
			('accept (s)' 			accept)
			('cancel (l)' 				cancel)
			-
			('more...' 				shiftedYellowButtonActivity));
		yourself
! !

!FileList methodsFor: 'file list menu' stamp: 'topa 3/16/2015 16:52'!
mainFileListMenu: aMenu
	<fileListMenu>
	fileName
		ifNil: [^ self noFileSelectedMenu: aMenu]
		ifNotNil: [^ self fileSelectedMenu: aMenu].
! !

!FileList methodsFor: 'volume menu' stamp: 'topa 3/16/2015 19:08'!
mainVolumeMenu: aMenu
	<volumeMenu>
	^ aMenu addTranslatedList: #(
			('recent...'				recentDirs)
			-
			('add server...'			askServerInfo)
			('remove server...'		removeServer)
			-
			('delete directory...'		deleteDirectory)
			-);
		yourself! !

!FileList methodsFor: 'initialization' stamp: 'tk 12/17/1999 18:00'!
modelSleep
	"User has exited or collapsed the window -- close any remote connection."

	directory ifNotNil: [directory sleep]! !

!FileList methodsFor: 'initialization' stamp: 'nk 1/19/2005 13:25'!
modelWakeUp
	"User has entered or expanded the window -- reopen any remote connection."

	(directory notNil and:[directory isRemoteDirectory])
		ifTrue: [[directory wakeUp] on: TelnetProtocolError do: [ :ex | self inform: ex printString ]] "It would be good to implement a null method wakeUp on the root of directory"! !

!FileList methodsFor: 'file list menu' stamp: 'sd 2/6/2002 21:25'!
myServicesForFile: fullName suffix: suffix

	^(FileStream isSourceFileSuffix: suffix)
		ifTrue: [ {self serviceBroadcastUpdate} ]
		ifFalse: [ #() ]! !

!FileList methodsFor: 'file list menu' stamp: 'SD 11/8/2001 20:34'!
noFileSelectedMenu: aMenu

	^ aMenu
		addServices: self itemsForNoFile 
		for: self
		extraLines: #()
		
! !

!FileList methodsFor: 'file list menu' stamp: 'sw 2/27/2001 13:52'!
offerAllFileOptions
	"Put up a menu offering all possible file options, whatever the suffix of the current selection may be.  Specially useful if you're wanting to keep the menu up"

	self offerMenuFrom: #fullFileListMenu:shifted: shifted: true! !

!FileList methodsFor: 'initialization' stamp: 'sbw 12/30/1999 15:53'!
optionalButtonHeight

	^ 15! !

!FileList methodsFor: 'initialization' stamp: 'sw 2/17/2002 00:07'!
optionalButtonSpecs
	"Answer a list of services underlying the optional buttons in their initial inception."

	^ {self serviceSortByName. self serviceSortByDate. self serviceSortBySize}! !

!FileList methodsFor: 'volume list and pattern' stamp: 'ar 10/4/2009 21:07'!
pathAndPattern
	"Answers both path and pattern"
	^directory fullName, directory slash, pattern! !

!FileList methodsFor: 'toolbuilder' stamp: 'topa 4/26/2015 23:32'!
pathAndPatternFont

	^ Preferences standardDefaultTextFont! !

!FileList methodsFor: 'toolbuilder' stamp: 'topa 4/26/2015 23:32'!
pathAndPatternHeight
	" Take a whole font line and 50 % for space "
	^ (self pathAndPatternFont height * 1.5) ceiling! !

!FileList methodsFor: 'volume list and pattern' stamp: 'ar 10/4/2009 21:46'!
pathAndPattern: stringOrText
	"Answers both path and pattern"
	| base pat aString |
	aString := stringOrText asString.
	base := aString copyUpToLast: directory pathNameDelimiter.
	pat := aString copyAfterLast: directory pathNameDelimiter.
	self changed: #pathAndPattern. "avoid asking if it's okToChange"
	pattern := pat.
	self directory: (FileDirectory on: base).
	self changed: #pathAndPattern.
	self changed: #selectedPath.! !

!FileList methodsFor: 'volume list and pattern' stamp: 'tk 4/7/98 15:26'!
pattern

	^ pattern ifNil: ['*']
! !

!FileList methodsFor: 'volume list and pattern' stamp: 'jmb 3/18/2010 11:18'!
pattern: textOrStringOrNil

	textOrStringOrNil
		ifNil: [pattern := '*']
		ifNotNil: [pattern := textOrStringOrNil asString].
	pattern isEmpty ifTrue: [pattern := '*'].
	self updateFileList.
	^ true
! !

!FileList methodsFor: 'menu messages' stamp: 'sw 11/30/2002 15:38'!
perform: selector orSendTo: otherTarget 
	"Selector was just chosen from a menu by a user.
	If it's one of the three sort-by items, handle it specially.
	If I can respond myself, then perform it on myself. 
	If not, send it to otherTarget, presumably the editPane from which the menu was invoked."

	^ (#(sortByDate sortBySize sortByName) includes: selector)
		ifTrue:
			[self resort: selector]
		ifFalse:
			[(#(get getHex copyName openImageInWindow importImage renameFile deleteFile addNewFile) includes: selector)
				ifTrue: [self perform: selector]
				ifFalse: [super perform: selector orSendTo: otherTarget]]! !

!FileList methodsFor: 'private' stamp: 'sumim 5/7/2010 11:17'!
put: aText
	"Private - put the supplied text onto the file"

	| ff type newName |
	brevityState == #fullFile ifTrue:
		[ff := directory newFileNamed: self fullName.
		Cursor write showWhile: [ff nextPutAll: aText asString; close].
		(directory = ff directory and: [fileName = ff localName])
			ifTrue: [contents := aText asString]
			ifFalse: 		"user changed the directory and/or renamed the file"
				[self clearUserEditFlag.
				directory ~= ff directory ifTrue: 
					[self directory: ff directory.
					self changed: #rootDirectoryList; changed: #selectedPath].
				self updateFileList.
				contents := aText asString.
				newName := ff localName.
				listIndex := list findFirst: [:item | (self fileNameFromFormattedItem: item) = newName].
				listIndex > 0 ifTrue: [fileName := newName].
				brevityState := #needToGetBrief.
				self changed: #fileListIndex].
		self changed: #relabel.
		^ true  "accepted"].

	listIndex = 0 ifTrue:
		[self inform: 'No fileName is selected' translated.
		^ false  "failed"].
	type := 'These'.
	brevityState = #briefFile ifTrue: [type := 'Abbreviated'].
	brevityState = #briefHex ifTrue: [type := 'Abbreviated'].
	brevityState = #fullHex ifTrue: [type := 'Hexadecimal'].
	brevityState = #FileList ifTrue: [type := 'Directory'].
	self inform: ('{1} contents cannot
meaningfully be saved at present.' translated format:{type translated}).
	^ false  "failed"
! !

!FileList methodsFor: 'private' stamp: 'yo 3/14/2005 13:55'!
readContentsAsEncoding: encodingName
	| f writeStream converter c |
	f := directory oldFileOrNoneNamed: self fullName.
	f ifNil: [^ 'For some reason, this file cannot be read'].
	writeStream := WriteStream on: String new.
	converter := TextConverter defaultConverterClassForEncoding: encodingName.
	converter ifNil: [^ 'This encoding is not supported'].
	f converter: converter new.
	f wantsLineEndConversion: true.
	[f atEnd or: [(c := f next) isNil]]
		whileFalse: [writeStream nextPut: c].
	f close.
	^ writeStream contents! !

!FileList methodsFor: 'private' stamp: 'tlk 11/13/2004 19:01'!
readContentsBrief: brevityFlag
	"Read the contents of the receiver's selected file, unless it is too long, in which case show just the first 5000 characters. Don't create a file if it doesn't already exist."
	| f fileSize first5000 |

	brevityFlag ifTrue: [
		directory isRemoteDirectory ifTrue: [^ self readServerBrief]].
	f := directory oldFileOrNoneNamed: self fullName.
	f ifNil: [^ 'For some reason, this file cannot be read' translated].
	f converter: (self defaultEncoderFor: self fullName).
	(brevityFlag not or: [(fileSize := f size) <= 100000]) ifTrue:
		[contents := f contentsOfEntireFile.
		brevityState := #fullFile.   "don't change till actually read"
		^ contents].

	"if brevityFlag is true, don't display long files when first selected"
	first5000 := f next: 5000.
	f close.
	contents := 'File ''{1}'' is {2} bytes long.
You may use the ''get'' command to read the entire file.

Here are the first 5000 characters...
------------------------------------------
{3}
------------------------------------------
... end of the first 5000 characters.' translated format: {fileName. fileSize. first5000}.
	brevityState := #briefFile.   "don't change till actually read"
	^ contents.
! !

!FileList methodsFor: 'private' stamp: 'ka 8/24/2000 18:55'!
readContentsCNGB
	| f writeStream |
	f := directory oldFileOrNoneNamed: self fullName.
	f ifNil: [^ 'For some reason, this file cannot be read'].
	writeStream := WriteStream on: String new.
	f converter: CNGBTextConverter new.
	[f atEnd]
		whileFalse: [writeStream nextPut: f next].
	f close.
	^ writeStream contents! !

!FileList methodsFor: 'private' stamp: 'ka 8/24/2000 18:31'!
readContentsEUCJP
	| f writeStream |
	f := directory oldFileOrNoneNamed: self fullName.
	f ifNil: [^ 'For some reason, this file cannot be read'].
	writeStream := WriteStream on: String new.
	f converter: EUCJPTextConverter new.
	[f atEnd]
		whileFalse: [writeStream nextPut: f next].
	f close.
	^ writeStream contents! !

!FileList methodsFor: 'private' stamp: 'ka 8/24/2000 18:56'!
readContentsEUCKR
	| f writeStream |
	f := directory oldFileOrNoneNamed: self fullName.
	f ifNil: [^ 'For some reason, this file cannot be read'].
	writeStream := WriteStream on: String new.
	f converter: EUCKRTextConverter new.
	[f atEnd]
		whileFalse: [writeStream nextPut: f next].
	f close.
	^ writeStream contents! !

!FileList methodsFor: 'private' stamp: 'BG 3/16/2005 08:22'!
readContentsHex: brevity
	"retrieve the contents from the external file unless it is too long.
	  Don't create a file here.  Check if exists."
	| f size data hexData s |

	f := directory oldFileOrNoneNamed: self fullName. 
	f == nil ifTrue: [^ 'For some reason, this file cannot be read' translated].
	f binary.
	((size := f size)) > 5000 & brevity
		ifTrue: [data := f next: 10000. f close. brevityState := #briefHex]
		ifFalse: [data := f contentsOfEntireFile. brevityState := #fullHex].

	s := WriteStream on: (String new: data size*4).
	0 to: data size-1 by: 16 do:
		[:loc | s nextPutAll: loc printStringHex; space;
			nextPut: $(; print: loc; nextPut: $); space; tab.
		loc+1 to: (loc+16 min: data size) do: [:i | s nextPutAll: (data at: i) printStringHex; space].
		s cr].
	hexData := s contents.

	^ contents := ((size > 5000) & brevity
		ifTrue: ['File ''{1}'' is {2} bytes long.
You may use the ''get'' command to read the entire file.

Here are the first 5000 characters...
------------------------------------------
{3}
------------------------------------------
... end of the first 5000 characters.' translated format: {fileName. size. hexData}]
		ifFalse: [hexData]).
! !

!FileList methodsFor: 'private' stamp: 'ka 8/26/2000 18:48'!
readContentsShiftJIS
	| f writeStream |
	f := directory oldFileOrNoneNamed: self fullName.
	f ifNil: [^ 'For some reason, this file cannot be read'].
	writeStream := WriteStream on: String new.
	f converter: ShiftJISTextConverter new.
	[f atEnd]
		whileFalse: [writeStream nextPut: f next].
	f close.
	^ writeStream contents! !

!FileList methodsFor: 'private' stamp: 'ka 6/23/2002 15:55'!
readContentsUTF8
	| f writeStream |
	f := directory oldFileOrNoneNamed: self fullName.
	f ifNil: [^ 'For some reason, this file cannot be read'].
	writeStream := WriteStream on: String new.
	f converter: UTF8TextConverter new.
	[f atEnd]
		whileFalse: [writeStream nextPut: f next].
	f close.
	^ writeStream contents! !

!FileList methodsFor: 'private' stamp: 'hmm 4/27/2010 20:45'!
readGraphicContents
	| form maxExtent ext |
	form := Form fromFileNamed: self fullName.
	maxExtent := lastGraphicsExtent := self availableGraphicsExtent.
	ext := form extent.
	(maxExtent notNil and: [form extent <= maxExtent]) ifFalse: [
		form := form magnify: form boundingBox by: (maxExtent x / form width min: maxExtent y / form height) asPoint smoothing: 3].
	contents :=  ('Image extent: ', ext printString) asText,
				(String with: Character cr),
				(Text string: ' '
					attribute: (TextFontReference toFont: 
						(FormSetFont new
							fromFormArray: (Array with: form)
							asciiStart: Character space asInteger
							ascent: form height))).
	brevityState := #graphic.
	^contents! !

!FileList methodsFor: 'file list' stamp: 'ul 12/12/2009 14:11'!
readOnlyStream
	"Answer a read-only stream on the selected file. For the various stream-reading services."

	^self directory ifNotNil: [ :dir | dir readOnlyFileNamed: self fileName ]! !

!FileList methodsFor: 'private' stamp: 'dgd 12/27/2003 12:09'!
readServerBrief
	| lString sizeStr fsize ff first5000 parts |
	"If file on server is known to be long, just read the beginning.  Cheat badly by reading the fileList string."

	listIndex = 0 ifTrue: [^ self].
	"Get size from file list entry"
	lString := list at: listIndex.
	parts := lString findTokens: '()'.
	sortMode = #name ifTrue: [sizeStr := (parts second findTokens: ' ') third].
	sortMode = #date ifTrue: [sizeStr := (parts first findTokens: ' ') third].
	sortMode = #size ifTrue: [sizeStr := (parts first findTokens: ' ') first].
	fsize := (sizeStr copyWithout: $,) asNumber.

	fsize <= 50000 ifTrue:
		[ff := directory oldFileOrNoneNamed: self fullName.
		ff ifNil: [^ 'For some reason, this file cannot be read' translated].
		contents := ff contentsOfEntireFile.
		brevityState := #fullFile.   "don't change till actually read"
		^ contents].

	"if brevityFlag is true, don't display long files when first selected"
	first5000 := directory getOnly: 3500 from: fileName.
	contents := 'File ''{1}'' is {2} bytes long.
You may use the ''get'' command to read the entire file.

Here are the first 3500 characters...
------------------------------------------
{3}
------------------------------------------
... end of the first 3500 characters.' translated format: {fileName. sizeStr. first5000}.
	brevityState := #briefFile.   "don't change till actually read"
	^ contents.

! !

!FileList methodsFor: 'private' stamp: 'ar 8/6/2009 18:50'!
recentDirs
	"Put up a menu and let the user select from the list of recently visited directories."

	| dirName |
	RecentDirs isEmpty ifTrue: [^self].
	dirName := UIManager default chooseFrom: RecentDirs values: RecentDirs.
	dirName == nil ifTrue: [^self].
	self directory: (FileDirectory on: dirName)! !

!FileList methodsFor: 'private' stamp: 'SD 11/8/2001 21:11'!
registeredFileReaderClasses
	"return the list of classes that provide file reader services"

	^ self class registeredFileReaderClasses! !

!FileList methodsFor: 'initialization' stamp: 'di 5/11/1999 22:25'!
release

	self modelSleep! !

!FileList methodsFor: 'server list' stamp: 'ar 8/6/2009 18:51'!
removeServer

	| choice names |
	self flag: #ViolateNonReferenceToOtherClasses.
	names := ServerDirectory serverNames asSortedArray.
	choice := UIManager default chooseFrom: names values: names.
	choice == nil ifTrue: [^ self].
	ServerDirectory removeServerNamed: choice! !

!FileList methodsFor: 'file menu action' stamp: 'rbb 3/1/2005 10:53'!
renameFile
	"Rename the currently selected file"
	| newName response |
	listIndex = 0 ifTrue: [^ self].
	self okToChange ifFalse: [^ self].
	(response := UIManager default request: 'NewFileName?' translated
 					initialAnswer: fileName)
		isEmpty ifTrue: [^ self].
	newName := response asFileName.
	newName = fileName ifTrue: [^ self].
	directory rename: fileName toBe: newName.
	self updateFileList.
	listIndex := list findFirst: [:item | (self fileNameFromFormattedItem: item) = newName].
	listIndex > 0 ifTrue: [fileName := newName].
	self changed: #fileListIndex.
! !

!FileList methodsFor: 'private' stamp: 'cmm 6/20/2010 16:33'!
resort: newMode 
	"Re-sort the list of files."
	| name |
	listIndex > 0 ifTrue: [ name := self fileNameFromFormattedItem: (list at: listIndex) ].
	sortMode := newMode.
	self pattern: pattern.
	name ifNotNil:
		[ fileName := name.
		listIndex := list findFirst:
			[ : item | (self fileNameFromFormattedItem: item) = name ].
		self
			 changed: #fileListIndex ;
			 changed: #labelString ].
	listIndex = 0 ifTrue: [ self changed: #contents ].
	self updateButtonRow! !

!FileList methodsFor: 'directory tree' stamp: 'ar 10/4/2009 20:34'!
rootDirectoryList
	| dirList dir servers |
	dir := FileDirectory on: ''.
	dirList := dir directoryNames collect:[:each| dir directoryNamed: each]..
	dirList isEmpty ifTrue:[dirList := Array with: FileDirectory default].
	servers := ServerDirectory serverNames collect: [ :n | ServerDirectory serverNamed: n].
	"This is so FileListPlus will work on ancient Squeak versions."
	servers := servers select:[:each| each respondsTo: #localName].
	^dirList, servers! !

!FileList methodsFor: 'private' stamp: 'bf 9/2/2009 12:45'!
selectEncoding

	| encodingItems |
	encodingItems := OrderedCollection new.
	TextConverter allSubclasses do: [:each | | names |
		names := each encodingNames.
		names notEmpty ifTrue: [ | label |
			label := '' writeStream.
			names do: [:eachName | label nextPutAll: eachName ] separatedBy: [ label nextPutAll: ', '].
			encodingItems add: {label contents. names first asSymbol}.
		].
	].
	brevityState := UIManager default
		chooseFrom: (encodingItems collect: [:spec| spec first])
		values: (encodingItems collect: [:spec| spec last]).
	brevityState ifNil: [brevityState := #needToGetBrief].
! !

!FileList methodsFor: 'directory tree' stamp: 'nice 12/27/2009 03:10'!
selectedPath
	| top here |
	top := FileDirectory root.
	here := directory.
	^(Array streamContents:[:s| | next |
		s nextPut: here.
		[next := here containingDirectory.
		top pathName = next pathName] whileFalse:[
			s nextPut: next.
			here := next.
		]]) reversed.! !

!FileList methodsFor: 'own services' stamp: 'KR 3/21/2008 21:09'!
serviceAddNewDirectory
	"Answer a service entry characterizing the 'add new directory' command"

	^ SimpleServiceEntry 
		provider: self 
		label: 'add new directory' translatedNoop
		selector: #addNewDirectory
		description: 'adds a new, empty directory (folder)'  translatedNoop! !

!FileList methodsFor: 'own services' stamp: 'KR 3/21/2008 21:09'!
serviceAddNewFile
	"Answer a service entry characterizing the 'add new file' command"

	^ SimpleServiceEntry 
			provider: self 
			label: 'add new file' translatedNoop
			selector: #addNewFile 
			description: 'create a new,. empty file, and add it to the current directory.' translatedNoop! !

!FileList methodsFor: 'own services' stamp: 'KR 3/21/2008 21:08'!
serviceAllFileOptions

	^ {SimpleServiceEntry 
			provider: self 
			label: 'more...' translatedNoop
			selector: #offerAllFileOptions 
			description: 'show all the options available' translatedNoop}! !

!FileList methodsFor: 'own services' stamp: 'KR 3/21/2008 21:04'!
serviceCompressFile
	"Answer a service for compressing a file"

	^ SimpleServiceEntry 
		provider: self 
		label: 'compress' translatedNoop
		selector: #compressFile 
		description: 'compress file' translatedNoop
		buttonLabel: 'compress' translatedNoop! !

!FileList methodsFor: 'own services' stamp: 'KR 3/21/2008 21:04'!
serviceCopyName

	^ (SimpleServiceEntry 
		provider: self 
		label: 'copy name to clipboard' translatedNoop
		selector: #copyName 
		description:'copy name to clipboard' translatedNoop )! !

!FileList methodsFor: 'own services' stamp: 'KR 3/21/2008 21:03'!
serviceDeleteFile

	^ (SimpleServiceEntry 
			provider: self 
			label: 'delete' translatedNoop
			selector: #deleteFile)
			description: 'delete the seleted item' translatedNoop! !

!FileList methodsFor: 'own services' stamp: 'KR 3/21/2008 21:01'!
serviceGet
	"Answer a service for getting the entire file"

	^  (SimpleServiceEntry 
			provider: self 
			label: 'get entire file' translatedNoop
			selector: #get
			description: 'if the file has only been partially read in, because it is very large, read the entire file in at this time.' translatedNoop)! !

!FileList methodsFor: 'own services' stamp: 'KR 3/21/2008 21:00'!
serviceGetEncodedText

	^  (SimpleServiceEntry 
			provider: self 
			label: 'view as encoded text' translatedNoop
			selector: #getEncodedText
			description: 'view as encoded text' translatedNoop)

! !

!FileList methodsFor: 'own services' stamp: 'KR 3/21/2008 21:00'!
serviceGetHex

	^  (SimpleServiceEntry 
			provider: self 
			label: 'view as hex' translatedNoop
			selector: #getHex
			description: 'view as hex' translatedNoop)
			
! !

!FileList methodsFor: 'own services' stamp: 'KR 3/21/2008 20:50'!
serviceRenameFile

	^ (SimpleServiceEntry 
			provider: self 
			label: 'rename' translatedNoop
			selector: #renameFile 
			description: 'rename file' translatedNoop)! !

!FileList methodsFor: 'own services' stamp: 'KR 3/21/2008 20:49'!
serviceSortByDate
	"Answer a service for sorting by date"

	^  (SimpleServiceEntry new
			provider: self 
			label: 'by date' translatedNoop 
			selector: #sortByDate 
			description: 'sort entries by date' translatedNoop)
		extraSelector: #sortingByDate;
		buttonLabel: 'date' translatedNoop! !

!FileList methodsFor: 'own services' stamp: 'KR 3/21/2008 20:48'!
serviceSortByName
	"Answer a service for soring by name"

	^ (SimpleServiceEntry new
		provider: self label: 'by name' translatedNoop
		selector: #sortByName 
		description: 'sort entries by name' translatedNoop)
		extraSelector: #sortingByName;
		buttonLabel: 'name' translatedNoop! !

!FileList methodsFor: 'own services' stamp: 'KR 3/21/2008 20:48'!
serviceSortBySize
	"Answer a service for sorting by size"

	^  (SimpleServiceEntry 
			provider: self 
			label: 'by size' translatedNoop
			selector: #sortBySize
			description: 'sort entries by size' translatedNoop)
				extraSelector: #sortingBySize;
				buttonLabel: 'size' translatedNoop! !

!FileList methodsFor: 'file list menu' stamp: 'topa 3/16/2015 16:56'!
servicesFileContentsMenu: aMenu
	"Construct aMenu to have items appropriate for the file browser's code pane, for the unshifted state"
	<fileContentsMenuShifted: false>
	
	fileName ifNotNil: [| services maybeLine extraLines | 
		services := OrderedCollection new.
		(#(briefHex briefFile needToGetBriefHex needToGetBrief) includes: brevityState) ifTrue:
			[services add: self serviceGet].
		(#(fullHex briefHex needToGetFullHex needToGetBriefHex) includes: brevityState) ifFalse:
			[services add: self serviceGetHex].
		(#(needToGetShiftJIS needToGetEUCJP needToGetCNGB needToGetEUCKR needToGetUTF8) includes: brevityState) ifFalse:
			[services add: self serviceGetEncodedText].
		maybeLine := services size.
		(FileStream sourceFileSuffixes includes: self suffixOfSelectedFile) ifTrue:
			[services addAll:
				(self servicesFromSelectorSpecs:
					#(fileIntoNewChangeSet: fileIn: browseChangesFile: browseFile:))].

		extraLines := OrderedCollection new.
		maybeLine > 0 ifTrue: [extraLines add: maybeLine].
		services size > maybeLine ifTrue: [extraLines add: services size].
		aMenu 
			addServices: services
			for: self
			extraLines: extraLines].
	^ aMenu
! !

!FileList methodsFor: 'own services' stamp: 'nice 12/27/2009 03:10'!
servicesFromSelectorSpecs: symbolArray
	"Answer an array of services represented by the incoming symbols, eliminating any that do not have a currently-registered service.  Pass the symbol #- along unchanged to serve as a separator between services"

	"FileList new servicesFromSelectorSpecs: #(fileIn: fileIntoNewChangeSet: browseChangesFile:)"

	| services col | 
	col := OrderedCollection new.
	services := self class allRegisteredServices, (self myServicesForFile: #dummy suffix: '*').
	symbolArray do: 
		[:sel | | res | 
			sel == #-
				ifTrue:
					[col add: sel]
				ifFalse:
					[res := services
							detect: [:each | each selector = sel] ifNone: [nil].
					res notNil
							ifTrue: [col add: res]]].
	^ col! !

!FileList methodsFor: 'volume menu' stamp: 'topa 3/16/2015 17:05'!
servicesVolumeMenu: aMenu
	<volumeMenu>
	aMenu
		addServices: (self itemsForDirectory: self directory)
		for: self
		extraLines: #().
	^aMenu.! !

!FileList methodsFor: 'own services' stamp: 'KR 3/21/2008 20:47'!
serviceViewContentsInWorkspace
	"Answer a service for viewing the contents of a file in a workspace"
	
	^ (SimpleServiceEntry provider: self label: 'workspace with contents' translatedNoop
			selector: #viewContentsInWorkspace)
			description: 'open a new Workspace whose contents are set to the contents of this file' translatedNoop! !

!FileList methodsFor: 'directory tree' stamp: 'ar 10/6/2009 00:54'!
setDirectoryTo: dir
	"Set the current directory shown in the FileList. 
	Does not allow setting the directory to nil since this blows up in various places."
	dir ifNil:[^self].
	self directory: dir.
	brevityState := #FileList.
	self changed: #fileList.
	self changed: #contents.
	self changed: #pathAndPattern.! !

!FileList methodsFor: 'initialization' stamp: 'cmm 6/20/2010 16:34'!
setFileStream: aStream
	"Used to initialize a spawned file editor.  Sets directory too."
	self directory: aStream directory.
	fileName := aStream localName.
	pattern := '*'.
	listIndex := 1.  "pretend a file is selected"
	aStream close.
	brevityState := #needToGetBrief.
	self 
		changed: #contents ; 
		changed: #labelString! !

!FileList methodsFor: 'file list menu' stamp: 'topa 3/16/2015 16:55'!
shiftedFileContentsMenu: aMenu
	"Delegate the shifted menu to the default shifted menu"
	<fileContentsMenuShifted: true>
	^ aMenu addList: self class shiftedYellowButtonMenuItems; yourself
! !

!FileList methodsFor: 'private' stamp: 'sma 11/11/2000 17:04'!
sortBlock
	"Answer block to decide what order to display the directory entries."

	^ [ :x :y |
			(x isDirectory = y isDirectory) 
				ifTrue: [  
					"sort by user-specified criterion"
					sortMode = #name 
						ifTrue: [(x name compare: y name) <= 2]
						ifFalse: [ sortMode = #date
							ifTrue: [ x modificationTime = y modificationTime
									ifTrue: [ (x name compare: y name) <= 2 ]
									ifFalse: [ x modificationTime > y modificationTime ] ]
							ifFalse: [ "size"
								x fileSize = y fileSize 
									ifTrue: [ (x name compare: y name) <= 2 ]
									ifFalse: [ x fileSize > y fileSize ] ] ] ]
				ifFalse: [
					"directories always precede files"
					x isDirectory ] ]! !

!FileList methodsFor: 'file menu action' stamp: 'di 4/15/98 12:36'!
sortByDate
	self resort: #date! !

!FileList methodsFor: 'file menu action' stamp: 'di 4/15/98 12:37'!
sortByName
	self resort: #name! !

!FileList methodsFor: 'file menu action' stamp: 'di 4/15/98 12:36'!
sortBySize
	self resort: #size! !

!FileList methodsFor: 'private' stamp: 'sw 1/7/2000 15:58'!
sortingByDate
	^ sortMode == #date! !

!FileList methodsFor: 'private' stamp: 'sw 1/7/2000 15:57'!
sortingByName
	^ sortMode == #name! !

!FileList methodsFor: 'private' stamp: 'sw 1/7/2000 15:58'!
sortingBySize
	^ sortMode == #size! !

!FileList methodsFor: 'file menu action' stamp: 'sd 2/1/2002 20:02'!
spawn: code
	"Open a simple Edit window"

	listIndex = 0 ifTrue: [^ self].
	self class openEditorOn: (directory readOnlyFileNamed: fileName)
				"read only just for initial look"
			editString: code! !

!FileList methodsFor: 'private' stamp: 'hmm 4/27/2010 17:02'!
step
	| ext |
	brevityState = #graphic ifTrue: [
		ext := self availableGraphicsExtent.
		ext = lastGraphicsExtent ifFalse: [
			lastGraphicsExtent := ext.
			brevityState := #needToGetGraphic.
			self changed: #contents]]! !

!FileList methodsFor: 'private' stamp: 'hmm 4/27/2010 17:03'!
stepTimeIn: aWindow
	^500! !

!FileList methodsFor: 'directory tree' stamp: 'ar 10/4/2009 20:34'!
subDirectoriesOf: aDirectory
	^aDirectory directoryNames collect:[:each| aDirectory directoryNamed: each].! !

!FileList methodsFor: 'file list menu' stamp: 'nk 12/7/2002 12:57'!
suffixOfSelectedFile
	"Answer the file extension of the receiver's selected file"
	^ self class suffixOf: self fullName.! !

!FileList methodsFor: 'toolbuilder' stamp: 'fbs 5/1/2010 22:56'!
topConstantHeightFrame: height fromLeft: leftFraction width: rightFraction
	^LayoutFrame new
		topFraction: 0 offset: 0;
		leftFraction: leftFraction offset: 0;
		rightFraction: (leftFraction + rightFraction) offset: 0;
		bottomFraction: 0 offset: height;
		yourself.! !

!FileList methodsFor: 'initialization' stamp: 'sw 2/17/2002 05:38'!
universalButtonServices
	"Answer a list of services underlying the universal buttons in their initial inception.  For the moment, only the sorting buttons are shown."

	^ {self serviceSortByName. self serviceSortByDate. self serviceSortBySize}! !

!FileList methodsFor: 'initialization' stamp: 'ar 10/4/2009 20:49'!
updateButtonRow
	"Dynamically update the contents of the button row, if any."
	self changed: #getButtonRow.! !

!FileList methodsFor: 'private' stamp: 'nk 12/10/2002 07:57'!
updateFileList
	"Update my files list with file names in the current directory  
	that match the pattern.
	The pattern string may have embedded newlines or semicolons; these separate different patterns."
	| patterns |
	patterns := OrderedCollection new.
	Cursor wait showWhile: [
	(pattern findTokens: (String with: Character cr with: Character lf with: $;))
		do: [ :each |
			(each includes: $*) | (each includes: $#)
					ifTrue: [ patterns add: each]
					ifFalse: [each isEmpty
										ifTrue: [ patterns add: '*']
										ifFalse: [ patterns add: '*' , each , '*']]].

	list := self listForPatterns: patterns.
	listIndex := 0.
	volListIndex := volList size.
	fileName := nil.
	contents := ''.
	self changed: #volumeListIndex.
	self changed: #fileList.
	self updateButtonRow]! !

!FileList methodsFor: 'updating' stamp: 'sw 11/30/2002 16:49'!
update: aParameter
	"Receive a change notice from an object of whom the receiver is a dependent"

	(aParameter == #fileListChanged) ifTrue: [self updateFileList].
	super update: aParameter! !

!FileList methodsFor: 'volume list and pattern' stamp: 'sw 3/6/1999 11:39'!
veryDeepFixupWith: deepCopier
	super veryDeepFixupWith: deepCopier.
	volListIndex := 1.
	self directory: FileDirectory default.
	self updateFileList! !

!FileList methodsFor: 'own services' stamp: 'edc 4/24/2007 09:49'!
viewContentsInWorkspace
	"View the contents of my selected file in a new workspace"
	
	| aString aFileStream aName |
	aString := (aFileStream := directory readOnlyFileNamed: self fullName) setConverterForCode contentsOfEntireFile.
	aName := aFileStream localName.
	aFileStream close.
	UIManager default edit: aString withSqueakLineEndings label: 'Workspace from ', aName! !

!FileList methodsFor: 'volume list and pattern' stamp: 'jm 5/3/1998 18:20'!
volumeList
	"Answer the current list of volumes."

	^ volList
! !

!FileList methodsFor: 'volume list and pattern' stamp: 'jm 5/3/1998 18:21'!
volumeListIndex
	"Answer the index of the currently selected volume."

	^ volListIndex
! !

!FileList methodsFor: 'volume list and pattern' stamp: 'ar 10/4/2009 21:08'!
volumeListIndex: index
	"Select the volume name having the given index."

	| delim path |
	volListIndex := index.
	index = 1 
		ifTrue: [self directory: (FileDirectory on: '')]
		ifFalse: [delim := directory pathNameDelimiter.
				path := String streamContents: [:strm |
					2 to: index do: [:i |
						strm nextPutAll: (volList at: i) withBlanksTrimmed.
						i < index ifTrue: [strm nextPut: delim]]].
				self directory: (directory on: path)].
	brevityState := #FileList.
	self addPath: path.
	self changed: #fileList.
	self changed: #contents.
	self updateButtonRow.! !

!FileList methodsFor: 'volume menu' stamp: 'topa 3/16/2015 17:04'!
volumeMenu: aMenu
	^ self menu: aMenu for: #(volumeMenu volumeMenuShifted:)
! !

!FileList methodsFor: 'drag''n''drop' stamp: 'mt 1/21/2016 15:27'!
wantsDraggedObject: anObject

	^ anObject class == DirectoryEntryFile! !

!FileList methodsFor: 'private' stamp: 'hmm 4/27/2010 16:56'!
wantsSteps
	^true! !

FileList subclass: #PluggableFileList
	instanceVariableNames: 'accepted fileFilterBlock canAcceptBlock validateBlock newFiles prompt resultBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-FileList'!

!PluggableFileList commentStamp: '<historical>' prior: 0!
I am a model for a modal dialog akin to "Standard File Services" on various modern GUI operating systems.  My protocol includes some methods to maintain upward compatibility with StandardFileMenu, which I hope to replace.

Sample doIts:

	"StandardFileDialog getFolder"-- ask user to select folder, answer corresponding FileDirectory
	"StandardFileDialog getFile"	-- ask user to select file, answer corresponding FileStream
	"StandardFileDialog putFile"	-- ask user to create new file, answer FileStream

  In addition to the instance variables inhereted from FileList, of which I am a subclass, I am pluggable via the following instance variables:

prompt			<String>
	Display a prompt between the buttons.

resultBlock		<BlockContext>
	Passed a file directory and a file name, answer the result to be answered by the dialog.	

canAcceptBlock	<BlockContext>	
	Answer whether the accept button should be "active"

fileFilterBlock	<BlockContext>	
	Passed a file directory entry, answer whether the entry should be added to the list.  The list can be further filtered (but not expanded) by the user through the pattern.

validateBlock	<BlockContent>
	Passed a file directory entry, a file name and a collection of newly created files, answer whether the dialog selection is valid.  validateBlock is checked after the user has pressed accept, for example to ask if the user really meant to delete a file.

newFiles		<OrderedCollection>

	newFiles is an OrderedCollection of fileNames of files added by the user to the current directory since the user last entered same!

!PluggableFileList class methodsFor: 'fileFilterBlocks' stamp: 'acg 2/10/2000 08:16'!
allFilesAndFoldersFileFilter

	^[:each | true]! !

!PluggableFileList class methodsFor: 'fileFilterBlocks' stamp: 'acg 2/10/2000 08:17'!
allFoldersFileFilter

	^[:each | each isDirectory]! !

!PluggableFileList class methodsFor: 'canAcceptBlocks' stamp: 'acg 2/10/2000 08:18'!
alwaysAcceptBlock

	^[:theDirectory :theFileName | true]! !

!PluggableFileList class methodsFor: 'validateBlocks' stamp: 'acg 2/10/2000 08:27'!
alwaysValidateBlock

	^[:theDirectory :theFileName :theNewFiles | true].! !

!PluggableFileList class methodsFor: 'validateBlocks' stamp: 'acg 2/12/2000 14:52'!
checkExistingFileValidateBlock

	^[:theDirectory :theFileName :theNewFiles | 
		(theNewFiles includes: theFileName) or:
			[(PluggableFileList okToOverwrite: theFileName)]].! !

!PluggableFileList class methodsFor: 'resultBlocks' stamp: 'acg 2/10/2000 08:19'!
directoryResultBlock

	^[:theDirectory :theFileName | theDirectory]! !

!PluggableFileList class methodsFor: 'validateBlocks' stamp: 'acg 2/12/2000 14:52'!
existingFileValidateBlock

	^[:theDirectory :theFileName :theNewFiles | 
		(theNewFiles includes: theFileName) or:
			[(PluggableFileList okToOverwrite: theFileName)]].! !

!PluggableFileList class methodsFor: 'resultBlocks' stamp: 'acg 2/10/2000 08:07'!
fileNameResultBlock

	^[:theDirectory :theFileName | theFileName]! !

!PluggableFileList class methodsFor: 'canAcceptBlocks' stamp: 'nice 8/27/2010 20:55'!
fileNameSelectedAcceptBlock

	^[:theDirectory :theFileName | theFileName notNil]! !

!PluggableFileList class methodsFor: 'standard dialog operations' stamp: 'acg 2/10/2000 08:24'!
getFile

	| result |
	result := self getFilePathName.
	^result ifNotNil: [FileStream oldFileNamed: result]! !

!PluggableFileList class methodsFor: 'standard dialog operations' stamp: 'acg 2/10/2000 08:31'!
getFilePathName

	^self getFilePathNameDialog open! !

!PluggableFileList class methodsFor: 'instance creation' stamp: 'acg 2/12/2000 14:52'!
getFilePathNameDialog

	^(self new)
		validateBlock: PluggableFileList alwaysValidateBlock;
		yourself! !

!PluggableFileList class methodsFor: 'instance creation' stamp: 'acg 2/12/2000 14:52'!
getFilePathNameDialogWithExistenceCheck

	^(self new)
		prompt: 'Select New File:';
		validateBlock: PluggableFileList checkExistingFileValidateBlock;
		yourself! !

!PluggableFileList class methodsFor: 'standard dialog operations' stamp: 'acg 2/10/2000 08:29'!
getFilePathNameWithExistenceCheck

	^self getFilePathNameDialogWithExistenceCheck open! !

!PluggableFileList class methodsFor: 'standard dialog operations' stamp: 'acg 2/9/2000 01:16'!
getFolder

	^self getFolderDialog open! !

!PluggableFileList class methodsFor: 'instance creation' stamp: 'acg 2/12/2000 14:52'!
getFolderDialog

	^(self new)
		prompt: 'Select a Folder';
		fileFilterBlock: PluggableFileList allFoldersFileFilter;
		canAcceptBlock: PluggableFileList alwaysAcceptBlock;
		resultBlock: PluggableFileList directoryResultBlock;
		validateBlock: PluggableFileList alwaysValidateBlock;
		yourself! !

!PluggableFileList class methodsFor: 'StandardFileMenu' stamp: 'BG 12/13/2002 15:31'!
newFileMenu: aDirectory

	"For compatibility with StandardFileMenu for now, answer a StandardFileMenuResult"
	^(self getFilePathNameDialogWithExistenceCheck)
		resultBlock: self sfmResultBlock;
		directory: aDirectory;
		yourself! !

!PluggableFileList class methodsFor: 'validateBlocks' stamp: 'rbb 2/18/2005 11:35'!
okToOpen: aFileNameString without: aSuffixString

	"Answer whether user confirms that it is ok to overwrite the file named in aString"
	^ 1 = (UIManager default 
				chooseFrom: #('overwrite that file' 'select another file')
				title:  aFileNameString, ' already exists.').
! !

!PluggableFileList class methodsFor: 'validateBlocks' stamp: 'rbb 2/18/2005 11:36'!
okToOverwrite: aString

	"Answer whether user confirms that it is ok to overwrite the file named in aString"
	^ 1 = (UIManager default chooseFrom: #('overwrite that file' 'select another file')
			title:  aString, ' already exists.')
! !

!PluggableFileList class methodsFor: 'StandardFileMenu' stamp: 'BG 12/13/2002 15:32'!
oldFileMenu: aDirectory

	"For compatibility with StandardFileMenu for now, answer a StandardFileMenuResult"
	^(self getFilePathNameDialog)
		resultBlock: self sfmResultBlock;
		directory: aDirectory;
		yourself! !

!PluggableFileList class methodsFor: 'instance creation' stamp: 'acg 2/9/2000 00:34'!
open

	^self new open! !

!PluggableFileList class methodsFor: 'resultBlocks' stamp: 'acg 2/10/2000 08:07'!
pathNameResultBlock

	^[:theDirectory :theFileName | 
		theFileName 
			ifNil: [theDirectory pathName]
			ifNotNil: [theDirectory fullNameFor: theFileName]].
! !

!PluggableFileList class methodsFor: 'standard dialog operations' stamp: 'acg 2/10/2000 08:29'!
putFile

	| result |
	result := self getFilePathNameWithExistenceCheck.
	^result ifNotNil: 
		[FileDirectory deleteFilePath: result.
		 FileStream newFileNamed: result]! !

!PluggableFileList class methodsFor: 'resultBlocks' stamp: 'acg 2/12/2000 15:08'!
sfmResultBlock

	^[:theDirectory :theFileName | 
		StandardFileMenuResult directory: theDirectory name: theFileName]! !

!PluggableFileList methodsFor: 'file list menu' stamp: 'rbb 3/1/2005 11:07'!
addNew: aString byEvaluating: aBlock
	"A parameterization of earlier versions of #addNewDirectory and
	#addNewFile.  Fixes the bug in each that pushing the cancel button
	in the FillInTheBlank dialog gave a walkback."

	| response newName index ending |
	self okToChange ifFalse: [^ self].
	(response := UIManager default request: 'New ',aString,' Name?'
 					initialAnswer: aString,'Name')
		isEmpty ifTrue: [^ self].
	newName := response asFileName.
	Cursor wait showWhile: [
		aBlock value: newName].
	self updateFileList.
	index := list indexOf: newName.
	index = 0 ifTrue: [ending := ') ',newName.
		index := list findFirst: [:line | line endsWith: ending]].
	self fileListIndex: index.
	newFiles add: newName
! !

!PluggableFileList methodsFor: 'accessing' stamp: 'acg 2/13/2000 15:33'!
beAccepted

	^accepted := true! !

!PluggableFileList methodsFor: 'accessing' stamp: 'acg 2/9/2000 09:10'!
canAccept

	^canAcceptBlock value: directory value: fileName! !

!PluggableFileList methodsFor: 'accessing' stamp: 'acg 2/9/2000 00:52'!
canAcceptBlock: aBlock

	^canAcceptBlock := aBlock! !

!PluggableFileList methodsFor: 'user interface' stamp: 'mt 7/9/2016 08:52'!
defaultWindowColor
	^ Color lightYellow! !

!PluggableFileList methodsFor: 'accessing' stamp: 'acg 2/9/2000 07:33'!
fileFilterBlock: aBlock

	^fileFilterBlock := aBlock! !

!PluggableFileList methodsFor: 'accessing' stamp: 'acg 2/18/2000 21:21'!
fileListIndex

	self changed: #fileString.
	^super fileListIndex! !

!PluggableFileList methodsFor: 'file list menu' stamp: 'asm 8/25/2003 18:37'!
fileSelectedMenu: aMenu
	| firstItems secondItems thirdItems n1 n2 n3 services |
	firstItems := self itemsForFile: self fullName asLowercase.
	secondItems := self itemsForAnyFile.
	thirdItems := self itemsForNoFile.
	n1 := firstItems size.
	n2 := n1 + secondItems size.
	n3 := n2 + thirdItems size.
	services := firstItems, secondItems, thirdItems, 
			(OrderedCollection with: (SimpleServiceEntry provider: self label: 'more...' selector: #offerAllFileOptions)).
	^ aMenu 
		addServices2: services 
		for: self
		extraLines: (Array with: n1 with: n2 with: n3)
! !

!PluggableFileList methodsFor: 'file string' stamp: 'acg 2/18/2000 21:23'!
fileString

	fileName ifNil: [^directory pathName].
	^directory fullNameFor: fileName! !

!PluggableFileList methodsFor: 'file string' stamp: 'acg 2/19/2000 01:02'!
fileString: aString

	"| textName index ending |
	textName := aString asString.
	(FileDirectory default fileExists: textName) ifTrue:
		[self directory: (FileDirectory forFileName: textName).
		 index := list indexOf: (FileDirectory localNameFor: textName).
		 index = 0 ifTrue: 
			[ending := ') ', (FileDirectory localNameFor: textName).
		  	 index := list findFirst: [:line | line endsWith: ending]].
		 self fileListIndex: index].
	(FileDirectory default directoryExists: textName) ifTrue:
		[self directory: (FileDirectory on: textName)]."
	self changed: #fileString.
	self changed: #contents.
	^true! !

!PluggableFileList methodsFor: 'accessing' stamp: 'acg 2/18/2000 21:21'!
fileVolumeIndex

	self changed: #fileString.
	^super fileVolumeIndex! !

!PluggableFileList methodsFor: 'initialize-release' stamp: 'acg 2/12/2000 14:52'!
initialize

	prompt := 'Select File'.
	directory := FileDirectory default.
	newFiles := OrderedCollection new.
	fileFilterBlock := PluggableFileList allFilesAndFoldersFileFilter.
	canAcceptBlock := PluggableFileList fileNameSelectedAcceptBlock.
	resultBlock := PluggableFileList pathNameResultBlock.
	validateBlock := PluggableFileList checkExistingFileValidateBlock.
! !

!PluggableFileList methodsFor: 'accepting/cancelling' stamp: 'acg 2/9/2000 01:05'!
leftButtonPressed

	accepted := false.
	self changed: #close.
! !

!PluggableFileList methodsFor: 'accepting/cancelling' stamp: 'acg 2/14/2000 22:40'!
leftButtonState

	^true! !

!PluggableFileList methodsFor: 'file list menu' stamp: 'nice 3/21/2010 21:27'!
listForPattern: pat
	"Make the list be those file names which match the pattern."
	| entries sizePad newList allFiles |
	entries := directory entries select: fileFilterBlock.
	sizePad := (entries inject: 0 into: [:mx :entry | mx max: entry fileSize])
					asStringWithCommas size - 1.

	newList := OrderedCollection new: entries size.

	allFiles := pat = '*'.
	entries do:
		[:entry | "<dirflag><name><creationTime><modificationTime><fileSize>"
		(allFiles or: [entry isDirectory or: [pat match: entry name]]) ifTrue:
			[newList add: entry]].
	newList := newList asArray sort: self sortBlock.
	newList := newList collect: [ :e | self fileNameFormattedFrom: e sizePad: sizePad ].

	volList size = 1 ifTrue:
		["Include known servers along with other desktop volumes" 
		^newList ,
			(ServerDirectory serverNames collect: [:n | '^' , n , self folderString])].
	newFiles := OrderedCollection new.
	^ newList! !

!PluggableFileList methodsFor: 'initialize-release' stamp: 'dtl 2/2/2010 21:14'!
open

	^ Project current
		dispatchTo: self
		addPrefixAndSend: #OpenLabel:in:
		withArguments: { prompt . self currentWorld }
! !

!PluggableFileList methodsFor: 'accessing' stamp: 'acg 2/9/2000 00:28'!
prompt: aString

	 prompt := aString! !

!PluggableFileList methodsFor: 'accepting/cancelling' stamp: 'acg 2/9/2000 00:38'!
result

	accepted ifFalse: [^nil].
	^resultBlock value: directory value: fileName! !

!PluggableFileList methodsFor: 'accessing' stamp: 'acg 2/9/2000 00:31'!
resultBlock: aBlock

	^resultBlock := aBlock! !

!PluggableFileList methodsFor: 'accepting/cancelling' stamp: 'acg 2/10/2000 07:58'!
rightButtonPressed

	(canAcceptBlock value: directory value: fileName) ifFalse: [^nil].
	(validateBlock value: directory value: fileName value: newFiles) ifFalse: [^nil].
	accepted := true.
	self changed: #close! !

!PluggableFileList methodsFor: 'accepting/cancelling' stamp: 'acg 2/14/2000 22:43'!
rightButtonState

	^self canAccept! !

!PluggableFileList methodsFor: 'StandardFileMenu' stamp: 'ar 3/18/2001 00:55'!
startUpWithCaption: captionOrNil
	"Display the menu, slightly offset from the cursor,
	so that a slight tweak is required to confirm any action."
	^ self startUpWithCaption: captionOrNil at: (ActiveHand ifNil:[Sensor cursorPoint]).! !

!PluggableFileList methodsFor: 'StandardFileMenu' stamp: 'acg 2/12/2000 15:35'!
startUpWithCaption: aString at: location

	self prompt: aString.
	^self open! !

!PluggableFileList methodsFor: 'accessing' stamp: 'acg 2/13/2000 15:28'!
validate

	^validateBlock value: directory value: fileName value: newFiles! !

!PluggableFileList methodsFor: 'accessing' stamp: 'acg 2/10/2000 08:02'!
validateBlock: aBlock

	^validateBlock := aBlock! !

StringHolder subclass: #Inspector
	instanceVariableNames: 'object selectionIndex timeOfLastListUpdate selectionUpdateTime context expression'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Inspector'!

!Inspector commentStamp: '<historical>' prior: 0!
I represent a query path into the internal representation of an object. As a StringHolder, the string I represent is the value of the currently selected variable of the observed object.!

Inspector subclass: #BasicInspector
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Inspector'!

!BasicInspector methodsFor: 'as yet unclassified' stamp: 'ajh 1/31/2003 15:49'!
inspect: anObject 
	"Initialize the receiver so that it is inspecting anObject. There is no 
	current selection."

	self initialize.
	object := anObject.
	selectionIndex := 0.
	contents := ''! !

Inspector subclass: #CompiledMethodInspector
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Inspector'!

!CompiledMethodInspector methodsFor: 'selecting' stamp: 'ajh 3/20/2003 00:17'!
contentsIsString
	"Hacked so contents empty when deselected"

	^ #(0 2 3) includes: selectionIndex! !

!CompiledMethodInspector methodsFor: 'accessing' stamp: 'sd 11/20/2005 21:27'!
fieldList

	| keys |
	keys := OrderedCollection new.
	keys add: 'self'.
	keys add: 'all bytecodes'.
	keys add: 'header'.
	1 to: object numLiterals do: [ :i |
		keys add: 'literal', i printString ].
	object initialPC to: object size do: [ :i |
		keys add: i printString ].
	^ keys asArray
	! !

!CompiledMethodInspector methodsFor: 'selecting' stamp: 'sd 11/20/2005 21:27'!
selection

	| bytecodeIndex |
	selectionIndex = 0 ifTrue: [^ ''].
	selectionIndex = 1 ifTrue: [^ object ].
	selectionIndex = 2 ifTrue: [^ object symbolic].
	selectionIndex = 3 ifTrue: [^ object headerDescription].
	selectionIndex <= (object numLiterals + 3) 
		ifTrue: [ ^ object objectAt: selectionIndex - 2 ].
	bytecodeIndex := selectionIndex - object numLiterals - 3.
	^ object at: object initialPC + bytecodeIndex - 1! !

!CompiledMethodInspector methodsFor: 'selecting' stamp: 'ajh 3/20/2001 11:56'!
selectionUnmodifiable
	"Answer if the current selected variable is unmodifiable via acceptance in the code pane.  For most inspectors, no selection and a selection of self (selectionIndex = 1) are unmodifiable"

	^ true! !

Inspector subclass: #ContextInspector
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Inspector'!

!ContextInspector methodsFor: 'accessing' stamp: 'ar 8/19/2010 12:24'!
fieldList
	"Answer the base field list plus an abbreviated list of indices."

	^ self baseFieldList , (object tempNames collect:[:t| '[',t,']'])! !

!ContextInspector methodsFor: 'accessing' stamp: 'ar 8/19/2010 12:21'!
selection
	"The receiver has a list of variables of its inspected object.
	One of these is selected. Answer the value of the selected variable."
	| basicIndex |
	selectionIndex = 0 ifTrue: [^ ''].
	selectionIndex = 1 ifTrue: [^ object].
	selectionIndex = 2 ifTrue: [^ object longPrintStringLimitedTo: 20000].
	(selectionIndex - 2) <= object class instSize
		ifTrue: [^ object instVarAt: selectionIndex - 2].
	basicIndex := selectionIndex - 2 - object class instSize.
	^object debuggerMap namedTempAt: basicIndex in: object
! !

Inspector subclass: #ContextVariablesInspector
	instanceVariableNames: 'fieldList'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Debugger'!

!ContextVariablesInspector commentStamp: '<historical>' prior: 0!
I represent a query path into the internal representation of a ContextPart. Typically this is a context at a point in the query path of a Debugger. As a StringHolder, the string I represent is the value of the currently selected variable of the observed temporary variable of the context.!

!ContextVariablesInspector methodsFor: 'selecting' stamp: 'bf 5/11/2016 18:47'!
contentsIsString
	"Hacked so contents empty when deselected and = long printString when item 3"

	^ (selectionIndex = 3) | (selectionIndex = 0) |
		(selectionIndex = 2 and: [object actualStackSize = 0])! !

!ContextVariablesInspector methodsFor: 'selecting' stamp: 'eem 3/12/2009 15:09'!
defaultIntegerBase
	"Answer the default base in which to print integers.
	 Defer to the class the code is compiled in."
	^[object method methodClass defaultIntegerBaseInDebugger]
		on: MessageNotUnderstood
		do: [:ex| 10]! !

!ContextVariablesInspector methodsFor: 'code' stamp: ''!
doItContext

	^object! !

!ContextVariablesInspector methodsFor: 'code' stamp: ''!
doItReceiver

	^object receiver! !

!ContextVariablesInspector methodsFor: 'accessing' stamp: 'eem 5/21/2008 12:31'!
fieldList 
	"Refer to the comment in Inspector|fieldList."

	object == nil ifTrue: [^Array with: 'thisContext'].
	^fieldList ifNil:[fieldList := (Array with: 'thisContext' with: 'stack top' with: 'all temp vars') , object tempNames]! !

!ContextVariablesInspector methodsFor: 'accessing' stamp: 'ar 4/11/2006 02:33'!
inspect: anObject 
	"Initialize the receiver so that it is inspecting anObject. There is no 
	current selection.
	
	Because no object's inspectorClass method answers this class, it is OK for this method to
	override Inspector >> inspect: "
	fieldList := nil.
	object := anObject.
	self initialize.
	! !

!ContextVariablesInspector methodsFor: 'selecting' stamp: 'eem 7/18/2008 11:18'!
replaceSelectionValue: anObject 
	"Refer to the comment in Inspector|replaceSelectionValue:."

	^selectionIndex = 1
		ifTrue: [object]
		ifFalse: [object namedTempAt: selectionIndex - 3 put: anObject]! !

!ContextVariablesInspector methodsFor: 'selecting' stamp: 'bf 5/11/2016 18:48'!
selection 
	"Refer to the comment in Inspector|selection."
	selectionIndex = 0 ifTrue:[^''].
	selectionIndex = 1 ifTrue: [^object].
	selectionIndex = 2 ifTrue: [^object actualStackSize > 0 ifTrue: [object top] ifFalse: ['']].
	selectionIndex = 3 ifTrue: [^object tempsAndValues].
	^object debuggerMap namedTempAt: selectionIndex - 3 in: object! !

Inspector subclass: #DictionaryInspector
	instanceVariableNames: 'keyArray'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Inspector'!

!DictionaryInspector methodsFor: 'menu' stamp: 'rbb 3/1/2005 10:51'!
addEntry
	| newKey aKey |

	newKey := UIManager default request:
'Enter new key, then type RETURN.
(Expression will be evaluated for value.)
Examples:  #Fred    ''a string''   3+4'.
	aKey := Compiler evaluate: newKey.
	object at: aKey put: nil.
	self calculateKeyArray.
	selectionIndex := self numberOfFixedFields + (keyArray indexOf: aKey).
	self changed: #inspectObject.
	self changed: #selectionIndex.
	self changed: #fieldList.
	self update! !

!DictionaryInspector methodsFor: 'selecting' stamp: 'apb 8/20/2004 21:41'!
addEntry: aKey
	object at: aKey put: nil.
	self calculateKeyArray.
	selectionIndex := self numberOfFixedFields + (keyArray indexOf: aKey).
	self changed: #inspectObject.
	self changed: #selectionIndex.
	self changed: #fieldList.
	self update! !

!DictionaryInspector methodsFor: 'selecting' stamp: 'di 3/8/2000 09:14'!
calculateKeyArray
	"Recalculate the KeyArray from the object being inspected"

	keyArray := object keysSortedSafely asArray.
	selectionIndex := 0.
! !

!DictionaryInspector methodsFor: 'selecting' stamp: 'di 9/22/1998 21:25'!
contentsIsString
	"Hacked so contents empty when deselected"

	^ (selectionIndex = 0)! !

!DictionaryInspector methodsFor: 'menu' stamp: 'apb 8/20/2004 21:19'!
copyName
	"Copy the name of the current variable, so the user can paste it into the 
	window below and work with is. If collection, do (xxx at: 1)."
	| sel |
	self selectionIndex <= self numberOfFixedFields
		ifTrue: [super copyName]
		ifFalse: [sel := String streamContents: [:strm | 
							strm nextPutAll: '(self at: '.
							(keyArray at: selectionIndex - self numberOfFixedFields)
								storeOn: strm.
							strm nextPutAll: ')'].
			Clipboard clipboardText: sel asText 			"no undo allowed"]! !

!DictionaryInspector methodsFor: 'accessing' stamp: 'apb 8/20/2004 23:06'!
fieldList
	^ self baseFieldList
		, (keyArray collect: [:key | key printString])! !

!DictionaryInspector methodsFor: 'initialize-release' stamp: 'PHK 7/21/2004 18:00'!
initialize
	super initialize.
	self calculateKeyArray! !

!DictionaryInspector methodsFor: 'menu commands' stamp: 'eem 11/20/2014 14:08'!
inspectKey
	"Create and schedule an Inspector on the receiver's model's currently selected key."

	selectionIndex >= self numberOfFixedFields ifTrue:
		[(keyArray at: selectionIndex - self numberOfFixedFields) inspect]! !

!DictionaryInspector methodsFor: 'menu' stamp: 'kfr 4/17/2015 16:42'!
mainFieldListMenu: aMenu

	^ aMenu addList: #(	
		('inspect'							inspectSelection)
		('inspect key'						inspectKey)
		('copy name'						copyName)
		('references'						selectionReferences)
		('objects pointing to this value'		objectReferencesToSelection)
		('senders of this key'				sendersOfSelectedKey)
		-
		('refresh view'						refreshView)
		('add key'							addEntry)
		('rename key'						renameEntry)
		-
		('remove'							removeSelection)
		('basic inspect'					inspectBasic));
	yourself
! !

!DictionaryInspector methodsFor: 'private' stamp: 'apb 8/20/2004 21:15'!
numberOfFixedFields
	^ 2 + object class instSize! !

!DictionaryInspector methodsFor: 'selecting' stamp: 'apb 8/20/2004 23:23'!
refreshView
	| i |
	i := selectionIndex.
	self calculateKeyArray.
	selectionIndex := i.
	self changed: #fieldList.
	self changed: #contents.! !

!DictionaryInspector methodsFor: 'menu' stamp: 'apb 8/20/2004 21:39'!
removeSelection
	selectionIndex = 0 ifTrue: [^ self changed: #flash].
	object removeKey: (keyArray at: selectionIndex - self numberOfFixedFields).
	selectionIndex := 0.
	contents := ''.
	self calculateKeyArray.
	self changed: #inspectObject.
	self changed: #selectionIndex.
	self changed: #fieldList.
	self changed: #selection.! !

!DictionaryInspector methodsFor: 'menu' stamp: 'rbb 3/1/2005 10:51'!
renameEntry
	| newKey aKey value |

	value := object at: (keyArray at: selectionIndex - self numberOfFixedFields).
	newKey := UIManager default request: 
'Enter new key, then type RETURN.
(Expression will be evaluated for value.)
Examples:  #Fred    ''a string''   3+4'
		 initialAnswer: (keyArray at: selectionIndex - self numberOfFixedFields) printString.
	aKey := Compiler evaluate: newKey.
	object removeKey: (keyArray at: selectionIndex - self numberOfFixedFields).
	object at: aKey put: value.
	self calculateKeyArray.
	selectionIndex := self numberOfFixedFields + (keyArray indexOf: aKey).
	self changed: #selectionIndex.
	self changed: #inspectObject.
	self changed: #fieldList.
	self update! !

!DictionaryInspector methodsFor: 'selecting' stamp: 'apb 8/20/2004 22:37'!
replaceSelectionValue: anObject 
	selectionIndex <= self numberOfFixedFields
		ifTrue: [^ super replaceSelectionValue: anObject].
	^ object
		at: (keyArray at: selectionIndex - self numberOfFixedFields)
		put: anObject! !

!DictionaryInspector methodsFor: 'selecting' stamp: 'apb 8/20/2004 21:55'!
selection

	selectionIndex <= (self numberOfFixedFields) ifTrue: [^ super selection].
	^ object at: (keyArray at: selectionIndex - self numberOfFixedFields) ifAbsent:[nil]! !

!DictionaryInspector methodsFor: 'menu' stamp: 'ar 10/31/2004 17:26'!
selectionReferences
	"Create a browser on all references to the association of the current selection."

	self selectionIndex = 0 ifTrue: [^ self changed: #flash].
	object class == MethodDictionary ifTrue: [^ self changed: #flash].
	self systemNavigation browseAllCallsOn: (object associationAt: (keyArray at: selectionIndex  - self numberOfFixedFields)).
! !

!DictionaryInspector methodsFor: 'menu' stamp: 'ar 4/10/2005 22:17'!
sendersOfSelectedKey
	"Create a browser on all senders of the selected key"
	| aKey |
	self selectionIndex = 0
		ifTrue: [^ self changed: #flash].
	((aKey := keyArray at: selectionIndex  - self numberOfFixedFields) isSymbol)
		ifFalse: [^ self changed: #flash].
	SystemNavigation default browseAllCallsOn: aKey! !

Inspector subclass: #ExternalStructureInspector
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Inspector'!

!ExternalStructureInspector methodsFor: 'accessing' stamp: 'hg 2/28/2000 14:20'!
fieldList
	^  (Array with: 'self: ', object defaultLabelForInspector with: 'all inst vars'), self recordFieldList! !

!ExternalStructureInspector methodsFor: 'accessing' stamp: 'gk 3/1/2005 12:07'!
recordFieldList
	| fields |
	fields := object class fields.
	(fields first isKindOf: Array) ifFalse: [fields := Array with: fields].
	^fields collect: [ :field | field first ] thenSelect: [:name | name notNil]! !

!ExternalStructureInspector methodsFor: 'selecting' stamp: 'hg 2/28/2000 14:12'!
replaceSelectionValue: anObject 
	"Add colon to fieldname to get setter selector, and send it to object with the argument.
	 Refer to the comment in Inspector|replaceSelectionValue:."

	selectionIndex = 1
		ifTrue: [^object]
		ifFalse: [^object perform: ((self fieldList at: selectionIndex), ':') asSymbol with: anObject]! !

!ExternalStructureInspector methodsFor: 'selecting' stamp: 'hg 2/28/2000 14:22'!
selection 
	"Refer to the comment in Inspector|selection."
	selectionIndex = 0 ifTrue:[^object printString].
	selectionIndex = 1 ifTrue: [^object].
	selectionIndex = 2 ifTrue:[^object longPrintString].
	selectionIndex > 2
		ifTrue: [^object perform: (self fieldList at: selectionIndex)]! !

Inspector subclass: #InspectorBrowser
	instanceVariableNames: 'fieldList msgList msgListIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Inspector'!

!InspectorBrowser commentStamp: 'mt 4/12/2016 17:29' prior: 0!
I am an inspector that also shows all messages the inspected objects can understand. I combine inspetor and code browser.

InspectorBrowser openOn: Smalltalk!

!InspectorBrowser methodsFor: 'tool builder' stamp: 'mt 4/12/2016 17:27'!
buildWith: builder

	| window fieldSpec valueSpec msgSpec codeSpec |
	
	window := builder pluggableWindowSpec new
		model: self;
		children: OrderedCollection new;
		yourself.
		
	fieldSpec := builder pluggableListSpec new
		model: self;
		list: #fieldList;
		getIndex: #selectionIndex;
		setIndex: #toggleIndex:;
		menu: #fieldListMenu:;
		frame: (0@0 corner: 0.3@0.5).
	
	valueSpec := builder pluggableTextSpec new
		model: self;
		getText: #contents;
		setText: #accept:;
		menu: #codePaneMenu:shifted:;
		frame: (0.3@0 corner: 1.0@0.5).

	msgSpec := builder pluggableListSpec new
		model: self;
		list: #msgList;
		getIndex: #msgListIndex;
		setIndex: #msgListIndex:;
		menu: #msgListMenu;
		frame: (0@0.5 corner: 0.3@1.0).
		
	codeSpec := builder pluggableTextSpec new
		model: self;
		getText: #msgText;
		setText: #msgAccept:from:;
		menu: #msgPaneMenu:shifted:;
		frame: (0.3@0.5 corner: 1.0@1.0).
	
	window children addAll: {fieldSpec. valueSpec. msgSpec. codeSpec}.
	^ builder build: window! !

!InspectorBrowser methodsFor: 'as yet unclassified' stamp: 'di 4/16/1998 15:51'!
fieldList
	fieldList ifNotNil: [^ fieldList].
	^ (fieldList := super fieldList)! !

!InspectorBrowser methodsFor: 'initialize-release' stamp: 'apb 7/26/2004 17:34'!
initialize

	super initialize.
	fieldList := nil.
	msgListIndex := 0.
	self changed: #msgText
! !

!InspectorBrowser methodsFor: 'initialize-release' stamp: 'apb 7/26/2004 17:34'!
inspect: anObject 
	"Initialize the receiver so that it is inspecting anObject. There is no current selection.
	Overriden so that my class is not changed to 'anObject inspectorClass'."
	
	object := anObject.
	self initialize
! !

!InspectorBrowser methodsFor: 'as yet unclassified' stamp: 'di 11/13/1998 15:27'!
msgAccept: newText from: editor
	| category |
	category := msgListIndex = 0
		ifTrue: [ClassOrganizer default]
		ifFalse: [object class organization categoryOfElement: (msgList at: msgListIndex)].
	^ (object class compile: newText classified: category notifying: editor) ~~ nil! !

!InspectorBrowser methodsFor: 'messages' stamp: 'apb 7/14/2004 13:50'!
msgList
	msgList ifNotNil: [^ msgList].
	^ (msgList := object class selectors asSortedArray)! !

!InspectorBrowser methodsFor: 'as yet unclassified' stamp: 'di 4/16/1998 14:18'!
msgListIndex 
	^msgListIndex! !

!InspectorBrowser methodsFor: 'as yet unclassified' stamp: 'di 4/16/1998 14:32'!
msgListIndex: anInteger
	"A selection has been made in the message pane"

	msgListIndex := anInteger.
	self changed: #msgText.! !

!InspectorBrowser methodsFor: 'messages' stamp: 'apb 7/14/2004 13:57'!
msgListMenu: aMenu 
	^ aMenu labels: 'Not yet implemented' lines: #(0) selections: #(flash)! !

!InspectorBrowser methodsFor: 'as yet unclassified' stamp: 'mt 11/13/2015 09:40'!
msgPaneMenu: aMenu shifted: shifted
	^ aMenu labels: 
'find... (f)
find again (g)
find and replace...
do/replace again (j)
undo (z)
redo (Z)
copy (c)
cut (x)
paste (v)
do it (d)
print it (p)
inspect it (i)
accept (s)
cancel (l)' 
		lines: #(0 4 6 9 12)
		selections: #(find findAgain findReplace again undo redo copySelection cut paste doIt printIt inspectIt accept cancel)! !

!InspectorBrowser methodsFor: 'as yet unclassified' stamp: 'di 4/16/1998 14:38'!
msgText
	msgListIndex = 0 ifTrue: [^ nil].
	^ object class sourceCodeAt: (msgList at: msgListIndex)! !

!InspectorBrowser methodsFor: 'as yet unclassified' stamp: 'di 4/18/1998 09:48'!
step
	| list fieldString msg |
	(list := super fieldList) = fieldList ifFalse:
		[fieldString := selectionIndex > 0 ifTrue: [fieldList at: selectionIndex] ifFalse: [nil].
		fieldList := list.
		selectionIndex := fieldList indexOf: fieldString ifAbsent: [0].
		self changed: #fieldList.
		self changed: #selectionIndex].
	list := msgList.  msgList := nil.  "force recomputation"
		list = self msgList ifFalse:
		[msg := msgListIndex > 0 ifTrue: [list at: msgListIndex] ifFalse: [nil].
		msgListIndex := msgList indexOf: msg ifAbsent: [0].
		self changed: #msgList.
		self changed: #msgListIndex].
	super step! !

!InspectorBrowser methodsFor: 'as yet unclassified' stamp: 'di 1/14/1999 09:01'!
wantsSteps
	^ true! !

!Inspector class methodsFor: 'instance creation' stamp: 'PHK 7/22/2004 17:04'!
inspect: anObject 
	"Answer an instance of me to provide an inspector for anObject."
	
	"We call basicNew to avoid a premature initialization; the instance method 
	inspect: anObject will do a self initialize."

	^self basicNew inspect: anObject! !

!Inspector class methodsFor: 'instance creation' stamp: 'ar 9/27/2005 18:30'!
openOn: anObject
	"Create and schedule an instance of me on the model, anInspector. "

	^ self openOn: anObject withEvalPane: true! !

!Inspector class methodsFor: 'instance creation' stamp: ''!
openOn: anObject withEvalPane: withEval 
	"Create and schedule an instance of me on the model, anInspector. "

	^ self openOn: anObject withEvalPane: withEval withLabel: anObject defaultLabelForInspector! !

!Inspector class methodsFor: 'instance creation' stamp: 'ar 8/7/2009 23:43'!
openOn: anObject withEvalPane: withEval withLabel: label
        ^ToolBuilder open: (self inspect: anObject) label: label! !

!Inspector methodsFor: 'styling' stamp: 'ul 5/5/2015 00:57'!
aboutToStyle: aStyler

	aStyler 
		classOrMetaClass: object class;
		parseAMethod: false.
	^true! !

!Inspector methodsFor: 'selecting' stamp: 'nice 3/31/2011 22:00'!
accept: aString 
	| result |
	result := self doItReceiver class evaluatorClass new
				evaluate: (ReadStream on: aString)
				in: self doItContext
				to: self doItReceiver
				notifying: nil  "fix this"
				ifFail: [self changed: #flash.
					^ false].
	self replaceSelectionValue: result.
	self changed: #contents.
	^ true! !

!Inspector methodsFor: 'menu commands' stamp: 'fbs 3/8/2013 08:50'!
addCollectionItemsTo: aMenu
	"If the current selection is an appropriate collection, add items to aMenu that cater to that kind of selection"

	| sel |
	((((sel := self selection) isMemberOf: Array) or: [sel isMemberOf: OrderedCollection]) and: 
		[sel size > 0]) ifTrue: [
			aMenu addList: #(
				('inspect element...'					inspectElement))].

	(sel respondsTo: #inspectElement) ifTrue: [
			aMenu addList: #(
				('inspect property...'				inspectElement))].! !

!Inspector methodsFor: 'accessing' stamp: ''!
baseFieldList
	"Answer an Array consisting of 'self'
	and the instance variable names of the inspected object."

	^ (Array with: 'self' with: 'all inst vars')
			, object class allInstVarNames! !

!Inspector methodsFor: 'toolbuilder' stamp: 'mt 5/1/2015 18:40'!
buildCodePaneWith: builder
	| textSpec |
	textSpec := builder pluggableCodePaneSpec new.
	textSpec 
		model: self;
		getText: #expression; 
		editText: #expression:;
		help: #helpText;
		selection: #contentsSelection; 
		menu: #codePaneMenu:shifted:;
		askBeforeDiscardingEdits: false.
	^textSpec! !

!Inspector methodsFor: 'toolbuilder' stamp: 'mt 4/12/2015 16:18'!
buildExploreButtonWith: builder
	
	^ builder pluggableButtonSpec new
		model: self;
		label: 'explore';
		action: #exploreObject;
		help: 'Switch to an explorer tool'! !

!Inspector methodsFor: 'toolbuilder' stamp: 'mt 2/17/2016 20:00'!
buildFieldListWith: builder

	| listSpec |
	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #fieldList; 
		getIndex: #selectionIndex; 
		setIndex: #toggleIndex:; 
		menu: #fieldListMenu:; 
		dragItem: #dragFromFieldList:;
		dropItem: #dropOnFieldList:at:;
		dropAccept: #wantsDropOnFieldList:type:source:;
		keyPress: #inspectorKey:from:.
	^listSpec! !

!Inspector methodsFor: 'toolbuilder' stamp: 'cmm 4/7/2016 15:55'!
buildValuePaneWith: builder
	| textSpec |
	textSpec := builder pluggableTextSpec new.
	textSpec 
		model: self;
		getText: #contents; 
		setText: #accept:; 
		help: 'Selection details.';
		selection: #contentsSelection; 
		menu: #codePaneMenu:shifted:.
	^textSpec! !

!Inspector methodsFor: 'toolbuilder' stamp: 'mt 5/4/2016 11:59'!
buildWith: builder
	"Inspector openOn: SystemOrganization"
	| windowSpec specs buttonOffset |
	buttonOffset := (Preferences standardButtonFont widthOfString: 'explore') * 3/2.
	
	specs := {
		(0@0 corner: 0.3@0.71)  -> [self buildFieldListWith: builder].
		(0.3@0.0corner: 1@0.71) -> [self buildValuePaneWith: builder].
		(LayoutFrame fractions: (0@0.71 corner: 1@1) offsets: (0@0 corner: buttonOffset negated@0)) -> [self buildCodePaneWith: builder].
		(LayoutFrame fractions: (1@0.71 corner: 1@1) offsets: (buttonOffset negated@0 corner: 0 @ 0)) -> [self buildExploreButtonWith: builder]
	}.
	
	windowSpec := self buildWindowWith: builder specs: specs. 
	windowSpec extent: self initialExtent.
	^builder build: windowSpec! !

!Inspector methodsFor: 'menu commands' stamp: 'ul 11/15/2010 11:43'!
chasePointers
	| selected  saved |
	self selectionIndex = 0 ifTrue: [^ self changed: #flash].
	selected := self selection.
	saved := self object.
	[self object: nil.
	(Smalltalk includesKey: #PointerFinder)
		ifTrue: [PointerFinder on: selected]
		ifFalse: [self inspectPointers]]
		ensure: [self object: saved]! !

!Inspector methodsFor: 'menu commands' stamp: 'tk 4/10/1998 17:53'!
classOfSelection
	"Answer the class of the receiver's current selection"

	self selectionUnmodifiable ifTrue: [^ object class].
	^ self selection class! !

!Inspector methodsFor: 'selecting' stamp: 'di 9/22/1998 21:24'!
contentsIsString
	"Hacked so contents empty when deselected and = long printString when item 2"

	^ (selectionIndex = 2) | (selectionIndex = 0)! !

!Inspector methodsFor: 'accessing' stamp: 'ajh 7/7/2004 15:55'!
context: ctxt
	"Set the context of inspection. Currently only used by my subclass ClosureEnvInspector. The inst var is here because we do primitiveChangeClassTo: between subclasses (see inspect:) between different subclasses, but also context could be used as a general concept in all inspectors"

	context := ctxt! !

!Inspector methodsFor: 'object fileIn' stamp: 'RAA 12/20/2000 17:47'!
convertToCurrentVersion: varDict refStream: smartRefStrm
	
	timeOfLastListUpdate ifNil: [timeOfLastListUpdate := 0].
	^super convertToCurrentVersion: varDict refStream: smartRefStrm.

! !

!Inspector methodsFor: 'menu commands' stamp: 'PHK 6/30/2004 11:47'!
copyName
	"Copy the name of the current variable, so the user can paste it into the 
	window below and work with is. If collection, do (xxx at: 1)."
	| sel aClass variableNames |
	self selectionUnmodifiable
		ifTrue: [^ self changed: #flash].
	aClass := self object class.
	variableNames := aClass allInstVarNames.
	(aClass isVariable and: [selectionIndex > (variableNames size + 2)])
		ifTrue: [sel := '(self basicAt: ' , (selectionIndex - (variableNames size + 2)) asString , ')']
		ifFalse: [sel := variableNames at: selectionIndex - 2].
	(self selection isKindOf: Collection)
		ifTrue: [sel := '(' , sel , ' at: 1)'].
	Clipboard clipboardText: sel asText! !

!Inspector methodsFor: 'selecting' stamp: 'eem 7/23/2014 06:17'!
defaultIntegerBase
	"Answer the default base in which to print integers.
	 Defer to the class of the instance."
	^[object class defaultIntegerBaseInDebugger]
		on: MessageNotUnderstood
		do: [:ex| 10]! !

!Inspector methodsFor: 'menu commands' stamp: 'sd 4/15/2003 16:14'!
defsOfSelection
	"Open a browser on all defining references to the selected instance variable, if that's what currently selected. "
	| aClass sel |

	self selectionUnmodifiable ifTrue: [^ self changed: #flash].
	(aClass := self object class) isVariable ifTrue: [^ self changed: #flash].

	sel := aClass allInstVarNames at: self selectionIndex - 2.
	self systemNavigation  browseAllStoresInto: sel from: aClass! !

!Inspector methodsFor: 'code' stamp: ''!
doItReceiver
	"Answer the object that should be informed of the result of evaluating a
	text selection."

	^object! !

!Inspector methodsFor: 'drag-drop' stamp: 'bf 9/6/2011 13:25'!
dragFromFieldList: index
	selectionIndex = index ifFalse: [self toggleIndex: index].
	^self selection! !

!Inspector methodsFor: 'drag-drop' stamp: 'bf 9/6/2011 13:38'!
dropOnFieldList: anObject at: index
	selectionIndex = index ifFalse: [self toggleIndex: index].
	self replaceSelectionValue: anObject.
	self changed: #contents.
	^ true! !

!Inspector methodsFor: 'toolbuilder' stamp: 'mt 5/4/2016 13:31'!
exploreObject
	"Switch to an explorer tool."
	
	| window currentBounds |
	currentBounds := ToolBuilder findDefault getBoundsForWindow: self containingWindow.

	"Close first because MVC fiddles around with processes."
	self changed: #close. 
		
	window := self object explore.
	
	"---- In MVC, the lines after this will not be executed ---"

	window model setExpression: self expression.
	ToolBuilder findDefault setBoundsForWindow: window to: currentBounds.! !

!Inspector methodsFor: 'menu commands' stamp: 'mt 4/2/2015 11:04'!
explorePointers
	self selectionIndex = 0 ifTrue: [^ self changed: #flash].
	PointerExplorer openOn: self selection.! !

!Inspector methodsFor: 'menu commands' stamp: 'sw 9/21/1999 12:16'!
exploreSelection

	self selectionIndex = 0 ifTrue: [^ self changed: #flash].
	^ self selection explore! !

!Inspector methodsFor: 'accessing' stamp: 'mt 4/12/2015 16:51'!
expression

	^ expression ifNil: ['']! !

!Inspector methodsFor: 'accessing' stamp: 'mt 4/12/2015 16:49'!
expression: aString

	expression := aString.! !

!Inspector methodsFor: 'accessing' stamp: ''!
fieldList
	"Answer the base field list plus an abbreviated list of indices."

	object class isVariable ifFalse: [^ self baseFieldList].
	^ self baseFieldList ,
		(object basicSize <= (self i1 + self i2)
			ifTrue: [(1 to: object basicSize)
						collect: [:i | i printString]]
			ifFalse: [(1 to: self i1) , (object basicSize-(self i2-1) to: object basicSize)
						collect: [:i | i printString]])! !

!Inspector methodsFor: 'menu commands' stamp: 'topa 3/16/2015 17:31'!
fieldListMenu: aMenu
	"Arm the supplied menu with items for the field-list of the receiver"
	^ self menu: aMenu for: #(fieldListMenu fieldListMenuShifted:)
! !

!Inspector methodsFor: 'accessing' stamp: 'cmm 4/24/2015 15:47'!
helpText
	^ 'evaluate expressions here'! !

!Inspector methodsFor: 'accessing' stamp: ''!
i1
	"This is the max index shown before skipping to the 
	last i2 elements of very long arrays"
	^ 100! !

!Inspector methodsFor: 'accessing' stamp: ''!
i2
	"This is the number of elements to show at the end
	of very long arrays"
	^ 10! !

!Inspector methodsFor: 'accessing' stamp: 'mt 4/12/2015 16:06'!
initialExtent
	"Answer the desired extent for the receiver when it is first opened on the screen.  "

	^ 350 @ 250! !

!Inspector methodsFor: 'initialize-release' stamp: 'apb 7/14/2004 14:45'!
initialize
	
	selectionIndex := 0.
	super initialize! !

!Inspector methodsFor: 'menu commands' stamp: 'tk 4/12/1998 08:49'!
inspectBasic
	"Bring up a non-special inspector"

	selectionIndex = 0 ifTrue: [^ object basicInspect].
	self selection basicInspect! !

!Inspector methodsFor: 'menu commands' stamp: 'fbs 3/8/2013 08:48'!
inspectElement
	| sel selSize countString count nameStrs |
	"Create and schedule an Inspector on an element of the receiver's model's currently selected collection."

	self selectionIndex = 0 ifTrue: [^ self changed: #flash].
	((sel := self selection) isKindOf: SequenceableCollection) ifFalse:
		[(sel respondsTo: #inspectElement) ifTrue: [^ sel inspectElement].
		^ sel inspect].
	(selSize := sel size) = 1 ifTrue: [^ sel first inspect].
	selSize <= 20 ifTrue:
		[nameStrs := (1 to: selSize) asArray collect: [:ii | 
			ii printString, '   ', (((sel at: ii) printStringLimitedTo: 25) replaceAll: Character cr with: Character space)].
		count := UIManager default chooseFrom: (nameStrs substrings) title: 'which element?'.
		count = 0 ifTrue: [^ self].
		^ (sel at: count) inspect].

	countString := UIManager default request: 'Which element? (1 to ', selSize printString, ')' initialAnswer: '1'.
	countString isEmptyOrNil ifTrue: [^ self].
	count := Integer readFrom: (ReadStream on: countString).
	(count > 0 and: [count <= selSize])
		ifTrue: [(sel at: count) inspect]
		ifFalse: [Beeper beep]! !

!Inspector methodsFor: 'menu commands' stamp: 'rhi 5/27/2004 17:09'!
inspectorKey: aChar from: view
	"Respond to a Command key issued while the cursor is over my field list"

	aChar == $i ifTrue: [^ self selection inspect].
	aChar == $I ifTrue: [^ self selection explore].
	aChar == $b ifTrue:	[^ self browseMethodFull].
	aChar == $h ifTrue:	[^ self classHierarchy].
	aChar == $c ifTrue: [^ self copyName].
	aChar == $p ifTrue: [^ self browseFullProtocol].
	aChar == $N ifTrue: [^ self browseClassRefs].
	aChar == $t ifTrue: [^ self tearOffTile].
	aChar == $v ifTrue: [^ self viewerForValue].

	^ self arrowKey: aChar from: view! !

!Inspector methodsFor: 'menu commands' stamp: 'apb 7/14/2004 13:16'!
inspectSelection
	"Create and schedule an Inspector on the receiver's model's currently selected object."

	self selectionIndex = 0 ifTrue: [^ self changed: #flash].
	self selection inspect.
	^ self selection! !

!Inspector methodsFor: 'initialize-release' stamp: 'bf 3/22/2012 16:13'!
inspect: anObject 
	"Initialize the receiver so that it is inspecting anObject. There is no current selection.
	
	Normally the receiver will be of the correct class (as defined by anObject inspectorClass),
	because it will have just been created by sedning inspect to anObject.   However, the
	debugger uses two embedded inspectors, which are re-targetted on the current receiver
	each time the stack frame changes.  The left-hand inspector in the debugger has its
	class changed by the code here.  Care should be taken if this method is overridden to
	ensure that the overriding code calls 'super inspect: anObject', or otherwise ensures that 
	the class of these embedded inspectors are changed back."

	| c |
	c := anObject inspectorClass.
	self class ~= c ifTrue: [
		self class format = c format
			ifTrue: [self primitiveChangeClassTo: c basicNew]
			ifFalse: [self becomeForward: (c basicNew copyFrom: self)]].
	"Set 'object' before sending the initialize message, because some implementations
	of initialize (e.g., in DictionaryInspector) require 'object' to be non-nil."
	
	object := anObject. 
	self initialize! !

!Inspector methodsFor: 'menu commands' stamp: 'topa 3/16/2015 17:29'!
mainFieldListMenu: aMenu
	"Arm the supplied menu with items for the field-list of the receiver"
	<fieldListMenu>
	"gets overriden by subclasses, _whithout_ the <fieldListMenu>"
	aMenu addStayUpItemSpecial.

	aMenu addList: #(
		('inspect (i)'						inspectSelection)
		('explore (I)'						exploreSelection)).

	self addCollectionItemsTo: aMenu.

	aMenu addList: #(
		-
		('method refs to this inst var'		referencesToSelection)
		('methods storing into this inst var'	defsOfSelection)
		('objects pointing to this value'		objectReferencesToSelection)
		('chase pointers'					chasePointers)
		('explore pointers'				explorePointers)
		-
		('browse full (b)'					browseMethodFull)
		('browse class'						browseClass)
		('browse hierarchy (h)'					classHierarchy)
		('browse protocol (p)'				browseFullProtocol)
		-
		('references... (r)'					browseVariableReferences)
		('assignments... (a)'					browseVariableAssignments)
		('class refs (N)'						browseClassRefs)
		-
		('copy name (c)'					copyName)		
		('basic inspect'						inspectBasic)).

	Smalltalk isMorphic ifTrue:
		[aMenu addList: #(
			-
			('tile for this value	(t)'			tearOffTile)
			('viewer for this value (v)'		viewerForValue))].

	^ aMenu


"			-
			('alias for this value'			aliasForValue)
			('watcher for this slot'			watcherForSlot)"

! !

!Inspector methodsFor: 'accessing' stamp: 'svp 3/14/2000 21:57'!
modelWakeUpIn: aWindow
	| newText |
	self updateListsAndCodeIn: aWindow.
	newText := self contentsIsString
		ifTrue: [newText := self selection]
		ifFalse: ["keep it short to reduce time to compute it"
			self selectionPrintString ].
	newText = contents ifFalse:
		[contents := newText.
		self changed: #contents]! !

!Inspector methodsFor: 'accessing' stamp: 'sw 10/30/1999 23:59'!
noteSelectionIndex: anInteger for: aSymbol
	aSymbol == #fieldList
		ifTrue:
			[selectionIndex := anInteger]! !

!Inspector methodsFor: 'accessing' stamp: ''!
object
	"Answer the object being inspected by the receiver."

	^object! !

!Inspector methodsFor: 'menu commands' stamp: 'ul 11/15/2010 11:44'!
objectReferencesToSelection
	"Open a list inspector on all the objects that point to the value of the selected instance variable, if any.  "

	self selectionIndex = 0 ifTrue: [^ self changed: #flash].
	self systemNavigation
		browseAllObjectReferencesTo: self selection
		except: (Array with: self object)
		ifNone: [:obj | self changed: #flash].
! !

!Inspector methodsFor: 'accessing' stamp: 'mt 4/12/2015 15:57'!
object: anObject 
	"Set anObject to be the object being inspected by the receiver."

	| oldIndex |
	anObject == object
		ifTrue: [self update]
		ifFalse:
			[oldIndex := selectionIndex <= 2 ifTrue: [selectionIndex] ifFalse: [0].
			self inspect: anObject.
			oldIndex := oldIndex min: self fieldList size.
			self changed: #inspectObject.
			oldIndex > 0
				ifTrue: [self toggleIndex: oldIndex].
			self changed: #fieldList.
			self changed: #contents.
			self changed: #helpText]! !

!Inspector methodsFor: 'private' stamp: 'cmm 12/14/2013 16:22'!
printStringErrorText
	| nm |
	nm := self selectionIndex < 3
		ifTrue: ['self']
		ifFalse: [self selectedSlotName].
	^ (nm
		ifNil: ['no selection']
		ifNotNil:
			[nm first isDigit
				ifTrue: ['<error in printString: evaluate "(self at: ' , nm , ') printString" to debug>']
				ifFalse: ['<error in printString: evaluate "' , nm , ' printString" to debug>'] ]) asText! !

!Inspector methodsFor: 'menu commands' stamp: 'sd 4/15/2003 16:14'!
referencesToSelection
	"Open a browser on all references to the selected instance variable, if that's what currently selected.  1/25/96 sw"
	| aClass sel |

	self selectionUnmodifiable ifTrue: [^ self changed: #flash].
	(aClass := self object class) isVariable ifTrue: [^ self changed: #flash].

	sel := aClass allInstVarNames at: self selectionIndex - 2.
	self systemNavigation   browseAllAccessesTo: sel from: aClass! !

!Inspector methodsFor: 'selecting' stamp: 'nice 11/8/2009 15:17'!
replaceSelectionValue: anObject 
	"The receiver has a list of variables of its inspected object. One of these 
	is selected. The value of the selected variable is set to the value, 
	anObject."
	| basicIndex si instVarIndex |
	selectionIndex <= 2 ifTrue: [
		self toggleIndex: (si := selectionIndex).  
		self toggleIndex: si.
		^ object].
	instVarIndex := selectionIndex - 2.
	instVarIndex > object class instSize
		ifFalse: [^ object instVarAt: instVarIndex put: anObject].
	object class isVariable or: [self error: 'Cannot replace selection'].
	basicIndex := selectionIndex - 2 - object class instSize.
	(object basicSize <= (self i1 + self i2)  or: [basicIndex <= self i1])
		ifTrue: [^object basicAt: basicIndex put: anObject]
		ifFalse: [^object basicAt: object basicSize - (self i1 + self i2) + basicIndex
					put: anObject]! !

!Inspector methodsFor: 'morphic ui' stamp: 'cmm 2/13/2010 19:51'!
representsSameBrowseeAs: anotherInspector
	^ self object == anotherInspector object! !

!Inspector methodsFor: 'accessing' stamp: 'tk 4/18/1998 15:37'!
selectedClass
	"Answer the class of the receiver's current selection"

	self selectionUnmodifiable ifTrue: [^ object class].
	^ self selection class! !

!Inspector methodsFor: 'selecting' stamp: 'eem 5/21/2008 11:46'!
selectedSlotName

	^ self fieldList at: self selectionIndex ifAbsent: []! !

!Inspector methodsFor: 'selecting' stamp: 'BG 11/7/2004 13:40'!
selection
	"The receiver has a list of variables of its inspected object.
	One of these is selected. Answer the value of the selected variable."
	| basicIndex |
	selectionIndex = 0 ifTrue: [^ ''].
	selectionIndex = 1 ifTrue: [^ object].
	selectionIndex = 2 ifTrue: [^ object longPrintStringLimitedTo: 20000].
	(selectionIndex - 2) <= object class instSize
		ifTrue: [^ object instVarAt: selectionIndex - 2].
	basicIndex := selectionIndex - 2 - object class instSize.
	(object basicSize <= (self i1 + self i2)  or: [basicIndex <= self i1])
		ifTrue: [^ object basicAt: basicIndex]
		ifFalse: [^ object basicAt: object basicSize - (self i1 + self i2) + basicIndex]! !

!Inspector methodsFor: 'selecting' stamp: ''!
selectionIndex
	"The receiver has a list of variables of its inspected object. One of these 
	is selected. Answer the index into the list of the selected variable."

	^selectionIndex! !

!Inspector methodsFor: 'selecting' stamp: 'eem 2/16/2009 18:26'!
selectionPrintString
	| text |
	selectionUpdateTime := [text := [self selection isInteger
										ifTrue: [self selection storeStringBase: self defaultIntegerBase]
										ifFalse: [self selection printStringLimitedTo: 5000]]
						on: Error
						do: [text := self printStringErrorText.
							text
								addAttribute: TextColor red
								from: 1
								to: text size.
							text]] timeToRun.
	^ text! !

!Inspector methodsFor: 'selecting' stamp: 'PHK 6/30/2004 11:50'!
selectionUnmodifiable
	"Answer if the current selected variable is modifiable via acceptance in the code pane.  For most inspectors, no selection and a selection of 'self' (selectionIndex = 1) and 'all inst vars' (selectionIndex = 2) are unmodifiable"

	^ selectionIndex <= 2! !

!Inspector methodsFor: 'code' stamp: 'mt 4/12/2015 16:50'!
setExpression: aString

	self expression: aString.
	self changed: #expression.! !

!Inspector methodsFor: 'stepping' stamp: 'apb 7/14/2004 14:28'!
stepAt: millisecondClockValue in: aWindow
	| newText |

	(Preferences smartUpdating and: [(millisecondClockValue - self timeOfLastListUpdate) > 8000]) "Not more often than once every 8 seconds"
		ifTrue:
			[self updateListsAndCodeIn: aWindow.
			timeOfLastListUpdate := millisecondClockValue].

	newText := self contentsIsString
		ifTrue: [self selection]
		ifFalse: ["keep it short to reduce time to compute it"
			self selectionPrintString ].
	newText = contents ifFalse:
		[contents := newText.
		self changed: #contents]! !

!Inspector methodsFor: 'accessing' stamp: 'sma 6/15/2000 16:48'!
stepTimeIn: aSystemWindow
	^ (selectionUpdateTime ifNil: [0]) * 10 max: 1000! !

!Inspector methodsFor: 'accessing' stamp: 'sw 10/20/1999 15:54'!
timeOfLastListUpdate
	^ timeOfLastListUpdate ifNil: [timeOfLastListUpdate := 0]! !

!Inspector methodsFor: 'selecting' stamp: 'sma 6/15/2000 16:50'!
toggleIndex: anInteger
	"The receiver has a list of variables of its inspected object. One of these 
	is selected. If anInteger is the index of this variable, then deselect it. 
	Otherwise, make the variable whose index is anInteger be the selected 
	item."

	selectionUpdateTime := 0.
	selectionIndex = anInteger
		ifTrue: 
			["same index, turn off selection"
			selectionIndex := 0.
			contents := '']
		ifFalse:
			["different index, new selection"
			selectionIndex := anInteger.
			self contentsIsString
				ifTrue: [contents := self selection]
				ifFalse: [contents := self selectionPrintString]].
	self changed: #selection.
	self changed: #contents.
	self changed: #selectionIndex.! !

!Inspector methodsFor: 'accessing' stamp: 'tk 4/10/1998 11:28'!
trash
	"What goes in the bottom pane"
	^ ''! !

!Inspector methodsFor: 'accessing' stamp: 'tk 6/11/1998 22:23'!
trash: newText
	"Don't save it"
	^ true! !

!Inspector methodsFor: 'accessing' stamp: 'hmm 7/12/2001 20:35'!
update
	"Reshow contents, assuming selected value may have changed."

	selectionIndex = 0
		ifFalse:
			[self contentsIsString
				ifTrue: [contents := self selection]
				ifFalse: [contents := self selectionPrintString].
			self changed: #contents.
			self changed: #selection.
			self changed: #selectionIndex]! !

!Inspector methodsFor: 'menu commands' stamp: 'ul 11/15/2010 11:44'!
viewerForValue
	"Open up a viewer on the value of the receiver's current selection"

	| objectToRepresent |
	objectToRepresent := self selectionIndex = 0 ifTrue: [object] ifFalse: [self selection].
	objectToRepresent beViewed
	! !

!Inspector methodsFor: 'drag-drop' stamp: 'mt 2/17/2016 20:03'!
wantsDropOnFieldList: anObject type: type source: source

	^ (source respondsTo: #model) and: [source model isKindOf: Inspector]! !

!Inspector methodsFor: 'accessing' stamp: 'di 1/13/1999 14:36'!
wantsSteps
	^ true! !

Inspector subclass: #OrderedCollectionInspector
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Inspector'!

!OrderedCollectionInspector methodsFor: 'as yet unclassified' stamp: 'btc 7/11/2011 23:11'!
fieldList
	object ifNil: [ ^ OrderedCollection new].
	^ self baseFieldList ,
		(self objectSize <= (self i1 + self i2)
			ifTrue: [(1 to: self objectSize)
						collect: [:i | i printString]]
			ifFalse: [(1 to: self i1) , (self objectSize - (self i2-1) to: self objectSize)
						collect: [:i | i printString]])
"
OrderedCollection new inspect
(OrderedCollection newFrom: #(3 5 7 123)) inspect
(OrderedCollection newFrom: (1 to: 1000)) inspect
"! !

!OrderedCollectionInspector methodsFor: 'as yet unclassified' stamp: 'btc 7/13/2011 00:07'!
objectSize
	"Single stepping through a debugger might observe the state of an OrderedCollection 
	instance after creation by basicNew but before initiialisation.  Thus 'object size' 
	throws a DNU error for arithmetic on a nil value that needs to be handled here." 

	^ [ object size ] on: Error do: [ 0 ]
! !

!OrderedCollectionInspector methodsFor: 'as yet unclassified' stamp: 'sw 9/16/97 22:38'!
replaceSelectionValue: anObject 
	"The receiver has a list of variables of its inspected object. One of these 
	is selected. The value of the selected variable is set to the value, anObject."

	(selectionIndex - 2) <= object class instSize
		ifTrue: [^ super replaceSelectionValue: anObject].
	object at: self selectedObjectIndex put: anObject! !

!OrderedCollectionInspector methodsFor: 'as yet unclassified' stamp: 'sw 9/16/97 22:28'!
selectedObjectIndex
	"Answer the index of the inspectee's collection that the current selection refers to."

	| basicIndex |
	basicIndex := selectionIndex - 2 - object class instSize.
	^ (object size <= (self i1 + self i2)  or: [basicIndex <= self i1])
		ifTrue: [basicIndex]
		ifFalse: [object size - (self i1 + self i2) + basicIndex]! !

!OrderedCollectionInspector methodsFor: 'as yet unclassified' stamp: 'sw 9/16/97 22:39'!
selection
	"The receiver has a list of variables of its inspected object.
	One of these is selected. Answer the value of the selected variable."

	(selectionIndex - 2) <= object class instSize
		ifTrue: [^ super selection].
	^ object at: self selectedObjectIndex! !

Inspector subclass: #SetInspector
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Inspector'!

!SetInspector commentStamp: '<historical>' prior: 0!
A verison of the Inspector specialized for inspecting Sets.  It displays the elements of the set like elements of an array.  Note that the indices, being phyical locations in the hash table, are not meaningful outside of the set.!

!SetInspector methodsFor: 'selecting' stamp: 'PHK 6/29/2004 15:33'!
arrayIndexForSelection
	^ (self fieldList at: selectionIndex) asInteger! !

!SetInspector methodsFor: 'menu commands' stamp: 'PHK 6/30/2004 12:25'!
copyName
	"Copy the name of the current variable, so the user can paste it into the 
	window below and work with is. If collection, do (xxx at: 1)."
	| sel |
	self selectionIndex <= (2 + object class instSize)
		ifTrue: [super copyName]
		ifFalse: [sel := '(self array at: '
						, (String streamContents: 
							[:strm | self arrayIndexForSelection storeOn: strm]) , ')'.
			Clipboard clipboardText: sel asText]! !

!SetInspector methodsFor: 'accessing' stamp: 'PHK 6/29/2004 14:50'!
fieldList
	object
		ifNil: [^ Set new].
	^ self baseFieldList
		, (object array
				withIndexCollect: [:each :i | each ifNotNil: [i printString]])
		  select: [:each | each notNil]! !

!SetInspector methodsFor: 'menu' stamp: 'topa 3/16/2015 19:01'!
mainFieldListMenu: aMenu

	^ aMenu labels: #(
			('inspect'							inspectSelection)
			('copy name'						copyName)
			('objects pointing to this value'		objectReferencesToSelection)
			('refresh view'						update)
			('remove'							removeSelection)
			-
			('basic inspect'						inspectBasic));
		yourself
! !

!SetInspector methodsFor: 'menu' stamp: 'PHK 6/30/2004 12:29'!
removeSelection
	(selectionIndex <= object class instSize) ifTrue: [^ self changed: #flash].
	object remove: self selection.
	selectionIndex := 0.
	contents := ''.
	self changed: #inspectObject.
	self changed: #fieldList.
	self changed: #selection.
	self changed: #selectionIndex.! !

!SetInspector methodsFor: 'selecting' stamp: 'PHK 6/29/2004 15:38'!
replaceSelectionValue: anObject
	^ object array at: self arrayIndexForSelection put: anObject! !

!SetInspector methodsFor: 'selecting' stamp: 'cmm 12/14/2013 16:20'!
selection
	selectionIndex = 0 ifTrue: [^ ''].
	selectionIndex = 1 ifTrue: [^ object].
	selectionIndex = 2 ifTrue: [^ object longPrintString].
	(selectionIndex - 2) <= object class instSize
		ifTrue: [^ object instVarAt: selectionIndex - 2].

	^ object array at: self arrayIndexForSelection ifAbsent: [ String empty ]! !

SetInspector subclass: #WeakSetInspector
	instanceVariableNames: 'flagObject'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Inspector'!

!WeakSetInspector commentStamp: '<historical>' prior: 0!
A verison of the SetInspector specialized for inspecting WeakSets.  It knows about the flag object used to indicate empty locations in the hash table.!

!WeakSetInspector methodsFor: 'accessing' stamp: 'nice 12/15/2007 11:59'!
fieldList
	| slotIndices |
	object ifNil: [^ Set new].
	
	"Implementation note: do not use objectArray withIndexCollect: as super
	because this might collect indices in a WeakArray, leading to constantly changing fieldList
	as explained at http://bugs.squeak.org/view.php?id=6812"
	
	slotIndices := (Array new: object size) writeStream.
	object array withIndexDo: [:each :i |
		(each notNil and: [each ~= flagObject]) ifTrue: [slotIndices nextPut: i printString]].
	
	^ self baseFieldList
		, slotIndices contents! !

!WeakSetInspector methodsFor: 'initialize-release' stamp: 'apb 8/21/2004 02:44'!
initialize
	super initialize.
	flagObject := object instVarNamed: 'flag'. ! !

StringHolder subclass: #SelectorBrowser
	instanceVariableNames: 'selectorIndex selectorList classListIndex classList'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Browser'!

!SelectorBrowser class methodsFor: 'instance creation' stamp: 'ar 8/7/2009 22:20'!
prototypicalToolWindow
	"Answer an example of myself seen in a tool window, for the benefit of parts-launching tools"

	| aWindow |
	aWindow := ToolBuilder build: self new.
	aWindow setLabel: 'Selector Browser'.
	aWindow applyModelExtent.
	^ aWindow! !

!SelectorBrowser methodsFor: 'toolbuilder' stamp: 'ar 8/14/2009 21:00'!
buildClassListWith: builder
	| listSpec |
	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #classList; 
		getIndex: #classListIndex; 
		setIndex: #classListIndex:; 
		keyPress: #arrowKey:from:.
	^listSpec
! !

!SelectorBrowser methodsFor: 'toolbuilder' stamp: 'ar 8/14/2009 21:04'!
buildEditViewWith: builder
	| textSpec |
	textSpec := builder pluggableInputFieldSpec new.
	textSpec 
		model: self;
		getText: #contents; 
		setText: #contents:notifying:; 
		selection: #contentsSelection; 
		menu: #codePaneMenu:shifted:.
	^textSpec! !

!SelectorBrowser methodsFor: 'toolbuilder' stamp: 'ar 8/14/2009 21:01'!
buildExamplePaneWith: builder
	| textSpec |
	textSpec := builder pluggableTextSpec new.
	textSpec 
		model: self;
		getText: #byExample; 
		setText: #byExample:; 
		selection: #contentsSelection; 
		menu: #codePaneMenu:shifted:.
	^textSpec! !

!SelectorBrowser methodsFor: 'toolbuilder' stamp: 'ar 8/14/2009 20:59'!
buildMessageListWith: builder
	| listSpec |
	listSpec := builder pluggableListSpec new.
	listSpec 
		model: self;
		list: #messageList; 
		getIndex: #messageListIndex; 
		setIndex: #messageListIndex:; 
		menu: #selectorMenu:; 
		keyPress: #messageListKey:from:.
	^listSpec
! !

!SelectorBrowser methodsFor: 'toolbuilder' stamp: 'ar 8/14/2009 21:04'!
buildWith: builder
	"Create a Browser that lets you type part of a selector, shows a list of selectors, shows the classes of the one you chose, and spawns a full browser on it.  Answer the window
	SelectorBrowser new open "
	| windowSpec |
	selectorIndex := classListIndex := 0.
	windowSpec := 	self buildWindowWith: builder specs: {
		(0@0 corner: 0.5@0.14) -> [self buildEditViewWith: builder].
		(0@0.14 corner: 0.5@0.6) -> [self buildMessageListWith: builder].
		(0.5@0 corner: 1@0.6) -> [self buildClassListWith: builder].
		(0@0.6 corner: 1@1) -> [self buildExamplePaneWith: builder].
	}.
	^builder build: windowSpec! !

!SelectorBrowser methodsFor: 'example pane' stamp: 'tk 12/1/2000 10:38'!
byExample
	"The comment in the bottom pane"

	false ifTrue: [MethodFinder methodFor: #( (4 3) 7  (0 5) 5  (5 5) 10)].
		"to keep the method methodFor: from being removed from the system"

	^ 'Type a fragment of a selector in the top pane.  Accept it.

Or, use an example to find a method in the system.  Type receiver, args, and answer in the top pane with periods between the items.  3. 4. 7

Or, in this pane, use examples to find a method in the system.  Select the line of code and choose "print it".  

	MethodFinder methodFor: #( (4 3) 7  (0 5) 5  (5 5) 10).
This will discover (data1 + data2).

You supply inputs and answers and the system will find the method.  Each inner array is a list of inputs.  It contains the receiver and zero or more arguments.  For Booleans and any computed arguments, use brace notation.

	MethodFinder methodFor: { {1. 3}. true.  {20. 10}. false}.
This will discover the expressions (data1 < data2), (data2 > data1), and many others.

	MethodFinder methodFor: { {''29 Apr 1999'' asDate}. ''Thursday''.  
		{''30 Apr 1999'' asDate}. ''Friday'' }.
This will discover the expression (data1 weekday)

Receiver and arguments do not have to be in the right order.
See MethodFinder.verify for more examples.'! !

!SelectorBrowser methodsFor: 'example pane' stamp: 'tk 3/29/1999 22:12'!
byExample: newText
	"Don't save it"
	^ true! !

!SelectorBrowser methodsFor: 'class list' stamp: 'tk 8/26/1998 14:20'!
classList
	^ classList! !

!SelectorBrowser methodsFor: 'class list' stamp: 'tk 8/26/1998 14:23'!
classListIndex
	^ classListIndex! !

!SelectorBrowser methodsFor: 'class list' stamp: 'mt 1/22/2016 13:10'!
classListIndex: anInteger

	classListIndex := anInteger.
	classListIndex > 0 ifTrue:
		[self changed: #startNewBrowser. "MVC view will terminate control to prepare for new browser"
		ToolSet browse: self selectedClass selector: self selectedMessageName.
		"classListIndex := 0"]
! !

!SelectorBrowser methodsFor: 'class list' stamp: 'tk 8/26/1998 14:33'!
classListSelectorTitle
	^ 'Class List Menu'! !

!SelectorBrowser methodsFor: 'example pane' stamp: 'nice 8/19/2010 14:08'!
contents: aString notifying: aController
	"Take what the user typed and find all selectors containing it"

	| tokens |
	contents := aString.
	classList := #().  classListIndex := 0.
	selectorIndex := 0.
	tokens := contents asString findTokens: ' .'.
	selectorList := Cursor wait showWhile: [
		tokens size = 1 
			ifTrue: [(Symbol selectorsContaining: contents asString) asArray
				sort: [:x :y | x asLowercase <= y asLowercase]]
			ifFalse: [self quickList]].	"find selectors from a single example of data"
	self changed: #messageList.
	self changed: #classList.
	^ true! !

!SelectorBrowser methodsFor: 'user interface' stamp: 'mt 7/9/2016 08:52'!
defaultWindowColor
	^ (Color r: 0.45 g: 0.9 b: 0.9)! !

!SelectorBrowser methodsFor: 'selector functions' stamp: 'sd 4/16/2003 09:45'!
implementors
	| aSelector |
	(aSelector := self selectedMessageName) ifNotNil:
		[self systemNavigation browseAllImplementorsOf: aSelector]! !

!SelectorBrowser methodsFor: 'message list' stamp: 'di 11/9/1999 10:07'!
initialExtent

	^ 350@250
! !

!SelectorBrowser methodsFor: 'selector finding' stamp: 'mt 8/12/2016 10:20'!
listFromResult: resultOC
	"ResultOC is of the form #('(data1 op data2)' '(...)'). Answer a sorted array."

	(resultOC first beginsWith: 'no single method') ifTrue: [^ #()].
	^ resultOC sorted: [:a :b | 
		(a copyFrom: 6 to: a size) < (b copyFrom: 6 to: b size)].

! !

!SelectorBrowser methodsFor: 'selector list' stamp: 'topa 3/18/2015 10:44'!
mainSelectorMenu: aMenu

	<selectorMenu>
	^ aMenu addList: #(
			('senders (n)'					senders)
			('implementors (m)'				implementors)
			('copy selector to clipboard'	copyName));
		yourself
! !

!SelectorBrowser methodsFor: 'message list' stamp: 'bf 8/20/2009 21:58'!
markMatchingClasses
	"If an example is used, mark classes matching the example instance with an asterisk."

	| unmarkedClassList firstPartOfSelector receiverString receiver |

	self flag: #mref.	"allows for old-fashioned style"

	"Only 'example' queries can be marked."
	(contents asString includes: $.) ifFalse: [^ self].

	unmarkedClassList := classList copy.

	"Get the receiver object of the selected statement in the message list."
	firstPartOfSelector := (Scanner new scanTokens: (selectorList at: selectorIndex)) second.
	receiverString := (ReadStream on: (selectorList at: selectorIndex))
						upToAll: firstPartOfSelector.
	receiver := Compiler evaluate: receiverString.

	unmarkedClassList do: [ :classAndMethod | | class |
		(classAndMethod isKindOf: MethodReference) ifTrue: [
			(receiver isKindOf: classAndMethod actualClass) ifTrue: [
				classAndMethod stringVersion: '*', classAndMethod stringVersionDefault.
			]
		] ifFalse: [
			class := Compiler evaluate:
					((ReadStream on: classAndMethod) upToAll: firstPartOfSelector).
			(receiver isKindOf: class) ifTrue: [
				classList add: '*', classAndMethod.
				classList remove: classAndMethod
			]
		].
	].
! !

!SelectorBrowser methodsFor: 'message list' stamp: 'tk 8/26/1998 14:19'!
messageList
	"Find all the selectors containing what the user typed in."

	^ selectorList! !

!SelectorBrowser methodsFor: 'message list' stamp: 'tk 8/26/1998 10:58'!
messageListIndex
	"Answer the index of the selected message selector."

	^ selectorIndex! !

!SelectorBrowser methodsFor: 'message list' stamp: 'sd 4/19/2003 12:13'!
messageListIndex: anInteger 
	"Set the selected message selector to be the one indexed by anInteger. 
	Find all classes it is in."
	selectorIndex := anInteger.
	selectorIndex = 0
		ifTrue: [^ self].
	classList := self systemNavigation allImplementorsOf: self selectedMessageName.
	self markMatchingClasses.
	classListIndex := 0.
	self changed: #messageListIndex.
	"update my selection"
	self changed: #classList! !

!SelectorBrowser methodsFor: 'message list' stamp: 'sma 2/6/2000 11:42'!
messageListKey: aChar from: view
	"Respond to a command key. Handle (m) and (n) here,
	else defer to the StringHolder behaviour."

	aChar == $m ifTrue: [^ self implementors].
	aChar == $n ifTrue: [^ self senders].
	super messageListKey: aChar from: view
! !

!SelectorBrowser methodsFor: 'toolbuilder' stamp: 'ar 8/7/2009 22:24'!
open
	"Create a Browser that lets you type part of a selector, shows a list of selectors,
	shows the classes of the one you chose, and spwns a full browser on it.
		SelectorBrowser new open
	"
	^ToolBuilder open: self! !

!SelectorBrowser methodsFor: 'selector finding' stamp: 'ul 12/6/2011 15:41'!
quickList
	"Compute the selectors for the single example of receiver and args, in the very top pane" 

	| data result resultArray dataStrings mf dataObjects aa statements |
	data := contents asString withBlanksTrimmed.
	mf := MethodFinder new.
	data := mf cleanInputs: data.	"remove common mistakes"
	dataObjects := Compiler evaluate: '{', data, '}'. "#( data1 data2 result )"
	statements := (Compiler new parse: 'zort ' , data in: Object notifying: nil)
				body statements select: [:each | (each isKindOf: ReturnNode) not].
 	dataStrings := statements collect:
				[:node | String streamContents:
					[:strm | (node isMessage) ifTrue: [strm nextPut: $(].
					node shortPrintOn: strm.
					(node isMessage) ifTrue: [strm nextPut: $)].]].
	dataObjects size < 2 ifTrue: [self inform: 'If you are giving an example of receiver, \args, and result, please put periods between the parts.\Otherwise just type one selector fragment' withCRs. ^#()].
 	dataObjects := Array with: dataObjects allButLast with: dataObjects last. "#( (data1
  data2) result )" 
	result := mf load: dataObjects; findMessage.
	(result first beginsWith: 'no single method') ifFalse: [
		aa := self testObjects: dataObjects strings: dataStrings.
		dataObjects := aa second.  dataStrings := aa third].
	resultArray := self listFromResult: result. 
	resultArray isEmpty ifTrue: [self inform: result first].

	dataStrings size = (dataObjects first size + 1) ifTrue:
		[resultArray := resultArray collect: [:expression | | newExp |
		newExp := expression.
		dataObjects first withIndexDo: [:lit :i |
			newExp := newExp copyReplaceAll: 'data', i printString
							with: (dataStrings at: i)].
		newExp, ' --> ', dataStrings last]].

 	^ resultArray! !

!SelectorBrowser methodsFor: 'example pane' stamp: 'dtl 12/28/2009 18:13'!
searchResult: anExternalSearchResult

	self contents: ''.
	classList := #(). classListIndex := 0.
	selectorIndex := 0.
	selectorList := self listFromResult: anExternalSearchResult.
 	self changed: #messageList.
	self changed: #classList.
	self changed: #contents
! !

!SelectorBrowser methodsFor: 'class list' stamp: 'RAA 5/29/2001 14:39'!
selectedClass
	"Answer the currently selected class."

	| pairString |

	self flag: #mref.	"allows for old-fashioned style"

	classListIndex = 0 ifTrue: [^nil].
	pairString := classList at: classListIndex.
	(pairString isKindOf: MethodReference) ifTrue: [
		^pairString actualClass
	].
	(pairString includes: $*) ifTrue: [pairString := pairString allButFirst].
	MessageSet 
		parse: pairString
		toClassAndSelector: [:cls :sel | ^ cls].! !

!SelectorBrowser methodsFor: 'accessing' stamp: 'tk 8/27/1998 17:48'!
selectedClassName
	"Answer the name of the currently selected class."

	classListIndex = 0 ifTrue: [^nil].
	^ self selectedClass name! !

!SelectorBrowser methodsFor: 'accessing' stamp: 'md 11/14/2003 17:21'!
selectedMessageName
	"Answer the name of the currently selected message."

	| example tokens |
	selectorIndex = 0 ifTrue: [^nil].
	example := selectorList at: selectorIndex.
	tokens := Scanner new scanTokens: example.
	tokens size = 1 ifTrue: [^ tokens first].
	tokens first == #'^' ifTrue: [^ nil].
	(tokens second includes: $:) ifTrue: [^ example findSelector].
	Symbol hasInterned: tokens second ifTrue: [:aSymbol | ^ aSymbol].
	^ nil! !

!SelectorBrowser methodsFor: 'selector list' stamp: 'dtl 12/28/2009 18:13'!
selectorList: anExternalList

	self contents: ''.
	classList := #(). classListIndex := 0.
	selectorIndex := 0.
	selectorList := anExternalList.
	self changed: #messageList.
	self changed: #classList.
	self changed: #contents

! !

!SelectorBrowser methodsFor: 'selector list' stamp: 'sw 9/2/1998 16:37'!
selectorMenuTitle
	^ self selectedMessageName ifNil: ['<no selection>']! !

!SelectorBrowser methodsFor: 'selector list' stamp: 'topa 3/18/2015 10:42'!
selectorMenu: aMenu
	^ self menu: aMenu for: #(selectorMenu selectorMenuShifted:)
! !

!SelectorBrowser methodsFor: 'selector functions' stamp: 'nk 6/26/2003 21:44'!
senders
	| aSelector |
	(aSelector := self selectedMessageName) ifNotNil:
		[self systemNavigation browseAllCallsOn: aSelector]! !

!SelectorBrowser methodsFor: 'selector finding' stamp: 'nice 12/27/2009 03:10'!
testObjects: dataObjects strings: dataStrings
	| dataObjs dataStrs selectors classes didUnmodifiedAnswer |
	"Try to make substitutions in the user's inputs and search for the selector again.
1 no change to answer.
2 answer Array -> OrderedCollection.
2 answer Character -> String
4 answer Symbol or String of len 1 -> Character
	For each of these, try straight, and try converting args:
Character -> String
Symbol or String of len 1 -> Character
	Return array with result, dataObjects, dataStrings.  Don't ever do a find on the same set of data twice."

dataObjs := dataObjects.  dataStrs := dataStrings.
selectors := {#asString. #first. #asOrderedCollection}.
classes := {Character. String. Array}.
didUnmodifiedAnswer := false.
selectors withIndexDo: [:ansSel :ansInd | | ds do result answerMod | "Modify the answer object"
	answerMod := false.
	do := dataObjs copyTwoLevel.  ds := dataStrs copy.
	(dataObjs last isKindOf: (classes at: ansInd)) ifTrue: [
		((ansSel ~~ #first) or: [dataObjs last size = 1]) ifTrue: [
			do at: do size put: (do last perform: ansSel).	"asString"
			ds at: ds size put: ds last, ' ', ansSel.
			result := MethodFinder new load: do; findMessage.
			(result first beginsWith: 'no single method') ifFalse: [
				"found a selector!!"
				^ Array with: result first with: do with: ds].	
			answerMod := true]].

	selectors allButLast withIndexDo: [:argSel :argInd | | ddo dds | "Modify an argument object"
			"for args, no reason to do Array -> OrderedCollection.  Identical protocol."
		didUnmodifiedAnswer not | answerMod ifTrue: [
		ddo := do copyTwoLevel.  dds := ds copy.
		dataObjs first withIndexDo: [:arg :ind |
			(arg isKindOf: (classes at: argInd))  ifTrue: [
				((argSel ~~ #first) or: [arg size = 1]) ifTrue: [
					ddo first at: ind put: ((ddo first at: ind) perform: argSel).	"asString"
					dds at: ind put: (dds at: ind), ' ', argSel.
					result := MethodFinder new load: ddo; findMessage.
					(result first beginsWith: 'no single method') ifFalse: [
						"found a selector!!"
						^ Array with: result first with: ddo with: dds]	.	
					didUnmodifiedAnswer not & answerMod not ifTrue: [
						didUnmodifiedAnswer := true].
					]]]]].
	].
^ Array with: 'no single method does that function' with: dataObjs with: dataStrs! !

!StringHolder class methodsFor: '*Tools-yellow button menu' stamp: 'ar 10/6/2009 10:36'!
codePaneMenu: aMenu shifted: shifted
	"Utility method for the 'standard' codePane menu"
	aMenu addList: (shifted 
		ifTrue:[self shiftedYellowButtonMenuItems]
		ifFalse:[self yellowButtonMenuItems]).
	^aMenu! !

!StringHolder class methodsFor: '*Tools' stamp: 'topa 3/18/2015 09:19'!
open
	^ (Smalltalk at: #Workspace ifAbsent:[self]) new openLabel: 'Workspace'
! !

!StringHolder class methodsFor: '*Tools' stamp: 'dtl 5/15/2011 20:46'!
openLabel: aString

	^self new openLabel: aString! !

!StringHolder class methodsFor: '*Tools-yellow button menu' stamp: 'topa 4/15/2015 16:53'!
shiftedYellowButtonMenuItems
	"Returns the standard yellow button menu items"
	| entries |
	entries := OrderedCollection withAll:
		{
			#-.
			{'explain' translated.					#explain}.
			{'pretty print' translated.				#prettyPrint}.
			{'pretty print with color' translated.		#prettyPrintWithColor}.
			{'file it in (G)' translated.					#fileItIn}.
			{'spawn (o)' translated.					#spawn}.
			#-.
			{'browse it (b)' translated.				#browseIt}.
			{'senders of it (n)' translated.			#sendersOfIt}.
			{'implementors of it (m)' translated.		#implementorsOfIt}.
			{'references to it (N)' translated.			#referencesToIt}.
			#-.
			{'selectors containing it (W)' translated.	#methodNamesContainingIt}.
			{'method strings with it (E)' translated.	#methodStringsContainingit}.
			{'method source with it' translated.		#methodSourceContainingIt}.
			{'class names containing it' translated.	#classNamesContainingIt}.
			{'class comments with it' translated.		#classCommentsContainingIt}.
			{'change sets with it' translated.			#browseChangeSetsWithSelector}.
			#-.
			{'save contents to file...' translated.		#saveContentsInFile}.
			{'send contents to printer' translated.		#sendContentsToPrinter}.
			{'printer setup' translated.				#printerSetup}.
			#-.
		}.
	Smalltalk isMorphic ifFalse: [ entries add: 
			{'special menu...' translated.				#presentSpecialMenu}.].
	entries add:
			{'more...' translated.					#yellowButtonActivity}.
	^ entries! !

!StringHolder class methodsFor: '*Tools-yellow button menu' stamp: 'mt 11/13/2015 09:41'!
yellowButtonMenuItems
	"Returns the standard yellow button menu items"
	^{
			#-.
			{'set font... (k)' translated.				#offerFontMenu}.
			{'set style... (K)' translated.				#changeStyle}.
			{'set alignment... (u)' translated.		#chooseAlignment}.
			#-.
			{'make project link (P)' translated.	#makeProjectLink}.
			#-.
			{'find...(f)' translated.					#find}.
			{'find again (g)' translated.				#findAgain}.
			{'find and replace ...' translated.			#findReplace}.
			{'do/replace again (j)' translated.				#again}.
			#-.
			{'undo (z)' translated.					#undo}.
			{'redo (Z)' translated.					#redo}.
			#-.
			{'copy (c)' translated.					#copySelection}.
			{'cut (x)' translated.						#cut}.
			{'paste (v)' translated.					#paste}.
			{'paste...' translated.					#pasteRecent}.
			#-.
			{'do it (d)' translated.		#doIt}.			     
		{'debug it (D)' translated.			#debugIt}.		     
		{'print it (p)' translated.		#printIt}.		     
		{'inspect it (i)' translated.		#inspectIt}.		     
		{'explore it (I)' translated.		#exploreIt}.		     
		{'button for it' translated.			#buttonForIt}.		     
		{'tally it' translated.			#tallyIt}.
		{'spy on it' translated.		#spyOnIt}.
			#-.
			{'accept (s)' translated.					#accept}.
			{'cancel (l)' translated.					#cancel}.
			#-.
			{'show bytecodes' translated.			#showBytecodes}.
			#-.
			{'copy html' translated.					#copyHtml}.
			#-.
			{'more...' translated.					#shiftedTextPaneMenuRequest}.
		}! !

!StringHolder methodsFor: '*Tools' stamp: 'sd 4/15/2003 22:46'!
browseAllMessages
	"Create and schedule a message set browser on all implementors of all the messages sent by the current method."

	| aClass aName method filteredList |
	(aName := self selectedMessageName) ifNotNil: [
		method := (aClass := self selectedClassOrMetaClass) compiledMethodAt: aName.
		filteredList := method messages reject: 
			[:each | #(new initialize = ) includes: each].
		self systemNavigation browseAllImplementorsOfList: filteredList asSortedCollection
			 title: 'All messages sent in ', aClass name, '.', aName]
! !

!StringHolder methodsFor: '*Tools' stamp: 'fbs 9/13/2013 17:29'!
browseClass
	"Open an class browser on this class and method"
	^ self browseMethodFull.! !

!StringHolder methodsFor: '*Tools-traits' stamp: 'al 12/6/2005 22:22'!
browseClassRefs

	| cls |
	cls := self selectedClass.
	(cls notNil and: [cls isTrait not])
		ifTrue: [self systemNavigation browseAllCallsOnClass: cls theNonMetaClass]
! !

!StringHolder methodsFor: '*Tools' stamp: 'sd 4/16/2003 08:42'!
browseLocalImplementors
	"Present a menu of all messages sent by the currently selected message. 
	Open a message set browser of all implementors of the message chosen in or below
	the selected class.
	Do nothing if no message is chosen."
	self getSelectorAndSendQuery: #browseAllImplementorsOf:localTo:
		to: self systemNavigation
		with: { self selectedClass }! !

!StringHolder methodsFor: '*Tools' stamp: 'eem 8/9/2014 10:05'!
browseLocalSenders
	self selectedClass ifNotNil:
		[:cls| | token |
		token := UIManager default request: 'browse for selector or literal'.
		token isEmpty ifTrue: [^self].
		Scanner new
			typedScan: token
			do: [:scannedToken :tokenType|
				token := tokenType = #word
							ifTrue:
								[(Symbol findInterned: token)
									ifNil: [^UIManager default  inform: 'no such selector: ', token]
									ifNotNil: [:symbol| symbol]]
							ifFalse:
								[scannedToken].
				self systemNavigation browseAllCallsOn: token localTo: cls]]! !

!StringHolder methodsFor: '*Tools' stamp: 'sd 4/16/2003 20:41'!
browseLocalSendersOfMessages
	"Present a menu of the currently selected message, as well as all
	messages sent by it.  Open a message set browser of all implementors
	of the message chosen in or below the selected class"

	self getSelectorAndSendQuery: #browseAllCallsOn:localTo:
		to: self systemNavigation
		with: { self selectedClass }! !

!StringHolder methodsFor: '*Tools' stamp: 'sd 4/16/2003 08:45'!
browseMessages
	"Present a menu of all messages sent by the currently selected message. 
	Open a message set browser of all implementors of the message chosen."

	self getSelectorAndSendQuery: #browseAllImplementorsOf: to: self systemNavigation! !

!StringHolder methodsFor: '*Tools' stamp: 'fbs 9/17/2013 22:52'!
browseMethodFull
	"Create and schedule a full Browser and then select the current class and message."
	^ self selectedClassOrMetaClass ifNotNil: [ :cls |
		self selectedMessageName
			ifNotNil: [ :sel |
				ToolSet browse: cls selector: sel]
			ifNil: [ToolSet browseClass: cls]].! !

!StringHolder methodsFor: '*Tools' stamp: 'sd 4/16/2003 20:40'!
browseSendersOfMessages
	"Present a menu of the currently selected message, as well as all messages sent by it.  Open a message set browser of all senders of the selector chosen."

	self getSelectorAndSendQuery: #browseAllCallsOn: to: self systemNavigation! !

!StringHolder methodsFor: '*Tools' stamp: 'sd 4/29/2003 20:20'!
browseUnusedMethods
	| classes unsent messageList cls |
	(cls := self selectedClass)
		ifNil: [^ self].
	classes := Array with: cls with: cls class.
	unsent := Set new.
	classes
		do: [:c | unsent addAll: c selectors].
	unsent := self systemNavigation allUnSentMessagesIn: unsent.
	messageList := OrderedCollection new.
	classes
		do: [:c | (c selectors
				select: [:s | unsent includes: s]) asSortedCollection
				do: [:sel | messageList add: c name , ' ' , sel]].
	self systemNavigation browseMessageList: messageList name: 'Unsent Methods in ' , cls name! !

!StringHolder methodsFor: '*Tools-traits' stamp: 'cmm 5/2/2011 09:55'!
browseVariableAssignments
	| cls |
	cls := self selectedClassOrMetaClass.
	(cls notNil and: [ cls isTrait not ]) ifTrue: [ self systemNavigation browseVariableAssignments: cls ]! !

!StringHolder methodsFor: '*Tools-traits' stamp: 'cmm 5/2/2011 09:45'!
browseVariableReferences
	| cls |
	cls := self selectedClassOrMetaClass.
	(cls notNil and: [cls isTrait not])
		ifTrue: [self systemNavigation browseVariableReferences: cls]! !

!StringHolder methodsFor: '*Tools' stamp: 'topa 6/23/2016 09:54'!
browseVersions
	"Create and schedule a Versions Browser, showing all versions of the 
	currently selected message. Answer the browser or nil."
	^ self classCommentIndicated
		ifTrue: [ClassCommentVersionsBrowser browseCommentOf: self selectedClass]
		ifFalse: [self selectedMessageName ifNotNil: 
			[:selector |
				ToolSet browseVersionsOf: self selectedClassOrMetaClass selector: selector]]! !

!StringHolder methodsFor: '*Tools' stamp: 'sd 1/16/2004 21:14'!
classHierarchy
	"Create and schedule a class list browser on the receiver's hierarchy."

	self systemNavigation
		spawnHierarchyForClass: self selectedClassOrMetaClass "OK if nil"
		selector: self selectedMessageName
! !

!StringHolder methodsFor: '*Tools' stamp: 'cmm 5/2/2011 09:56'!
classListKey: aChar from: view 
	"Respond to a Command key.  I am a model with a list of classes and a 
	code pane, and I also have a listView that has a list of methods.  The 
	view knows how to get the list and selection."

	aChar == $f ifTrue: [^ self findMethod].
	aChar == $r ifTrue: [^ self browseVariableReferences].
	aChar == $a ifTrue: [^ self browseVariableAssignments].
	aChar == $h ifTrue: [^ self spawnHierarchy].
	aChar == $x ifTrue: [^ self removeClass].
	^ self messageListKey: aChar from: view! !

!StringHolder methodsFor: '*Tools-pluggable menus - hooks' stamp: 'topa 3/27/2015 01:20'!
codePaneMenuHook: aMenu shifted: aBoolean
	<codePaneMenu>
	<menuPriority: 400>
	^ self menuHook: aMenu named: #codePaneMenu shifted: aBoolean.
! !

!StringHolder methodsFor: '*Tools-code pane menu' stamp: 'topa 3/13/2015 18:50'!
codePaneMenu: aMenu shifted: shifted
	^ self menu: aMenu for: #(codePaneMenu codePaneMenuShifted:) shifted: shifted
! !

!StringHolder methodsFor: '*Tools' stamp: 'topa 3/16/2015 16:06'!
codeTextMorph
	
	^ self someTextPaneWithSelector: #contents! !

!StringHolder methodsFor: '*Tools-code pane menu' stamp: 'ar 8/8/2009 22:58'!
contents: aString notifying: aController 
	"Accept text"
	^self acceptContents: aString! !

!StringHolder methodsFor: '*Tools' stamp: 'ar 1/15/2001 18:39'!
copyName
	"Copy the current selector to the clipboard"
	| selector |
	(selector := self selectedMessageName) ifNotNil:
		[Clipboard clipboardText: selector asString asText]! !

!StringHolder methodsFor: '*Tools' stamp: 'cmm 9/20/2010 13:59'!
copyReference
	"Copy the selected selector to the clipboard"
	self selectedMessageName ifNotNilDo:
		[ : selector | Clipboard clipboardText:
			(self selectedClassOrMetaClass
				ifNil: [ selector asString ]
				ifNotNilDo: [ : cls | (cls >> selector) reference ]) ]! !

!StringHolder methodsFor: '*Tools' stamp: 'sw 8/5/2002 16:53'!
copySelector
	"Copy the selected selector to the clipboard"

	| selector |
	(selector := self selectedMessageName) ifNotNil:
		[Clipboard clipboardText: selector asString]! !

!StringHolder methodsFor: '*Tools' stamp: 'topa 3/16/2015 16:05'!
dependentTextPaneWithSelector: aSelector

	^ self dependents detect:
		[:aMorph | (aMorph isKindOf: PluggableTextMorph) and:
			[aMorph getTextSelector == aSelector]]
		ifNone: [nil]
! !

!StringHolder methodsFor: '*Tools' stamp: 'nice 3/14/2010 19:43'!
fileOutAllMessages
	"Put a description of the all the message list on a file."

	FileStream
		writeSourceCodeFrom:
			(String new: 16000 streamContents: [:str |
				self messageList do: [:e |
					e actualClass printMethodChunk: e methodSymbol withPreamble: true
					on: str moveSource: false toFile: 0]])
		baseName: (self messageListSelectorTitle replaceAll: Character space with: $_) isSt: true useHtml: false! !

!StringHolder methodsFor: '*Tools' stamp: 'nice 3/14/2010 19:43'!
fileOutMessage
	"Put a description of the selected message on a file.
	If no message is selected, put a description of ALL the message list."

	self selectedMessageName
		ifNil:
			[self fileOutAllMessages]
		ifNotNil:
			[Cursor write showWhile:
				[self selectedClassOrMetaClass fileOutMethod: self selectedMessageName]]! !

!StringHolder methodsFor: '*Tools' stamp: 'di 5/6/1998 17:03'!
findMethodInChangeSets
	"Find and open a changeSet containing the current method."

	| aName |
	(aName := self selectedMessageName) ifNotNil: [
		ChangeSorter browseChangeSetsWithClass: self selectedClassOrMetaClass
					selector: aName]! !

!StringHolder methodsFor: '*Tools' stamp: 'tk 4/21/1998 09:23'!
inspectInstances
	"Inspect all instances of the selected class."

	| myClass |
	(myClass := self selectedClassOrMetaClass) ifNotNil:
		[myClass theNonMetaClass inspectAllInstances]. 
! !

!StringHolder methodsFor: '*Tools' stamp: 'tk 4/21/1998 09:00'!
inspectSubInstances
	"Inspect all instances of the selected class and all its subclasses"

	| aClass |
	(aClass := self selectedClassOrMetaClass) ifNotNil: [
		aClass theNonMetaClass inspectSubInstances].
! !

!StringHolder methodsFor: '*Tools-code pane menu' stamp: 'topa 3/16/2015 10:58'!
mainCodePaneMenu: aMenu shifted: shifted
	<codePaneMenu>
	"Fill in the given menu with additional items. The menu is prepoulated with the 'standard' text commands that the editor supports. Note that unless we override perform:orSendTo:, the editor will respond to all menu items in a text pane"
	^ self class codePaneMenu: aMenu shifted: shifted.
! !

!StringHolder methodsFor: '*Tools-code pane menu' stamp: 'eem 7/18/2014 11:42'!
menuHook: aMenu named: aSymbol shifted: aBool
	"Provide a hook for supplemental menu items.  Answer the appropriately-enhanced menu."
	^ aMenu! !

!StringHolder methodsFor: '*Tools' stamp: 'nice 3/14/2010 19:33'!
messageList
	^Array new! !

!StringHolder methodsFor: '*Tools' stamp: 'mt 1/22/2016 13:08'!
messageListKey: aChar from: view
	"Respond to a Command key.  I am a model with a code pane, and I also
	have a listView that has a list of methods.  The view knows how to get
	the list and selection."

	| sel class |
	aChar == $D ifTrue: [^ self toggleDiffing].

	sel := self selectedMessageName.
	aChar == $m ifTrue:  "These next two put up a type in if no message selected"
		[^ self useSelector: sel orGetSelectorAndSendQuery: #browseAllImplementorsOf: to: self systemNavigation].
	aChar == $n ifTrue: 
		[^ self useSelector: sel orGetSelectorAndSendQuery: #browseAllCallsOn: to: self systemNavigation].

	"The following require a class selection"
	(class := self selectedClassOrMetaClass) ifNil: [^ self arrowKey: aChar from: view].
	aChar == $b ifTrue: [^ ToolSet browse: class selector: sel].
	aChar == $N ifTrue: [^ self browseClassRefs].
	aChar == $i ifTrue: [^ self methodHierarchy].
	aChar == $h ifTrue: [^ self classHierarchy].
	aChar == $p ifTrue: [^ self browseFullProtocol].

	"The following require a method selection"
	sel ifNotNil: 
		[aChar == $o ifTrue: [^ self fileOutMessage].
		aChar == $c ifTrue: [^ self copySelector].
		aChar == $C ifTrue: [^ self copyReference].
		aChar == $v ifTrue: [^ self browseVersions].
		aChar == $x ifTrue: [^ self removeMessage]].

	^ self arrowKey: aChar from: view! !

!StringHolder methodsFor: '*Tools' stamp: 'ul 11/15/2010 12:05'!
messageListSelectorTitle
	| selector aString aStamp aSize |

	(selector := self selectedMessageName)
		ifNil:
			[aSize := self messageList size.
			^ (aSize = 0 ifTrue: ['no'] ifFalse: [aSize printString]), ' message', (aSize = 1 ifTrue: [''] ifFalse: ['s'])]
		ifNotNil:
			[Preferences timeStampsInMenuTitles
				ifFalse:	[^ nil].
			aString := selector truncateWithElipsisTo: 28.
			^ (aStamp := self timeStamp) size > 0
				ifTrue:
					[aString, String cr, aStamp]
				ifFalse:
					[aString]]! !

!StringHolder methodsFor: '*Tools' stamp: 'sd 1/16/2004 21:10'!
methodHierarchy
	"Create and schedule a method browser on the hierarchy of implementors."

	self systemNavigation 
			methodHierarchyBrowserForClass: self selectedClassOrMetaClass 
			selector: self selectedMessageName
! !

!StringHolder methodsFor: '*Tools' stamp: 'sw 5/22/2001 16:17'!
offerDurableMenuFrom: menuRetriever shifted: aBoolean
	"Pop up (morphic only) a menu whose target is the receiver and whose contents are provided by sending the menuRetriever to the receiver.  The menuRetriever takes two arguments: a menu, and a boolean representing the shift state; put a stay-up item at the top of the menu."

	| aMenu |
	aMenu := MenuMorph new defaultTarget: self.
	aMenu addStayUpItem.
	self perform: menuRetriever with: aMenu with: aBoolean.
		aMenu popUpInWorld! !

!StringHolder methodsFor: '*Tools' stamp: 'ar 1/10/2010 10:24'!
offerMenuFrom: menuRetriever shifted: aBoolean
	"Pop up a menu whose target is the receiver and whose contents are provided
	by sending the menuRetriever to the receiver. The menuRetriever takes two
	arguments: a menu, and a boolean representing the shift state."

	Project current offerMenu: menuRetriever from: self shifted: aBoolean
! !

!StringHolder methodsFor: '*Tools' stamp: 'ar 8/7/2009 22:27'!
openLabel: aString 
	"Create a standard system view of the model, me, a StringHolder and open it.  If in mvc, terminate the active controller so that the new window will immediately be activated."
	^ToolBuilder open: self label: aString! !

!StringHolder methodsFor: '*Tools-code pane menu' stamp: 'wod 5/29/1998 16:35'!
perform: selector orSendTo: otherTarget
	"Selector was just chosen from a menu by a user.  If can respond, then
perform it on myself. If not, send it to otherTarget, presumably the
editPane from which the menu was invoked."

	(self respondsTo: selector)
		ifTrue: [^ self perform: selector]
		ifFalse: [^ otherTarget perform: selector]! !

!StringHolder methodsFor: '*Tools' stamp: 'tk 4/28/1998 18:16'!
printOutMessage
	"Write a file with the text of the selected message, for printing by a web browser"

	self selectedMessageName ifNotNil: [
		self selectedClassOrMetaClass fileOutMethod: self selectedMessageName
							asHtml: true]! !

!StringHolder methodsFor: '*Tools' stamp: 'sd 5/23/2003 14:42'!
removeFromCurrentChanges
	"Tell the changes mgr to forget that the current msg was changed."

	ChangeSet current removeSelectorChanges: self selectedMessageName 
			class: self selectedClassOrMetaClass.
	self changed: #annotation! !

!StringHolder methodsFor: '*Tools' stamp: 'sw 1/28/1999 12:34'!
revertAndForget
	"Revert to the previous version of the current method, and tell the changes mgr to forget that it was ever changed.  Danger!!  Use only if you really know what you're doing!!"

	self okToChange ifFalse: [^ self].
	self revertToPreviousVersion.
	self removeFromCurrentChanges.
	self contentsChanged
! !

!StringHolder methodsFor: '*Tools' stamp: 'nb 6/17/2003 12:25'!
revertToPreviousVersion
	"Revert to the previous version of the current method"
	| aClass aSelector  changeRecords |
	self okToChange ifFalse: [^ self].
	aClass := self selectedClassOrMetaClass.
	aClass ifNil: [^ self changed: #flash].
	aSelector := self selectedMessageName.
	changeRecords := aClass changeRecordsAt: aSelector.
	(changeRecords == nil or: [changeRecords size <= 1]) ifTrue: [self changed: #flash.  ^ Beeper beep].
	changeRecords second fileIn.
	self contentsChanged
! !

!StringHolder methodsFor: '*Tools' stamp: 'eem 9/7/2010 12:04'!
selectMessageAndEvaluate: aBlock
	"Allow the user to choose one selector, chosen from the currently selected message's selector, as well as those of all messages sent by it, and evaluate aBlock on behalf of chosen selector.  If there is only one possible choice, simply make it; if there are multiple choices, put up a menu, and evaluate aBlock on behalf of the the chosen selector, doing nothing if the user declines to choose any"

	| selector method |
	(selector := self selectedMessageName) ifNil: [^ self].
	method := (self selectedClassOrMetaClass ifNil: [^aBlock value: selector])
				compiledMethodAt: selector
				ifAbsent: [].
	method isNil ifTrue: [^aBlock value: selector].
	^self withSelectorAndMessagesIn: method evaluate: aBlock! !

!StringHolder methodsFor: '*Tools-code pane menu' stamp: 'tk 4/6/98 11:43'!
showBytecodes
	"We don't know how to do this"

	^ self changed: #flash! !

!StringHolder methodsFor: '*Tools' stamp: 'topa 3/16/2015 18:38'!
someTextPaneWithSelector: aSelector
	" Try to find the text pane for aSelector in my dependents or my window. Not beautiful tho"
	^ (self dependentTextPaneWithSelector: aSelector)
		ifNil: [self textPaneWithSelector: aSelector]! !

!StringHolder methodsFor: '*Tools' stamp: 'ar 9/27/2005 20:47'!
spawn: contentsString

	UIManager default edit: contentsString label: 'Workspace'
! !

!StringHolder methodsFor: '*Tools' stamp: 'mt 4/12/2015 19:45'!
textPaneWithSelector: aSelector
	"If, among my window's paneMorphs, there is a text pane defined with aSelector as its retriever, answer it, else answer nil"

	| aWindow |
	Smalltalk isMorphic ifFalse: [^ nil].
	^ (aWindow := self containingWindow) ifNotNil:
		[aWindow paneMorphSatisfying:
			[:aMorph | (aMorph isKindOf: PluggableTextMorph) and:
				[aMorph getTextSelector == aSelector]]]! !

!StringHolder methodsFor: '*Tools' stamp: 'fbs 3/4/2013 20:50'!
timeStamp
	"Answer the time stamp for the chosen class and method, if any, else an empty string"

	|  selector  aMethod |
	(selector := self selectedMessageName) ifNotNil:
		[self selectedClassOrMetaClass 
			ifNil:
				[^ String new]
			ifNotNil:
				[aMethod := self selectedClassOrMetaClass compiledMethodAt: selector ifAbsent: [nil].
				aMethod ifNotNil: [^ aMethod timeStamp]]].
	^ String new! !

!StringHolder methodsFor: '*Tools-optional panes' stamp: 'sw 1/24/2001 21:25'!
wantsAnnotationPane
	"Answer whether the receiver, seen in some browser window, would like to have the so-called  annotationpane included.  By default, various browsers defer to the global preference 'optionalButtons' -- but individual subclasses can insist to the contrary."

	^ Preferences annotationPanes! !

!StringHolder methodsFor: '*Tools-optional panes' stamp: 'sw 1/24/2001 18:57'!
wantsOptionalButtons
	"Answer whether the receiver, seen in some browser window, would like to have the so-called optional button pane included.  By default, various browsers defer to the global preference 'optionalButtons' -- but individual subclasses can insist to the contrary."

	^ Preferences optionalButtons! !

!StringHolder methodsFor: '*Tools' stamp: 'ul 8/8/2013 13:12'!
withSelectorAndMessagesIn: aCompiledMethod evaluate: aBlock
	"Allow the user to choose one selector, chosen from the currently selected message's selector, as well as those of all messages sent by it, and evaluate aBlock on behalf of chosen selector.  If there is only one possible choice, simply make it; if there are multiple choices, put up a menu, and evaluate aBlock on behalf of the the chosen selector, doing nothing if the user declines to choose any"

	| selectorOrNil litGetter messages |
	selectorOrNil := aCompiledMethod selector.
	messages := SystemNavigation thoroughSenders
					ifTrue:
						[litGetter := [:set :l|
										(l isSymbol and: [l size > 0 and: [l first isLowercase]]) ifTrue:
											[set add: l].
										l isArray ifTrue:
											[l inject: set into: litGetter copy].
										set].
						aCompiledMethod allLiterals
						, (aCompiledMethod pragmas collect: [:pragma| pragma keyword])
							inject: aCompiledMethod messages into: litGetter copy]
					ifFalse: [aCompiledMethod messages].
	(messages isEmpty	"if no messages, use only selector"
	or: [messages size = 1 and: [messages includes: selectorOrNil]]) ifTrue:
		[^selectorOrNil ifNotNil: [aBlock value: selectorOrNil]].  "If only one item, there is no choice"

	self systemNavigation 
		showMenuOf: messages
		withFirstItem: selectorOrNil
		ifChosenDo: aBlock! !

StringHolder subclass: #SyntaxError
	instanceVariableNames: 'class selector debugger notification'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Debugger'!

!SyntaxError commentStamp: '<historical>' prior: 0!
I represent syntax error report for syntax errors encountered when filing in class descriptions from a non-interactive source such as an external file. As a StringHolder, the string to be viewed is the method code or expression containing the error.

The user may fix the error and accept the method to continue the fileIn.
!

!SyntaxError class methodsFor: 'instance creation' stamp: 'dtl 2/9/2010 08:11'!
open: aSyntaxError
	"Answer a standard system view whose model is an instance of me."
	
	<primitive: 19> "Simulation guard"
	^ Project current
		dispatchTo: self
		addPrefixAndSend: #Open:
		withArguments: {aSyntaxError}
! !

!SyntaxError methodsFor: 'other' stamp: 'nice 9/18/2013 21:24'!
contents: aString notifying: aController
	"Accept the code editions if syntax is correct.
	Then let the notification signalerContext proceed and restart compilation with these newSource."

	"In case of failure, return false so as to not accept the edits"
	notification reparse: aString notifying: aController ifFail: [^false].

	"else accept edits and proceed"
	aController hasUnacceptedEdits: false.
	self proceed! !

!SyntaxError methodsFor: 'menu' stamp: 'RAA 12/1/2000 14:24'!
debug
	"Show the stack of the process leading to this syntax editor, typically showing the stack of the compiler as called from fileIn."

	debugger openFullNoSuspendLabel: 'Stack of the Syntax Error'.
	Smalltalk isMorphic ifFalse: [Processor terminateActive].
! !

!SyntaxError methodsFor: 'message list' stamp: 'nice 9/20/2013 01:58'!
list
	"Answer an array of one element made up of the class name and message selector in which the syntax error was found. This is the single item in the message list of a view/browser on the receiver."

	^ Array with: (class name, '  ', (selector ifNil: ['<none>']))
! !

!SyntaxError methodsFor: 'message list' stamp: 'jm 5/3/1998 13:48'!
listIndex
	"There is always exactly one element in my list and it is always selected."

	^ 1
! !

!SyntaxError methodsFor: 'menu' stamp: 'jm 5/3/1998 14:22'!
listMenu: aMenu

	^ aMenu labels:
'proceed
debug calling process
browse full'
	lines: #()
	selections: #(proceed debug browseMethodFull)
! !

!SyntaxError methodsFor: 'menu' stamp: 'di 5/5/1998 00:06'!
proceed
	"The user has has edited and presumably fixed the syntax error and the filein can now proceed."

	debugger proceed: self topView.
! !

!SyntaxError methodsFor: 'initialize-release' stamp: 'nice 9/18/2013 21:30'!
release
	debugger ifNotNil:
		[debugger interruptedProcess ifNotNil:
			[:p |	p isTerminated ifFalse:
				[p terminate]]].! !

!SyntaxError methodsFor: 'text menu support' stamp: 'jm 5/3/1998 14:15'!
selectedClass
	"Answer the class in which the syntax error occurred."

	^ class
! !

!SyntaxError methodsFor: 'text menu support' stamp: 'jm 5/3/1998 14:33'!
selectedClassOrMetaClass
	"Answer the class of the method being compiled."

	^ class
! !

!SyntaxError methodsFor: 'text menu support' stamp: 'jm 5/3/1998 14:17'!
selectedMessageName
	"Answer the selector of the method being compiled."

	^ selector
! !

!SyntaxError methodsFor: 'accessing' stamp: 'nice 9/20/2013 01:57'!
setNotification: aSyntaxErrorNotification

	| types printables badChar code |
	notification := aSyntaxErrorNotification.
	class := aSyntaxErrorNotification errorClass.
	debugger := Debugger context: aSyntaxErrorNotification signalerContext.
	code := aSyntaxErrorNotification errorCode.
	selector := class newParser parseSelector: code.
	types := Scanner classPool at: #TypeTable.	"dictionary"
	printables := '!!@#$%&*-_=+<>{}?/\,·£¢§¶ªºÚæÚ¯×¿«»`~`' asSet.
	badChar := code detect: [:aChar | (types at: aChar asciiValue ifAbsent: [#xLetter]) == #xBinary and: [
			(printables includes: aChar) not]] ifNone: [nil].
	contents := badChar 
		ifNil: [code]
		ifNotNil: ['<<<This string contains a character (ascii value ', 
			badChar asciiValue printString,
			') that is not normally used in code>>> ', code].! !

StringHolder subclass: #Workspace
	instanceVariableNames: 'bindings acceptDroppedMorphs acceptAction mustDeclareVariables shouldStyle'
	classVariableNames: 'LookupPools ShouldStyle'
	poolDictionaries: ''
	category: 'Tools-Base'!

!Workspace commentStamp: 'fbs 6/2/2012 20:46' prior: 0!
A Workspace is a text area plus a lot of support for executable code.  It is a great place to execute top-level commands to compute something useful, and it is a great place to develop bits of a program before those bits get put into class methods.

To open a new workspace, execute:

	Workspace open


A workspace can have its own variables, called "workspace variables", to hold intermediate results.  For example, if you type into a workspace "x := 5" and do-it, then later you could type in "y := x * 2" and y would become 10.

Additionally, in Morphic, a workspace can gain access to morphs that are on the screen.  If acceptDroppedMorphs is turned on, then whenever a morph is dropped on the workspace, a variable will be created which references that morph.  This functionality is toggled with the window-wide menu of a workspace.


The instance variables of this class are:

	bindings  -  holds the workspace variables for this workspace

	acceptDroppedMorphs - whether dropped morphs should create new variables!

!Workspace class methodsFor: 'lookup pools' stamp: 'jcg 6/15/2010 15:21'!
addLookupPool: sharedPool
	"Add a SharedPool that bindings should be looked up in."
	LookupPools ifNil: [LookupPools := IdentitySet new].
	LookupPools add: sharedPool! !

!Workspace class methodsFor: 'lookup pools' stamp: 'jcg 6/15/2010 11:08'!
clearLookupPools
	LookupPools := nil.
! !

!Workspace class methodsFor: 'instance creation' stamp: 'ar 8/7/2009 22:29'!
prototypicalToolWindow
	"Answer an example of myself seen in a tool window, for the benefit of parts-launching tools"

	| aWorkspace |
	aWorkspace := ToolBuilder build: self new.
	^ aWorkspace! !

!Workspace class methodsFor: 'lookup pools' stamp: 'jcg 6/15/2010 15:21'!
removeLookupPool: sharedPool
	"Workspaces should no longer look up bindings in the specified SharedPool."
	LookupPools ifNil: [^self].
	LookupPools remove: sharedPool ifAbsent: [].! !

!Workspace class methodsFor: 'preferences' stamp: 'tpr 8/12/2016 17:19'!
shouldStyle
	<preference: 'Shout styling in Workspace' 
		category: 'browsing' 
		description: 'If true, new workspaces use Shout to style their contents.' 
		type: #Boolean>
	^ ShouldStyle ifNil: [ false ]! !

!Workspace class methodsFor: 'preferences' stamp: 'kb 2/22/2010 23:56'!
shouldStyle: aBoolean

	ShouldStyle := aBoolean! !

!Workspace methodsFor: 'code pane' stamp: 'kb 2/22/2010 23:58'!
aboutToStyle: aStyler

	self shouldStyle ifFalse: [ ^false ].
	aStyler 
		classOrMetaClass: nil;
		workspace: self.
	^true! !

!Workspace methodsFor: 'accessing' stamp: 'ar 7/16/2005 17:56'!
acceptAction
	^acceptAction! !

!Workspace methodsFor: 'accessing' stamp: 'ar 7/16/2005 17:56'!
acceptAction: anAction
	acceptAction := anAction.! !

!Workspace methodsFor: 'accessing' stamp: 'ar 7/16/2005 17:58'!
acceptContents: aString
	acceptAction ifNotNil:[acceptAction value: aString].
	^super acceptContents: aString.! !

!Workspace methodsFor: 'drag and drop' stamp: 'em 3/24/2005 11:21'!
acceptDroppedMorphsWording

	^ self acceptsDroppingMorphForReference
		ifTrue: ['<yes> create textual references to dropped morphs' translated]
		ifFalse: ['<no> create textual references to dropped morphs' translated]
! !

!Workspace methodsFor: 'drag and drop' stamp: 'bf 4/20/2016 18:10'!
acceptDroppingMorph: dropee event: evt inMorph: targetMorph 
	"Return the dropee to its old position, and add a reference to it at the
	cursor point."
	| bindingName externalName reference |
	(dropee isKindOf: TransferMorph)
		ifTrue: [reference := dropee passenger.
			externalName := dropee passenger className]
		ifFalse: [reference := dropee.
			dropee externalName].
	externalName := externalName isOctetString
				ifTrue: [externalName]
				ifFalse: ['a' , externalName].
	bindingName := externalName withFirstCharacterDownshifted , reference identityHash printString.
	targetMorph correctSelectionWithString: bindingName , ' '.
	(self bindingOf: bindingName)
		value: reference.
	(dropee isKindOf: TransferMorph)
		ifFalse: [dropee rejectDropMorphEvent: evt].
	^ true"success"! !

!Workspace methodsFor: 'drag and drop' stamp: 'jcg 7/8/2000 00:10'!
acceptsDroppingMorphForReference

	^ acceptDroppedMorphs
		
! !

!Workspace methodsFor: 'drag and drop' stamp: 'jcg 7/8/2000 00:09'!
acceptsDroppingMorphForReference: trueFalse

	acceptDroppedMorphs := trueFalse
		
! !

!Workspace methodsFor: 'menu commands' stamp: 'nice 4/20/2011 01:30'!
addModelItemsToWindowMenu: aMenu 
	
	aMenu addLine.
	aMenu
		add: 'save contents to file...'
		target: self
		action: #saveContentsInFile.
	aMenu
		add: 'inspect variables'
		target: self
		action: #inspectBindings.
	aMenu
		add: 'reset variables'
		target: self
		action: #initializeBindings.
	aMenu
		addUpdating: #mustDeclareVariableWording
		target: self
		action: #toggleVariableDeclarationMode.
	aMenu
		addUpdating: #acceptDroppedMorphsWording
		target: self
		action: #toggleDroppingMorphForReference.

	self addToggleStylingMenuItemTo: aMenu.
! !

!Workspace methodsFor: 'code pane menu' stamp: 'kb 2/22/2010 23:53'!
addToggleStylingMenuItemTo: aMenu
	
	aMenu
		addUpdating: #toggleStylingLabel
		target: self
		action: #toggleStyling! !

!Workspace methodsFor: 'menu commands' stamp: 'sw 11/8/2003 14:24'!
appendContentsOfFile
	"Prompt for a file, and if one is obtained, append its contents to the contents of the receiver.   Caution: as currently implemented this abandons any custom style information previously in the workspace.  Someone should fix this.  Also, for best results you should accept the contents of the workspace before requesting this."

	| aFileStream |
	(aFileStream := FileList2 modalFileSelector) ifNil: [^ self].
	contents := (contents ifNil: ['']) asString, aFileStream contentsOfEntireFile.
	aFileStream close.
	self changed: #contents! !

!Workspace methodsFor: 'binding' stamp: 'nice 4/20/2011 01:08'!
bindingOf: aString 
	mustDeclareVariables ifTrue: [^ nil].
	"I want to have workspace that force the user to declare  
	variables. Still subclasses may want to do something else"
	(bindings includesKey: aString) ifFalse: [
		(self lookupInPools: aString)
			ifNotNil: [:val |
				"Caveat: since we're not inserting the pool's Association into
				 the dictionary, assigning a new value to the variable will only
				 affect the one Workspace (i.e. you can't assign to a variable in a
				 SharedPool)."
				bindings at: aString put: val]
			ifNil: [aString first isUppercase
					ifTrue: [^nil]
					ifFalse: [bindings at: aString put: nil]]
	].
	^ bindings associationAt: aString! !

!Workspace methodsFor: 'toolbuilder' stamp: 'kb 2/22/2010 22:12'!
buildCodePaneWith: builder
	| textSpec |
	textSpec := builder pluggableCodePaneSpec new.
	textSpec 
		model: self;
		getText: #contents; 
		setText: #contents:notifying:; 
		selection: #contentsSelection; 
		menu: #codePaneMenu:shifted:.
	^textSpec! !

!Workspace methodsFor: 'accessing' stamp: 'fbs 4/28/2010 21:48'!
contents: textOrString
	super contents: textOrString.
	self changed: #contents.! !

!Workspace methodsFor: 'object fileIn' stamp: 'RAA 12/20/2000 17:50'!
convertToCurrentVersion: varDict refStream: smartRefStrm
	
	acceptDroppedMorphs ifNil: [acceptDroppedMorphs := false].
	^super convertToCurrentVersion: varDict refStream: smartRefStrm.

! !

!Workspace methodsFor: 'binding' stamp: 'nice 4/20/2011 01:08'!
hasBindingOf: aString 
	^bindings includesKey: aString! !

!Workspace methodsFor: 'initialize-release' stamp: 'wiz 2/13/2006 21:18'!
initialExtent
 "Start small.  Window aspect ratio is 5 sqrt::1 . Good asthetics. -wiz"
	
	^ 447@200! !

!Workspace methodsFor: 'initialize-release' stamp: 'nice 4/20/2011 01:06'!
initialize
	
	super initialize.
	self initializeBindings.
	acceptDroppedMorphs := false.
	mustDeclareVariables := false! !

!Workspace methodsFor: 'binding' stamp: 'sd 3/20/2004 14:31'!
initializeBindings
	
	bindings := Dictionary new! !

!Workspace methodsFor: 'binding' stamp: 'nice 4/20/2011 01:29'!
inspectBindings
	
	bindings inspect! !

!Workspace methodsFor: 'binding' stamp: 'jcg 6/15/2010 15:00'!
lookupInPools: aString 
	"Iterate through all of the lookup pools to find a match for the specified name."
	LookupPools ifNil: [^nil].
	LookupPools do: [:pool | (pool bindingOf: aString) ifNotNil: [:assoc | ^assoc value]].
	^nil
! !

!Workspace methodsFor: 'accessing' stamp: 'sd 3/20/2004 15:17'!
mustDeclareVariables: aBoolean

	mustDeclareVariables := aBoolean! !

!Workspace methodsFor: 'variable declarations' stamp: 'sumim 11/26/2006 03:52'!
mustDeclareVariableWording
	
	^ mustDeclareVariables not
		ifTrue: ['<yes> automatically create variable declaration' translated]
		ifFalse: ['<no> automatically create variable declaration' translated]! !

!Workspace methodsFor: 'menu commands' stamp: 'dtl 2/10/2010 17:00'!
saveContentsInFile
	"Pass along this message to the controller or morph.  (Possibly this Workspace menu item could be deleted, since it's now in the text menu.)"

	self changed: #saveContents
! !

!Workspace methodsFor: 'accessing' stamp: 'jsp 3/23/1999 12:19'!
setBindings: aDictionary
	"Sets the Workspace to use the specified dictionary as its namespace"

	bindings := aDictionary.
! !

!Workspace methodsFor: 'code pane menu' stamp: 'kb 2/22/2010 23:53'!
shouldStyle

	^shouldStyle ifNil: [ self class shouldStyle ]! !

!Workspace methodsFor: 'code pane menu' stamp: 'ar 4/21/2010 21:23'!
shouldStyle: aBool
	"If true, contents will by rendered w/ syntax-highlighting"

	shouldStyle := aBool! !

!Workspace methodsFor: 'drag and drop' stamp: 'jcg 7/8/2000 00:11'!
toggleDroppingMorphForReference

	acceptDroppedMorphs := acceptDroppedMorphs not.
		
! !

!Workspace methodsFor: 'code pane menu' stamp: 'kb 2/23/2010 00:44'!
toggleStyling

	shouldStyle := self shouldStyle not.
	" Ugly hack, to restyle our contents. "
	self codeTextMorph in: [ :codeTextMorph |
		codeTextMorph setText:
			codeTextMorph textMorph text asString asText ]! !

!Workspace methodsFor: 'code pane menu' stamp: 'ar 2/26/2010 01:19'!
toggleStylingLabel

	^self shouldStyle 
		ifTrue: [ '<on> syntax highlighting' ]
		ifFalse: [ '<off> syntax highlighting' ]! !

!Workspace methodsFor: 'variable declarations' stamp: 'sd 3/20/2004 14:39'!
toggleVariableDeclarationMode

	mustDeclareVariables := mustDeclareVariables not! !

!Workspace methodsFor: 'accessing' stamp: 'ar 8/8/2009 17:17'!
wantsAnnotationPane
	"We have nothing to annotate"
	^false! !

!Workspace methodsFor: 'drag and drop' stamp: 'bf 4/20/2016 18:02'!
wantsDroppedMorph: dropee event: evt inMorph: target

	^ acceptDroppedMorphs or: [dropee isKindOf: TransferMorph]
		
! !

!Workspace methodsFor: 'accessing' stamp: 'ar 8/8/2009 17:17'!
wantsOptionalButtons
	"We have no optional buttons (yet)"
	^false! !

!Message methodsFor: '*Tools-Debugger' stamp: 'fbs 4/19/2013 08:31'!
createStubMethod
	| argNames |
	argNames := Set new.
	^ String streamContents: [ :s |
		self selector keywords doWithIndex: [ :key :i |
			| aOrAn argName arg argClassName |
			s nextPutAll: key.
			((key last = $:) or: [self selector isInfix]) ifTrue: [
				arg := self arguments at: i.
				argClassName := arg canonicalArgumentName.
				aOrAn := argClassName first isVowel ifTrue: ['an'] ifFalse: ['a'].
				argName := aOrAn, argClassName.
				[argNames includes: argName] whileTrue: [argName := argName, i asString].
				argNames add: argName.
				s nextPutAll: ' '; nextPutAll: argName; space
			].
		].
		s cr; tab.
		s nextPutAll: 'self shouldBeImplemented'
	].! !

!String class methodsFor: '*Tools-icons' stamp: 'topa 3/26/2015 16:09'!
toolIcon

	^ #string! !

!String methodsFor: '*Tools-Debugger' stamp: ''!
canonicalArgumentName
	^ 'String'! !

!TheWorldMenu methodsFor: '*Tools' stamp: 'ar 7/16/2005 19:47'!
browseRecentLog
	ChangeList browseRecentLog! !

!TheWorldMenu methodsFor: '*Tools' stamp: 'ar 8/7/2009 22:21'!
openChangeSorter1

	ToolBuilder open: ChangeSorter new! !

!TheWorldMenu methodsFor: '*Tools' stamp: 'ar 8/7/2009 22:21'!
openChangeSorter2

	ToolBuilder open: DualChangeSorter new! !

!TheWorldMenu methodsFor: '*Tools' stamp: 'sw 7/28/2001 02:11'!
openMessageNames
	"Bring a MessageNames tool to the front"

	MessageNames openMessageNames! !

!TheWorldMenu methodsFor: '*Tools' stamp: 'ar 7/16/2005 20:02'!
openProcessBrowser
	ProcessBrowser open! !

!TheWorldMenu methodsFor: '*Tools' stamp: 'ar 8/7/2009 22:21'!
openSelectorBrowser

	ToolBuilder open: SelectorBrowser new! !

!TheWorldMenu methodsFor: '*Tools' stamp: 'mt 2/16/2016 09:46'!
startThenBrowseMessageTally
	(self confirm: 'MessageTally will start now,
and stop when the cursor goes
to the top of the screen')
		ifTrue: [TimeProfileBrowser
				onBlock: [[Sensor peekPosition y > 10]
						whileTrue: [World doOneCycle]]]! !

!Collection class methodsFor: '*Tools-icons' stamp: 'topa 3/26/2015 16:08'!
toolIcon

	^ #collection! !

!Collection methodsFor: '*Tools-Debugger' stamp: ''!
canonicalArgumentName
	^ 'Collection'! !

!MethodContext methodsFor: '*Tools-Inspector' stamp: 'ar 8/19/2010 12:16'!
inspectorClass
	"Answer the class of the inspector to be used on the receiver.  Called by inspect; 
	use basicInspect to get a normal (less useful) type of inspector."

	^ ContextInspector! !

!FileDirectory class methodsFor: '*Tools-FileList' stamp: 'mt 10/22/2015 10:45:06'!
serviceBrowseDirectory

	^ (SimpleServiceEntry
		provider: FileList
		label: 'browse directory'
		selector: #openOn:
		description: 'browse directory'
		buttonLabel: 'browse')
			argumentGetter:  [:directory | directory];
			yourself! !

!FileStream class methodsFor: '*Tools-Changes' stamp: 'mt 10/7/2015 09:26'!
edit: fullNameOrStream

	^ (fullNameOrStream isString
		ifTrue: [self fileNamed: fullNameOrStream]
		ifFalse: [fullNameOrStream]) edit! !

!FileStream class methodsFor: '*Tools-Changes' stamp: 'mt 10/7/2015 10:17'!
serviceEditFile

	^ SimpleServiceEntry 
		provider: self 
		label: 'edit as text file'
		selector: #edit:
		description: 'edit as text file'
		buttonLabel: 'edit'! !

!FileStream class methodsFor: '*Tools-Changes' stamp: 'mt 10/7/2015 10:17'!
serviceEditFileSuffixes

	^ #('*')! !

!FileStream methodsFor: '*Tools-Changes' stamp: 'di 5/20/1998 23:20'!
edit
	"Create and schedule an editor on this file."

	FileList openEditorOn: self editString: nil.
! !

!FileStream methodsFor: '*Tools-Changes' stamp: 'ar 9/6/2010 10:34'!
fileIntoNewChangeSet
	"File all of my contents into a new change set." 

	self readOnly.
	ChangesOrganizer newChangesFromStream: self named: self localName! !

!WeakSet methodsFor: '*Tools-Inspector' stamp: 'apb 8/20/2004 23:48'!
inspectorClass 
	^ WeakSetInspector! !

!PasteUpMorph methodsFor: '*Tools' stamp: 'cmm 10/19/2015 15:24:47'!
defaultDesktopCommandKeyTriplets
	"Answer a list of triplets of the form
		<key> <receiver> <selector>   [+ optional fourth element, a <description> for use in desktop-command-key-help]
that will provide the default desktop command key handlers.  If the selector takes an argument, that argument will be the command-key event"

	| noviceKeys expertKeys |

	noviceKeys := {
		{ $o.	ActiveWorld.						#activateObjectsTool.						'Activate the "Objects Tool"'}.
		{ $r.	ActiveWorld.						#restoreMorphicDisplay.					'Redraw the screen'}.		
		{ $z.	self.								#undoOrRedoCommand.					'Undo or redo the last undoable command'}.
		{ $F.	Project current.					#toggleFlapsSuppressed.					'Toggle the display of flaps'}.
		{ $N.	self.								#toggleClassicNavigatorIfAppropriate.	'Show/Hide the classic Navigator, if appropriate'}.
		{ $M.	self.								#toggleShowWorldMainDockingBar.		'Show/Hide the Main Docking Bar'}.
		{ $].	Smalltalk.							#saveSession.								'Save the image.'}.
	}.

	Preferences noviceMode
			ifTrue:[^ noviceKeys].

	expertKeys := {
		{ $b.	SystemBrowser.					#defaultOpenBrowser.						'Open a new System Browser'}.
		{ $k.	StringHolder.					#open.										'Open a new, blank Workspace'}.
		{ $m.	self.								#putUpNewMorphMenu.					'Put up the "New Morph" menu'}.
		{ $O.	self.								#findAMonticelloBrowser.					'Bring a Monticello window into focus.'}.
		{ $t.	self.	 							#findATranscript:.							'Make a System Transcript visible'}.
		{ $w.	SystemWindow.					#closeTopWindow.							'Close the topmost window'}.
		{ Character escape.	SystemWindow.	#closeTopWindow.							'Close the topmost window'}.

		{ $C.	self.								#findAChangeSorter:.						'Make a Change Sorter visible'}.

		{ $L.	self.								#findAFileList:.								'Make a File List visible'}.
		{ $P.	self.								#findAPreferencesPanel:.					'Activate the Preferences tool'}.
		{ $R.	Utilities. 								#browseRecentSubmissions.		'Make a Recent Submissions browser visible'}.

		{ $W.	self. 								#findAMessageNamesWindow:.			'Make a MessageNames tool visible'}.
		{ $Z.	ChangeList. 						#browseRecentLog.							'Browse recently-logged changes'}.

		{ $\.	SystemWindow. 					#sendTopWindowToBack.					'Send the top window to the back'}.
		{ $_.	Smalltalk.							#quitPrimitive.								'Quit the image immediately.'}.
	}.

	^ noviceKeys, expertKeys
! !

!PasteUpMorph methodsFor: '*Tools' stamp: 'eem 12/17/2012 11:19'!
findAMonticelloBrowser
	(Smalltalk classNamed: #MCWorkingCopyBrowser) ifNotNil:
		[:mc|
		self
			findAWindowSatisfying: [:aWindow | aWindow model isMemberOf: mc]
			orMakeOneUsing: [mc open]]! !

AppRegistry subclass: #SystemBrowser
	instanceVariableNames: ''
	classVariableNames: 'BrowseWithDragNDrop BrowseWithPrettyPrint'
	poolDictionaries: ''
	category: 'Tools-Base'!

!SystemBrowser commentStamp: '<historical>' prior: 0!
This is the AppRegistry class for class browsing!

!SystemBrowser class methodsFor: 'registration' stamp: 'hpt 9/30/2004 20:53'!
addRegistryMenuItemsTo: aMenu inAccountOf: aBrowser 
	"Add some useful options related Browser registry to the
	browsers windows menu"
	aMenu addLine;
		add: 'Register this Browser as default'
		target: [self default: aBrowser class]
		action: #value;
		add: 'Choose new default Browser'
		target: self
		action: #askForDefault! !

!SystemBrowser class methodsFor: 'preferences' stamp: 'fbs 11/30/2013 11:40'!
browserShowsPackagePane
	<preference: 'Browser shows package pane' category: 'browsing' description: 'If true, then the various ''browse full'' and ''browse it'' commands (usually invoked via cmd-b) will open a Package Browser rather than a System Browser.  The Package Browser includes a package pane which groups system categories into packages based on the initial portion of their category name.' type: #Boolean>
	^ SystemNavigation default browserClass == PackagePaneBrowser.! !

!SystemBrowser class methodsFor: 'preferences' stamp: 'fbs 11/30/2013 11:40'!
browserShowsPackagePane: aBoolean
	| theOtherOne |
	self registeredClasses size = 2
		ifTrue: [theOtherOne := (self registeredClasses copyWithout: PackagePaneBrowser) first]
		ifFalse: [theOtherOne := nil].
	aBoolean
		ifTrue: [self default: PackagePaneBrowser]
		ifFalse: [self default: theOtherOne].
	SystemNavigation default browserClass: self default.! !

!SystemBrowser class methodsFor: 'preferences' stamp: 'fbs 12/29/2013 17:08'!
browseWithDragNDrop
	<preference: 'Browse with drag ''n'' drop' category: 'browsing' description: 'If true, new browsers will open with drag & drop functionality (only in Morphic)' type: #Boolean>
	^ BrowseWithDragNDrop ifNil: [false].! !

!SystemBrowser class methodsFor: 'preferences' stamp: 'fbs 12/29/2013 17:08'!
browseWithDragNDrop: aBoolean
	BrowseWithDragNDrop := aBoolean.! !

!SystemBrowser class methodsFor: 'preferences' stamp: 'fbs 12/29/2013 16:54'!
browseWithPrettyPrint
	<preference: 'Browse with pretty-print' category: 'browsing' description: 'If true, browsers will automatically format their contents' type: #Boolean>
	^ BrowseWithPrettyPrint ifNil: [false].! !

!SystemBrowser class methodsFor: 'preferences' stamp: 'fbs 12/29/2013 16:54'!
browseWithPrettyPrint: aBoolean
	BrowseWithPrettyPrint := aBoolean.! !

!SystemBrowser class methodsFor: 'instance creation' stamp: 'hpt 8/5/2004 20:27'!
defaultOpenBrowser
	^self default openBrowser! !

!OrderedCollection methodsFor: '*Tools-Inspector' stamp: 'apb 7/14/2004 12:19'!
inspectorClass 
	"Answer the class of the inspector to be used on the receiver.  Called by inspect; 
	use basicInspect to get a normal (less useful) type of inspector."

	^OrderedCollectionInspector! !

!Number methodsFor: '*Tools-Debugger' stamp: ''!
canonicalArgumentName
	^ 'Number'! !

!SystemNavigation methodsFor: '*tools-browser' stamp: 'mu 3/11/2004 15:46'!
browserClass
	browserClass ifNil: [browserClass := self defaultBrowserClass].
	^browserClass! !

!SystemNavigation methodsFor: '*tools-browser' stamp: 'mu 3/11/2004 15:46'!
browserClass: aBrowserClass
	browserClass := aBrowserClass! !

!SystemNavigation methodsFor: '*tools-browser' stamp: 'nk 2/25/2005 09:42'!
defaultBrowserClass
	^SystemBrowser default! !

!SystemNavigation methodsFor: '*tools-browser' stamp: 'mu 3/11/2004 15:49'!
defaultHierarchyBrowserClass
	^self class environment at: #HierarchyBrowser ifAbsent:[]! !

!SystemNavigation methodsFor: '*tools-browser' stamp: 'mu 3/11/2004 15:50'!
hierarchyBrowserClass
	hierarchyBrowserClass ifNil: [hierarchyBrowserClass := self defaultHierarchyBrowserClass].
	^hierarchyBrowserClass! !

!SystemNavigation methodsFor: '*tools-browser' stamp: 'mu 3/11/2004 15:47'!
hierarchyBrowserClass: aBrowserClass
	hierarchyBrowserClass := aBrowserClass! !

!Dictionary methodsFor: '*Tools-Inspector' stamp: 'apb 7/14/2004 12:18'!
inspectorClass
	"Answer the class of the inspector to be used on the receiver.  Called by inspect; 
	use basicInspect to get a normal (less useful) type of inspector."

	^ DictionaryInspector! !

!Utilities class methodsFor: '*Tools' stamp: 'cmm 12/2/2013 10:52'!
browseRecentSubmissions
	"Open up a browser on the most recent methods submitted in the image."
	^ RecentMessageSet 
		openMessageList: RecentMessages default methodReferences
		name: 'Recent Submissions' 
		autoSelect: nil! !

!Utilities class methodsFor: '*Tools' stamp: 'dtl 2/12/2010 22:33'!
closeAllDebuggers
	"Utilities closeAllDebuggers"
	Smalltalk isMorphic
	ifTrue:
		[((Smalltalk at: #SystemWindow) allSubInstances select: [:w | w model isKindOf: Debugger])
			do: [:w | w delete]]
	ifFalse:
		[((Smalltalk at: #StandardSystemController) allInstances select: [:w | w model isKindOf: Debugger])
			do: [:w | w closeAndUnscheduleNoTerminate]]! !

!Utilities class methodsFor: '*Tools' stamp: 'cmm 10/12/2013 16:51'!
openRecentSubmissionsBrowser
	"Open up a browser on the most recent methods submitted in the image; reuse any existing one found in the world."
self deprecated: 'use #browseRecentSubmissions'.
	self browseRecentSubmissions! !

!Utilities class methodsFor: '*Tools' stamp: 'cmm 12/2/2013 10:51'!
recentSubmissionsWindow
	"Answer a SystemWindow holding recent submissions"
	| messageSet |
	messageSet := RecentMessageSet messageList: RecentMessages default methodReferences.
	messageSet autoSelectString: nil.
	^ ToolBuilder build: messageSet! !

!Class methodsFor: '*Tools-Debugger' stamp: ''!
canonicalArgumentName
	^ 'Class'! !

!Morph class methodsFor: '*Tools-icons' stamp: 'topa 3/26/2015 16:09'!
toolIcon

	^ #morph! !

!TranscriptStream class methodsFor: '*Tools' stamp: 'dtl 10/4/2009 23:44'!
buildWith: aBuilder
	^(Smalltalk at: #Transcript) buildWith: aBuilder! !

!TranscriptStream class methodsFor: '*Tools' stamp: 'ar 8/7/2009 22:24'!
openMorphicTranscript
	"Have the current project's transcript open up as a morph"

	^ToolBuilder open: self! !

!TranscriptStream methodsFor: '*Tools' stamp: 'ar 2/11/2005 20:36'!
buildWith: builder
	| windowSpec textSpec |
	windowSpec := builder pluggableWindowSpec new.
	windowSpec model: self.
	windowSpec label: 'Transcript'.
	windowSpec children: OrderedCollection new.

	textSpec := builder pluggableTextSpec new.
	textSpec 
		model: self;
		menu: #codePaneMenu:shifted:;
		frame: (0@0corner: 1@1).
	windowSpec children add: textSpec.

	^builder build: windowSpec! !

!TranscriptStream methodsFor: '*Tools' stamp: 'di 5/27/1998 16:44'!
codePaneMenu: aMenu shifted: shifted
	"Note that unless we override perform:orSendTo:, PluggableTextController will respond to all menu items"
	^ StringHolder basicNew codePaneMenu: aMenu shifted: shifted
! !

!TranscriptStream methodsFor: '*Tools' stamp: 'mp 2/2/2010 20:36'!
open
	| openCount |
	openCount := self countOpenTranscripts.
	openCount = 0
		ifTrue: [self openLabel: 'Transcript']
		ifFalse: [self openLabel: 'Transcript #' , (openCount+1) printString]! !

!TranscriptStream methodsFor: '*Tools' stamp: 'ar 8/7/2009 22:28'!
openLabel: aString 
	"Open a window on this transcriptStream"
	^ToolBuilder open: self label: aString! !

!Integer methodsFor: '*Tools-Debugger' stamp: ''!
canonicalArgumentName
	^ 'Integer'! !
CPUWatcher initialize!
ProcessBrowser initialize!
ChangesOrganizer initialize!
DebuggerMethodMap initialize!
PopUpMenu initialize!
StandardToolSet initialize!
ArchiveViewer initialize!
Browser initialize!
FileContentsBrowser initialize!
ChangeList initialize!
ChangeSorter initialize!
CodeHolder initialize!
Debugger initialize!
DependencyBrowser initialize!
MessageTrace initialize!
FileList initialize!
